#!/bin/sh
# -*- mode: landisk-sh; coding: utf-8; -*-
#
# initialize.sh: System initialization script for HDL-AA series.
# Copyright (C) 2013-2016 I-O DATA DEVICE, INC.  All rights reserved.
#

DEBUG=
#DEBUG=1

D_MP_DISKLABEL="/mnt/disklabel"
D_DISKLABEL="${D_MP_DISKLABEL}/disklabel"

D_MP_PART1="/mnt/part1"

D_MP_ILOG="/mnt/log"
D_MP_IROOT="/mnt/install"
D_MP_IBOOT="${D_MP_IROOT}/boot"
D_MP_IDATA="${D_MP_IROOT}/mnt/data"
D_MP_IVFSEXT="${D_MP_IROOT}/mnt/vfsext"
D_MP_ISATARAID1="${D_MP_IROOT}/mnt/sataraid1"

F_TMP_POWER_LOG="/tmp/power.log"
F_RESET_OK="${D_INIT_FW}/reset_ok"
F_RESET_NG="${D_INIT_FW}/reset_ng"

F_PROC_BTNLED="/proc/nas_ctrl/btnled"


###############################################################################
# LED を設定する
###############################################################################
__initialization_set_led()
{
	local color="${1}"
	local action="${2}"

	if [ ! -e "${F_PROC_BTNLED}" ]; then
		return 1
	fi

	echo "${color} ${action}" > "${F_PROC_BTNLED}"
	return 0
}


###############################################################################
# STATUS LED を設定する
# 入力:
#   arg1: 色(green, red, orange)
#   arg2: 動作(on, blink)
###############################################################################
__initialization_set_status_led()
{
	local color="${1}"
	local action="${2}"

	case "${color}" in
		green)
			__initialization_set_led "led_green" "${action}"
			__initialization_set_led "led_red" "off"
			__initialization_set_led "led_orange" "off"
			;;
		red)
			__initialization_set_led "led_green" "off"
			__initialization_set_led "led_red" "${action}"
			__initialization_set_led "led_orange" "off"
			;;
		orange)
			__initialization_set_led "led_green" "off"
			__initialization_set_led "led_red" "off"
			__initialization_set_led "led_orange" "${action}"
			;;
		*)
			;;
	esac

	return 0
}


###############################################################################
# エラー表示して初期化を終了する
# 入力: 引数1=表示メッセージ
###############################################################################
__initialization_error()
{
	local msg="${1}"

	echo "###### ERROR ######"
	echo "${msg}"

	# 初期化 NG のフラグを立てる
	touch "${F_RESET_NG}"

	# RAID の動作を停止する
	/sbin/mdadm --stop /dev/md13 1>/dev/null 2>/dev/null
	/sbin/mdadm --stop /dev/md6 1>/dev/null 2>/dev/null
	/sbin/mdadm --stop /dev/md5 1>/dev/null 2>/dev/null
	/sbin/mdadm --stop /dev/md2 1>/dev/null 2>/dev/null

	exit 1
}


###############################################################################
# 使用するツールをコピーする
###############################################################################
__initialization_provide_mandatory_tools()
{
	if [ -e /sbin/mkswap ]; then
		# BusyBox の mkswap は先に削除しておく
		rm -f /sbin/mkswap 1>/dev/null 2>/dev/null
	fi

	cp -r ${D_INIT_FW}/lib/* /lib
	cp ${D_INIT_FW}/sbin/* /sbin
	cp -r ${D_INIT_FW}/etc/* /etc

	depmod -a
	modprobe vfsext
}


###############################################################################
# 内蔵ディスクの存在を確認する
# 入力: $1=ディスク数
# 出力: 0=異常なし  1〜63=異常あり(異常ディスクのビットが 1)
###############################################################################
__initialization_check_internal_disk_existence()
{
	local disk_count="${1}"
	local retval=0
	local i=1

	while [ ${i} -le ${disk_count} ];
	do
		if [ ! -e "/dev/satad${i}" ]; then
			# ディスクがない
			echo "SATAD${i} not found."
			retval=$((${retval} + (1 << (${i} - 1))))
		fi

		i=$((${i} + 1))
	done

	return ${retval}
}


###############################################################################
# 内蔵ディスクの容量が十分か確認する
# 入力: $1=元ディスクデバイス名, $2=ディスク数
# 表示: "(セクター数)"=異常なし  "0"=異常あり
###############################################################################
__initialization_check_internal_disk_size()
{
	local srcdisk="${1}"
	local disk_count="${2}"
	local srcdisk_sector=$(
		__initialization_get_sector_from_model "${srcdisk}"
	)
	local i=1

	while [ ${i} -le ${disk_count} ];
	do
		if [ "x${srcdisk}" = "x/dev/satad${i}" ]; then
			# 元ディスクなので次へ
			i=$((${i} + 1))
			continue
		fi

		# 容量確認
		local curdisk_sector=$(
			/bin/cat "/sys/block/$(readlink /dev/satad${i})/size" \
				2>/dev/null
		)

		if [ ${srcdisk_sector} -gt ${curdisk_sector} ]; then
			# ディスク容量不足
			# ディスクのランプを赤点灯にする

			echo "0"
			return 1
		fi

		i=$((${i} + 1))
	done

	echo "${srcdisk_sector}"
	return 0
}


###############################################################################
# GB 容量からセクター数を取得する(IDEMA 準拠)
# 入力: $1=GB容量
# 表示: (セクター数)
###############################################################################
__initialization_get_idema_sector()
{
	local gigabyte="${1}"

	echo "$((97696368 + (1953504 * (${gigabyte} - 50))))"
	return 0
}


###############################################################################
# ディスクラベルの型番からセクター数を取得する
# 入力: $1=元ディスクデバイス名
# 表示: "(セクター数)"=異常なし  "0"=異常あり
###############################################################################
__initialization_get_sector_from_model()
{
	local srcdisk="${1}"
	local model=$(__initialization_get_model "${srcdisk}")
	local model_disk_count=$(
		echo "${model}" | \
			sed -e 's/HDL\([0-9]*\)-AA[0-9]*W*/\1/' \
			2>/dev/null
	)
	local model_total_size=$(
		echo "${model}" | \
			sed -e 's/HDL[0-9]*-AA\([0-9]*\)W*/\1/' \
			2>/dev/null
	)

	if [ "x${model_disk_count}" = "x" ]; then
		# HDL-AA 用
		model_disk_count=1
	fi

	local model_disk_size=$((${model_total_size} / ${model_disk_count}))
	local gigabyte=$((${model_disk_size} * 1000))

	echo "$(__initialization_get_idema_sector ${gigabyte})"
	return 0
}


###############################################################################
# ディスクからセクター数を取得する
# 入力: $1=元ディスクデバイス名
# 出力: "(セクター数)"=異常なし  "0"=異常あり
###############################################################################
__initialization_get_sector_from_disk()
{
	local srcdisk="${1}"
	local sector="0"
	local sector_srcdisk=$(
		/bin/cat "/sys/block/$(readlink ${srcdisk})/size" \
			2>/dev/null
	)

	if [ "x${sector_srcdisk}" != "x" ]; then
		sector="${sector_srcdisk}"
	fi

	echo "${sector}"
	return 0
}


###############################################################################
# ディスクラベルから型番を読み込む
# 入力: $1=元ディスクデバイス名
# 表示: "(型番文字列)"=異常なし  "NG"=異常あり
###############################################################################
__initialization_get_model()
{
	local srcdisk="${1}"
	local model=

	if [ ! -e "${srcdisk}p4" ]; then
		# ディスクラベルパーティションが存在しない(通常ここにはこない)
		echo "NG"
		return 1
	fi

	if [ ! -e "${D_MP_DISKLABEL}" ]; then
		/bin/mkdir -p \
			"${D_MP_DISKLABEL}" \
			1>/dev/null 2>/dev/null
	fi

	/bin/mount -t ext4 \
		"${srcdisk}p4" \
		"${D_MP_DISKLABEL}" \
		1>/dev/null 2>/dev/null
	if [ ${?} != 0 ]; then
		# マウントできない
		echo "NG"
		return 1
	fi

	model=$(
		/bin/cat "${D_DISKLABEL}/model" \
			2>/dev/null
	)

	/bin/umount \
		"${D_MP_DISKLABEL}" \
		1>/dev/null 2>/dev/null

	echo "${model}"
	return 0
}


###############################################################################
# 内蔵ディスクのパーティション構成が正しいか
# 入力: $1=ディスクデバイス名
# 出力: 0=異常なし  1=異常あり
###############################################################################
__initialization_is_valid_partition_table()
{
	local disk="${1}"

	local table=$(
		/sbin/parted -s \
			"${disk}" \
			unit s \
			p \
			2>/dev/null \
			| grep "^Partition Table:" 2>/dev/null \
			| cut -d: -f2 2>/dev/null \
			| sed -n 's/^ *//p' 2>/dev/null
	)

	if [ "x${table}" != "xgpt" ]; then
		# GPT でない
		return 1
	fi

	# 構成確認
	local part1=$(
		/sbin/parted -s \
			"${disk}" \
			unit s \
			p \
			2>/dev/null \
			| grep "^ 1" 2>/dev/null \
			| grep " 2048s" 2>/dev/null \
			| grep " 1049599s" 2>/dev/null \
			| grep " 1047552s" 2>/dev/null \
			| grep "ext4" 2>/dev/null \
			| grep "boot" 2>/dev/null \
			| grep "boot" 2>/dev/null \
			| wc -l 2>/dev/null
	)
	if [ "x${part1}" != "x1" ]; then
		# 構成が正しくない
		return 1
	fi

	local part2=$(
		/sbin/parted -s \
			"${disk}" \
			unit s \
			p \
			2>/dev/null \
			| grep "^ 2" 2>/dev/null \
			| grep " 1049600s" 2>/dev/null \
			| grep " 5243903s" 2>/dev/null \
			| grep " 4194304s" 2>/dev/null \
			| grep "rootfs" 2>/dev/null \
			| grep "raid" 2>/dev/null \
			| wc -l 2>/dev/null
	)
	if [ "x${part2}" != "x1" ]; then
		# 構成が正しくない
		return 1
	fi

	local part3=$(
		/sbin/parted -s \
			"${disk}" \
			unit s \
			p \
			2>/dev/null \
			| grep "^ 3" 2>/dev/null \
			| grep " 5243904s" 2>/dev/null \
			| grep " 7341055s" 2>/dev/null \
			| grep " 2097152s" 2>/dev/null \
			| grep "linux-swap(v1)" 2>/dev/null \
			| grep "swap" 2>/dev/null \
			| wc -l 2>/dev/null
	)
	if [ "x${part3}" != "x1" ]; then
		# 構成が正しくない
		return 1
	fi

	local part4=$(
		/sbin/parted -s \
			"${disk}" \
			unit s \
			p \
			2>/dev/null \
			| grep "^ 4" 2>/dev/null \
			| grep " 7341056s" 2>/dev/null \
			| grep " 7603199s" 2>/dev/null \
			| grep " 262144s" 2>/dev/null \
			| grep "ext4" 2>/dev/null \
			| grep "disklabel" 2>/dev/null \
			| wc -l 2>/dev/null
	)
	if [ "x${part4}" != "x1" ]; then
		# 構成が正しくない
		return 1
	fi

	local part5=$(
		/sbin/parted -s \
			"${disk}" \
			unit s \
			p \
			2>/dev/null \
			| grep "^ 5" 2>/dev/null \
			| grep " 7603200s" 2>/dev/null \
			| grep " 9700351s" 2>/dev/null \
			| grep " 2097152s" 2>/dev/null \
			| grep "data" 2>/dev/null \
			| grep "raid" 2>/dev/null \
			| wc -l 2>/dev/null
	)
	if [ "x${part5}" != "x1" ]; then
		# 構成が正しくない
		return 1
	fi

	local part6=$(
		/sbin/parted -s \
			"${disk}" \
			unit s \
			p \
			2>/dev/null \
			| grep "^ 6" 2>/dev/null \
			| grep " 9700352s" 2>/dev/null \
			| grep "sataraid1" 2>/dev/null \
			| grep "raid" 2>/dev/null \
			| wc -l 2>/dev/null
	)
	if [ "x${part6}" != "x1" ]; then
		# 構成が正しくない
		return 1
	fi

	return 0
}


###############################################################################
# power.log を保存する
# 入力: $1=元ディスクデバイス名
# 出力: 0=保存成功  1=保存失敗
###############################################################################
__initialization_save_power_log()
{
	local srcdisk="${1}"

	if [ ! -e "${D_MP_ILOG}" ]; then
		/bin/mkdir -p \
			"${D_MP_ILOG}" \
			1>/dev/null 2>/dev/null
	fi

	/sbin/mdadm \
		--assemble /dev/md5 \
		--no-degraded \
		"${srcdisk}p5" \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	/sbin/mdadm \
		--misc /dev/md5 \
		--readonly \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		/sbin/mdadm \
			--stop /dev/md5 \
			1>/dev/null 2>/dev/null
		return 1
	fi

	/sbin/mdadm \
		--run /dev/md5 \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		/sbin/mdadm \
			--stop /dev/md5 \
			1>/dev/null 2>/dev/null
		return 1
	fi

	/bin/mount \
		-t ext4 \
		-o ro \
		/dev/md5 \
		"${D_MP_ILOG}" \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		/sbin/mdadm \
			--stop /dev/md5 \
			1>/dev/null 2>/dev/null
		return 1
	fi

	cp -f \
		"${D_MP_ILOG}/log/power.log" \
		"${F_TMP_POWER_LOG}" \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		/bin/umount "${D_MP_ILOG}" \
			1>/dev/null 2>/dev/null
		/sbin/mdadm \
			--stop /dev/md5 \
			1>/dev/null 2>/dev/null
		return 1
	fi

	/bin/umount \
		"${D_MP_ILOG}" \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		/sbin/mdadm \
			--stop /dev/md5 \
			1>/dev/null 2>/dev/null
		return 1
	fi

	/sbin/mdadm \
		--stop /dev/md5 \
		1>/dev/null 2>/dev/null
	if [ ${?} != 0 ]; then
		return 1
	fi

	return 0
}


###############################################################################
# 内蔵ディスクのパーティションを構成する
# 入力: $1=比較元ディスクデバイス名, $2=ディスク数
# 出力: 0=異常なし  1〜63=異常あり(異常ディスクのビットが 1)
###############################################################################
__initialization_create_internal_disk_partitions()
{
	local srcdisk="${1}"
	local disk_count="${2}"
	local retval=0
	local i=1

	while [ ${i} -le ${disk_count} ];
	do
		if [ "x${srcdisk}" = "x/dev/satad${i}" ]; then
			# 元ディスクなので次へ
			i=$((${i} + 1))
			continue
		fi

		__initialization_is_valid_partition_table "/dev/satad${i}"
		if [ ${?} = 0 ]; then
			# パーティションが正しいので次へ
			i=$((${i} + 1))
			continue
		fi

		local max_sector=$(
			__initialization_get_sector_from_disk "/dev/satad${i}"
		)
		local max_part=$((${max_sector} - 64 - 1))

		/sbin/parted -s "/dev/satad${i}" \
			mktable gpt \
			unit s \
			mkpart boot      ext4          2048 1049599 \
			mkpart rootfs    ext4       1049600 5243903 \
			mkpart swap      linux-swap 5243904 7341055 \
			mkpart disklabel ext4       7341056 7603199 \
			mkpart data      ext4       7603200 9700351 \
			mkpart sataraid1 ext4       9700352 ${max_part} \
			set 1 boot on \
			set 2 raid on \
			set 5 raid on \
			set 6 raid on \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# パーティションを作成できない
			retval=$((${retval} + (1 << (${i} - 1))))
		fi
		/bin/sleep 1

		i=$((${i} + 1))
	done

	# デバイスファイルの再構成
	. /scripts/landisk/probe
	landisk_probe

	return ${retval}
}


###############################################################################
# errorcount 値を取得する
# 入力: $1=ディスクデバイス名
# 表示: エラーカウント数。取得できない場合は 0 とする
###############################################################################
__initialization_get_errorcount()
{
	local disk="${1}"
	local errorcount=0

	if [ ! -e "${disk}" ]; then
		# デバイスが存在しない(通常ここにはこない)
		echo "${errorcount}"
		return 1
	fi

	if [ ! -e "${disk}p4" ]; then
		# パーティションが存在しない(通常ここにはこない)
		echo "${errorcount}"
		return 1
	fi

	if [ ! -e "${D_MP_DISKLABEL}" ]; then
		/bin/mkdir -p \
			"${D_MP_DISKLABEL}" \
			1>/dev/null 2>/dev/null
	fi

	/bin/mount -t ext4 \
		"${disk}p4" \
		"${D_MP_DISKLABEL}" \
		1>/dev/null 2>/dev/null
	if [ ${?} != 0 ]; then
		# マウントできない
		echo "${errorcount}"
		return 1
	fi

	if [ ! -f "${D_DISKLABEL}/errorcount" ]; then
		# errorcount が存在しない(通常ここにはこない)
		/bin/umount \
			"${D_MP_DISKLABEL}" \
			1>/dev/null 2>/dev/null
		echo "${errorcount}"
		return 1
	fi

	errorcount=$(
		/bin/cat "${D_DISKLABEL}/errorcount" \
			2>/dev/null
	)

	/bin/umount \
		"${D_MP_DISKLABEL}" \
		1>/dev/null 2>/dev/null

	echo "${errorcount}"
	return 0
}


###############################################################################
# 第 1 パーティションを使用可能にする
# 入力: $1=元ディスクデバイス名, $2=ゼロクリアするかどうか, $3=ディスク数
# 出力: 0=異常なし  1〜63=異常あり(異常ディスクのビットが 1)
###############################################################################
__initialization_create_volume1()
{
	local srcdisk="${1}"
	local zeroclear="${2}"
	local disk_count="${3}"
	local retval=0
	local i=1

	if [ ! -e "${D_MP_PART1}" ]; then
		/bin/mkdir -p \
			"${D_MP_PART1}" \
			1>/dev/null 2>/dev/null
	fi

	while [ ${i} -le ${disk_count} ];
	do
		if [ "x${srcdisk}" = "x/dev/satad${i}" ]; then
			# 元ディスクなので次へ
			i=$((${i} + 1))
			continue
		fi

		if [ "x${zeroclear}" = "x1" ]; then
			# ゼロクリアする
			echo "###### zero-clear start satad${i}p1 ######"
			/bin/dd if="/dev/zero" of="/dev/satad${i}p1" bs=1M \
				1>/dev/null 2>/dev/null
			echo "###### zero-clear complete satad${i}p1 ######"
		fi

		/sbin/mkfs.ext4 \
			-E lazy_itable_init=0 \
			"/dev/satad${i}p1" \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# フォーマットできない
			retval=$((${retval} + (1 << (${i} - 1))))

			i=$((${i} + 1))
			continue
		fi

		/bin/mount -t ext4 \
			"/dev/satad${i}p1" \
			"${D_MP_PART1}" \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# マウントできない
			retval=$((${retval} + (1 << (${i} - 1))))

			i=$((${i} + 1))
			continue
		fi

		cp -r ${D_MP_INIT}/* "${D_MP_PART1}" 1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# コピーできない
			/bin/umount "${D_MP_PART1}" 1>/dev/null 2>/dev/null
			retval=$((${retval} + (1 << (${i} - 1))))

			i=$((${i} + 1))
			continue
		fi

		# 今動作させているスクリプトはコピーしない
		if [ -e "${D_MP_PART1}/landisk/script.sh" ]; then
			rm -f \
				"${D_MP_PART1}/landisk/script.sh" \
				1>/dev/null 2>/dev/null
		fi

		/bin/umount "${D_MP_PART1}" 1>/dev/null 2>/dev/null

		i=$((${i} + 1))
	done

	return ${retval}
}


###############################################################################
# 第 2 パーティションを使用可能にする
# 入力: $1=ゼロクリアするかどうか, $2=ディスク数
# 出力: 0=異常なし  1=異常あり
###############################################################################
__initialization_create_volume2()
{
	local zeroclear="${1}"
	local disk_count="${2}"
	local i=1
	local disk_list=""

	# 前処理
	while [ ${i} -le ${disk_count} ];
	do
		/sbin/mdadm --zero-superblock --force \
			"/dev/satad${i}p2" \
			1>/dev/null 2>/dev/null

		i=$((${i} + 1))
	done

	# ディスク一覧生成
	for i in 1 2 3
	do
		if [ ${i} -le ${disk_count} ]; then
			disk_list="${disk_list} /dev/satad${i}p2"
		else
			disk_list="${disk_list} missing"
		fi
	done

	# RAID 構成
	yes y | /sbin/mdadm \
		--create "/dev/md2" \
		--level=raid1 \
		--raid-devices=3 \
		${disk_list} \
		1>/dev/null 2>/dev/null
	if [ ! -b "/dev/md2" ]; then
		# RAID を構成できない
		return 1
	fi

	if [ "x${zeroclear}" = "x1" ]; then
		# ゼロクリアする
		echo "###### zero-clear start md2 ######"
		/bin/dd if="/dev/zero" of="/dev/md2" bs=1M \
			1>/dev/null 2>/dev/null
		echo "###### zero-clear complete md2 ######"
	fi

	# フォーマット
	/sbin/mkfs.ext4 \
		-E lazy_itable_init=0 \
		"/dev/md2" \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		# フォーマットできない
		return 1
	fi

	return 0
}


###############################################################################
# 第 3 パーティションを使用可能にする
# 入力: $1=元ディスクデバイス名, $2=ゼロクリアするかどうか, $3=ディスク数
# 出力: 0=異常なし  1〜63=異常あり(異常ディスクのビットが 1)
###############################################################################
__initialization_create_volume3()
{
	local srcdisk="${1}"
	local zeroclear="${2}"
	local disk_count="${3}"
	local retval=0
	local i=1

	while [ ${i} -le ${disk_count} ];
	do
		if [ "x${srcdisk}" = "x/dev/satad${i}" ]; then
			# 元ディスクなので次へ
			i=$((${i} + 1))
			continue
		fi

		if [ "x${zeroclear}" = "x1" ]; then
			# ゼロクリアする
			echo "###### zero-clear start satad${i}p3 ######"
			/bin/dd if="/dev/zero" of="/dev/satad${i}p3" bs=1M \
				1>/dev/null 2>/dev/null
			echo "###### zero-clear complete satad${i}p3 ######"
		fi

		/sbin/mkswap \
			"/dev/satad${i}p3" \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# フォーマットできない
			retval=$((${retval} + (1 << (${i} - 1))))
		fi

		i=$((${i} + 1))
	done

	return ${retval}
}


###############################################################################
# 第 4 パーティションを使用可能にする
# 入力: $1=元ディスクデバイス名, $2=ゼロクリアするかどうか,
#       $3=モデル名文字列, $4=ディスク数
# 出力: 0=異常なし  1〜63=異常あり(異常ディスクのビットが 1)
###############################################################################
__initialization_create_volume4()
{
	local srcdisk="${1}"
	local zeroclear="${2}"
	local disk_count="${4}"

	local errorcount=0
	local magic="HDLAA"
	local mindisk="1"
	local model="${3}"
	local raidmode="vfsext"
	local usage="RAID"
	local version="3"

	local retval=0
	local i=1

	if [ ! -e "${D_MP_DISKLABEL}" ]; then
		/bin/mkdir -p \
			"${D_MP_DISKLABEL}" \
			1>/dev/null 2>/dev/null
	fi

	while [ ${i} -le ${disk_count} ];
	do
		# errorcount を取得する
		errorcount=$(__initialization_get_errorcount "/dev/satad${i}")

		if [ "x${srcdisk}" = "x/dev/satad${i}" ]; then
			# 元ディスク
			/bin/mount -t ext4 \
				"/dev/satad${i}p4" \
				"${D_MP_DISKLABEL}" \
				1>/dev/null 2>/dev/null
			if [ ${?} -ne 0 ]; then
				# マウントできない(通常ここにはこない)
				retval=$((${retval} + (1 << (${i} - 1))))
			fi

			# データを書き込む
			echo "${mindisk}" > "${D_DISKLABEL}/mindisk"
			echo "${raidmode}" > "${D_DISKLABEL}/raidmode"
			echo "${usage}" > "${D_DISKLABEL}/usage"
			/bin/cat \
				"${D_DISKLABEL}/magic" \
				"${D_DISKLABEL}/mindisk" \
				"${D_DISKLABEL}/model" \
				"${D_DISKLABEL}/raidmode" \
				"${D_DISKLABEL}/usage" \
				"${D_DISKLABEL}/version" \
				| md5sum > "${D_DISKLABEL}/md5sum"

			/bin/umount \
				"${D_MP_DISKLABEL}" \
				1>/dev/null 2>/dev/null

			i=$((${i} + 1))
			continue
		fi

		if [ "x${zeroclear}" = "x1" ]; then
			# ゼロクリアする
			echo "###### zero-clear start satad${i}p4 ######"
			/bin/dd if="/dev/zero" of="/dev/satad${i}p4" bs=1M \
				1>/dev/null 2>/dev/null
			echo "###### zero-clear complete satad${i}p4 ######"
		fi

		/sbin/mkfs.ext4 \
			-E lazy_itable_init=0 \
			"/dev/satad${i}p4" \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# フォーマットできない
			retval=$((${retval} + (1 << (${i} - 1))))

			i=$((${i} + 1))
			continue
		fi

		/bin/mount -t ext4 \
			"/dev/satad${i}p4" \
			"${D_MP_DISKLABEL}" \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# マウントできない
			retval=$((${retval} + (1 << (${i} - 1))))

			i=$((${i} + 1))
			continue
		fi

		/bin/mkdir -p "${D_DISKLABEL}" 1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# ディレクトリーを作成できない
			retval=$((${retval} + (1 << (${i} - 1))))

			i=$((${i} + 1))
			continue
		fi

		# データを書き込む
		echo "${errorcount}" > "${D_DISKLABEL}/errorcount"
		echo "${magic}" > "${D_DISKLABEL}/magic"
		echo "${mindisk}" > "${D_DISKLABEL}/mindisk"
		echo "${model}" > "${D_DISKLABEL}/model"
		echo "${raidmode}" > "${D_DISKLABEL}/raidmode"
		echo "${usage}" > "${D_DISKLABEL}/usage"
		echo "${version}" > "${D_DISKLABEL}/version"
		/bin/cat \
			"${D_DISKLABEL}/magic" \
			"${D_DISKLABEL}/mindisk" \
			"${D_DISKLABEL}/model" \
			"${D_DISKLABEL}/raidmode" \
			"${D_DISKLABEL}/usage" \
			"${D_DISKLABEL}/version" \
			| md5sum > "${D_DISKLABEL}/md5sum"

		chown -R 0:0 "${D_DISKLABEL}" 1>/dev/null 2>/dev/null
		chmod 755 "${D_DISKLABEL}" 1>/dev/null 2>/dev/null
		chmod 644 ${D_DISKLABEL}/* 1>/dev/null 2>/dev/null

		/bin/umount "${D_MP_DISKLABEL}" 1>/dev/null 2>/dev/null

		i=$((${i} + 1))
	done

	return ${retval}
}


###############################################################################
# 第 5 パーティションを使用可能にする
# 入力: $1=ゼロクリアするかどうか, $2=ディスク数
# 出力: 0=異常なし  1=異常あり
###############################################################################
__initialization_create_volume5()
{
	local zeroclear="${1}"
	local disk_count="${2}"
	local i=1
	local disk_list=""

	# 前処理
	while [ ${i} -le ${disk_count} ];
	do
		/sbin/mdadm --zero-superblock --force \
			"/dev/satad${i}p5" \
			1>/dev/null 2>/dev/null

		i=$((${i} + 1))
	done

	# ディスク一覧生成
	for i in 1 2 3
	do
		if [ ${i} -le ${disk_count} ]; then
			disk_list="${disk_list} /dev/satad${i}p5"
		else
			disk_list="${disk_list} missing"
		fi
	done

	# RAID 構成
	yes y | /sbin/mdadm \
		--create "/dev/md5" \
		--level=raid1 \
		--raid-devices=3 \
		${disk_list} \
		1>/dev/null 2>/dev/null
	if [ ! -b "/dev/md5" ]; then
		# RAID を構成できない
		return 1
	fi

	if [ "x${zeroclear}" = "x1" ]; then
		# ゼロクリアする
		echo "###### zero-clear start md5 ######"
		/bin/dd if="/dev/zero" of="/dev/md5" bs=1M \
			1>/dev/null 2>/dev/null
		echo "###### zero-clear complete md5 ######"
	fi

	# フォーマット
	/sbin/mkfs.ext4 \
		-E lazy_itable_init=0 \
		"/dev/md5" \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		# フォーマットできない
		return 1
	fi

	return 0
}


###############################################################################
# sataraid1 構成デバイスリストを取得する
# 入力:
#   arg1: ディスク数
# 表示:
#   構成デバイスリスト(例: /dev/sarad1p6 missing)
###############################################################################
__initialization_get_device_list_p6()
{
	local disk_count="${1}"
	local device_list=

	for i in 1 2
	do
		if [ ${i} -le ${disk_count} ]; then
			device_list="${device_list} /dev/satad${i}p6"
		else
			device_list="${device_list} missing"
		fi

		i=$((${i} + 1))
	done

	echo "${device_list}"
	return 0
}


###############################################################################
# sataraid1 をフォーマットする
# 入力:
#   arg1: ディスク数
# 出力:
#   0: 異常なし
#   1: 異常あり
###############################################################################
__initialization_format_sataraid1()
{
	local disk_count="${1}"

	if [ ${disk_count} -eq 1 ]; then
		__initialization_format_sataraid1_md13 ${disk_count}
	else
		__initialization_format_sataraid1_vfsext ${disk_count}
	fi

	return ${?}
}


###############################################################################
# sataraid1 をフォーマットする(1 ドライブ)
# 入力:
#   arg1: ディスク数
# 出力:
#   0: 異常なし
#   1: 異常あり
###############################################################################
__initialization_format_sataraid1_md13()
{
	local disk_count="${1}"

	# md13 の構築
	echo "##### create md13 ######"
	local device_list=$(__initialization_get_device_list_p6 ${disk_count})

	yes y | /sbin/mdadm \
			--create "/dev/md13" \
			--level=raid1 \
			--raid-devices=3 \
			${device_list} \
			"missing" \
			1>/dev/null 2>/dev/null
	if [ ! -b "/dev/md13" ]; then
		# md13 を構築できない
		return 1
	fi

	# md13 のフォーマット
	echo "##### format md13 ######"
	/sbin/mkfs.xfs \
		-f \
		/dev/md13 \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		# md13 をフォーマットできない
		/sbin/mdadm --stop /dev/md13 1>/dev/null 2>/dev/null
		return 1
	fi

	return 0
}


###############################################################################
# sataraid1 をフォーマットする(2 ドライブ)
# 入力:
#   arg1: ディスク数
# 出力:
#   0: 異常なし
#   1: 異常あり
###############################################################################
__initialization_format_sataraid1_vfsext()
{
	local disk_count="${1}"

	# vfsext の構築
	echo "##### create vfsext ######"

	local i=1

	while [ ${i} -le ${disk_count} ];
	do
		echo "##### format satad${i}p6 ######"
		/sbin/mkfs.xfs \
			-f \
			"/dev/satad${i}p6" \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# フォーマットできない
			return 1
		fi

		i=$((${i} + 1))
	done

	return 0
}


###############################################################################
# 第 6 パーティションを使用可能にする
# 入力: $1=ゼロクリアするかどうか, $2=ディスク数
# 出力: 0=異常なし  1=異常あり
###############################################################################
__initialization_create_volume6()
{
	local zeroclear="${1}"
	local disk_count="${2}"
	local i=1

	# 前処理
	while [ ${i} -le ${disk_count} ];
	do
		/sbin/mdadm --zero-superblock --force \
			"/dev/satad${i}p6" \
			1>/dev/null 2>/dev/null

		i=$((${i} + 1))
	done

	if [ "x${zeroclear}" = "x1" ]; then
		# ゼロクリアする
		i=1
		while [ ${i} -le ${disk_count} ];
		do
			echo "###### zero-clear start satad${i}p6 ######"
			/bin/dd if="/dev/zero" of="/dev/satad${i}p6" bs=1M \
				1>/dev/null 2>/dev/null
			echo "###### zero-clear complete satad${i}p6 ######"

			i=$((${i} + 1))
		done
	fi

	# フォーマット
	__initialization_format_sataraid1 ${disk_count}
	if [ ${?} -ne 0 ]; then
		# フォーマットできない
		return 1
	fi

	return 0
}


###############################################################################
# sataraid1 をマウントする
# 入力:
#   arg1: ディスク数
# 出力:
#   0: 正常終了
#   1: 異常終了
###############################################################################
__initialization_mount_sataraid1()
{
	local disk_count="${1}"

	if [ ${disk_count} -eq 1 ]; then
		__initialization_mount_sataraid1_md13 ${disk_count}
	else
		__initialization_mount_sataraid1_vfsext ${disk_count}
	fi

	return ${?}
}


###############################################################################
# sataraid1 をマウントする(RAID ドライブ)
# 入力:
#   arg1: ディスク数
# 出力:
#   0: 正常終了
#   1: 異常終了
###############################################################################
__initialization_mount_sataraid1_md13()
{
	local disk_count="${1}"

	if [ ! -b "/dev/md13" ]; then
		local device_list=$(
			__initialization_get_device_list_p6 ${disk_count}
		)

		/sbin/mdadm \
			--assemble "/dev/md13" \
			--run \
			${device_list} \
			"missing" \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne  0 ]; then
			# マウントできない
			echo "Failed to assemble md13."
			return 1
		fi
	fi

	mount -t xfs "/dev/md13" "${D_MP_ISATARAID1}" 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		# マウントできない
		echo "Failed to mount satad1p6."
		/sbin/mdadm --stop "/dev/md13" 1>/dev/null 2>/dev/null
		return 1
	fi

	return 0
}


###############################################################################
# sataraid1 をマウントする(拡張ボリューム)
# 入力:
#   arg1: ディスク数
# 出力:
#   0: 正常終了
#   1: 異常終了
###############################################################################
__initialization_mount_sataraid1_vfsext()
{
	local disk_count="${1}"
	local i=1

	while [ ${i} -le ${disk_count} ];
	do
		mount -t xfs \
		      "/dev/satad${i}p6" \
		      "${D_MP_IVFSEXT}/sata${i}" \
		      1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# マウントできない
			echo "Failed to mount satad${i}p6."
			__initialization_umount_sataraid1_vfsext ${disk_count}
			return 1
		fi

		i=$((${i} + 1))
	done

	mount -t vfsext \
	      -o "${D_MP_IVFSEXT}/sata1:${D_MP_IVFSEXT}/sata2" \
	      none \
	      "${D_MP_ISATARAID1}" \
	      1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		# マウントできない
		__initialization_umount_sataraid1_vfsext ${disk_count}
		return 1
	fi

	return 0
}


###############################################################################
# sataraid1 をアンマウントする
# 入力:
#   arg1: ディスク数
# 出力:
#   0: 正常終了
#   1: 異常終了
###############################################################################
__initialization_umount_sataraid1()
{
	local disk_count="${1}"

	umount /mnt/sataraid1 1>/dev/null 2>/dev/null

	if [ ${disk_count} -eq 1 ]; then
		__initialization_umount_sataraid1_md13 ${disk_count}
	else
		__initialization_umount_sataraid1_vfsext ${disk_count}
	fi

	return ${?}
}


###############################################################################
# sataraid1 をアンマウントする(RAID ドライブ)
# 入力:
#   arg1: ディスク数
# 出力:
#   0: 正常終了
#   1: 異常終了
###############################################################################
__initialization_umount_sataraid1_md13()
{
	local disk_count="${1}"

	/sbin/mdadm --stop /dev/md13 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		return 1
		fi

	return 0
}


###############################################################################
# sataraid1 をアンマウントする(拡張ボリューム)
# 入力:
#   arg1: ディスク数
# 出力:
#   0: 正常終了
#   1: 異常終了
###############################################################################
__initialization_umount_sataraid1_vfsext()
{
	local disk_count="${1}"
	local i=1

	while [ ${i} -le ${disk_count} ];
	do
		umount "${D_MP_IVFSEXT}/sata${i}" 1>/dev/null 2>/dev/null

		i=$((${i} + 1))
	done

	return 0
}


###############################################################################
# 拡張ボリュームの構成ボリュームのマウントポイントを作成する
# 入力:
#   arg1: ディスク数
###############################################################################
__initialization_mkdir_vfsext_components()
{
	local disk_count="${1}"
	local i=1

	while [ ${i} -le ${disk_count} ];
	do
		mkdir -p "${D_MP_IVFSEXT}/sata${i}" 1>/dev/null 2>/dev/null
		i=$((${i} + 1))
	done
}


###############################################################################
# ファイルをインストールする
# 入力: $1=ディスク数
# 出力: 0=異常なし  1=異常あり
###############################################################################
__initialization_install_files()
{
	local disk_count="${1}"
	local i=1
	local opt_vfs=""

	# md2 マウントポイントを生成
	if [ ! -e "${D_MP_IROOT}" ]; then
		/bin/mkdir -p "${D_MP_IROOT}" 1>/dev/null 2>/dev/null
	fi

	# md2 をマウント
	/bin/mount -t ext4 "/dev/md2" "${D_MP_IROOT}" 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		# マウントできない
		echo "Failed to mount md2."
		return 1
	fi

	# その他マウントポイントを生成
	if [ ! -e "${D_MP_IDATA}" ]; then
		/bin/mkdir -p "${D_MP_IDATA}" 1>/dev/null 2>/dev/null
	fi
	if [ ! -e "${D_MP_IVFSEXT}" ]; then
		/bin/mkdir -p "${D_MP_IVFSEXT}" 1>/dev/null 2>/dev/null
		__initialization_mkdir_vfsext_components ${disk_count}
	fi
	if [ ! -e "${D_MP_ISATARAID1}" ]; then
		mkdir -p "${D_MP_ISATARAID1}" 1>/dev/null 2>/dev/null
	fi

	# md5 をマウント
	/bin/mount -t ext4 "/dev/md5" "${D_MP_IDATA}" 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		# マウントできない
		echo "Failed to mount md5."
		/bin/umount "${D_MP_IROOT}" 1>/dev/null 2>/dev/null
		return 1
	fi

	# sataraid1 をマウント
	__initialization_mount_sataraid1 ${disk_count}
	if [ ${?} -ne 0 ]; then
		# マウントできない
		echo "Failed to mount sataraid1."
		__initialization_umount_sataraid1 ${disk_count}
		/bin/umount "${D_MP_IDATA}" 1>/dev/null 2>/dev/null
		/bin/umount "${D_MP_IROOT}" 1>/dev/null 2>/dev/null
		return 1
	fi

	# ファイルを展開
	tar zxp \
		-f "${D_INIT_FW}/rootfs.tgz" \
		-C "${D_MP_IROOT}" \
		1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		# 展開できない
		echo "Failed to untar rootfs."
		__initialization_umount_sataraid1 ${disk_count}
		umount "${D_MP_IDATA}" 1>/dev/null 2>/dev/null
		umount "${D_MP_IROOT}" 1>/dev/null 2>/dev/null
		return 1
	fi

	# power.log を書き戻す
	if [ -e "${F_TMP_POWER_LOG}" ]; then
		mv -f \
			"${F_TMP_POWER_LOG}" \
			"${D_MP_IDATA}/log/power.log" \
			1>/dev/null 2>/dev/null
	fi

	# rc.local は新しいものに更新する
	if [ -e "${D_INIT_FW}/rc.local" ]; then
		mv -f \
			"${D_INIT_FW}/rc.local" \
			"${D_MP_IROOT}/etc" \
			1>/dev/null 2>/dev/null
	fi

	/bin/umount "${D_MP_ISATARAID1}" 1>/dev/null 2>/dev/null
	__initialization_umount_sataraid1 ${disk_count}
	/bin/umount "${D_MP_IDATA}" 1>/dev/null 2>/dev/null
	/bin/umount "${D_MP_IROOT}" 1>/dev/null 2>/dev/null

	return 0
}


###############################################################################
# 異なる場合のみ上書きする
# 入力:
#   arg1: 比較・コピー元ファイルパス
#   arg2: 比較・コピー先ファイルパス
# 出力:
#   0: 異常なし
#   1: 異常あり
###############################################################################
__initialization_update_file()
{
	local src="${1}"
	local dst="${2}"

	diff -q "${src}" "${dst}" 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		cp -f "${src}" "${dst}" 1>/dev/null 2>/dev/null
	fi

	return 0;
}


###############################################################################
# 起動に必要なファイルを書き込む
# 入力: $1=ディスク数
# 出力: 0=異常なし  1=異常あり
###############################################################################
__initialization_write_files_for_boot()
{
	local disk_count="${1}"
	local i=1

	# md2 マウントポイントを生成
	if [ ! -e "${D_MP_IROOT}" ]; then
		/bin/mkdir -p "${D_MP_IROOT}" 1>/dev/null 2>/dev/null
	fi

	# md2 をマウント
	/bin/mount -t ext4 "/dev/md2" "${D_MP_IROOT}" 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		# マウントできない
		echo "Failed to mount md2."
		return 1
	fi

	local file_vmlinuz=$(
		find "${D_MP_IBOOT}" -maxdepth 1 -name 'vmlinuz-*'
	)
	local file_initrd=$(
		find "${D_MP_IBOOT}" -maxdepth 1 -name 'initrd.img-*'
	)

	if [ "x${file_vmlinuz}" = "x" ]; then
		# vmlinuz がない
		umount "${D_MP_IROOT}" 1>/dev/null 2>/dev/null
		echo "vmlinuz not found."
		return 1
	fi
	if [ "x${file_initrd}" = "x" ]; then
		# initrd.img がない
		umount "${D_MP_IROOT}" 1>/dev/null 2>/dev/null
		echo "initrd.img not found."
		return 1
	fi

	# 第 1 パーティション用のデータを生成する
	cp \
		-a \
		"${file_vmlinuz}" \
		"${D_MP_IBOOT}/zImage" \
		1>/dev/null 2>/dev/null
	/sbin/mkimage \
		-A arm \
		-T ramdisk \
		-C none \
		-n HDL-AA \
		-d "${file_initrd}" \
		"${D_MP_IBOOT}/uInitrd" \
		1>/dev/null 2>/dev/null

	# 第 1 パーティション用のデータを退避する
	cp -a "${D_MP_IBOOT}" /tmp 1>/dev/null 2>/dev/null

	while [ ${i} -le ${disk_count} ];
	do
		# p1 マウントする
		/bin/mount -t ext4 \
			"/dev/satad${i}p1" \
			"${D_MP_IBOOT}" \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
			# マウントできない
			echo "Failed to mount satad${i}p1."
			return 1
		fi

		# p1 用データを書き込む
		cp -a /tmp/boot/* "${D_MP_IBOOT}" 1>/dev/null 2>/dev/null

		# p1 アンマウントする
		/bin/umount "${D_MP_IBOOT}" 1>/dev/null 2>/dev/null

		i=$((${i} + 1))
	done

	# md2 アンマウントする
	/bin/umount "${D_MP_IROOT}" 1>/dev/null 2>/dev/null

	return 0
}


###############################################################################
# RTC アラーム設定を無効にする
###############################################################################
__initialization_disable_rtc_alarm()
{
	if [ ! -f /sys/class/rtc/rtc0/wakealarm ]; then
		return 1
	fi

	echo '' > /sys/class/rtc/rtc0/wakealarm
	return 0
}


###############################################################################
# AC 電源連動設定を無効にする
# 出力:
#   0: 異常なし
#   1: 異常あり
###############################################################################
__initialization_disable_ac_auto_power_on()
{
	# AC auto power on setting (offset 29 (0x1d))
	/sbin/i2cset -y 0 0x50 0x1d 0x00 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	return 0
}


###############################################################################
# WoL 設定を有効にする
###############################################################################
__initialization_enable_wol()
{
	# eth0
	__initialization_enable_eth_wol eth0
	if [ ${?} != 0 ]; then
		return 1
	fi

	return 0
}


###############################################################################
# 指定 NIC の WoL 設定を有効にする
###############################################################################
__initialization_enable_eth_wol()
{
	local iface="${1}"

	# WOL setting (offset 30 (0x1e))
	/sbin/i2cset -y 0 0x50 0x1e 0x01 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	/sbin/ethtool -s "${iface}" wol g 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	return 0
}


###############################################################################
# ブザーを有効にする
# 出力:
#   0: 異常なし
#   1: 異常あり
###############################################################################
__initialization_enable_buzzer()
{
	# buzzer setting (offset 31 (0x1f))
	/sbin/i2cset -y 0 0x50 0x1f 0x01 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	return 0
}


###############################################################################
# LED 輝度を設定する
# 出力:
#   0: 異常なし
#   1: 異常あり
###############################################################################
__initialize_set_led_brightness()
{
	# LED brightness setting (offset 32 (0x20))
	/sbin/i2cset -y 0 0x50 0x20 0xff 1>/dev/null 2>/dev/null
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	if [ ! -e "${F_PROC_BTNLED}" ]; then
		return 1
	fi

	echo "led_brightness high" > "${F_PROC_BTNLED}"

	return 0
}


###############################################################################
# 初期化完了後の処理
###############################################################################
__initialization_finishing()
{
	# 初期化 OK のフラグを立てる
	touch "${F_RESET_OK}"

	# RAID の動作を停止する
	/sbin/mdadm --stop /dev/md13 1>/dev/null 2>/dev/null
	/sbin/mdadm --stop /dev/md6 1>/dev/null 2>/dev/null
	/sbin/mdadm --stop /dev/md5 1>/dev/null 2>/dev/null
	/sbin/mdadm --stop /dev/md2 1>/dev/null 2>/dev/null

	return 0
}


###############################################################################
# main
###############################################################################

SRCDISK="${1}"
ZEROCLEAR="${2}"

# 使用するツールをコピーする
__initialization_provide_mandatory_tools
__initialization_set_status_led "green" "blink"

# ディスク確認
echo "### INITIALIZATION: check internal HDDs #################################"

echo "###### read source disklabel ######"
local model_name=$(__initialization_get_model "${SRCDISK}")
local model_disk_count=0
local max_sector=0
if [ "x${model_name}" = "x" -o "x${model_name}" = "xNG" ]; then
	# モデル名を取得できない
	__initialization_error "Failed to get model."
fi

model_disk_count=$(
	echo "${model_name}" | \
		sed -e 's/HDL\([0-9]*\)-AA[0-9]*W*/\1/' \
		2>/dev/null
)
if [ "x${model_disk_count}" = "x" ]; then
	# HDL-AA 用
	model_disk_count=1
fi
echo "OK"

echo "###### check internal HDD existence ######"
__initialization_check_internal_disk_existence "${model_disk_count}"
if [ ${?} -ne 0 ]; then
	# ディスクがない
	__initialization_error "Internal disk(s) not found."
fi
echo "OK"

echo "###### check internal HDD size ######"
max_sector=$(
	__initialization_check_internal_disk_size \
		"${SRCDISK}" \
		"${model_disk_count}"
)
if [ "x${max_sector}" = "x" -o "x${max_sector}" = "x0" ]; then
	# ディスク容量不足
	__initialization_error "The size of internal disk is not valid."	
fi
echo "OK"

echo "###### save power.log ######"
__initialization_save_power_log "${SRCDISK}"
if [ ${?} -ne 0 ]; then
	# 保存できない
	__initialization_error "Failed to save power.log."
fi
echo "OK"


# パーティション作成
echo "### INITIALIZATION: create internal volumes #############################"

echo "###### partition internal disks ######"
__initialization_create_internal_disk_partitions \
	"${SRCDISK}" \
	"${model_disk_count}"
if [ ${?} -ne 0 ]; then
	# パーティションを作成できない
	__initialization_error "Failed to create partition of internal disk(s)."
fi
echo "OK"

echo "###### create volume 1 ######"
__initialization_create_volume1 \
	"${SRCDISK}" \
	"${ZEROCLEAR}" \
	"${model_disk_count}"
if [ ${?} -ne 0 ]; then
	# 第 1 パーティションを構成できない
	__initialization_error "Failed to create volume 1."
fi
echo "OK"

echo "###### create volume 2 ######"
__initialization_create_volume2 \
	"${ZEROCLEAR}" \
	"${model_disk_count}"
if [ ${?} -ne 0 ]; then
	# 第 2 パーティションを構成できない
	__initialization_error "Failed to create volume 2."
fi
echo "OK"

echo "###### create volume 3 ######"
__initialization_create_volume3 \
	"${SRCDISK}" \
	"${ZEROCLEAR}" \
	"${model_disk_count}"
if [ ${?} -ne 0 ]; then
	# 第 3 パーティションを構成できない
	__initialization_error "Failed to create volume 3."
fi
echo "OK"

echo "###### create volume 4 ######"
__initialization_create_volume4 \
	"${SRCDISK}" \
	"${ZEROCLEAR}" \
	"${model_name}" \
	"${model_disk_count}"
if [ ${?} -ne 0 ]; then
	# 第 4 パーティションを構成できない
	__initialization_error "Failed to create volume 4."
fi
echo "OK"

echo "###### create volume 5 ######"
__initialization_create_volume5 \
	"${ZEROCLEAR}" \
	"${model_disk_count}"
if [ ${?} -ne 0 ]; then
	# 第 5 パーティションを構成できない
	__initialization_error "Failed to create volume 5."
fi
echo "OK"

echo "###### create volume 6 ######"
__initialization_create_volume6 \
	"${ZEROCLEAR}" \
	"${model_disk_count}"
if [ ${?} -ne 0 ]; then
	# 第 6 パーティションを構成できない
	__initialization_error "Failed to create volume 6."
fi
echo "OK"


# ファイルをインストールする
echo "### INITIALIZATION: install files #######################################"
__initialization_install_files "${model_disk_count}"
if [ ${?} -ne 0 ]; then
	# ファイルをインストールできない
	__initialization_error "Failed to install files."
fi
echo "OK"


# 基板側設定の初期化
echo "### INITIALIZATION: initialize board config #############################"

# RTC アラームの無効化
echo "###### disable RTC alarm ######"
__initialization_disable_rtc_alarm
if [ ${?} -ne 0 ]; then
	# RTC アラームを無効にできない
	__initialization_error "Failed to disable RTC alarm."
fi
echo "OK"

# AC 電源連動設定を無効にする
echo "###### disable AC auto power on ######"
__initialization_disable_ac_auto_power_on
if [ ${?} -ne 0 ]; then
	# AC 電源連動設定を無効にできない
	__initialization_error "Failed to disable AC auto power on."
fi
echo "OK"

# WoL のリセット
echo "###### enable WoL ######"
__initialization_enable_wol
if [ ${?} -ne 0 ]; then
	# WoL を有効にできない
	__initialization_error "Failed to enable WoL."
fi
echo "OK"

# ブザーの初期化
echo "##### enable buzzer #####"
__initialization_enable_buzzer
if [ ${?} -ne 0 ]; then
	# ブザーを有効にできない
	__initialization_error "Failed to initialize buzzer."
fi
echo "OK"

# LED 輝度の初期化
__initialize_set_led_brightness
if [ ${?} -ne 0 ]; then
	# LED 輝度を設定できない
	__initialization_error "Failed to set LED brightness."
fi
echo "OK"

# 後処理
echo "### INITIALIZATION: finishing initialization ############################"
__initialization_finishing
