#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling announce function"""

import os
from iobase.xmldata import XmlConfNode
from iobase.fileut import FLock
import ionas.request as request
import libfal.falcmd as falcmd
import fal_msginfo as msginfo
import pal_info
import fal_config as config
from fal_info import (Results, ReqTypes)
from libfal.util import (RequestError, dispatch_event)

### XML Path###
ANNOUNCE_NAME = "announce"
LEVEL_XPATH = "option/@level"
CATEGORY_XPATH = "option/@category"
CODE_XPATH = "option/@code"
IDX_XPATH = "./index"
CODE_NAME = "code"

ROOT_XPATH = "./."
MESSAGEINFO_XPATH = "messageinfolist/messageinfo[code='%s']"
RELATEDCODELIST_XPATH = "noticeinfo/relatedcodelist"
NOTIFY_XPATH = "noticeinfo/notify"

### XML Attribute ###
LEVEL_ATTR = "level"
CATEGORY_ATTR = "category"

### XML Attribute Value ###
ALL_LEVEL = "all"
LEVEL_WARN = "1"
LEVEL_ERROR = "2"
ALL_CATEGORY = ""

LCM_CATEGORY = "lcm"
CLEAR_ACTION = "clear"

ANNOUNCE_LOCK_FILE = "announce_lock"

ANNOUNCE_XML_DEFAULT = "<announcelist/>"


class FalAnnounce(falcmd.FalCmd):
    """FAL Announce implementation class."""

    def __is_file_exists(self, target_path):
        """Check path existence."""

        target = target_path
        if not os.path.isabs(target):
            target = os.path.abspath(target_path)

        return os.path.exists(target)

    def __load_announce(self):
        """Load a announce file and return XML data ."""

        if not self.__is_file_exists(config.ANNOUNCE_FILE):
            return XmlConfNode(ANNOUNCE_XML_DEFAULT)

        response_entry = self.send_entry_with_file(
                                            request.Entry(ReqTypes.LOAD),
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.ANNOUNCE_FILE)

        if response_entry.get_result() == Results.FILE_NOT_FOUND:
            return XmlConfNode(ANNOUNCE_XML_DEFAULT)

        elif response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

        return response_entry.get()

    def __save_announce(self, announce_xml):
        """Save the "announce_xml" XML data to a announce file."""

        response_entry = self.send_entry_with_file(
                                request.Entry(ReqTypes.SAVE, announce_xml),
                                pal_info.PAL_CONFFILE_PATH,
                                config.ANNOUNCE_FILE)

        if response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

    def __load_messages(self):
        """Load messages.xml"""

        return self.send_entry_with_file(
            request.Entry(ReqTypes.LOAD),
            pal_info.PAL_CONFFILE_PATH, config.MESSAGES_XML_FILE).get()

    def __clear_announce_node(self, level, category, announce_list_node):
        """Clear announce node from announce list node."""

        clr_target_level = {ALL_LEVEL: [LEVEL_WARN, LEVEL_ERROR],
                            LEVEL_WARN: [LEVEL_WARN],
                            LEVEL_ERROR: [LEVEL_ERROR]}

        if category == ALL_CATEGORY:
            category_attr = {}
        else:
            category_attr = {CATEGORY_ATTR: category}

        for level in clr_target_level[level]:
            attr = {LEVEL_ATTR: level}
            attr.update(category_attr)

            announce_list_node.delete_children(ANNOUNCE_NAME, attr)

    def __clear_related_nodes(self, code, announce_list_node):
        """Clear announce nodes related to specified code."""

        CLEAR_TARGET_XPATH = "./announce[code='%s']"

        is_clear = False

        try:
            messages = self.__load_messages()
            messageinfo = messages.get_node(MESSAGEINFO_XPATH % code)
            relatedcodelist = messageinfo.get_node(RELATEDCODELIST_XPATH)
            relatedcode_nodes = relatedcodelist.get_node_list(CODE_NAME)
        except:
            return False

        r_codes = [node.get_text() for node in relatedcode_nodes]
        a_codes = [
            node.get_value(CODE_NAME) \
            for node in announce_list_node.get_node_list(ANNOUNCE_NAME)]
        for r_code in r_codes:
            for a_code in a_codes:
                if r_code == a_code:
                    announce_list_node.delete_nodes(
                        ROOT_XPATH, CLEAR_TARGET_XPATH % a_code)
                    is_clear = True

        return is_clear

    def __clear_same_category_nodes(self, category, announce_list_node):
        """Clear announce nodes same category."""

        CLEAR_TARGET_XPATH = "./announce[@category='%s']"

        is_clear = False

        a_category_list = [
            node.get_attr_value(CATEGORY_ATTR) \
            for node in announce_list_node.get_node_list(ANNOUNCE_NAME)]
        if category in a_category_list:
            try:
                announce_list_node.delete_nodes(
                    ROOT_XPATH, CLEAR_TARGET_XPATH % category)
                is_clear = True
            except:
                pass

        return is_clear

    def __clear_recoverd_nodes(self, code, announce_list_node):
        """Clear announce nodes recoverd by event of code."""

        if not self.__clear_related_nodes(code, announce_list_node):
            category = msginfo.get_category_code(code)
            return self.__clear_same_category_nodes(
                category, announce_list_node)

        return True

    def __is_enable_announce(self, code):
        """Check if the code is announce target."""

        try:
            messages = self.__load_messages()
            messageinfo = messages.get_node(MESSAGEINFO_XPATH % code)
        except:
            self.exception("message load failed?")
            return False

        try:
            if messageinfo.get_node(RELATEDCODELIST_XPATH):
                return True
        except:
            pass

        try:
            return messageinfo.get_node(NOTIFY_XPATH).is_enabled()
        except:
            return False

    #
    # Request Handler
    #

    def __query_handler(self, entry):
        """Handle clear request"""

        try:
            annouce_xml = self.__load_announce()
            entry.set(annouce_xml)
            entry.set_result(Results.OK)
        except RequestError, err:
            entry.set_result(err.get_result())

        return entry

    def __clear_handler(self, entry):
        """Handle clear request"""

        announce = entry.get()
        level = announce.get_value(LEVEL_XPATH, LEVEL_WARN)
        category = announce.get_value(CATEGORY_XPATH, ALL_CATEGORY)
        code = announce.get_value(CODE_XPATH, None)

        with FLock(config.LOCK_DIR, ANNOUNCE_LOCK_FILE, exclusive=True):
            try:
                announces = self.__load_announce()
                if code:
                    self.__clear_recoverd_nodes(code, announces)
                else:
                    self.__clear_announce_node(level, category, announces)

                self.__save_announce(announces)
                entry.set_result(Results.OK)

                try:
                    dispatch_event(LCM_CATEGORY, CLEAR_ACTION, ANNOUNCE_NAME)
                except:
                    self.exception("fail to clear event")
                    pass

            except RequestError, err:
                entry.set_result(err.get_result())

            return entry

    def __delete_handler(self, entry):
        """Handle delete request"""

        with FLock(config.LOCK_DIR, ANNOUNCE_LOCK_FILE, exclusive=True):
            try:
                announces = self.__load_announce()
                code = entry.get().get_value(CODE_NAME)
                if code:
                    idx = 0
                    announce_list = announces.get_node_list(ANNOUNCE_NAME)
                    for announce in announce_list:
                        if announce.get_value(CODE_NAME) == code:
                            break
                        idx += 1
                    if idx >= len(announce_list):
                        entry.set_result(Results.OK)
                        return entry
                else:
                    idx = int(entry.get().get_value(IDX_XPATH))

                try:
                    announces.delete_child(ANNOUNCE_NAME, index=idx)
                except:
                    pass

                self.__save_announce(announces)
                entry.set_result(Results.OK)

            except RequestError, err:
                entry.set_result(err.get_result())

            return entry

    def __add_handler(self, entry):
        """Handle add request."""

        try:
            entry_announce_node = entry.get()
            if entry_announce_node.get_node_name() != ANNOUNCE_NAME:
                entry.set_result(Results.INVALID_XML)
                return entry

            entry_code = entry_announce_node.get_value(CODE_NAME, None)
            if not entry_code:
                entry.set_result(Results.INVALID_XML)
                return entry

            if not self.__is_enable_announce(entry_code):
                entry.set_result(Results.OK)
                return entry
        except:
            entry.set_result(Results.INVALID_XML)
            return entry

        with FLock(config.LOCK_DIR, ANNOUNCE_LOCK_FILE, exclusive=True):
            try:
                announces = self.__load_announce()
                announce_list = announces.get_node_list(ANNOUNCE_NAME)
                for announce in announce_list:
                    if announce.compare(entry_announce_node, True, True) == 0:
                        entry.set_result(Results.OK)
                        return entry

                announces.append_child(entry_announce_node.get_clone())
                self.__save_announce(announces)
                entry.set_result(Results.OK)

            except RequestError, err:
                entry.set_result(err.get_result())

            return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.CLEAR, self.__clear_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)
        self.set_handler(ReqTypes.ADD, self.__add_handler)
#
# Main
#

if __name__ == "__main__":
    FalAnnounce(option=falcmd.FalCmdOption()).handle_request()
