#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Backup Settings."""

import os
import string
import re

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info
from fal_info import ReqTypes
from fal_info import Results
from iobase.xmldata import (ANY_CHILD_NAME, xpath_escape)
from iobase.exception import (NotFoundError, InvalidArgError)

import fal_config as config

JOB = "job"
JOBLIST = "joblist"
JOB_PATH = "./%s/joblist/job"
JOBLIST_PATH = "./%s/joblist"
BACKUP = "backup"
SCHEDULE = "schedule"
ID = "id"
ID_ENTRY_PATH = "./job/id"
NAME_ENTRY_PATH = "./job/name"

DESTINATION_NAME = "destination"
PATH_NAME = "path"

OPTION_NAME = "option"
DETAIL_ATTR = "detail"

UNC_SEP = "\\"


class FalBackup(falcmd.FalCmd):
    """FAL Backup command implementation class."""

    def __get_backup_config(self):
        """Load backup config and return XML data ."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH, config.BACKUP_CONF_FILE).get()

    def __exec_handler(self, entry):
        """Handle detail or execute request and set entry to result."""

        exec_result = self.send_entry(entry, pal_info.PAL_BACKUPS_PATH)
        if exec_result.get_result() != Results.OK:
            entry.set_result(exec_result.get_result())
            return entry
        return exec_result

    def __load_config(self, conffile_name):
        """Load config data"""

        load_entry = request.Entry(ReqTypes.LOAD)
        return self.send_entry_with_file(load_entry,
                                         pal_info.PAL_CONFFILE_PATH,
                                         conffile_name).get()

    def _get_local_sub_dir_from_unc(self, unc):
    
        UNC_SUB_DIR_RE = r"^[^\\]+\\(.+)"
    
        sub_dir = ""
        match = re.match(UNC_SUB_DIR_RE, unc)
        if match:
            sub_dir = re.match(UNC_SUB_DIR_RE, unc).group(1)

        return sub_dir

    def __get_local_share_path(self, unc):

        VOL_SEP = "/"
        SHARE_PATH_XPATH = "sharelist/share[name=%s]/path"
        UNC_SHARE_RE = r"^\\\\[^\\]+\\([^\\]+)"
        LOCAL_SHARE_RE = r"([^\\\\]+)"

        share_name = re.match(LOCAL_SHARE_RE, unc).group(1)
        share_config = self.__load_config(config.SHARES_CONF_FILE)
        
        sub_dir = self._get_local_sub_dir_from_unc(unc).replace(UNC_SEP, VOL_SEP)
        if sub_dir:
            sub_dir = VOL_SEP + sub_dir

        return share_config.get_value(
            SHARE_PATH_XPATH % xpath_escape(share_name), "") + sub_dir

    def __insert_share_path(self, joblist_node):

        UNC_PATH = "destination/unc"

        for job_node in joblist_node.get_node_list(JOB):

            try:
                share_path = \
                    self.__get_local_share_path(job_node.get_value(UNC_PATH))
            except:
                continue

            # fixme is remote
            job_node.get_node(
                 DESTINATION_NAME).append_child(PATH_NAME, share_path)

    def __get_joblist_node_from_config(self):

        JOBLIST_PATH = "backup/joblist"
        return self.__load_config(
                   config.BACKUP_CONF_FILE).get_node(JOBLIST_PATH)

    def __do_job_exceed_limit(self, type, new_num):
        """Return "new_num" exceed the limit of "type" job."""

        if (type in config.MAX_JOB_NUM_DICT and
            new_num > config.MAX_JOB_NUM_DICT[type]):

            return True

        return False

    def __query_handler(self, entry):

        try:
            if entry.get().get_node(
                   OPTION_NAME).is_set_to_true(DETAIL_ATTR):

                joblist_node = self.__get_joblist_node_from_config()
                self.__insert_share_path(joblist_node)
                entry.get().get_node(
                    OPTION_NAME).append_child(joblist_node)

        except NotFoundError:
            pass

        result = self.send_entry(entry, pal_info.PAL_BACKUPS_PATH)
        result.get().delete_nodes(OPTION_NAME, JOBLIST)
        if result.get_result() != Results.OK:
            entry.set_result(result.get_result())
            return entry
        return result

    def __apply_handler(self, entry):

        result = self.send_entry(entry, pal_info.PAL_BACKUPS_PATH)
        if result.get_result() != Results.OK:
            entry.set_result(result.get_result())
            return entry
        return result

    @falcmd.conf_change_handler_wrapper
    def __delete_handler(self, entry):
        """Handle delete request and set entry to result."""

        delete_result = self.send_entry(entry, pal_info.PAL_BACKUPS_PATH)
        if delete_result.get_result() != Results.OK:
            entry.set_result(delete_result.get_result())
            return entry

        type = entry.get().get_node(ANY_CHILD_NAME).get_node_name()
        job_name = entry.get().get_node(ANY_CHILD_NAME).get_node(
            NAME_ENTRY_PATH).get_text(False)
        backup_config = self.__get_backup_config()

        backup_config.delete_nodes(JOBLIST_PATH % type,
                                "./job[name='%s']" % job_name)
        save_entry = request.Entry(ReqTypes.SAVE, backup_config)
        self.send_entry_with_file(save_entry, pal_info.PAL_CONFFILE_PATH,
                                              config.BACKUP_CONF_FILE)
        entry.set_result(Results.OK)
        return entry

    @falcmd.conf_change_handler_wrapper
    def __modify_handler(self, entry):
        """Handle modify request and set entry to result."""

        modify_result = self.send_entry(entry, pal_info.PAL_BACKUPS_PATH)
        if modify_result.get_result() != Results.OK:
            entry.set_result(modify_result.get_result())
            return entry

        type = entry.get().get_node(ANY_CHILD_NAME).get_node_name()
        try:
            job_id = entry.get().get_node(ANY_CHILD_NAME).get_node(
                ID_ENTRY_PATH).get_text()
        except NotFoundError:
            raise InvalidArgError("No job ID Node in modify entry.")

        backup_config = self.__get_backup_config()
        idx = 0
        for job in backup_config.get_node_list(JOB_PATH % type):
            if job.get_node(ID).get_text() == job_id:
                break
            idx += 1

        if self.__do_job_exceed_limit(type, idx):
            entry.set_result(Results.MAX_EXCEED_ERROR)
            return entry

        try:
            backup_config.delete_nodes(JOBLIST_PATH % type,
                                        "./job[id='%s']" % job_id)
        except NotFoundError:
            pass
        backup_config.get_node(JOBLIST_PATH % type).insert_child(idx,
            entry.get().get_node(ANY_CHILD_NAME).get_node(JOB))

        save_entry = request.Entry(ReqTypes.SAVE, backup_config)
        self.send_entry_with_file(save_entry, pal_info.PAL_CONFFILE_PATH,
                                              config.BACKUP_CONF_FILE)
        entry.set_result(Results.OK)
        return entry

    @falcmd.conf_change_handler_wrapper
    def __add_handler(self, entry):
        """Handle add request and set entry to result."""

        type = entry.get().get_node(ANY_CHILD_NAME).get_node_name()
        id_list = []
        backup_config = self.__get_backup_config()
        try:
            for job in backup_config.get_node_list(JOB_PATH % type):
                id_list.append(job.get_node(ID).get_text())
        except NotFoundError:
            pass

        specified_job_id = \
            entry.get().get_node(ANY_CHILD_NAME).get_value(ID_ENTRY_PATH)
        if not specified_job_id or not specified_job_id.isdigit():
            job_id = 0
            while str(job_id) in id_list:
                job_id += 1
            entry.get().get_node(ANY_CHILD_NAME).get_node(
                "./job/id").set_text(str(job_id))
        else:
            if specified_job_id in id_list:
                entry.set_result(Results.ALREADY_EXIST)
                return entry

        if self.__do_job_exceed_limit(type, len(id_list) + 1):
            entry.set_result(Results.MAX_EXCEED_ERROR)
            return entry

        add_result = self.send_entry(entry, pal_info.PAL_BACKUPS_PATH)
        if add_result.get_result() != Results.OK:
            entry.set_result(add_result.get_result())
            return entry

        backup_config.get_node(
                JOBLIST_PATH % type).append_child(
                entry.get().get_node(ANY_CHILD_NAME).get_node(JOB))

        save_entry = request.Entry(ReqTypes.SAVE, backup_config)
        self.send_entry_with_file(save_entry, pal_info.PAL_CONFFILE_PATH,
                                              config.BACKUP_CONF_FILE)
        entry.set_result(Results.OK)
        return entry

    def __restore_handler(self, entry):

        REQ_JOB_PATH = "backup/job"
        REQ_JOBID_PATH = "backup/job/id"
        CONF_JOBID_PATH_TMPL = "backup/joblist/job[id='%s']"

        job_id = entry.get().get_value(REQ_JOBID_PATH)
        conf_job_node = \
            self.__get_backup_config().get_node(CONF_JOBID_PATH_TMPL % job_id)

        pal_entry = entry.get_clone()
        pal_entry.get().get_node(
            REQ_JOB_PATH).update(conf_job_node, override=True)

        result = self.send_entry(pal_entry, pal_info.PAL_BACKUPS_PATH)
        entry.set_result(result.get_result())
        return entry

    def __stop_handler(self, entry):

        return self.send_entry(entry, pal_info.PAL_BACKUPS_PATH)

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                 pal_info.PAL_CONFFILE_PATH, config.BACKUP_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.EXECUTE, self.__exec_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)
        self.set_handler(ReqTypes.MODIFY, self.__modify_handler)
        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.RESTORE, self.__restore_handler)
        self.set_handler(ReqTypes.STOP, self.__stop_handler)

#
# Main
#

if __name__ == "__main__":
    FalBackup(option=falcmd.FalCmdOption()).handle_request()
