#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for buzzer beep"""

import os

import ionas.request as request
import libfal.falcmd as falcmd
import pal_info
import fal_config as config
import iobase.fileut as fileut
from libfal.melody_data import (MELODY_TABLE,)
from fal_info import (ReqTypes, Results, BUZZER_MELODY)
from fal_config import (MELODY_PRIORYTY_TABLE,)

### XML Tag Name ###
MELODY_NAME = "melody"
SCORE_NAME = "score"
STATUS_NAME = "status"

### XML Attribute  ###
TYPE_ATTR = "type"

MELODY_OFF = "off"

STOP_REQ_XML = '<buzzer><score>S</score></buzzer>'
MELODY_LOCK_FILE = "melody_lock"


class FalBuzzer(falcmd.FalCmd):
    """FAL Buzzer command implementation class."""

    def __set_cur_melody_type(self, cur_melody_type):
        """Save current melody to tmp file"""

        melody_file_path = \
            os.path.join(config.MELODY_CACHE_FILE_TMPL)

        dir_path = os.path.dirname(melody_file_path)
        if not os.path.isdir(dir_path):
            os.makedirs(dir_path)

        try:
            with fileut.FLock(config.LOCK_DIR, MELODY_LOCK_FILE,
                              exclusive=False):
                with open(melody_file_path, "w") as fh:
                    fh.write(cur_melody_type)
        except:
            self.exception("fail to save current melody type")
            pass

        return

    def __is_current_buzzer_stop(self):

        try:
            query_entry = request.Entry(ReqTypes.QUERY)
            result_entry = \
                self.send_entry(query_entry, pal_info.PAL_BUZZER_PATH)
            melody_status = result_entry.get().get_value(STATUS_NAME, "")
            if MELODY_OFF == melody_status:
                return True
        except:
            pass

        return False

    def __is_able_to_play(self, melody_type):
        """Check whether melody"""

        if self.__is_current_buzzer_stop():
            return True

        melody_file_path = \
            os.path.join(config.MELODY_CACHE_FILE_TMPL)

        if not os.path.isfile(melody_file_path):
            return True

        try:
            with fileut.FLock(config.LOCK_DIR, MELODY_LOCK_FILE,
                              exclusive=False):
                with open(melody_file_path, "r") as fh:
                    cur_melody_type = fh.read().rstrip()

                    if BUZZER_MELODY == cur_melody_type or \
                       BUZZER_MELODY == melody_type:
                        return True

                    cur_pri = MELODY_PRIORYTY_TABLE[cur_melody_type]
                    pri = MELODY_PRIORYTY_TABLE[melody_type]
                    if cur_pri < pri:
                        return False
        except:
            self.exception("fail to load current melody type")
            pass

        return True

    def __stop_buzzer(self):

        play_entry = request.Entry(ReqTypes.PLAY, STOP_REQ_XML)
        return self.send_entry(play_entry, pal_info.PAL_BUZZER_PATH)

    def __play_handler(self, entry):
        """Handle play request."""

        melody_type = \
            entry.get().get_node(MELODY_NAME).get_attr_value(TYPE_ATTR)

        if not self.__is_able_to_play(melody_type):
            entry.set_result(Results.OK)
            return entry

        self.__stop_buzzer()
        entry.get().append_child(SCORE_NAME, MELODY_TABLE[melody_type])
        result_entry = self.send_entry(entry, pal_info.PAL_BUZZER_PATH)

        self.__set_cur_melody_type(melody_type)
        return result_entry

    def __is_need_stop_melody(self, entry):

        melody_type = None
        try:
            melody_type = \
                entry.get().get_node(MELODY_NAME).get_attr_value(TYPE_ATTR)
        except:
            return True

        if self.__is_current_buzzer_stop():
            return True  # 念のためSTOP処理させる

        melody_file_path = os.path.join(config.MELODY_CACHE_FILE_TMPL)

        if os.path.isfile(melody_file_path) and melody_type:

            cur_melody_type = None
            try:
                with fileut.FLock(config.LOCK_DIR, MELODY_LOCK_FILE,
                                  exclusive=False):
                    with open(melody_file_path, "r") as fh:
                        cur_melody_type = fh.read().rstrip()
            except:
                self.exception("fail to load current melody type")
                return True

            if not cur_melody_type:
                return True

            if cur_melody_type == melody_type:
                return True
            else:
                # 指定されたメロディと現在メロディが異なる場合は停止しない
                return False

        return True

    def __stop_handler(self, entry):
        """Handle stop request."""

        try:
            if not self.__is_need_stop_melody(entry):
                entry.set_result(Results.OK)
                return entry
        except:
            self.exception("fail to __is_need_stop_melody")
            pass

        entry.set_result(self.__stop_buzzer().get_result())
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.PLAY, self.__play_handler)
        self.set_handler(ReqTypes.STOP, self.__stop_handler)

#
# Main
#

if __name__ == "__main__":
    FalBuzzer(option=falcmd.FalCmdOption()).handle_request()
