#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling diskctrl function"""

import libfal.falcmd as falcmd
import ionas.request as request
import libfal.util as util
import pal_info
from fal_info import (ReqTypes, Results)
from iobase.exception import (NotFoundError)

FAIL_STATUS = "fail"
CAPSHORTAGE_STATUS = "capshortage"
SMARTERROR_STATUS = "smarterror"
UNPLUG_DISK_STATUS = "unplug"
NORAML_DISK_STATUS = "normal"

STATUS_NAME = "status"
OPTION_NAME = "option"

INC_ERR_ATTR = "incerr"


class FalDiskCtrl(falcmd.FalCmd):
    """FAL DiskCtrl command implementation class."""

    class __DeleteError(Exception):
        "Delete Error"
        pass

    class __CheckError(Exception):
        "Check Error"
        pass

    class __NotSupported(Exception):
        "Not Supported"
        pass

    def __delete_disk_form_system(self, disk_name):

        DELETE_REQ_XML = \
            "<diskctrl><disk><name>%s</name></disk></diskctrl>"

        entry = request.Entry(ReqTypes.DELETE, DELETE_REQ_XML % disk_name)
        result_entry = self.send_entry(entry, pal_info.PAL_DISKCTRL_PATH)
        if result_entry.get_result() != Results.OK:
            raise self.__DeleteError
        return

    def __is_physical_hdd_exist(self, disk_name):

        CHK_EXIST_REQ_XML = \
            "<diskctrl><disk><name>%s</name></disk></diskctrl>"

        entry = request.Entry(ReqTypes.CHK_EXIST,
                              CHK_EXIST_REQ_XML % disk_name)
        result_entry = self.send_entry(entry, pal_info.PAL_DISKCTRL_PATH)
        result_code = result_entry.get_result()

        if Results.OK == result_code:
            return True

        if Results.DEVICE_NOT_FOUND == result_code:
            return False

        if Results.NOT_SUPPORTED == result_code:
            raise self.__NotSupported

        raise self.__CheckError

    def __update_handler(self, entry):

        disk_name = entry.get().get_node("./disk/name").get_text()
        status = entry.get().get_node("./disk/status").get_text()

        try:
            if entry.get().get_node(OPTION_NAME).is_set_to_true(INC_ERR_ATTR):
                is_inc_err = True
            else:
                is_inc_err = False
        except NotFoundError:
            is_inc_err = False

        if UNPLUG_DISK_STATUS == status:
            cur_status = util.get_disk_info(disk_name, STATUS_NAME)
            if NORAML_DISK_STATUS != cur_status:
                entry.set_result(Results.OK)
                return entry

        util.set_disk_info(disk_name, STATUS_NAME, status)

        if util.FAIL_DISK_STATUS == status:
            try:
                if is_inc_err:
                    util.increment_disk_errorcount(disk_name)
            except:
                self.exception("fail to increment disk errorcount")
                pass

        entry.set_result(Results.OK)
        return entry

    def __query_handler(self, entry):

        disk_name = entry.get().get_node("./disk/name").get_text()
        status = util.get_disk_info(disk_name, STATUS_NAME)

        if disk_name.startswith("satad"):
            try:
                if not self.__is_physical_hdd_exist(disk_name):
                    # 物理的にHDDが接続されてない
                    util.set_disk_info(disk_name,
                                       STATUS_NAME, UNPLUG_DISK_STATUS)
                    entry.set_result(Results.DEVICE_NOT_FOUND)
                    return entry
            except self.__CheckError:
                self.exception("fail to __is_physical_hdd_exist(%s)" % \
                               disk_name)
                pass

            # 物理的な接続状態を確認できない環境
            except self.__NotSupported: 
                pass

        result_entry = self.send_entry(entry, pal_info.PAL_DISKCTRL_PATH)
        if Results.OK != result_entry.get_result():

            if status in [FAIL_STATUS, CAPSHORTAGE_STATUS, SMARTERROR_STATUS]:
                entry.get().get_node("./disk").append_child(STATUS_NAME)
                entry.get().get_node("./disk/status").set_text(status)
                entry.set_result(Results.OK)
                return entry

            return result_entry

        result_entry.get().get_node("./disk").append_child(STATUS_NAME)

        if not status:
            result_entry.get().get_node("./disk/status").set_text("normal")
            util.set_disk_info(disk_name, STATUS_NAME, "normal")
        else:
            result_entry.get().get_node("./disk/status").set_text(status)

        return result_entry

    def __delete_handler(self, entry):

        result_entry = self.send_entry(entry, pal_info.PAL_DISKCTRL_PATH)
        util.dispatch_event("disk", "update", "diskctrl")
        util.dispatch_event("raid", "update", "raidctrl")
        return result_entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        pass_handler = \
            lambda entry: self.send_entry(entry, pal_info.PAL_DISKCTRL_PATH)

        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.UPDATE, self.__update_handler)
        self.set_handler(ReqTypes.CHECK, pass_handler)
        self.set_handler(ReqTypes.MKPART, pass_handler)
        self.set_handler(ReqTypes.DELPART, pass_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)
        self.set_handler(ReqTypes.CONTROL, pass_handler)


#
# Main
#

if __name__ == "__main__":
    FalDiskCtrl(option=falcmd.FalCmdOption()).handle_request()
