#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling mail function"""

from __future__ import with_statement

import os
import shutil
import iobase.xmldata as xmldata
import ionas.request as request
import libfal.falcmd as falcmd
import pal_info
import fal_config as config
import iobase.fileut as fileut
from libfal.util import dispatch_event
from iobase.exception import (NotFoundError)
from iobase.xmldata import (XmlNode)
from iobase.cmdexec import (CmdExec, NULL)
from datetime import datetime
from fal_info import ReqTypes
from fal_info import Results
from fal_config import (LOGMAIL_CMD)
from fal_config import EVENTLOG_LOCK_NAME
from lxml import etree


# For fal_eventlog
EVENTLOG_FILE_NAME = "eventlog.xml"
EVENTLOG_MAX_FILE_NUM = 10
EVENTLOG_MAX_LOG_NUM = 1000

# XML Tag Name
EVENTLOG_NAME = "eventlog"
LOGLIST_NAME = "loglist"
LOG_NAME = "log"
OPTION_NAME = "option"
FILTER_NAME = "filter"
LEVEL_LIST_NAME = "levellist"
LEVEL_NAME = "level"
CATEGORY_LIST_NAME = "categorylist"
CATEGORY_NAME = "category"
MESSAGEINFO_LIST_NAME = "messageinfolist"
TIMESTAMP_LIST_NAME = "timestamplist"
TIMESTAMP_NAME = "timestamp"
CODE_NAME = "code"
LEVEL_NAME = "level"

# XML Attribute
CATEGORY_ATTR = "category"
TIMESTAMP_ATTR = "timestamp"
DATE_ATTR = "date"
SORT_ATTR = "sort"
REVERSE_ATTR = "reverse"
FILTER_ATTR = "filter"
INDEX_ATTR = "index"
MAX_NUM_ATTR = "maxnum"
TOTAL_ATTR = "total"
FROM_ATTR = "from"
TO_ATTR = "to"
LEVEL_ATTR = "level"
CAUSE_ATTR = "cause"

# XML Attribute Value
TRUE_ATTR_VALUE = "true"
NONE_ATTR_VALUE = "none"
TIMESTAMP_ATTR_VALUE = "timestamp"

# Blank eventlog XML
BLANK_EVENT_LOG_XML_STR = '<eventlog><loglist/></eventlog>'

# Base eventlog XML
BASE_EVENT_LOG_XML_STR = '<eventlog/>'
BASE_LOGLIST_XML_STR = '<loglist/>'
BASE_MSG_NODE_FOR_EVH_XML = \
    '<messages><categorylist/><messageinfolist/></messages>'

# Dictionary KEY
FROM_KEY = "FROM"
TO_KEY = "TO"
CATEGORY_KEY = "CATEGORY"
LEVEL_KEY = "LEVEL"
TIMESTAMP_KEY = "TIMESTAMP"
INDEX_KEY = "INDEX"
MAXNUM_KEY = "MAXNUM"
SORT_KEY = "SORT"
REVERSE_KEY = "REVERSE"
FILTER_KEY = "FILTER"

# For MesssageXML
MESSAGE_CODE_UNIT = 10000
CATEGORY_CODE_UNIT = 100
UNKNOWN_CATEGORY = "0"
WARN_LEVEL = "2"
NORMAL_LEVEL = "0"

MSG_NOTIFY_PATH_TMPL = \
    'messageinfolist/messageinfo/code[text()="%s"]/..'
CATE_PATH_TMPL = \
    'categorylist/category[code="%s"]'
MSGINFO_PATH_TMPL = \
    'messageinfolist/messageinfo[code="%s"]'


class FalEventLog(falcmd.FalCmd):
    """FAL EventLog command implementation class."""

    def __is_enable_filter(self):
        """Return filter flag"""
        return self.__is_filter

    def __is_exist_root_node(self, entry):
        """Check whether entry node has root node"""

        try:
            entry.get()
        except NotFoundError:
            return False
        return True

    def __is_exist_node(self, node, node_path):
        """Check whether node_path exists in specified node"""

        try:
            node.get_node(node_path)
        except NotFoundError:
            return False
        return True

    def __is_exist_attr(self, node, attr_val):
        """Check whether attr_val exists in specified node"""

        try:
            node.get_attr_value(attr_val)
        except NotFoundError:
            return False
        return True

    def __get_messages_xml(self):
        """Load "messages.xml" and return XML data ."""

        res = self.send_entry_with_file(
            request.Entry(ReqTypes.LOAD), pal_info.PAL_CONFFILE_PATH,
            config.MESSAGES_XML_FILE)
        node = res.get()
        return node

    def __get_message_level(self, msg_code):
        """Return the category code for the "msg_code"."""

        try:
            messageinfo_node = self.messages_xml.get_node(
                MSG_NOTIFY_PATH_TMPL % msg_code)
            level_node = messageinfo_node.get_node(LEVEL_NAME)
            return level_node.get_text()
        except:
            return NORMAL_LEVEL

    def __get_message_cause(self, msg_code):
        """Return the category code for the "msg_code"."""

        try:
            messageinfo_node = self.messages_xml.get_node(
                MSG_NOTIFY_PATH_TMPL % msg_code)
            level_node = messageinfo_node.get_node(LEVEL_NAME)
            return level_node.get_attr_value(CAUSE_ATTR)
        except:
            return ""

    def __get_category_code(self, msg_code):
        """Return the category code for the "msg_code"."""

        try:
            if not isinstance(msg_code, int):
                msg_code = int(msg_code)
            return str(
                (msg_code / MESSAGE_CODE_UNIT / CATEGORY_CODE_UNIT) *
                CATEGORY_CODE_UNIT)
        except:
            return UNKNOWN_CATEGORY

    def __get_entry_file_attr(self, entry):
        """Get file path of entry node """

        try:
            file_path = entry.get_file()
        except NotFoundError:
            return None

        return file_path

    def __is_need_log_rotate(self, logfile_path):
        """Check wheather log rotate"""

        LOG_XPATH = "loglist/log"

        if not os.path.exists(logfile_path):
            return False

        log_node_list = \
            xmldata.XmlData(logfile_path).get_node_list(LOG_XPATH)

        if len(log_node_list) > EVENTLOG_MAX_LOG_NUM:
            return True
        return False

    def __send_log_mail(self, org_logfile_path):

        logfile_path = \
            os.path.join(config.LOG_DIR,
                         os.path.basename(org_logfile_path) + '.1')

        LOG_ENABLE_MAILCONF_XPATH = \
            "./mailnotice/mailconflist" + \
            "/mailconf[@enable='true']" + \
            "/notifylist" + \
            "/notify[@enable='true' and @name='eventlog']"

        notice_node = self.__load_config(config.NOTICE_CONF_FILE)

        try:
            notice_node.get_node(LOG_ENABLE_MAILCONF_XPATH)
        except:
            return

        file_opt = "--file=" + logfile_path
        cmd_list = [LOGMAIL_CMD, file_opt, "--eventlog"]
        proc = CmdExec(cmd_list, stdout=NULL, stderr=NULL,
                       wait=True, raise_exc=False, close_fds=True).execute()
        proc.wait()
        return

    def __sub_log_rotate(self, org_logfile_path):

        logfile_path = \
            os.path.join(config.LOG_DIR, os.path.basename(org_logfile_path))

        if os.path.exists(logfile_path + "." + str(EVENTLOG_MAX_FILE_NUM)):
            os.remove(logfile_path + "." + str(EVENTLOG_MAX_FILE_NUM))

        for no in range(EVENTLOG_MAX_FILE_NUM - 1, 0, -1):

            srcdir = logfile_path + '.' + str(no)
            destdir = logfile_path + '.' + str(no + 1)
            if os.path.exists(srcdir):
                shutil.move(srcdir, destdir)

        if os.path.exists(org_logfile_path):
            shutil.move(org_logfile_path, logfile_path + '.1')

    def __log_rotate(self, org_logfile_path, is_judge=False):
        """Do log rotate in specified log directory"""

        is_rotate_done = False
        with fileut.FLock(config.LOCK_DIR, EVENTLOG_LOCK_NAME, True):
            if is_judge:
                if not self.__is_need_log_rotate(org_logfile_path):
                    return

            self.__sub_log_rotate(org_logfile_path)
            is_rotate_done = True

        if is_rotate_done:
            self.__send_log_mail(org_logfile_path)
        return

    def __create_new_logfile(self, logfile_path):
        """Create new log file. Insert "eventlog" node."""

        fh = open(logfile_path, "w")
        fh.write(BLANK_EVENT_LOG_XML_STR + "\n")
        fh.close()

    def __get_mes_node_for_evh(self, mes_code, cate_code):

        mes_node = XmlNode(BASE_MSG_NODE_FOR_EVH_XML)
        catelist_node = mes_node.get_node(CATEGORY_LIST_NAME)
        mesinfolist_node = mes_node.get_node(MESSAGEINFO_LIST_NAME)

        catelist_node.append_child(self.messages_xml.get_node(
            CATE_PATH_TMPL % cate_code).get_clone())
        mesinfolist_node.append_child(self.messages_xml.get_node(
            MSGINFO_PATH_TMPL % mes_code).get_clone())

        mes_node.append_child(catelist_node)
        mes_node.append_child(mesinfolist_node)

        return mes_node

    def __write_log(self, logfile_path, log_node):
        """Write event of specified event node into logfile"""

        # メッセージカテゴリ、レベルを挿入
        mes_code = log_node.get_value(CODE_NAME)
        cate_code = self.__get_category_code(mes_code)
        msg_level = self.__get_message_level(mes_code)
        msg_cause = self.__get_message_cause(mes_code)

        log_node.set_attrs({CATEGORY_ATTR: cate_code})
        log_node.set_attrs({LEVEL_ATTR: msg_level})
        if msg_cause:
            log_node.set_attrs({CAUSE_ATTR: msg_cause})

        with fileut.FLock(config.LOCK_DIR, EVENTLOG_LOCK_NAME, True):

            if not os.path.exists(logfile_path):
                self.__create_new_logfile(logfile_path)

            eventlog_data = xmldata.XmlData(logfile_path)
            loglist_node = eventlog_data.get_root_node().get_node(LOGLIST_NAME)
            loglist_node.append_child(log_node)
            fh = open(logfile_path, "w")
            fh.write(str(eventlog_data.get_root_node()))
            fh.write("\n")
            fh.close()

        # "イベントログを保存したよ"イベント
        event_detail_node = XmlNode(EVENTLOG_NAME)
        event_detail_node.append_child(
            log_node)
        event_detail_node.append_child(
            self.__get_mes_node_for_evh(mes_code, cate_code))
        try:
            dispatch_event('eventlog', 'add', event_detail_node)
        except:
            pass
        return

    def __set_current_time_to_node(self, log_node):
        """Set current time to log node"""

        now = datetime.now()
        timestamp = now.strftime("%s")  # epoch time
        date = now.strftime("%Y/%m/%d %H:%M:%S")
        log_node.set_attrs({TIMESTAMP_ATTR: timestamp, DATE_ATTR: date})
        return

    def __load_config(self, file_name):
        """Load a 'file_name' file setting and return XML data ."""

        entry = request.Entry(ReqTypes.LOAD)
        entry.set_file(file_name)
        target = request.Request(pal_info.PAL_CONFFILE_PATH)
        response_data = target.request(request.RequestData(entry))
        return response_data.get_entrylist().get_entry().get()

    def __get_level_filter_list(self, filter_node):
        """Get filter parameter from levellist node"""

        level_list = []
        level_list_node = filter_node.get_node(LEVEL_LIST_NAME)
        level_node_list = level_list_node.get_node_list(LEVEL_NAME)

        for level_node in level_node_list:
            level_list += [level_node.get_text()]

        return level_list

    def __get_category_filter_list(self, filter_node):
        """Get filter parameter from levellist node"""

        category_list = []
        category_list_node = filter_node.get_node(CATEGORY_LIST_NAME)
        category_node_list = category_list_node.get_node_list(CATEGORY_NAME)

        for category_node in category_node_list:
            category_list += [category_node.get_text()]

        return category_list

    def __get_timestamp_filter_list(self, filter_node):
        """Get timestamp range filter from timestamplist node"""

        timestamp_list = []
        timestamp_list_node = filter_node.get_node(TIMESTAMP_LIST_NAME)

        timestamp_node_list = timestamp_list_node.get_node_list(TIMESTAMP_NAME)
        for timestamp_node in timestamp_node_list:

            from_timestamp = timestamp_node.get_attr_value(FROM_ATTR)

            to_timestamp = timestamp_node.get_attr_value(TO_ATTR)

            timestamp_list.append({FROM_KEY: from_timestamp,
                                  TO_KEY: to_timestamp})

        return timestamp_list

    def __load_filter_param(self):
        """Get filter parameter from filter node."""

        filter_node = \
            self.__load_config(config.EVENTLOG_CONF_FILE).get_node(FILTER_NAME)

        level_list = self.__get_level_filter_list(filter_node)
        category_list = self.__get_category_filter_list(filter_node)
        timestamp_list = self.__get_timestamp_filter_list(filter_node)

        self.__fileter_param_dict = {LEVEL_KEY: level_list,
                                     CATEGORY_KEY: category_list,
                                     TIMESTAMP_KEY: timestamp_list}
        return

    def __load_option_param(self, entry):
        """Load option node parameter"""

        if not self.__is_exist_node(entry.get(), OPTION_NAME):
            return

        option_node = entry.get().get_node(OPTION_NAME)

        if self.__is_exist_attr(option_node, INDEX_ATTR):
            self.__option_dict[INDEX_KEY] = \
                int(option_node.get_attr_value(INDEX_ATTR))

        if self.__is_exist_attr(option_node, MAX_NUM_ATTR):
            self.__option_dict[MAXNUM_KEY] = \
                int(option_node.get_attr_value(MAX_NUM_ATTR))

        if self.__is_exist_attr(option_node, SORT_ATTR):
            self.__option_dict[SORT_KEY] = \
                option_node.get_attr_value(SORT_ATTR)

        if self.__is_exist_attr(option_node, REVERSE_ATTR):
            self.__option_dict[REVERSE_KEY] = \
                option_node.get_attr_value(REVERSE_ATTR)

        if self.__is_exist_attr(option_node, FILTER_ATTR):
            self.__option_dict[FILTER_KEY] = \
                option_node.get_attr_value(FILTER_ATTR)

    def __is_filter_target(self, log_node):
        """Check wheather specified node is filter target"""

        level = log_node.get_attr_value(LEVEL_ATTR)
        if len(self.__fileter_param_dict[LEVEL_KEY]) > 0:
            if level not in self.__fileter_param_dict[LEVEL_KEY]:
                return False

        category = log_node.get_attr_value(CATEGORY_ATTR)
        if len(self.__fileter_param_dict[CATEGORY_KEY]) > 0:
            if category not in self.__fileter_param_dict[CATEGORY_KEY]:
                return False

        if len(self.__fileter_param_dict[TIMESTAMP_KEY]) == 0:
            return True
        timestamp = log_node.get_attr_value(TIMESTAMP_ATTR)
        for timestamp_range in self.__fileter_param_dict[TIMESTAMP_KEY]:

            log_utime = int(timestamp)

            try:
                from_utime = int(timestamp_range[FROM_KEY])
            except:
                from_utime = 0

            try:
                to_utime = int(timestamp_range[TO_KEY])
            except:
                to_utime = 100000000000  # とりあえずでかい値

            if from_utime <= log_utime and log_utime <= to_utime:
                return True

        return False

    def __is_valid_log_node(self, log_node):
        """Check log node validity"""

        try:
            log_node.get_attr_value("date")
            log_node.get_attr_value("timestamp")
            log_node.get_attr_value("category")
            log_node.get_attr_value("level")
            return True
        except:
            return False

    def __append_log_node_with_filter(self,
                                      dst_loglist_node,
                                      src_loglist_node):
        """Append log node of "src_loglist_node" to "dst_loglist_node"
           with filtering.
        """

        src_log_node_list = src_loglist_node.get_node_list(LOG_NAME)

        for src_log_node in src_log_node_list:

            if not self.__is_valid_log_node(src_log_node):
                continue

            if TRUE_ATTR_VALUE == self.__option_dict[FILTER_KEY] and \
               not self.__is_filter_target(src_log_node):
                continue

            dst_loglist_node.append_child(src_log_node)
            self.__total_log_count += 1

        return

    def __get_loglist_from_file(self, logfile_path):
        """Get loglist node from specified log file"""

        parser = etree.XMLParser(recover=True)
        tree = etree.parse(logfile_path, parser)
        eventlog_xml = etree.tostring(tree)
        return XmlNode(eventlog_xml).get_node(LOGLIST_NAME)

    def __get_specific_range_log_list(self, log_node_list):
        """Get specified range log list"""

        index = self.__option_dict[INDEX_KEY]
        maxnum = self.__option_dict[MAXNUM_KEY]

        if index is None and maxnum is None:
            return log_node_list

        if index is None:
            return log_node_list[:maxnum]

        if maxnum is None:
            return log_node_list[index:]

        if index >= 0:
            return log_node_list[index:index + maxnum]

        if (index + maxnum) >= 0:
            return log_node_list[index:]

        return log_node_list[index: index + maxnum]

    def __append_all_eventlog_with_filter(self, loglist_node):
        """Append all event log to loglist node with filter setting"""

        self.__total_log_count = 0

        main_log_path = os.path.join(config.SYS_LOG_DIR, EVENTLOG_FILE_NAME)
        if os.path.exists(main_log_path):
            tmp_loglist_node = self.__get_loglist_from_file(main_log_path)
            self.__append_log_node_with_filter(loglist_node, tmp_loglist_node)

        rotated_log_path = os.path.join(config.LOG_DIR, EVENTLOG_FILE_NAME)
        ext_max_num = EVENTLOG_MAX_FILE_NUM
        for ext in ["." + str(no) for no in range(1, ext_max_num + 1)]:

            logfile_path = rotated_log_path + ext

            if not os.path.exists(logfile_path):
                continue

            tmp_loglist_node = self.__get_loglist_from_file(logfile_path)

            self.__append_log_node_with_filter(loglist_node, tmp_loglist_node)

        return

    def __remake_loglist_with_option(self, loglist_node):
        """Re-make loglist node with option"""

        log_node_list = loglist_node.get_node_list(LOG_NAME)

        log_node_list = self.__sort_eventlog(log_node_list)

        log_node_list = self.__reverse_eventlog(log_node_list)

        log_node_list = self.__get_specific_range_log_list(log_node_list)

        new_loglist_node = XmlNode(BASE_LOGLIST_XML_STR)
        new_loglist_node.set_attrs({TOTAL_ATTR: str(self.__total_log_count)})
        for log_node in log_node_list:
            new_loglist_node.append_child(log_node)

        return new_loglist_node

    def __sort_eventlog(self, log_node_list):
        """Sort log_node_list"""

        if self.__option_dict[SORT_KEY] is None:
            return log_node_list

        if self.__option_dict[SORT_KEY] == NONE_ATTR_VALUE:
            return log_node_list

        if self.__option_dict[SORT_KEY] == TIMESTAMP_ATTR_VALUE:
            def sort_target(node):
                return int(node.get_attr_value(TIMESTAMP_ATTR))

            return sorted(log_node_list, key=sort_target)

        # 同じソートキーの中では
        # タイムスタンプを優先させる
        def sort_target(node):
            return (
                node.get_attr_value(self.__option_dict[SORT_KEY]),
                int(node.get_attr_value(TIMESTAMP_ATTR)))

        return sorted(log_node_list, key=sort_target)

    def __reverse_eventlog(self, log_node_list):
        """Reverse log_node_list"""

        if TRUE_ATTR_VALUE == self.__option_dict[REVERSE_KEY]:
            log_node_list.reverse()

        return log_node_list

    def __load_eventlog(self):
        """Load eventlog.
           This method handles filter and option parameter.
        """

        loglist_node = XmlNode(BASE_LOGLIST_XML_STR)

        self.__append_all_eventlog_with_filter(loglist_node)

        new_loglist_node = self.__remake_loglist_with_option(loglist_node)

        eventlog_node = XmlNode(BASE_EVENT_LOG_XML_STR)
        eventlog_node.append_child(new_loglist_node)
        return eventlog_node

    def __output_log(self, outfile_path, eventlog_node):
        """Out put log to specified file"""

        fh = open(outfile_path, "w")
        fh.write(str(eventlog_node))
        fh.write("\n")
        fh.close()
        return

    #
    # Request handlers
    #

    def __add_handler(self, entry):
        """Handle "add" request."""

        self.messages_xml = self.__get_messages_xml()

        logfile_path = os.path.join(config.SYS_LOG_DIR, EVENTLOG_FILE_NAME)
        log_node = entry.get().get_node(LOG_NAME)
        self.__set_current_time_to_node(log_node)
        self.__log_rotate(logfile_path, True)
        self.__write_log(logfile_path, log_node)

        entry.set_result(Results.OK)
        return entry

    def __rotate_handler(self, entry):
        """Handle "rotate" request."""

        logfile_path = os.path.join(config.SYS_LOG_DIR, EVENTLOG_FILE_NAME)
        self.__log_rotate(logfile_path)
        entry.set_result(Results.OK)
        return entry

    def __get_handler(self, entry):
        """Handle "get" request."""

        if self.__is_exist_root_node(entry):
            if self.__is_exist_node(entry.get(), OPTION_NAME):
                self.__load_option_param(entry)

        self.__load_filter_param()

        eventlog_node = self.__load_eventlog()

        outfile_path = self.__get_entry_file_attr(entry)

        if outfile_path is not None:

            if not os.path.isfile(outfile_path):
                entry.set_result(Results.FILE_NOT_FOUND)
                return entry

            self.__output_log(outfile_path, eventlog_node)

        else:
            entry.set(eventlog_node)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.__total_log_count = 0
        self.__fileter_param_dict = \
            {LEVEL_KEY: [],
             CATEGORY_KEY: [],
             TIMESTAMP_KEY: []}

        self.__option_dict = \
            {INDEX_KEY: None,
             MAXNUM_KEY: None,
             SORT_KEY: None,
             REVERSE_KEY: None,
             FILTER_KEY: "false"}

        def save_and_load_handler(entry):
            return self.send_entry_with_file(
                entry,
                pal_info.PAL_CONFFILE_PATH,
                config.EVENTLOG_CONF_FILE)

        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.ROTATE, self.__rotate_handler)
        self.set_handler(ReqTypes.GET, self.__get_handler)
        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)

#
# Main
#

if __name__ == "__main__":
    FalEventLog(option=falcmd.FalCmdOption()).handle_request()
