#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for producing the information about the product features."""


import pal_info
from iobase import xmldata
from iobase.exception import (InvalidArgError,)
from ionas import request
from libfal import falcmd
from libfal.util import (RequestError, get_current_raid_mode)
from fal_info import (ReqTypes, Results)

import fal_config as config


FEATURES_NAME = "features"
FUNCTIONLIST_NAME = "functionlist"
RAIDVOLUMELIST_NAME = "raidvolumelist"
DEVICEINFO_NAME = "deviceinfo"
FUNCTION_PATH = FUNCTIONLIST_NAME + "/function"
FUNCTION_XPATH_TMPL = """function[text()="%s"]"""

USB_MODELIST_PATH = DEVICEINFO_NAME + "/usblist/usb/modelist"
MODE_NAME = "mode"
VFSEXT_RAID_MODE = "vfsext"
VFSEXT_PORT_MODE = "vfsext"
MIRRORING_PORT_MODE = "mirroring"


class FalFeatures(falcmd.FalCmd):
    """FAL Features command implementation class."""

    def __check_body_node(self, node):
        """Check if entry's body node is valid."""

        if node.get_node_name() != FEATURES_NAME:
            raise InvalidArgError(node.get_node_name())

    def __load_config(self):
        """Load a configuration file for "features" and return XML data."""

        response_entry = self.send_entry_with_file(
                                            request.Entry(ReqTypes.LOAD),
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.FEATURES_CONF_FILE)

        if response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

        return response_entry.get()

    def __save_config(self, xml_data):
        """Save the "xml_data" XML data to a configuration file for
           "features".
        """

        entry = request.Entry(ReqTypes.SAVE)
        entry.set(xml_data)
        response_entry = self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.FEATURES_CONF_FILE)

        if response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

    def __delete_mode(self, modelist, mode_text):
        """Delete mode whose text is "mode_text" in "modelist" XML data ."""

        idx = 0
        for mode in modelist.get_node_list(MODE_NAME):
            if mode.get_text() == mode_text:
                modelist.delete_child(MODE_NAME, index=idx)
                continue
            idx += 1

    def __update_usb_modelist(self, features):
        """Update modelist of usb in "features" XML data."""

        raid_mode = get_current_raid_mode(config.INTERNAL_VOL)
        if raid_mode == VFSEXT_RAID_MODE:
            for modelist in features.get_node_list(USB_MODELIST_PATH):
                self.__delete_mode(modelist, MIRRORING_PORT_MODE)
        else:
            for modelist in features.get_node_list(USB_MODELIST_PATH):
                self.__delete_mode(modelist, VFSEXT_PORT_MODE)

    def __get_function_list(self, config):
        """Return a "funcion"'s text list ."""

        return [function.get_text() for function in
                        config.get_node_list(FUNCTION_PATH)]

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        self.__check_body_node(entry.get())

        try:
            config = self.__load_config()
            features = entry.get()
            features.delete_children(xmldata.ANY_CHILD_PATH)
            features.append_child(config.get_node(FUNCTIONLIST_NAME))
            features.append_child(config.get_node(RAIDVOLUMELIST_NAME))
            features.append_child(config.get_node(DEVICEINFO_NAME))

            self.__update_usb_modelist(features)
            entry.set_result(Results.OK)

        except RequestError, err:
            entry.set_result(err.get_result())

        return entry

    def __add_handler(self, entry):
        """Handle add request and set entry to result."""

        self.__check_body_node(entry.get())

        try:
            config = self.__load_config()

            config_functionlist = config.get_node(FUNCTIONLIST_NAME)
            for function in entry.get().get_node_list(FUNCTION_PATH):
                if not function.get_text() in self.__get_function_list(config):
                    config_functionlist.append_child(function.get_clone())

            self.__save_config(config)
            entry.set_result(Results.OK)

        except RequestError, err:
            entry.set_result(err.get_result())

        return entry

    def __delete_handler(self, entry):
        """Handle delete request and set entry to result."""

        self.__check_body_node(entry.get())

        try:
            config = self.__load_config()

            for function in entry.get().get_node_list(FUNCTION_PATH):
                config.delete_nodes(FUNCTIONLIST_NAME,
                                FUNCTION_XPATH_TMPL % (function.get_text(),))

            self.__save_config(config)
            entry.set_result(Results.OK)

        except RequestError, err:
            entry.set_result(err.get_result())

        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.FEATURES_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)


#
# Main
#

if __name__ == "__main__":
    FalFeatures(option=falcmd.FalCmdOption()).handle_request()
