#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Firmware Updating."""

import sys
import os
import hashlib

import iobase.xmldata as xmldata
from iobase.exception import NotFoundError

import ionas.request as request
import ionas.nascmd as nascmd

import libfal.falcmd as falcmd
import libfal.util as util

from fal_info import (ReqTypes, Results)
import fal_config as config

import pal_info


MD5_TYPE = "md5"
CHECKSUM_NAME = "checksum"
CHECKSUMTYPE_NAME = "checksumtype"

APPLY_TAG_NAME_LIST = ("updatenotice", "autoupdate", "time")


class FalFirmware(falcmd.FalCmd):
    """FAL Firmware command implementation class."""

    def __download_firm_file(self, filename):
        """File download."""

        proxy = None
        full_url = ""
        try:
            usb_path_list = self.firmware_config.get_node_list(
                            "config/localinfo/pathlist/path")
            for node in usb_path_list:
                file_path = os.path.join(node.get_text(), filename)
                if os.path.exists(file_path):
                    full_url = "file://" + file_path
                    break
        except:
            pass

        if full_url == "":
            serverinfo = self.firmware_config.get_node("config/serverinfo")
            protocol = serverinfo.get_node("protocol").get_text()
            host = serverinfo.get_node("hostname").get_text()
            path = serverinfo.get_node("path").get_text()
            full_url = protocol + "://" + host + "/" + path + "/" + filename
            try:
                proxy = util.get_proxy(full_url)
            except:
                pass

        if not os.path.exists(config.FIRMWARE_TEMP_DIR):
            os.mkdir(config.FIRMWARE_TEMP_DIR)
        local_path = os.path.join(config.FIRMWARE_TEMP_DIR, filename)

        util.urlaccess_receive(full_url, local_path, proxy=proxy)
        return

    def __get_firm_file_node(self):
        """Get tgz file node. """

        file_path = os.path.join(config.FIRMWARE_TEMP_DIR,
                                    self.__get_infofile_name())
        if os.path.exists(file_path):
            data = xmldata.XmlData(file_path)
            return data.get_node("software/file")
        else:
            return

    def __get_infofile_name(self):
        """Get infofile name."""

        return self.firmware_config.get_value("config/serverinfo/infofile")

    def __update_handler(self, entry):
        """Handle update request and set entry to result."""

        file_node = self.__get_firm_file_node()
        name = file_node.get_text()
        if file_node.get_attr_value(CHECKSUMTYPE_NAME) != MD5_TYPE:
            self.error("No supported checksum type: %s" %
                            (file_node.get_attr_value(CHECKSUMTYPE_NAME),))
            entry.set_result(Results.INVALID_FILE)
            return entry
        checksum = file_node.get_attr_value(CHECKSUM_NAME).lower()

        firm_path = ""
        try:
            self.__download_firm_file(name)

            firm_path = os.path.join(config.FIRMWARE_TEMP_DIR, name)
            md5check = hashlib.md5()
            f = file(firm_path, "r")
            while True:
                blkdata = f.read(100)
                if blkdata == "":
                    break
                md5check.update(blkdata)
            if md5check.hexdigest() != checksum:
                self.error("Checksum error: %s - %s" %
                            (md5check.hexdigest(), checksum))
                entry.set_result(Results.INVALID_FILE)
                return entry
        except:
            self.exception("Download error")
            entry.set_result(Results.DOWNLOAD_ERROR)
            return entry

        entry.set_file(firm_path)
        result_entry = self.send_entry(entry, pal_info.PAL_FIRMWARE_PATH)
        return result_entry

    def __execute_handler(self, entry):
        """Handle execute request and set entry to result."""

        pal_entry = request.Entry(ReqTypes.UPDATE)
        pal_entry.set_file(entry.get_file())
        result_entry = self.send_entry(pal_entry, pal_info.PAL_FIRMWARE_PATH)
        return result_entry

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        info_file = self.__get_infofile_name()
        try:
            self.__download_firm_file(info_file)
            firm_info = xmldata.XmlData(os.path.join(config.FIRMWARE_TEMP_DIR,
                                            info_file))
            entry.get().append_child(firm_info.get_node("software"))
            entry.set_result(Results.OK)
        except:
            self.exception("Download error")
            entry.set_result(Results.DOWNLOAD_ERROR)
        return entry

    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        for tag_name in APPLY_TAG_NAME_LIST:
            try:
                src_node = entry.get().get_node(tag_name)
            except:
                continue

            try:
                dest_node = self.firmware_config.get_node(tag_name)
            except NotFoundError:
                self.firmware_config.append_child(src_node)
                continue

            dest_node.update(src_node, override=True)

        util.save_config(config.FIRMWARE_CONF_FILE, self.firmware_config)

        # event
        util.dispatch_event("firmware", "apply", entry.get())

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(
            entry, pal_info.PAL_CONFFILE_PATH, config.FIRMWARE_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.UPDATE, self.__update_handler)
        self.set_handler(ReqTypes.EXECUTE, self.__execute_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)

        try:
            self.firmware_config = util.load_config(config.FIRMWARE_CONF_FILE)
        except:
            self.exception("load configuration failed.\n")
            sys.exit(nascmd.INTERNAL_ERROR)


#
# Main
#

if __name__ == "__main__":
    FalFirmware(option=falcmd.FalCmdOption()).handle_request()
