#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting FTP"""

import os
import time

from iobase.exception import NotFoundError
import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info
import libfal.util as util
from fal_info import (ReqTypes, Results)
import fal_config as config

FTP = "ftp"
FTP_ROOT_NAME = "ftp"
FTP_PACKAGE_NAME = "ftp"
FTP_CTRL_PORT_NAME = "ftp_control"
FTP_SERVICE_NAME = config.FUNC_TO_SERVICE_MAP["ftp"]


class FalFtp(falcmd.FalCmd):
    """FAL FTP command implementation class."""
    
    def __set_used_port(self, entry, unset=False):

        if not unset:
            port = entry.get().get_value("port[@type='control']/number")
            util.set_used_port(port, FTP_CTRL_PORT_NAME)
        else:
            util.unset_used_port(FTP_CTRL_PORT_NAME)

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        if not util.is_installed_package(FTP_PACKAGE_NAME):
            entry.set_result(Results.OK)
            return entry

        root_name = entry.get().get_node_name()

        if root_name == FTP_ROOT_NAME:
            # ftpサービス設定処理
            if entry.get().get_node("service").is_enabled():
                util.servicectrl_stop([], [FTP_SERVICE_NAME])
                apply_result = self.send_entry(entry, pal_info.PAL_PROFTPD_PATH)
                if apply_result.get_result() != Results.OK:
                    return apply_result

                util.enable_service(FTP, enable=True)
                util.servicectrl_start([], [FTP_SERVICE_NAME])
                self.__set_used_port(entry)
            else:
                util.enable_service(FTP, enable=False, do_start_stop=True)
                self.__set_used_port(entry, unset=True)

            save_entry = request.Entry(ReqTypes.SAVE, str(entry.get()))
            save_result_entry = self.send_entry_with_file(
                    save_entry,
                    pal_info.PAL_CONFFILE_PATH,
                    config.FTP_CONF_FILE)
            if save_result_entry.get_result() != Results.OK:
                return save_result_entry
        else:
            # ftpホスト名設定処理
            apply_result = self.send_entry(entry, pal_info.PAL_PROFTPD_PATH)
            if apply_result.get_result() != Results.OK:
                return apply_result

        entry.set_result(Results.OK)
        return entry

    # FIXME 共有設定時のハンドラは
    # FALレイヤーとして動くので、
    # fal_ftpにhandle_apply_handlerは不要か?
    def __handle_apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        return self.send_entry(entry, pal_info.PAL_PROFTPD_PATH)

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_load_handler = lambda entry: self.send_entry_with_file(entry,
                        pal_info.PAL_CONFFILE_PATH,
                        config.FTP_CONF_FILE)

        update_handler = lambda entry: self.send_entry(entry,
                        pal_info.PAL_PROFTPD_PATH)

        self.set_handler(ReqTypes.LOAD, save_load_handler)
        self.set_handler(ReqTypes.SAVE, save_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.HANDLE_APPLY, self.__handle_apply_handler)
        self.set_handler(ReqTypes.UPDATE, update_handler)

#
# Main
#

if __name__ == "__main__":
    FalFtp(option=falcmd.FalCmdOption()).handle_request()
