#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Group Settings."""


import os

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info
from fal_info import ReqTypes
from fal_info import Results
from iobase import xmldata
from iobase.xmldata import ANY_CHILD_NAME
from iobase.exception import (NotFoundError, InvalidArgError, ExecError)
from libfal.util import dispatch_event

import fal_config as config

ID = "id"
GID = "gid"
NAME = "name"
GROUP = "group"
GROUPLIST_PATH = "./grouplist"
GROUP_PATH = GROUPLIST_PATH + "/group"
ID_PATH = GROUP_PATH + "/id"
NAME_ENTRY_PATH = "./group/name"
ID_ENTRY_PATH = "./group/id"
USER_ENTRY_PATH = "./group/userlist/user"
USER_ENTRY = "<users/>"
XPATH_GROUP_BYNAME = "grouplist/group" + \
        "[(not(@hidden) or @hidden!='true') and not(@domain) and name='%s']"

MIN_ID = 1000
MAX_GROUP_NUM = 100
QUOTA_VOL = config.VOL_INFO[config.STOR_IF_TO_VOL['internal'][1]]['mnt_point']


class FalGroups(falcmd.FalCmd):
    """FAL Group command implementation class."""

    def __is_empty(self, val):
        if val in (None, "", [], (), {}):
            return True
        return False

    def __set_quota(self, group, limit):
        quota_node = xmldata.XmlNode("""<quota>
            <grouplist>
                <group>
                    <name />
                    <quota><limit /></quota>
                </group>
            </grouplist>
        </quota>""")
        quota_node.get_node('grouplist/group/name').set_text(group)
        quota_node.get_node('grouplist/group/quota').set_attrs(
                {'volume': QUOTA_VOL})
        quota_node.get_node('grouplist/group/quota/limit').set_text(limit)
        entry = self.send_entry(
                request.Entry(ReqTypes.APPLY, quota_node),
                pal_info.PAL_QUOTA_PATH)
        if entry.get_result() == Results.OK:
            return True
        else:
            return False

    def __get_user_config(self):
        """Load user config and return XML data ."""

        return self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                config.USERS_CONF_FILE).get()

    def __get_group_config(self):
        """Load group config and return XML data ."""

        return self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                config.GROUPS_CONF_FILE).get()

    def __query_handler(self, entry):
        """Handle group quota information request and set entry to resutl."""

        # pal_quotaからグループ毎の使用容量を取得
        try:

            def __pal_quota_query(self):
                xml = '<quota><volume>%s</volume></quota>' % QUOTA_VOL
                return self.send_entry(
                        request.Entry(ReqTypes.QUERY, xml),
                        pal_info.PAL_QUOTA_PATH)

            def __get_group_node_list(self):
                xpath = 'grouplist/group'
                return __pal_quota_query(self).get().get_node_list(xpath)

            def __get_used_dict(self):
                return dict([(n.get_value('name'), n.get_value('quota/used')) \
                        for n in __get_group_node_list(self)])

            used_dict = __get_used_dict(self)
        except:
            raise ExecError(pal_info.PAL_QUOTA_PATH)

        # レスポンスentry作成
        groups_node = entry.get()
        groups_node.append_child('grouplist')
        grouplist_node = groups_node.get_node('grouplist')
        for group_node_conf in \
                self.__get_group_config().get_node_list('grouplist/group'):
            name = group_node_conf.get_value('name')
            group_node = xmldata.XmlNode('group')
            group_node.append_child('name', text=name)
            group_node.append_child('quota')
            group_node.get_node('quota').append_child(
                    'used', text=used_dict.get(name, '0'))
            grouplist_node.append_child(group_node)
        entry.set_result(Results.OK)

        return entry

    def __get_group_id_list(self):
        """Get a list of group id."""

        group_config = self.__get_group_config()
        return [id_node.get_text() for id_node in \
                group_config.get_node_list(ID_PATH)]

    def __check_groupid_enable(self, entry, id_list):
        """Check if the group ID in entry is enable."""

        group_id = entry.get().get_node(ID_ENTRY_PATH).get_text()
        if group_id != "":
            if group_id in id_list:
                raise InvalidArgError("Group ID %s has been used." % group_id)
        else:
            id = MIN_ID
            while str(id) in id_list:
                id += 1
            entry.get().get_node(GROUP).get_node(ID).set_text(str(id))
        return

    def __clear_users_primary_group(self, target_group, user_config,
                                    user_list=[]):
        """Clear users' primary group when the user is deleted from
           the userlist of group modify entry.
        """

        group_id = target_group.get_node(ID).get_text()
        target_users = []
        try:
            target_users = user_config.get_node_list(
                            "./userlist/user[gid='%s']" % group_id)
        except NotFoundError:
            pass
        for user in target_users:
            user_name = user.get_node(NAME).get_text()
            if user_name not in user_list:
                user.get_node(GID).delete_text()
                user_set_entry = request.Entry(ReqTypes.MODIFY, USER_ENTRY)
                user_set_entry.get().append_child(user.get_clone())
                result = self.send_entry(user_set_entry,
                                                pal_info.PAL_USERS_PATH)
                if result.get_result() != Results.OK:
                    return result.get_result()

        return Results.OK

#    def __detail_handler(self, entry):
#        """Handle group detail information request and set entry to result."""
#
#        group_name = entry.get().get_node(GROUP).get_node(NAME).get_text()
#        group_config = self.__get_group_config()
#        target_group = group_config.get_node(XPATH_GROUP_BYNAME % group_name)
#        entry.get().delete_child(GROUP)
#        entry.get().append_child(target_group)
#        return entry

    def __delete_handler(self, entry):
        """Handle group delete request and set entry to result."""

        # event
        try:
            dispatch_event(
                    'groups', 'delete', entry.get().get_clone(), previous=True)
        except:
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        result = self.send_entry(entry, pal_info.PAL_GROUPS_PATH)
        if result.get_result() != Results.OK:
            entry.set_result(result.get_result())
            return entry

        delete_name = entry.get().get_node(NAME_ENTRY_PATH).get_text()
        user_config = self.__get_user_config()
        group_config = self.__get_group_config()
        target_group = group_config.get_node(XPATH_GROUP_BYNAME % delete_name)
# FIXME イベントハンドラでやる
#        result = self.__clear_users_primary_group(target_group, user_config)
#        if result != Results.OK:
#            entry.set_result(result)
#            return entry
        group_config.delete_nodes(GROUPLIST_PATH,
                 "./group[(not(@hidden) or @hidden!='true') and \
                 not(@domain) and name='%s']" % delete_name)

        group_save_entry = request.Entry(ReqTypes.SAVE, group_config)
        self.send_entry_with_file(
                group_save_entry,
                pal_info.PAL_CONFFILE_PATH,
                config.GROUPS_CONF_FILE)
#        user_save_entry = request.Entry(ReqTypes.SAVE, user_config)
#        self.send_entry_with_file(
#                user_save_entry,
#                pal_info.PAL_CONFFILE_PATH,
#                config.USERS_CONF_FILE)

        # event
        try:
            dispatch_event('groups', 'delete', entry.get().get_clone())
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __add_handler(self, entry):
        """Handle group add request and set entry to result."""

        # event
        try:
            dispatch_event(
                    'groups', 'add', entry.get().get_clone(), previous=True)
        except:
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        # FIXME 全体的に関数化検討
        group_config = self.__get_group_config()

        # グループ数確認(既に上限数に達していたらエラー)
        # FIXME 上限越えたら隠しユーザーも作成できなくなる
        #       このチェックを非隠しユーザーの場合のみに修正する必要有
        if len(group_config.get_node_list(
                'grouplist/group[@hidden="false"]')) >= MAX_GROUP_NUM:
            entry.set_result(Results.MAX_EXCEED_ERROR)
            return entry

        # リクエストパラメータ取得
        # FIXME domain等の属性パラメータの受け取り
        group_node = entry.get().get_node('group')
        name = group_node.get_value('name', None)
        if self.__is_empty(name):
            raise InvalidArgError('Group name empty!!')
        #gid = group_node.get_value('gid', None)
        gid = group_node.get_value('id', None)
        quota = group_node.get_value('quota/limit', None)
        members = [n.get_value('name', None) \
                for n in group_node.get_node_list('userlist/user')]
        hidden = group_node.is_set_to_true('hidden')
        reserved = group_node.is_set_to_true('reserved')
        domain = group_node.get_value('@domain', None)

        # gid処理
        gids = [int(node.get_text()) \
                for node in group_config.get_node_list('grouplist/group/id')]
#                for node in group_config.get_node_list('grouplist/group/gid')]
        if gid is None:
            gid = int(MIN_ID)
            while gid in gids:
                gid += 1
        else:
            if int(gid) in gids:
                raise InvalidArgError('Group ID %s has been used.' % gid)

        # FIXME 同名グループのチェックは不要か?
        #       pal_groups にリクエストするとエラーになるはず

        # pal_groups に add リクエスト
        add_groups_node = xmldata.XmlNode('<groups><group /></groups>')
        add_group_node = add_groups_node.get_node('group')
        add_group_node.append_child('name', text=name)
        add_group_node.append_child('gid', text=str(gid))
        add_group_node.append_child('userlist')
        add_userlist_node = add_group_node.get_node('userlist')
        for member in members:
            # FIXME 実在するユーザーかチェックした方がいいかも
            if self.__is_empty(member):
                continue
            add_user_node = xmldata.XmlNode('<user />')
            add_user_node.append_child('name', text=member)
            add_userlist_node.append_child(add_user_node)
        add_res_entry = self.send_entry(
                request.Entry(ReqTypes.ADD, add_groups_node),
                pal_info.PAL_GROUPS_PATH)
        if add_res_entry.get_result() != Results.OK:
            entry.set_result(add_res_entry.get_result())
            return entry

        # pal_quota にリクエストして使用量制限を設定
        if quota is not None and not self.__set_quota(name, quota):
            quota = None

        # 設定ファイルに書き出す
        # FIXME 書き出すxmlの体裁をaddとmodifyで統一するためのメソッド作る
        added_group_node = add_res_entry.get().get_node('group')
        added_group_node.append_child('id', added_group_node.get_value('gid'))
        added_group_node.delete_child('gid')
        if not quota is None:
            added_group_node.append_child('quota')
            added_group_node.get_node('quota').append_child('limit', str(quota))
        if domain is not None:
            added_group_node.set_attrs({'domain': domain})
        added_group_node.set_bool_attr('hidden', hidden)
        added_group_node.set_bool_attr('reserved', reserved)
        group_config.get_node('grouplist').append_child(added_group_node)
        self.send_entry_with_file(
                request.Entry(ReqTypes.SAVE, group_config),
                pal_info.PAL_CONFFILE_PATH,
                config.GROUPS_CONF_FILE)

        # 正常終了を返す
        group_config = self.__get_group_config()
        groups_node = entry.get()
        groups_node.clear()
        groups_node.append_child(
                group_config.get_node('grouplist/group[name="%s"]' % name))

        # event
        try:
            dispatch_event('groups', 'add', entry.get().get_clone())
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __modify_handler(self, entry):
        """Handle group modify request and set entry to result."""

        # event
        try:
            dispatch_event(
                    'groups', 'modify', entry.get().get_clone(), previous=True)
        except:
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        # リクエストパラメータ取得
        # FIXME ドメイン等の属性パラメータ
        group_node = entry.get().get_node('group')
        name = group_node.get_value('name', None)
        if self.__is_empty(name):
            raise InvalidArgError('Group name empty!!')
        #gid = group_node.get_value('gid', None)
        gid = group_node.get_value('id', None)
        quota = group_node.get_value('quota/limit', None)
        try:
            members = [node.get_value('name', None) for node in \
                    group_node.get_node('userlist').get_node_list('user')]
        except NotFoundError:
            members = None

        # 存在しないグループが指定されたらエラー
        group_config = self.__get_group_config()
        try:
            config_group_node = group_config.get_node(
                    'grouplist/group[name="%s"]' % name)
        except NotFoundError:
            raise InvalidArgError("%s is not exists!!" % name)

        # pal_groups に modify リクエスト
        # FIXME
        mod_res_entry = self.send_entry(entry, pal_info.PAL_GROUPS_PATH)
        result = mod_res_entry.get_result()
        if result != Results.OK:
            entry.set_result(result)
            return entry

        # FIXME ユーザーがメンバーから外れたときユーザー設定のgidをケアする
        #       イベントハンドラに登録しておけばいいと思われる

        # 使用量制限が変わっていたら変える
        if quota is not None and not self.__set_quota(name, quota):
            quota = None

        # 設定ファイルに書き出す
        config_group_node.update(group_node)
        if members is not None:
            userlist_node = config_group_node.get_node('userlist')
            userlist_node.delete_children('user')
            for member in members:
                user_node = xmldata.XmlNode('<user />')
                user_node.append_child('name', text=member)
                userlist_node.append_child(user_node)
        self.send_entry_with_file(
                request.Entry(ReqTypes.SAVE, group_config),
                pal_info.PAL_CONFFILE_PATH,
                config.GROUPS_CONF_FILE)

        # 正常終了を返す
        group_config = self.__get_group_config()
        entry_groups_node = entry.get()
        entry_groups_node.clear()
        entry_groups_node.append_child(
                group_config.get_node('grouplist/group[name="%s"]' % name))

        # event
        try:
            dispatch_event('groups', 'modify', entry.get().get_clone())
        except:
            pass

        entry.set_result(Results.OK)
        return entry
#
#        result = self.send_entry(entry, pal_info.PAL_GROUPS_PATH)
#        if result.get_result() != Results.OK:
#            entry.set_result(result.get_result())
#            return entry
#
#        user_list = []
#        try:
#            entry_userlist = entry.get().get_node_list(USER_ENTRY_PATH)
#            for gp_user in entry_userlist:
#                user_list.append(gp_user.get_node(NAME).get_text())
#        except NotFoundError:
#            pass
#        entry_name = entry.get().get_node(NAME_ENTRY_PATH).get_text()
#        user_config = self.__get_user_config()
#        group_config = self.__get_group_config()
#        target_group = group_config.get_node(XPATH_GROUP_BYNAME % entry_name)
#        result = self.__clear_users_primary_group(target_group,
#                                                user_config, user_list)
#        if result != Results.OK:
#            entry.set_result(result)
#            return entry
#        infos = entry.get().get_node(GROUP).get_node_list(ANY_CHILD_NAME)
#        for node in infos:
#            if node.get_node_name() == ID:
#                continue
#            try:
#                target_group.delete_child(node.get_node_name())
#            except NotFoundError:
#                pass
#            target_group.append_child(node)
#
#        gp_save_entry = request.Entry(ReqTypes.SAVE, group_config)
#        self.send_entry_with_file(gp_save_entry, pal_info.PAL_CONFFILE_PATH,
#                                              config.GROUPS_CONF_FILE)
#        user_save_entry = request.Entry(ReqTypes.SAVE, user_config)
#        self.send_entry_with_file(user_save_entry, pal_info.PAL_CONFFILE_PATH,
#                                              config.USERS_CONF_FILE)
#        entry.set_result(Results.OK)
#        return entry

    def __get_workgroup(self):
        msnd = self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                config.MSNETWORK_CONF_FILE).get()
        method = msnd.get_value('assign/@method')
        if method == "wg":
            return msnd.get_value('assign/wg/workgroup', None)
        elif method == "nt":
            return msnd.get_value('assign/nt/domain', None)
        elif method == "ad":
            return msnd.get_value('assign/ad/netbios', None)
        return None

    def __get_domain_group(self):
        entry = self.send_entry(
                request.Entry(ReqTypes.LIST, '<winbind />'),
                pal_info.PAL_WINBIND_PATH)
        glist = [gnd.get_value('name') \
                for gnd in entry.get().get_node_list('grouplist/group') \
                    if gnd.get_value('name', None)]
        workgroup = self.__get_workgroup() or ""
        glist_ = []
        for upn in glist:
            if not upn:
                continue
            (domain, name) = upn.split(config.WINBIND_SEPARATOR)[:2]
            if not domain.lower() == workgroup.lower():
                continue
            glist_.append(upn)
        return glist_

    def __domain_list_handler(self, entry):
        """<entry type="domain_list">
            <groups />
        </entry>"""
        """<entry type="domain_list" result="0">
            <groups>
                <grouplist>
                    <group>
                        <upn>DOMAIN@GROUP001</upn>
                        <domain>DOMAIN</domain>
                        <name>GROUP001</name>
                        <quota><limit>100</limit></quota>
                    </group>
                </grouplist>
            </groups>
        </entry>"""

        glist = self.__get_domain_group()

        # quota設定値の取得
        try:

            def __pal_quota_query(self):
                xml = '<quota><volume>%s</volume></quota>' % QUOTA_VOL
                return self.send_entry(
                        request.Entry(ReqTypes.QUERY, xml),
                        pal_info.PAL_QUOTA_PATH)

            def __get_group_node_list(self):
                xpath = 'grouplist/group'
                return __pal_quota_query(self).get().get_node_list(xpath)

            def __get_limit_dict(self):
                return dict([(n.get_value('name'), n.get_value('quota/limit')) \
                        for n in __get_group_node_list(self)])

            limit_dict = __get_limit_dict(self)
        except:
            raise ExecError(pal_info.PAL_QUOTA_PATH)

        groups_node = entry.get()
        groups_node.set_node_name('groups')  # 念のため
        groups_node.clear()  # 念のため
        groups_node.append_child('grouplist')
        grouplist_node = groups_node.get_node('grouplist')
        for gname in glist:
            upn = gname
            try:
                (domain, name) = upn.split(config.WINBIND_SEPARATOR)[:2]
                gnode = xmldata.XmlNode('group')
                gnode.append_child('upn', text=upn)
                gnode.append_child('domain', text=domain)
                gnode.append_child('name', text=name)
                gnode.append_child('quota')
                gnode.get_node('quota').append_child('limit', text=limit_dict.get(upn, '0'))
                grouplist_node.append_child(gnode)
            except:
                continue

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.GROUPS_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.MODIFY, self.__modify_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)
#        self.set_handler(ReqTypes.DETAIL, self.__detail_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.DOMAIN_LIST, self.__domain_list_handler)


#
# Main
#

if __name__ == "__main__":
    FalGroups(option=falcmd.FalCmdOption()).handle_request()
