#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Initialize."""


import os
import re

import ionas.request as request
from iobase.xmldata import XmlNode
from libfal import (util, falcmd)
import pal_info
from fal_info import ReqTypes
from fal_info import Results
import fal_config as config

REBOOT_ENTRY = "<powersetting><option force='true'/></powersetting>"
LABEL_VER = "3"
LABEL_USAGE = "RAID"

INITIALIZE_NAME = "initialize"
DISPACH_OK = 0
DISPACH_FAIL = 1


class FalInitialize(falcmd.FalCmd):
    """FAL Initialize command implementation class."""

    def __dispatch_initialize_event(self):
        try:
            initialize_node = XmlNode(INITIALIZE_NAME)
            util.dispatch_event('system', 'initialize', initialize_node, True)
        except:
            self.exception("fail to __dispatch_initialize_event")
            return DISPACH_FAIL
        return DISPACH_OK

    def __get_init_disk_list(self):
        disk_list = []
        hdd_dev_list_key = min(config.HDD_DEV_LIST_IN_VOL.keys())
        for disk in config.HDD_DEV_LIST_IN_VOL[hdd_dev_list_key]["sataraid1"]:
            if disk in config.SATA_HDD_DEV_LIST:
                disk_list.append(disk)

        return disk_list

    def __is_disk_label_correct(self):
        for disk in self.__get_init_disk_list():
            disklabel = util.get_disk_label_info(disk)
            if disklabel.get("version") == LABEL_VER:
                if disklabel.get("usage") == LABEL_USAGE:
                    if disklabel.get("model"):
                        if re.match(config.LABEL_MODEL_RE, disklabel["model"]):
                            return True
        return False

    def __all_disk_exist(self):
        for name in self.__get_init_disk_list():
            disk_path = os.path.join(config.DEV_DIR, name)
            if not os.path.exists(disk_path):
                return False
        return True

    def __initialize_handler(self, entry):
        """Handle initialize request and set entry to result."""

        if self.__dispatch_initialize_event() == DISPACH_FAIL:
            entry.set_result(Results.INITIALIZE_ERROR)
            return entry

        if not self.__all_disk_exist():
            entry.set_result(Results.DEVICE_NOT_FOUND)
            return entry

        if not self.__is_disk_label_correct():
            entry.set_result(Results.INITIALIZE_ERROR)
            return entry

        pal_result = self.send_entry(entry, pal_info.PAL_INITIALIZE_PATH)
        if pal_result.get_result() == Results.OK:
            shutdown_entry = request.Entry(ReqTypes.REBOOT, REBOOT_ENTRY)
            shutdown_result = self.send_entry(shutdown_entry,
                                                pal_info.PAL_POWER_PATH)
            if shutdown_result.get_result() != Results.OK:
                entry.set_result(shutdown_result.get_result())
                return entry
        return pal_result

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.INITIALIZE, self.__initialize_handler)


#
# Main
#

if __name__ == "__main__":
    FalInitialize(option=falcmd.FalCmdOption()).handle_request()
