#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for LED."""

from __future__ import with_statement

import os
import ionas.request as request
from iobase.exception import (NotFoundError,)
import libfal.falcmd as falcmd
import pal_info
import fal_config as config
from fal_info import (ReqTypes, Results)
import product_conf
import iobase.fileut as fileut
from iobase.xmldata import (XmlNode, XmlConfNode)
from libfal.util import (RequestError,)

### XML Tag Name ###
BRIGHTNESS_NAME = "brightness"
STATUS_NAME = "status"
HDD_NAME = "hdd"

### XML Element Value ###
BRIGHTNESS_HIGH_VALUE = "high"
BRIGHTNESS_MIDDLE_VALUE = "middle"
BRIGHTNESS_LOW_VALUE = "low"

### XML Attribute ###
COLOR_ATTR = "color"
HDDNUM_ATTR = "hddnum"
TYPE_ATTR = "type"

### XML Attribute Value ###
ACT_ATTR_VALUE = "act"

### LEDのFAL/PAL輝度レベルのマップ ###
BRIGHTNESS_MAP = {"high": "100",
                  "middle": "75",
                  "low": "50"}

### SYSTEM STATE ###
SYS_ERROR_STATE = "syserr"
SYS_WARN_STATE = "syswarn"
SYS_NORMAL_STATE = "sysnor"

### Some value ###
ANY_CHILD_NAME = "*"

SETTING_LED_COUNT_DIR = os.path.join(product_conf.TEMP_DIR, "busy_led")
SETTING_LED_COUNT_FILE = "busy_led_count"
SETTING_LED_COUNT_PATH = os.path.join(SETTING_LED_COUNT_DIR,
                                      SETTING_LED_COUNT_FILE)

STATUS_GREEN_ON_XML = \
    '<led><status color="green">on</status></led>'
ANNOUNCE_XML_DEFAULT = "<announcelist/>"


class FalLed(falcmd.FalCmd):
    """FAL LED command implementation class."""

    def __is_file_exists(self, target_path):
        """Check path existence."""

        target = target_path
        if not os.path.isabs(target):
            target = os.path.abspath(target_path)

        return os.path.exists(target)

    def __update_counter(self, count):
        """Update contents of a counter file with "count".
           A directory and file for counter are created, if it doesn't exist.
        """

        if not os.path.exists(SETTING_LED_COUNT_DIR):
            os.makedirs(SETTING_LED_COUNT_DIR)

        with open(SETTING_LED_COUNT_PATH, "w") as counter:
            counter.write(str(count))

    def __get_count(self):
        """Return a current count of "busy led" counter."""

        try:
            with open(SETTING_LED_COUNT_PATH) as counter:
                return int(counter.read())
        except:
            return 0

    def __count_down_setting_led_count(self):
        """Remount root filesystem as read-only."""

        with fileut.FLock(product_conf.LOCK_DIR, SETTING_LED_COUNT_FILE, True):
            count = self.__get_count()
            if count <= 0:
                return -1

            count -= 1
            self.__update_counter(count)
            return count

    def __count_up_setting_led_count(self):
        """Remount root filesystem as write."""

        with fileut.FLock(product_conf.LOCK_DIR, SETTING_LED_COUNT_FILE, True):
            count = self.__get_count() + 1
            self.__update_counter(count)
            return count

    def __is_exist_node(self, node, node_path):
        """Check whether node_path exists in specified node"""

        try:
            node.get_node(node_path)
        except NotFoundError:
            return False
        return True

    def __update_brightness_text(self, entry):
        """Update "brightness" node's text for "pal_led"."""

        brightness = entry.get().get_node(BRIGHTNESS_NAME)
        brightness.set_text(BRIGHTNESS_MAP[brightness.get_text()])

    def __get_announce_node(self):
        """Load announce node from config file."""

        if not self.__is_file_exists(config.ANNOUNCE_FILE):
            return XmlConfNode(ANNOUNCE_XML_DEFAULT)

        response_entry = self.send_entry_with_file(
            request.Entry(ReqTypes.LOAD),
            pal_info.PAL_CONFFILE_PATH,
            config.ANNOUNCE_FILE)

        if response_entry.get_result() == Results.FILE_NOT_FOUND:
            return XmlConfNode(ANNOUNCE_XML_DEFAULT)
        elif response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

        return response_entry.get()

    def __get_system_state(self):

        # announceファイルを基にシステムの状態取得
        # level=2：エラー状態
        # level=1：警告状態
        # announceなし：通常状態

        ERR_ANNOUNCE_XPATH = "announce[@level='2']"
        WARN_ANNOUNCE_XPATH = "announce[@level='1']"

        try:
            announce_list_node = self.__get_announce_node()
        except:
            return SYS_NORMAL_STATE

        if self.__is_exist_node(announce_list_node, ERR_ANNOUNCE_XPATH):
            return SYS_ERROR_STATE

        if self.__is_exist_node(announce_list_node, WARN_ANNOUNCE_XPATH):
            return SYS_WARN_STATE

        return SYS_NORMAL_STATE

    def __convert_status_led_node(self, led_node):

        LED_NORMAL = "n-on"
        LED_SETTING = "n-bl"
        LED_WARN = "warn"
        LED_ERROR = "err"
        GREEN = "green"
        RED = "red"

        _ON = "on"
        _OFF = "off"
        _BLINK = "blink"

        def _action_set_error(led_node):

            led_node.get_node(STATUS_NAME).set_attrs({COLOR_ATTR: RED})
            led_node.get_node(STATUS_NAME).set_text(_BLINK)

        def _action_set_warn(led_node):

            led_node.get_node(STATUS_NAME).set_attrs({COLOR_ATTR: RED})
            led_node.get_node(STATUS_NAME).set_text(_ON)

        def _action_filter_out(led_node):

            led_node.delete_child(STATUS_NAME)

        def _pass(led_node):
            pass

        # 現在のシステム状態
        # - 上位からリクエストされた LED 設定
        # - 変換関数
        # 例)
        # システムがエラー状態
        # - 正常点灯リクエスト
        # - エラー表示
        STATUS_LED_CONVERTER_TABLE = \
            {SYS_ERROR_STATE: {LED_NORMAL: _action_set_error,
                               LED_SETTING: _pass,
                               LED_WARN: _action_set_error,
                               LED_ERROR: _pass},
             SYS_WARN_STATE: {LED_NORMAL: _action_set_warn,
                              LED_SETTING: _pass,
                              LED_WARN: _pass,
                              LED_ERROR: _pass},
             SYS_NORMAL_STATE: {LED_NORMAL: _pass,
                                LED_SETTING: _pass,
                                LED_WARN: _pass,
                                LED_ERROR: _pass}}
        LED_TYPE_TABLE = \
            {GREEN: {_ON: LED_NORMAL, _BLINK: LED_SETTING},
             RED: {_ON: LED_WARN, _BLINK: LED_ERROR}}

        system_state = self.__get_system_state()

        color = led_node.get_node(STATUS_NAME).get_attr_value(COLOR_ATTR)
        mode = led_node.get_node(STATUS_NAME).get_text()
        if _OFF == mode:
            return

        led_type = LED_TYPE_TABLE[color][mode]

        # 緑点滅(システム BSY 状態)にする人が複数いた
        # 場合、全員が BSY 解除して初めて緑点灯状態に
        # したいのでカウンタを使う
        if LED_NORMAL == led_type:
            # まだ他の人が緑点滅させてる場合
            if 1 <= self.__count_down_setting_led_count():
                led_node.get_node(STATUS_NAME).set_attrs({COLOR_ATTR: GREEN})
                led_node.get_node(STATUS_NAME).set_text(_BLINK)
                led_type = LED_SETTING

        elif LED_SETTING == led_type:
            # BSY状態のカウントアップ
            self.__count_up_setting_led_count()

        STATUS_LED_CONVERTER_TABLE[system_state][led_type](led_node)
        return

    def __control_handler(self, entry):
        """Handle control request."""

        pal_entry = entry.get_clone()

        try:
            self.__update_brightness_text(pal_entry)
        except NotFoundError:
            pass

        led_node = pal_entry.get()

        if self.__is_exist_node(led_node, STATUS_NAME):
            self.__convert_status_led_node(led_node)

        # 変換した結果空になってたらやることないので
        # そのまま終了
        if len(pal_entry.get().get_node_list(ANY_CHILD_NAME)) == 0:
            entry.set_result(Results.OK)
            return entry

        result_entry = self.send_entry(pal_entry, pal_info.PAL_LED_PATH)
        entry.set_result(result_entry.get_result())
        return entry

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        ctrl_entry = request.Entry(ReqTypes.CONTROL,
                            pal_info.PAL_LED_REQ_BASE_XML[ReqTypes.CONTROL])
        ctrl_entry.get().append_child(
                        entry.get().get_node(BRIGHTNESS_NAME).get_clone())
        self.__update_brightness_text(ctrl_entry)
        result_entry = self.send_entry(ctrl_entry, pal_info.PAL_LED_PATH)
        if Results.OK != result_entry.get_result():
            entry.set_result(result_entry.get_result())
            return entry

        response_entry = self.send_entry_with_file(
                                    request.Entry(ReqTypes.LOAD),
                                    pal_info.PAL_CONFFILE_PATH,
                                    config.LED_CONF_FILE)
        if Results.OK != response_entry.get_result():
            entry.set_result(response_entry.get_result())
            return entry

        setting = entry.get().get_node(BRIGHTNESS_NAME).get_text()
        response_entry.get().get_node(BRIGHTNESS_NAME).set_text(setting)

        save_entry = request.Entry(ReqTypes.SAVE, response_entry.get())
        response_entry = self.send_entry_with_file(save_entry,
                                  pal_info.PAL_CONFFILE_PATH,
                                  config.LED_CONF_FILE)
        if Results.OK != result_entry.get_result():
            entry.set_result(result_entry.get_result())
            return entry

        entry.set_result(Results.OK)
        return entry

    def __update_handler(self, entry):
        """Handle update request and set entry to result."""

        if self.__get_count() > 0:
            entry.set_result(Results.OK)
            return entry

        led_node = XmlNode(STATUS_GREEN_ON_XML)
        self.__convert_status_led_node(led_node)

        # 変換した結果空になってたらやることないので
        # そのまま終了
        if len(led_node.get_node_list(ANY_CHILD_NAME)) == 0:
            entry.set_result(Results.OK)
            return entry

        control_entry = request.Entry(ReqTypes.CONTROL, led_node)
        control_entry.set(led_node)

        result_entry = self.send_entry(control_entry, pal_info.PAL_LED_PATH)
        if Results.OK != result_entry.get_result():
            entry.set_result(result_entry.get_result())
            return entry

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_handler = lambda entry: self.send_entry_with_file(entry,
                                                pal_info.PAL_CONFFILE_PATH,
                                                config.LED_CONF_FILE)

        self.set_handler(ReqTypes.LOAD, load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.CONTROL, self.__control_handler)
        self.set_handler(ReqTypes.UPDATE, self.__update_handler)
#
# Main
#

if __name__ == "__main__":
    FalLed(option=falcmd.FalCmdOption()).handle_request()
