#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for LOCKKEY configuration."""

from __future__ import with_statement

import os
import glob
import re
import ionas.request as request
import libfal.falcmd as falcmd
import libfal.util as util
import fal_config as config
import pal_info
from iobase.exception import (InvalidArgError, NotFoundError)
from iobase.xmldata import (XmlNode, XmlConfNode)
from fal_info import (ReqTypes, Results)


### XML Base String ###

KEYSERVER_LIST_XML = '<keyserverlist/>'

QUERY_XML_STR = '<lockkey>' + \
                ' <device><name type="usb"></name></device>' + \
                ' <path type="encode"/>' + \
                '</lockkey>'

USB_DEV_XML_STR = '<device><name type="usb">%s</name></device>'
ENC_FILE_PATH_XML_STR = '<path type="encode">%s</path>'

### Reqest XML ###
CHK_EXIST_REQ_XML = '<devctrl><device><name>%s</name></device></devctrl>'

MOUNT_REQ_XML = '<volctrl><volume>' + \
                ' <name type="partition">%s</name>' + \
                ' <dst>%s</dst>' + \
                '</volume></volctrl>'

UNMOUNT_REQ_XML = '<volctrl><volume>' + \
                  ' <dst>%s</dst>' + \
                  '</volume></volctrl>'

MKPART_REQ_XML = '<diskctrl>' + \
                 ' <disk>' + \
                 '  <name>%s</name>' + \
                 '  <partitionlist ptable="msdos">' + \
                 '   <partition>' + \
                 '    <number>1</number>' + \
                 '    <start>2048</start>' + \
                 '    <end>112457</end>' + \
                 '    <name>lockkey</name>' + \
                 '    <filesystem>fat32</filesystem>' + \
                 '   </partition>' + \
                 '  </partitionlist>' + \
                 ' </disk>' + \
                 '</diskctrl>'

MKFS_REQ_XML = '<volctrl>' + \
               ' <option wait="true"/>' + \
               ' <volume>' + \
               '  <name type="partition">%s</name>' + \
               '  <filesystem>fat</filesystem>' + \
               ' </volume>' + \
               '</volctrl>'

MKFS_WITH_CLEAR_REQ_XML = '<volctrl>' + \
                          ' <option zeroclr="true"/>' + \
                          ' <volume>' + \
                          '  <name type="partition">%s</name>' + \
                          '  <filesystem>fat</filesystem>' + \
                          ' </volume>' + \
                          '</volctrl>'

ALLDELPART_REQ_XML = '<diskctrl>' + \
                     ' <option clear="true"/>' + \
                     ' <disk>' + \
                     '   <name>%s</name>' + \
                     ' </disk>' + \
                     '</diskctrl>'

ZERO_CLEAR_REQ_XML = '<diskctrl>' + \
                     ' <disk>' + \
                     '  <name>%(name)s</name>' + \
                     '  <start>%(start)s</start>' + \
                     '  <end>%(end)s</end>' + \
                     ' </disk>' + \
                     '</diskctrl>'

EXTRACT_USB_LOCKKEY_REQ_XML = \
    '<lockkey>' + \
    '    <option operation="extract" keyserver="false"/>' + \
    '    <mac>%s</mac>' + \
    '    <path type="encode">%s</path>' + \
    '    <path type="decode">%s</path>' + \
    '    <device><name type="usb">%s</name></device>' + \
    '</lockkey>'

EXTRACT_KEYSRV_REQ_BASE_XML = \
    '<lockkey>' + \
    '    <option operation="extract" keyserver="true"/>' + \
    '    <mac>%s</mac>' + \
    '    <path type="decode">%s</path>' + \
    '</lockkey>'

UNMOUNT_XML_STR = \
    '<volctrl><volume><dst>%s</dst></volume></volctrl>'

MOUNT_UNMOUNT_EVENT_BODY_BASE_XML = \
    '<volume>' + \
    ' <name>%s</name>' + \
    '</volume>'

QUERY_FOR_KEYSERVER_BASE_XML = "<lockkey><keyserverlist/></lockkey>"

WRITE_USB_LOCKKEY_REQ_XML_TMPL = """<lockkey>
    <option operation="write" keyserver="false"/>
    <key>%s</key>
    <mac>%s</mac>
    <path type="encode">%s</path>
    <device><name type="usb">%s</name></device>
</lockkey>"""

CRYPT_OPTION_XML_TMPL = """<option crypt="%s"/>"""


### X path ###
KEYSERVER_XPATH = "./keyserverlist/keyserver"
USBDEV_NAME_XPATH = "./device/name[@type='usb']"
ENCODE_FILE_PATH_XPATH = "./path[@type='encode']"
DECODE_FILE_PATH_XPATH = "./path[@type='decode']"
KEY_PATH = "./key"

### XML Tag ###
LOCKKEY_NAME = "lockkey"
OPTION_NAME = "option"
MAC_NAME = "mac"
KEYSERVERLIST_NAME = "keyserverlist"
KEYSERVER_NAME = "keyserver"
REGISTNO_NAME = "registno"
KEY_NAME = "key"

### XML Attribute ###
CREATE_ATTR_VAL = "create"
WRITE_ATTR_VAL = "write"
DELETE_ATTR_VAL = "delete"
EXTRACT_ATTR_VAL = "extract"
TRUE_ATTR_VAL = "true"
FALSE_ATTR_VAL = "false"
REMOVE_KEY_ATTR_VAL = "removekey"
TRANSFORM_KEY_ATTR_VAL = "transform"
DOWNLOAD_KEY_ATTR_VAL = "download"
MATCH_ATTR = "match"
REGISTERED_ATTR = "registered"

### Request option ###
OPERATION_OPT = "operation"
CRYPT_OPT = "crypt"

### Method Return Code ###
MOUNT_OK = "OK"
MOUNT_FAIL = "FAIL"

LOCKKEY_DISK_MIN_SIZE = 112458


class FalLockkey(falcmd.FalCmd):
    """FAL Lockkey command implementation class."""

    class __DeviceNotFound(Exception):
        "Lockkey Device Not Found Error"
        pass

    class __KeyFileNotFound(Exception):
        "Key File Not Found Error"
        pass

    class __MkpartFail(Exception):
        "Mkpart Fail"
        pass

    class __DeleteError(Exception):
        "Delete Error"
        pass

    class __MkfsFail(Exception):
        "Mkfs Fail"
        pass

    class __ExtractLockkeyError(Exception):
        "Extract Lockkey Error"
        pass

    class __KeyserverNotFound(Exception):
        "Keyserver Not Found"
        pass

    class __GetError(Exception):
        "Get Error"
        pass

    class __CapShortage(Exception):
        "Cap Shortage"
        pass

    class __UnmountFail(Exception):
        "Unmount Fail"
        pass

    class __AlreadyUnmount(Exception):
        "Alerady unmount"
        pass

    def __vol_name2dev_name(self, vol_name):
        """Convert vol name to partition name."""

        return config.VOL_INFO[vol_name]["dev"]

    def __vol_name2alt_mount_point(self, vol_name):
        """Convert vol name to alternate mount point name."""

        return config.VOL_INFO[vol_name]["alt_mnt_point"]

    def __vol_name2mount_point(self, vol_name):
        """Convert vol name to mount point name."""

        return config.VOL_INFO[vol_name]["mnt_point"]

    def __send_request(self, target_module, type, child_node=""):
        """Send request to pal module(single entry)"""

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __load_config(self, conffile_name):
        """Load config data"""

        load_entry = request.Entry(ReqTypes.LOAD)
        result_entry = self.send_entry_with_file(load_entry,
                                                 pal_info.PAL_CONFFILE_PATH,
                                                 conffile_name)
        return result_entry.get()

    def __dispatch_unmount_event(self, vol_name, is_previous=False):

        try:
            volume_node = XmlNode(MOUNT_UNMOUNT_EVENT_BODY_BASE_XML % vol_name)
            util.dispatch_event('volume', 'unmount', volume_node, is_previous)
        except:
            self.exception("fail to dispatch volume-unmount event")

        return

    def __force_make_dirs(self, dir_path):
        """make specified directory."""
        if not os.path.isdir(dir_path):
            os.makedirs(dir_path)

    def __get_macaddr(self):
        """Get system MAC address"""

        try:
            mac_file_path = os.path.join(config.CONF_DIR, "lockkey", "mac")
            self.__force_make_dirs(os.path.dirname(mac_file_path))

            if not os.path.isfile(mac_file_path):
                mac = util.get_mac_address("netif1").replace(":", "").upper()
                with open(mac_file_path, "w") as fh:
                    fh.write(mac)
                return mac

            with open(mac_file_path, "r") as fh:
                mac = fh.read().rstrip()
                return mac
        except:
            try:
                return util.get_mac_address("netif1").replace(":", "").upper()
            except:
                pass
        return "000000000000"

    def __sub_get_current_lockkey_info(self, vol_name):
        """Get MAC and lockkey file path
           MAC addr is NOT gotten from pal_network. MAC addr is gotten by
           parsing lockkey file name.
        """

        mnt_path = config.VOL_INFO[vol_name]["mnt_point"]

        key_file_pattern = os.path.join(mnt_path, config.LOCKKEY_DIR_PREFIX)
        key_file_list = glob.glob(key_file_pattern + "*/*")

        if 0 == len(key_file_list):
            raise self.__KeyFileNotFound

        match = \
            re.match(key_file_pattern + "[0-9A-Za-z]{12}/([0-9A-Za-z]{12})",
                     key_file_list[0])
        if not match:
            raise self.__KeyFileNotFound

        return (match.group(1), key_file_list[0])

    def __get_current_lockkey_info(self):
        """Get lockkey infomation from connected lockkey."""

        for vol_name in config.LOCKKEY_VOL_LIST:

            try:
                (mac, lockkey_path) = \
                    self.__sub_get_current_lockkey_info(vol_name)

                lockkey_dev_name = config.VOL_INFO[vol_name]["dev"]

                return (mac, lockkey_dev_name, lockkey_path)

            except self.__KeyFileNotFound:
                continue

        raise self.__KeyFileNotFound

    def __is_device_exist(self, dev_name):
        """Check wheather specified device exists."""

        result_entry = self.__send_request(pal_info.PAL_DEVCTRL_PATH,
                                           ReqTypes.CHK_EXIST,
                                           CHK_EXIST_REQ_XML % dev_name)
        if Results.OK == result_entry.get_result():
            return True

        return False

    def __mount_lockkey(self, lockkey_vol_name):
        """Mount lockkey device."""

        part_name = config.VOL_INFO[lockkey_vol_name]["part_base"] + "1"

        mnt_dir = config.VOL_INFO[lockkey_vol_name]["mnt_point"]

        result_entry = \
            self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                ReqTypes.MOUNT,
                                MOUNT_REQ_XML % (part_name, mnt_dir))

        result_code = result_entry.get_result()

        if Results.OK == result_code:
            return MOUNT_OK

        if Results.ALREADY_MOUNT == result_code:
            return MOUNT_OK

        return MOUNT_FAIL

    def __do_bind_unmount(self, vol_name):

        alt_mnt_point = self.__vol_name2alt_mount_point(vol_name)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.UNMOUNT,
                                           UNMOUNT_XML_STR % (alt_mnt_point))

        result_code = result_entry.get_result()

        if Results.OK == result_code or Results.ALREADY_UNMOUNT == result_code:
            return

        raise self.__UnmountFail

    def __do_unmount(self, vol_name):

        mnt_point = self.__vol_name2mount_point(vol_name)
        req_xml = UNMOUNT_XML_STR % (mnt_point)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.UNMOUNT,
                                           req_xml)

        result_code = result_entry.get_result()

        if Results.OK == result_code or Results.ALREADY_UNMOUNT == result_code:
            return

        raise self.__UnmountFail

    def __unmount_shared_volume(self, vol_name):

        self.__dispatch_unmount_event(vol_name, True)

        try:
            self.__do_bind_unmount(vol_name)
        except:
            self.exception("fail to __do_bind_unmount(%s)" % vol_name)
            self.__dispatch_unmount_event(vol_name)
            raise

        try:
            self.__do_unmount(vol_name)
        except:
            self.exception("fail to __do_unmount(%s)" % vol_name)
            self.__dispatch_unmount_event(vol_name)
            raise

        self.__dispatch_unmount_event(vol_name)

        return

    def __unmount_lockkey(self, lockkey_vol_name):
        """Unount lockkey device."""

        mnt_dir = config.VOL_INFO[lockkey_vol_name]["mnt_point"]

        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.UNMOUNT,
                                           UNMOUNT_REQ_XML % (mnt_dir))
        if Results.OK == result_entry.get_result():
            return True

        return False

    def __delete_all_partition(self, lockkey_vol_name):

        arry_num = util.get_cuurent_array_num()
        dev_name = config.HDD_DEV_LIST_IN_VOL[arry_num][lockkey_vol_name]

        if dev_name in config.STOR_IF_TO_VOL["internal"].values():
            raise InvalidArgError

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.DELPART,
                                           ALLDELPART_REQ_XML % dev_name)
        if Results.OK != result_entry.get_result():
            raise self.__DeleteError
        return

    def __send_query_to_diskctrl(self, disk_name):
        """Send query request to pal_diskctrl"""

        QUERY_REQ_BASE_XML = '<diskctrl><disk><name/></disk></diskctrl>'
        DISK_NAME_XPATH = "./disk/name"

        diskctrl_node = XmlNode(QUERY_REQ_BASE_XML)
        diskctrl_node.get_node(DISK_NAME_XPATH).set_text(disk_name)

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.QUERY,
                                           diskctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__GetError

        return result_entry

    def __get_disk_size(self, disk_name):

        SIZE_XPATH = "./disk/size"
        diskctrl_node = self.__send_query_to_diskctrl(disk_name).get()
        return int(diskctrl_node.get_node(SIZE_XPATH).get_text())

    def __mkpart_lockkey(self, lockkey_vol_name):
        """Mkapr lockkey device."""

        arry_num = util.get_cuurent_array_num()
        dev_name = config.HDD_DEV_LIST_IN_VOL[arry_num][lockkey_vol_name]

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.MKPART,
                                           MKPART_REQ_XML % (dev_name))

        if Results.OK == result_entry.get_result():
            return

        raise self.__MkpartFail

    def __zero_clear_lockkey(self, lockkey_vol_name):

        disk_name = config.VOL_INFO[lockkey_vol_name]["dev"]
        req_xml = ZERO_CLEAR_REQ_XML % {"name": disk_name,
                                        "start": "0",
                                        "end": "112457"}
        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.CLEAR,
                                           req_xml)
        if Results.OK != result_entry.get_result():
            raise self.__DeleteError

        return

    def __mkfs_lockkey(self, lockkey_vol_name, clear_flg=False):
        """Mkfs lockkey device."""

        part_name = config.VOL_INFO[lockkey_vol_name]["part_base"] + "1"

        if clear_flg:
            req_xml = MKFS_WITH_CLEAR_REQ_XML % (part_name)
        else:
            req_xml = MKFS_REQ_XML % (part_name)

        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.MKFS,
                                           req_xml)

        if Results.OK == result_entry.get_result():
            return

        raise self.__MkfsFail

    def __make_volume_for_lockkey(self, vol_name):
        """Mkpart and Mkfs for creating lockkey."""

        lockkey_disk_size = \
            self.__get_disk_size(self.__vol_name2dev_name(vol_name))

        if LOCKKEY_DISK_MIN_SIZE > lockkey_disk_size:
            raise self.__CapShortage

        self.__delete_all_partition(vol_name)
        self.__mkpart_lockkey(vol_name)
        self.__mkfs_lockkey(vol_name)
        return

    def __get_enable_lockkey_volume(self):
        """If the device of lockkey volume is connected return the volume."""

        for vol_name in config.LOCKKEY_VOL_LIST:

            lockkey_dev_name = config.VOL_INFO[vol_name]["dev"]

            if self.__is_device_exist(lockkey_dev_name):
                return vol_name

        raise self.__DeviceNotFound

    def __check_key_file_exist(self, lockkey_vol_name):
        """Check wheather lockkey file exists."""

        mnt_dir = config.VOL_INFO[lockkey_vol_name]["mnt_point"]

        key_file_pattern = os.path.join(mnt_dir, config.LOCKKEY_DIR_PREFIX)
        key_file_list = glob.glob(key_file_pattern + "*/*")

        if 0 == len(key_file_list):
            return Results.OK

        match = \
            re.match(key_file_pattern + "([0-9A-Za-z]{12})/([0-9A-Za-z]{12})",
                     key_file_list[0])
        if not match:
            return Results.OK

        mac = self.__get_macaddr()
        if mac != match.group(1):
            return Results.UNKNOWN_LOCKKEY

        if mac != match.group(2):
            return Results.UNKNOWN_LOCKKEY

        return Results.ALREADY_EXIST

    def __lockkey_create(self, entry):
        """Create lockkey."""

        vol_name = self.__get_enable_lockkey_volume()

        self.__unmount_shared_volume(vol_name)

        self.__mount_lockkey(vol_name)

        # 既にキーが存在する場合は新たにキーを作らない
        chk_result = self.__check_key_file_exist(vol_name)
        if Results.OK != chk_result:
            entry.set_result(chk_result)
            util.set_volume_cache_value(vol_name,
                                   util.STATUS_VOL_CACHE,
                                   util.LOCKKEY_STATUS)
            self.__unmount_lockkey(vol_name)
            return entry

        self.__unmount_lockkey(vol_name)

        self.__make_volume_for_lockkey(vol_name)

        if MOUNT_FAIL == self.__mount_lockkey(vol_name):
            entry.set_result(Results.CREATE_ERROR)
            return entry

        lockkey_dev_name = config.VOL_INFO[vol_name]["dev"]
        mnt_dir = config.VOL_INFO[vol_name]["mnt_point"]
        mac = self.__get_macaddr()
        lockkey_file_path = \
            os.path.join(mnt_dir, config.LOCKKEY_DIR_PREFIX + mac, mac)

        entry.get().append_child(MAC_NAME, mac)
        entry.get().append_child(XmlNode(USB_DEV_XML_STR % lockkey_dev_name))
        entry.get().append_child(XmlNode(ENC_FILE_PATH_XML_STR %
                                         lockkey_file_path))
        result_entry = self.send_entry(entry, pal_info.PAL_LOCKKEY_PATH)

        self.__unmount_lockkey(vol_name)

        util.set_volume_cache_value(vol_name,
                               util.STATUS_VOL_CACHE,
                               util.LOCKKEY_STATUS)

        return result_entry

    def __lockkey_write(self, entry):
        """Save key to specified file path.
           The key is encoded before saving.
        """

        vol_name = self.__get_enable_lockkey_volume()

        self.__unmount_shared_volume(vol_name)

        self.__make_volume_for_lockkey(vol_name)

        if MOUNT_FAIL == self.__mount_lockkey(vol_name):
            entry.set_result(Results.CREATE_ERROR)
            return entry

        # MAC は App 側から指定された値を使う
        # (create とは異なる)
        mac = entry.get().get_node(MAC_NAME).get_text()
        lockkey_dev_name = config.VOL_INFO[vol_name]["dev"]
        mnt_dir = config.VOL_INFO[vol_name]["mnt_point"]
        lockkey_file_path = \
            os.path.join(mnt_dir, config.LOCKKEY_DIR_PREFIX + mac, mac)

        entry.get().append_child(XmlNode(USB_DEV_XML_STR % lockkey_dev_name))
        entry.get().append_child(XmlNode(ENC_FILE_PATH_XML_STR %
                                         lockkey_file_path))

        result_entry = self.send_entry(entry, pal_info.PAL_LOCKKEY_PATH)

        self.__unmount_lockkey(vol_name)

        util.clear_volume_cache_value(vol_name, "status")
        return result_entry

    def __lockkey_delete(self, entry):
        """Zero clear the partition of lockkey and execute mkfs"""

        vol_name = self.__get_enable_lockkey_volume()

        self.__unmount_shared_volume(vol_name)

        if MOUNT_FAIL == self.__mount_lockkey(vol_name):
            entry.set_result(Results.FILE_NOT_FOUND)
            return entry

        try:
            self.__get_current_lockkey_info()
        except:
            entry.set_result(Results.FILE_NOT_FOUND)
            self.__unmount_lockkey(vol_name)
            return entry

        try:
            self.__unmount_lockkey(vol_name)
            self.__zero_clear_lockkey(vol_name)
            self.__mkpart_lockkey(vol_name)
            self.__mkfs_lockkey(vol_name)
            entry.set_result(Results.OK)
        except:
            entry.set_result(Results.DELETE_ERROR)
        finally:
            util.clear_volume_cache_value(vol_name, "status")

        return entry

    def __get_keyserverlist(self, enabled_only=False, no_list=[]):
        """Return "keyserverlist" node include specified "keyserver" node."""

        lockkey_config = self.__load_config(config.LOCKKEY_CONF_FILE)
        keyserverlist_node = XmlNode(KEYSERVER_LIST_XML)
        is_found = False
        for keyserver in lockkey_config.get_node_list(KEYSERVER_XPATH):
            if enabled_only and not keyserver.is_enabled():
                continue

            registno = keyserver.get_value(REGISTNO_NAME, "")
            if len(no_list) and registno not in no_list:
                continue

            keyserverlist_node.append_child(keyserver.get_clone())
            is_found = True

        if not is_found:
            raise self.__KeyserverNotFound

        return keyserverlist_node

    def __is_registered(self, keyserver):
        """Check if the key is registered in "keyserver"."""

        query_node = XmlNode(QUERY_FOR_KEYSERVER_BASE_XML)
        query_node.get_node(KEYSERVERLIST_NAME).append_child(
                                                    keyserver.get_clone())
        query_node.append_child(MAC_NAME, self.__get_macaddr())
        result_entry = self.__send_request(pal_info.PAL_LOCKKEY_PATH,
                                               ReqTypes.QUERY,
                                               query_node)

        if result_entry.get_result() != Results.OK:
            return False
        return True

    def __do_extract_keyserver_lockkey(self):

        req_xml = EXTRACT_KEYSRV_REQ_BASE_XML % (
            self.__get_macaddr(), config.DECODED_LOCK_KEY_FILE_PATH)

        req_node = XmlNode(req_xml)
        keyserverlist_node = self.__get_keyserverlist(enabled_only=True)
        req_node.append_child(keyserverlist_node)

        result_entry = self.__send_request(pal_info.PAL_LOCKKEY_PATH,
                                           ReqTypes.SET_LOCKKEY,
                                           str(req_node))

        if Results.OK != result_entry.get_result():
            raise self.__KeyserverNotFound
        return

    def __do_extract_usb_lockkey(self):

        vol_name = self.__get_enable_lockkey_volume()
        self.__mount_lockkey(vol_name)

        try:
            (key_mac, key_dev, key_path) = self.__get_current_lockkey_info()
            mac = self.__get_macaddr()
            if key_mac != mac:
                raise self.__KeyFileNotFound
        except:
            self.__unmount_lockkey(vol_name)
            raise

        req_xml = EXTRACT_USB_LOCKKEY_REQ_XML % (
            mac, key_path, config.DECODED_LOCK_KEY_FILE_PATH, key_dev)
        result_entry = self.__send_request(
            pal_info.PAL_LOCKKEY_PATH, ReqTypes.SET_LOCKKEY, req_xml)
        self.__unmount_lockkey(vol_name)

        if Results.OK != result_entry.get_result():
            raise self.__ExtractLockkeyError
        return result_entry

    def __lockkey_extract(self, entry):
        """Extract enckey"""

        # try extract from keyserver
        try:
            self.__do_extract_keyserver_lockkey()
            entry.set_result(Results.OK)
            return entry
        except:
            pass

        # try extract from USB memory
        try:
            self.__do_extract_usb_lockkey()
            entry.set_result(Results.OK)
            return entry
        except self.__DeviceNotFound:
            if os.path.exists(config.DECODED_LOCK_KEY_FILE_PATH):
                entry.set_result(Results.OK)
                return entry
            else:
                raise self.__DeviceNotFound
        except self.__KeyFileNotFound:
            raise self.__KeyFileNotFound
        except:
            raise self.__ExtractLockkeyError

    def __lockkey_remove_extract_key(self, entry):

        if os.path.exists(config.DECODED_LOCK_KEY_FILE_PATH):

            with open(config.DECODED_LOCK_KEY_FILE_PATH, "w") as fh:
                fh.write("                                ")
            os.remove(config.DECODED_LOCK_KEY_FILE_PATH)

    def __lockkey_transform_key(self, entry):

        vol_name = self.__get_enable_lockkey_volume()
        self.__mount_lockkey(vol_name)

        try:
            (key_mac, dev_name, key_path) = self.__get_current_lockkey_info()
        except:
            self.exception("Get a current Lock Key info fail!!")
            self.__unmount_lockkey(vol_name)
            raise

        try:
            query_node = XmlNode(QUERY_XML_STR)
            query_node.get_node(USBDEV_NAME_XPATH).set_text(dev_name)
            query_node.get_node(ENCODE_FILE_PATH_XPATH).set_text(key_path)
            query_node.append_child(
                            XmlNode(CRYPT_OPTION_XML_TMPL % (key_mac,)))
            query_response = self.__send_request(pal_info.PAL_LOCKKEY_PATH,
                                                   ReqTypes.QUERY,
                                                   str(query_node)).get()
            key = query_response.get_node(KEY_PATH).get_text()
        except:
            self.exception("Get a Key fail!!")
            self.__unmount_lockkey(vol_name)
            raise

        self.__unmount_lockkey(vol_name)
        try:
            self.__make_volume_for_lockkey(vol_name)
        except:
            self.exception("Make a volume for lockkey fail!!")
            raise

        if MOUNT_FAIL == self.__mount_lockkey(vol_name):
            entry.set_result(Results.CREATE_ERROR)
            return entry

        mac = self.__get_macaddr()
        mnt_dir = config.VOL_INFO[vol_name]["mnt_point"]
        lockkey_file_path = \
            os.path.join(mnt_dir, config.LOCKKEY_DIR_PREFIX + mac, mac)

        req_xml = XmlNode(WRITE_USB_LOCKKEY_REQ_XML_TMPL % \
                                    (key, mac, lockkey_file_path, dev_name))
        req_xml.get_node(OPTION_NAME).set_attrs({CRYPT_OPT: mac})
        result_entry = self.__send_request(
                    pal_info.PAL_LOCKKEY_PATH, ReqTypes.SET_LOCKKEY, req_xml)

        self.__unmount_lockkey(vol_name)

        util.set_volume_cache_value(vol_name,
                               util.STATUS_VOL_CACHE,
                               util.LOCKKEY_STATUS)

        return result_entry

    def __lockkey_download_key(self, entry):

        lockkey_node = entry.get().get_clone()
        keyserver = lockkey_node.get_node(KEYSERVER_NAME)
        keyserverlist = self.__get_keyserverlist(
                                no_list=[keyserver.get_value(REGISTNO_NAME)])
        config_keyserver = keyserverlist.get_node(KEYSERVER_NAME)
        keyserver.update(config_keyserver, override=True)

        try:
            query_node = XmlNode(QUERY_FOR_KEYSERVER_BASE_XML)
            query_node.get_node(KEYSERVERLIST_NAME).append_child(keyserver)
            query_node.append_child(MAC_NAME, self.__get_macaddr())

            query_response = self.__send_request(pal_info.PAL_LOCKKEY_PATH,
                                                   ReqTypes.QUERY,
                                                   query_node)
            result = query_response.get_result()
            if Results.OK != result:
                self.exception("Request fail!! - %s" % (result,))
                entry.set_result(result)
                return entry

            key = query_response.get().get_node(KEY_PATH).get_text()
        except:
            self.exception("Get a Key fail!!")
            raise

        vol_name = self.__get_enable_lockkey_volume()
        self.__mount_lockkey(vol_name)
        if self.__check_key_file_exist(vol_name) == Results.UNKNOWN_LOCKKEY:
            self.__unmount_lockkey(vol_name)
            entry.set_result(Results.UNKNOWN_LOCKKEY)
            return entry

        self.__unmount_lockkey(vol_name)
        try:
            self.__make_volume_for_lockkey(vol_name)
        except:
            self.exception("Make a volume for lockkey fail!!")
            raise

        if MOUNT_FAIL == self.__mount_lockkey(vol_name):
            entry.set_result(Results.CREATE_ERROR)
            return entry

        mac = self.__get_macaddr()
        dev_name = config.VOL_INFO[vol_name]["dev"]
        mnt_dir = config.VOL_INFO[vol_name]["mnt_point"]
        lockkey_file_path = \
            os.path.join(mnt_dir, config.LOCKKEY_DIR_PREFIX + mac, mac)

        req_xml = XmlNode(WRITE_USB_LOCKKEY_REQ_XML_TMPL % \
                                    (key, mac, lockkey_file_path, dev_name))
        req_xml.get_node(OPTION_NAME).set_attrs({CRYPT_OPT: mac})
        result_entry = self.__send_request(
                    pal_info.PAL_LOCKKEY_PATH, ReqTypes.SET_LOCKKEY, req_xml)

        self.__unmount_lockkey(vol_name)

        util.set_volume_cache_value(vol_name,
                               util.STATUS_VOL_CACHE,
                               util.LOCKKEY_STATUS)

        return result_entry

    #
    # Request handler
    #

    def __query_handler(self, entry):
        """Handle query request."""
        try:
            vol_name = self.__get_enable_lockkey_volume()
        except self.__DeviceNotFound:
            entry.set_result(Results.DEVICE_NOT_FOUND)
            return entry

        self.__mount_lockkey(vol_name)

        match_mac_attr_val = TRUE_ATTR_VAL
        if Results.UNKNOWN_LOCKKEY == self.__check_key_file_exist(vol_name):
            match_mac_attr_val = FALSE_ATTR_VAL

        try:
            (mac, key_dev_name, key_path) = self.__get_current_lockkey_info()
        except self.__KeyFileNotFound:
            entry.set_result(Results.DEVICE_NOT_FOUND)
            self.__unmount_lockkey(vol_name)
            return entry

        query_node = XmlNode(QUERY_XML_STR)
        query_node.get_node(USBDEV_NAME_XPATH).set_text(key_dev_name)
        query_node.get_node(ENCODE_FILE_PATH_XPATH).set_text(key_path)
        result_entry = self.__send_request(pal_info.PAL_LOCKKEY_PATH,
                                           ReqTypes.QUERY,
                                           str(query_node))

        if result_entry.get_result() != Results.OK:
            entry.set_result(Results.QUERY_ERROR)
            self.__unmount_lockkey(vol_name)
            return entry

        result_entry.get().append_child(MAC_NAME, mac,
                                        {MATCH_ATTR: match_mac_attr_val})

        self.__unmount_lockkey(vol_name)
        return result_entry

    def __set_lockkey_handler(self, entry):
        """Handle set_lockkey request."""

        op_funcs = {CREATE_ATTR_VAL: self.__lockkey_create,
                    WRITE_ATTR_VAL: self.__lockkey_write,
                    DELETE_ATTR_VAL: self.__lockkey_delete,
                    EXTRACT_ATTR_VAL: self.__lockkey_extract,
                    REMOVE_KEY_ATTR_VAL: self.__lockkey_remove_extract_key,
                    TRANSFORM_KEY_ATTR_VAL: self.__lockkey_transform_key,
                    DOWNLOAD_KEY_ATTR_VAL: self.__lockkey_download_key}

        op = entry.get_options()[OPERATION_OPT]

        try:
            result_entry = op_funcs[op](entry.get_clone())
            entry.set_result(result_entry.get_result())

        except self.__DeviceNotFound:
            entry.set_result(Results.DEVICE_NOT_FOUND)

        except self.__KeyFileNotFound:
            entry.set_result(Results.NO_LOCKKEY)

        except self.__ExtractLockkeyError:
            entry.set_result(Results.GET_ERROR)

        except self.__CapShortage:
            entry.set_result(Results.CAP_SHORTAGE)

        return entry

    def __check_keyserver_handler(self, entry):
        """Handle check_keyserver request."""

        return self.send_entry(entry, pal_info.PAL_LOCKKEY_PATH)

    @falcmd.conf_change_handler_wrapper
    def __save_handler(self, entry):
        """Handle save request and set entry to result."""

        return self.send_entry_with_file(entry,
                                pal_info.PAL_CONFFILE_PATH,
                                config.LOCKKEY_CONF_FILE)

    def __list_handler(self, entry):
        """Handle list request."""

        entry.set(XmlConfNode(LOCKKEY_NAME))

        entry.get().append_child(
                self.__get_keyserverlist(no_list=config.KEY_SERVER_NO_LIST))

        for keyserver in entry.get().get_node_list(KEYSERVER_XPATH):
            if not keyserver.is_enabled():
                continue

            try:
                if self.__is_registered(keyserver):
                    keyserver.set_bool_attr(REGISTERED_ATTR, True)
                else:
                    keyserver.set_bool_attr(REGISTERED_ATTR, False)
            except:
                pass

        entry.set_result(Results.OK)
        return entry

    def __register_handler(self, entry):
        """Handle register request."""

        try:
            vol_name = self.__get_enable_lockkey_volume()
        except self.__DeviceNotFound:
            self.exception("A lock key not found!!")
            entry.set_result(Results.DEVICE_NOT_FOUND)
            return entry

        self.__mount_lockkey(vol_name)

        try:
            (key_mac, dev_name, key_path) = self.__get_current_lockkey_info()
        except:
            self.exception("Get a current Lock Key info fail!!")
            self.__unmount_lockkey(vol_name)
            entry.set_result(Results.NO_LOCKKEY)
            return entry

        try:
            query_node = XmlNode(QUERY_XML_STR)
            query_node.get_node(USBDEV_NAME_XPATH).set_text(dev_name)
            query_node.get_node(ENCODE_FILE_PATH_XPATH).set_text(key_path)
            query_node.append_child(
                            XmlNode(CRYPT_OPTION_XML_TMPL % (key_mac,)))
            query_response = self.__send_request(pal_info.PAL_LOCKKEY_PATH,
                                                   ReqTypes.QUERY,
                                                   str(query_node))
            key = query_response.get().get_node(KEY_PATH).get_text()
        except:
            self.exception("Get a Key fail!!")
            self.__unmount_lockkey(vol_name)
            entry.set_result(Results.GET_ERROR)
            return entry

        self.__unmount_lockkey(vol_name)

        lockkey_node = entry.get().get_clone()
        keyserver = lockkey_node.get_node(KEYSERVER_NAME)
        keyserverlist = self.__get_keyserverlist(
                                no_list=[keyserver.get_value(REGISTNO_NAME)])
        config_keyserver = keyserverlist.get_node(KEYSERVER_NAME)
        keyserver.update(config_keyserver, override=True)

        lockkey_node.append_child(KEY_NAME, key)
        lockkey_node.append_child(MAC_NAME, self.__get_macaddr())

        result_entry = self.__send_request(pal_info.PAL_LOCKKEY_PATH,
                                           ReqTypes.REGISTER,
                                           lockkey_node)

        entry.set_result(result_entry.get_result())
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_save_handler = lambda entry: self.send_entry_with_file(entry,
                                pal_info.PAL_CONFFILE_PATH,
                                config.LOCKKEY_CONF_FILE)

        self.set_handler(ReqTypes.LOAD, load_save_handler)
        self.set_handler(ReqTypes.SAVE, self.__save_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.SET_LOCKKEY, self.__set_lockkey_handler)
        self.set_handler(ReqTypes.CHECK_KEYSERVER,
                         self.__check_keyserver_handler)
        self.set_handler(ReqTypes.LIST, self.__list_handler)
        self.set_handler(ReqTypes.REGISTER, self.__register_handler)

#
# Main
#

if __name__ == "__main__":
    FalLockkey(option=falcmd.FalCmdOption()).handle_request()
