#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling mailsend function"""

import pal_info
import libfal.falcmd as falcmd
import ionas.request as request
import fal_config as config
from iobase.exception import NotFoundError
from fal_info import ReqTypes

### XML Tag Name ###
MAILSEND_NAME = "mailsend"
MAILCONF_NAME = "mailconf"
MESSAGE_NAME = "message"
TOADDRLIST_NAME = "toaddrlist"
TOADDR_NAME = "toaddr"
MAILBASIC_NAME = "mailbasic"
MAILCONFLIST_NAME = "mailconflist"
MAILNOTICE_NAME = "mailnotice"

### XML Attribute ###
DETAIL_ATTR = "detail"
PROFILENUM_ATTR = "profilenum"
RETRYCOUNT_ATTR = "retrycount"

### XML Attribute Value ###
TRUE_ATTR_VAL = "true"


class FalMailsend(falcmd.FalCmd):
    """FAL Mailsend command implementation class."""

    def __is_exist_attr(self, node, attr_val):
        """Check whether node_path exists in specified node"""

        try:
            node.get_attr_value(attr_val)
        except NotFoundError:
            return False
        return True

    def __load_config(self, file_name):
        """Load a 'file_name' file setting and return XML data ."""

        entry = request.Entry(ReqTypes.LOAD)
        entry.set_file(file_name)
        target = request.Request(pal_info.PAL_CONFFILE_PATH)
        response_data = target.request(request.RequestData(entry))
        return response_data.get_entrylist().get_entry().get()

    def __set_smtp_auth_data(
            self, auth_type, detail_mailconf_node, basic_mailconf_node):
        """Set SMTP auth data."""

        USER_XPATH = "./auth[@type='%s']/user"
        PASSWORD_XPATH = "./auth[@type='%s']/password"

        user_node = detail_mailconf_node.get_node(USER_XPATH % auth_type)
        pass_node = detail_mailconf_node.get_node(PASSWORD_XPATH % auth_type)

        if user_node == "":
            user = basic_mailconf_node.get_value(USER_XPATH % auth_type)
            user_node.set_text(user)
        if pass_node == "":
            passwd = basic_mailconf_node.get_value(PASSWORD_XPATH % auth_type)
            pass_node.set_text(passwd)

    def __set_pop_before_smtp_data(
            self, detail_mailconf_node, basic_mailconf_node):
        """Set POP before SMTP data."""

        POP_HOST_XPATH = "./auth[@type='pop']/pop3/host"
        POP_PORT_XPATH = "./auth[@type='pop']/pop3/port"

        pop_host_node = detail_mailconf_node.get_node(POP_HOST_XPATH)
        pop_port_node = detail_mailconf_node.get_node(POP_PORT_XPATH)

        if pop_host_node == "":
            host = basic_mailconf_node.get_value(POP_HOST_XPATH)
            pop_host_node.set_text(host)
        if pop_port_node == "":
            port = basic_mailconf_node.get_value(POP_PORT_XPATH)
            pop_port_node.set_text(port)

    def __is_none_auth(self, auth_type):
        """Check auth_type none."""

        NONE_AUTH_ATTR_VAL = "no"

        if auth_type == NONE_AUTH_ATTR_VAL:
            return True

        return False

    def __is_smtp_auth(self, auth_type):
        """Check auth_type SMTP auth."""

        PLAIN_AUTH_ATTR_VAL = "plain"
        LOGIN_AUTH_ATTR_VAL = "login"
        MD5_AUTH_ATTR_VAL = "crammd5"

        if auth_type == PLAIN_AUTH_ATTR_VAL:
            return True
        if auth_type == LOGIN_AUTH_ATTR_VAL:
            return True
        if auth_type == MD5_AUTH_ATTR_VAL:
            return True

        return False

    def __is_pop_before_smtp_auth(self, auth_type):
        """Check auth_type POP before SMTP."""

        POP_BEFORE_SMTP_AUTH_ATTR_VAL = "pop"

        if auth_type == POP_BEFORE_SMTP_AUTH_ATTR_VAL:
            return True

        return False

    def __merge_auth_node(self, detail_mailconf_node, basic_mailconf_node):
        """Merge auth node."""

        AUTH_XPATH = "./auth"
        TYPE_ATTR = "type"

        auth_node = detail_mailconf_node.get_node(AUTH_XPATH)
        type_attr = auth_node.get_attr_value(TYPE_ATTR)

        if self.__is_none_auth(type_attr):
            return
        if self.__is_smtp_auth(type_attr):
            self.__set_smtp_auth_data(
                type_attr, detail_mailconf_node, basic_mailconf_node)
        if self.__is_pop_before_smtp_auth(type_attr):
            self.__set_pop_before_smtp_data(
                detail_mailconf_node, basic_mailconf_node)

    def __merge_mail_conf_node(self,
                               detail_mailconf_node,
                               basic_mailconf_node):

        HOST_XPATH = "./host[@type='smtp']"
        PORT_XPATH = "./port[@type='smtp']"
        FROMADDR_XPATH = "./fromaddr"
        ENCODE_XPATH = "./encode"

        param_xpaths = \
            (HOST_XPATH, PORT_XPATH, FROMADDR_XPATH, ENCODE_XPATH)

        for xpath in param_xpaths:

            if "" == detail_mailconf_node.get_value(xpath):
                value = basic_mailconf_node.get_value(xpath)
                detail_mailconf_node.get_node(xpath).set_text(value)

        self.__merge_auth_node(basic_mailconf_node, detail_mailconf_node)

    def __load_mail_profile(self, prof_num):
        """Load profile from config file"""

        conf_xml = self.__load_config(config.NOTICE_CONF_FILE)

        basic_mailconf_node = \
            conf_xml.get_node(MAILBASIC_NAME).get_node(MAILCONF_NAME)

        mailconf_list_node = \
            conf_xml.get_node(MAILNOTICE_NAME).get_node(MAILCONFLIST_NAME)

        mailconf_node = \
            mailconf_list_node.get_node_list(MAILCONF_NAME)[prof_num]

        toaddrlist_node = mailconf_node.get_node(TOADDRLIST_NAME)

        if TRUE_ATTR_VAL != mailconf_node.get_attr_value(DETAIL_ATTR):
            basic_mailconf_node.append_child(toaddrlist_node)
            return basic_mailconf_node

        self.__merge_mail_conf_node(mailconf_node, basic_mailconf_node)

        return mailconf_node

    def __get_landisk_name(self):
        """Get landisk name form config file"""

        NAME_XPATH = "./hostdesc/name"

        sysinfo_node = \
            self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                      pal_info.PAL_CONFFILE_PATH,
                                      config.SYSINFO_CONF_FILE).get()

        return sysinfo_node.get_value(NAME_XPATH, "")

    def __attach_landisk_name_to_subject(self, mailsend_node):
        """Attached landisk name string to specified subject"""

        SUBJECT_STR_TMPL = "[%(landiskname)s] %(title)s"
        SUBJECT_XPATH = "message/header/subject"

        subject = mailsend_node.get_node(SUBJECT_XPATH).get_text()

        landisk_name = self.__get_landisk_name()

        if not landisk_name:
            return  # 通常ありえないパス

        new_subject = SUBJECT_STR_TMPL % {"landiskname": landisk_name,
                                      "title": subject}

        mailsend_node.get_node(SUBJECT_XPATH).set_text(new_subject)
        return

    def __attach_additional_message_to_body(self, mailsend_node):
        """Attach additional message to message body."""

        # FIXME: ロケールに応じた処理になっていない
        MAIL_BODY_STR_TMPL = """
株式会社アイ・オー・データ機器 LAN DISK %(landisk_name)s からのお知らせです。

%(mail_to)s 様

%(mail_body)s

このメールはお客様の設定に基づきお送りしています。
"""
        MAIL_BODY_XPATH = "message/body"
        MAIL_TO_XPATH = "mailconf/toaddrlist/toaddr"

        try:
            message_node = mailsend_node.get_node(MESSAGE_NAME)
            message_node.get_attr_value(RETRYCOUNT_ATTR)
            return
        except:
            pass

        mail_body = mailsend_node.get_node(MAIL_BODY_XPATH).get_text()
        mail_to = mailsend_node.get_node(MAIL_TO_XPATH).get_text()
        landisk_name = self.__get_landisk_name()

        if not mail_to:
            return
        if not landisk_name:
            return

        new_mail_body = MAIL_BODY_STR_TMPL % {
            "mail_body": mail_body,
            "mail_to": mail_to,
            "landisk_name": landisk_name}

        mailsend_node.get_node(MAIL_BODY_XPATH).set_text(new_mail_body)

    #
    # Request handlers
    #

    def __send_handler(self, entry):
        """Handle "send" request."""

        mailsend_node = entry.get()
        mailconf_node = mailsend_node.get_node(MAILCONF_NAME)

        self.__attach_landisk_name_to_subject(mailsend_node)

        if not self.__is_exist_attr(mailconf_node, PROFILENUM_ATTR):
            self.__attach_additional_message_to_body(mailsend_node)
            return self.send_entry(entry, pal_info.PAL_MAILSEND_PATH)

        prof_num = mailconf_node.get_attr_value(PROFILENUM_ATTR)
        mailconf_node = self.__load_mail_profile(int(prof_num))
        mailsend_node.delete_children(MAILCONF_NAME)
        mailsend_node.append_child(mailconf_node)
        self.__attach_additional_message_to_body(mailsend_node)

        return self.send_entry(entry, pal_info.PAL_MAILSEND_PATH)

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.SEND, self.__send_handler)

#
# Main
#

if __name__ == "__main__":
    FalMailsend(option=falcmd.FalCmdOption()).handle_request()
