#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for messages."""


import iobase.xmldata as xmldata
import ionas.request as request
import libfal.falcmd as falcmd
import fal_msginfo as msginfo
import pal_info
from fal_info import ReqTypes
from fal_info import Results
from iobase.exception import (InvalidArgError, NotFoundError)

import fal_config as config


MESSAGEINFO_PATH = "messageinfo"
CODE_PATH = MESSAGEINFO_PATH + "/code"

OPTION_NAME = "option"
CATEGORY_NAME = "category"
MESSAGEINFO_NAME = "messageinfo"
CODE_NAME = "code"
MESSAGE_NAME = "message"
LINE_NAME = "line"
ARGLIST_NAME = "arglist"
ARG_NAME = "arg"
NAME_ATTR_NAME = "name"
LCM_ATTR_NAME = "for_lcm"

MESSAGE_PATH_TMPL = 'messageinfolist/messageinfo/code[text()="%s"]/../message'
LEVEL_PATH_TMPL = 'messageinfolist/messageinfo/code[text()="%s"]/../level'
LCM_LINE_PATH_TMPL = 'messageinfolist/messageinfo/code[text()="%s"]/../' + \
                'noticeinfo/lcm/display'
ARGTYPE_PATH_TMPL = 'messageinfolist/messageinfo/code[text()="%s"]/..' + \
                        '/argtypelist/argtype'

GOOD_LEVEL = "0"
CODE_CATEGORY_UNIT = 10000

XML_LANG_ATTR_NAME = "{http://www.w3.org/XML/1998/namespace}lang"


class FalMessages(falcmd.FalCmd):
    """FAL messages command implementation class."""

    def __get_messages_xml(self):
        """Load "messages.xml" and return XML data ."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                             pal_info.PAL_CONFFILE_PATH,
                                             config.MESSAGES_XML_FILE).get()

    def __get_lang_value(self, node):
        """Return the value of "xml:lang" attribute of "node".
           empty string is returned if attribute doesn't exist.
        """

        if msginfo.XML_LANG_ATTR_NAME in node.get_attrs():
            return node.get_attrs()[XML_LANG_ATTR_NAME]
        return ""

    def __get_lcm_msg_strs(self, code):
        """Return the LCM string corresponding to "xpath_tmpl" and "code" ."""

        strs = {"l1": "", "l2": ""}
        try:
            lcm_node = self.messages_xml.get_node(LCM_LINE_PATH_TMPL % code)
            lang_value = self.__get_lang_value(lcm_node)
            if len(lang_value) <= 0:
                strs["l1"] = lcm_node.get_node("./line[@no='1']").get_text()
                strs["l2"] = lcm_node.get_node("./line[@no='2']").get_text()

            if self.target_lang == lang_value:
                strs["l1"] = lcm_node.get_node("./line[@no='1']").get_text()
                strs["l2"] = lcm_node.get_node("./line[@no='2']").get_text()
        except:
            pass
        return strs

    def __get_lang_str(self, xpath_tmpl, code):
        """Return the string corresponding to "xpath_tmpl" and "code" ."""

        default = ""
        for node in self.messages_xml.get_node_list(xpath_tmpl % (code,)):
            lang_value = self.__get_lang_value(node)
            if len(lang_value) <= 0:
                default = node.get_text()

            if self.target_lang == lang_value:
                return node.get_text()

        return default

    def __get_sorted_arg_text_tuple(self, arglist, argtype_list):
        """Return the text list of element of the sorted "arg_list"
           based on the order of "argtype_list".
        """

        sorted_list = []
        for argtype in argtype_list:
            try:
                sorted_list.append(arglist.get_node(ARG_NAME,
                                        {NAME_ATTR_NAME: argtype}).get_text())
            except:
                self.exception("get_value for " + argtype)
                sorted_list.append("")

        return tuple(sorted_list)

    def __get_code_str(self, code):
        """Return the code string for display "code"."""

        return str(int(code) / CODE_CATEGORY_UNIT) + "-" + \
                "%.4d" % (int(code) % CODE_CATEGORY_UNIT,)

    def __get_message_str(self, code, arglist):
        """Return the message string corresponding to "code"."""

        message_base = self.__get_lang_str(MESSAGE_PATH_TMPL, code)
        level = self.messages_xml.get_value(LEVEL_PATH_TMPL % (code,),
                                            GOOD_LEVEL)
        argtype_list = self.messages_xml.get_value_list(
                                        ARGTYPE_PATH_TMPL % (code,))

        if level != GOOD_LEVEL:
            try:
                message_base += " (" + self.__get_code_str(code) + ")"
            except:
                self.exception("can't convert code: " + str(code))
                # don't modify message_base

        if len(argtype_list) <= 0:
            return message_base

        arg_list = arglist.get_node_list(xmldata.ANY_CHILD_NAME)
        if len(arg_list) < len(argtype_list):
            raise InvalidArgError("too few arg for message!!(" +
                str(len(arg_list)) + " < " + str(len(argtype_list)) + ")")

        return message_base % \
                self.__get_sorted_arg_text_tuple(arglist, argtype_list)

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        messageinfo = entry.get().get_node(MESSAGEINFO_PATH)
        code = entry.get().get_node(CODE_PATH).get_text()
        try:
            if entry.get().get_node(OPTION_NAME).is_set_to_true(LCM_ATTR_NAME):
                for_lcm = True
            else:
                for_lcm = False
        except NotFoundError:
            for_lcm = False

        if for_lcm:
            msg_strs = self.__get_lcm_msg_strs(code)
            messageinfo.append_child(MESSAGE_NAME)
            messageinfo.get_node(MESSAGE_NAME).append_child(
                    LINE_NAME, msg_strs["l1"], attrs={"no": "1"})
            messageinfo.get_node(MESSAGE_NAME).append_child(
                    LINE_NAME, msg_strs["l2"], attrs={"no": "2"})
        else:
            try:
                arglist = messageinfo.get_node(ARGLIST_NAME)
            except NotFoundError:
                arglist = xmldata.XmlNode(ARGLIST_NAME)
            msg_str = self.__get_message_str(code, arglist)
            messageinfo.append_child(MESSAGE_NAME, msg_str)
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.target_lang = msginfo.get_lang()

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.MESSAGES_XML_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)

        self.messages_xml = self.__get_messages_xml()


#
# Main
#

if __name__ == "__main__":
    FalMessages(option=falcmd.FalCmdOption()).handle_request()
