#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Microsoft Network Setting."""


import os

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
from fal_info import (ReqTypes, Results)
import pal_info
from iobase.xmldata import ANY_CHILD_NAME
from iobase.exception import NotFoundError
import libfal.util as util
import fal_config as config


MSNETWORK = "msnetwork"
SERVICE = "service"
ASSIGN = "assign"
WINSSRV = "winssrv"


class FalMSNetwork(falcmd.FalCmd):
    """FAL MSNetwork command implementation class."""

    def __get_msnetwork_config(self):
        """Load microsoft network config setting and return XML data ."""

        return self.send_entry_with_file(
            request.Entry(ReqTypes.LOAD), pal_info.PAL_CONFFILE_PATH,
            config.MSNETWORK_CONF_FILE).get()

    def __save_service_config(self, service):
        """Save msnetwork config service."""

        msn_config = self.__get_msnetwork_config()
        msn_config.get_node("service").enable(service)

        save_entry = request.Entry(ReqTypes.SAVE, msn_config)
        self.send_entry_with_file(
            save_entry, pal_info.PAL_CONFFILE_PATH,
            config.MSNETWORK_CONF_FILE)
        return

    def __save_assign_config(self, entry):
        """Save msnetwork config assign and winssrv."""

        # assign
        entry_assign_node = entry.get().get_node(ASSIGN)
        method = entry_assign_node.get_attr_value("method")
        entry_method_node = entry_assign_node.get_node(method)

        msnetwork_config = self.__get_msnetwork_config()
        config_assign_node = msnetwork_config.get_node(ASSIGN)
        config_assign_node.set_attrs({"method": method})
        try:
            config_assign_node.delete_child(method)
        except NotFoundError:
            pass
        config_assign_node.append_child(entry_method_node.get_clone())

        # winssrv
        entry_winssrv_node = entry.get().get_node(WINSSRV)
        msnetwork_config.delete_child(WINSSRV)
        msnetwork_config.append_child(entry_winssrv_node)

        # save
        save_entry = request.Entry(ReqTypes.SAVE, msnetwork_config)
        self.send_entry_with_file(
            save_entry, pal_info.PAL_CONFFILE_PATH,
            config.MSNETWORK_CONF_FILE)
        return

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        # service
        result_service = True
        try:
            new_msnetwork_node = entry.get().get_node(SERVICE)
            result_service = False
            new_service_enable = new_msnetwork_node.is_enabled()
            util.enable_service(
                MSNETWORK, enable=new_service_enable, do_start_stop=True)
            self.__save_service_config(new_service_enable)
            result_service = True
        except:
            pass

        # assign/winssrv
        result_assign = True
        try:
            new_assign_node = entry.get().get_node(ASSIGN)
            result_assign = False
            entry_pal = entry.get_clone()
            try:
                entry_pal.get().delete_child(SERVICE)
            except NotFoundError:
                pass
            result_entry = self.send_entry(entry_pal, pal_info.PAL_SAMBA_PATH)
            if result_entry.get_result() != Results.OK:
                entry.set_result(result_entry.get_result())
                return entry

            util.servicectrl_suspend(
                category_list=[], service_list=["winbind"])
            util.servicectrl_resume(
                category_list=[], service_list=["winbind"])
            self.__save_assign_config(entry)
            result_assign = True
        except:
            pass

        if result_service == True and result_assign == True:
            entry.set_result(Results.OK)
        else:
            entry.set_result(Results.INVALID_XML)
        return entry

    def __list_handler(self, entry):
        """Handle list request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_SAMBA_PATH)
        return result_entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(
            entry, pal_info.PAL_CONFFILE_PATH, config.MSNETWORK_CONF_FILE)

        update_handler = lambda entry: self.send_entry(
            entry, pal_info.PAL_SAMBA_PATH)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.LIST, self.__list_handler)
        self.set_handler(ReqTypes.UPDATE, update_handler)


#
# Main
#

if __name__ == "__main__":
    FalMSNetwork(option=falcmd.FalCmdOption()).handle_request()
