#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling Narsus function"""

import os
import re
import types
from datetime import datetime
from urlparse import urlparse
from libfal.narsus_data import *
from fal_info import (ReqTypes, Results)
from iobase.exception import (NotFoundError, InvalidArgError)
from iobase.xmldata import (XmlNode, XmlConfNode)
import pal_info
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import fal_config as config
import ionas.request as request
import libfal.util as util
import fal_msginfo as msginfo

import urllib
from subprocess import (Popen, PIPE)

try:
    # For Python 2.5.x and later
    from xml.etree.cElementTree import (ElementTree, SubElement, Element,
        XMLTreeBuilder, fromstring, tostring)
    import xml.etree.cElementTree as etree
except ImportError:
    # For old Python
    try:
        from cElementTree import (ElementTree, SubElement, Element,
            XMLTreeBuilder, fromstring, tostring)
        import cElementTree as etree
    except:
        from elementtree.ElementTree import (ElementTree, SubElement,
            Element, XMLTreeBuilder, fromstring, tostring)
        import elementtree.ElementTree as etree

NAESUR_BASE_XML = "<narsus/>"
SERVERINFO_BASE_XML = "<serverinfo/>"
DEFAULT_VALUE = "default"

### X Path ###
URL_XPATH = "./urllist/url"
SERVERINFO_XPATH = "./serverinfolist/serverinfo"

### XML Tag Name ###
NARSUS_NAME = "narsus"
LANDISKINFO_NAME = "landiskinfo"
USECODE_NAME = "usecode"
HOSTNAME_NAME = "hostname"
PORT_NAME = "port"
PROTOCOL_NAME = "protocol"
PATH_NAME = "path"
SERVERINFOLIST_NAME = "serverinfolist"
URLLIST_NAME = "urllist"
URL_NAME = "url"
LANDISK_NAME = "landisk"
PRODUCT_NAME = "product"
STATUSINFO_NAME = "statusinfo"
SYSTEMINFO_NAME = "systeminfo"
DEVICEPORT_NAME = "deviceport"
VOLLIST_NAME = "vollist"
NETWORK_NAME = "network"
KEY_NAME = "key"
SATAPORT_NAME = "sataport"
USBPORT_NAME = "usbport"
SERIALPORT_NAME = "serialport"
SERVICE_NAME = "service"
ANNOUNCE_NAME = "announce"

### XML Attribute ###
TYPE_ATTR = "type"
VERSION_ATTR = "version"
UPDATE_ATTR = "update"
ISSKEY_ATTR = "isskey"
CONNECTED_ATTR = "connected"
CATEGORY_ATTR = "category"
LEVEL_ATTR = "level"
ANNOUNCE_ERROR_LEVEL = "2"
ANNOUNCE_WARNING_LEVEL = "1"
ANNOUNCE_OK_LEVEL = "0"
CONDITION_ATTR = "condition"
CAUSE_ATTR = "cause"

### XML Attribute Value ###
TRUE_ATTR_VAL = "true"

# DEVICE
DEV_EXIST_KEY = "dev_exist"

# SATA DISK
SATA_DISK_VENDOR_KEY = "sata_disk_product"
SATA_DISK_MODEL_KEY = "sata_disk_vendor"
SATA_DISK_REV_KEY = "sata_disk_rev"

# USB DISK
USB_DISK_PRODUCT_KEY = "usb_disk_product"
USB_DISK_VENDOR_KEY = "usb_disk_vendor"
USB_DISK_REV_KEY = "usb_disk_rev"

# UPS DISK
USB_UPS_PRODUCT_KEY = "usb_ups_product"
USB_UPS_VENDOR_KEY = "usb_ups_vendor"
USB_UPS_REV_KEY = "usb_ups_rev"
# LPR DISK
USB_LPR_PRODUCT_KEY = "usb_lpr_product"
USB_LPR_VENDOR_KEY = "usb_lpr_vendor"
USB_LPR_REV_KEY = "usb_lpr_rev"

VOL_SIZE_KEY = "vol_size"
VOL_USESIZE_KEY = "vol_usesize"
SMART_STATUS_KEY = "smart_status"
RAID_LEVEL_KEY = "raid_level"

USECODE_LEN = 20
USECODE_MAC_ADDR_LEN = 9
BASE_54_CHARS = ("Z", "Y", "X", "W", "V", "U", "T", "S", "R",
                 "Q", "P", "N", "M", "L", "K", "J", "H", "G",
                 "F", "E", "D", "B", "A", "z", "y", "x", "w",
                 "v", "u", "t", "s", "r", "q", "p", "n", "m",
                 "k", "j", "i", "h", "g", "f", "e", "d", "b",
                 "a", "9", "8", "7", "6", "5", "4", "3", "2")

BASE_54_CHAR_TO_INT = dict(zip(BASE_54_CHARS, range(0, 54)))
DIVISIONS_FOR_MAC_ADDR = (7, 16)
VALID_DIVISIONS = (8, 17) + DIVISIONS_FOR_MAC_ADDR

### For __send_multiple_request() ###
REQ_TYPE_KEY = "reqtype"
REQ_NODE_KEY = "reqnode"

WGET_CMD_ARGS = ['/usr/bin/wget']
WGET_OPTS = ['-q', '-O', '-', '--timeout=60', '--tries=1', \
                    '--no-check-certificate']
STATUS_NAME = "status"
KEY_TAG_PATH = 'key'
INVALID_KEY = 'null'

NARSUS_SRV_NAME = "narsus"

# for status info
QUOTA_KEY = "quota"
CAPACITY_KEY = "capacity"
UPS_KEY = "ups"
DISKERROR_KEY = "diskerror"

CONDITION_OK = "OK"
CONDITION_NG = "NG"
CONDITION_NOTIFY = "NOTIFY"
ON_PARAM = "ON"
OFF_PARAM = "OFF"

VFSEXT_PORT_MODE = "vfsext"

# serial upsの暫定対策
# 存在しないUSBポート(ポート5)に設定するために
SERIAL_PORT_NUM = str(len(config.USB_PORT_NO_LIST) + 1)


class FalNarsus(falcmd.FalCmd):
    """FAL Narsus implementation class."""

    class __UrlAccessError(Exception):
        pass

    class __InvalidUseCode(Exception):
        pass

    def __init_param(self):
        """Init sevral parameter"""

        SATA_DISK_SCSI_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_sata_disk_arg_map_list

        SATA_DISK_ERR_CHK_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_sata_disk_arg_map_list

        SATA_DISK_ATA_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_sata_disk_arg_map_list

        USB_DISK_SCSI_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_usb_disk_arg_map_list

        INTERNAL_VOLUME_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_internal_volume_arg_map_list

        USB_VOLUME_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_usb_volume_arg_map_list

        USB_VFSEXT_VOLUME_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_usb_vfsext_volume_arg_map_list

        USB_UPS_COMMON_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_usb_ups_arg_map_list

        USB_UPS_DETAIL_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_usb_ups_arg_map_list

        USB_LPR_COMMON_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_usb_lpr_arg_map_list

        # serial ups用暫定対策
        SERIAL_UPS_COMMON_DATA_TRANSFER[ARG_GETTER] = \
            self.__get_serial_ups_arg_map_list

        SMART_EXT_INFO_TRANSFER[ARG_GETTER] = \
            self.__get_sata_disk_arg_map_list

        SMART_EXT_INFO_TRANSFER[RESULT_CONVERTER] = \
            self.__get_smartdaalist_node_for_smart_ext

        self.__dev_exist_info = self.__check_dvice_exist(config.ALL_DEV_LIST)

    def __send_request(self, target_module, type,
                       child_node="", file_attr_value=""):
        """Send request to pal module(single entry)"""

        entry = request.Entry(type, child_node)
        if file_attr_value:
            return self.send_entry_with_file(entry,
                                             target_module,
                                             file_attr_value)

        return self.send_entry(entry, target_module)

    def __send_multiple_request(self, target_module, req_param_list=[]):
        """Send request to pal module(multiple entry)"""

        entrylist = request.EntryList()
        for req_param in req_param_list:
            entrylist.append_entry(request.Entry(req_param[REQ_TYPE_KEY],
                                                 req_param[REQ_NODE_KEY]))
        return self.send_entrylist(entrylist, target_module).get_entry_list()

    def __check_dvice_exist(self, dev_name_list):
        """Check device exist"""

        CHK_EXIST_XML = '<devctrl><device><name>%s</name></device></devctrl>'

        req_param_list = []
        for dev_name in dev_name_list:
            req_param_list.append({REQ_TYPE_KEY: ReqTypes.CHK_EXIST,
                                   REQ_NODE_KEY: CHK_EXIST_XML % dev_name})

        result_entry_list = \
            self.__send_multiple_request(pal_info.PAL_DEVCTRL_PATH,
                                         req_param_list)

        NAME_XPATH = "./device/name"
        dev_exist_info_dict = {}
        for result_entry in result_entry_list:
            dev_name = result_entry.get().get_node(NAME_XPATH).get_text()
            dev_exist_info_dict[dev_name] = False
            if Results.OK == result_entry.get_result():
                dev_exist_info_dict[dev_name] = True

        return dev_exist_info_dict

    def __append_exist_device_node(self, dev_port_node):
        """Append exist device node."""

        DEV_LIST_KEY = "devlist"
        XPATH_KEY = "xpath"
        XML_DATA_KEY = "xmldata"

        SATA_PORT_XPATH = "./sataport/port[@number='%s']"
        USB_PORT_XPATH = "./usbport/port[@number='%s']"

        dev_param_list = [
            {DEV_LIST_KEY: util.get_ideal_sata_hdd_list(),
             XPATH_KEY: SATA_PORT_XPATH,
             XML_DATA_KEY: SATA_DISK_XML},

            {DEV_LIST_KEY: config.USB_HDD_DEV_LIST,
             XPATH_KEY: USB_PORT_XPATH,
             XML_DATA_KEY: USB_DISK_XML},

            {DEV_LIST_KEY: config.USB_UPS_DEV_LIST,
             XPATH_KEY: USB_PORT_XPATH,
             XML_DATA_KEY: USB_UPS_XML},

            {DEV_LIST_KEY: config.USB_LP_DEV_LIST,
             XPATH_KEY: USB_PORT_XPATH,
             XML_DATA_KEY: USB_LPR_XML},

            {DEV_LIST_KEY: config.SERIAL_UPS_DEV_LIST,
             XPATH_KEY: USB_PORT_XPATH,
             XML_DATA_KEY: USB_UPS_XML}]

        for dev_name, flg_exist in self.__dev_exist_info.items():

            if not flg_exist:
                continue

            # デバイスのタイプに対応するXMLノードをappend
            for dev_param in dev_param_list:

                if dev_name not in dev_param[DEV_LIST_KEY]:
                    continue

                if dev_name in config.SERIAL_UPS_DEV_LIST:
                    # serial upsの暫定対策
                    port_xpath = \
                        dev_param[XPATH_KEY] % SERIAL_PORT_NUM
                else:
                    port_xpath = \
                        dev_param[XPATH_KEY] % config.DEV_TO_PORT_NO[dev_name]

                port_node = dev_port_node.get_node(port_xpath)
                port_node.set_attrs({CONNECTED_ATTR: TRUE_ATTR_VAL})

                node = XmlNode(dev_param[XML_DATA_KEY])
                port_node.append_child(node)
        return

    def __append_device_port_node(self, narsusu_node):
        """Append "deviceport" node.
           The node is blank node.
        """

        dev_port_node = XmlNode(DEVICE_PORT_XML)
        sata_port_node = dev_port_node.get_node(SATAPORT_NAME)
        usb_port_node = dev_port_node.get_node(USBPORT_NAME)

        # 最初は各ポートのconnected属性はfalseで作る
        for port in util.get_enable_sata_port_list():
            sata_port_node.append_child(XmlNode(PORT_XML % port))

        for port in config.USB_PORT_NO_LIST:
            usb_port_node.append_child(XmlNode(PORT_XML % port))

        # serial upsの暫定対策
        usb_port_node.append_child(XmlNode(PORT_XML % SERIAL_PORT_NUM))

        # デバイスが存在していたらその
        # デバイスに対するノードを挿入
        self.__append_exist_device_node(dev_port_node)

        narsusu_node.append_child(dev_port_node)
        return

    def __make_static_narsusu_node(self):
        """Make static narsus node."""

        # make landiskinfo node
        iss_key = self.__get_iss_key()
        date = self.__get_current_datetime()
        narsus_node_xml = \
            LANDISK_INFO_BASE_XML % (iss_key, date, NARSUS_XML_VER)
        narsus_node = XmlNode(narsus_node_xml)
        landiskinfo_node = narsus_node.get_node(LANDISKINFO_NAME)
        landiskinfo_node.append_child(XmlNode(PRODUCT_BASE_XML))
        landiskinfo_node.append_child(XmlNode(SYS_INFO_BASE_XML))
        landiskinfo_node.append_child(XmlNode(STATUS_INFO_BASE_XML))
        landiskinfo_node.append_child(XmlNode(VOLLIST_BASE_XML))
        landiskinfo_node.append_child(XmlNode(NETWORK_BASE_XML))
        self.__append_device_port_node(landiskinfo_node)

        return narsus_node

    def __set_transfer_args(self, target_value, need_arg_types, arg_dict):
        """Set args"""

        args_tpl = ()
        for arg_type in  need_arg_types:
            args_tpl += (arg_dict[arg_type],)
        return target_value % args_tpl

    def __update_landiskinfo_node(self, narsus_node, transfer_dict):
        """Update landisk info node."""

        if ARG_GETTER not in transfer_dict:
            self.__sub_update_landiskinfo(narsus_node, transfer_dict)
            return

        arg_map_list = []
        # 引数取得用関数 引数のマップのリスト
        # [{a:b, c:d}, {e:f}, {} ... ]
        # それぞれのリストの要素数だけnodeが存在する
        arg_map_list = transfer_dict[ARG_GETTER]()

        for arg_map in arg_map_list:
            self.__sub_update_landiskinfo(narsus_node,
                                          transfer_dict,
                                          arg_map)
        return

    def __sub_update_landiskinfo(self, narsus_node, transfer_dict, arg_map={}):
        """Update landisk info node.
           step1: Send request to pal module by transfer_dict.
           step2: Get the value from result entry by transfer_dict.
           step3: Set the value into landiskinfo node by transfer_dict.
        """

        module = transfer_dict[PAL_MODULE]
        req_type = transfer_dict[REQ_TYPE]
        req_child_xml = transfer_dict[REQ_XML]

        if REQ_XML_ARG_TYPES in transfer_dict:  # 引数を取る場合
            arg_type_list = transfer_dict[REQ_XML_ARG_TYPES]
            req_child_xml = \
                self.__set_transfer_args(req_child_xml, arg_type_list, arg_map)

        file_attr_value = ""
        if WITH_FILE in transfer_dict:  # entryにfile属性を取る場合
            file_attr_value = transfer_dict[WITH_FILE]

        # PALへのリクエストXML
        result_entry = self.__send_request(module,
                                           req_type,
                                           req_child_xml,
                                           file_attr_value)

        # result nodeを変換する必要があるかどうか
        if transfer_dict.get(RESULT_CONVERTER):
            result_node = transfer_dict[RESULT_CONVERTER](result_entry)
        else:
            result_node = result_entry.get()

        for xpath_dict in transfer_dict[SRC_DST_LST]:

            dst_path = xpath_dict[DST_PATH]  # コピー先(To landiskinfo)

            # landiskinfoノードに取得した値を挿入
            if DST_ARG_TYPES in xpath_dict:  # 引数を取る場合
                arg_type_list = xpath_dict[DST_ARG_TYPES]
                dst_path = \
                    self.__set_transfer_args(dst_path, arg_type_list, arg_map)

            src_path = xpath_dict[SRC_PATH]  # コピー元(From pal)

            # ノードごとコピーする場合
            if IS_TRANSFER_NODE in xpath_dict.keys():
                if xpath_dict[IS_TRANSFER_NODE]:
                    src_node = result_node.get_node(src_path)
                    narsus_node.get_node(dst_path).append_child(src_node)
                    continue

            # ノードをupdateする場合
            if xpath_dict.get(IS_UPDATE_NODE):
                src_node = result_node.get_node(src_path)
                narsus_node.get_node(dst_path).update(src_node)
                continue

            # ノードの要素か属性に挿入する
            # テキストを挿入する
            value = result_node.get_node(src_path).get_text()

            if TEXT_CONVERTER in xpath_dict.keys():
                converter = xpath_dict[TEXT_CONVERTER]
                if isinstance(converter, types.FunctionType):
                    value = converter(value)
                else:
                    value = converter[value]

            if DST_ATTR in xpath_dict:  # 属性値に挿入する場合
                attr = xpath_dict[DST_ATTR]
                narsus_node.get_node(dst_path).set_attrs({attr: value})
            else:
                narsus_node.get_node(dst_path).set_text(value)

    def __get_internal_volume_arg_map_list(self):
        """Get internal volume name arg map"""

        arg_map_list = []
        vol_name = config.STOR_IF_TO_VOL["internal"][1]
        enable_part_name = util.get_volume_cache_value(vol_name, "name")
        arg_map_list.append({DEVNAME_KEY: enable_part_name})
        return arg_map_list

    def __get_usb_volume_arg_map_list(self):
        """Get arg for USB_VOLUME_DATA_TRANSFER"""

        arg_map_list = []

        for dev_name, flg_exist in self.__dev_exist_info.iteritems():

            if dev_name not in config.USB_HDD_DEV_LIST:
                continue

            if not flg_exist:
                continue

            usb_port_num = config.DEV_TO_PORT_NO[dev_name]

            try:
                if VFSEXT_PORT_MODE == \
                       util.get_current_port_mode("usb", usb_port_num):
                    continue
            except:
                self.exception("fail to get_current_port_mode()")
                continue

            arg_map_list.append({DEVNAME_KEY: dev_name,
                                 PORTNUM_KEY: usb_port_num})
        return arg_map_list

    def __get_usb_vfsext_volume_arg_map_list(self):
        """Get arg for USB_VOLUME_DATA_TRANSFER"""

        arg_map_list = []

        for dev_name, flg_exist in self.__dev_exist_info.iteritems():

            if dev_name not in config.USB_HDD_DEV_LIST:
                continue

            if not flg_exist:
                continue

            usb_port_num = config.DEV_TO_PORT_NO[dev_name]

            try:
                if not VFSEXT_PORT_MODE == \
                       util.get_current_port_mode("usb", usb_port_num):
                    continue
            except:
                self.exception("fail to get_current_port_mode()")
                continue

            arg_map_list.append({DEVNAME_KEY: dev_name,
                                 PORTNUM_KEY: usb_port_num})
        return arg_map_list

    def __get_sata_disk_arg_map_list(self):
        """Get arg for SATA_DISK_SCSI_DATA_TRANSFER
            and SATA_DISK_ERR_CHK_DATA_TRANSFER
            and SATA_DISK_ATA_DATA_TRANSFER
        """

        return self.__get_device_port_arg_map_list(
                   util.get_ideal_sata_hdd_list())

    def __get_usb_disk_arg_map_list(self):
        """Get arg for USB_DISK_SCSI_DATA_TRANSFER"""

        return self.__get_device_port_arg_map_list(config.USB_HDD_DEV_LIST)

    def __get_usb_ups_arg_map_list(self):
        """Get arg for
              USB_UPS_COMMON_DATA_TRANSFER
              USB_UPS_DETAIL_DATA_TRANSFER
        """

        return self.__get_device_port_arg_map_list(config.USB_UPS_DEV_LIST)

    def __get_usb_lpr_arg_map_list(self):
        """Get arg for USB_LPR_COMMON_DATA_TRANSFER"""

        return self.__get_device_port_arg_map_list(config.USB_LP_DEV_LIST)

    def __get_serial_ups_arg_map_list(self):
        """Get arg for
              SERIAL_UPS_COMMON_DATA_TRANSFER
        """

        return self.__get_device_port_arg_map_list(config.SERIAL_UPS_DEV_LIST)

    def __get_smartdaalist_node_for_smart_ext(self, pal_result_entry):

        SMART_DATA_TMPL = '<smartdata>' + \
                             '<id>%(id)s</id>' + \
                             '<flag>0x%(signature)s</flag>' + \
                             '<value>255</value>' + \
                             '<worstvalue>255</worstvalue>' + \
                             '<threshold>0</threshold>' + \
                             '<rawvalue>%(rawvalue)s</rawvalue>' + \
                           '</smartdata>'

        diskctrl_node = pal_result_entry.get()

        statistics_node = diskctrl_node.get_node("disk/statistics")
        signature = statistics_node.get_node("./signature").get_text()
        smartdatalist_node = XmlNode("smartdatalist")
        for id, xpath in SMART_EXT_ID_XPATH_MAP.iteritems():
            raw_value = statistics_node.get_value(xpath, None)
            if raw_value:
                smart_data_xml = SMART_DATA_TMPL % {"id": id,
                                                    "signature": signature,
                                                    "rawvalue": raw_value}
                smartdatalist_node.append_child(XmlNode(smart_data_xml))

        diskctrl_node.get_node("disk").append_child(smartdatalist_node)
        return diskctrl_node

    def __get_device_port_arg_map_list(self, dev_list):
        """Get args for device data transfer"""

        arg_map_list = []
        for dev_name, flg_exist in self.__dev_exist_info.items():

            if dev_name not in dev_list:
                continue
            if not flg_exist:
                continue

            # serial upsの暫定対策
            if dev_name in config.SERIAL_UPS_DEV_LIST:
                arg_map_list.append({DEVNAME_KEY: dev_name,
                                     PORTNUM_KEY: SERIAL_PORT_NUM})
            else:
                arg_map_list.append(
                    {DEVNAME_KEY: dev_name,
                     PORTNUM_KEY: config.DEV_TO_PORT_NO[dev_name]})

        return arg_map_list

    def __get_mounted_usb_partition_name(self, usb_disk_name):
        """Get mounted usb partition name."""

        req_base_xml = '<volctrl><volume>' + \
                       '<name type="partition">%s</name>' + \
                       '</volume></volctrl>'

        STATUS_XPATH = "./volume/status"
        MOUNTED_VAL = "mounted"

        vol_name = config.DEV_TO_VOL[usb_disk_name]
        return util.get_volume_cache_value(vol_name, "name")

    def __get_mac_addr(self):
        """Return MAC address string."""

        return util.get_mac_address("netif1")

    def __url_access(self, url, data):
        """Send "data"(string or ElementTree) to "url" by POST method."""

        try:
            proxy = util.get_proxy(url)
        except:
            proxy = None

        try:
            response_data = util.urlaccess_get(url, data, proxy=proxy)
        except:
            self.exception("fail to urlaccess_get()")
            raise self.__UrlAccessError

        response_node = XmlNode(response_data)
        status = response_node.get_node(STATUS_NAME).get_text()
        if "0" == status:
            return response_node

    def __enable_narsus(self, narsus_node):

        # get param
        sendflag = "0"
        if narsus_node.get_node(SERVICE_NAME).is_enabled():
            sendflag = "1"
            util.enable_service(NARSUS_SRV_NAME,
                                enable=True, do_start_stop=True)
        else:
            sendflag = "0"
            util.enable_service(NARSUS_SRV_NAME,
                                enable=False, do_start_stop=True)

        usecode = narsus_node.get_node(USECODE_NAME).get_text(do_strip=False)
        isskey = narsus_node.get_node(KEY_NAME).get_text()
        mac_addr = self.__get_mac_addr()
        mode_name = self.__get_model_name()
        CONFIG_URL_XPATH = "./serverinfolist/serverinfo[@type='config']"
        config_node = narsus_node.get_node(CONFIG_URL_XPATH)
        config_url = self.__conv_serverinfo_node_and_url_str(config_node)

        # make request string
        request_str = 'model=' + urllib.quote(mode_name)
        request_str += '&macaddress=' + urllib.quote(mac_addr)
        request_str += '&key=' + urllib.quote(isskey)
        request_str += '&sendflag=' + urllib.quote(sendflag)
        if 0 < len(usecode):
            request_str += '&usecode=' + urllib.quote(usecode)

        response_node = self.__url_access(config_url, request_str)

        if self.__is_exist_node(response_node, KEY_NAME):
            key = response_node.get_node(KEY_NAME).get_text()
            # FIXME: Test server often return INVALID_KEY. So ignore it now.
            #if INVALID_KEY == key:
            #    key = ""
            #narsus_node.get_node(KEY_NAME).set_text(key)
            if INVALID_KEY != key:
                narsus_node.get_node(KEY_NAME).set_text(key)

        if self.__is_exist_node(response_node, USECODE_NAME):
            usecode = response_node.get_node(
                USECODE_NAME).get_text(do_strip=False)
            if usecode:
                self.__validate_usecode(usecode)
                narsus_node.get_node(USECODE_NAME).set_text(usecode)
                narsus_node.get_node(USECODE_NAME).enable()

        return

    def __is_exist_node(self, node, node_path):
        """Check whether node_path exists in specified node"""

        try:
            node.get_node(node_path)
        except NotFoundError:
            return False
        return True

    def __load_config(self, conffile=config.NARSUS_CONF_FILE):
        """Load config data and return a response entry data."""

        load_entry = request.Entry(ReqTypes.LOAD)
        result_entry = self.send_entry_with_file(load_entry,
                                                 pal_info.PAL_CONFFILE_PATH,
                                                 conffile)
        return result_entry

    def __is_current_raid_mode_vfsext(self):

        conf_node = self.__load_config(config.RAIDCTRL_CONF_FILE).get()
        level = conf_node.get_value("raidlist/raid/level")

        return config.RAID_MODE_INFO[level]["is_ext_raid"]

    def __save_config(self, xml_config):
        """Save "xml_config" data and return a response entry data"""

        save_entry = request.Entry(ReqTypes.SAVE, xml_config)
        result_entry = self.send_entry_with_file(save_entry,
                                                 pal_info.PAL_CONFFILE_PATH,
                                                 config.NARSUS_CONF_FILE)
        return result_entry

    def __get_current_datetime(self):
        """Get current datetime."""

        now = datetime.now()
        return now.strftime("%Y/%m/%d %H:%M:%S")

    def __get_iss_key(self):
        """Get iss key from config file."""

        narsus_node = self.__load_config().get()
        return narsus_node.get_node(KEY_NAME).get_text()

    def __conv_serverinfo_node_and_url_str(self, serverinfo_node):
        """parse URL string and return each parts dictionary."""

        protocol = serverinfo_node.get_node(PROTOCOL_NAME).get_text()
        hostname = serverinfo_node.get_node(HOSTNAME_NAME).get_text()
        path = serverinfo_node.get_node(PATH_NAME).get_text()
        port = serverinfo_node.get_node(PORT_NAME).get_text()

        if port == DEFAULT_VALUE:
            url = "{0}://{1}{2}".format(protocol, hostname, path)
        else:
            url = "{0}://{1}:{2}{3}".format(protocol, hostname, port, path)
        return url

    def __update_status_info_node(self, landiskinfo_node):
        """Update status info node"""

        # load_entry = request.Entry(ReqTypes.LOAD)
        # result_entry = self.send_entry_with_file(load_entry,
        #                                        pal_info.PAL_CONFFILE_PATH,
        #                                         config.ANNOUNCE_FILE)

        return

    def __get_model_name(self):
        """Return Model name."""

        MODEL_NAME = "model"
        res_entry = \
            self.__send_request(pal_info.PAL_SYSINFO_PATH, ReqTypes.QUERY)

        return res_entry.get().get_node(MODEL_NAME).get_text()

    def __to_shuffled_idx(self, idx):
        """Return the index shuffled from "idx" for the MAC address."""

        return ((idx + 7) % 4) * 5 + (idx + 7) / 4

    def __base_54_str_to_int(self, base_54_str):
        """Convert base 54 string to integer."""

        int_value = 0
        for value_char in base_54_str:
            if value_char == 0:
                continue
            int_value = int_value * 54 + BASE_54_CHAR_TO_INT[value_char]

        return int_value

    def __to_check_weight(self, idx):
        """Return the weight value for the check digit."""

        return ((18 - idx) % 6 + 2)

    def __validate_check_digit(self, usecode):
        """Validate the usecode's check digit."""

        sum = 0
        for (idx, code_char) in enumerate(usecode[0:USECODE_LEN - 1]):
            sum += BASE_54_CHAR_TO_INT[code_char] * self.__to_check_weight(idx)

        check_digit = 53 - sum % 54
        if check_digit != BASE_54_CHAR_TO_INT[usecode[-1]]:
            raise self.__InvalidUseCode("invalid usecode as check digit.")

        return

    def __validate_mac_addr(self, usecode):
        """Validate the MAC address in usecode."""

        base_54_str = ""
        for idx in range(0, USECODE_MAC_ADDR_LEN):
            base_54_str += usecode[self.__to_shuffled_idx(idx)]

        mac_addr = re.sub(":", "", self.__get_mac_addr())

        if self.__base_54_str_to_int(base_54_str) != int(mac_addr, 16):
            raise self.__InvalidUseCode("invalid usecode as MAC.")

        return

    def __validate_usecode(self, usecode):
        """Validate the usecode data."""

        if len(usecode) != USECODE_LEN:
            raise self.__InvalidUseCode("usecode length is invalid.")

        self.__validate_check_digit(usecode)

        division = BASE_54_CHAR_TO_INT[usecode[0]]
        if division not in VALID_DIVISIONS:
            raise self.__InvalidUseCode("usecode is invalid division.")

        if division in DIVISIONS_FOR_MAC_ADDR:
            self.__validate_mac_addr(usecode)

        return

    def __get_announcelist_node(self):

        if not self.__announcelist_node:
            self.__announcelist_node = \
                self.__load_config(config.ANNOUNCE_FILE).get()

        return self.__announcelist_node

    def __get_status_error_map_from_announce(self):

        status_error_map = {QUOTA_KEY: False,
                            CAPACITY_KEY: False,
                            UPS_KEY: False,
                            DISKERROR_KEY: False}

        announcelist_node = self.__get_announcelist_node()
        for announce_node in announcelist_node.get_node_list(ANNOUNCE_NAME):

            try:
                cause = announce_node.get_attr_value(CAUSE_ATTR)
            except NotFoundError:
                continue

            status_error_map[cause] = True

        return status_error_map

    def __get_notify_condition(self):

        is_notify = False

        announcelist_node = self.__get_announcelist_node()
        for announce_node in announcelist_node.get_node_list(ANNOUNCE_NAME):

            level = announce_node.get_attr_value(LEVEL_ATTR)

            if ANNOUNCE_ERROR_LEVEL == level:
                return CONDITION_NG

            if ANNOUNCE_ERROR_LEVEL == level:
                is_notify = True

        if is_notify:
            return CONDITION_NOTIFY

        return CONDITION_OK

    def __set_status_info(self, landiskinfo_node):

        STATUS_XPATH = "./statusinfo/status"
        NOTIFY_XPATH = "./statusinfo/status/notify[@type='notify']/param"
        STATUS_ERROR_XPATH_TMPL = "./statusinfo/status/error[@type='%s']/param"

        status_error_map = self.__get_status_error_map_from_announce()

        for type, is_error in status_error_map.iteritems():

            param_node = \
                landiskinfo_node.get_node(STATUS_ERROR_XPATH_TMPL % type)

            if is_error:
                param_node.set_text(CONDITION_NG)
            else:
                param_node.set_text(CONDITION_OK)

        # status condition値
        notify_condition = self.__get_notify_condition()
        status_node = landiskinfo_node.get_node(STATUS_XPATH)
        status_node.set_attrs({CONDITION_ATTR: notify_condition})

        # お知らせ有無
        notify_node = landiskinfo_node.get_node(NOTIFY_XPATH)
        if CONDITION_OK == notify_condition:
            notify_node.set_text(OFF_PARAM)
        else:
            notify_node.set_text(ON_PARAM)

        return

    #
    # Request Handler
    #

    def __get_handler(self, entry):
        """Handle "get" request."""

        self.__init_param()

        # 最初にNarsus用ノードを作ってしまう
        # (ただしノードの各値は空)
        narsus_node = self.__make_static_narsusu_node()
        landiskinfo_node = narsus_node.get_node(LANDISKINFO_NAME)

        # status infoをセット
        try:
            self.__set_status_info(landiskinfo_node)
        except:
            self.exception("Fail to [__set_status_info]")
            pass

        if self.__is_current_raid_mode_vfsext():
            internal_vol_transfer = VFSEXT_INTERNAL_VOLUME_DATA_TRANSFER
        else:
            internal_vol_transfer = INTERNAL_VOLUME_DATA_TRANSFER

        # 作られたノードに値を挿入していく
        data_transfer_list = [
            SYSINFO_DATA_TRANSFER,
            NETWORK_DATA_TRANSFER,
            internal_vol_transfer,
            RAID_DATA_TRANSFER,
            SATA_DISK_SCSI_DATA_TRANSFER,
            SATA_DISK_ERR_CHK_DATA_TRANSFER,
            SATA_DISK_ATA_DATA_TRANSFER,
            SMART_EXT_INFO_TRANSFER,
            USB_DISK_SCSI_DATA_TRANSFER,
            USB_VOLUME_DATA_TRANSFER,
            USB_VFSEXT_VOLUME_DATA_TRANSFER,
            USB_UPS_COMMON_DATA_TRANSFER,
            USB_UPS_DETAIL_DATA_TRANSFER,
            # serial ups用暫定対策
            SERIAL_UPS_COMMON_DATA_TRANSFER,
            USB_LPR_COMMON_DATA_TRANSFER,
            VOL_LINK_LIST_TRANSFER
        ]
        for data_transfer in data_transfer_list:

            # 拡張ボリュームの時はpalからRAIDモード
            # が取得できない
            # 特別対応
            if RAID_DATA_TRANSFER == data_transfer:
                if self.__is_current_raid_mode_vfsext():
                    vol_node = landiskinfo_node.get_node("./vollist/vol")
                    vol_node.set_attrs({"level": VFSEXT_STR})
                    continue

            try:
                self.__update_landiskinfo_node(landiskinfo_node, data_transfer)
            except:
                self.exception("Fail to update landisk info")
                pass

        self.__update_status_info_node(landiskinfo_node)

        entry.set(narsus_node)
        entry.set_result(Results.OK)
        return entry

    def __try_update_usecode(self):
        """Try updating usecode, if it is not saved."""

        try:
            xml_config = self.__load_config().get()
            if not xml_config.get_node(SERVICE_NAME).is_enabled():
                return

            usecode_node = xml_config.get_node(USECODE_NAME)
            if (not usecode_node.is_enabled() or
                not usecode_node.get_text(do_strip=False)):

                self.__enable_narsus(xml_config)
                self.__save_config(xml_config)
        except:
            pass

    def __query_handler(self, entry):
        """Handle "query" request."""

        self.__try_update_usecode()
        result_entry = self.__load_config()

        result_entry.get().append_child(URLLIST_NAME)
        urllist_node = result_entry.get().get_node(URLLIST_NAME)

        serverinfo_node_list = \
            result_entry.get().get_node_list(SERVERINFO_XPATH)

        for serverinfo_node in serverinfo_node_list:
            type = serverinfo_node.get_attr_value(TYPE_ATTR)
            url_str = self.__conv_serverinfo_node_and_url_str(serverinfo_node)
            urllist_node.append_child(URL_NAME, url_str, {TYPE_ATTR: type})

        result_entry.get().delete_children(SERVERINFOLIST_NAME)
        return result_entry

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle "apply" request."""

        saved_narsus_node = self.__load_config().get()
        narsus_node = XmlConfNode(entry.get())

        if self.__is_exist_node(narsus_node, USECODE_NAME):

            if narsus_node.get_node(USECODE_NAME).is_enabled():

                usecode = narsus_node.get_node(
                    USECODE_NAME).get_text(do_strip=False)
                try:
                    self.__validate_usecode(usecode)
                except self.__InvalidUseCode:
                    entry.set_result(Results.INVALID_CODE)
                    return entry

                saved_narsus_node.get_node(USECODE_NAME).set_text(usecode)
                saved_narsus_node.get_node(USECODE_NAME).enable()
            else:
                saved_narsus_node.get_node(USECODE_NAME).set_text("")
                saved_narsus_node.get_node(USECODE_NAME).enable(False)

        if self.__is_exist_node(narsus_node, SERVICE_NAME):

            saved_narsus_node.delete_children(SERVICE_NAME)
            saved_narsus_node.append_child(narsus_node.get_node(SERVICE_NAME))

            try:
                self.__enable_narsus(saved_narsus_node)
            except self.__UrlAccessError:
                entry.set_result(Results.APPLY_ERROR)
                return entry

        result_entry = self.__save_config(saved_narsus_node)

        entry.set_result(result_entry.get_result())
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.__dev_exist_info = {}
        self.__announcelist_node = None

        save_load_handler = lambda entry: self.send_entry_with_file(entry,
                             pal_info.PAL_CONFFILE_PATH,
                             config.NARSUS_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_load_handler)
        self.set_handler(ReqTypes.LOAD, save_load_handler)

        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.GET, self.__get_handler)
#
# Main
#

if __name__ == "__main__":
    FalNarsus(option=falcmd.FalCmdOption()).handle_request()
