#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting network and getting network condition."""


import os

import iobase.xmldata as xmldata
import ionas.request as request
import ionas.nascmd as nascmd

import libfal.falcmd as falcmd
from libfal.util import dispatch_event, servicectrl_suspend, servicectrl_resume
import pal_info
from fal_info import (ReqTypes, Results)
from iobase.exception import (NotFoundError,)

import fal_config as config


FAMILYCONFIG_NAME = "familyconfig"
FAMILY_INET6 = "inet6"
FAMILY_INET4 = "inet4"
DNS_NAME = "dns"
GATEWAY_NAME = "gateway"
NETIFLIST_NAME = "netiflist"
NETIF_NAME = "netif"

FAMILY_INET6_PATH = FAMILYCONFIG_NAME + "/" + FAMILY_INET6
FAMILY_INET4_PATH = FAMILYCONFIG_NAME + "/" + FAMILY_INET4
DNS_PATH = FAMILY_INET6_PATH + "/" + DNS_NAME
DNS_INET4_PATH = FAMILY_INET4_PATH + "/" + DNS_NAME
GATEWAY_PATH = FAMILY_INET6_PATH + "/" + GATEWAY_NAME
GATEWAY_INET4_PATH = FAMILY_INET4_PATH + "/" + GATEWAY_NAME
NETIF_PATH = NETIFLIST_NAME + "/" + NETIF_NAME
PROTOCOL_PATH = "./protocollist/protocol"

SOURCE_ATTR = "source"
NAME_ATTR = "name"
DEV_ATTR = "dev"
FAMILY_ATTR = "family"
ENABLE_ATTR = xmldata.XmlConfNode.ENABLE_NAME

NETIF_ATTR_LIST = [ENABLE_ATTR, "wol", "mtu"]


class FalNetwork(falcmd.FalCmd):
    """FAL Network command implementation class."""

    def __is_exist_node(self, node, node_path):
        """Check whether node_path exists in specified node"""

        try:
            node.get_node(node_path)
        except NotFoundError:
            return False
        return True

    def __set_source_attr_value(self, node, to_values):
        """Set the source attribute's value to value in "to_values"."""

        source = node.get_attr_value(SOURCE_ATTR)
        if source in to_values:
            node.set_attrs({SOURCE_ATTR: to_values[source]})

    def __change_net_dev_to_if(self, network):
        """Change the net devince name to the interface name."""

        for netif in network.get_node_list(NETIF_PATH):
            attrs = netif.delete_attrs((DEV_ATTR,))
            if not DEV_ATTR in attrs:
                raise NotFoundError(DEV_ATTR, attrs)

            dev = attrs[DEV_ATTR]
            if dev in config.NET_DEV_TO_IF:
                netif.set_attrs({NAME_ATTR: config.NET_DEV_TO_IF[dev]})

        for src_path in (DNS_PATH, DNS_INET4_PATH,
                            GATEWAY_PATH, GATEWAY_INET4_PATH):

            try:
                self.__set_source_attr_value(network.get_node(src_path),
                                            config.NET_DEV_TO_IF)
            except:
                pass

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        response_entry = self.send_entry(entry, pal_info.PAL_NETWORK_PATH)
        self.__change_net_dev_to_if(response_entry.get())
        return response_entry

    def __change_net_if_to_dev(self, network):
        """Change the net interface name to the devince name."""

        for netif in network.get_node_list(NETIF_PATH):
            name = netif.get_attr_value(NAME_ATTR)
            if name in config.NET_IF_TO_DEV:
                netif.set_attrs({DEV_ATTR: config.NET_IF_TO_DEV[name]})

        for src_path in (DNS_PATH, DNS_INET4_PATH,
                            GATEWAY_PATH, GATEWAY_INET4_PATH):

            try:
                self.__set_source_attr_value(network.get_node(src_path),
                                            config.NET_IF_TO_DEV)
            except:
                pass

    def __load_config(self):
        """Load network configuration and return it."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                    pal_info.PAL_CONFFILE_PATH,
                                    config.NETWORK_CONF_FILE).get()

    def __save_config(self, network):
        """Save "network" configuration."""

        return self.send_entry_with_file(request.Entry(ReqTypes.SAVE, network),
                                    pal_info.PAL_CONFFILE_PATH,
                                    config.NETWORK_CONF_FILE).get()

    def __must_be_netif_enabled(self, netif):
        """Return if the "netif" must be enabled or not."""

        for protocol in netif.get_node_list(PROTOCOL_PATH):
            if protocol.is_enabled():
                return True

        return False

    def __update_netif_enable(self, network):
        """Update the "enable" attribute of each "netif" in "network"."""

        for netif in network.get_node_list(NETIF_PATH):
            netif.enable(self.__must_be_netif_enabled(netif))

    def __update_netif_attribute(self, netif, other):
        """Update attributes of "netif" by other."""

        attrs = other.get_attrs()
        for attr in list(attrs.keys()):
            if not attr in NETIF_ATTR_LIST:
                del attrs[attr]
        netif.set_attrs(attrs)

    def __update_netif(self, netif, other):
        """Update the "netif" tree by other."""

        self.__update_netif_attribute(netif, other)

        for protocol in other.get_node_list(PROTOCOL_PATH):
            config_protocol = netif.get_node(PROTOCOL_PATH,
                        {FAMILY_ATTR: protocol.get_attr_value(FAMILY_ATTR)})
            config_protocol.update(protocol, override=True)

    def __merge_conf(self, entry, pal_network):

        pal_network.update(self.__load_config())

        try:
            entry_inet4 = entry.get().get_node(FAMILY_INET4_PATH)
            pal_network_inet4 = pal_network.get_node(FAMILY_INET4_PATH)
            pal_network_inet4.update(entry_inet4)
        except:
            pass

        try:
            entry_inet6 = entry.get().get_node(FAMILY_INET6_PATH)
            pal_network_inet6 = pal_network.get_node(FAMILY_INET6_PATH)
            pal_network_inet6.update(entry_inet6)
        except:
            pass

        for pal_network_netif in pal_network.get_node_list(NETIF_PATH):
            try:
                entry_netif = entry.get().get_node(
                    NETIF_PATH,
                    {NAME_ATTR: pal_network_netif.get_attr_value(NAME_ATTR)})
                self.__update_netif(pal_network_netif, entry_netif)
            except:
                pass

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        pal_entry = request.Entry(type="apply", source="<network/>")
        pal_network = pal_entry.get()
        if config.CONFIG_NETWORK_SUPPORTED:
            # 設定ファイル + entry の内容で pal に request する
            self.__merge_conf(entry, pal_network)
        else:
            pal_network.update(entry.get().get_clone())

        self.__update_netif_enable(pal_network)
        self.__change_net_if_to_dev(pal_network)
        response_entry = self.send_entry(pal_entry, pal_info.PAL_NETWORK_PATH)

        self.__change_net_dev_to_if(response_entry.get())

        if (response_entry.get_result() != Results.OK or
            not config.CONFIG_NETWORK_SUPPORTED):

            entry.set_result(response_entry.get_result())
            return response_entry

        ## 設定ファイル更新 ##
        network = self.__load_config()

        try:
            familyconfig = network.get_node(FAMILYCONFIG_NAME)

            try:
                entry_inet4 = entry.get().get_node(FAMILY_INET4_PATH)
                inet4 = network.get_node(FAMILY_INET4_PATH)
                inet4.update(entry_inet4, override=True)
            except NotFoundError:
                pass

            try:
                entry_inet6 = entry.get().get_node(FAMILY_INET6_PATH)
                inet6 = network.get_node(FAMILY_INET6_PATH)
                inet6.update(entry_inet6, override=True)
            except NotFoundError:
                pass

        except NotFoundError:
            pass

        for netif in entry.get().get_node_list(NETIF_PATH):
            try:
                config_netif = network.get_node(
                    NETIF_PATH,
                    {NAME_ATTR: netif.get_attr_value(NAME_ATTR)})
                self.__update_netif(config_netif, netif)
            except:
                pass

        self.__update_netif_enable(network)
        self.__save_config(network)

        # event
        try:
            dispatch_event('network', 'apply', entry.get().get_clone())
        except:
            pass

        return response_entry

    def __restart_handler(self, entry):
        """Handle restart request"""

        try:
            servicectrl_suspend(category_list=['network'], service_list=[])
        except:
            pass

        entry = self.send_entry(entry, pal_info.PAL_NETWORK_PATH)

        try:
            servicectrl_resume(category_list=['network'], service_list=[])
        except:
            pass

        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.NETWORK_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.RESTART, self.__restart_handler)


#
# Main
#

if __name__ == "__main__":
    FalNetwork(option=falcmd.FalCmdOption()).handle_request()
