#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for managing packages."""


import sys
import os
import shutil
import urlparse

import iobase.xmldata as xmldata
import ionas.request as request
import ionas.nascmd as nascmd

import libfal.falcmd as falcmd
import libfal.util as util
import pal_info
from fal_info import (ReqTypes, Results)
from iobase.exception import (NotFoundError,)
from libfal.util import dispatch_event

import fal_config as config


PROTOCOL_PATH = "config/serverinfo/protocol"
HOSTNAME_PATH = "config/serverinfo/hostname"
PATH_PATH = "config/serverinfo/path"
INFOFILE_PATH = "config/serverinfo/infofile"
LOCAL_PATH_PATH = "config/localinfo/pathlist/path"

PACKAGELIST_NAME = "packagelist"
PACKAGE_NAME = "package"
UPDATE_ATTR = "update"
AVAILABLE_ATTR = "available"
TRUE_TEXT = "true"
FALSE_TEXT = "false"
PACKAGE_PATH = "packagelist/package"
PACKAGEID_NAME = "packageid"
VERSION_NAME = "version"
FILE_NAME = "file"
INTERNALNAME_PATH = "package/internalname"
INTERNALNAME_NAME = "internalname"
ICON_NAME = "icon"
TARGET_PATH = "targetlist/target"
NAME_NAME = "name"

TARGET_KEY = "target"
AVAILABLE_KEY = AVAILABLE_ATTR

APPLY_TAG_NAME_LIST = ("updatenotice", "autoupdate", "time")

PACKAGES_BASE_XML_STR = '<packages/>'
PACKAGELIST_BASE_XML_STR = '<packagelist/>'
EMPTY_PACKAGE_LIST_XML_STR = '<packages><packagelist/></packages>'


class DownloadError(Exception):
    """Download Error."""

    pass


class InstallError(Exception):
    """Install Error."""

    pass


class UninstallError(Exception):
    """Uninstall Error."""

    pass


class FalPackages(falcmd.FalCmd):
    """FAL Packages command implementation class."""

    def __load_config(self, file_name):
        """Load a 'file_name' file setting and return XML data ."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                        pal_info.PAL_CONFFILE_PATH,
                                        file_name).get()

    def __save_config(self, file_name, xml_data):
        """Save the "xml_data" XML data to a 'file_name' file."""

        entry = request.Entry(ReqTypes.SAVE)
        entry.set(xml_data)
        return self.send_entry_with_file(entry,
                                pal_info.PAL_CONFFILE_PATH, file_name)

    def __get_installed_packages_info(self):
        """Return installed packages info."""

        try:
            return self.__load_config(config.INSTALLED_PACKAGES_FILE)
        except:
            return xmldata.XmlNode(EMPTY_PACKAGE_LIST_XML_STR)

    def __get_info_file_name(self):
        """Return a info file name from configuration."""

        return self.package_config.get_node(INFOFILE_PATH).get_text()

    def __get_local_info_file_path(self):
        """Return a local info file path."""

        return os.path.join(config.PACKAGES_DIR,
                        os.path.basename(self.__get_info_file_name()))

    def __get_packageid(self, package):
        """Return a packageid for "package"."""

        return package.get_node(PACKAGEID_NAME).get_text()

    def __get_package_target_version(self, package, series, model):
        """Return a target version for "package"."""

        series_version = 0.0
        model_version = 0.0
        for target in package.get_node_list(TARGET_PATH):
            try:
                name = target.get_node(NAME_NAME).get_text()
                if name == series:
                    series_version = float(target.get_value(VERSION_NAME))
                if name == model:
                    model_version = float(target.get_value(VERSION_NAME))
            except:
                continue

        if model_version > 0.0:
            return model_version
        return series_version

    def __try_delete_package_entry(self, packagelist, packageid):
        """Delete entry which packageid is "packageid" from "packagelist"."""

        for idx, package in enumerate(packagelist.get_node_list(PACKAGE_NAME)):
            if self.__get_packageid(package) == packageid:
                packagelist.delete_child(PACKAGE_NAME, index=idx)

    def __get_local_released_packagelist(self):
        """Try merge the local released packagelist to the downloaded list."""

        for path in self.package_config.get_node_list(LOCAL_PATH_PATH):
            info_path = os.path.join(path.get_text(),
                                os.path.basename(self.__get_info_file_name()))

            if not os.path.exists(info_path):
                continue

            return xmldata.XmlData(info_path).get_root_node()

    def __get_released_packagelist(self):
        """Return the released packagelist."""

        try:
            if self.__is_there_local_package_info():
                packagelist = self.__get_local_released_packagelist()
            else:
                self.__download_package_info_file()
                packagelist = xmldata.XmlData(
                    self.__get_local_info_file_path()).get_root_node()

        except:
            packagelist = xmldata.XmlNode(PACKAGELIST_BASE_XML_STR)

        self.__update_packagelist_for_available(packagelist)

        index = 0
        for package in packagelist.get_node_list(PACKAGE_NAME):
            try:
                if package.get_attr_value(AVAILABLE_ATTR) != TRUE_TEXT:
                    packagelist.delete_child(PACKAGE_NAME, index=index)
                    continue

                index += 1

            except:
                pass

        return packagelist

    def __get_local_icon_path(self, package):
        """Return a local icon path for package."""

        return os.path.join(config.PACKAGES_ICONS_DIR,
                    os.path.basename(package.get_node(ICON_NAME).get_text()))

    def __update_package_icon_path(self, package):
        """update icon path for a package."""

        package.get_node(ICON_NAME).set_text(
                                        self.__get_local_icon_path(package))

    def __get_version(self, package):
        """Return a version for "package"."""

        return package.get_node(VERSION_NAME).get_text()

    def __is_new_package_released(self, package, released_package_list):
        """Check the new version of "package" is released."""

        for released_package in released_package_list:
            if (self.__get_packageid(package) ==
                self.__get_packageid(released_package)):

                if (float(self.__get_version(released_package)) >
                        float(self.__get_version(package))):

                    return True
                else:
                    return False

        return False

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and return the result entry."""

        for tag_name in APPLY_TAG_NAME_LIST:
            try:
                src_node = entry.get().get_node(tag_name)
            except:
                continue

            try:
                dest_node = self.package_config.get_node(tag_name)
            except NotFoundError:
                self.package_config.append_child(src_node)
                continue

            dest_node.update(src_node, override=True)

        response_entry = self.__save_config(config.PACKAGES_CONF_FILE,
                                            self.package_config)

        if response_entry.get_result() != Results.OK:
            entry.set_result(Results.APPLY_ERROR)
        else:
            entry.set_result(Results.OK)

        # event
        util.dispatch_event("packages", "apply", entry.get())
        return entry

    def __list_handler(self, entry):
        """Handle list request and return the result entry."""

        installed_info = self.__get_installed_packages_info()
        released_package_list = \
                self.__get_released_packagelist().get_node_list(PACKAGE_NAME)

        for package in installed_info.get_node_list(PACKAGE_PATH):
            self.__update_package_icon_path(package)
            if self.__is_new_package_released(package, released_package_list):
                package.set_attrs({UPDATE_ATTR: TRUE_TEXT})
            else:
                package.set_attrs({UPDATE_ATTR: FALSE_TEXT})

        entry.set(installed_info)
        entry.set_result(Results.OK)
        return entry

    def __get_server_root_url(self):
        """Return a server root url from configuration and return it."""

        return urlparse.urlunparse(
                (self.package_config.get_node(PROTOCOL_PATH).get_text(),
                    self.package_config.get_node(HOSTNAME_PATH).get_text(),
                    self.package_config.get_node(PATH_PATH).get_text(),
                    "", "", ""))

    def __get_url(self, path):
        """Return a URL for "path"."""

        return os.path.join(self.__get_server_root_url(), path)

    def __download_file(self, url, dest):
        """Download a file specified "url" to dest with a same file name and
           return a local file name.
        """

        if not os.path.exists(dest):
            os.makedirs(dest)

        local_name = os.path.join(dest, os.path.basename(url))
        try:
            proxy = util.get_proxy(url)
        except:
            proxy = None
        util.urlaccess_receive(url, local_name, proxy=proxy)
        return local_name

    def __download_package_info_file(self):
        """Download a package information file."""

        self.__download_file(self.__get_url(self.__get_info_file_name()),
                        config.PACKAGES_DIR)

    def __try_copy_local_package_icons(self, icon_file):
        """Try copying a local package icon."""

        if not os.path.exists(config.PACKAGES_ICONS_DIR):
            os.makedirs(config.PACKAGES_ICONS_DIR)

        for path in self.package_config.get_node_list(LOCAL_PATH_PATH):
            icon_path = os.path.join(path.get_text(), icon_file)

            if not os.path.exists(icon_path):
                continue
            try:
                shutil.copyfile(icon_path,
                        os.path.join(config.PACKAGES_ICONS_DIR, icon_file))
                return
            except:
                pass

        raise NotFoundError(icon_file)

    def __download_package_icon(self, package):
        """Download a package icon."""

        icon_path = package.get_node(ICON_NAME).get_text()
        if not icon_path:
            return

        try:
            self.__try_copy_local_package_icons(os.path.basename(icon_path))
        except:
            self.__download_file(self.__get_url(icon_path),
                            config.PACKAGES_ICONS_DIR)

    def __download_package_icons(self, packagelist):
        """Download a package icons."""

        for package in packagelist.get_node_list(PACKAGE_NAME):
            self.__download_package_icon(package)

    def __update_packagelist_icon_path(self, packagelist):
        """update icon path for each package in packagelist."""

        for package in packagelist.get_node_list(PACKAGE_NAME):
            self.__update_package_icon_path(package)

    def __get_latest_package_info_dict(self, packagelist, series, model,
            firmware_version):
        """Return the dictionary of latest package information."""

        latest_dict = {}
        for package in packagelist.get_node_list(PACKAGE_NAME):
            try:
                packageid = self.__get_packageid(package)
                version = float(self.__get_version(package))
                target_version = self.__get_package_target_version(package,
                                                            series, model)
                if not packageid in latest_dict:
                    latest_dict[packageid] = {}
                    latest_dict[packageid][TARGET_KEY] = target_version
                    if target_version <= firmware_version:
                        latest_dict[packageid][AVAILABLE_KEY] = version
                    else:
                        latest_dict[packageid][AVAILABLE_KEY] = 0.0
                    continue

                if latest_dict[packageid][TARGET_KEY] < target_version:
                    latest_dict[packageid][TARGET_KEY] = target_version

                if (target_version <= firmware_version and
                    latest_dict[packageid][AVAILABLE_KEY] < version):

                    latest_dict[packageid][AVAILABLE_KEY] = version
            except:
                continue
        return latest_dict

    def __is_ignored_package(self, package, latest_info, series, model,
            firmware_version):
        """Return if package should be ignored."""

        version = float(self.__get_version(package))
        target_version = self.__get_package_target_version(package,
                                                series, model)

        if version < latest_info[AVAILABLE_KEY]:
            return True

        if (target_version > firmware_version and
            target_version < latest_info[TARGET_KEY]):
          
            return True

        return False

    def __update_packagelist_for_available(self, packagelist):
        """Update "packagelist" for availability on the current firmware."""

        try:
            (series, model, firmware_version) = util.get_sysinfo(False)

            # Try update if "series" or "model" is empty.
            if not firmware_version:
                return

            firmware_version = float(firmware_version)
        except:
            return

        latest_info_dict = self.__get_latest_package_info_dict(packagelist,
                                            series, model, firmware_version)
        index = 0
        for package in packagelist.get_node_list(PACKAGE_NAME):
            try:
                latest_info = latest_info_dict[self.__get_packageid(package)]
                if self.__is_ignored_package(package, latest_info,
                            series, model, firmware_version):

                    packagelist.delete_child(PACKAGE_NAME, index=index)
                    continue

                index += 1
                if (float(self.__get_version(package)) <=
                    latest_info[AVAILABLE_KEY]):

                    package.set_attrs({AVAILABLE_ATTR: TRUE_TEXT})
                else:
                    package.set_attrs({AVAILABLE_ATTR: FALSE_TEXT})

            except:
                packagelist.delete_child(PACKAGE_NAME, index=index)

    def __is_there_local_package_info(self):
        """Check if there is the local package info file."""

        for path in self.package_config.get_node_list(LOCAL_PATH_PATH):
            info_path = os.path.join(path.get_text(),
                                os.path.basename(self.__get_info_file_name()))

            if not os.path.exists(info_path):
                continue

            try:
                xmldata.XmlData(info_path).get_root_node()
                return True
            except:
                pass

        return False

    def __query_handler(self, entry):
        """Handle query request and return the result entry."""

        try:
            download_fail = False

            is_there_local_package_info = self.__is_there_local_package_info()

            if is_there_local_package_info:
                packagelist = self.__get_local_released_packagelist()
            else:
                try:
                    self.__download_package_info_file()
                    packagelist = xmldata.XmlData(
                            self.__get_local_info_file_path()).get_root_node()
                except:
                    download_fail = True
                    packagelist = xmldata.XmlNode(PACKAGELIST_BASE_XML_STR)

            self.__update_packagelist_for_available(packagelist)
            try:
                self.__download_package_icons(packagelist)
            except:
                pass

            self.__update_packagelist_icon_path(packagelist)
            entry.set(PACKAGES_BASE_XML_STR)
            entry.get().append_child(packagelist)
            if download_fail:
                self.error("download fail for query")
                entry.set(EMPTY_PACKAGE_LIST_XML_STR)
                entry.set_result(Results.DOWNLOAD_ERROR)
            else:
                entry.set_result(Results.OK)
        except:
            self.exception("fail for query")
            entry.set(EMPTY_PACKAGE_LIST_XML_STR)
            entry.set_result(Results.DOWNLOAD_ERROR)

        return entry

    def __try_copy_local_package(self, package_file):
        """Try copying a local package file."""

        if not os.path.exists(config.PACKAGES_TEMP_DIR):
            os.makedirs(config.PACKAGES_TEMP_DIR)

        for path in self.package_config.get_node_list(LOCAL_PATH_PATH):
            package_path = os.path.join(path.get_text(), package_file)

            if not os.path.exists(package_path):
                continue
            try:
                shutil.copyfile(package_path,
                        os.path.join(config.PACKAGES_TEMP_DIR, package_file))
                return
            except:
                pass

        raise NotFoundError(package_file)

    def __prepare_install(self, package):
        """Prepare for installing "package" and return path to package file."""

        try:
            package_file = os.path.join(config.PACKAGES_TEMP_DIR,
                    os.path.basename(package.get_node(FILE_NAME).get_text()))
            self.__try_copy_local_package(os.path.basename(package_file))
        except:
            package_file = self.__download_file(
                            self.__get_url(
                                package.get_node(FILE_NAME).get_text()),
                            config.PACKAGES_TEMP_DIR)

        return package_file

    def __install_package(self, package):
        """Install package."""

        try:
            package_file = self.__prepare_install(package)
        except:
            self.exception("prepare install error")
            raise DownloadError

        entry = request.Entry(ReqTypes.INSTALL,
                    pal_info.PAL_PACKAGES_REQ_BASE_XML[ReqTypes.INSTALL])
        entry.get().get_node(INTERNALNAME_PATH).set_text(
                    package.get_node(INTERNALNAME_NAME).get_text())

        result_entry = self.send_entry_with_file(entry,
                                    pal_info.PAL_PACKAGES_PATH,
                                    package_file)

        os.remove(package_file)

        if result_entry.get_result() != Results.OK:
            raise InstallError

    def __update_installed_packages_file(self, package, uninstalled=False):
        """Update a "installed-packages.xml"."""

        installed_info = self.__get_installed_packages_info()
        installed_list = installed_info.get_node(PACKAGELIST_NAME)

        self.__try_delete_package_entry(installed_list,
                                        self.__get_packageid(package))

        if not uninstalled:
            installed_list.append_child(package)

        self.__save_config(config.INSTALLED_PACKAGES_FILE, installed_info)

    def __install_handler(self, entry):
        """Handle install request and return the result entry."""

        package_list = \
                self.__get_released_packagelist().get_node_list(PACKAGE_NAME)
        release_packages = dict(
            zip([self.__get_packageid(package) for package in package_list],
                package_list))

        for package in entry.get().get_node_list(PACKAGE_PATH):
            packageid = self.__get_packageid(package)
            if not packageid in release_packages:
                continue

            target_package = release_packages[packageid]
            try:
                icon_path = self.__get_local_icon_path(target_package)
                if not os.path.exists(icon_path):
                    self.__download_package_icon(target_package)

                self.__install_package(target_package)
                self.__update_installed_packages_file(target_package)
            except InstallError:
                self.exception("install error")
                entry.set_result(Results.INSTALL_ERROR)
                return entry
            except DownloadError:
                self.exception("download error for install")
                entry.set_result(Results.DOWNLOAD_ERROR)
                return entry

        # event
        dispatch_event('packages', 'install', entry.get().get_clone())

        entry.set_result(Results.OK)
        return entry

    def __uninstall_package(self, package):
        """Uninstall package."""

        entry = request.Entry(ReqTypes.UNINSTALL,
                    pal_info.PAL_PACKAGES_REQ_BASE_XML[ReqTypes.UNINSTALL])
        entry.get().get_node(INTERNALNAME_PATH).set_text(
                    package.get_node(INTERNALNAME_NAME).get_text())

        result_entry = self.send_entry(entry, pal_info.PAL_PACKAGES_PATH)
        if result_entry.get_result() != Results.OK:
            raise UninstallError

    def __uninstall_handler(self, entry):
        """Handle uninstall request and return the result entry."""

        installed_info = self.__get_installed_packages_info()

        package_list = installed_info.get_node_list(PACKAGE_PATH)
        installed_packages = dict(zip([self.__get_packageid(package)
                                        for package in package_list],
                                    package_list))

        for package in entry.get().get_node_list(PACKAGE_PATH):
            packageid = self.__get_packageid(package)
            if not packageid in installed_packages:
                continue

            target_package = installed_packages[packageid]
            try:
                self.__uninstall_package(target_package)
                self.__update_installed_packages_file(target_package, True)
            except UninstallError:
                self.exception("uninstall error")
                entry.set_result(Results.UNINSTALL_ERROR)
                return entry

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.PACKAGES_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.LIST, self.__list_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.INSTALL, self.__install_handler)
        self.set_handler(ReqTypes.UNINSTALL, self.__uninstall_handler)

        try:
            self.package_config = self.__load_config(config.PACKAGES_CONF_FILE)
        except:
            self.exception("load configuration fail!!\n")
            sys.exit(nascmd.INTERNAL_ERROR)


#
# Main
#

if __name__ == "__main__":
    FalPackages(option=falcmd.FalCmdOption()).handle_request()
