#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting Ports"""

from __future__ import with_statement

import os

import ionas.request as request
import libfal.falcmd as falcmd
import libfal.util as util

import pal_info
from fal_info import ReqTypes, Results
import fal_config as config
from iobase.exception import NotFoundError, InvalidArgError
from iobase.xmldata import XmlNode
from libfal.util import dispatch_event

MIRRORING_PORT_MODE = "mirroring"
VFSEXT_PORT_MODE = "vfsext"
DISK_STATUS = "status"
DISK_FAIL_STATUS = "fail"

UNMOUNT_XML_STR = \
    '<volctrl><volume><dst>%s</dst></volume></volctrl>'

MOUNT_UNMOUNT_EVENT_BODY_BASE_XML = \
    '<volume>' + \
    ' <name>%s</name>' + \
    '</volume>'

VOLUME_CACHE_FILE_TMPL = \
    os.path.join(config.CONF_TEMP_DIR, "volume", "%s", "%s")


class FalPorts(falcmd.FalCmd):
    """FAL Ports command implementation class."""

    class __UnmountFail(Exception):
        "Unmount Fail"
        pass

    def _load_config(self):
        return self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                config.PORTS_CONF_FILE).get()  # portconfig_node

    def _save_config(self, portconfig_node):
        return self.send_entry_with_file(
                request.Entry(ReqTypes.SAVE, portconfig_node),
                pal_info.PAL_CONFFILE_PATH,
                config.PORTS_CONF_FILE)

    def __vol_name2alt_mount_point(self, vol_name):
        """Convert vol name to alternate mount point name."""

        return config.VOL_INFO[vol_name]["alt_mnt_point"]

    def __vol_name2mount_point(self, vol_name):
        """Convert vol name to mount point name."""

        return config.VOL_INFO[vol_name]["mnt_point"]

    def __vol_name2dev_name(self, vol_name):
        """Convert vol name to partition name."""

        return config.VOL_INFO[vol_name]["dev"]

    def __send_request(self, target_module, type, child_xml=""):
        """Send request to pal module(single entry)"""

        entry = request.Entry(type, child_xml)
        return self.send_entry(entry, target_module)

    def __dispatch_unmount_event(self, vol_name, is_previous=False):

        volume_node = XmlNode(MOUNT_UNMOUNT_EVENT_BODY_BASE_XML % vol_name)
        dispatch_event('volume', 'unmount', volume_node, is_previous)
        return

    def __dispatch_raid_update_event(self):
        try:
            dispatch_event('raid', 'update', "raidctrl")
        except:
            self.exception("fail to __dispatch_raid_update_event")
        return

    def __do_bind_unmount(self, vol_name):

        alt_mnt_point = self.__vol_name2alt_mount_point(vol_name)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.UNMOUNT,
                                           UNMOUNT_XML_STR % (alt_mnt_point))
        if Results.OK != result_entry.get_result():
            raise self.__UnmountFail

        return

    def __do_unmount(self, vol_name):

        mnt_point = self.__vol_name2mount_point(vol_name)
        req_xml = UNMOUNT_XML_STR % (mnt_point)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.UNMOUNT,
                                           req_xml)
        if Results.OK != result_entry.get_result():
            raise self.__UnmountFail
        return

    def __get_raid_add_result(self, vol_name, port_mode):

        MIRRORING_STATUS = "mirroring"
        CAPSHORTAGE_STATUS = "capshortage"
        WAITING_STATUS = "waiting"
        VFSEXT_STATUS = "vfsext"
        OTHERMIRROR_STATUS = "othermirror"
        RECOVERABLEMIRROR_STATUS = "recoverablemirror"

        file_path = \
            os.path.join(VOLUME_CACHE_FILE_TMPL % (vol_name, "status"))

        if not os.path.isfile(file_path):
            return Results.APPLY_ERROR

        status = ""
        with open(file_path, "r") as fh:
            status = fh.read().rstrip()

        if status in [OTHERMIRROR_STATUS, RECOVERABLEMIRROR_STATUS]:
            return Results.WAITING

        disk_name = self.__vol_name2dev_name(vol_name)
        disk_status = util.get_disk_info(disk_name, DISK_STATUS)

        if DISK_FAIL_STATUS == disk_status:
            return Results.APPLY_ERROR

        if MIRRORING_PORT_MODE == port_mode:
            if MIRRORING_STATUS == status:
                return Results.OK

        if VFSEXT_PORT_MODE == port_mode:
            if VFSEXT_STATUS == status:
                return Results.OK

        if CAPSHORTAGE_STATUS == status:
            return Results.CAP_SHORTAGE

        if WAITING_STATUS == status:
            return Results.APPLY_ERROR

        return Results.APPLY_ERROR

    def __add_volume_to_raid(self, vol_name, port_mode):
        """Dispatch hdd plug event."""

        plug_event_xml = \
            '<device>' + \
            ' <option force="true"/>' + \
            ' <name>%s</name>' + \
            '</device>'

        if not vol_name.startswith("usb"):
            raise InvalidArgError("Specified I/F is wrong!!")

        try:
            self.__dispatch_unmount_event(vol_name, True)
        except:
            self.exception("Fail to pre unmount event(%s)" % vol_name)
            pass

        try:
            self.__do_bind_unmount(vol_name)
        except self.__UnmountFail:
            pass

        try:
            self.__do_unmount(vol_name)
        except self.__UnmountFail:
            pass

        try:
            self.__dispatch_unmount_event(vol_name, False)
        except:
            self.exception("Fail to post unmount event(%s)" % vol_name)
            pass

        array_num = util.get_cuurent_array_num()

        if vol_name in config.HDD_DEV_LIST_IN_VOL[array_num].keys():

            hdd_name_list = config.HDD_DEV_LIST_IN_VOL[array_num][vol_name]

            if len(hdd_name_list) > 1:
                raise InvalidArgError("Specified I/F or PORT is wrong!!")

            added_hdd_name = hdd_name_list[0]
            if util.is_device_exist(added_hdd_name):
                util.set_volume_cache_value(vol_name,
                                            util.STATUS_VOL_CACHE,
                                            util.WAITING_STATUS)
                device_node = XmlNode(plug_event_xml % added_hdd_name)
                try:
                    util.dispatch_event('usbd', 'plug', device_node)
                except:
                    pass

                return self.__get_raid_add_result(vol_name, port_mode)

        return Results.DEVICE_NOT_FOUND

    def __remove_vfsext_volume(self, vol_name):

        # convert part name
        part_base = config.VOL_INFO[vol_name]["part_base"]

        if not part_base:
            return

        for element in config.EXT_RAID_DEV_INFO.keys():
            if element.startswith(part_base):
                break
        else:
            return

        # check exist
        vfsext_element_list = util.get_current_vfs_element_list()
        if not element in vfsext_element_list:
            return

        # remove
        util.remove_vfs_partition(element)
        return

    def __is_need_vfsext_remove(self, is_enable, old_func, func):

        if is_enable:
            if VFSEXT_PORT_MODE == old_func and func != VFSEXT_PORT_MODE:
                return True

        if not is_enable:
            if VFSEXT_PORT_MODE == func:
                return True

        return False

    def __get_supported_mode_list(self, devtype, portnum):
        try:
            return config.PORT_TO_MODE_LIST[devtype][portnum]
        except:
            return ['share', 'quickcopy', 'mirroring', 'vfsext']

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):

        entry_ports_node = entry.get()
        config_ports_node = self._load_config()
        result_code = Results.OK
        exclusive = getattr(config, 'IS_PORT_MODE_EXCLUSIVE', True)

        for type_ in ('sata', 'usb'):
            for entry_type_node in \
                    entry_ports_node.get_node_list(
                            '%(type)slist/%(type)s' % {'type': type_}):

                # リクエストされた情報を取得
                port = entry_type_node.get_value('@port', None)
                if port is None:
                    raise InvalidArgError("Port is empty!!")
                try:
                    port = int(port)
                except ValueError:
                    raise InvalidArgError("Port is not integer!!")

                # 対象ポートを設定ファイルから探す
                try:
                    config_type_node = config_ports_node.get_node(
                            '%(type)slist/%(type)s[@port="%(number)s"]' % {
                                'type': type_,
                                'number': port})
                except NotFoundError:
                    # FIXME 無ければどうする? 作る? -> 取り敢えずエラー
                    raise InvalidArgError(
                            '%(type)s port %(number)s is not found!!' % {
                                'type': type_,
                                'number': port})

                # 対象ポートのノードにリクエストされたmode設定を反映する
                config_modelist_node = config_type_node.get_node('modelist')

                _mode = None
                for mode in self.__get_supported_mode_list(type_, port):

                    config_mode_node, entry_mode_node = None, None
                    try:
                        config_mode_node = config_modelist_node.get_node(
                                'mode[text()="%s"]' % mode)
                    except:
                        config_mode_node = None
                    try:
                        entry_mode_node = entry_type_node.get_node(
                                'modelist/mode[text()="%s"]' % mode)
                    except:
                        entry_mode_node = None

                    # この type_, port, mode での設定をまとめる
                    enable, share = False, None  # コードの見た目のため
                    if entry_mode_node is not None:
                        enable = entry_mode_node.is_set_to_true('enable')
                        share = entry_mode_node.get_value('@share', None)
                    elif config_mode_node is not None:
                        enable = config_mode_node.is_set_to_true('enable')
                        share = config_mode_node.get_value('@share', None)
                    else:
                        enable = False
                        share = None

                    # モードが排他設定時のための処理
                    if exclusive and _mode:
                        enable = False
                    if enable and _mode is None:
                        _mode = mode

                    if config_mode_node is None:
                        config_modelist_node.append_child('mode', text=mode)
                        config_mode_node = config_modelist_node.get_node(
                                'mode[text()="%s"]' % mode)
                    config_mode_node.set_bool_attr('enable', enable)
                    if mode == 'quickcopy':
                        config_mode_node.set_attrs({'share': share})

                    # ミラーか拡張ボリュームの場合はRAIDに追加
                    if enable:
                        if mode == MIRRORING_PORT_MODE \
                               or mode == VFSEXT_PORT_MODE:

                            self._save_config(config_ports_node)

                            tmp_result_code = \
                                self.__add_volume_to_raid(type_ + str(port),
                                                          mode)
                            if Results.OK != tmp_result_code:
                                result_code = tmp_result_code

                    # 拡張ボリュームモードから別のモードに変更した場合
#                    if self.__is_need_vfsext_remove(enable, old_func, func):
                    if mode == VFSEXT_PORT_MODE and not enable:

                        try:
                            self.__remove_vfsext_volume(type_ + str(port))
                        except:
                            result_code = Results.DELETE_ERROR

                # volumeのステータスファイルをクリア
                util.set_volume_cache_value(type_ + str(port),
                                            util.STATUS_VOL_CACHE,
                                             "")

        # 変更を設定ファイルに保存する
        self._save_config(config_ports_node)

        # event
        try:
            util.dispatch_event("ports", "apply", entry_ports_node)
        except:
            self.exception("fail to dispatch port-apply")
            pass

        self.__dispatch_raid_update_event()

        entry.set_result(result_code)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                pal_info.PAL_CONFFILE_PATH,
                                config.PORTS_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
#
# Main
#

if __name__ == "__main__":
    FalPorts(option=falcmd.FalCmdOption()).handle_request()
