#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Power Settings."""


import os

import ionas.request as request
import iobase.xmldata as xmldata
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info
from fal_info import ReqTypes
from fal_info import Results
from iobase.exception import NotFoundError
import fal_config as config


class FalPower(falcmd.FalCmd):
    """FAL Power command implementation class."""

    def __get_power_config(self):
        """Load power config and return XML data ."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.POWER_CONF_FILE).get()

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        pal_result = self.send_entry(entry, pal_info.PAL_POWER_PATH)
        if pal_result.get_result() != Results.OK:
            return pal_result

        power_config = self.__get_power_config()
        optlist = entry.get().get_node_list(xmldata.ANY_CHILD_NAME)
        for opt in optlist:
            try:
                power_config.delete_child(opt.get_node_name())
            except NotFoundError:
                pass
            power_config.append_child(opt)
        save_entry = request.Entry(ReqTypes.SAVE, power_config)
        self.send_entry_with_file(save_entry,
                                  pal_info.PAL_CONFFILE_PATH,
                                  config.POWER_CONF_FILE)
        return pal_result

    def __shutdown_and_reboot_handler(self, entry):
        """Handle shutdown and reboot request and set entry to result."""

        pal_result = self.send_entry(entry, pal_info.PAL_POWER_PATH)
        return pal_result

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.POWER_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.SHUTDOWN, self.__shutdown_and_reboot_handler)
        self.set_handler(ReqTypes.REBOOT, self.__shutdown_and_reboot_handler)


#
# Main
#

if __name__ == "__main__":
    FalPower(option=falcmd.FalCmdOption()).handle_request()
