#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting quota"""

import os

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
from iobase.exception import (InvalidArgError, NotFoundError)

import pal_info
from fal_info import ReqTypes
from fal_info import Results
import fal_config as config

QUOTA_NAME = "quota"

VOLUMELIST_NAME = "volumelist"
DELETE_VOLUME_XPATH = 'volume[@result="false"]'
VOLUME_LIST_PATH = "volumelist/volume"
USERLIST_PATH = "userlist"
GROUPLIST_PATH = "grouplist"

RESULT_ATTR = "result"

INVALID_ROOT_TAG_MSG = "Invalid tag name!!"


class FalQuota(falcmd.FalCmd):
    """FAL Quota command implementation class."""

    def __check_root_tag(self, entry):
        """Check root tag."""

        root_name = entry.get().get_node_name()

        if root_name != QUOTA_NAME:
            raise InvalidArgError(INVALID_ROOT_TAG_MSG)

    def __get_save_node(self, node):

        try:
            node.delete_child(USERLIST_PATH)
        except NotFoundError:
            pass

        try:
            node.delete_child(GROUPLIST_PATH)
        except NotFoundError:
            pass

        for volume_node in node.get_node_list(VOLUME_LIST_PATH):
            volume_node.delete_attrs([RESULT_ATTR])
        return node

    def __is_exec_save(self, entry):

        VOLUME_ENABLE_PATH = "volumelist/volume/@enable"
        if entry.get().get_value(VOLUME_ENABLE_PATH):
            return True
        return False

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        self.__check_root_tag(entry)

        apply_result_entry = self.send_entry(entry, pal_info.PAL_QUOTA_PATH)
        if apply_result_entry.get_result() != Results.OK:
            return apply_result_entry

        if not self.__is_exec_save(apply_result_entry):
            return apply_result_entry

        save_entry = request.Entry(ReqTypes.SAVE,
            self.__get_save_node(entry.get().get_clone()))

        save_result_entry = self.send_entry_with_file(
                save_entry,
                pal_info.PAL_CONFFILE_PATH,
                config.QUOTA_CONF_FILE)

        apply_result_entry.set_result(save_result_entry.get_result())

        return apply_result_entry

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        self.__check_root_tag(entry)

        return self.send_entry(entry, pal_info.PAL_QUOTA_PATH)

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.QUOTA_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)

#
# Main
#

if __name__ == "__main__":
    FalQuota(option=falcmd.FalCmdOption()).handle_request()
