#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling raid function"""

import os
import re
import datetime
import time
import copy
import ionas.request as request
import libfal.falcmd as falcmd
import libfal.util as util
import pal_info
import fal_config as config
import iobase.fileut as fileut
from fal_info import (ReqTypes, Results)
from iobase.xmldata import XmlNode
from libfal.util import dispatch_event
from iobase.exception import (NotFoundError, InvalidArgError)
from fal_config import (DECODED_LOCK_KEY_FILE_PATH,
                        INTERNAL_VOL, INTERNAL_DEV, SINGLE_RAID_LEVEL,
                        MIRROR_DEV, DEV_TO_ENC_DEV)

### OPTION ###
OPERATION_OPT = "operation"
SCHEDULE_OPT_VAL = "schedule"
ENC_STOP_OPT_VAL = "encstop"
TYPE_OPT_VAL = "type"

###  XML Path ###
RAIDCTRL_NAME = "raidctrl"
RAID_NAME = "raid"
RAIDDEV_NAME = "raiddev"
NAME_NAME = "name"
DISK_NAME = "disk"
PARTITIONLIST_NAME = "partitionlist"
PARTITION_NAME = "partition"
UUID_NAME = "uuid"
EVENTS_NAME = "events"
UPDATETIME_NAME = "updatetime"
NUMBER_NAME = "number"
OPTION_NAME = "option"
ACTIVEREPAIR_NAME = "activerepair"
VOLUME_NAME = "volume"
STATUS_NAME = "status"
LEVEL_NAME = "level"
SIZE_NAME = "size"
PROGRESS_NAME = "progress"
START_NAME = "start"
END_NAME = "end"
MODELIST_NAME = "modelist"
MODE_NAME = "mode"
ELEMENTLIST_NAME = "elementlist"
ELEMENT_NAME = "element"

### XML Attribute ###
NAME_ATTR = "name"
ELEMENT_ATTR = "element"
MISSING_ATTR = "missing"
TYPE_ATTR = "type"
FORCE_ATTR = "force"
AUTOREBUILD_ATTR = "autorebuild"
DEPTH_ATTR = "depth"
JOIN_ATTR = "join"
CRYPT_ATTR_VAL = "crypt"
PTABLE_ATTR = "ptable"
RECOVERY_ATTR = "recovery"
REDUDANCY_ATTR = "redudancy"
CRASH_ATTR_VAL = "crash"
GET_OPERATION_ATTR_VAL = "getoperation"
USER_ATTR = "user"
RAID_ATTR_VAL = "raid"
AUTO_EXTEND_ATTR = "autoextend"
CHANGE_MODE_ATTR_VAL = "changemode"
ADD_DISK_ATTR_VAL = "adddisk"
OTHER_MIRROR_ATTR_VAL = "othermirror"

### XML Data ###
ROOT_NODE_XML_STR = "<raid/>"
PARTITION_LIST_XML_STR = "<raid><partitionlist/></raid>"
RAIDDEV_NAME = "raiddev"
RAID_NAME_XML = '<raid><name type="raid">%s</name></raid>'
UNMOUNT_XML_STR = \
    '<volctrl><volume><dst>%s</dst></volume></volctrl>'
CHK_EXIST_REQ_XML = '<devctrl><device><name>%s</name></device></devctrl>'

MKFS_VFSADD_REQ_XML = \
    '<volctrl>' + \
    ' <option vfsadd="true"/>' + \
    ' <volume encrypt="%s">' + \
    '  <name type="partition">%s</name>' + \
    '  <filesystem>xfs</filesystem>' + \
    ' </volume>' + \
    '</volctrl>'

MOUNT_UNMOUNT_EVENT_BODY_BASE_XML = \
    '<volume>' + \
    ' <name>%s</name>' + \
    '</volume>'

MOUNT_MOUNT_EVENT_BODY_BASE_XML = \
    '<volume>' + \
    ' <name>%s</name>' + \
    '</volume>'

SATAD_PLUG_EVENT_BODY_BASE_XML = \
    '<device>' + \
    ' <name>%s</name>' + \
    '</device>'

MKFS_NONNRAID_REQ_XML = \
    '<volctrl>' + \
     '<option mknonraid="true" externalvfsext="%(external)s"/>' + \
     '<src><disk><name>%(src)s</name></disk></src>' + \
     '<dst><disk><name>%(dst)s</name></disk></dst>' + \
    '</volctrl>'

REBUILD_START_EVENT_BODY_XML_TMPL = \
    '<progress state="start">' + \
    ' <name>%s</name>' + \
    '</progress>'

RAID_START_EVENT_BODY_XML_TMPL = \
    '<raid><name>%s</name></raid>'

RAID_CREATE_EVENT_BODY_XML_TMPL = \
    '<raid><name>%s</name></raid>'

VFS_QUERY_XML = '<volctrl>' + \
                ' <volume>' + \
                '  <filesystem>vfsext</filesystem>' + \
                ' </volume>' + \
                '</volctrl>'

WRITE_SIG_REQ_XML_TMPL = \
    '<volctrl>' + \
    '  <option operation="writeencsig" clear="false"/>' + \
    '  <volume>' + \
    '   <name type="disk">%s</name>' + \
    '  </volume>' + \
    '</volctrl>'

CLEAR_SIG_REQ_XML_TMPL = \
    '<volctrl>' + \
    '  <option operation="writeencsig" clear="true"/>' + \
    '  <volume>' + \
    '   <name type="disk">%s</name>' + \
    '  </volume>' + \
    '</volctrl>'

START_RESULT_XML_TMPL = \
    '<result>' + \
    '  <extend>' + \
    '   <level>%(level)s</level>' + \
    '   <status>%(status)s</status>' + \
    '  </extend>' + \
    '</result>'


### XML PATH ###
START_XPATH_TMPL = "./disk/partitionlist/partition[number='%d']/start"
END_XPATH_TMPL = "./disk/partitionlist/partition[number='%d']/end"

### Some Value ###
MISSING_DEVICE = "missing"
TYPE_KEY = "type"
NODE_KEY = "node"
DATA_PARTITION_NAME = "sataraid1"
VFSEXT_FSTYPE = "vfsext"
FORCE_START_DONE = "force_start_done"

# volume status
WAITING_STATUS = "waiting"
MIRRORING_STATUS = "mirroring"
MOUNTED_STATUS = "mounted"
CAPSHORTAGE_STATUS = "capshortage"
OTHER_MIRROR_STATUS = "othermirror"
RECOVERABLE_MIRROR_STATUS = "recoverablemirror"

# raid status
RUNNING_STATUS = "running"
CRASH_STATUS = "crash"
STOP_STATUS = "stop"
FAIL_STATUS = "fail"
DEGRADE_STATUS = "degrade"

# disk status
DISK_STATUS = "status"
DISK_NORMAL_STATUS = "normal"
DISK_CAPSHORTAGE_STATUS = "capshortage"
DISK_UNPLUGGED_STATUS = "unplugged"
DISK_FAIL_STATUS = "fail"
DISK_SMARTERR_STATUS = "smarterror"
DISK_FAIL_PENDING_STATUS = "failpending"
DISK_SMARTERR_PENDING_STATUS = "smarterrorpending"
PENDINF_STATUS_MAP = \
    {DISK_FAIL_STATUS: DISK_FAIL_PENDING_STATUS,
     DISK_SMARTERR_STATUS: DISK_SMARTERR_PENDING_STATUS}

# extend status
EXTEND_SUCCESS = "success"
EXTEND_FAIL = "fail"
EXTEND_NOEXEC = "noexec"

USER_REMOVE = "user_remove"

MIRROR_RAID_SYMBL_DEV = config.VOL_INFO[INTERNAL_VOL]["dev"]
MIRROR_USB_VOLUME = config.MIRRORING_VOL_LIST[0]
MIRROR_USB_DISK = config.VOL_INFO[MIRROR_USB_VOLUME]["dev"]

SATA_DISK_BASE_NAME = "satad"

SWAP_PART_SUFFIX = "p3"

MBR_PART_START = 2048
GPT_PART_START = 40
END_SECTOR = "-1"

SHARE_PORT_MODE = "share"
MIRRORING_PORT_MODE = "mirroring"
VFSEXT_PORT_MODE = "vfsext"

TRUE_TEXT = "true"
FALSE_TEXT = "false"

RAID_EXTEND_LOCK_NAME = "raidextend"

DISPACH_OK = 0
DISPACH_FAIL = 1


class FalRaid(falcmd.FalCmd):
    """FAL Raid command implementation class."""

    class __CreateError(Exception):
        """Create Error"""
        pass

    class __StartError(Exception):
        """Start Error"""
        pass

    class __StopError(Exception):
        """Stop Error"""
        pass

    class __GetError(Exception):
        """Get Error"""
        pass

    class __DelpartError(Exception):
        """DelPart Error"""
        pass

    class __MkpartError(Exception):
        """Mkpart Error"""
        pass

    class __UnmountFail(Exception):
        """Unmount Fail"""
        pass

    class __GetElementInfoError(Exception):
        """Get element info Error"""
        pass

    class __RemoveError(Exception):
        """Remove Error"""
        pass

    class __ActiverepairError(Exception):
        """Activerepair Error"""
        pass

    class __MkfsError(Exception):
        """Mkfs Error"""
        pass

    class __AlreadyExist(Exception):
        """Already Exist"""
        pass

    class __AddError(Exception):
        """Add Error"""
        pass

    class __Capshortage(Exception):
        """Capshortage"""
        pass

    class __DeviceNotFound(Exception):
        """DeviceNotFound"""
        pass

    class __FileNotFound(Exception):
        """FileNotFound"""
        pass

    class __RequestError(Exception):
        """RequestError"""
        pass

    class __ErrorDiskDetected(Exception):
        """Error Disk Detected"""
        pass

    class __WaitingDiskDetected(Exception):
        """Waiting mirroring disk detected"""
        pass

    class __AlreadyStop(Exception):
        """Already Stop"""
        pass

    class __NonSupportAutoExtend(Exception):
        """NonSupportAutoExtend"""
        pass

    class __DoneNothing(Exception):
        """DoneNothing"""
        pass

    class __ExtendError(Exception):
        """ExtendError"""
        pass
    
    class __CannotChange(Exception):
        """Cannot Change"""
        pass
    

    def __vol_name2raiddev_name(self, vol_name):
        """Convert volume name to partition name."""

        return config.VOL_INFO[vol_name]["dev"]

    def __vol_name2enc_part_name(self, vol_name):

        return DEV_TO_ENC_DEV[config.VOL_INFO[vol_name]["dev"]]

    def __vol_name2dev_name(self, vol_name):
        """Convert vol name to partition name."""

        return config.VOL_INFO[vol_name]["dev"]

    def __raiddev2elements(self, raiddev_name, raid_mode=None,
                           is_need_missing=False,
                           is_need_mirror_info=False):
        """Convert RAID device to element partition list."""

        element_list = []

        if raid_mode is None:
            raid_mode = util.get_current_raid_mode()

        try:
            element_list = \
                list(util.get_raid_element_info(raid_mode)[raiddev_name])
        except:
            self.exception("fail to __raiddev2elements(%s) " % raiddev_name)

        array_num = config.RAID_MODE_INFO[raid_mode]["array_num"]
        is_ext_raid = config.RAID_MODE_INFO[raid_mode]["is_ext_raid"]

        # ポートモードがvfsextでないUSBデバイスはremove
        if is_need_mirror_info:
            if  is_ext_raid and \
                   raiddev_name in config.EXT_RAID_TO_ELEMENT[array_num].keys():

                external_disk_list = \
                    [config.VOL_INFO[vol]["dev"] for vol in \
                        self.__get_mode_setting_vol_list(VFSEXT_PORT_MODE)]

                tmp_element_list = list(element_list)
                for element in tmp_element_list:
                    if element.startswith("usb"):
                        disk_name = self.__partition2disk(element)
                        if disk_name not in external_disk_list:
                            element_list.remove(element)

            # ポートモードがmirroringでないUSBデバイスはremove
            else:
                mirroring_disk_list = \
                    [config.VOL_INFO[vol]["dev"] for vol in \
                        self.__get_mode_setting_vol_list(MIRRORING_PORT_MODE)]

                tmp_element_list = list(element_list)
                for element in tmp_element_list:
                    if element.startswith("usb"):
                        disk_name = self.__partition2disk(element)
                        if disk_name not in mirroring_disk_list:
                            element_list.remove(element)

        if not is_need_missing:
            while MISSING_DEVICE in element_list:
                element_list.remove(MISSING_DEVICE)

        return element_list

    def __partition2disk(self, part_name):
        """Convert partition name to corresponding disk name."""

        match = re.match("([a-zA-Z]+[0-9]+)", part_name)

        if not match:
            raise NotFoundError

        disk_name = match.group(1)

        if disk_name not in config.SATA_HDD_DEV_LIST + config.USB_HDD_DEV_LIST:
            raise NotFoundError

        return disk_name

    def __disk_name2vfs_element_name(self, disk_name):

        for element_name in util.get_vfsext_dev_info().keys():

            if element_name.startswith(disk_name):
                return element_name

        raise NotFoundError

    def __vol_name2mount_point(self, vol_name):
        """Convert vol name to mount point name."""

        return config.VOL_INFO[vol_name]["mnt_point"]

    def __vol_name2alt_mount_point(self, vol_name):
        """Convert vol name to alternate mount point name."""

        return config.VOL_INFO[vol_name]["alt_mnt_point"]

    def __dev_name2vol_name(self, dev_name):

        for vol_name, vol_info in config.VOL_INFO.iteritems():

            if dev_name.startswith(vol_info["dev"]):
                return vol_name

        return ""

    def __convert_to_appropriate_raid_level(self, raid_level):
        """If specified raid level is not in supported raid level
           convert appropriate raid level.
        """

        def level2real_level(level):
            match = re.match("([a-zA-Z0-9]+)", raid_level)
            if match:
                return match.group(1)
            return None

        level_list_map = config.VOL_INFO[INTERNAL_VOL]["mode_list"]
        for level_list in level_list_map.values():
            if raid_level in level_list:
                return raid_level

        real_raid_level = level2real_level(raid_level)
        if not real_raid_level:
            return raid_level

        max_array_num = util.get_max_array_num()
        array_num_list = sorted(level_list_map.keys())
        for array_num in array_num_list:
            if max_array_num < array_num:
                continue
            for raid_level in level_list_map[array_num]:
                if real_raid_level == level2real_level(raid_level):
                    return raid_level

        return None

    def __is_device_exist(self, dev_name):
        """Check wheather specified device exists."""

        result_entry = self.__send_request(pal_info.PAL_DEVCTRL_PATH,
                                           ReqTypes.CHK_EXIST,
                                           CHK_EXIST_REQ_XML % dev_name)
        if Results.OK == result_entry.get_result():
            return True

        return False

    def __is_partition_exist_in_raid(self, raid_name, part_name):

        CHK_PART_EXIST_REQ_XML = \
            """<raidctrl>
                <raid><name>%s</name></raid>
                <partition><name>%s</name></partition>
               </raidctrl>"""

        result_entry = \
            self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                ReqTypes.CHK_EXIST,
                                CHK_PART_EXIST_REQ_XML % \
                                    (raid_name, part_name))
        if Results.OK != result_entry.get_result():
            return False

        return True

    def __is_disk_exist_in_raid(self, disk_name):

        CHK_DISK_EXIST_REQ_XML = \
            """<raidctrl>
                <disk><name>%s</name></disk>
               </raidctrl>"""

        result_entry = \
            self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                ReqTypes.CHK_EXIST,
                                CHK_DISK_EXIST_REQ_XML % disk_name)
        if Results.OK != result_entry.get_result():
            return False

        return True

    def __is_raid_device(self, dev_name):
        """Check wheather specified device is raid device"""

        if dev_name in config.RAID_DEV_LIST:
            return True

        return False

    def __is_exist_node(self, node, node_path):
        """Check whether node_path exists in specified node"""

        try:
            node.get_node(node_path)
        except NotFoundError:
            return False
        return True

    def __is_exist_attr(self, node, attr_val):
        """Check whether node_path exists in specified node"""

        try:
            node.get_attr_value(attr_val)
        except NotFoundError:
            return False
        return True

    def __is_able_to_change_mode(self, from_level, to_level):

        from sets import Set

        from_array_num = config.RAID_MODE_INFO[from_level]["array_num"]
        to_array_num = config.RAID_MODE_INFO[to_level]["array_num"]

        from_disk_list = config.HDD_DEV_LIST_IN_VOL[from_array_num][INTERNAL_VOL]
        to_disk_list = config.HDD_DEV_LIST_IN_VOL[to_array_num][INTERNAL_VOL]

        diff_list = list(Set(to_disk_list) - Set(from_disk_list))

        if len(diff_list) == 0:
            return False

        for disk in diff_list:

            if not disk.startswith("satad"):
                continue

            if DISK_NORMAL_STATUS != util.get_disk_info(disk, DISK_STATUS):
                return False

        return True

    def __save_user_remove_flag(self, disk_name, is_remove=False):

        util.set_disk_info(disk_name,
                           USER_REMOVE,
                           "1" if is_remove else "0")

    def __get_mode_setting_vol_list(self, mode):

        vol_name_list = []
        try:
            for iface, info in config.PORT_TO_MODE_LIST.iteritems():

                if not iface.startswith("usb"):
                    continue

                for port_num in info.keys():
                    try:
                        if mode == util.get_current_port_mode(iface, port_num):
                            vol_name_list.append(iface + str(port_num))
                    except:
                        pass
        except:
            self.exception("fail to make vol_name_list")
            pass

        return vol_name_list

    def __remake_creating_element_list(self, raiddev_name, element_list):

        remake_element_list = element_list[:]
        element_num = len(remake_element_list)
        is_missing = False

        if MISSING_DEVICE not in remake_element_list:
            return is_missing, element_num, remake_element_list

        # ミラー設定用
        for idx, element in enumerate(remake_element_list):

            if MISSING_DEVICE == element:
                is_missing = True
                continue

            if not element.startswith("usbd"):
                continue

            if not self.__is_device_exist(element):
                is_missing = True
                remake_element_list[idx] = MISSING_DEVICE
                continue

            if MIRRORING_STATUS != \
                util.get_volume_cache_value(MIRROR_USB_VOLUME, "status"):
                is_missing = True
                remake_element_list[idx] = MISSING_DEVICE
                continue

        while MISSING_DEVICE in remake_element_list:
            remake_element_list.remove(MISSING_DEVICE)

        return is_missing, element_num, remake_element_list

    def __remake_start_element_list(self, raiddev_name, element_list):

        remake_element_list = element_list[:]

        element_num = len(remake_element_list)
        is_missing = False

        actual_element_info_dict = self.__get_element_info(raiddev_name, True)

        # ミラー設定用
        for idx, element in enumerate(remake_element_list):

            if MISSING_DEVICE == element:
                is_missing = True
                continue

            if not element.startswith("usbd"):
                continue

            # 現在ミラー状態じゃない場合
            if not self.__is_usb_mirroring_setting():
                is_missing = True
                remake_element_list[idx] = MISSING_DEVICE
                continue

            # ミラー設定されてるが、USBミラー状態じゃない場合
            if not actual_element_info_dict[element][JOIN_ATTR]:
                is_missing = True
                remake_element_list[idx] = MISSING_DEVICE
                continue

        while MISSING_DEVICE in remake_element_list:
            remake_element_list.remove(MISSING_DEVICE)

        return is_missing, element_num, remake_element_list

    def __dispatch_pre_unmount_event(self, vol_name):

        volume_node = XmlNode(MOUNT_UNMOUNT_EVENT_BODY_BASE_XML % vol_name)
        dispatch_event('volume', 'unmount', volume_node, True)
        return

    def __dispatch_mount_event(self, vol_name, is_previous=False):

        volume_node = XmlNode(MOUNT_MOUNT_EVENT_BODY_BASE_XML % vol_name)
        dispatch_event('volume', 'mount', volume_node, is_previous)
        return

    def __dispatch_raid_update_event(self):
        try:
            dispatch_event('raid', 'update', "raidctrl")
        except:
            self.exception("fail to __dispatch_raid_update_event")
        return

    def __dispatch_disk_update_event(self):
        try:
            dispatch_event('disk', 'update', "raidctrl")
        except:
            self.exception("fail to __dispatch_disk_update_event")
        return

    def __dispatch_raid_start_event(self, raiddev_name):

        try:
            body_node = \
                XmlNode(RAID_START_EVENT_BODY_XML_TMPL % raiddev_name)
            dispatch_event('raid', 'start', body_node)
        except:
            self.exception("fail to __dispatch_raid_start_event")
        return

    def __dispatch_raid_create_event(self, raiddev_name=INTERNAL_DEV,
            previous=False):

        try:
            body_node = \
                XmlNode(RAID_CREATE_EVENT_BODY_XML_TMPL % raiddev_name)
            dispatch_event('raid', 'create', body_node, previous)
        except:
            self.exception("fail to __dispatch_raid_create_event")
            return DISPACH_FAIL
        return DISPACH_OK

    def __get_real_raiddev_name(self, raiddev_name):
        """Get real raiddev name from devinfo.py"""

        if raiddev_name in config.RAID_TO_REAL_NAME:
            return config.RAID_TO_REAL_NAME[raiddev_name]

        return raiddev_name

    def __get_default_raid_level(self, raiddev_name):
        """Get RAID level
           The level is get from "product_conf".
        """

        return config.RAID_TO_LEVEL[raiddev_name]

    def __is_in_missing(self, element_list):
        """Check wheather RAID element include missing device"""

        if MISSING_DEVICE in element_list:
            return True

        return False

    def __load_config(self, conffile_name=config.RAIDCTRL_CONF_FILE):
        """Load config and return XML data ."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                         pal_info.PAL_CONFFILE_PATH,
                                         conffile_name).get()

    def __is_usb_mirroring_setting(self):

        USB_MIRRORIGN_XPATH_TMPL = \
            "usblist/usb[@port='%s']/modelist" + \
            "/mode[@enable='true' and text()='mirroring']"

        ports_node = self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                               pal_info.PAL_CONFFILE_PATH,
                                               config.PORTS_CONF_FILE).get()
        try:
            ports_node.get_node(USB_MIRRORIGN_XPATH_TMPL %
                                (config.DEV_TO_PORT_NO[MIRROR_USB_DISK],))
        except:
            return False

        return True

    def __save_config(self, conf_node, conffile=config.RAIDCTRL_CONF_FILE):
        """Load config and return XML data ."""

        save_entry = request.Entry(ReqTypes.SAVE, conf_node)
        return self.send_entry_with_file(save_entry,
                                         pal_info.PAL_CONFFILE_PATH,
                                         conffile)

    def __delete_disk_from_system(self, disk_name):

        try:
            util.delete_disk_from_system(disk_name)
        except:
            self.exception("fail to delete_disk_from_system(%s)" % disk_name)
            pass

    def __remove_mirror_disk_from_sys_raid(self):

        if self.__is_usb_mirroring_setting():
            self.__remove_disk_from_all_raid(MIRROR_USB_DISK, is_sys_only=True)

    def __save_config_share_port_mode(self, vol_name_list):

        def remove_mirror_disk_from_sys_raid():

            self.__remove_disk_from_all_raid(MIRROR_USB_DISK,
                                             is_sys_only=True)

        PORTS_USBLIST_XML = "<ports><usblist/></ports>"
        PORT_MODE_XPATH = \
            "./usblist/%(iface)s[@port='%(port)s']/modelist/mode"
        USB_PORT_XPATH = "./usblist/usb[@port='%s']"

        config_ports_node = self.__load_config(config.PORTS_CONF_FILE)

        for vol_name in vol_name_list:

            disk_name = config.VOL_INFO[vol_name]["dev"]
            iface_name = config.DEV_TO_PORT_IF[disk_name]
            port_num = config.DEV_TO_PORT_NO[disk_name]

            xpath = PORT_MODE_XPATH % {"iface": iface_name,
                                       "port": port_num}
            mode_node_list = config_ports_node.get_node_list(xpath)

            # shareモードをenable, 他のモードをdisable
            for mode_node in mode_node_list:

                mode = mode_node.get_text()

                if SHARE_PORT_MODE == mode:
                    mode_node.enable(True)

                elif MIRRORING_PORT_MODE == mode:
                    if not mode_node.is_enabled():
                        continue

                    remove_mirror_disk_from_sys_raid()
                    mode_node.enable(False)

                elif VFSEXT_PORT_MODE == mode:
                    if not mode_node.is_enabled():
                        continue

                    for part_name in config.EXT_RAID_DEV_INFO.keys():
                        if part_name.startswith(
                               config.VOL_INFO[vol_name]["dev"]):
                            try:
                                self.__remove_vfs_partition(part_name)
                            except:
                                self.exception(
                                    "fail to __remove_vfs_partition")

                    mode_node.enable(False)

                else:
                    if mode_node.is_enabled():
                        mode_node.enable(False)

                util.clear_volume_cache_value(vol_name, "status")

        self.__save_config(config_ports_node, config.PORTS_CONF_FILE)

        for vol_name in vol_name_list:
            try:
                disk_name = config.VOL_INFO[vol_name]["dev"]
                iface_name = config.DEV_TO_PORT_IF[disk_name]
                port_num = config.DEV_TO_PORT_NO[disk_name]

                ports_node = XmlNode(PORTS_USBLIST_XML)
                usblist_node = ports_node.get_node("./usblist")
                usblist_node.append_child(
                    config_ports_node.get_node(USB_PORT_XPATH % port_num))
                util.dispatch_event("ports", "apply", ports_node)
            except:
                self.exception("fail to dispatch port-apply")
                pass

    def __is_current_raid_mode_vfsext(self):

        conf_node = self.__load_config()
        level = conf_node.get_value("raidlist/raid/level")
        if VFSEXT_FSTYPE == config.RAID_MODE_TO_REAL_RAID_MODE[level]:
            return True
        return False

    def __is_enough_element_num(self, raid_level, num):

        if util.get_raid_min_num(raid_level) > num:
            return False

        return True

    def __is_internal_volume_encrypted(self):

        INTRNAL_ENCRIPT_XPATH = \
            "./volumelist/volume[name='sataraid1']/encrypt"

        volctrl_conf_node = self.__load_config(config.VOLCTRL_CONF_FILE)
        encrypt_node = volctrl_conf_node.get_node(INTRNAL_ENCRIPT_XPATH)
        if encrypt_node.is_enabled():
            return True
        return False

    def __get_config_raid_level(self):

        return self.__load_config().get_value("raidlist/raid/level")

    def __flat_all_data_partition_size(self):

        def delete_data_partition(disk_name):

            DELPART_REQ_XML = """<diskctrl>
                                  <disk>
                                   <name>%s</name>
                                   <partitionlist>
                                    <partition><number>6</number></partition>
                                   </partitionlist>
                                  </disk>
                                 </diskctrl>"""

            result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                               ReqTypes.DELPART,
                                               DELPART_REQ_XML % disk_name)
            if Results.OK != result_entry.get_result():
                raise self.__RemoveError
            return

        def get_disk_partlist_node(disk_name):

            result_entry = self.__send_query_to_diskctrl(disk_name)
            disk_node = result_entry.get().get_node(DISK_NAME)
            return disk_node.get_node(PARTITIONLIST_NAME)

        def make_data_partition(disk_name,
                                partlist_node,
                                min_end_sector):
            PART6_XPATH = "./partition[number='6']"

            add_partlist_node = XmlNode(PARTITIONLIST_NAME)
            part_node = partlist_node.get_node(PART6_XPATH)
            add_partlist_node.append_child(part_node)

            self.__send_addpart_to_diskctrl(disk_name, add_partlist_node)
            return

        # get_elementでmd2のjoin=trueの全diskをチェック
        # disk part6のサイズをサーチ
        # minを求める
        # minよりでかいpart6を発見したらremove partしてmkpart

        PART6_END_XPATH = "./partition[number='6']/end"
        element_info_dict = self.__get_element_info("md2", True)
        min_end_sector = None
        end_sector_map = {}
        right_partlist_node = None  # あるべきpartitionlistノード
        for element, info in element_info_dict.iteritems():

            if not element.startswith("satad"):
                continue

            disk_name = self.__partition2disk(element)
            if not self.__is_device_exist(disk_name):
                continue

            try:
                partlist_node = get_disk_partlist_node(disk_name)
                end_sector_str = partlist_node.get_value(PART6_END_XPATH)
                end_sector = int(end_sector_str)
                end_sector_map[disk_name] = end_sector
            except:
                continue

            if info[JOIN_ATTR]:

                if not end_sector:
                    continue

                if min_end_sector is None:
                    min_end_sector = end_sector
                    right_partlist_node = partlist_node
                elif min_end_sector > end_sector:
                    min_end_sector = end_sector
                    right_partlist_node = partlist_node

        for disk_name, end_sector in end_sector_map.iteritems():

            if disk_name.startswith("usbd"):
                continue

            if min_end_sector is not None:
                if min_end_sector != end_sector or not end_sector:
                    if end_sector:
                        delete_data_partition(disk_name)
                    make_data_partition(disk_name,
                                        right_partlist_node.get_clone(),
                                        min_end_sector)

    def __do_enc_start_partition(self, part_name, enc_part_name):

        ENC_START_OPT_VAL = "encstart"
        CTRL_ENC_BASE_XML = '<volctrl>' + \
                            ' <volume>' + \
                            '  <name type="partition">%s</name>' + \
                            '  <name type="crypt">%s</name>' + \
                            ' </volume>' + \
                            '</volctrl>'

        ctrl_entry = \
            request.Entry(ReqTypes.CONTROL,
                          CTRL_ENC_BASE_XML % (part_name, enc_part_name))
        # set key file
        if not os.path.isfile(DECODED_LOCK_KEY_FILE_PATH):
            raise self.__FileNotFound
        ctrl_entry.set_file(DECODED_LOCK_KEY_FILE_PATH)

        # set option
        ctrl_entry.set_options({OPERATION_OPT: ENC_START_OPT_VAL})

        # send request
        result_entry = self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)
        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        return

    def __vfs_add_partition(self, part_name, is_enc=False):

        VFS_ADD_BASE_REQ_XML = \
            '<volctrl>' + \
            ' <option vfsadd="true"/>' + \
            ' <volume>' + \
            '  <filesystem encrypt="%(is_enc)s"/>' + \
            '  <partition><name>%(part)s</name></partition>' + \
            ' </volume>' + \
            '</volctrl>'

        req_xml = VFS_ADD_BASE_REQ_XML % {"is_enc": str(is_enc).lower(),
                                          "part": part_name}
        return self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                   ReqTypes.MOUNT,
                                   req_xml)

    def __mkfs_vfs_partition(self, part_name, is_enc=False):

        def wait_mkfs_end():

            FILESYSTEM_CATEGORY = "filesystem"
            FORMATTING_STATE = "formatting"

            while(True):
                time.sleep(10)
                state = util.statemgr_query(FILESYSTEM_CATEGORY)
                if FORMATTING_STATE != state:
                    break

        for idx in range(0, 100):  # 100回LOOPして終わらなければアウト

            req_xml = MKFS_VFSADD_REQ_XML % (str(is_enc).lower(), part_name)
            result_entry = \
                self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                    ReqTypes.MKFS,
                                    req_xml)

            result_code = result_entry.get_result()
            if Results.OK == result_code:
                wait_mkfs_end()

                try:
                    disk_name = self.__partition2disk(part_name)
                    if "1" == util.get_disk_info(disk_name,
                                                 config.VFS_ADD_MKFS_ERR):
                        raise self.__MkfsError
                except:
                    self.exception(
                        "VFS_ADD_MKFS_ERR(%s)" % disk_name)
                    raise self.__MkfsError

                break

            if Results.BUSY_ERROR == result_code:
                time.sleep(10)
                continue

            raise self.__MkfsError

        return

    def __remove_vfs_partition(self, part_name):

        VFS_REMOVE_BASE_REQ_XML = \
            '<volctrl>' + \
            ' <option vfsremove="true"/>' + \
            ' <volume>' + \
            '  <filesystem encrypt="%(is_enc)s"/>' + \
            '  <partition><name>%(part)s</name></partition>' + \
            ' </volume>' + \
            '</volctrl>'

        # vfs専用remount実行
        is_enc = self.__is_internal_volume_encrypted()

        req_xml = VFS_REMOVE_BASE_REQ_XML % {"is_enc": str(is_enc).lower(),
                                             "part": part_name}
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.MOUNT,
                                           req_xml)
        if Results.OK != result_entry.get_result():
            self.error("fail to __remove_vfs_partition : " + part_name)
            raise self.__RemoveError

        # 対象パーティションを普通にunmount
        vfsext_dev_info = util.get_vfsext_dev_info()
        mnt_point = vfsext_dev_info[part_name]["mnt_point"]
        req_xml = UNMOUNT_XML_STR % (mnt_point)
        self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                            ReqTypes.UNMOUNT,
                            req_xml)
        if is_enc:
            enc_part_name = DEV_TO_ENC_DEV[part_name]
            self.__vfs_enc_stop_partition(enc_part_name)

        return

    def __mount_internal_volume(self):

        # マウント
        MOUNT_XML_STR = \
            """<volctrl><volume>
                <name type="partition">%s</name>
                <dst>%s</dst>
               </volume></volctrl>"""

        vol_name = config.STOR_IF_TO_VOL['internal'][1]
        part_name = config.VOL_INFO[vol_name]["dev"]
        mnt_point = config.VOL_INFO[vol_name]["mnt_point"]

        result_entry = \
            self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                ReqTypes.MOUNT,
                                MOUNT_XML_STR % (part_name, mnt_point))
        if Results.OK != result_entry.get_result():
            raise self.__MountFail

        # バインドマウント
        BIND_MOUNT_XML_STR = \
            """<volctrl>
                <option bind="true"/>
                <volume>
                 <src>%s</src>
                 <dst>%s</dst>
                </volume>
               </volctrl>"""

        alt_mnt_point = config.VOL_INFO[vol_name]["alt_mnt_point"]
        result_entry = \
            self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                ReqTypes.MOUNT,
                                BIND_MOUNT_XML_STR % \
                                    (mnt_point, alt_mnt_point))
        if Results.OK != result_entry.get_result():
            raise self.__MountFail

        return

    def __unmount(self, vol_name):

        # VFS以外はここ
        try:
            self.__do_bind_unmount(vol_name)
        except self.__UnmountFail:
            pass

        try:
            self.__do_unmount(vol_name)
        except self.__UnmountFail:
            pass

        # VFSは特別扱い
        if self.__is_current_raid_mode_vfsext():
            try:
                util.vfsext_unmount()
            except:
                pass
            return

        return

    def __do_bind_unmount(self, vol_name):

        alt_mnt_point = self.__vol_name2alt_mount_point(vol_name)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.UNMOUNT,
                                           UNMOUNT_XML_STR % (alt_mnt_point))
        if Results.OK != result_entry.get_result():
            raise self.__UnmountFail

        return

    def __do_unmount(self, vol_name):

        mnt_point = self.__vol_name2mount_point(vol_name)
        req_xml = UNMOUNT_XML_STR % (mnt_point)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.UNMOUNT,
                                           req_xml)
        if Results.OK != result_entry.get_result():
            raise self.__UnmountFail
        return

    def __mdadm_time2epoch(self, mdadm_datetime):

        dt = datetime.datetime.strptime(mdadm_datetime,
                                        "%a %b %d %H:%M:%S %Y")
        return int(time.mktime(dt.timetuple()))

    def __send_request(self, target_module, type, child_xml=""):
        """Send entry to specified target pal module."""

        entry = request.Entry(type, child_xml)
        return self.send_entry(entry, target_module)

    def __send_multiple_request(self, target_module, type_node_list):
        """Send multipule request to specified target pal module.
           This method append entry tags into entrylist tag.
        """

        entrylist = request.EntryList()

        for type_node in type_node_list:
            entrylist.append_entry(request.Entry(type_node[TYPE_KEY],
                                                 type_node[NODE_KEY]))

        return self.send_entrylist(entrylist, target_module)

    def __send_query_to_diskctrl(self, disk_name):
        """Send query request to pal_diskctrl"""

        QUERY_REQ_BASE_XML = \
            '<diskctrl>' + \
            '<option getpartitioninfo="true"/>' + \
            '<disk><name/></disk>' + \
            '</diskctrl>'
        DISK_NAME_XPATH = "./disk/name"

        diskctrl_node = XmlNode(QUERY_REQ_BASE_XML)
        diskctrl_node.get_node(DISK_NAME_XPATH).set_text(disk_name)

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.QUERY,
                                           diskctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__GetError

        return result_entry

    def __send_delpart_to_diskctrl_with_clear(self, disk_name):
        """Send delpart request to pal_diskctrl."""

        DELPART_REQ_BASE_XML = '<diskctrl>' + \
                               '  <option clear="true"/>' + \
                               '  <disk><name/></disk>' + \
                               '</diskctrl>'
        DISK_NAME_XPATH = "./disk/name"

        diskctrl_node = XmlNode(DELPART_REQ_BASE_XML)
        diskctrl_node.get_node(DISK_NAME_XPATH).set_text(disk_name)

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.DELPART,
                                           diskctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__DelpartError

        return

    def __send_addpart_to_diskctrl(self, disk_name, partlist_node):
        """Send addpart request to pal_diskctrl"""

        ADDPART_REQ_BASE_XML = '<diskctrl>' +\
                               '  <disk><name/></disk>' + \
                               '</diskctrl>'
        DISK_NAME_XPATH = "./disk/name"

        diskctrl_node = XmlNode(ADDPART_REQ_BASE_XML)
        diskctrl_node.get_node(DISK_NAME_XPATH).set_text(disk_name)
        diskctrl_node.get_node(DISK_NAME).append_child(partlist_node)

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.ADDPART,
                                           diskctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__MkpartError

        return

    def __send_mkpart_to_diskctrl(self, disk_name, partlist_node):
        """Send mkpart request to pal_diskctrl"""

        MKPART_REQ_BASE_XML = '<diskctrl>' +\
                               '  <disk><name/></disk>' + \
                               '</diskctrl>'
        DISK_NAME_XPATH = "./disk/name"

        diskctrl_node = XmlNode(MKPART_REQ_BASE_XML)
        diskctrl_node.get_node(DISK_NAME_XPATH).set_text(disk_name)
        diskctrl_node.get_node(DISK_NAME).append_child(partlist_node)

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.MKPART,
                                           diskctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__MkpartError

        return

    def __get_element_info(self, raiddev_name, is_check_join=False,
                           is_need_mirror_info=False):
        """Get RAID element info by sending
           get_element_info request to pal_raidctrl.
        """

        result_entry = \
            self.__send_get_element_info_request(raiddev_name, is_check_join)
        if result_entry.get_result() != Results.OK:
            raise self.__GetElementInfoError

        PART_XPATH = "./partitionlist/partition"
        element_dict = {}
        part_node_list = result_entry.get().get_node_list(PART_XPATH)

        for part_node in part_node_list:

            part_name = part_node.get_node(NAME_NAME).get_text()

            if not part_node.is_set_to_true(ELEMENT_ATTR):
                element_dict[part_name] = {}
                if is_check_join:
                    is_join = part_node.is_set_to_true(JOIN_ATTR)
                    element_dict[part_name][JOIN_ATTR] = is_join
                continue

            updatetime_str = part_node.get_node(UPDATETIME_NAME).get_text()
            updatetime = self.__mdadm_time2epoch(updatetime_str)
            uuid = part_node.get_node(UUID_NAME).get_text()
            events = part_node.get_node(EVENTS_NAME).get_text()

            element_dict[part_name] = {UUID_NAME: uuid,
                                       UPDATETIME_NAME: updatetime,
                                       EVENTS_NAME: events}

            if is_check_join:
                is_join = part_node.is_set_to_true(JOIN_ATTR)
                element_dict[part_name][JOIN_ATTR] = is_join

        return element_dict

    def __send_get_element_info_request(self, raiddev_name,
                                        is_check_join=False,
                                        is_need_mirror_info=False):

        partlist_node = XmlNode(PARTITIONLIST_NAME)
        part_name_list = self.__raiddev2elements(raiddev_name,
                             is_need_mirror_info=is_need_mirror_info)
        for part_name in part_name_list:

            part_node = XmlNode(PARTITION_NAME)
            part_node.append_child(NAME_NAME, part_name)
            partlist_node.append_child(part_node)

        raidctrl_node = XmlNode(RAIDCTRL_NAME)
        raidctrl_node.append_child(partlist_node)

        if is_check_join:
            raid_node = XmlNode(RAID_NAME_XML % raiddev_name)
            raidctrl_node.append_child(raid_node)

        return self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                   ReqTypes.GET_ELEMENT_INFO,
                                   raidctrl_node)

    def __send_add_request(self, raiddev_name, part_name):

        ADD_REQ_BASE_XML = '<raidctrl><raid>' +\
                           '  <name type="raid"/>' +\
                           '  <partition><name/></partition>' + \
                           '</raid></raidctrl>'
        RAID_NAME_XPATH = "./raid/name"
        PART_NAME_XPATH = "./raid/partition/name"

        raidctrl_node = XmlNode(ADD_REQ_BASE_XML)
        raidctrl_node.get_node(RAID_NAME_XPATH).set_text(raiddev_name)
        raidctrl_node.get_node(PART_NAME_XPATH).set_text(part_name)

        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.ADD,
                                           raidctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__AddError

        return

    def __send_query_request(self, raiddev_name):

        RAID_QUERY_REQ_XML_TMPL = \
            '<raidctrl><raid><name type="raid">%s</name></raid></raidctrl>'

        result_entry = \
            self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                ReqTypes.QUERY,
                                RAID_QUERY_REQ_XML_TMPL % raiddev_name)

        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        return result_entry.get()

    def __send_query_request_with_element(self, raiddev_name):

        QUERY_REQ_BASE_XML = '<raidctrl>' + \
                             ' <option element="true"/>' + \
                             ' <raid>' + \
                             '  <name type="raid">%s</name>' + \
                             ' </raid>' + \
                             '</raidctrl>'

        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.QUERY,
                                           QUERY_REQ_BASE_XML % raiddev_name)

        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        return result_entry.get()

    def __get_current_raid_element_list(self, raiddev_name):

        PART_NAME_XPATH = "./raid/elementlist/element/name"
        try:
            raidctrl_node = \
                self.__send_query_request_with_element(raiddev_name)
        except self.__RequestError:
            self.exception("fail to query to raidctrl(%s)" % raiddev_name)
            return []

        return [name_node.get_text() for name_node in \
                   raidctrl_node.get_node_list(PART_NAME_XPATH)]

    def __is_system_raid_full_element_for_vfs(self):

        try:
            for raiddev_name in config.SYSTEM_RAID_DEV_LIST:

                full_element_list = self.__raiddev2elements(raiddev_name)
                cur_element_list = \
                    self.__get_current_raid_element_list(raiddev_name)

                full_sata_element_list = \
                    [el for el in full_element_list if el.startswith("satad")]

                cur_sata_element_list = \
                    [el for el in cur_element_list if el.startswith("satad")]

                if len(cur_sata_element_list) < len(full_sata_element_list):
                    return False
        except:
            return False

        return True

    def __get_raid_query_status(self, raiddev_name):

        STATUS_XPATH = "raid/status"
        raidctrl_node = self.__send_query_request(raiddev_name)
        return raidctrl_node.get_value(STATUS_XPATH)

    def __send_start_request(self, org_raidctrl_node,
                             raiddev_name, part_name_list):
        """Send START request to pal_raidcrtrl"""

        raidctrl_node = XmlNode(org_raidctrl_node)
        raid_node = raidctrl_node.get_node(RAID_NAME)
        raid_node.append_child(NAME_NAME, raiddev_name,
                               {TYPE_ATTR: RAID_ATTR_VAL})

        partlist_node = XmlNode(PARTITIONLIST_NAME)
        for part_name in part_name_list:

            part_node = XmlNode(PARTITION_NAME)
            part_node.append_child(NAME_NAME, part_name)
            partlist_node.append_child(part_node)

        raid_node.append_child(partlist_node)

        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.START,
                                           raidctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__StartError

        return

    def __send_stop_request(self, raiddev_name):

        STOP_REQ_BASE_XML = '<raidctrl><raid>' + \
                            '  <name type="raid"/>' + \
                            '</raid></raidctrl>'
        NAME_XPATH = "./raid/name"

        raidctrl_node = XmlNode(STOP_REQ_BASE_XML)
        raidctrl_node.get_node(NAME_XPATH).set_text(raiddev_name)
        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.STOP,
                                           raidctrl_node)

        result_code = result_entry.get_result()
        if result_code == Results.ALREADY_STOP:
            raise self.__AlreadyStop

        if result_code != Results.OK:
            raise self.__StopError

        return

    def __crash_raid(self, raiddev_name):

        RAID_CRASH_REQ_XML_TMPL = \
            """<raidctrl><option crash="true"/>
                <raid><name type="raid">%s</name></raid>
               </raidctrl>"""

        result_entry = \
            self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                ReqTypes.STOP,
                                RAID_CRASH_REQ_XML_TMPL % raiddev_name)

        result_code = result_entry.get_result()
        if result_code == Results.ALREADY_STOP or result_code == Results.OK:
            return

        self.error("fail to crash raid (%s)" % raiddev_name)

    def __send_remove_request(self, raiddev_name, part_name):

        REMOVE_REQ_BASE_XML = '<raidctrl><raid>' + \
                              '  <name type="raid"/>' + \
                              '  <partition><name/></partition>' + \
                              '</raid></raidctrl>'
        RAID_NAME_XPATH = "./raid/name"
        PART_NAME_XPATH = "./raid/partition/name"

        raidctrl_node = XmlNode(REMOVE_REQ_BASE_XML)
        raidctrl_node.get_node(RAID_NAME_XPATH).set_text(raiddev_name)
        raidctrl_node.get_node(PART_NAME_XPATH).set_text(part_name)

        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.REMOVE,
                                           raidctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__RemoveError

        return

    def __get_raiddev_vs_part_name_table(self, disk_name, is_sys_only=False):

        table = {}

        for raiddev_name, element_list in \
                util.get_raid_element_info().iteritems():

            if is_sys_only:
                if raiddev_name not in config.SYSTEM_RAID_DEV_LIST:
                    continue

            for element in element_list:
                if element.startswith(disk_name):
                    table.update({raiddev_name: element})
        return table

    def __remove_disk_from_all_raid(self, disk_name, is_sys_only=False):

        REMOVE_REQ_BASE_XML = \
            '<raidctrl><raid>' + \
            '  <name type="raid"/>' + \
            '  <partition><name/></partition>' +\
            '</raid></raidctrl>'

        is_reqest_error = False

        # ディスクがどのRAIDにどのパーティションとして存在するか取得
        raid_part_table = \
            self.__get_raiddev_vs_part_name_table(disk_name, is_sys_only)

        raidctrl_node = XmlNode(REMOVE_REQ_BASE_XML)
        for raiddev_name, part_name in raid_part_table.iteritems():
            raidctrl_node.get_node("raid/name").set_text(raiddev_name)
            raidctrl_node.get_node("raid/partition/name").set_text(part_name)

            result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                               ReqTypes.REMOVE,
                                               raidctrl_node)

            if result_entry.get_result() != Results.OK:
                is_reqest_error = True

        if is_reqest_error:
            raise self.__RemoveError

        return

    def __remove_disk(self, disk_name):

        RAID_REMOVE_REQ_XML_TMPL = \
            '<raidctrl><disk><name>%s</name></disk></raidctrl>'

        raidctrl_node = XmlNode(RAID_REMOVE_REQ_XML_TMPL % disk_name)
        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.REMOVE,
                                           raidctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__RemoveError

        return

    def __is_usb_mirror_element(self, disk_name):

        if not disk_name.startswith("usb"):
            return False

        array_num = util.get_cuurent_array_num()
        if disk_name in config.HDD_DEV_LIST_IN_VOL[array_num][INTERNAL_VOL]:
            return True
        return False

    def __do_encrypt_stop(self, enc_dev_name):

        CTRL_ENC_BASE_XML = '<volctrl><volume/></volctrl>'
        volctrl_node = XmlNode(CTRL_ENC_BASE_XML)

        # append partition name
        vol_node = volctrl_node.get_node(VOLUME_NAME)

        vol_node.append_child(NAME_NAME,
                              enc_dev_name,
                              {TYPE_ATTR: CRYPT_ATTR_VAL})

        ctrl_entry = request.Entry(ReqTypes.CONTROL, volctrl_node)

        # set option
        ctrl_entry.set_options({OPERATION_OPT: ENC_STOP_OPT_VAL})

        # send request
        result_entry = self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)

        return result_entry.get_result()

    def __stop_encrypt(self, vol_name):

        # fixme
        enc_dev_name = self.__vol_name2enc_part_name(vol_name)
        if self.__is_device_exist(enc_dev_name):

            # stop encrypt
            result_code = self.__do_encrypt_stop(enc_dev_name)
            if Results.OK != result_code and \
               Results.DEVICE_NOT_FOUND != result_code:
                raise self.__StopError
        return

    def __vfs_enc_stop_partition(self, vfs_enc_name):

        CTRL_ENC_BASE_XML = \
            '<volctrl>' + \
             '<option operation="encstop" />' + \
             '<volume>' + \
              '<name type="crypt" />' + \
             '</volume>' + \
            '</volctrl>'

        ctrl_entry = request.Entry(ReqTypes.CONTROL, CTRL_ENC_BASE_XML)
        name_node = ctrl_entry.get().get_node('volume/name')

        if not self.__is_device_exist(vfs_enc_name):
            return

        name_node.set_text(vfs_enc_name)
        result_entry = self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)
        if Results.OK != result_entry.get_result():
            raise self.__StopError
        return

    def __vfs_enc_stop(self):

        VFS_ENC_LIST = \
            [DEV_TO_ENC_DEV[element] \
                for element in util.get_vfs_element_list_on_config()]

        CTRL_ENC_BASE_XML = \
            '<volctrl>' + \
             '<option operation="encstop" />' + \
             '<volume>' + \
              '<name type="crypt" />' + \
             '</volume>' + \
            '</volctrl>'
        ctrl_entry = request.Entry(ReqTypes.CONTROL, CTRL_ENC_BASE_XML)
        name_node = ctrl_entry.get().get_node('volume/name')

        for vfs_enc_name in VFS_ENC_LIST:

            if not self.__is_device_exist(vfs_enc_name):
                continue

            name_node.set_text(vfs_enc_name)
            result_entry = \
                self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)

            if Results.OK != result_entry.get_result():
                raise self.__StopError
        return

    def __get_usable_elements(self, part_dict):

        # 各要素の中で最新のupdatetimeを取得する
        latest_update_time = \
            max([param[UPDATETIME_NAME] for param in part_dict.values() \
                 if UPDATETIME_NAME in param])

        # latestに該当する要素からさらにevents数のmaxを取得する
        latest_events = \
            max([param[EVENTS_NAME] for param in part_dict.values() \
                if UPDATETIME_NAME in param and \
                   latest_update_time == param[UPDATETIME_NAME]])

        # 最新のupdatetimeのリストを作る
        actual_part_name_list = []
        latest_uuid = None

        is_mirroring = self.__is_usb_mirroring_setting()

        for part_name, part_param in part_dict.items():

            if part_name.startswith("usb"):
                if not is_mirroring:
                    continue

            if UPDATETIME_NAME not in part_param:
                continue

            if latest_update_time == part_param[UPDATETIME_NAME] and \
                latest_events == part_param[EVENTS_NAME]:

                if latest_uuid is None:
                    actual_part_name_list.append(part_name)
                    latest_uuid = part_param[UUID_NAME]
                    continue

                if latest_uuid == part_param[UUID_NAME]:
                    actual_part_name_list.append(part_name)

        return actual_part_name_list

    def __make_partition_for_external_vfs(self, disk_name):

        DATA_PART_NUM = 6
        DISK_LABEL_PART_NUM = 4
        PARTLIST_XPATH = "./disk/partitionlist"
        PART_XPATH_TMPL = \
            "./partition/number[text()='%d']/.."

        # errorcountは常に保持するため
        try:
            disk_info = util.get_disk_label_info(disk_name)
            saved_errorcount = disk_info["errorcount"]
        except:
            saved_errorcount = "0"

        # duplicate元のHDDをsystem RAIDから取得
        src_disk_name = \
            self.__get_duplicate_src_disk_for_add_disk(
                config.SYSTEM_RAID_DEV_LIST,
                disk_name)

        result_entry = self.__send_query_to_diskctrl(src_disk_name)
        partlist_node = result_entry.get().get_node(PARTLIST_XPATH)

        # 先頭パーティションの開始位置を求めるための初期値を取得
        part_node = partlist_node.get_node(PART_XPATH_TMPL % 1)
        end_sector = int(part_node.get_node(START_NAME).get_text()) + 1

        for part_num in range(1, DATA_PART_NUM + 1):

            part_node = partlist_node.get_node(PART_XPATH_TMPL % part_num)

            # パーティション開始セクタ
            start_sector = end_sector + 1

            # パーティション最終セクタ
            if DISK_LABEL_PART_NUM == part_num:
                tmp_top = int(part_node.get_node(START_NAME).get_text())
                tmp_end = int(part_node.get_node(END_NAME).get_text())
                end_sector = start_sector + tmp_end - tmp_top
            elif DATA_PART_NUM == part_num:
                end_sector = \
                    util.get_external_vfsext_end_part_sector(disk_name)

                max_end_sector = \
                    util.get_external_vfsext_max_end_part_sector(disk_name)

                if end_sector > max_end_sector:
                      end_sector = max_end_sector

            else:
                # ディスクラベルとデータ以外は最小の8セクタ
                # (4K HDDのパフォーマンス考慮)
                end_sector = start_sector + 8 - 1

            part_node.get_node(START_NAME).set_text(str(start_sector))
            part_node.get_node(END_NAME).set_text(str(end_sector))

        self.__send_delpart_to_diskctrl_with_clear(disk_name)
        self.__send_mkpart_to_diskctrl(disk_name, partlist_node)

        try:
            self.__make_non_raid_partition(src_disk_name, disk_name, True)
        except self.__MkfsError:
            self.exception("fail to make non raid partition")
            pass

        # 元のerrorcountに置き換える
        try:
            util.set_disklabel_value(disk_name,
                                     "errorcount",
                                     saved_errorcount)
        except:
            self.exception("fail to errorcount copy")
            pass

        return

    def __get_swap_size_from_disk_label(self, disk_name):

        disk_info = util.get_disk_label_info(disk_name)
        swap_size = disk_info[config.DEFAULT_SWAP_VALUE]
        if swap_size:
            return int(swap_size)
        raise NotFoundError

    def __duplicate_disk_partition(self, src_disk_name, dst_disk_name,
                                   is_max_end_part=False):
        # errorcountは常に保持するため
        try:
            disk_info = util.get_disk_label_info(dst_disk_name)
            saved_errorcount = disk_info["errorcount"]
        except:
            saved_errorcount = "0"

        # get_partitioninfo from pal_diskctrl
        default_swap_size = None
        if dst_disk_name.startswith("usb"):

            default_swap_size = self.__get_swap_partition_size(src_disk_name)

            disk_node = \
                self.__get_reduced_swap_diskctrl_node(
                    src_disk_name, dst_disk_name).get_node(DISK_NAME)
        else:
            result_entry = self.__send_query_to_diskctrl(src_disk_name)
            disk_node = result_entry.get().get_node(DISK_NAME)

        partlist_node = disk_node.get_node(PARTITIONLIST_NAME)

        # ミラーからの復帰の際
        if src_disk_name.startswith("usb"):
            try:
                swap_size = self.__get_swap_size_from_disk_label(dst_disk_name)
                partlist_node = \
                    self.__change_swap_size_of_partlist_node(partlist_node,
                                                             swap_size)
            except:
                pass

        if is_max_end_part:
            data_part_node = \
                partlist_node.get_node("partition[name='%s']" % \
                                       DATA_PARTITION_NAME)

            # データパーティションはHDDの最後部まで取るようにする
            data_part_node.get_node(END_NAME).set_text("-1")

        self.__send_delpart_to_diskctrl_with_clear(dst_disk_name)
        self.__send_mkpart_to_diskctrl(dst_disk_name, partlist_node)

        try:
            self.__make_non_raid_partition(src_disk_name, dst_disk_name)
        except self.__MkfsError:
            self.exception("fail to make non raid partition")
            pass

        # 元のerrorcountに置き換える
        try:
            util.set_disklabel_value(dst_disk_name,
                                     "errorcount",
                                     saved_errorcount)
        except:
            self.exception("fail to errorcount copy")
            pass

        try:
            if default_swap_size:
                util.set_disklabel_value(dst_disk_name,
                                         config.DEFAULT_SWAP_VALUE,
                                         str(default_swap_size))
        except:
            self.exception("fail to default_swap_size copy")
            pass

        return

    def __make_non_raid_partition(self, src_disk_name, dst_disk_name,
                                  is_external_vfsext=False):

        req_xml = MKFS_NONNRAID_REQ_XML % \
                      {"external": str(is_external_vfsext).lower(),
                       "src": src_disk_name,
                       "dst": dst_disk_name}
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.MKFS,
                                           req_xml)
        if result_entry.get_result() != Results.OK:
            raise self.__MkfsError

    def __get_usable_external_vfsext_port_element_list(self):
        """Get usable external vfsext port.
           ex.
              When the port mode of usb3 is vfsext and
              usb3 HDD is usable this device name is add to list
        """

        usable_element_list = []
        for element_name in util.get_support_external_vfsext_element_list():

            disk_name = self.__partition2disk(element_name)
            iface_name = config.DEV_TO_PORT_IF[disk_name]
            port_num = config.DEV_TO_PORT_NO[disk_name]

            if VFSEXT_PORT_MODE == \
                   util.get_current_port_mode(iface_name, port_num):

                # 存在確認
                if not self.__is_device_exist(disk_name):
                    continue

                usable_element_list.append(element_name)

        return usable_element_list

    def __get_auto_rebuild_disk_for_vfsext(self,
                                           ignore_disk_list=[]):
        vfsext_dev_info = util.get_vfsext_dev_info()
        vfs_element_list = vfsext_dev_info.keys()

        # 外付け拡張ボリュームelement
        external_element_list = \
            self.__get_usable_external_vfsext_port_element_list()

        for element, info in vfsext_dev_info.iteritems():

            if info["is_external"]:
                if element not in external_element_list:
                    if element in vfs_element_list:
                        vfs_element_list.remove(element)
                        continue

            disk_name = self.__partition2disk(element)
            if not self.__is_device_exist(disk_name):
                vfs_element_list.remove(element)
                continue

            if disk_name in ignore_disk_list:
                vfs_element_list.remove(element)

        try:
            (vfs_element_list, un_match_element_list) = \
                util.get_vfsext_uuid_match_element_list(vfs_element_list)
        except NotFoundError:
            # uuidが存在しない場合は全部含めてmount(インストール直後など)
            un_match_element_list = []

        return ([self.__partition2disk(el) for el in vfs_element_list],
                [self.__partition2disk(el) for el in un_match_element_list])

    def __get_auto_rebuild_disk(self, ignore_disk_list=[]):

        def make_add_disk_list(non_disk_map):
            # どのRAIDにも属していないパーティションはディスク全体をaddする
            # そのディスクのリストを作る

            add_disk_list = []
            for raiddev, disk_list in non_disk_map.iteritems():

                for disk_name in disk_list:
                    for tmp_raiddev, tmp_disk_list in non_disk_map.iteritems():

                        if raiddev == tmp_raiddev:
                            continue

                        if MIRROR_DEV == tmp_raiddev or \
                           MIRROR_RAID_SYMBL_DEV == tmp_raiddev:
                            if disk_name.startswith("satad"):
                                continue

                        if INTERNAL_DEV == tmp_raiddev:
                            if disk_name.startswith("usbd"):
                                continue

                        if disk_name not in tmp_disk_list:
                            break

                    else:  # loop over(全RAIDに該当ディスクが存在した)
                        if disk_name not in add_disk_list:
                            add_disk_list.append(disk_name)

            return add_disk_list

        def remake_non_element_map(non_element_map, add_disk_list):

            # 各RAIDに属していないパーティションのディスクがadd_disk_list
            # にあったらそのパーティションはリストから削除
            # (そのパーティションはどうせディスク全体としてaddする)

            tmp_non_element_map = copy.deepcopy(non_element_map)

            for raiddev, element_list in tmp_non_element_map.iteritems():

                for element in element_list:

                    disk_name = self.__partition2disk(element)

                    if disk_name in add_disk_list:
                        while element in non_element_map[raiddev]:
                            non_element_map[raiddev].remove(element)

            return non_element_map

        def remake_add_disk_list(non_element_map, add_disk_list):

            for raiddev, element_list in non_element_map.iteritems():

                for element in element_list:

                    disk_name = self.__partition2disk(element)
                    if disk_name in add_disk_list:
                        add_disk_list.remove(disk_name)

            return add_disk_list

        # 歯抜けのRAID要素のマップ
        # {md2:[satad1p2, satad6p2], md5[satad1p5]}
        non_element_map = {}
        non_disk_map = {}
        for raiddev_name in util.get_raid_element_info().keys():

            if self.__is_current_raid_mode_vfsext():
                if raiddev_name not in config.SYSTEM_RAID_DEV_LIST:
                    continue

            if not self.__is_device_exist(raiddev_name):
                continue
            else:
                non_element_map[raiddev_name] = []
                non_disk_map[raiddev_name] = []

            actual_element_info_dict = \
                self.__get_element_info(raiddev_name, True)

            element_list = self.__raiddev2elements(raiddev_name)

            for element in element_list:
                if not actual_element_info_dict[element][JOIN_ATTR]:

                    if element.startswith("md"):
                        non_element_map[raiddev_name].append(element)
                        continue

                    try:
                        disk_name = self.__partition2disk(element)
                    except:
                        continue

                    if disk_name in ignore_disk_list:
                        continue

                    if not self.__is_device_exist(disk_name):
                        continue

                    if disk_name.startswith(MIRROR_USB_DISK):
                        if not self.__is_usb_mirroring_setting():
                            continue

                    non_element_map[raiddev_name].append(element)
                    non_disk_map[raiddev_name].append(disk_name)

        add_disk_list = make_add_disk_list(non_disk_map)
        non_element_map = \
            remake_non_element_map(non_element_map, add_disk_list)

        add_disk_list = remake_add_disk_list(non_element_map, add_disk_list)

        return non_element_map, add_disk_list

    def __save_vfs_autorebuild_flag(self, disk_name):
        """Save disk added flag
           This flag file is for VFS only
        """

        if self.__is_current_raid_mode_vfsext():
            util.set_disk_info(disk_name, config.NEED_VFS_AUTO_REBUILD, "1")

    def __is_need_vfs_autorebuild(self, disk_name):
        """Check need vfs autorebuild flag"""

        try:
            if "1" == util.get_disk_info(disk_name,
                                         config.NEED_VFS_AUTO_REBUILD):
                return True
        except:
            pass

        return False

    def __is_error_disk(self, disk_name, is_include_pending=False):

        disk_status = util.get_disk_info(disk_name, DISK_STATUS)

        if is_include_pending:
            if disk_status in [DISK_FAIL_STATUS,
                               DISK_SMARTERR_STATUS,
                               DISK_FAIL_PENDING_STATUS,
                               DISK_SMARTERR_PENDING_STATUS]:
                return True
            return False

        if disk_status in [DISK_FAIL_STATUS, DISK_SMARTERR_STATUS]:
            return True
        return False

    def __get_error_disk_list(self, element_list):

        error_disk_name_list = []
        for element in element_list:

            if not element.startswith("satad"):
                if not element.startswith("usbd"):
                    continue

            disk_name = self.__partition2disk(element)
            if self.__is_error_disk(disk_name):
                error_disk_name_list.append(disk_name)

        return error_disk_name_list

    def __remove_all_error_disk(self, is_add_fail=False):

        element_list = []
        for raiddev_name in [INTERNAL_DEV, MIRROR_RAID_SYMBL_DEV]:
            if raiddev_name:
                element_list += self.__raiddev2elements(raiddev_name)

        for element in element_list:

            if not element.startswith("satad"):
                if not element.startswith("usbd"):
                    continue

            disk_name = self.__partition2disk(element)
            disk_status = util.get_disk_info(disk_name, DISK_STATUS)
            if disk_status in [DISK_FAIL_STATUS, DISK_SMARTERR_STATUS,
                               DISK_CAPSHORTAGE_STATUS]:
                util.handle_error_disk(disk_name, disk_status, is_add_fail)
        return

    def __do_auto_rebuild(self):

        part_name_list = []
        for raiddev_name in [INTERNAL_DEV, MIRROR_RAID_SYMBL_DEV]:
            if raiddev_name:
                part_name_list += self.__raiddev2elements(raiddev_name)

        error_disk_list = self.__get_error_disk_list(part_name_list)
        non_element_map, add_disk_list = \
            self.__get_auto_rebuild_disk(error_disk_list)

        if self.__is_current_raid_mode_vfsext():
            vfs_disk_name_list, vfs_add_disk_list = \
                self.__get_auto_rebuild_disk_for_vfsext(
                    add_disk_list + error_disk_list)
            add_disk_list += vfs_add_disk_list

            for vfs_disk_name in vfs_disk_name_list:
                util.set_vfsexe_element_flag(vfs_disk_name)

        # パーティションごとに追加
        for raiddev, element_list in non_element_map.iteritems():
            for element in element_list:

                try:
                    self.__send_add_request(raiddev, element)
                except:
                    self.exception("fail to __send_add_request() - do nothing")

                self.__save_vfs_autorebuild_flag(
                    self.__partition2disk(element))

        # ディスク丸ごと追加
        is_capshortage_occred = False
        is_add_error_occured = False
        for add_disk_name in add_disk_list:

            try:
                self.__add_disk_to_raid(add_disk_name)
            except self.__Capshortage:
                is_capshortage_occred = True
                util.set_disk_info(add_disk_name,
                                   DISK_STATUS,
                                   DISK_CAPSHORTAGE_STATUS)
            except:
                is_add_error_occured = True

        if is_capshortage_occred:
            raise self.__Capshortage

        if is_add_error_occured:
            raise self.__AddError

        return

    def __spindown_unuse_disk(self):

        try:
            util.spindown_unuse_disk()
        except:
            self.exception("fail to util.spindown_unuse_disk()")

    def do_auto_extend(self):

        CONF_RAID_XPATH = "raidlist/raid[name='sataraid1']"
        CONF_RAID_LEVEL_XPATH = "raidlist/raid[name='sataraid1']/level"

        conf_node = self.__load_config()

        cur_raid_level = conf_node.get_value(CONF_RAID_LEVEL_XPATH)
        if not config.RAID_MODE_INFO[cur_raid_level]["is_ext_raid"]:
            return "", EXTEND_NOEXEC

        if not conf_node.get_node(
                   CONF_RAID_XPATH).is_set_to_true(AUTO_EXTEND_ATTR):
            return "", EXTEND_NOEXEC

        for ext_raid_level in config.EXTEND_RAID_MODE_MAP.get(cur_raid_level):
            if self.__is_able_to_change_mode(cur_raid_level, ext_raid_level):
                result_code = self.__do_extend_disk(ext_raid_level, True)
                if result_code != Results.OK:
                     return ext_raid_level, EXTEND_FAIL

                return ext_raid_level, EXTEND_SUCCESS

        return "", EXTEND_NOEXEC

    def __do_start_raid(self, org_raidctrl_node, raiddev_name,
                        is_force, is_auto_rebuild):
        """Do start RAID.
           If the element of specified raid device is other raid device
           call this method as recursively.
        """

        def remove_error_element(element_list, error_disk_list, raid_level):
            # element_listからエラーHDDのelementを消す

            tmp_element_list = list(element_list)

            for element in tmp_element_list:

                if not element.startswith("satad") and \
                       not element.startswith("usbd"):
                    continue

                disk_name = self.__partition2disk(element)
                if disk_name in error_disk_list:

                    if util.get_raid_min_num(raid_level) == len(element_list):
                        # これを外すと崩壊してしまうのでキープ
                        disk_status = \
                            util.get_disk_info(disk_name, DISK_STATUS)

                        if disk_status in PENDINF_STATUS_MAP.keys():
                            util.set_disk_info(disk_name,
                                               DISK_STATUS,
                                               PENDINF_STATUS_MAP[disk_status])
                        return

                    element_list.remove(element)

        is_capshortage_disk_exist = False
        part_name_list = self.__raiddev2elements(raiddev_name,
                                                 raid_mode=None,
                                                 is_need_missing=True)

        error_disk_list = self.__get_error_disk_list(part_name_list)

        if self.__is_current_raid_mode_vfsext():
            return

        for part_name in part_name_list:

            if not self.__is_raid_device(part_name):
                continue

            # RAIDデバイスのエレメントが更にRAIDデバイスの場合は再帰
            try:
                self.__do_start_raid(org_raidctrl_node, part_name,
                                     is_force, is_auto_rebuild)
            except self.__Capshortage:
                is_capshortage_disk_exist = True
            except:
                self.exception("fail to __do_start_raid()")
                self.__crash_raid(part_name)
                pass

        real_raiddev_name = self.__get_real_raiddev_name(raiddev_name)

        raid_level = config.RAID_TO_LEVEL[raiddev_name]
        if not raid_level:
            raid_level = self.__get_config_raid_level()

        # force start mode
        if is_force:
            is_missing, number, actual_element_part_list = \
                self.__remake_start_element_list(raiddev_name, part_name_list)

            if not self.__is_enough_element_num(raid_level,
                                                len(actual_element_part_list)):
                raise self.__StartError

            remove_error_element(actual_element_part_list,
                                 error_disk_list,
                                 raid_level)

            self.__send_start_request(org_raidctrl_node.get_clone(),
                                      real_raiddev_name,
                                      actual_element_part_list)
            return

        # normal start mode
        else:
            element_dict = self.__get_element_info(raiddev_name)
            actual_part_name_list = self.__get_usable_elements(element_dict)

            remove_error_element(actual_part_name_list,
                                 error_disk_list,
                                 raid_level)

            if not self.__is_enough_element_num(raid_level,
                                                len(actual_part_name_list)):
                raise self.__StartError

            self.__send_start_request(org_raidctrl_node.get_clone(),
                                      real_raiddev_name,
                                      actual_part_name_list)

        if is_capshortage_disk_exist:
            raise self.__Capshortage

        return

    def __do_create_raid(self, org_raidctrl_node, raiddev_name):
        """Do create RAID.
           If the element of specified raid device is other raid device
           call this method as recursively.
        """

        LEVEL_XPATH = "./raid/level"

        # product_confで指定されているRAIDモード
        product_raid_level = \
            self.__get_default_raid_level(raiddev_name)

        # 上位から指定されたRAIDモード
        specified_raid_level = \
            org_raidctrl_node.get_node(LEVEL_XPATH).get_text()

        # 上位から指定されたraidmodeを元に構成要素を取得
        element_part_list = \
            self.__raiddev2elements(raiddev_name,
                                    raid_mode=specified_raid_level,
                                    is_need_missing=True)

        raid_level = product_raid_level
        if not product_raid_level:
            raid_level = specified_raid_level

        # get element param
        is_missing, number, actual_element_part_list = \
            self.__remake_creating_element_list(raiddev_name,
                                                element_part_list)

        # デバイスが存在する場合はRAIDをストップ
        if self.__is_device_exist(raiddev_name):
            self.__send_stop_request(raiddev_name)

        # RAIDのエレメントが更にRAIDの場合は再帰
        for part_name in element_part_list:
            if not self.__is_raid_device(part_name):
                continue
            self.__do_create_raid(org_raidctrl_node.get_clone(), part_name)

        # make new raidctrl node
        raidctrl_node = XmlNode(org_raidctrl_node)
        raid_node = raidctrl_node.get_node(RAID_NAME)
        raid_node.append_child(NAME_NAME,
                               self.__get_real_raiddev_name(raiddev_name),
                               {TYPE_ATTR: RAID_ATTR_VAL})

        real_raid_level = config.RAID_MODE_TO_REAL_RAID_MODE[raid_level]
        raidctrl_node.get_node(LEVEL_XPATH).set_text(real_raid_level)

        # set number
        missing_attr = {MISSING_ATTR: "true" if is_missing else "false"}
        raidctrl_node.get_node(RAID_NAME).append_child(NUMBER_NAME,
                                                       str(number),
                                                       missing_attr)
        # set partitionlist node
        raidctrl_node.get_node(RAID_NAME).append_child(PARTITIONLIST_NAME)
        partlist_node = \
            raidctrl_node.get_node(RAID_NAME).get_node(PARTITIONLIST_NAME)

        for part_name in actual_element_part_list:
            part_node = XmlNode(PARTITION_NAME)
            part_node.append_child(NAME_NAME, part_name)
            partlist_node.append_child(part_node)

        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.CREATE,
                                           raidctrl_node)

        if result_entry.get_result() != Results.OK:
            raise self.__CreateError

        return

    def __do_stop_raid(self, org_raidctrl_node, raiddev_name):
        """Do stop RAID.
           If the element of specified raid device is other raid device
           call this method as recursively.
        """

        # VFS
        if self.__is_current_raid_mode_vfsext():
            return

        if self.__is_device_exist(raiddev_name):
            try:
                self.__send_stop_request(raiddev_name)
            except self.__AlreadyStop:
                pass

        part_name_list = self.__raiddev2elements(raiddev_name)

        for part_name in part_name_list:

            if not self.__is_raid_device(part_name):
                continue

            # RAIDデバイスのエレメントが更にRAIDデバイスの場合は再帰
            try:
                self.__do_stop_raid(org_raidctrl_node, part_name)
            except self.__AlreadyStop:
                continue

        return

    def __add_vfsext_extend_disk(self, disk_name_list):

        for disk_name in disk_name_list:

            if util.is_errorcount_over(disk_name):
                self.error("errcount over detected(%s)" % disk_name)
                util.set_disk_info(disk_name,
                                   DISK_STATUS,
                                   DISK_FAIL_STATUS)
                return Results.DISK_ERROR

            if util.is_smart_error(disk_name):
                self.error("smart error detected(%s)" % disk_name)
                util.set_disk_info(disk_name,
                                   DISK_STATUS,
                                   DISK_SMARTERR_STATUS)
                return Results.DISK_ERROR

    def __add_partitions_to_vfsext(self, part_name_list, is_enc=False):

        VFS_ADD_BASE_REQ_XML = \
            '<volctrl>' + \
            ' <option vfsmultiadd="true"/>' + \
            ' <volume>' + \
            '  <filesystem encrypt="%(is_enc)s"/>' + \
            '  <partitionlist/>' + \
            ' </volume>' + \
            '</volctrl>'

        PARTITION_XML_TMPL = '<partition><name>%s</name></partition>'

        req_node = \
            XmlNode(VFS_ADD_BASE_REQ_XML % {"is_enc": str(is_enc).lower()})

        partlist_node = req_node.get_node("volume/partitionlist")

        for part_name in part_name_list:
            partlist_node.append_child(XmlNode(PARTITION_XML_TMPL % part_name))

        return self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                   ReqTypes.MOUNT,
                                   req_node)

    def __do_extend_disk(self, ext_raid_level, is_during_start=False):

        CONF_RAID_LEVEL_XPATH = "raidlist/raid/level"

        try:
            with fileut.FLock(config.LOCK_DIR, RAID_EXTEND_LOCK_NAME,
                              exclusive=True, timeout=0):

                result_code = self.__sub_do_extend_disk(
                                  ext_raid_level, is_during_start)

                conf_node = self.__load_config()
                conf_node.get_node(CONF_RAID_LEVEL_XPATH).set_text(ext_raid_level)
                self.__save_config(conf_node)
                return result_code
        except:
            return Results.EXTEND_ERROR

    def __sub_do_extend_disk(self, ext_raid_level, is_during_start=False):

        CONF_RAID_LEVEL_XPATH = "raidlist/raid/level"

        result_code = Results.EXTEND_ERROR
        conf_node = self.__load_config()
        cur_raid_level = conf_node.get_value(CONF_RAID_LEVEL_XPATH)
        if not config.RAID_MODE_INFO[cur_raid_level]["is_ext_raid"]:
            return Results.EXTEND_ERROR

        if not self.__is_able_to_change_mode(cur_raid_level, ext_raid_level):
            raise self.__CannotChange

        cur_array_num = config.RAID_MODE_INFO[cur_raid_level]["array_num"]
        ext_array_num = config.RAID_MODE_INFO[ext_raid_level]["array_num"]
        cur_disk_list = config.HDD_DEV_LIST_IN_VOL[cur_array_num][INTERNAL_VOL]
        ext_disk_list = config.HDD_DEV_LIST_IN_VOL[ext_array_num][INTERNAL_VOL]

        is_enc = self.__is_internal_volume_encrypted()
        vfsext_element_list = []
        for disk in ext_disk_list:

            if disk in cur_disk_list:
                continue

            if not disk.startswith("satad"):
                continue

            if not self.__is_device_exist(disk):
                result_code = Results.DEVICE_NOT_FOUND
                continue

            # パーティション切ってシステムRAIDに追加
            try:
                self.__standard_add_disk_to_raid(disk, ext_raid_level)
            except self.__Capshortage:
                self.exception("Capshortage(%s)" % disk)
                result_code = Results.CAP_SHORTAGE
            except self.__ErrorDiskDetected:
                self.exception("ErrorDiskDetected(%s)" % disk)
                result_code = Results.DISK_ERROR
            except self.__AddError:
                self.exception("AddError(%s)" % disk)
                result_code = Results.EXTEND_ERROR
            except:
                self.exception("UnknownError(%s)" % disk)
                result_code = Results.EXTEND_ERROR

            if is_during_start:
                util.clear_vfsexe_element_flag(disk)
                self.__save_vfs_autorebuild_flag(disk)
                continue

            # データパーティションをフォーマット
            try:
                part_name = util.disk_name2vfs_element_name(
                                disk, ext_raid_level)
                if is_enc:
                    enc_part_name = DEV_TO_ENC_DEV[part_name]
                    self.__do_enc_start_partition(part_name,
                                                  enc_part_name)
                    self.__mkfs_vfs_partition(enc_part_name, True)
                else:
                    self.__mkfs_vfs_partition(part_name)

                vfsext_element_list.append(part_name)

            except self.__MkfsError:
                self.exception("fail to mkfs_vfs_partition(%s)" % part_name)
                result_code = Results.EXTEND_ERROR

        if is_during_start:
            return Results.OK

        # 成功したディスクがなければ終了
        if len(vfsext_element_list) == 0:
            return result_code

        # まとめてvfsextパーティションをadd
        result_entry = \
            self.__add_partitions_to_vfsext(vfsext_element_list, is_enc)
        if result_entry.get_result() == Results.OK:
            return Results.OK

        return result_code

    def __is_need_max_size_partition(self, src_disk_name, added_disk_name):

        if src_disk_name.startswith("usbd") and \
               added_disk_name.startswith("satad"):
            return True

        if added_disk_name.startswith("satad"):
            if self.__is_current_raid_mode_vfsext():
                return True

        if added_disk_name.startswith("usb"):
            return True

        return False

    def __external_vfs_add_disk_to_raid(self, disk_name):

        # 容量確認
        if not util.is_enough_disk_size_vfsext(disk_name, is_use_flag=True):
            util.set_volume_cache_value(self.__dev_name2vol_name(disk_name),
                                        STATUS_NAME,
                                        CAPSHORTAGE_STATUS)
            raise self.__Capshortage

        # 待機中かどうか確認
        if not util.is_have_system_partition(disk_name):
            vol_name = self.__dev_name2vol_name(disk_name)
            if util.is_volume_exist_any_files(vol_name):
                util.set_volume_cache_value(vol_name, STATUS_NAME,
                                            WAITING_STATUS)
                raise self.__WaitingDiskDetected

        # パーティションを切る
        self.__make_partition_for_external_vfs(disk_name)

    def __standard_add_disk_to_raid(self, added_disk_name, raid_mode=None,
                                    is_sys_only=False):
        """Add disk to Raid device"""

        if self.__is_error_disk(added_disk_name):
            raise self.__ErrorDiskDetected

        if self.__is_usb_mirror_element(added_disk_name):
            if added_disk_name not in config.DEV_TO_VOL.keys():
                raise self.__AddError

            if not self.__is_disk_label_exist(added_disk_name):
                # USBミラーの場合は
                if util.is_volume_exist_any_files(
                    config.DEV_TO_VOL[added_disk_name]):
                    raise self.__WaitingDiskDetected

        # ADD先のRAIDデバイスのリスト取得
        tgt_raiddev_name_list = \
            self.__get_raiddev_name_list_for_add_disk(
                added_disk_name, raid_mode)

        # まずHDDのパーティションを複製するための複製元HDDを取得
        src_disk_name = \
            self.__get_duplicate_src_disk_for_add_disk(tgt_raiddev_name_list,
                                                       added_disk_name)
        if not src_disk_name:
            raise self.__AddError

        if (self.__is_current_raid_mode_vfsext() and not raid_mode) or \
           (raid_mode and config.RAID_MODE_INFO[raid_mode].get("is_ext_raid")):
            if not util.is_enough_disk_size_vfsext(added_disk_name,
                                                   is_use_flag=True,
                                                   vfsext_mode=raid_mode):
                raise self.__Capshortage
        else:
            if not self.__is_enough_disk_size(src_disk_name, added_disk_name):
                raise self.__Capshortage

        # swapパーティションが存在する状態で立ち上がった場合は
        # swapが動作しているのでとりあえずswap off
        try:
            self.__swap_off_added_disk(added_disk_name)
        except:
            pass

        is_max_end_part = \
            self.__is_need_max_size_partition(src_disk_name, added_disk_name)

        # 取得した複製元HDDのパーティション構成を追加されたHDDに複製
        self.__duplicate_disk_partition(src_disk_name,
                                        added_disk_name,
                                        is_max_end_part)

        try:
            self.__swap_on_added_disk(added_disk_name,
                                      self.__is_internal_volume_encrypted())
        except:
            self.exception(("fail to __swap_on_added_disk(%s)" +
                           " - do nothing") % added_disk_name)

        is_add_fail = False
        # 全ターゲットRAIDデバイスにADDしていく
        for tgt_raiddev_name in tgt_raiddev_name_list:

            if is_sys_only:
                if tgt_raiddev_name in [MIRROR_RAID_SYMBL_DEV,
                                        MIRROR_DEV, INTERNAL_DEV]:
                    continue

            raid_status = self.__get_raid_query_status(tgt_raiddev_name)
            if raid_status in [CRASH_STATUS, STOP_STATUS]:
                continue

            element_name_list = \
                self.__raiddev2elements(tgt_raiddev_name, raid_mode)

            for element_name in element_name_list:
                if element_name.startswith(added_disk_name):
                    try:
                        self.__send_add_request(tgt_raiddev_name, element_name)
                    except self.__AddError:
                        is_add_fail = True
                        continue

        if is_add_fail:
            raise self.__AddError

        return

    def __add_disk_to_raid(self, added_disk_name, raid_mode=None):

        vol_name = self.__dev_name2vol_name(added_disk_name)

        vol_status = util.get_volume_cache_value(vol_name, STATUS_NAME)
        if vol_status == OTHER_MIRROR_STATUS:
            return

        if vol_status == RECOVERABLE_MIRROR_STATUS:
            (is_mirrordisk, is_recoverable, is_match_system) = \
                util.check_mirrordisk(self.__vol_name2dev_name(vol_name))
            if not is_match_system:
                return

        if self.__is_need_to_add_as_external_vfs_disk(added_disk_name):
            self.__external_vfs_add_disk_to_raid(added_disk_name)
        else:
            self.__standard_add_disk_to_raid(added_disk_name, raid_mode)

        if self.__is_current_raid_mode_vfsext():
            self.__save_vfs_autorebuild_flag(added_disk_name)
        return

    def __add_disk_to_system_raid(self, added_disk_name, raid_mode=None):

        if self.__is_need_to_add_as_external_vfs_disk(added_disk_name):
            self.__external_vfs_add_disk_to_raid(added_disk_name)
        else:
            self.__standard_add_disk_to_raid(added_disk_name, raid_mode,
                                             is_sys_only=True)
        return

    def __is_specified_raid_name(self, entry):

        RAID_NAME_XPATH = "./raid/name[@type='raid']"
        try:
            entry.get().get_node(RAID_NAME_XPATH)
        except:
            return False

        return True

    def __is_specified_system_raid_name(self, entry):

        RAID_NAME_XPATH = "./raid/name[@type='raid']"

        raid_name = entry.get().get_value(RAID_NAME_XPATH)
        if raid_name in config.SYSTEM_RAID_DEV_LIST:
            return True

        return False

    def __swap_on(self, swap_part_name):
        """Enable swap"""

        SWAP_ON_REQ_XML = \
            """<volctrl>
                 <option operation="swapon"/>
                 <volume>
                   <name type="partition">%s</name>
                 </volume>
               </volctrl>"""

        return self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                   ReqTypes.CONTROL,
                                   SWAP_ON_REQ_XML % swap_part_name)

    def __swap_create(self, swap_part_name):
        """Enable swap"""

        SWAP_CREATE_REQ_XML = \
            """<volctrl>
                 <option operation="swapcreate"/>
                 <volume>
                   <name type="partition">%s</name>
                 </volume>
               </volctrl>"""

        return self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                   ReqTypes.CONTROL,
                                   SWAP_CREATE_REQ_XML % swap_part_name)

    def __swap_off(self, swap_part_name):
        """Disable swap"""

        SWAP_OFF_REQ_XML = \
            """<volctrl>
                 <option operation="swapoff"/>
                 <volume>
                   <name type="partition">%s</name>
                 </volume>
               </volctrl>"""

        return self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                   ReqTypes.CONTROL,
                                   SWAP_OFF_REQ_XML % swap_part_name)

    def __swap_off_added_disk(self, added_disk_name):

        self.debug("__swap_off_added_disk")
        swap_part_name = added_disk_name + SWAP_PART_SUFFIX
        enc_swap_part_name = DEV_TO_ENC_DEV[swap_part_name]

        for swap in [swap_part_name, enc_swap_part_name]:
            try:
                self.__swap_off(swap)
            except:
                continue

    def __swap_on_added_disk(self, added_disk_name, is_enc=False):

        self.debug("__add_swap")
        swap_part_name = added_disk_name + SWAP_PART_SUFFIX

        try:
            self.__swap_off(swap_part_name)
        except:
            pass

        if is_enc:
            enc_swap_part_name = DEV_TO_ENC_DEV[swap_part_name]

            try:
                self.__swap_off(enc_swap_part_name)
            except:
                pass

            self.__do_enc_start_partition(swap_part_name, enc_swap_part_name)
            swap_part_name = enc_swap_part_name

        # create
        try:
            self.__swap_create(swap_part_name)
        except:
            self.error("fail to __swap_create(%s)" % swap_part_name)
            pass

        # on
        try:
            self.__swap_on(swap_part_name)
        except:
            self.error("fail to __swap_on(%s)" % swap_part_name)
            pass

        return

    def __start_raid_service(self):

        START_RAID_SERVICE_REQ_XML = \
            """<raidctrl>
                 <option operation="startservice"/>
               </raidctrl>"""

        return self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                   ReqTypes.CONTROL,
                                   START_RAID_SERVICE_REQ_XML)

    def __stop_raid_service(self):

        STOP_RAID_SERVICE_REQ_XML = \
            """<raidctrl>
                 <option operation="stopservice"/>
               </raidctrl>"""

        return self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                   ReqTypes.CONTROL,
                                   STOP_RAID_SERVICE_REQ_XML)

    def __recover_internal(self, entry):

        RAID_CREATE_REQ_XML = """<raidctrl>
                                   <option operation="silent"/>
                                   <raid>
                                    <name type="raid">%(raid_name)s</name>
                                    <level>%(level)s</level>
                                    <number missing="true">%(number)s</number>
                                    <partitionlist/>
                                   </raid>
                                 </raidctrl>"""

        src_disk_name = MIRROR_USB_DISK

        raid_level = self.__get_config_raid_level()
        if "vfsext" == config.RAID_MODE_TO_REAL_RAID_MODE[raid_level]:
            entry.set_result(Results.CREATE_ERROR)
            return entry

        # 念のためmd13からmd6を削除
        try:
            self.__send_remove_request(MIRROR_DEV, INTERNAL_DEV)
        except:
            self.exception("fail to __send_remove_request")
            pass

        # 念のためmd6を停止
        try:
            self.__send_stop_request(INTERNAL_DEV)
        except:
            self.exception("fail to __send_stop_request")
            pass

        dst_disk_name_list = []
        for dst_disk_name in config.SATA_HDD_DEV_LIST:

            if not self.__is_device_exist(dst_disk_name):
                continue

            try:
                self.__remove_disk(dst_disk_name)
            except:
                pass

            try:
                self.__swap_off_added_disk(dst_disk_name)
            except:
                pass

            # まず最初にdiskのパーティション複製
            try:
                self.__duplicate_disk_partition(src_disk_name,
                                                dst_disk_name,
                                                True)
            except:
                self.exception("fail to duplicate_disk_partition")
                continue

            # md2, md5にadd
            for sys_raiddev_name in config.SYSTEM_RAID_DEV_LIST:

                match = re.match("[a-zA-Z]+([0-9]+)", sys_raiddev_name)
                if not match:
                    continue
                num = match.group(1)

                try:
                    self.__send_add_request(sys_raiddev_name,
                                            dst_disk_name + "p" + num)
                except:
                    self.exception("fail to adding disk to system raid")
                    pass

            util.set_disk_info(dst_disk_name, DISK_STATUS, DISK_NORMAL_STATUS)
            dst_disk_name_list.append(dst_disk_name)

        if not self.__is_enough_element_num(raid_level,
                                            len(dst_disk_name_list)):
            raise self.__CreateError

        # データパーティションのRAIDデバイス作成
        internal_raiddev_name = config.INTERNAL_RAID_DEV_LIST[0]
        match = re.match("[a-zA-Z]+([0-9]+)", internal_raiddev_name)
        if not match:
            raise self.__CreateError
        num = match.group(1)

        raidctrl_node = \
            XmlNode(RAID_CREATE_REQ_XML % \
                {"raid_name": INTERNAL_DEV,
                 "number": str(len(dst_disk_name_list)),
                 "level": config.RAID_MODE_TO_REAL_RAID_MODE[raid_level]})
        partlist_node = \
            raidctrl_node.get_node(RAID_NAME).get_node(PARTITIONLIST_NAME)

        for disk_name in dst_disk_name_list:
            part_name = disk_name + "p" + num
            part_node = XmlNode(PARTITION_NAME)
            part_node.append_child(NAME_NAME, part_name)
            partlist_node.append_child(part_node)

        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.CREATE,
                                           raidctrl_node)
        if result_entry.get_result() != Results.OK:
            raise self.__CreateError

        try:
            self.__send_add_request(MIRROR_DEV,
                                    internal_raiddev_name)
        except:
            self.exception("fail to adding disk to system raid")
            raise self.__CreateError

        self.__dispatch_raid_update_event()
        self.__dispatch_disk_update_event()

        entry.set_result(Results.OK)
        return entry

    def __delete_raid_info(self, raid_name):

        DELETE_REQ_BASE_XML = "<raidctrl>" + \
                              "<raid><name type='raid'>%s</name></raid>" + \
                              "</raidctrl>"
        
        # mdデバイス削除
        raidctrl_node = XmlNode(DELETE_REQ_BASE_XML % raid_name)
        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.DELETE,
                                           raidctrl_node)
        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

    def __recover_form_other_mirror(self, entry):

        REBOOT_ENTRY = "<powersetting><option force='true'/></powersetting>"

        for raid_name in \
            config.SYSTEM_RAID_DEV_LIST + config.INTERNAL_RAID_DEV_LIST:

            try:
                self.__delete_raid_info(raid_name)
            except:
                self.exception(
                    "fail to __recover_form_other_mirror(%s) % raid_name")

        shutdown_entry = request.Entry(ReqTypes.REBOOT, REBOOT_ENTRY)
        shutdown_result = self.send_entry(shutdown_entry,
                                          pal_info.PAL_POWER_PATH)
        if Results.OK != shutdown_result.get_result():
            raise self.__RequestError

    #
    # Request Handler
    #

    # FIXME 暫定対処
    def __get_vfs_query(self, entry):

        def is_mirror_target(entry):

            RAID_NAME_XPATH = \
                "./raid/name[@type='raid' and text()='%s']" % MIRROR_DEV
            DEPTH1_XPATH = "./raid[@depth='1']"

            try:
                entry.get().get_node(RAID_NAME_XPATH)
                return True
            except:
                pass

            try:
                entry.get().get_node(DEPTH1_XPATH)
                return True
            except:
                pass

            return False

        def is_full_vfs_element(cur_part_node_list):

            cur_element_list = [node.get_text() for node in cur_part_node_list]
            for element, info in util.get_vfsext_dev_info().iteritems():
                if info["is_external"]:
                    continue
                if element not in cur_element_list:
                    return False
            return True

        def is_full_external_vfs_element(cur_part_node_list):

            external_vfs_port_mode_element_list = \
                util.get_external_vfsext_port_mode_element_list()
            cur_element_list = [node.get_text() for node in cur_part_node_list]
            for element, info in util.get_vfsext_dev_info().iteritems():

                if not info["is_external"]:
                    continue

                if element not in external_vfs_port_mode_element_list:
                    continue

                if element not in cur_element_list:
                    if self.__is_need_vfs_autorebuild(
                           self.__partition2disk(element)):
                        continue

                    return False

            return True

        TOTAL_CAPA_XPATH = "volume/capacity/total"
        STATUS_XPATH = "volume/status"
        PARTITION_XPATH = "volume/partitionlist/partition"
        PROGRESS_XPATH = "volume/progress"
        MOUNTED_STATUS = "mounted"
        RUNNING_STATUS = "running"
        REBUILDING_STATUS = "rebuilding"
        DEGRADE_STATUS = "degrade"
        STOP_STATUS = "stop"
        RAID_XPATH = "./raid"

        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.QUERY,
                                           VFS_QUERY_XML)

        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        part_node_list = []
        if MOUNTED_STATUS == result_entry.get().get_value(STATUS_XPATH):
            status = RUNNING_STATUS
            part_node_list = result_entry.get().get_node_list(PARTITION_XPATH)
            if not is_full_vfs_element(part_node_list):
                if self.__is_system_raid_full_element_for_vfs():
                    # system raidの構成台数が最大で、VFSの構成数が足りない
                    # 場合はautorebuildが走るのでとりあえずrunningにする
                    status = RUNNING_STATUS
                else:
                    status = DEGRADE_STATUS

            if RUNNING_STATUS == status:
                # external vfsモードなのにvfsに組み込まれてないならdegrade
                if not is_full_external_vfs_element(part_node_list):
                    status = DEGRADE_STATUS

        else:
            if self.__is_internal_volume_encrypted():
                status = RUNNING_STATUS
            else:
                status = STOP_STATUS

        if is_mirror_target(entry):
            entry.get().get_node(RAID_XPATH).append_child(STATUS_NAME,
                                                          status)
            return

        progress = result_entry.get().get_value(PROGRESS_XPATH)
        if progress:
            status = REBUILDING_STATUS

        size = result_entry.get().get_value(TOTAL_CAPA_XPATH)

        raid_node = entry.get().get_node(RAID_NAME)
        raid_node.append_child(STATUS_NAME, status)

        # 冗長性は常にTrue
        raid_node.get_node(STATUS_NAME).set_bool_attr(REDUDANCY_ATTR, True)

        raid_node.append_child(SIZE_NAME, size)
        raid_node.append_child(LEVEL_NAME, VFSEXT_FSTYPE)
        if progress:
            raid_node.append_child(PROGRESS_NAME, progress)

        # elementのリスト
        elementlist_node = XmlNode(ELEMENTLIST_NAME)
        for part_node in part_node_list:
            element_node = XmlNode(ELEMENT_NAME)
            element_node.append_child(NAME_NAME, part_node.get_text())
            elementlist_node.append_child(element_node)
        raid_node.append_child(elementlist_node)

        return

    def __convert_mirroring_status(self, result_entry):

        STAUS_XPATH = "./raid/status"
        PART_NAME_XPATH = "./raid/elementlist/element/name"

        status = result_entry.get().get_value(STAUS_XPATH)
        if DEGRADE_STATUS == status:

            element_list = \
                [name_node.get_text() for name_node in \
                    result_entry.get().get_node_list(PART_NAME_XPATH)]

            if len(element_list) == 1:
                if element_list[0].startswith("usb"):
                    result_entry.get().get_node(STAUS_XPATH).\
                        set_text(FAIL_STATUS)
                    return

            if not self.__is_usb_mirroring_setting():
                result_entry.get().get_node(STAUS_XPATH).\
                    set_text(RUNNING_STATUS)
                return
        return

    def __convert_raid_level_to_config_raid_level(self, entry):

        try:
            entry.get().get_node(RAID_NAME).delete_children(LEVEL_NAME)
            config_raid_level = self.__get_config_raid_level()
            if config_raid_level:
                entry.get().get_node(
                    RAID_NAME).append_child(LEVEL_NAME, config_raid_level)
        except NotFoundError:
            pass
        except:
            self.exception("fail to __convert_raid_level_to_config_raid_level")
            pass

    def __query_handler(self, entry):
        """Handle "query" request."""

        VOL_NAME_XPATH = "./name[@type='volume']"
        RAID_NAME_XPATH = "./raid/name[@type='raid']"
        OPTION_ELEMENT_XML = '<option element="true"/>'

        # VFS 仮実装
        if self.__is_current_raid_mode_vfsext():
            if not self.__is_specified_system_raid_name(entry):
                self.__get_vfs_query(entry)
                entry.set_result(Results.OK)
                self.__convert_raid_level_to_config_raid_level(entry)
                return entry

        # element属性をつけておく
        entry.get().append_child(XmlNode(OPTION_ELEMENT_XML))

        # raidデバイス名を直接指定された
        if self.__is_specified_raid_name(entry):

            result_entry = self.send_entry(entry, pal_info.PAL_RAIDCTRL_PATH)

            raiddev_name = entry.get().get_value(RAID_NAME_XPATH)

            if raiddev_name in config.SYSTEM_RAID_DEV_LIST + \
                   (MIRROR_DEV, MIRROR_RAID_SYMBL_DEV,):

                self.__convert_mirroring_status(result_entry)
            if raiddev_name == INTERNAL_DEV:
                self.__convert_raid_level_to_config_raid_level(result_entry)

            return result_entry

        # depth属性で指定された
        raid_node = entry.get().get_node(RAID_NAME)
        depth = int(raid_node.get_attr_value(DEPTH_ATTR))
        vol_name = raid_node.get_node(VOL_NAME_XPATH).get_text()
        raiddev_name = self.__vol_name2raiddev_name(vol_name)

        for idx in range(1, depth):

            element_name_list = self.__raiddev2elements(raiddev_name)

            for element_name in element_name_list:
                if self.__is_raid_device(element_name):
                    raiddev_name = element_name
                    break
            else:  # loop over(=>RAID device is not found)
                if len(element_name_list) > 0:
                    raid_node.append_child(LEVEL_NAME, SINGLE_RAID_LEVEL)
                    if self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.QUERY,
                                           """<diskctrl><disk><name>%(diskname)s</name></disk></diskctrl>""" % {
                                               "diskname": element_name_list[0],
                                           }).get_result() == Results.DEVICE_NOT_FOUND:
                        raid_node.append_child(STATUS_NAME, CRASH_STATUS)
                    else:
                        raid_node.append_child(STATUS_NAME, RUNNING_STATUS)
                    entry.set_result(Results.OK)
                else:
                    entry.set_result(Results.DEVICE_NOT_FOUND)
                return entry

        raid_node.append_child(NAME_NAME, raiddev_name,
                               {TYPE_ATTR: RAID_ATTR_VAL})

        result_entry = self.send_entry(entry, pal_info.PAL_RAIDCTRL_PATH)
        if Results.OK != result_entry.get_result():
            return result_entry

        if raiddev_name in config.SYSTEM_RAID_DEV_LIST + \
               (MIRROR_DEV, MIRROR_RAID_SYMBL_DEV,):

            self.__convert_mirroring_status(result_entry)

        self.__convert_raid_level_to_config_raid_level(result_entry)
        return result_entry

    def __start_handler(self, entry):
        """Handle "start" request."""

        VOL_NAME_XPATH = "./raid/name[@type='volume']"
        raidctrl_node = entry.get()
        vol_name = raidctrl_node.get_node(VOL_NAME_XPATH).get_text()
        raiddev_name = self.__vol_name2raiddev_name(vol_name)

        opt_node = raidctrl_node.get_node(OPTION_NAME)
        is_force = opt_node.is_set_to_true(FORCE_ATTR)
        is_auto_rebuild = opt_node.is_set_to_true(AUTOREBUILD_ATTR)

        try:
            self.__do_start_raid(raidctrl_node, raiddev_name,
                                 is_force, is_auto_rebuild)
        except self.__StartError:
            entry.set_result(Results.START_ERROR)
            self.__crash_raid(raiddev_name)

        except self.__AddError:
            # メインのSTARTは成功しているのでとりあえずOK
            entry.set_result(Results.OK)

        except self.__Capshortage:
            entry.set_result(Results.CAP_SHORTAGE)

        except:
            entry.set_result(Results.START_ERROR)
            self.__crash_raid(raiddev_name)

        else:
            entry.set_result(Results.OK)

        try:
            if is_auto_rebuild:
                self.__do_auto_rebuild()
        except self.__Capshortage:
            entry.set_result(Results.CAP_SHORTAGE)
        except:
            pass

        ext_raid_level = ""
        extend_status = EXTEND_NOEXEC
        try:
            if is_auto_rebuild:
                ext_raid_level, extend_status = self.do_auto_extend()
        except:
            pass
        finally:
            self.__spindown_unuse_disk()

        try:
            self.__remove_all_error_disk(is_add_fail=True)
        except:
            self.exception("fail to __remove_all_error_disk() - do nothing")

        if not self.__is_current_raid_mode_vfsext():
            if is_force:
                # fal_volctrlでmount後にraid-updateかけるか判別用
                util.set_raid_cache_value(MIRROR_DEV,
                                          FORCE_START_DONE,
                                          "1")
                if INTERNAL_DEV:
                    util.set_raid_cache_value(INTERNAL_DEV,
                                              FORCE_START_DONE,
                                              "1")

        if is_force:
            # LEDがRAID崩壊状態になってるはずなのでLEDの更新は必要
            self.__dispatch_disk_update_event()

        # 結果挿入
        entry.get().append_child(
            XmlNode(START_RESULT_XML_TMPL % {"level": ext_raid_level,
                                             "status": extend_status}))
        return entry

    def __stop_handler(self, entry):
        """Handle "stop" request."""

        # PAL pass through
        if self.__is_specified_raid_name(entry):
            return self.send_entry(entry, pal_info.PAL_RAIDCTRL_PATH)

        VOL_NAME_XPATH = "./raid/name[@type='volume']"
        raidctrl_node = entry.get()
        vol_name = raidctrl_node.get_node(VOL_NAME_XPATH).get_text()
        raiddev_name = self.__vol_name2raiddev_name(vol_name)

        self.__dispatch_pre_unmount_event(vol_name)

        self.__unmount(vol_name)

        try:
            self.__do_stop_raid(raidctrl_node, raiddev_name)
        except self.__StopError:
            entry.set_result(Results.STOP_ERROR)
        else:
            entry.set_result(Results.OK)

        self.__dispatch_raid_update_event()
        return entry

    def __get_need_add_disk_list_for_create(self, raid_mode):

        add_disk_list = []

        # RAID崩壊していた場合は追加すべきHDDを追加
        internal_status = self.__get_raid_query_status(INTERNAL_DEV)
        mirror_status = self.__get_raid_query_status(MIRROR_DEV)
        if mirror_status in [CRASH_STATUS, STOP_STATUS]:
            if internal_status in [CRASH_STATUS, STOP_STATUS]:
                un_use_arg, add_disk_list = self.__get_auto_rebuild_disk()

        set_raid_element_info = util.get_raid_element_info(raid_mode)

        for raid_name in config.SYSTEM_RAID_DEV_LIST:

            cur_element_list = \
                    self.__get_current_raid_element_list(raid_name)

            for element in set_raid_element_info[raid_name]:

                if MISSING_DEVICE == element:
                    continue

                if element in cur_element_list:
                    continue

                if not element.startswith("satad"):
                    continue

                add_disk_list.append(self.__partition2disk(element))

        add_disk_list = sorted(set(add_disk_list), key=add_disk_list.index)
        return add_disk_list

    def __remove_non_element_disk_form_all_raid(self):

        removed_disk_list = []

        for raiddev_name, element_list in \
                util.get_raid_element_info().iteritems():

            for cur_element in \
                    self.__get_current_raid_element_list(raiddev_name):

                if cur_element not in element_list:
                    # 不要なelement
                    disk_name = self.__partition2disk(cur_element)
                    self.__remove_disk(disk_name)

                    if disk_name not in removed_disk_list:
                        removed_disk_list.append(disk_name)

        for disk_name in removed_disk_list:
            try:
                self.__swap_off_added_disk(disk_name)

                enc_swap_part_name = \
                    DEV_TO_ENC_DEV.get(disk_name + SWAP_PART_SUFFIX)
                if enc_swap_part_name:
                    if self.__is_device_exist(enc_swap_part_name):
                        self.__do_encrypt_stop(enc_swap_part_name)

            except:
                self.exception("fail to disable swap (%s)" % disk_name)
                pass

        return


    def __create_handler(self, entry):
        """Handle "create" request."""

        VOL_NAME_XPATH = "./raid/name[@type='volume']"
        CONF_RAID_LEVEL_XPATH = "raidlist/raid/level"

        def set_all_disk_status(status):

            for disk_name in config.SATA_HDD_DEV_LIST:
                util.set_disk_info(disk_name, DISK_STATUS, status)
                self.__save_user_remove_flag(disk_name, False)

        def check_all_disks_valid(raid_level):

            if config.RAID_MODE_INFO[raid_level]["is_ext_raid"]:
                raiddev_name = MIRROR_RAID_SYMBL_DEV
            else:
                raiddev_name = INTERNAL_DEV

            for element in self.__raiddev2elements(raiddev_name, raid_level):

                if not element.startswith("satad"):
                    continue

                try:
                    disk_name = self.__partition2disk(element)
                except:
                    self.exception(
                        "fail to set __partition2disk(%s)" % element)
                    continue

                # 存在確認
                if not self.__is_device_exist(disk_name):
                    raise self.__DeviceNotFound

                if self.__is_error_disk(disk_name, True):
                    self.error("error disk detected(%s)" % disk_name)
                    raise self.__ErrorDiskDetected

            return

        def set_all_disk_label(raid_level):

            if self.__is_current_raid_mode_vfsext():
                raiddev_name = MIRROR_RAID_SYMBL_DEV
            else:
                raiddev_name = INTERNAL_DEV

            for element in self.__raiddev2elements(raiddev_name):

                if not element.startswith("satad"):
                    continue

                try:
                    disk_name = self.__partition2disk(element)
                    util.set_disklabel_value(disk_name, "raidmode", raid_level)
                except:
                    self.exception("fail to set disklabel(%s)" % disk_name)
                    continue
            return

        def add_disk_to_system_raid(raid_mode):

            add_disk_list = self.__get_need_add_disk_list_for_create(raid_mode)
            is_fail = False
            for add_disk_name in add_disk_list:
                try:
                    self.__add_disk_to_system_raid(add_disk_name, raid_mode)
                except:
                    is_fail = True

            if is_fail:
                raise self.__CreateError

            return

        def restore_config_raid_mode(conf_node, raid_mode, vol_name):

            conf_node.get_node(CONF_RAID_LEVEL_XPATH).set_text(raid_mode)
            self.__save_config(conf_node)
            util.save_cuurent_array_num(vol_name)

        if self.__is_exist_node(entry.get(), OPTION_NAME):
            if entry.get().get_node(OPTION_NAME).is_set_to_true(RECOVERY_ATTR):

                self.__stop_raid_service()
                
                if RECOVERABLE_MIRROR_STATUS == \
                    util.get_volume_cache_value(MIRROR_USB_VOLUME, STATUS_NAME):

                    try:
                        self.__recover_form_other_mirror(entry)
                    except:
                        entry.set_result(Results.CREATE_ERROR)
                        self.__start_raid_service()
                        return entry
                    else:
                        entry.set_result(Results.OK)
                        self.__start_raid_service()
                        return entry

                try:
                    self.__recover_internal(entry)
                except:
                    entry.set_result(Results.CREATE_ERROR)
                    self.__start_raid_service()
                    return entry
                else:
                    entry.set_result(Results.OK)
                    self.__start_raid_service()
                    return entry

        if self.__dispatch_raid_create_event(previous=True) == DISPACH_FAIL:
            entry.set_result(Results.CREATE_ERROR)
            return entry

        raidctrl_node = entry.get()
        raid_level = raidctrl_node.get_value("./raid/level")
        raid_level = self.__convert_to_appropriate_raid_level(raid_level)
        if not raid_level:
            entry.set_result(Results.CREATE_ERROR)
            return entry
        vol_name = raidctrl_node.get_node(VOL_NAME_XPATH).get_text()
        raiddev_name = self.__vol_name2raiddev_name(vol_name)

        # まず使えるHDDかチェック
        try:
            check_all_disks_valid(raid_level)
        except self.__DeviceNotFound:
            self.exception("device not found and cannot create raid")
            entry.set_result(Results.DEVICE_NOT_FOUND)
            return entry
        except self.__Capshortage:
            self.exception("Capshortage and cannot create raid")
            entry.set_result(Results.CAP_SHORTAGE)
            return entry
        except self.__ErrorDiskDetected:
            self.exception("Disk error and cannot create raid")
            entry.set_result(Results.DISK_ERROR)
            return entry

        try:
            add_disk_to_system_raid(raid_level)
        except self.__CreateError:
            entry.set_result(Results.CREATE_ERROR)
            self.__dispatch_raid_update_event()
            self.__dispatch_disk_update_event()
            return entry

        self.__dispatch_pre_unmount_event(vol_name)
        self.__unmount(vol_name)

        if self.__is_internal_volume_encrypted():

            if self.__is_current_raid_mode_vfsext():
                try:
                    self.__vfs_enc_stop()
                except self.__StopError:
                    entry.set_result(Results.STOP_ERROR)
                    return entry

            try:
                self.__stop_encrypt(vol_name)
            except self.__StopError:
                entry.set_result(Results.STOP_ERROR)
                return entry

        # VFSフォーマットの場合は処理がかなり違うので特別扱い
        if VFSEXT_FSTYPE == config.RAID_MODE_TO_REAL_RAID_MODE[raid_level]:
            try:
                self.__do_stop_raid(raidctrl_node, raiddev_name)
            except self.__StopError:
                entry.set_result(Results.STOP_ERROR)
            else:
                conf_node = self.__load_config()
                conf_node.get_node(CONF_RAID_LEVEL_XPATH).set_text(raid_level)

                # 設定ファイル更新して現在のarray numを更新
                self.__save_config(conf_node)
                util.save_cuurent_array_num(vol_name)
                self.__remove_non_element_disk_form_all_raid()

                set_all_disk_label(raid_level)
                set_all_disk_status(DISK_NORMAL_STATUS)
                entry.set_result(Results.OK)
                self.__dispatch_raid_update_event()
                self.__dispatch_disk_update_event()
                self.__dispatch_raid_create_event()

            self.__save_config_share_port_mode(
                self.__get_mode_setting_vol_list(MIRRORING_PORT_MODE))

            self.__spindown_unuse_disk()
            return entry

        # 通常のRAIDモード
        cur_raid_level = None
        try:
            self.__do_stop_raid(raidctrl_node, raiddev_name)
            self.__flat_all_data_partition_size()

            # 設定ファイル更新して現在のarray numを更新
            conf_node = self.__load_config()
            cur_raid_level = \
                conf_node.get_node(CONF_RAID_LEVEL_XPATH).get_text()
            conf_node.get_node(CONF_RAID_LEVEL_XPATH).set_text(raid_level)
            self.__save_config(conf_node)
            util.save_cuurent_array_num(vol_name)

            self.__do_create_raid(raidctrl_node, raiddev_name)
            self.__remove_non_element_disk_form_all_raid()

            set_all_disk_label(raid_level)

            self.__save_config_share_port_mode(
                self.__get_mode_setting_vol_list(VFSEXT_PORT_MODE))

        except self.__CreateError:
            self.exception("create error")
            restore_config_raid_mode(conf_node, cur_raid_level, vol_name)
            entry.set_result(Results.CREATE_ERROR)
        except self.__StopError:
            self.exception("stop error")
            restore_config_raid_mode(conf_node, cur_raid_level, vol_name)
            entry.set_result(Results.STOP_ERROR)
        else:
            set_all_disk_status(DISK_NORMAL_STATUS)
            entry.set_result(Results.OK)

        self.__dispatch_raid_update_event()
        self.__dispatch_disk_update_event()
        self.__dispatch_raid_create_event()
        self.__spindown_unuse_disk()
        return entry

    def __get_raiddev_name_list_for_add_disk(self, disk_name, raid_mode=None):

        target_raiddev_list = []
        if not raid_mode:
            is_vfs_setting = self.__is_current_raid_mode_vfsext()
        else:
            is_vfs_setting = (VFSEXT_FSTYPE == \
                                 config.RAID_MODE_TO_REAL_RAID_MODE[raid_mode])

        for raiddev, element_name_list in \
                util.get_raid_element_info(raid_mode).iteritems():

            if is_vfs_setting:
                if raiddev == INTERNAL_DEV:
                    continue
                if raiddev == MIRROR_DEV:
                    continue
                if raiddev == MIRROR_RAID_SYMBL_DEV:
                    continue

            for element_name in element_name_list:

                if MISSING_DEVICE == element_name:
                    continue

                if element_name.startswith(disk_name):
                    target_raiddev_list.append(raiddev)

        return target_raiddev_list

    def __get_duplicate_src_disk_for_add_disk(self,
                                              target_raiddev_name_list,
                                              dst_disk_name):

        # RAIDデバイス名から複製元のHDD名を取得
        for target_raiddev_name in target_raiddev_name_list:

            element_dict = self.__get_element_info(target_raiddev_name)
            tmp_element_dict = copy.deepcopy(element_dict)
            for element, info in tmp_element_dict.iteritems():
                if element.startswith(dst_disk_name):
                    element_dict.pop(element)

            actual_part_name_list = self.__get_usable_elements(element_dict)

            for actual_part_name in actual_part_name_list:
                match = re.match("([a-zA-Z]+[0-9]+)p[0-9]+", actual_part_name)
                if not match:
                    continue

                src_disk_name = match.group(1)

                if dst_disk_name == src_disk_name:
                    continue

                return src_disk_name

        return ""

    def __get_raiddev_size(self, raiddev_name):

        SIZE_XPATH = "./raid/size"
        raidctrl_node = self.__send_query_request(raiddev_name)
        return int(raidctrl_node.get_value(SIZE_XPATH))

    def __get_disk_size(self, disk_name):

        SIZE_XPATH = "./disk/size"
        diskctrl_node = self.__send_query_to_diskctrl(disk_name).get()
        return int(diskctrl_node.get_node(SIZE_XPATH).get_text())

    def __get_partition_size(self, diskctrl_node, part_no):

        start_sector = int(diskctrl_node.get_value(START_XPATH_TMPL % part_no))
        end_sector = int(diskctrl_node.get_value(END_XPATH_TMPL % part_no))
        return end_sector - start_sector + 1

    def __is_disk_join_vfsext(self, disk_name):

        PARTITION_XPATH = "volume/partitionlist/partition"

        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.QUERY,
                                           VFS_QUERY_XML)
        self.debug(str(result_entry))
        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        part_node_list = result_entry.get().get_node_list(PARTITION_XPATH)
        for part_node in part_node_list:
            self.debug(str(part_node))
            part_name = part_node.get_text()
            self.debug(" part name " + part_name)
            if part_name.startswith(disk_name):
                return True
        return False

    def __is_enough_disk_size_vfsext(self, dst_disk_name):

        def get_min_size_vfsext_disk_size():

            DEFAULT_MIN_SIZE = 0

            STATUS_XPATH = "volume/status"
            PARTITION_XPATH = "volume/partitionlist/partition"

            result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                               ReqTypes.QUERY,
                                               VFS_QUERY_XML)
            if result_entry.get_result() != Results.OK:
                raise self.__RequestError

            if MOUNTED_STATUS != result_entry.get().get_value(STATUS_XPATH):
                return DEFAULT_MIN_SIZE

            part_node_list = result_entry.get().get_node_list(PARTITION_XPATH)
            min_disk_size = None
            for part_node in part_node_list:
                part_name = part_node.get_text()

                disk_size = self.__get_disk_size(
                                self.__partition2disk(part_name))

                if min_disk_size is None:
                    min_disk_size = disk_size

                elif min_disk_size > disk_size:
                    min_disk_size = disk_size

            if min_disk_size is None:
                return DEFAULT_MIN_SIZE

            return min_disk_size

        def get_vfs_disk_num(dst_disk_name):

            match = re.match(SATA_DISK_BASE_NAME + "([0-9]+)", dst_disk_name)
            if not match:
                raise self.__GetError

            return int(match.group(1))

        def get_mirror_set_partner_disk_name(dst_disk_name):

            disk_num = get_vfs_disk_num(dst_disk_name)

            # ミラーセットの番号
            set_num = 1 + (disk_num - 1) / 2

            # ミラーのペアになるディスクの番号
            par_disk_num = set_num * 2 - ((disk_num + 1) % 2)
            return SATA_DISK_BASE_NAME + str(par_disk_num)

        self.debug("__is_enough_disk_size_vfsext")

        pair_disk_name = get_mirror_set_partner_disk_name(dst_disk_name)
        self.debug("pair_disk_name " + pair_disk_name)

        # ミラーセットのペアがvfsに組み込まれている場合はそっちと比較
        # 存在しない場合は最小のディスクと比較
        if self.__is_disk_join_vfsext(pair_disk_name):
            self.debug("__is_disk_join_vfsext True")
            src_disk_size = self.__get_disk_size(pair_disk_name)
            self.debug("src_disk_size" + str(src_disk_size))
        else:
            self.debug("__is_disk_join_vfsext False")
            src_disk_size = get_min_size_vfsext_disk_size()
            self.debug("src_disk_size" + str(src_disk_size))

        dst_disk_size = self.__get_disk_size(dst_disk_name)
        self.debug("dst_disk_size" + str(dst_disk_size))
        if src_disk_size > dst_disk_size:
            self.debug("   return False")
            return False

        self.debug("   return True")
        return True

    def __get_swap_partition_size(self, disk_name):

        SWAP_PART_NUM = 3

        diskctrl_node = self.__send_query_to_diskctrl(disk_name).get()
        return self.__get_partition_size(diskctrl_node, SWAP_PART_NUM)

    def __get_reduced_swap_diskctrl_node(self, src_disk_name, added_disk_name):
        """pal_diskctrlにqueryして返ってきたdiskctrl_nodeを代入する
           関数はそのdiskctrl_nodeのswapサイズを指定されたディスクサイズ
           にフィットするように一部編集して返す
        """

        SWAP_PART_NUM = 3
        MD5_PART_NUM = 5
        DATA_PART_NUM = 6
        LEAST_SWAP_SIZE = 512 * 1024  # sector(256MB)

        diskctrl_node = self.__send_query_to_diskctrl(src_disk_name).get()
        added_disk_size = self.__get_disk_size(added_disk_name)

        swap_org_size = self.__get_partition_size(diskctrl_node, SWAP_PART_NUM)

        # md5
        total_system_size = int(diskctrl_node.get_value(
                                    END_XPATH_TMPL % MD5_PART_NUM)) + 1
        # internal
        if INTERNAL_DEV:
            internal_raiddev_size = self.__get_raiddev_size(INTERNAL_DEV)
        else:
            internal_raiddev_size = self.__get_partition_size(diskctrl_node,
                                                            DATA_PART_NUM)

        # 足りないサイズ
        not_enough_size = \
            total_system_size + internal_raiddev_size + \
                GPT_PART_START - added_disk_size

        if not_enough_size <= 0:
            return diskctrl_node

        # 縮小後のswapパーティションのサイズ
        new_swap_size = swap_org_size - not_enough_size
        new_swap_size = new_swap_size & ~(new_swap_size & 7)  # 8セクタ境界
        if new_swap_size < LEAST_SWAP_SIZE:
            raise self.__Capshortage

        reduce_size = swap_org_size - new_swap_size

        for part_num in range(SWAP_PART_NUM, DATA_PART_NUM + 1):

            start_node = diskctrl_node.get_node(START_XPATH_TMPL % part_num)
            start_sector = int(start_node.get_text())

            end_node = diskctrl_node.get_node(END_XPATH_TMPL % part_num)
            end_sector = int(end_node.get_text())

            if SWAP_PART_NUM == part_num:
                end_sector -= reduce_size
                end_node.set_text(str(end_sector))
            else:
                start_sector -= reduce_size
                end_sector -= reduce_size
                start_node.set_text(str(start_sector))
                end_node.set_text(str(end_sector))

        return diskctrl_node

    def __change_swap_size_of_partlist_node(self, partlist_node, swap_size):

        SWAP_PART_NUM = 3
        DATA_PART_NUM = 6
        START_XPATH_TMPL = "./partition[number='%d']/start"
        END_XPATH_TMPL = "./partition[number='%d']/end"

        start_node = partlist_node.get_node(START_XPATH_TMPL % SWAP_PART_NUM)
        start_sector = int(start_node.get_text())

        end_node = partlist_node.get_node(END_XPATH_TMPL % SWAP_PART_NUM)
        end_sector = int(end_node.get_text())
        new_end_sector = start_sector + swap_size - 1
        diff_size = new_end_sector - end_sector

        for part_num in range(SWAP_PART_NUM, DATA_PART_NUM + 1):

            start_node = partlist_node.get_node(START_XPATH_TMPL % part_num)
            start_sector = int(start_node.get_text())

            end_node = partlist_node.get_node(END_XPATH_TMPL % part_num)
            end_sector = int(end_node.get_text())

            if SWAP_PART_NUM == part_num:
                end_sector += diff_size
                end_node.set_text(str(end_sector))
            else:
                start_sector += diff_size
                end_sector += diff_size
                start_node.set_text(str(start_sector))
                end_node.set_text(str(end_sector))

        return partlist_node

    def __is_enough_disk_size(self, src_disk_name, dst_disk_name):

        PATR5_END_XPATH = "./disk/partitionlist/partition[number='5']/end"
        PATR6_END_XPATH = "./disk/partitionlist/partition[number='6']/end"

        if src_disk_name.startswith("usb"):
            return True

        result_entry = self.__send_query_to_diskctrl(src_disk_name)
        if Results.OK != result_entry.get_result():
            self.critical("cannot get query data for checking enough size")
            return False

        if dst_disk_name.startswith("usb"):
            if INTERNAL_DEV:
                sys_end_sector = result_entry.get().get_value(PATR5_END_XPATH)
                sys_size = int(sys_end_sector) + 1
                needed_size = sys_size + \
                                self.__get_raiddev_size(INTERNAL_DEV) + \
                                GPT_PART_START
            else:
                data_end_sector = result_entry.get().get_value(PATR6_END_XPATH)
                needed_size = int(data_end_sector) + 1

        else:
            if self.__is_current_raid_mode_vfsext():
                return self.__is_enough_disk_size_vfsext(dst_disk_name)

            data_end_sector = result_entry.get().get_value(PATR6_END_XPATH)
            needed_size = int(data_end_sector) + 1

        dst_size = self.__get_disk_size(dst_disk_name)
        if needed_size > dst_size:
            if not dst_disk_name.startswith("usb"):
                return False

            try:
                self.__get_reduced_swap_diskctrl_node(src_disk_name,
                                                      dst_disk_name)
            except:
                self.exception("fail to __get_reduced_swap_diskctrl_node")
                return False  # swapパーティションを縮小しても容量不足
            else:
                return True  # swapパーティションを縮小すれば容量足りる

        return True

    def __is_enough_disk_size_for_recovery_sata_disk(self,
                                                     usb_src_disk_name,
                                                     dst_sata_disk_name):

        src_size = self.__get_disk_size(usb_src_disk_name)
        dst_size = self.__get_disk_size(dst_sata_disk_name)

        if src_size > dst_size:
            return False
        return True

    def __is_disk_label_exist(self, disk_name):

        try:
            util.get_disk_label_info(disk_name)
        except:
            self.exception("fail to get_disk_label_info")
            return False

        return True

    def __is_mirror_volume_waiting(self):

        if WAITING_STATUS == util.get_volume_cache_value(MIRROR_USB_VOLUME,
                                                         "status"):
            return True

        return False

    def __is_volume_waiting(self, vol_name):

        if WAITING_STATUS == util.get_volume_cache_value(vol_name,
                                                         "status"):
            return True

        return False

    def __is_need_to_add_as_external_vfs_disk(self, added_disk_name):

        VFSEXT_PORT_MODE = "vfsext"

        # RAIDモードがVFSEXTか
        if not self.__is_current_raid_mode_vfsext():
            return False

        # 設定ファイル上でポートがVFSEXTか
        iface_name = config.DEV_TO_PORT_IF[added_disk_name]
        port_num = config.DEV_TO_PORT_NO[added_disk_name]

        try:
            port_mode = util.get_current_port_mode(iface_name, port_num)
        except:
            return False

        if VFSEXT_PORT_MODE != port_mode:
            return False

        # product_conf上でexternal vfsextに対応しているか
        external_vfs_disk_list = util.get_support_external_vfsext_element_list(
                                     is_get_as_disk_name=True)
        if added_disk_name not in external_vfs_disk_list:
            return False

        return True

    def __is_crash_if_remove(self, tgt_disk_name):

        RESULT_NAME = "result"
        CRASH_RESULT = "crash"
        RAID_XPATH = "result/raidlist/raid"

        RAID_CHK_CRASH_WITH_DISK_REQ_XML_TMPL = \
            """<raidctrl>
                 <option type="crash"/>
                 <disk><name>%s</name></disk>
               </raidctrl>"""

        if tgt_disk_name in config.HDD_DEV_LIST:
            req_xml = RAID_CHK_CRASH_WITH_DISK_REQ_XML_TMPL % tgt_disk_name
        else:
            raise InvalidArgError

        result_entry = \
            self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                ReqTypes.CHECK,
                                req_xml)

        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        raid_node_list = result_entry.get().get_node_list(RAID_XPATH)
        for raid_node in raid_node_list:

            result = raid_node.get_node(RESULT_NAME).get_text()

            if CRASH_RESULT == result:
                return True

        return False

    def __save_enc_flg_to_external_vfs_disk(self, disk_name, is_enc=False):

        ENCRYPT_TEXT = "encrypt"

        vol_name = self.__dev_name2vol_name(disk_name)

        if is_enc:
            req_xml = WRITE_SIG_REQ_XML_TMPL % disk_name
            util.set_volume_cache_value(vol_name, ENCRYPT_TEXT, TRUE_TEXT)
        else:
            req_xml = CLEAR_SIG_REQ_XML_TMPL % disk_name
            util.set_volume_cache_value(vol_name, ENCRYPT_TEXT, FALSE_TEXT)

        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.CONTROL,
                                           req_xml)
        if Results.OK != result_entry.get_result():
            raise self.__SaveError

        return

    def __external_vfs_add_handler(self, entry):

        DISK_NAME_XPATH = "./raid/name[@type='disk']"

        raidctrl_node = entry.get()
        added_disk_name = raidctrl_node.get_node(DISK_NAME_XPATH).get_text()

        if util.is_errorcount_over(added_disk_name):
            self.error("errcount over detected(%s)" % added_disk_name)
            util.set_disk_info(added_disk_name,
                               DISK_STATUS,
                               DISK_FAIL_STATUS)
            entry.set_result(Results.DISK_ERROR)
            return entry

        # 容量確認
        if not util.is_enough_disk_size_vfsext(added_disk_name):

            util.set_disk_info(added_disk_name,
                               DISK_STATUS,
                               DISK_CAPSHORTAGE_STATUS)

            entry.set_result(Results.CAP_SHORTAGE)
            return entry

        if not self.__is_volume_waiting(
                   self.__dev_name2vol_name(added_disk_name)):

            if not self.__is_disk_label_exist(added_disk_name):
                if util.is_volume_exist_any_files(
                    config.DEV_TO_VOL[added_disk_name]):
                    # waiting mode
                    entry.set_result(Results.WAITING)
                    return entry

        if not util.is_uuid_same_to_cur_vfsext_element(added_disk_name):
            is_need_make_part = True
        else:
            is_need_make_part = False

        # フォーマットしてVFSEXTに追加
        element_name = \
            util.disk_name2external_vfs_element_name(added_disk_name)

        # パーティションを切る
        try:
            if is_need_make_part:
                self.__make_partition_for_external_vfs(added_disk_name)
        except:
            self.exception("fail to mkfs_vfs_partition")
            entry.set_result(Results.MKFS_ERROR)
            util.set_disk_info(added_disk_name,
                               DISK_STATUS,
                               DISK_FAIL_STATUS)
            return entry

        try:
            is_enc = self.__is_internal_volume_encrypted()

            self.__save_enc_flg_to_external_vfs_disk(added_disk_name, is_enc)

            if is_enc:
                enc_part_name = DEV_TO_ENC_DEV[element_name]

                self.__do_enc_start_partition(element_name,
                                              enc_part_name)
                if is_need_make_part:
                    self.__mkfs_vfs_partition(enc_part_name, True)
            else:
                if is_need_make_part:
                    self.__mkfs_vfs_partition(element_name)

            self.__vfs_add_partition(element_name, is_enc)

            util.set_disk_info(added_disk_name,
                               DISK_STATUS,
                               DISK_NORMAL_STATUS)

            entry.set_result(Results.OK)
            return entry

        except self.__MkfsError:
            self.exception("fail to mkfs_vfs_partition")
            entry.set_result(Results.MKFS_ERROR)
            util.set_disk_info(added_disk_name,
                               DISK_STATUS,
                               DISK_FAIL_STATUS)
            return entry
        except self.__SaveError:
            self.exception("fail to mkfs_vfs_partition")
            entry.set_result(Results.SAVE_ERROR)
            util.set_disk_info(added_disk_name,
                               DISK_STATUS,
                               DISK_FAIL_STATUS)
            return entry

        entry.set_result(Results.ADD_ERROR)
        return entry

    def __standard_add_handler(self, entry):
        """Handle "add" request."""

        DISK_NAME_XPATH = "./raid/name[@type='disk']"

        raidctrl_node = entry.get()
        added_disk_name = raidctrl_node.get_node(DISK_NAME_XPATH).get_text()

        if util.is_errorcount_over(added_disk_name):
            self.error("errcount over detected(%s)" % added_disk_name)
            util.set_disk_info(added_disk_name,
                               DISK_STATUS,
                               DISK_FAIL_STATUS)
            entry.set_result(Results.DISK_ERROR)
            return entry

        if util.is_smart_error(added_disk_name):
            self.error("smart error detected(%s)" % added_disk_name)
            util.set_disk_info(added_disk_name,
                               DISK_STATUS,
                               DISK_SMARTERR_STATUS)
            entry.set_result(Results.DISK_ERROR)
            return entry

        is_mirror_hdd = self.__is_usb_mirror_element(added_disk_name)
        if is_mirror_hdd:
            if not self.__is_mirror_volume_waiting():
                if not self.__is_disk_label_exist(added_disk_name):
                    # USBミラーの場合は
                    if util.is_volume_exist_any_files(
                        config.DEV_TO_VOL[added_disk_name]):
                        # waiting mode
                        entry.set_result(Results.WAITING)
                        return entry

        # ADD先のRAIDデバイスのリスト取得
        tgt_raiddev_name_list = \
            self.__get_raiddev_name_list_for_add_disk(added_disk_name)

        # 既にADD先にADDされている変な状況の場合はエラー終了
        for raiddev_name in tgt_raiddev_name_list:
            cur_element_info = self.__get_element_info(raiddev_name, True)
            for part_name, info in cur_element_info.iteritems():

                if not part_name.startswith(added_disk_name):
                    continue

                if not info[JOIN_ATTR]:
                    continue

                # 対象のRAIDに既に指定されたディスクが存在していた
                self.critical("adddisk already exist in target raid !!")
                entry.set_result(Results.ALREADY_EXIST)
                return entry

        # まずHDDのパーティションを複製するための複製元HDDを取得
        src_disk_name = \
            self.__get_duplicate_src_disk_for_add_disk(tgt_raiddev_name_list,
                                                       added_disk_name)
        if not src_disk_name:
            entry.set_result(Results.ADD_ERROR)
            return entry

        if not self.__is_enough_disk_size(src_disk_name, added_disk_name):

            util.set_volume_cache_value(
                self.__dev_name2vol_name(added_disk_name),
                STATUS_NAME,
                CAPSHORTAGE_STATUS)
            entry.set_result(Results.CAP_SHORTAGE)
            return entry

        # swapパーティションが存在する状態でシステム起動した場合は
        # swapが動作しているのでとりあえずswap off
        try:
            self.__swap_off_added_disk(added_disk_name)
        except:
            pass

        is_max_end_part = \
            self.__is_need_max_size_partition(src_disk_name, added_disk_name)

        # 取得した複製元HDDのパーティション構成を追加されたHDDに複製
        try:
            self.__duplicate_disk_partition(src_disk_name,
                                            added_disk_name,
                                            is_max_end_part)
        except self.__MkpartError:
            entry.set_result(Results.DISK_ERROR)
            return entry

        # swap追加
        try:
            self.__swap_on_added_disk(added_disk_name,
                                      self.__is_internal_volume_encrypted())
        except:
            self.exception(("fail to __swap_on_added_disk(%s)" +
                           " - do nothing") % added_disk_name)

        is_add_fail = False
        # 全ターゲットRAIDデバイスにADDしていく
        for tgt_raiddev_name in tgt_raiddev_name_list:

            raid_status = self.__get_raid_query_status(tgt_raiddev_name)
            if raid_status in [CRASH_STATUS, STOP_STATUS]:
                continue

            element_name_list = self.__raiddev2elements(tgt_raiddev_name)
            for element_name in element_name_list:
                if element_name.startswith(added_disk_name):
                    try:
                        self.__send_add_request(tgt_raiddev_name, element_name)
                    except self.__AddError:
                        is_add_fail = True
                        continue

        if self.__is_current_raid_mode_vfsext():

            try:
                vfs_part_name = None
                for element, info in util.get_vfsext_dev_info().iteritems():
                    if element.startswith(added_disk_name):
                        vfs_part_name = element
                        break
                else:
                    raise NotFoundError

                is_enc = self.__is_internal_volume_encrypted()
                if is_enc:

                    enc_part_name = DEV_TO_ENC_DEV[vfs_part_name]

                    self.__do_enc_start_partition(vfs_part_name,
                                                  enc_part_name)

                    self.__mkfs_vfs_partition(enc_part_name, True)
                else:
                    self.__mkfs_vfs_partition(vfs_part_name)

                self.__vfs_add_partition(vfs_part_name, is_enc)

            except self.__MkfsError:
                self.exception("fail to mkfs_vfs_partition")
                entry.set_result(Results.MKFS_ERROR)
                return entry

        if is_add_fail:
            entry.set_result(Results.ADD_ERROR)
        else:
            util.set_disk_info(added_disk_name,
                               DISK_STATUS,
                               DISK_NORMAL_STATUS)

            entry.set_result(Results.OK)

        return entry

    def __add_handler(self, entry):
        """Handle "add" request."""

        DISK_NAME_XPATH = "./raid/name[@type='disk']"

        raidctrl_node = entry.get()
        added_disk_name = raidctrl_node.get_node(DISK_NAME_XPATH).get_text()

        if added_disk_name.startswith("usb"):
            vol_name = self.__dev_name2vol_name(added_disk_name)

            vol_status = util.get_volume_cache_value(vol_name, STATUS_NAME)
            if vol_status == OTHER_MIRROR_STATUS:
                entry.set_result(Results.ADD_ERROR)
                return entry

            if vol_status == RECOVERABLE_MIRROR_STATUS:
                (is_mirrordisk, is_recoverable, is_match_system) = \
                    util.check_mirrordisk(self.__vol_name2dev_name(vol_name))
                if not is_match_system:
                    entry.set_result(Results.ADD_ERROR)
                    return entry

        self.__save_user_remove_flag(added_disk_name, False)

        if self.__is_need_to_add_as_external_vfs_disk(added_disk_name):
            return self.__external_vfs_add_handler(entry)

        return self.__standard_add_handler(entry)

    def __is_disk_already_removed(self, disk_name):

        return not util.is_disk_exist_in_raid(disk_name)

    def __sub_remove_handler_for_partition(self, entry, raid_name, part_name):

        if self.__is_disk_already_removed(self.__partition2disk(part_name)):
            return Results.ALREADY_REMOVED

        if self.__is_current_raid_mode_vfsext() and \
               part_name in util.get_vfsext_dev_info().keys():

            try:
                self.__remove_vfs_partition(part_name)
            except:
                self.exception("fail to __remove_vfs_partition()")
            return Results.OK

        if not self.__is_partition_exist_in_raid(raid_name, part_name):
            return Results.OK

        return self.send_entry(entry,
                               pal_info.PAL_RAIDCTRL_PATH).get_result()

    def __sub_remove_handler_for_disk(self, entry, disk_name):

        if self.__is_disk_already_removed(disk_name):
            if self.__is_current_raid_mode_vfsext():
                if not self.__is_disk_join_vfsext(disk_name):
                    return Results.ALREADY_REMOVED
                else:
                    pass
            else:
                return Results.ALREADY_REMOVED

        # まずswapパーティションをOFFにしとく
        try:
            self.__swap_off_added_disk(disk_name)

            enc_swap_part_name = \
                DEV_TO_ENC_DEV.get(disk_name + SWAP_PART_SUFFIX)
            if enc_swap_part_name:
                if self.__is_device_exist(enc_swap_part_name):
                    self.__do_encrypt_stop(enc_swap_part_name)

        except:
            self.exception("fail to swap remove handling(%s)" %
                           disk_name)

        if self.__is_current_raid_mode_vfsext():
            try:
                vfs_element_name = self.__disk_name2vfs_element_name(disk_name)
                self.__remove_vfs_partition(vfs_element_name)
            except:
                self.exception("fail to __remove_vfs_partition()")
            # md2,5からもremoveするのでreturnしない

        if not self.__is_disk_exist_in_raid(disk_name):
            return Results.OK

        return self.send_entry(entry,
                               pal_info.PAL_RAIDCTRL_PATH).get_result()

    def __remove_handler(self, entry):

        disk_name = entry.get().get_value("./disk/name", None)
        part_name = entry.get().get_value("./raid/partition/name", None)
        raid_name = entry.get().get_value("./raid/name[@type='raid']", None)

        try:
            if entry.get().get_node(OPTION_NAME).is_set_to_true(USER_ATTR):
                is_user = True
            else:
                is_user = False
        except NotFoundError:
            is_user = False

        if part_name and raid_name:
            try:
                result = self.__sub_remove_handler_for_partition(entry,
                                                                 raid_name,
                                                                 part_name)
                entry.set_result(result)
                return entry

            except self.__RemoveError:
                self.exception("fail to" +
                               "__sub_remove_handler_for_partition" +
                               "(%s %s)" % (raid_name, part_name))
                entry.set_result(Results.DELETE_ERROR)
                return entry

        if disk_name:
            if is_user and not self.__is_current_raid_mode_vfsext():
                entry.set_result(Results.DELETE_ERROR)
                return entry

            # CGIからHDD取り外しなのかどうかのフラグを保存
            self.__save_user_remove_flag(disk_name, is_user)

            try:
                result = self.__sub_remove_handler_for_disk(entry, disk_name)
                entry.set_result(result)
                return entry

            except self.__RemoveError:
                self.exception("fail to" +
                               "__sub_remove_handler_for_disk(%s)" %
                               disk_name)
                entry.set_result(Results.DELETE_ERROR)
                return entry

        entry.set_result(Results.INVALID_XML)
        return entry

    @falcmd.conf_change_handler_wrapper
    def __activerepair_handler(self, entry):
        """Handle "activerepair" request."""

        op = entry.get_options()[OPERATION_OPT]
        if SCHEDULE_OPT_VAL == op:

            # pal_raidctrlでcronに登録させる
            result_entry = self.send_entry(entry, pal_info.PAL_RAIDCTRL_PATH)
            if Results.OK != result_entry.get_result():
                return result_entry

            # 成功したら設定ファイルに保存
            conf_node = self.__load_config()
            conf_raidctrl_node = XmlNode(RAIDCTRL_NAME)
            conf_raidctrl_node.append_child(
                entry.get().get_node(ACTIVEREPAIR_NAME))
            conf_node.update(conf_raidctrl_node)

            result_save_entry = self.__save_config(conf_node)
            if Results.OK != result_save_entry.get_result():
                entry.set_result(result_save_entry.get_result())
                return entry

            entry.set_result(Results.OK)
            return entry

        # Activerepair実行
        return self.send_entry(entry,
                               pal_info.PAL_RAIDCTRL_PATH,
                               preserve_file=True)

    def __chk_exist_in_vfs_element(self, entry):

        PARTITION_XPATH = "./volume/partitionlist/partition"

        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.QUERY,
                                           VFS_QUERY_XML)
        if result_entry.get_result() != Results.OK:
            # 想定外
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        disk_name = entry.get().get_value("./disk/name", None)
        part_name = entry.get().get_value("./partition/name", None)

        if not disk_name and not part_name:
            entry.set_result(Results.INVALID_XML)
            return entry

        # 実際の構成パーティションに指定デバイスが存在するかチェック
        part_node_list = result_entry.get().get_node_list(PARTITION_XPATH)
        for part_node in part_node_list:
            actual_element_name = part_node.get_text()

            if disk_name is not None:
                if actual_element_name.startswith(disk_name):  # 存在した
                    entry.set_result(Results.OK)
                    return entry
                else:
                    continue

            if part_name is not None:
                if actual_element_name == part_name:  # 存在した
                    entry.set_result(Results.OK)
                    return entry
                else:
                    continue

        # どの構成デバイスにも該当しなかった
        entry.set_result(Results.DEVICE_NOT_FOUND)
        return entry

    def __chk_exist_handler(self, entry):

        result_entry = self.send_entry(entry, pal_info.PAL_RAIDCTRL_PATH)

        if result_entry.get_result() == Results.DEVICE_NOT_FOUND:
            if self.__is_current_raid_mode_vfsext():
                # system RAIDには存在しないがvfsの構成デバイスには
                # 存在するかもしれない
                return self.__chk_exist_in_vfs_element(entry)

        return result_entry

    def __check_crash(self, entry):

        RAIDLIST_NAME = "raidlist"
        RESULT_NAME = "result"
        CRASH_RESULT = "crash"
        NO_CRASH_RESULT = "nocrash"
        PART_NAME_XPATH = "volume/partitionlist/partition"
        RESULT_XML_TMPL = \
            "<raid><name>%(name)s</name><result>%(result)s</result></raid>"

        result_entry = self.send_entry(entry, pal_info.PAL_RAIDCTRL_PATH)

        # 拡張ボリュームの場合は結果に追加
        if self.__is_current_raid_mode_vfsext():

            tgt_disk_nmame = entry.get().get_value("./disk/name", None)
            if tgt_disk_nmame is None:
                return result_entry

            vfs_result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                                   ReqTypes.QUERY,
                                                   VFS_QUERY_XML)
            try:
                part_node_list = \
                    vfs_result_entry.get().get_node_list(PART_NAME_XPATH)

                element_disk_name_list = \
                    [self.__partition2disk(node.get_text()) \
                        for node in part_node_list]

                config_raid_level = self.__get_config_raid_level()

                if tgt_disk_nmame not in element_disk_name_list:
                    return result_entry
                elif len(part_node_list) <= \
                         util.get_raid_min_num(config_raid_level):
                    raid_status = CRASH_RESULT
                else:
                    raid_status = NO_CRASH_RESULT

                for raid_name in [MIRROR_DEV, INTERNAL_DEV]:
                    result_node = \
                        XmlNode(RESULT_XML_TMPL % {"name": raid_name,
                                                   "result": raid_status})
                    raidlist_node = result_entry.get().get_node(
                                        RESULT_NAME).get_node(RAIDLIST_NAME)
                    raidlist_node.append_child(result_node)
            except:
                self.exception("fail to get vfs element.")
                return result_entry

        return result_entry

    def __check_operation(self, entry):

        return self.send_entry(entry, pal_info.PAL_RAIDCTRL_PATH)

    def __check_change_mode(self, entry):

        VOL_NAME_XPATH = "./name[@type='volume']"
        CONF_RAID_LEVEL_XPATH = "raidlist/raid/level"

        raid_node = entry.get().get_node(RAID_NAME)
        vol_name = raid_node.get_node(VOL_NAME_XPATH).get_text()
        raid_level = raid_node.get_value(LEVEL_NAME)

        conf_node = self.__load_config()
        cur_raid_level = conf_node.get_value(CONF_RAID_LEVEL_XPATH)
        if not config.RAID_MODE_INFO[cur_raid_level]["is_ext_raid"]:
            entry.set_result(Results.CHANGE_ERROR)
            return entry

        if not config.RAID_MODE_INFO[raid_level]["is_ext_raid"]:
            entry.set_result(Results.CHANGE_ERROR)
            return entry

        if not self.__is_able_to_change_mode(cur_raid_level, raid_level):
            entry.set_result(Results.CHANGE_ERROR)
            return entry
        entry.set_result(Results.OK)
        return entry

    def __check_add_disk(self, entry):

        DISK_NAME_XPATH = "./disk/name"
        LEVEL_XPATH = "./raid/level"

        raidctrl_node = entry.get()
        added_disk_name = raidctrl_node.get_node(DISK_NAME_XPATH).get_text()
        raid_level = raidctrl_node.get_value(LEVEL_XPATH, None)

        # errorカウントチェック
        if util.is_errorcount_over(added_disk_name):
            entry.set_result(Results.DISK_ERROR)
            return entry

        # smartチェック
        if util.is_smart_error(added_disk_name):
            entry.set_result(Results.DISK_ERROR)
            return entry

        # 容量チェック
        tgt_raiddev_name_list = \
            self.__get_raiddev_name_list_for_add_disk(added_disk_name,
                                                      raid_level)

        src_disk_name = \
            self.__get_duplicate_src_disk_for_add_disk(tgt_raiddev_name_list,
                                                       added_disk_name)
        if not src_disk_name:
            entry.set_result(Results.ADD_ERROR)
            return entry

        if not self.__is_enough_disk_size(src_disk_name, added_disk_name):
            entry.set_result(Results.CAP_SHORTAGE)
            return entry

        entry.set_result(Results.OK)
        return entry

    def __check_other_mirror(self, entry):

        RESULT_XML_TMPL = \
            "<result>%(result)s</result>"
        VOLUME_NAME_XPATH = "./volume/name"

        RESULT_NO_MIRROR = "nomirror"

        raidctrl_node = entry.get()
        vol_name = raidctrl_node.get_node(VOLUME_NAME_XPATH).get_text()

        (is_mirrordisk, is_recoverable, is_match_system) = \
            util.check_mirrordisk(self.__vol_name2dev_name(vol_name))
        
        result = RESULT_NO_MIRROR
        if is_mirrordisk:
            if is_recoverable:
                result = RECOVERABLE_MIRROR_STATUS
            else:
                result = OTHER_MIRROR_STATUS

        entry.get().append_child(XmlNode(RESULT_XML_TMPL % {"result": result}))
        entry.set_result(Results.OK)
        return entry
  
    def __check_handler(self, entry):

        chk_funcs = {CRASH_ATTR_VAL: self.__check_crash,
                     GET_OPERATION_ATTR_VAL: self.__check_operation,
                     CHANGE_MODE_ATTR_VAL: self.__check_change_mode,
                     ADD_DISK_ATTR_VAL: self.__check_add_disk,
                     OTHER_MIRROR_ATTR_VAL: self.__check_other_mirror}

        try:
            chk_type = entry.get_options()[TYPE_OPT_VAL]
        except:
            chk_type = entry.get_options()[OPERATION_OPT]

        try:
            return chk_funcs[chk_type](entry.get_clone())
        except:
            entry.set_result(Results.CHECK_ERROR)

        return entry

    def __getraidmodelist_handler(self, entry):

        VOL_NAME_XPATH = "./name[@type='volume']"
        raid_node = entry.get().get_node(RAID_NAME)
        vol_name = raid_node.get_node(VOL_NAME_XPATH).get_text()

        max_array_num = util.get_max_array_num()

        array_num_list = config.VOL_INFO[vol_name]["mode_list"].keys()
        if max_array_num not in array_num_list:
            max_array_num = min(array_num_list)

        modelist = config.VOL_INFO[vol_name]["mode_list"][max_array_num]

        modelist_node = XmlNode(MODELIST_NAME)
        for mode in modelist:
            modelist_node.append_child(MODE_NAME, mode)

        raid_node.append_child(modelist_node)
        entry.set_result(Results.OK)
        return entry

    def __extend_handler(self, entry):

        VOL_NAME_XPATH = "./name[@type='volume']"
        CONF_RAID_LEVEL_XPATH = "raidlist/raid/level"

        raid_node = entry.get().get_node(RAID_NAME)
        vol_name = raid_node.get_node(VOL_NAME_XPATH).get_text()
        raid_level = raid_node.get_value(LEVEL_NAME)

        try:
            result_code = self.__do_extend_disk(raid_level)
            if result_code != Results.OK:
                entry.set_result(result_code)
                return entry
        except:
            self.exception("fail to __do_extend_disk...")
            entry.set_result(Results.EXTEND_ERROR)
            return entry
        finally:
            self.__spindown_unuse_disk()

        entry.set_result(Results.OK)
        return entry

    def __apply_handler(self, entry):

        RAID_XPATH = "raidlist/raid"
        VOL_NAME_XPATH = "./name[@type='volume']"
        CONF_RAID_XPATH_TMPL = \
            "raidlist/raid/name[@type='volume'][text()='%s']/.."

        conf_node = self.__load_config()
        raid_node_list = entry.get().get_node_list(RAID_XPATH)

        for raid_node in raid_node_list:
            vol_name = raid_node.get_value(VOL_NAME_XPATH)
            is_auto_extend = raid_node.is_set_to_true(AUTO_EXTEND_ATTR)
            conf_node.get_node(CONF_RAID_XPATH_TMPL % vol_name).set_bool_attr(
                AUTO_EXTEND_ATTR, is_auto_extend)

        self.__save_config(conf_node)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_save_handler = lambda entry: self.send_entry_with_file(entry,
                                pal_info.PAL_CONFFILE_PATH,
                                config.RAIDCTRL_CONF_FILE)

        self.set_handler(ReqTypes.LOAD, load_save_handler)
        self.set_handler(ReqTypes.SAVE, load_save_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.CREATE, self.__create_handler)
        self.set_handler(ReqTypes.START, self.__start_handler)
        self.set_handler(ReqTypes.STOP, self.__stop_handler)
        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.ACTIVEREPAIR, self.__activerepair_handler)
        self.set_handler(ReqTypes.REMOVE, self.__remove_handler)
        self.set_handler(ReqTypes.CHK_EXIST, self.__chk_exist_handler)
        self.set_handler(ReqTypes.CHECK, self.__check_handler)
        self.set_handler(ReqTypes.GETRAIDMODELIST,
                         self.__getraidmodelist_handler)
        self.set_handler(ReqTypes.EXTEND, self.__extend_handler)

#
# Main
#

if __name__ == "__main__":
    FalRaid(option=falcmd.FalCmdOption()).handle_request()
