#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Replication Settings."""


import sys
import os

import iobase.xmldata as xmldata
import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info
from fal_info import ReqTypes, Results
from iobase.exception import NotFoundError, InvalidArgError


import fal_config as config

REPORT_DATA_PATH = os.path.join(config.CONF_DIR, 'replication', 'report')


class FalReplication(falcmd.FalCmd):
    """FAL Replication command implementation class."""

    def __report_handler(self, entry):
        """
        <entry type="report">
            <replication>
                <sharelist>
                    <share>
                        <name>disk1</name>
                        <timestamp>1234567890</timestamp>
                        <result>0</result>
                    </share>
                </sharelist>
            </replication>
        </entry>
        """
        entry = entry.get_clone()

        NAME = 'name'
        TIMESTAMP = 'timestamp'  # NOTE epoch
        RESULT = 'result'

        entry.set_result(Results.INTERNAL_ERROR)
        replication_node = entry.get()

        sharelist = dict()
        # 現在データ取得
        for share_node in self.__read_handler(entry).get().get_node_list(
                'sharelist/share'):
            name = share_node.get_value(NAME, None)
            timestamp = share_node.get_value(TIMESTAMP)
            result = share_node.get_value(RESULT)
            if name is None:
                raise InvalidArgError('Share name is empty!!')
            sharelist[name] = dict({
                NAME: name,
                TIMESTAMP: timestamp,
                RESULT: result,
            })
        # リクエストデータをマージ
        for share_node in replication_node.get_node_list('sharelist/share'):
            name = share_node.get_value(NAME, None)
            timestamp = share_node.get_value(TIMESTAMP)
            result = share_node.get_value(RESULT)
            if name is None:
                raise InvalidArgError('Share name is empty!!')
            # FIXME 共有フォルダーの存在確認をするか?
            sharelist[name] = dict({
                NAME: name,
                TIMESTAMP: timestamp,
                RESULT: result,
            })

        _replication_node = xmldata.XmlNode(
                """<replication><sharelist /></replication>""")
        _sharelist_node = _replication_node.get_node('sharelist')
        for share in sharelist.values():
            _share_node = xmldata.XmlNode('<share />')
            _share_node.append_child(NAME, text=share[NAME])
            _share_node.append_child(TIMESTAMP, text=share[TIMESTAMP])
            _share_node.append_child(RESULT, text=share[RESULT])
            _sharelist_node.append_child(_share_node)
        if Results.OK == self.send_entry_with_file(
                request.Entry(ReqTypes.SAVE, _replication_node),
                pal_info.PAL_CONFFILE_PATH,
                REPORT_DATA_PATH,
                ).get_result():
            entry.set_result(Results.OK)
        else:
            entry.set_result(Results.INTERNAL_ERROR)

        return entry

    def __read_handler(self, entry):
        """
        <entry type="read" result="0">
            <replication>
                <sharelist>
                    <share>
                        <name>disk1</name>
                        <timestamp>1234567890</timestamp>
                        <result>0</result>
                    </share>
                </sharelist>
            </replication>
        </entry>
        """
        entry = entry.get_clone()

        NAME = 'name'
        TIMESTAMP = 'timestamp'  # NOTE epoch
        RESULT = 'result'

        _entry = self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                REPORT_DATA_PATH,
                )
        if Results.OK != _entry.get_result():
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        _replication_node = _entry.get()
        _sharelist = dict()
        for _share_node in _replication_node.get_node_list('sharelist/share'):
            _name = _share_node.get_value(NAME, None)
            _timestamp = _share_node.get_value(TIMESTAMP)
            _result = _share_node.get_value(RESULT)
            if _name is None:
                raise InvalidArgError('Share name is empty!!')
            _sharelist[_name] = dict({
                NAME: _name,
                TIMESTAMP: _timestamp,
                RESULT: _result,
            })

        replication_node = xmldata.XmlNode(
                """<replication><sharelist /></replication>""")
        sharelist_node = replication_node.get_node('sharelist')
        for _share in _sharelist.values():
            share_node = xmldata.XmlNode('<share />')
            share_node.append_child(NAME, text=_share[NAME])
            share_node.append_child(TIMESTAMP, text=_share[TIMESTAMP])
            share_node.append_child(RESULT, text=_share[RESULT])
            sharelist_node.append_child(share_node)
        entry.set(replication_node)

        entry.set_result(Results.OK)
        return entry

    def __delete_handler(self, entry):
        """
        <entry type="delete">
            <replication>
                <sharelist>
                    <share>
                        <name>disk1</name>
                    </share>
                </sharelist>
            </replication>
        </entry>
        """
        NAME = 'name'
        TIMESTAMP = 'timestamp'  # NOTE epoch
        RESULT = 'result'
        entry = entry.get_clone()

        # リクエストデータ取得
        sharelist = dict()
        replication_node = entry.get()
        for share_node in replication_node.get_node_list('sharelist/share'):
            name = share_node.get_value(NAME, None)
            if name is None:
                raise InvalidArgError('Share name is empty!!')
            sharelist[name] = dict({
                NAME: name,
            })

        # 現在データ取得
        _sharelist = dict()
        for share_node in self.__read_handler(entry).get().get_node_list(
                'sharelist/share'):
            name = share_node.get_value(NAME, None)
            timestamp = share_node.get_value(TIMESTAMP)
            result = share_node.get_value(RESULT)
            if name is None:
                raise InvalidArgError('Share name is empty!!')
            _sharelist[name] = dict({
                NAME: name,
                TIMESTAMP: timestamp,
                RESULT: result,
            })

        # データ削除
        for name in sharelist.keys():
            if name in _sharelist.keys():
                del _sharelist[name]

        # 保存
        _replication_node = xmldata.XmlNode(
                """<replication><sharelist /></replication>""")
        _sharelist_node = _replication_node.get_node('sharelist')
        for _share in _sharelist.values():
            _share_node = xmldata.XmlNode('<share />')
            _share_node.append_child(NAME, text=_share[NAME])
            _share_node.append_child(TIMESTAMP, text=_share[TIMESTAMP])
            _share_node.append_child(RESULT, text=_share[RESULT])
            _sharelist_node.append_child(_share_node)
        if Results.OK == self.send_entry_with_file(
                request.Entry(ReqTypes.SAVE, _replication_node),
                pal_info.PAL_CONFFILE_PATH,
                REPORT_DATA_PATH,
                ).get_result():
            entry.set_result(Results.OK)
        else:
            entry.set_result(Results.INTERNAL_ERROR)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.REPLICATION_CONF_FILE)

        pass_handler = lambda entry: self.send_entry(entry,
                                            pal_info.PAL_REPLICATION_PATH)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.CHECK, pass_handler)
        self.set_handler(ReqTypes.SET, pass_handler)
        self.set_handler(ReqTypes.UNSET, pass_handler)
        self.set_handler(ReqTypes.SUSPEND, pass_handler)
        self.set_handler(ReqTypes.RESUME, pass_handler)
        self.set_handler('feature', pass_handler)
        self.set_handler('report', self.__report_handler)
        self.set_handler('read', self.__read_handler)
        self.set_handler('delete', self.__delete_handler)

#
# Main
#

if __name__ == "__main__":
    FalReplication(option=falcmd.FalCmdOption()).handle_request()
