#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for controlling the service."""


import os

import ionas.nascmd as nascmd
import ionas.request as request
from iobase.exception import (InvalidArgError,)

import libfal.falcmd as falcmd
import libfal.util as util
import fal_config as config
import pal_info
from fal_info import (ReqTypes, Results)


SERVICECTRL_NAME = "servicectrl"

CATEGORY_NAME_PATH = "./categorylist/category/name"
SERVICELIST_NAME = "servicelist"
SERVICE_PATH = "./servicelist/service"
SERVICE_NAME_PATH = SERVICE_PATH + "/name"
NAME_NAME = "name"


class FalServiceCtrl(falcmd.FalCmd):
    """FAL Service Control command implementation class."""

    def __check_body_node(self, node):
        """Check if entry's body node is valid."""

        if node.get_node_name() != SERVICECTRL_NAME:
            raise InvalidArgError(node.get_node_name())

    def __load_config(self):
        """Load config setting and return XML data ."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                        pal_info.PAL_CONFFILE_PATH,
                                        config.SERVICECTRL_CONF_FILE).get()

    def __save_config(self, config_xml):
        """Save the "config_xml" XML data."""

        entry = request.Entry(ReqTypes.SAVE)
        entry.set(config_xml)
        return self.send_entry_with_file(entry,
                                pal_info.PAL_CONFFILE_PATH,
                                config.SERVICECTRL_CONF_FILE)

    def __get_category_list(self, entry):
        """Return category name list specified by "entry"."""

        return entry.get().get_value_list(CATEGORY_NAME_PATH)

    def __get_service_list(self, entry):
        """Return service name list specified by "entry"."""

        return entry.get().get_value_list(SERVICE_NAME_PATH)

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle "apply" request and return the result entry."""

        self.__check_body_node(entry.get())

        config_xml = self.__load_config()
        service_setting_list = config_xml.get_node_list(SERVICE_PATH)
        service_setting = dict(zip([service.get_node(NAME_NAME).get_text()
                                    for service in service_setting_list],
                                service_setting_list))

        for service in entry.get().get_node_list(SERVICE_PATH):
            service_name = service.get_node(NAME_NAME).get_text()
            if not service_name in service_setting:
                # Ignore service of which setting is not in config file
                continue

            if service.is_enabled():
                service_setting[service_name].enable(True)
            else:
                service_setting[service_name].enable(False)

        self.__save_config(config_xml)
        entry.set_result(Results.OK)
        return entry

    def __query_handler(self, entry):
        """Handle "query" request and set entry to result."""

        self.__check_body_node(entry.get())

        try:
            query_result = util.servicectrl_query(
                                    self.__get_category_list(entry),
                                    self.__get_service_list(entry),
                                    self.dry_run, self.debug_on)

            try:
                entry.get().delete_child(SERVICELIST_NAME)
            except:
                pass

            config_xml = self.__load_config()
            service_setting_list = config_xml.get_node_list(SERVICE_PATH)
            service_setting = dict(zip([service.get_node(NAME_NAME).get_text()
                                    for service in service_setting_list],
                                service_setting_list))

            for service in query_result.get_node_list(SERVICE_PATH):
                service_name = service.get_node(NAME_NAME).get_text()

                if not service_name in service_setting:
                    raise InvalidArgError(service_name)

                if service_setting[service_name].is_enabled():
                    service.enable(True)
                else:
                    service.enable(False)

            entry.get().append_child(query_result.get_node(SERVICELIST_NAME))
            entry.set_result(Results.OK)
        except:
            self.exception("Query error")
            entry.set_result(Results.QUERY_ERROR)

        return entry

    def __start_handler(self, entry):
        """Handle "start" request and set entry to result."""

        self.__check_body_node(entry.get())

        try:
            util.servicectrl_start(self.__get_category_list(entry),
                                    self.__get_service_list(entry),
                                    self.dry_run, self.debug_on)
            entry.set_result(Results.OK)
        except:
            self.exception("Start error")
            entry.set_result(Results.START_ERROR)

        return entry

    def __stop_handler(self, entry):
        """Handle "stop" request and set entry to result."""

        self.__check_body_node(entry.get())

        try:
            util.servicectrl_stop(self.__get_category_list(entry),
                                    self.__get_service_list(entry),
                                    self.dry_run, self.debug_on)
            entry.set_result(Results.OK)
        except:
            self.exception("Stop error")
            entry.set_result(Results.STOP_ERROR)

        return entry

    def __suspend_handler(self, entry):
        """Handle "suspend" request and set entry to result."""

        self.__check_body_node(entry.get())

        try:
            util.servicectrl_suspend(self.__get_category_list(entry),
                                    self.__get_service_list(entry),
                                    self.dry_run, self.debug_on)
            entry.set_result(Results.OK)
        except:
            self.exception("Suspend error")
            entry.set_result(Results.SUSPEND_ERROR)

        return entry

    def __resume_handler(self, entry):
        """Handle "resume" request and set entry to result."""

        self.__check_body_node(entry.get())

        try:
            util.servicectrl_resume(self.__get_category_list(entry),
                                    self.__get_service_list(entry),
                                    self.dry_run, self.debug_on)
            entry.set_result(Results.OK)
        except:
            self.exception("Resume error")
            entry.set_result(Results.RESUME_ERROR)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.SERVICECTRL_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.START, self.__start_handler)
        self.set_handler(ReqTypes.STOP, self.__stop_handler)
        self.set_handler(ReqTypes.SUSPEND, self.__suspend_handler)
        self.set_handler(ReqTypes.RESUME, self.__resume_handler)


#
# Main
#
if __name__ == "__main__":
    FalServiceCtrl(option=falcmd.FalCmdOption()).handle_request()
