#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Share Folder Settings."""

import os

import ionas.request as request
import ionas.nascmd as nascmd
from iobase.xmldata import XmlConfNode, XmlNode
import libfal.falcmd as falcmd
from fal_info import ReqTypes
from fal_info import Results
import pal_info

from iobase.exception import (NotFoundError, InvalidArgError, ExecError)
from iobase.xmldata import (ANY_CHILD_NAME)
from libfal.util import (dispatch_event,
    servicectrl_suspend, servicectrl_resume)

import fal_config as config

SHARE_NAME = "share"
SHARELIST_NAME = "sharelist"
NAME_NAME = "name"
UNAME_NAME = "uname"
UID_NAME = "uid"
SHARELIST_PATH = "sharelist/*"
SHARE_PATH = "sharelist/share"
NAME_PATH = "share/name"
VOLUME_PATH = "share/volume"
PATH_NAME = "path"
PATH_PATH = "share/" + PATH_NAME
PATH_XPATH = "sharelist/share[name='%s']/path"

NOT_FOUND_MSG = "share-folder not found"
FIND_MSG = "find error"

MIN_UID = 3006
MAX_UID = 4000
DEFAULT_UID = 65534  # nobody
DEFAULT_GID = 65534  # nogroup
QUOTA_VOL = config.VOL_INFO[config.STOR_IF_TO_VOL['internal'][1]]['mnt_point']

MAX_SHARE_NUM = 300

COUNT_SHARE_PATH = 'sharelist/share[@undeletable!="true" or not(@undeletable)]'


def _servicectrl_suspend(category_list, service_list,
                         dry_run=None, debug=None):
    try:
        if config.FAL_CTRL_SERVICE:
            servicectrl_suspend(category_list, service_list,
                                dry_run=None, debug=None)
    except:
        pass


def _servicectrl_resume(category_list, service_list,
                        dry_run=None, debug=None):
    try:
        if config.FAL_CTRL_SERVICE:
            servicectrl_resume(category_list, service_list,
                               dry_run=None, debug=None)
    except:
        pass


def _dispatch_event(category, action, body,
                    previous=False, dry_run=None, debug=None):
    try:
        if config.FAL_DISPATCH_EVENT:
            dispatch_event(category, action, body,
                           previous, dry_run, debug)
    except:
        raise


class FalShares(falcmd.FalCmd):
    """FAL Shares command implementation class."""

    def __make_uname(self, share_name):
        import hashlib
        import base64
        md5 = hashlib.md5()
        md5.update(share_name)
        return ("?%s" % base64.b64encode(md5.hexdigest()))[:16]  # FIXME

    def __useradd_for_share(self, share_name):
        uname = self.__make_uname(share_name)
        users_node = XmlNode('<users><user /></users>')
        user_node = users_node.get_node('user')
        user_node.append_child('name', text=uname)
        uids = [int(n.get_text()) for n in \
                self.__get_share_config().get_node_list('sharelist/share/uid')]
        uid = MIN_UID
        while uid in uids:
            uid += 1
        user_node.append_child('uid', text=str(uid))
        entry = self.send_entry(
                request.Entry(ReqTypes.ADD, users_node),
                pal_info.PAL_USERS_PATH)
        if entry.get_result() != Results.OK:
            raise ExecError(pal_info.PAL_USERS_PATH)
        return (entry.get().get_value('user/uid'), uname)

    def __userdel_for_share(self, share_name):
        uname = self.__make_uname(share_name)
        entry = self.send_entry(
                request.Entry(ReqTypes.DELETE, """<users>
                    <user><name>%s</name></user>
                </users>""" % uname),
                pal_info.PAL_USERS_PATH)
        if entry.get_result() != Results.OK:
            raise ExecError(pal_info.PAL_USERS_PATH)
        return (entry.get().get_value('user/uid'), uname)

    def __set_quota(self, uname, qlimit):
        quota_node = XmlNode("""<quota>
            <userlist>
                <user>
                    <name />
                    <quota><limit /></quota>
                </user>
            </userlist>
        </quota>""")
        quota_node.get_node('userlist/user/name').set_text(uname)
        quota_node.get_node('userlist/user/quota').set_attrs(
                {'volume': QUOTA_VOL})
        quota_node.get_node('userlist/user/quota/limit').set_text(str(qlimit))
        entry = self.send_entry(
                request.Entry(ReqTypes.APPLY, quota_node),
                pal_info.PAL_QUOTA_PATH)
        if entry.get_result() != Results.OK:
            raise ExecError(pal_info.PAL_QUOTA_PATH)

    def __get_share_config(self):
        """Load share config and return XML data."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.SHARES_CONF_FILE).get()

    def __find_share_node(self, entry, share_config):

        name = entry.get().get_node(NAME_PATH).get_text()
        sharelist = share_config.get_node_list(SHARELIST_PATH)

        index = 0
        find = False

        for share in sharelist:
            if share.get_node(NAME_NAME).get_text() == name:
                find = True
                break
            index += 1

        if find == False:
            return None

        return index

    def __add_share_config(self, entry):
        """Add share-folder config."""

        entry = entry.get_clone()
        share_config = self.__get_share_config()
        # FIXME 既存のものはエラーとせず上書き
        # xr用だが、6hもこうする?
        idx = self.__find_share_node(entry, share_config)
        if idx == None:
            share_config.get_node(SHARELIST_NAME).append_child(
                entry.get().get_node(SHARE_NAME))
        else:
            share_config.get_node(SHARELIST_NAME).delete_child(
                    SHARE_NAME, index=idx)
            share_config.get_node(SHARELIST_NAME).insert_child(
                    idx, entry.get().get_node(SHARE_NAME))
        save_entry = request.Entry(ReqTypes.SAVE, share_config)
        return self.send_entry_with_file(
                save_entry,
                pal_info.PAL_CONFFILE_PATH,
                config.SHARES_CONF_FILE)

    def __modify_share_config(self, entry):
        """Modify share-folder config."""

        entry_share_node = entry.get().get_node(SHARE_NAME)
        name = entry_share_node.get_value(NAME_NAME)

        config_shares_node = self.__get_share_config()
        config_share_node = config_shares_node.get_node(
                'sharelist/share[name="%s"]' % name)

        config_share_node.update(entry_share_node)

        xpath_detailpermission = 'permission/detailpermission'
        xpath_readonly = '%s/readonlymember' % xpath_detailpermission
        xpath_readwrite = '%s/readwritemember' % xpath_detailpermission
        for xpath in (
                "%s/userlist" % xpath_readonly,
                "%s/grouplist" % xpath_readonly,
                "%s/userlist" % xpath_readwrite,
                "%s/grouplist" % xpath_readwrite):
            try:
                entry_list_node = entry_share_node.get_node(xpath)
                config_list_node = config_share_node.get_node(xpath)
                config_list_node.delete_children('*')
                config_list_node.update(entry_list_node)
            except NotFoundError:
                continue

        return self.send_entry_with_file(
                request.Entry(ReqTypes.SAVE, config_shares_node),
                pal_info.PAL_CONFFILE_PATH,
                config.SHARES_CONF_FILE)

    def __delete_share_config(self, entry):
        """Delete share-folder config."""

        share_config = self.__get_share_config()
        idx = self.__find_share_node(entry, share_config)

        share_config.get_node(SHARELIST_NAME).delete_child(
                SHARE_NAME, index=idx)

        save_entry = request.Entry(ReqTypes.SAVE, share_config)
        return self.send_entry_with_file(
                save_entry,
                pal_info.PAL_CONFFILE_PATH,
                config.SHARES_CONF_FILE)

    def __get_path_from_volume(self, volume, share_name):
        # VOL_INFO[volume]["share_root"]が"share"などだったら
        # その下のshare_name名のdirが共有dir。
        # share_rootが""だったらmnt_point自体が共有dir。
        if config.VOL_INFO[volume]["share_root"]:
            return os.path.join(config.VOL_INFO[volume]["mnt_point"],
                config.VOL_INFO[volume]["share_root"], share_name)
        return os.path.join(config.VOL_INFO[volume]["mnt_point"])

    def __get_include_path_entry(self, entry, default_volume=""):

        if entry.get().get_value(PATH_PATH):
            return entry

        share_name = entry.get().get_node(NAME_PATH).get_text()
        try:
            volume = entry.get().get_node(VOLUME_PATH).get_text()
            path = self.__get_path_from_volume(volume, share_name)
        except NotFoundError:
            if default_volume:
                volume = default_volume
                path = self.__get_path_from_volume(volume, share_name)
            else:
                share_config = self.__get_share_config()
                path = share_config.get_node(
                    PATH_XPATH % share_name).get_text()
        try:
            entry.get().get_node(PATH_PATH).set_text(path)
        except NotFoundError:
            entry.get().get_node(SHARE_NAME).append_child(PATH_NAME, path)

        return entry

    def __add_handler(self, entry):
        """Handle share folder add request and set entry to result."""

        # event
        try:
            _dispatch_event(
                    'shares', 'add', entry.get().get_clone(), previous=True)
        except:
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        config_shares_node = self.__get_share_config()

        # 共有数確認(既に上限数に達していたらエラー)
        if len(config_shares_node.get_node_list(
            COUNT_SHARE_PATH)) >= MAX_SHARE_NUM:
            entry.set_result(Results.MAX_EXCEED_ERROR)
            return entry

        # 変更共有フォルダ名取得
        try:
            entry_share_node = entry.get().get_node(SHARE_NAME)
        except NotFoundError:
            raise InvalidArgError("Share name is empty!!")
        name = entry_share_node.get_value(NAME_NAME, None)
        if name is None:
            raise InvalidArgError("Share name is empty!!")
        if config_shares_node.get_node_list('sharelist/share/name[.="%s"]' % name):
            raise InvalidArgError("%s is already exists!!" % name)

        # リクエストされた情報を取得
        qlimit = entry_share_node.get_value(
                'shareservice/msshare/quota/limit', None)
        qlimit = qlimit if qlimit is None else float(qlimit)

        entry = self.__get_include_path_entry(entry, config.DEFAULT_VOLUME)

        try:
            if not config.FAL_SAVE_ONLY:
                # 共有用ユーザー作成
                (uid, uname) = self.__useradd_for_share(name)

                try:
                    entry.get().get_node('share/uid').set_text(uid)
                except NotFoundError:
                    entry.get().get_node('share').append_child('uid', text=uid)

                try:
                    entry.get().get_node('share/uname').set_text(uname)
                except NotFoundError:
                    entry.get().get_node('share').append_child('uname', text=uname)

                # 作成したユーザーでquotaかける
                if qlimit is None:
                    pass
                elif qlimit >= 0:
                    self.__set_quota(uname, qlimit)

                try:
                    entry.get().get_node('share/gid').set_text(str(DEFAULT_GID))
                except NotFoundError:
                    entry.get().get_node('share').append_child('gid', text=str(DEFAULT_GID))

                add_result_entry = self.send_entry(entry, pal_info.PAL_SHARES_PATH)
                if add_result_entry.get_result() != Results.OK:
                    return add_result_entry

            save_result_entry = self.__add_share_config(entry)
            if save_result_entry.get_result() != Results.OK:
                return save_result_entry
        except:
            self.__userdel_for_share(name)
            raise

        # event
        try:
            _servicectrl_suspend(['share'], [])
            _dispatch_event('shares', 'add', entry.get().get_clone())
            _servicectrl_resume(['share'], [])
        except:
            _servicectrl_resume(['share'], [])

        entry.set_result(Results.OK)
        return entry

    def __modify_handler(self, entry):
        """Handle share folder modify request and set entry to result."""

        # event
        try:
            _dispatch_event(
                    'shares', 'modify', entry.get().get_clone(), previous=True)
        except:
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        # 変更共有フォルダ名取得
        try:
            entry_share_node = entry.get().get_node(SHARE_NAME)
        except NotFoundError:
            raise InvalidArgError("Share name is empty!!")
        name = entry_share_node.get_value(NAME_NAME, None)
        if name is None:
            raise InvalidArgError("Share name is empty!!")

        # 存在する共有フォルダかチェック
        config_shares_node = self.__get_share_config()
        try:
            config_share_node = config_shares_node.get_node(
                    'sharelist/share[name="%s"]' % name)
        except:
            raise InvalidArgError("%s is not exists!!" % name)

        # リクエストされた情報を取得
        qlimit = entry_share_node.get_value(
                'shareservice/msshare/quota/limit', None)
        qlimit = qlimit if qlimit is None else float(qlimit)

        entry = self.__get_include_path_entry(entry)

        if not config.FAL_SAVE_ONLY:
            # Quota関係処理
            if qlimit is None:
                pass
            elif qlimit >= 0:
                uid = config_share_node.get_value('uid', None)
                uname = config_share_node.get_value(UNAME_NAME, None)
                gid = config_share_node.get_value('gid', DEFAULT_GID)
                if uid is None or uname is None:
                    (uid, uname) = self.__useradd_for_share(name)
                entry_share_node.append_child('uid', text=str(uid))
                entry_share_node.append_child(UNAME_NAME, text=str(uname))

                self.__set_quota(uname, qlimit)
                if qlimit > 0:
                    # pal_sharesにリクエストしてオーナー変更する
                    modify_result_entry = self.send_entry(
                        entry, pal_info.PAL_SHARES_PATH)
                    if modify_result_entry.get_result() != Results.OK:
                        return modify_result_entry

        save_result_entry = self.__modify_share_config(entry)
        if save_result_entry.get_result() != Results.OK:
            return save_result_entry

        # event
        try:
            _servicectrl_suspend(['share'], [])
            _dispatch_event('shares', 'modify', entry.get().get_clone())
            _servicectrl_resume(['share'], [])
        except:
            _servicectrl_resume(['share'], [])

        entry.set_result(Results.OK)
        return entry

    def __delete_handler(self, entry):
        """Handle share folder delete request and set entry to result."""

        entry = self.__get_include_path_entry(entry)
        # event
        try:
            _servicectrl_suspend(['share'], [])
            _dispatch_event(
                    'shares', 'delete', entry.get().get_clone(), previous=True)
            _servicectrl_resume(['share'], [])
        except:
            _servicectrl_resume(['share'], [])
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        # 変更共有フォルダ名取得
        try:
            entry_share_node = entry.get().get_node(SHARE_NAME)
        except NotFoundError:
            raise InvalidArgError("Share name is empty!!")
        name = entry_share_node.get_value(NAME_NAME, None)
        if name is None:
            raise InvalidArgError("Share name is empty!!")

        # 存在する共有フォルダかチェック
        config_shares_node = self.__get_share_config()
        try:
            config_share_node = config_shares_node.get_node(
                    'sharelist/share[name="%s"]' % name)
        except:
            raise InvalidArgError("%s is not exists!!" % name)

        if not config.FAL_SAVE_ONLY:
            delete_result_entry = self.send_entry(
                entry, pal_info.PAL_SHARES_PATH)
            if delete_result_entry.get_result() != Results.OK:
                return delete_result_entry

            # 共有フォルダー用ユーザーの削除
            uname = config_share_node.get_value('uname', None)
            if uname is not None:
                if self.send_entry(
                        request.Entry(ReqTypes.DELETE, """<users>
                            <user><name>%s</name></user>
                        </users>""" % uname),
                        pal_info.PAL_USERS_PATH).get_result() != Results.OK:
                    raise ExecError(pal_info.PAL_USERS_PATH)

        save_result_entry = self.__delete_share_config(entry)
        if save_result_entry.get_result() != Results.OK:
            return save_result_entry

        # event
        try:
            _servicectrl_suspend(['share'], [])
            _dispatch_event('shares', 'delete', entry.get().get_clone())
            _servicectrl_resume(['share'], [])
        except:
            _servicectrl_resume(['share'], [])

        entry.set_result(Results.OK)
        return entry

    def __detail_handler(self, entry):
        """
        Handle share folder detail information request
        and set entry to result.
        """

        try:
            share_config = self.__get_share_config()
            idx = self.__find_share_node(entry, share_config)

            share = share_config.get_node(SHARE_PATH, index=idx)

            result_entry = request.Entry(
                    entry.get_type(), str(share.get_clone()))
            result_entry.set_result(Results.OK)

            #####################################
            #使用量情報を追加する処理を追加する予定
            #####################################

        except:
            result_entry = entry
            result_entry.set_result(Results.DETAIL_FAILED)

        return result_entry

    def __query_handler(self, entry):
        try:

            def _dict_share2uname(self):
                config_shares_node = self.__get_share_config()
                xpath = 'sharelist/share'
                return dict([(n.get_value(NAME_NAME), n.get_value(UNAME_NAME))\
                        for n in config_shares_node.get_node_list(xpath)])

            def _entry_quota_query(self):
                xml = '<quota><volume>%s</volume></quota>' % QUOTA_VOL
                return self.send_entry(
                        request.Entry(ReqTypes.QUERY, xml),
                        pal_info.PAL_QUOTA_PATH)

            def _user_node_list(self):
                xpath = 'userlist/user'
                return _entry_quota_query(self).get().get_node_list(xpath)

            def _dict_uname2used(self):
                return dict([(n.get_value('name'), n.get_value('quota/used')) \
                        for n in _user_node_list(self)])

            uname2used = _dict_uname2used(self)
            share2used = dict([(share, uname2used.get(uname, '0.0')) \
                    for share, uname in _dict_share2uname(self).iteritems()])
        except:
            raise ExecError(pal_info.PAL_QUOTA_PATH)

        shares_node = entry.get()
        shares_node.append_child('sharelist')
        sharelist_node = shares_node.get_node('sharelist')
        for name, used in share2used.iteritems():
            share_node = XmlNode('<share><name/><quota><used/></quota></share>')
            share_node.get_node('name').set_text(name)
            share_node.get_node('quota/used').set_text(used)
            sharelist_node.append_child(share_node)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_save_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.SHARES_CONF_FILE)

        self.set_handler(ReqTypes.LOAD, load_save_handler)
        self.set_handler(ReqTypes.SAVE, load_save_handler)
        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.MODIFY, self.__modify_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)
        self.set_handler(ReqTypes.DETAIL, self.__detail_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)


#
# Main
#

if __name__ == "__main__":
    FalShares(option=falcmd.FalCmdOption()).handle_request()
