#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for System Information ."""

import libfal.falcmd as falcmd
import libfal.util as util

from fal_info import (ReqTypes, Results)

import fal_config as config

import pal_info


### XML Tag Name and Xpath ###
NAME_NAME = "name"
DESC_NAME = "desc"
LOCATION_NAME = "location"
HOSTDESC_NAME = "hostdesc"
HOSTDESC_NAME_XPATH = "hostdesc/name"
HOSTDESC_DESC_XPATH = "hostdesc/desc"
HOSTDESC_LOCATION_XPATH = "hostdesc/location"

# Some Value
PARENT_PATH = "parent"
CHLD_NAME = "child"


class FalSysinfo(falcmd.FalCmd):
    """FAL Sysinfo command implementation class."""

    #
    # Internal methods
    #

    def __restart_samba(self):
        """Restart samba service."""

        util.servicectrl_suspend(category_list=[], service_list=["samba"])
        util.servicectrl_resume(category_list=[], service_list=["samba"])

    def __update_config_file(self, entry):
        """Update config file with entry contents."""

        src_root_node = entry.get()
        module_config = util.load_config(config.SYSINFO_CONF_FILE)
        module_config.update(src_root_node, override=True)
        util.save_config(config.SYSINFO_CONF_FILE, module_config)

    #
    # Request handlers
    #

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_SYSINFO_PATH)
        if Results.OK != result_entry.get_result():
            return result_entry

        module_config = util.load_config(config.SYSINFO_CONF_FILE)
        desc_node = module_config.get_node(HOSTDESC_DESC_XPATH)
        location_node = module_config.get_node(HOSTDESC_LOCATION_XPATH)
        result_entry.get().get_node(HOSTDESC_NAME).append_child(desc_node)
        result_entry.get().get_node(HOSTDESC_NAME).append_child(location_node)
        return result_entry

    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_SYSINFO_PATH)
        if result_entry.get_result() != Results.OK:
            entry.set_result(result_entry.get_result())
            return entry

        # event
        util.dispatch_event('sysinfo', 'apply', entry.get().get_clone())

        # save to pal_conffile
        try:
            self.__update_config_file(entry)
        except:
            entry.set_result(Results.APPLY_ERROR)
            return entry

        entry.set_result(Results.OK)
        return entry

    def __clear_handler(self, entry):
        """Handle clear request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_SYSINFO_PATH)
        if result_entry.get_result() != Results.OK:
            entry.set_result(result_entry.get_result())
            return entry

        entry.set_result(Results.OK)
        return entry

    #
    # Constructor
    #

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                                pal_info.PAL_CONFFILE_PATH,
                                                config.SYSINFO_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.CLEAR, self.__clear_handler)

#
# Main
#

if __name__ == "__main__":
    FalSysinfo(option=falcmd.FalCmdOption()).handle_request()
