#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for System Log Setting."""


import re
import codecs

import ionas.request as request
import libfal.falcmd as falcmd
import pal_info
from fal_info import (ReqTypes, Results)
import fal_config as config
import libfal.util as util

SMB_APPLY_ENTRY = "<msnetwork/>"
GET_ENTRY = "<systemlog><accesslog/></systemlog>"
LOG_CSV_FMT = \
    "%(date)s," + \
    "%(user)s," + \
    "%(clientname)s," + \
    "%(client)s," + \
    "%(operation)s," + \
    "%(result)s," + \
    "%(targetshare)s," + \
    "%(targetfile)s"
LOG_XPATH = "./accesslog/loglist/log"
USERNAME_XPATH = "./accessinfo/username"
CLIENT_XPATH = "./accessinfo/client"
CLIENTNAME_XPATH = "./accessinfo/clientname"
OPERATION_XPATH = "./accessinfo/operation"
TARGETSHARE_XPATH = "./accessinfo/targetshare"
TARGETFILE_XPATH = "./accessinfo/targetfile"


class FalSystemLog(falcmd.FalCmd):
    """FAL System Log command implementation class."""

    def __get_systemlog_config(self):
        """Load system log config setting and return XML data."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
            pal_info.PAL_CONFFILE_PATH, config.SYSTEMLOG_CONF_FILE).get()

    def __convert_for_csv_element(self, src_string):
        """Convert string for CSV output."""

        contain_quotes = False
        contain_commas = False

        if re.match(r'.*".*', src_string):
            contain_quotes = True
            src_string = src_string.replace('"', '""')

        if re.match(r'.*,.*', src_string):
            contain_commas = True

        if contain_quotes or contain_commas:
            src_string = '"' + src_string + '"'

        return src_string

    def __get_log_date(self, log_node):
        """Get date from log."""

        log_date = ""
        try:
            log_date = log_node.get_attr_value("date")
            log_date = self.__convert_for_csv_element(log_date)
        except:
            pass

        return log_date

    def __get_log_user(self, log_node):
        """Get user name from log."""

        log_user = ""
        try:
            log_user = log_node.get_value(USERNAME_XPATH)
            log_user = self.__convert_for_csv_element(log_user)
        except:
            pass

        return log_user

    def __get_log_clientname(self, log_node):
        """Get client host name from log."""

        log_clientname = ""
        try:
            log_clientname = log_node.get_value(CLIENTNAME_XPATH)
            log_clientname = self.__convert_for_csv_element(log_clientname)
        except:
            pass

        return log_clientname

    def __get_log_client(self, log_node):
        """Get client IP address from log."""

        log_client = ""
        try:
            log_client = log_node.get_value(CLIENT_XPATH)
            log_client = self.__convert_for_csv_element(log_client)
        except:
            pass

        return log_client

    def __get_log_operation(self, log_node):
        """Get operation from log."""

        log_operation = ""
        try:
            log_operation = log_node.get_value(OPERATION_XPATH)
            log_operation = self.__convert_for_csv_element(log_operation)
        except:
            pass

        return log_operation

    def __get_log_result(self, log_node):
        """Get result from log."""

        log_result = ""
        try:
            log_operation_node = log_node.get_node(OPERATION_XPATH)
            log_result = log_operation_node.get_attr_value("result")
            log_result = self.__convert_for_csv_element(log_result)
        except:
            pass

        return log_result

    def __get_log_targetshare(self, log_node):
        """Get target share from log."""

        log_targetshare = ""
        try:
            log_targetshare = log_node.get_value(TARGETSHARE_XPATH)
            log_targetshare = self.__convert_for_csv_element(log_targetshare)
        except:
            pass

        return log_targetshare

    def __get_log_targetfile(self, log_node):
        """Get target file from log."""

        log_targetfile = ""
        try:
            log_targetfile = log_node.get_value(TARGETFILE_XPATH)
            log_targetfile = self.__convert_for_csv_element(log_targetfile)
        except:
            pass

        return log_targetfile

    def __get_handler(self, entry):
        """Handle get request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_SYSTEMLOG_PATH)
        return result_entry

    def __csv_transform_handler(self, entry):
        """Handle csv_transform request and set entry to result."""

        csv_file = entry.get_file()
        get_entry = request.Entry(ReqTypes.GET, GET_ENTRY)
        result_entry = self.send_entry(get_entry, pal_info.PAL_SYSTEMLOG_PATH)
        if result_entry.get_result() != Results.OK:
            entry.set_result(result_entry.get_result())
            return entry

        loglist = result_entry.get().get_node_list(LOG_XPATH)
        open(csv_file, "w").write(codecs.BOM_UTF8)

        for log in loglist:
            log_string = LOG_CSV_FMT % {
                "date": self.__get_log_date(log),
                "user": self.__get_log_user(log),
                "clientname": self.__get_log_clientname(log),
                "client": self.__get_log_client(log),
                "operation": self.__get_log_operation(log),
                "result": self.__get_log_result(log),
                "targetshare": self.__get_log_targetshare(log),
                "targetfile": self.__get_log_targetfile(log)}
            open(csv_file, "a").write(log_string + "\r\n")

        entry.set_result(Results.OK)
        return entry

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        util.servicectrl_suspend([], ["samba"])
        accesslog = entry.get().get_node("accesslog")
        smb_entry = request.Entry(ReqTypes.SET_SYSTEMLOG, SMB_APPLY_ENTRY)
        smb_entry.get().append_child(accesslog)
        result_entry = self.send_entry(smb_entry, pal_info.PAL_SAMBA_PATH)
        util.servicectrl_resume([], ["samba"])
        if result_entry.get_result() != Results.OK:
            entry.set_result(result_entry.get_result())
            return entry

        log_config = self.__get_systemlog_config()
        log_config.delete_child("accesslog")
        log_config.append_child(accesslog)

        save_entry = request.Entry(ReqTypes.SAVE, log_config)
        self.send_entry_with_file(save_entry, pal_info.PAL_CONFFILE_PATH,
                                         config.SYSTEMLOG_CONF_FILE)
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
            pal_info.PAL_CONFFILE_PATH, config.SYSTEMLOG_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.GET, self.__get_handler)
        self.set_handler(ReqTypes.CSV_TRANSFORM, self.__csv_transform_handler)

#
# Main
#

if __name__ == "__main__":
    FalSystemLog(option=falcmd.FalCmdOption()).handle_request()
