#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for UPS Setting."""


import string
import os
import time

import iobase.xmldata as xmldata
import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info
from fal_info import (ReqTypes, Results)
from iobase.exception import (NotFoundError, ExecError)
import libfal.util as util
import fal_config as config


NETSHUTDOWN = "netshutdown"
TIME = "time"
WARNING = "warning"
UPS_SERVICE_LIST = ["apcupsd", "omronssd", "nut"]

NETIF_XPATH_LAN = "./netiflist/netif[@name='%s']"
NETIF_XPATH_DEV = "./netiflist/netif[@dev='%s']"
PROTOCOL_XPATH = "./protocollist/protocol[@family='%s']"
ADDR_XPATH = "./addrlist/addr"
TARGETLIST_XPATH = "./netshutdown/targetlandisklist"
TARGET = "targetlandisk"


class FalUps(falcmd.FalCmd):
    """FAL UPS command implementation class."""

    def __get_config(self, conf_file):
        """Load config setting and return XML data ."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                             pal_info.PAL_CONFFILE_PATH, conf_file).get()

    def __get_host_name(self):
        entry = request.Entry(ReqTypes.QUERY)
        result_entry = self.send_entry(entry, pal_info.PAL_SYSINFO_PATH)
        if Results.OK != result_entry.get_result():
            return None
        try:
            name = result_entry.get().get_node("hostdesc/name").get_text()
            if name:
                return name
            else:
                return None
        except:
            return None

    def __get_ipaddr(self, net_conf, query_data, netif="lan1", inet="inet4"):
        dev_map = config.NET_IF_TO_DEV
        ipaddr = ""
        netif_node = net_conf.get_node(NETIF_XPATH_LAN % netif)
        if netif == "lan2":
            if not netif_node.is_enabled():
                return ipaddr

        protocol_node = netif_node.get_node(PROTOCOL_XPATH % inet)
        if not protocol_node.is_enabled():
            return ipaddr
        if protocol_node.get_node("dhcp").is_enabled():
            if query_data:
                try:
                    addr_node = query_data.get_node(
                            NETIF_XPATH_DEV % dev_map[netif]).get_node(
                            PROTOCOL_XPATH % inet).get_node(ADDR_XPATH)
                    ipaddr = addr_node.get_text()
                except NotFoundError:
                    pass
        else:
            ipaddr = protocol_node.get_node(ADDR_XPATH).get_text()
        return ipaddr

    def __query_net_data(self):
        entry = request.Entry(ReqTypes.QUERY, "<network/>")
        result_entry = self.send_entry(entry, pal_info.PAL_NETWORK_PATH)
        if Results.OK != result_entry.get_result():
            return None
        return result_entry.get()

    def __get_ipaddr_list(self):
        ipaddr_list = []
        net_conf = self.__get_config(config.NETWORK_CONF_FILE)
        query_data = self.__query_net_data()
        for netif in config.NET_IF_LIST:
            for inet in ["inet4", "inet6"]:
                addr = self.__get_ipaddr(
                    net_conf, query_data, netif=netif, inet=inet)
                if addr:
                    ipaddr_list.append(addr)

        return ipaddr_list

    def __get_self_names(self):
        self_names = ["LOCALHOST", "127.0.0.1"]
        
        host_name = self.__get_host_name()
        if host_name:
            self_names.append(host_name.upper())
        self_names += self.__get_ipaddr_list()

        return self_names

    def __update_handler(self, entry):
        """Update requet."""

        result_entry = self.send_entry(entry, pal_info.PAL_UPS_PATH)
        return result_entry

    def __netshutdown_handler(self, entry):
        """netshutdown setting."""

        ups_config = self.__get_config(config.UPS_CONF_FILE)
        del_list = []
        targetlist_node = ups_config.get_node(TARGETLIST_XPATH)
        for target in targetlist_node.get_node_list(TARGET):
            host = target.get_node("host").get_text()
            if host.upper() in self.__get_self_names():
                del_list.append(target.get_attr_value("number"))
        for num in del_list:
            targetlist_node.delete_child(TARGET, attrs={"number": num})

        entry.set(ups_config.get_node(NETSHUTDOWN))
        try:
            result_entry = self.send_entry(entry, pal_info.PAL_UPS_PATH)
        except:
            raise ExecError(pal_info.PAL_UPS_PATH)
        return result_entry

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        try:
            result_entry = self.send_entry(entry, pal_info.PAL_UPS_PATH)
        except:
            raise ExecError(pal_info.PAL_UPS_PATH)
        if result_entry.get_result() != Results.OK:
            entry.set_result(result_entry.get_result())
            return entry

        status = result_entry.get().get_node("status")
        ups_config = self.__get_config(config.UPS_CONF_FILE)
        if ups_config.get_node("warning").is_enabled():
            if status.get_text() == "ONLINE":
                status.set_bool_attr("warning", true=False)
            else:
                status.set_bool_attr("warning")
        else:
            status.set_bool_attr("warning", true=False)

        return result_entry

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        util.servicectrl_suspend([], UPS_SERVICE_LIST)
        result_entry = self.send_entry(entry, pal_info.PAL_UPS_PATH)
        util.servicectrl_resume([], UPS_SERVICE_LIST)

        if result_entry.get_result() != Results.OK:
            return result_entry

        ups_config = self.__get_config(config.UPS_CONF_FILE)
        for node in entry.get().get_node_list(xmldata.ANY_CHILD_NAME):
            try:
                ups_config.delete_child(node.get_node_name())
            except NotFoundError:
                pass
            ups_config.append_child(node)

        save_entry = request.Entry(ReqTypes.SAVE, ups_config)
        self.send_entry_with_file(save_entry, pal_info.PAL_CONFFILE_PATH,
                                         config.UPS_CONF_FILE)

        # FIXME 暫定
        try:
            from iobase.cmdexec import CmdExec, NULL
            time.sleep(2)
            cmd = ['/usr/local/bin/interval_alert', '--once', '--ups']
            CmdExec(cmd, stdout=NULL, stderr=NULL).execute()
        except:
            pass

        return result_entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.UPS_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.UPDATE, self.__update_handler)
        self.set_handler(ReqTypes.NETSHUTDOWN, self.__netshutdown_handler)
        self.set_handler(ReqTypes.CHECK, self.__netshutdown_handler)

#
# Main
#

if __name__ == "__main__":
    FalUps(option=falcmd.FalCmdOption()).handle_request()
