#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for User Settings."""


import os
import logging

import ionas.request as request
import iobase.xmldata as xmldata
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info
from fal_info import ReqTypes
from fal_info import Results
from iobase.exception import (NotFoundError, InvalidArgError, ExecError)
from libfal.util import dispatch_event

import fal_config as config

USERLIST_NAME = "userlist"
USERS_NAME = "users"
USER_NAME = "user"
NAME_NAME = "name"
FULLNAME_NAME = "fullname"
DESC_NAME = "desc"
UID_NAME = "uid"
GID_NAME = "gid"
PASSWD_NAME = "passwd"
QUOTA_NAME = "quota"
MAILADDR_NAME = "mailaddr"

USERLIST_PATH = "./userlist"
USER_PATH = USERLIST_PATH + "/user"
UID_PATH = USER_PATH + "/uid"
GROUPLIST_PATH = "./grouplist"
GROUP_PATH = GROUPLIST_PATH + "/group"
UID_ENTRY_PATH = "./user/uid"
GID_ENTRY_PATH = "./user/gid"
NAME_ENTRY_PATH = "./user/name"

DEL = "del"
MOD = "mod"
ADD = "add"
DOMAIN = "domain"
GROUP_ENTRY = "<groups/>"
AUTH_USER_XPATH = "userlist/user[not(@domain) and name='%s']"
DETAIL_USER_XPATH = "userlist/user" + \
        "[(not(@hidden) or @hidden!='true') and not(@domain) and name='%s']"

MIN_UID = 1000
MAX_USER_NUM = 300
QUOTA_VOL = config.VOL_INFO[config.STOR_IF_TO_VOL['internal'][1]]['mnt_point']
DEFAULT_GID = 100  # FIXME 100


class FalUsers(falcmd.FalCmd):
    """FAL User command implementation class."""

    def __is_empty(self, val):
        if val in (None, "", [], (), {}):
            return True
        return False

    def __quota_is_enabled(self):
        return self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                config.QUOTA_CONF_FILE).get().get_node(
                        'volume[="%s"]' % QUOTA_VOL).is_enabled()

    def __set_quota(self, user, limit):
        quota_node = xmldata.XmlNode("""<quota>
            <userlist>
                <user>
                    <name />
                    <quota><limit /></quota>
                </user>
            </userlist>
        </quota>""")
        quota_node.get_node('userlist/user/name').set_text(user)
        quota_node.get_node('userlist/user/quota').set_attrs(
                {'volume': QUOTA_VOL})
        quota_node.get_node('userlist/user/quota/limit').set_text(limit)
        entry = self.send_entry(
                request.Entry(ReqTypes.APPLY, quota_node),
                pal_info.PAL_QUOTA_PATH)
        if entry.get_result() == Results.OK:
            return True
        else:
            return False

    def __get_user_config(self):
        """Load user config and return XML data ."""

        return self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                config.USERS_CONF_FILE).get()

    def __get_group_config(self):
        """Load group config and return XML data ."""

        return self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                config.GROUPS_CONF_FILE).get()

    def __pal_add_user_to_group(self, group, name):
        """Add user to a given group ."""

        new_user = xmldata.XmlNode(USER_NAME)
        new_user.append_child(NAME_NAME, name)
        group.get_node(USERLIST_NAME).append_child(new_user)
        gp_set_entry = request.Entry(ReqTypes.MODIFY, GROUP_ENTRY)
        gp_set_entry.get().append_child(group.get_clone())
        result_entry = self.send_entry(gp_set_entry, pal_info.PAL_GROUPS_PATH)
        return result_entry

    def __group_delete_user(self, group, name):
        """Delete user from a given group ."""

        try:
            group.delete_nodes(USERLIST_PATH, "./user[name=\"%s\"]" % name)
            gp_set_entry = request.Entry(ReqTypes.MODIFY, GROUP_ENTRY)
            gp_set_entry.get().append_child(group.get_clone())
            gp_result = self.send_entry(gp_set_entry, pal_info.PAL_GROUPS_PATH)
            return gp_result
        except NotFoundError:
            pass

    def __primary_group_operation(self, gid, entry_gid):
        """check the operation of primary group. """

        if entry_gid == "" and gid != "":
            return DEL
        elif entry_gid != "" and gid == "":
            return ADD
        elif entry_gid != "" and gid != "" and gid != entry_gid:
            return MOD

    def __get_user_id_list(self, user_config):
        """Get a list of user id."""

        return [uid_node.get_text() for uid_node in \
                user_config.get_node_list(UID_PATH)]

    def __check_userid_enable(self, entry, uid_list):
        """Check if the user ID in entry is enable."""

        user_id = entry.get().get_node(UID_ENTRY_PATH).get_text()
        if user_id != "":
            if user_id in uid_list:
                raise InvalidArgError("User ID %s has been used." % user_id)
        else:
            uid = MIN_UID
            while str(uid) in uid_list:
                uid += 1
            entry.get().get_node(UID_ENTRY_PATH).set_text(str(uid))
        return

    def __group_add_user(self, entry, user_name, group_id):
        """Add a user to group when the user's gid is enable."""

        group_config = self.__get_group_config()
        grouplist = group_config.get_node(GROUPLIST_PATH)

        target_group = grouplist.get_node('group[id="%s"]' % group_id)
        result = self.__pal_add_user_to_group(target_group, user_name)
        if result.get_result() != Results.OK:
            entry.set_result(result.get_result())
            return entry

        gp_save_entry = request.Entry(ReqTypes.SAVE, group_config)
        save_result = self.send_entry_with_file(gp_save_entry,
                pal_info.PAL_CONFFILE_PATH, config.GROUPS_CONF_FILE)
        save_result.set_result(Results.OK)
        return save_result

    def __modify_user(self, entry_gid, gid, user_name, group_config):
        """Modify user in this function."""

        grouplist = group_config.get_node(GROUPLIST_PATH)
        primary_gp_opt = self.__primary_group_operation(gid, entry_gid)

        if primary_gp_opt == DEL:
            group = grouplist.get_node('group[id="%s"]' % gid)
            result = self.__group_delete_user(group, user_name)
        elif primary_gp_opt == ADD:
            group = grouplist.get_node('group[id="%s"]' % entry_gid)
            result = self.__pal_add_user_to_group(group, user_name)
        elif primary_gp_opt == MOD:
            group = grouplist.get_node('group[id="%s"]' % gid)
            result = self.__group_delete_user(group, user_name)
            group = grouplist.get_node('group[id="%s"]' % entry_gid)
            if result.get_result() == Results.OK:
                result = self.__pal_add_user_to_group(group, user_name)

        return result.get_result()

#    def __detail_handler(self, entry):
#        """Handle user detail information request and set entry to result."""
#
#        user_name = entry.get().get_node(NAME_ENTRY_PATH).get_text()
#        user_config = self.__get_user_config()
#        try:
#            target_user = user_config.get_node(DETAIL_USER_XPATH % user_name)
#            entry.get().delete_child(USER_NAME)
#            entry.get().append_child(target_user)
#            entry.set_result(Results.OK)
#        except NotFoundError:
#            entry.set_result(Results.NotFoundError)    # FIXME
#        return entry

    def __query_handler(self, entry):
        """Handle user quota information request and set entry to result."""

        # pal_quotaからユーザー毎の使用容量を取得
        try:

            def __pal_quota_query(self):
                xml = '<quota><volume>%s</volume></quota>' % QUOTA_VOL
                return self.send_entry(
                        request.Entry(ReqTypes.QUERY, xml),
                        pal_info.PAL_QUOTA_PATH)

            def __get_user_node_list(self):
                xpath = 'userlist/user'
                return __pal_quota_query(self).get().get_node_list(xpath)

            def __get_used_dict(self):
                return dict([(n.get_value('name'), n.get_value('quota/used')) \
                        for n in __get_user_node_list(self)])

            used_dict = __get_used_dict(self)
        except:
            raise ExecError(pal_info.PAL_QUOTA_PATH)    # FIXME

        # レスポンスentry作成
        users_node = entry.get()
        users_node.append_child('userlist')
        userlist_node = users_node.get_node('userlist')
        for user_node_conf in \
                    self.__get_user_config().get_node_list('userlist/user'):
            name = user_node_conf.get_value('name')
            user_node = xmldata.XmlNode('<user />')
            user_node.append_child('name', text=name)
            user_node.append_child('quota')
            user_node.get_node('quota').append_child(
                    'used', text=used_dict.get(name, '0'))
            userlist_node.append_child(user_node)
        entry.set_result(Results.OK)

        return entry

    def __is_domain_user(self, user):
        """Check whether the user is a domain user or not."""

        if DOMAIN in user.get_attrs().keys():
            sysinfo = self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH, config.SYSINFO_CONF_FILE).get()
            hostname = sysinfo.get_node("hostdesc/name").get_text()
            if user.get_attr_value(DOMAIN) == hostname:
                return False
            else:
                return True
        else:
            return False

    def __auth_handler(self, entry):
        """Handle login authentication request and set entry to result."""

        user = entry.get().get_node(USER_NAME)
        if self.__is_domain_user(user):
            result_entry = self.send_entry(entry, pal_info.PAL_SAMBA_PATH)
            return result_entry
        else:
            name = user.get_node(NAME_NAME).get_text()
            entry_pswd = user.get_node(PASSWD_NAME).get_text(do_strip=False)
            user_config = self.__get_user_config()
            try:
                target_user = user_config.get_node(AUTH_USER_XPATH % name)
                if target_user.get_node(PASSWD_NAME).get_text(
                    do_strip=False) == entry_pswd:
                    entry.set_result(Results.OK)
                else:
                    entry.set_result(Results.AUTH_ERROR)   # FIXME
            except NotFoundError:
                entry.set_result(Results.AUTH_ERROR)     # FIXME
        return entry

    def __delete_handler(self, entry):
        """Handle user delete request and set entry to result."""

        # event
        try:
            dispatch_event(
                    'users', 'delete', entry.get().get_clone(), previous=True)
        except:
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        # リクエストされた削除ユーザー名を取得
        user_node = entry.get().get_node('user')
        name = user_node.get_value('name', None)
        if self.__is_empty(name):
            raise InvalidArgError("User name empty!!")

        # 存在しないユーザー名を指定されたらエラー
        user_config = self.__get_user_config()
        try:
            target = user_config.get_node('userlist/user[name="%s"]' % name)
        except NotFoundError:
            raise InvalidArgError("%s is not exists!!" % name)

        # ドメインユーザーは削除不可
        if user_config.get_value('@domain', None):
            raise InvalidArgError("%s is domain user!!" % name)
        # FIXME 管理者、予約、隠しユーザーはどうするか?

#        user_config.delete_nodes(
#                USERLIST_PATH,
#                "./user[(not(@hidden) or @hidden!='true') and \
#                not(@domain) and name='%s']" % delete_name)

        # pal_users に delete リクエスト
        del_users_node = xmldata.XmlNode('<users><user /></users>')
        del_users_node.get_node('user').append_child('name', text=name)
        del_res_entry = self.send_entry(
                request.Entry(ReqTypes.DELETE, del_users_node),
                pal_info.PAL_USERS_PATH)
        result = del_res_entry.get_result()
        if result != Results.OK:
            entry.set_result(result)
            return entry

        # FIXME グループのメンバーから削除

        # FIXME quota から除外

        # 設定ファイルに書き出し
        user_config.delete_nodes('userlist', 'user[name="%s"]' % name)
        self.send_entry_with_file(
                request.Entry(ReqTypes.SAVE, user_config),
                pal_info.PAL_CONFFILE_PATH,
                config.USERS_CONF_FILE)

        # event
        try:
            dispatch_event('users', 'delete', entry.get().get_clone())
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __add_handler(self, entry):
        """Handle add request and set entry to result."""

        # event
        try:
            dispatch_event(
                    'users', 'add', entry.get().get_clone(), previous=True)
        except:
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        # FIXME 全体的に関数化検討
        user_config = self.__get_user_config()

        # ユーザー数確認(既に上限数に達していたらエラー)
        if len(user_config.get_node_list(
                'userlist/user[@hidden="false"]')) >= MAX_USER_NUM:
            entry.set_result(Results.MAX_EXCEED_ERROR)
            return entry

        # リクエストパラメータ取得
        # FIXME userタグの属性
        user_node = entry.get().get_node(USER_NAME)
        admin = user_node.is_set_to_true('admin')
        reserved = user_node.is_set_to_true('reserved')
        hidden = user_node.is_set_to_true('hidden')
        domain = user_node.get_value('@domain', None)
        name = user_node.get_value(NAME_NAME, None)
        fullname = user_node.get_value(FULLNAME_NAME, '')
        desc = user_node.get_value(DESC_NAME, '')
        uid = user_node.get_value(UID_NAME, None)
        gid = user_node.get_value(GID_NAME, None)
        if gid == '' or gid is None:
            gid = DEFAULT_GID
        passwd = user_node.get_value(PASSWD_NAME, '', do_strip=False)
        quota = user_node.get_value('quota/limit', None)
        mailaddr = user_node.get_value(MAILADDR_NAME, None)
        if self.__is_empty(name):
            raise InvalidArgError("User name empty!!")

        # uid 処理
        uids = [int(n.get_text()) \
                for n in user_config.get_node_list('userlist/user/uid')]
        if self.__is_empty(uid):
            uid = int(MIN_UID)
            while uid in uids:
                uid += 1
        else:
            if int(uid) in uids:
                raise InvalidArgError("User ID %s has been used." % uid)

        # FIXME 同名ユーザーが存在するか確認しなくていいのか?
        #       (pal_users にリクエストしたらエラーになるはず)

        # pal_users に add リクエスト
        add_users_node = xmldata.XmlNode('<users><user /></users>')
        add_user_node = add_users_node.get_node(USER_NAME)
        add_user_node.append_child(NAME_NAME, text=name)
        add_user_node.append_child(UID_NAME, text=str(uid))
        add_user_node.append_child(GID_NAME, text=str(gid))
        add_user_node.append_child(PASSWD_NAME, text=passwd)
        add_user_node.append_child(FULLNAME_NAME, text=fullname)
        add_res_entry = self.send_entry(
                request.Entry(ReqTypes.ADD, add_users_node),
                pal_info.PAL_USERS_PATH)
        if add_res_entry.get_result() != Results.OK:
            entry.set_result(add_res_entry.get_result())
            return entry

        # pal_quota にリクエストして使用量制限を設定
        if quota is not None and not self.__set_quota(name, quota):
            quota = None

#        # FIXME group にユーザーを追加する イベントで処理する
#        user_name = entry.get().get_node(NAME_ENTRY_PATH).get_text()
#        group_id = entry.get().get_node(GID_ENTRY_PATH).get_text()
#        if group_id != "":
#            result = self.__group_add_user(entry, user_name, group_id)
#            if result.get_result() != Results.OK:
#                entry.set_result(result.get_result())
#                return entry

        # 設定ファイルに書き出す
        added_user_node = add_res_entry.get().get_node('user')
        added_user_node.set_bool_attr('admin', admin)
        added_user_node.set_bool_attr('reserved', reserved)
        if not self.__is_empty(domain):
            added_user_node.set_attrs({'domain': domain})
        added_user_node.set_bool_attr('hidden', hidden)
        added_user_node.append_child(DESC_NAME, text=desc)
        if not quota is None:
            added_user_node.append_child(QUOTA_NAME)
            added_user_node.get_node(QUOTA_NAME).append_child('limit', text=quota)
        user_config.get_node('userlist').append_child(added_user_node)
        self.send_entry_with_file(
                request.Entry(ReqTypes.SAVE, user_config),
                pal_info.PAL_CONFFILE_PATH,
                config.USERS_CONF_FILE)

        # 正常終了を返す
        user_config = self.__get_user_config()
        users_node = entry.get()
        users_node.clear()
        users_node.append_child(
                user_config.get_node('userlist/user[name="%s"]' % name))

        # event
        try:
            dispatch_event('users', 'add', entry.get().get_clone())
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __modify_handler(self, entry):
        """Handle modify request and set entry to result."""

        # event
        try:
            dispatch_event(
                    'users', 'modify', entry.get().get_clone(), previous=True)
        except:
            entry.set_result(Results.INTERNAL_ERROR)
            return entry

        user_node = entry.get().get_node(USER_NAME)
        name = user_node.get_value(NAME_NAME, None)
        fullname = user_node.get_value(FULLNAME_NAME, None)
        desc = user_node.get_value(DESC_NAME, None)
        uid = user_node.get_value(UID_NAME, None)
        gid = user_node.get_value(GID_NAME, None)
        if gid == '':
            gid = DEFAULT_GID
            user_node.get_node(GID_NAME).set_text(str(DEFAULT_GID))
        passwd = user_node.get_value(PASSWD_NAME, None, do_strip=False)
        quota = user_node.get_value('quota/limit', None)
        mailaddr = user_node.get_value(MAILADDR_NAME, None)  # FIXME adminのみ
        if self.__is_empty(name):
            raise InvalidArgError("User name empty!!")

        # 存在しないユーザー名を指定されたらエラー
        user_config = self.__get_user_config()
        try:
            target = user_config.get_node('userlist/user[name="%s"]' % name)
        except NotFoundError:
            raise InvalidArgError("%s is not exists!!" % name)

        # pal_users に modify リクエスト
        # FIXME
        mod_res_entry = self.send_entry(entry, pal_info.PAL_USERS_PATH)
        result = mod_res_entry.get_result()
        if result != Results.OK:
            entry.set_result(result)
            return entry

        # 使用量制限が変わっていたら変える
        if quota is not None and not self.__set_quota(name, quota):
            quota = None

        # FIXME gid が変わっていたら所属グループを変える
#        user_id = entry.get().get_node(UID_ENTRY_PATH).get_text()
#        user_name = entry.get().get_node(NAME_ENTRY_PATH).get_text()
#        entry_gid = entry.get().get_node(GID_ENTRY_PATH).get_text()
#        user_config = self.__get_user_config()
#        userlist = user_config.get_node(USERLIST_PATH)
#        group_config = self.__get_group_config()
#        target_user = userlist.get_node('user[uid="%s"]' % user_id)
#        gid = target_user.get_node(GID_NAME).get_text()
#        if entry_gid != gid:
#            result = self.__modify_user(entry_gid,
#                                    gid, user_name, group_config)
#            if result != Results.OK:
#                entry.set_result(result)
#                return entry

        # 設定ファイルに書き出す
        for node in entry.get().get_node_list('user/*'):
            if node.get_node_name() == 'name':
                continue
            try:
                target.delete_child(node.get_node_name())
            except NotFoundError:
                pass
            target.append_child(node)
        self.send_entry_with_file(
                request.Entry(ReqTypes.SAVE, user_config),
                pal_info.PAL_CONFFILE_PATH,
                config.USERS_CONF_FILE)

        # 正常終了を返す
        user_config = self.__get_user_config()
        users_node = entry.get()
        users_node.clear()
        users_node.append_child(
                user_config.get_node('userlist/user[name="%s"]' % name))

        # event
        try:
            dispatch_event('users', 'modify', entry.get().get_clone())
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __get_workgroup(self):
        msnd = self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                config.MSNETWORK_CONF_FILE).get()
        method = msnd.get_value('assign/@method')
        if method == "wg":
            return msnd.get_value('assign/wg/workgroup', None)
        elif method == "nt":
            return msnd.get_value('assign/nt/domain', None)
        elif method == "ad":
            return msnd.get_value('assign/ad/netbios', None)
        return None

    def __get_domain_user(self):
        entry = self.send_entry(
                request.Entry(ReqTypes.LIST, '<winbind />'),
                pal_info.PAL_WINBIND_PATH)
        ulist = [und.get_value('name') \
                for und in entry.get().get_node_list('userlist/user') \
                    if und.get_value('name', None)]
        workgroup = self.__get_workgroup() or ""
        ulist_ = []
        for upn in ulist:
            if not upn:
                continue
            (domain, name) = upn.split(config.WINBIND_SEPARATOR)[:2]
            if not domain.lower() == workgroup.lower():
                continue
            ulist_.append(upn)
        return ulist_

    def __domain_list_handler(self, entry):
        """<entry type="domain_list">
            <users />
        </entry>"""
        """<entry type="domain_list" result="0">
            <users>
                <userlist>
                    <user>
                        <upn>DOMAIN@USER001</upn>
                        <domain>DOMAIN</domain>
                        <name>USER001</name>
                        <quota><limit>10</limit></quota>
                    </user>
                </userlist>
            </users>
        </entry>"""

        ulist = self.__get_domain_user()

        # FIXME quota設定値の取得
        # pal_quotaからユーザー毎の使用容量を取得
        try:

            def __pal_quota_query(self):
                xml = '<quota><volume>%s</volume></quota>' % QUOTA_VOL
                return self.send_entry(
                        request.Entry(ReqTypes.QUERY, xml),
                        pal_info.PAL_QUOTA_PATH)

            def __get_user_node_list(self):
                xpath = 'userlist/user'
                return __pal_quota_query(self).get().get_node_list(xpath)

            def __get_limit_dict(self):
                return dict([(n.get_value('name'), n.get_value('quota/limit')) \
                        for n in __get_user_node_list(self)])

            limit_dict = __get_limit_dict(self)
        except:
            raise ExecError(pal_info.PAL_QUOTA_PATH)    # FIXME

        users_node = entry.get()
        users_node.set_node_name('users')  # 念のため
        users_node.clear()  # 念のため
        users_node.append_child('userlist')
        userlist_node = users_node.get_node('userlist')
        for uname in ulist:
            upn = uname
            try:
                (domain, name) = upn.split(config.WINBIND_SEPARATOR)[:2]
                unode = xmldata.XmlNode('user')
                unode.append_child('upn', text=upn)
                unode.append_child('domain', text=domain)
                unode.append_child('name', text=name)
                unode.append_child('quota')
                unode.get_node('quota').append_child('limit', text=limit_dict.get(upn, '0'))
                userlist_node.append_child(unode)
            except:
                continue

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.USERS_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.MODIFY, self.__modify_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)
#        self.set_handler(ReqTypes.DETAIL, self.__detail_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.AUTH, self.__auth_handler)
        self.set_handler(ReqTypes.DOMAIN_LIST, self.__domain_list_handler)


#
# Main
#

if __name__ == "__main__":
    FalUsers(option=falcmd.FalCmdOption()).handle_request()
