#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Validation."""

import os

import iobase.xmldata as xmldata
from iobase.exception import NotFoundError
import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
from pal_info import(PAL_CONFFILE_PATH)
from fal_info import ReqTypes
from fal_info import Results
import fal_config as config
import libfal.validators.validator as validator

VALIDATE_NAME = "validate"
NAME_NAME = "name"
TARGET_NAME = "target"

VALIDATOR_ATTR = "validator"
NAME_ATTR = "name"

IN_ITEM_CLASS = "InItem"

VALIDATE_LIST_CHILD_PATH = "validatelist/*"
ALTERNATIVE_LIST_CHILD_PATH = "featurelist/feature/alternativelist/*"

VALIDATE_FMT = "validatelist/validate[name='%s']"
ALT_FMT = "altlist/alt[@name='%s']"

RESULT_NOT_EXIST_VALIDATOR = "65536"


class NotExistValidatorError(Exception):
    """Not exist validator error."""

    pass


class FalValidate(falcmd.FalCmd):
    """FAL Validate command implementation class."""

    def __get_validate_ins(self, validator_name, entry):
        """get validate instance."""

        try:
            node = self.validate_xml_node.get_node(
                VALIDATE_FMT % validator_name)
            return getattr(validator, validator_name)(
                    target_node=entry.get().get_node(TARGET_NAME),
                    validate_config_node=node)

        except NotFoundError:
            try:
                node = self.validate_xml_node.get_node(
                    ALT_FMT % validator_name)
                return getattr(validator, validator_name)(
                        target_node=entry.get().get_node(TARGET_NAME))

            except NotFoundError:
                raise NotExistValidatorError


    def __validate_handler(self, entry):
        """Handle validate request and set entry to result."""

        try:
            validator_ins = self.__get_validate_ins(
                    entry.get().get_node(
                    TARGET_NAME).get_attr_value(VALIDATOR_ATTR),
                    entry)

            entry.get().append_child(validator_ins.validate())
            entry.set_result(Results.OK)

        except NotExistValidatorError:
            entry.set_result(RESULT_NOT_EXIST_VALIDATOR)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_entry = request.Entry(ReqTypes.LOAD)

        self.validate_xml_node = xmldata.XmlNode()
        self.validate_xml_node.set(
                self.send_entry_with_file(load_entry,
                PAL_CONFFILE_PATH,
                config.VALIDATES_XML_FILE).get())

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            PAL_CONFFILE_PATH,
                                            config.VALIDATES_XML_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.VALIDATE, self.__validate_handler)


#
# Main
#

if __name__ == "__main__":
    FalValidate(option=falcmd.FalCmdOption()).handle_request()
