#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling volume function"""

from __future__ import with_statement

import os
import time
import re
import glob
import pal_info
import fal_config as config
import ionas.request as request
import libfal.falcmd as falcmd
import libfal.util as util
import iobase.fileut as fileut
from libfal.util import dispatch_event
from iobase.exception import (NotFoundError, InvalidArgError)
from iobase.xmldata import XmlNode
from fal_info import (ReqTypes, Results)
from fal_config import (DECODED_LOCK_KEY_FILE_PATH,
                        INTERNAL_DEV, MIRROR_DEV, INTERNAL_VOL,
                        SINGLE_RAID_LEVEL, DEV_TO_ENC_DEV, VOL_INFO,)

### option ###
OPERATION_OPT = "operation"
ENC_START_OPT_VAL = "encstart"
ENC_STOP_OPT_VAL = "encstop"
VFSREMOUNT_ATTR = "vfsremount"
VFSADD_ATTR = "vfsadd"

### XML Tag Name ###
NAME_NAME = "name"
VOLUME_NAME = "volume"
OPTION_NAME = "option"
STATUS_NAME = "status"
FSYS_NAME = "filesystem"
TOTAL_NAME = "total"
USE_NAME = "use"
CAPACITY_NAME = "capacity"
PARTITION_NAME = "partition"
PARTITION_LIST_NAME = "partitionlist"
NUMBER_NAME = "number"
VFSEXT_NAME = "vfsext"
AUTOREBUILD_NAME = "autorebuild"

VOL_NAME_PATH = "volume/name"

### XML Path ###
VOL_NAME_XPATH = "./volume/name[@type='volume']"
PART_NAME_XPATH = "./volume/name[@type='partition']"
VOL_STATUS_XPATH = "volume/status"
FILESYSTEM_XPATH = "volume/filesystem"
TOTAL_SIZE_XPATH = "volume/capacity/total"
USE_SIZE_XPATH = "volume/capacity/use"
ENC_ATTR_XPATH = "volume/status[@encrypt]"
INTRNAL_ENCRIPT_XPATH = \
    "./volumelist/volume[name='sataraid1']/encrypt"

### XML Attribute ###
TYPE_ATTR = "type"
ENCRYPT_ATTR = "encrypt"
CLEAR_ATTR = "clear"
BIND_ATTR = "bind"
NOUPDATE_ATTR = "noupdate"
MOUNT_FAIL_ATTR = "mountfail"

### XML Attribute Value ###
DISK_ATTR_VAL = "disk"
PARTITION_ATTR_VAL = "partition"
TRUE_ATTR_VAL = "true"
FALSE_ATTR_VAL = "false"
CRYPT_ATTR_VAL = "crypt"
MOUNT_ATTR_VAL = "mount"
SWAP_CREATE_ATTR_VAL = "swapcreate"
SWAP_ON_ATTR_VAL = "swapon"
SWAP_OFF_ATTR_VAL = "swapoff"
ENC_START_ATTR_VAL = "encstart"
ENC_STOP_ATTR_VAL = "encstop"
WRITE_ENC_SIG_ATTR_VAL = "writeencsig"
CHK_ENC_SIG_ATTR_VAL = "chkencsig"

### XML Value ###
MOUNTED = "mounted"
UNMOUNTED = "unmounted"
LOCKKEY_STATUS = "lockkey"
KEYWAITING_STATUS = "keywaiting"
UNCONNECTED_STATUS = "unconnected"

SUCCESS_TEXT = "success"
FAIL_TEXT = "fail"

### For __send_multiple_request() ###
REQ_TYPE_KEY = "reqtype"
REQ_NODE_KEY = "reqnode"

QUERY_XML_STR = \
    '<volctrl><volume><name type="partition">%s</name></volume></volctrl>'

MOUNT_XML_STR = \
    '<volctrl><volume>' + \
    '<name type="partition">%s</name>' + \
    '<dst>%s</dst>' + \
    '</volume></volctrl>'

BIND_MOUNT_XML_STR = \
    '<volctrl>' + \
    ' <option bind="true"/>' + \
    ' <volume>' + \
    '  <src>%s</src>' + \
    '  <dst>%s</dst>' + \
    ' </volume>' + \
    '</volctrl>'

UNMOUNT_XML_STR = \
    '<volctrl><volume><dst>%s</dst></volume></volctrl>'

CHK_EXIST_REQ_XML = '<devctrl><device><name>%s</name></device></devctrl>'

ALLDELPART_REQ_XML = '<diskctrl>' + \
                     ' <option clear="true"/>' + \
                     ' <disk>' + \
                     '   <name>%s</name>' + \
                     ' </disk>' + \
                     '</diskctrl>'

MKPART_REQ_XML = '<diskctrl>' + \
                 ' <disk>' + \
                 '   <name>%s</name>' + \
                 '   <partitionlist ptable="%s">' + \
                 '     <partition>' + \
                 '      <number>1</number>' + \
                 '      <start>%s</start>' + \
                 '      <end>%s</end>' + \
                 '      <name>%s</name>' + \
                 '      <filesystem>%s</filesystem>' + \
                 '     </partition>' + \
                 '   </partitionlist>' + \
                 ' </disk>' + \
                 '</diskctrl>'

QUERY_DISK_REQ_XML = \
    '<diskctrl><disk><name>%s</name></disk></diskctrl>'

EXTRACT_LOCKKEY_KEYSRV_REQ_XML = \
    '<lockkey>' + \
    ' <option operation="extract" keyserver="true"/>' + \
    ' <mac>%s</mac>' + \
    ' <path type="decode">%s</path>' + \
    ' <keyserverlist>' + \
    '  <keyserver>' + \
    '    <hostname>%s</hostname>' + \
    '    <port>%s</port>' + \
    '  </keyserver>' + \
    ' </keyserverlist>' + \
    '</lockkey>'

EXTRACT_USB_LOCKKEY_REQ_XML = \
    '<lockkey>' + \
    '    <option operation="extract" keyserver="false"/>' + \
    '    <mac>%s</mac>' + \
    '    <path type="encode">%s</path>' + \
    '    <path type="decode">%s</path>' + \
    '    <device><name type="usb">%s</name></device>' + \
    '</lockkey>'

EXTRACT_KEYSRV_REQ_BASE_XML = \
    '<lockkey>' + \
    '    <option operation="extract" keyserver="true"/>' + \
    '    <mac>%s</mac>' + \
    '    <path type="decode">%s</path>' + \
    '</lockkey>'

MOUNT_UNMOUNT_EVENT_BODY_BASE_XML = \
    '<volume>' + \
    ' <name>%s</name>' + \
    '</volume>'

MKFSPREPARE_EVENT_BODY_BASE_XML = \
    '<volume>' + \
    ' <name>%s</name>' + \
    '</volume>'

CHECK_ENC_SIG_REQ_XML_TMPL = \
    '<volctrl>' + \
    '  <option operation="chkencsig"/>' + \
    '  <volume>' + \
    '   <name type="disk">%s</name>' + \
    '  </volume>' + \
    '</volctrl>'

MKFS_VFSADD_REQ_XML = \
    '<volctrl>' + \
    ' <option vfsadd="true"/>' + \
    ' <volume encrypt="%s">' + \
    '  <name type="partition">%s</name>' + \
    '  <filesystem>xfs</filesystem>' + \
    ' </volume>' + \
    '</volctrl>'

CTRL_ENC_BASE_XML = '<volctrl>' + \
                    ' <volume>' + \
                    '  <name type="partition">%s</name>' + \
                    '  <name type="crypt">%s</name>' + \
                    ' </volume>' + \
                    '</volctrl>'

VFS_QUERY_XML = '<volctrl>' + \
                ' <volume>' + \
                '  <filesystem>vfsext</filesystem>' + \
                ' </volume>' + \
                '</volctrl>'


WRITE_SIG_REQ_XML_TMPL = \
    '<volctrl>' + \
    '  <option operation="writeencsig" clear="false"/>' + \
    '  <volume>' + \
    '   <name type="disk">%s</name>' + \
    '  </volume>' + \
    '</volctrl>'

CLEAR_SIG_REQ_XML_TMPL = \
    '<volctrl>' + \
    '  <option operation="writeencsig" clear="true"/>' + \
    '  <volume>' + \
    '   <name type="disk">%s</name>' + \
    '  </volume>' + \
    '</volctrl>'

KEYSERVER_LIST_XML = '<keyserverlist/>'

### Volume Status ###
MOUNTED_STATUS = "mounted"
ENC_MOUNTED_STATUS = "encmounted"
UNMOUNTED_STATUS = "unmounted"
LOCKKEY_STATUS = "lockkey"
KEYWAITING_STATUS = "keywaiting"
MOUNTFAIL_STATUS = "mountfail"
WAITING_STATUS = "waiting"
CAPSHORTAGE_STATUS = "capshortage"
CHECKING_STATUS = "checking"
ENC_CHECKING_STATUS = "encchecking"
FORMATTING_STATUS = "formatting"
ENC_FORMATTING_STATUS = "encformatting"
UNFORMATTED_STATUS = "unformated"
MIRRORING_STATUS = "mirroring"
EXIST_DISK_LABEL_STATUS = "existdisklabel"
FAIL_STATUS = "fail"
VFSEXT_STATUS = "vfsext"
UNKNOWN_STATUS = "unknown"
OTHER_MIRROR_STATUS = "othermirror"
RECOVERABLE_MIRROR_STATUS = "recoverablemirror"

# disk status
DISK_STATUS = "status"
DISK_CAPSHORTAGE_STATUS = "capshortage"
# disk status
DISK_NORMAL_STATUS = "normal"
DISK_FAIL_PENDING_STATUS = "failpending"
DISK_SMARTERR_PENDING_STATUS = "smarterrorpending"

VOLUME_STATUS_LIST = (MOUNTED_STATUS, UNMOUNTED_STATUS, LOCKKEY_STATUS,
                      KEYWAITING_STATUS, MOUNTFAIL_STATUS,
                      WAITING_STATUS, CAPSHORTAGE_STATUS,
                      CHECKING_STATUS, FORMATTING_STATUS,
                      ENC_CHECKING_STATUS, ENC_FORMATTING_STATUS,
                      UNFORMATTED_STATUS, MIRRORING_STATUS, OTHER_MIRROR_STATUS, RECOVERABLE_MIRROR_STATUS)
VOLUME_CACHE_FILE_TMPL = \
    os.path.join(config.CONF_TEMP_DIR, "volume", "%s", "%s")

# disk status
DISK_STATUS = "status"
DISK_FAIL_STATUS = "fail"
DISK_SMARTERR_STATUS = "smarterror"
DISK_ADDED = "added"

### Some Value ###
MBR_PART_START = 2048
GPT_PART_START = 40
MAX_2TB_SECTORS = 4294967296
LABEL_STR = "DISK"
MBR_PTABLE = "msdos"
GPT_PTABLE = "gpt"
FAT_FSYS = "fat"

### Lock Name ###
VOL_LOCK_NAME = "volume_info_cache"

MIRROR_USB_VOLUME = "usb3"

FILESYSTEM_CATEGORY = "filesystem"
NOOPERATION_STATE = "nooperation"
FORMATTING_STATE = "formatting"
CHECKING_STATE = "checking"

SATA_DISK_BASE_NAME = "satad"

FORCE_START_DONE = "force_start_done"

TRUE_TEXT = "true"
FALSE_TEXT = "false"

VFSEXT_PORT_MODE = "vfsext"

DISPACH_OK = 0
DISPACH_FAIL = 1


class FalVolctrl(falcmd.FalCmd):
    """FAL Volume Control command implementation class."""

    class __UpdateError(Exception):
        "Update Error"
        pass

    class __DeviceNotFound(Exception):
        "Device Not Found"
        pass

    class __MountFail(Exception):
        "Mount Fail"
        pass

    class __UnmountFail(Exception):
        "Unmount Fail"
        pass

    class __DeleteError(Exception):
        "Delete Error"
        pass

    class __MakepartError(Exception):
        "Make Error"
        pass

    class __KeyFileNotFound(Exception):
        "Key File Not Found Error"
        pass

    class __ExtractLockkeyError(Exception):
        "Extract Lockkey Error"
        pass

    class __KeyserverNotFound(Exception):
        "Keyserver Not Found"
        pass

    class __RequestError(Exception):
        "Request Error"
        pass

    class __InternalError(Exception):
        "Internal Error"
        pass

    class __DiskLabelExist(Exception):
        "DiskLabel Exist"
        pass

    class __RemoveError(Exception):
        "Remove Error"
        pass

    class __AlreadyUnmount(Exception):
        "Alerady unmount"
        pass

    class __AlreadyMount(Exception):
        "Alerady mount"
        pass

    class __MkfsError(Exception):
        "Mkfs Error"
        pass

    class __GetError(Exception):
        "Get Error"
        pass

    class __SaveError(Exception):
        "Save Error"
        pass

    class __StartError(Exception):
        "Start Error"
        pass

    def __is_exist_node(self, node, node_path):
        """Check whether node_path exists in specified node"""

        try:
            node.get_node(node_path)
        except NotFoundError:
            return False
        return True

    def __vol_name2hdd_name_list(self, vol_name):
        """Convert vol name to HDD name list."""

        return config.HDD_DEV_LIST_IN_VOL[
                   util.get_cuurent_array_num()][vol_name]

    def __vol_name2dev_name(self, vol_name):
        """Convert vol name to partition name."""

        return VOL_INFO[vol_name]["dev"]

    def __vol_name2enc_part_name(self, vol_name):

        return DEV_TO_ENC_DEV[VOL_INFO[vol_name]["dev"]]

    def __vol_name2first_part_name(self, vol_name):
        """Convert vol name to partition name."""

        if not VOL_INFO[vol_name]["part_base"]:
            return VOL_INFO[vol_name]["dev"]

        return VOL_INFO[vol_name]["part_base"] + "1"

    def __vol_name2mount_point(self, vol_name):
        """Convert vol name to mount point name."""

        return VOL_INFO[vol_name]["mnt_point"]

    def __vol_name2alt_mount_point(self, vol_name):
        """Convert vol name to alternate mount point name."""

        return VOL_INFO[vol_name]["alt_mnt_point"]

    def __partition2disk(self, part_name):
        """Convert partition name to corresponding disk name."""

        match = re.match("([a-zA-Z]+[0-9]+)", part_name)

        disk_name = match.group(1)

        if disk_name not in config.SATA_HDD_DEV_LIST + config.USB_HDD_DEV_LIST:
            raise NotFoundError

        return disk_name

    def __dev_name2vol_name(self, dev_name):

        for vol_name, vol_info in VOL_INFO.iteritems():

            if dev_name.startswith(vol_info["dev"]):
                return vol_name

        return ""

    def __append_partition_name(self, vol_node, is_enc=False, is_vfs=False):
        """Append the name tag with partition type attribure"""

        # vfsext用のelement指定
        if is_vfs:
            vfs_element_list = \
                self.__get_enable_vfs_element_list(is_enc=is_enc,
                                                   is_chk_external_status=True)
            vol_node.append_child(VFSEXT_NAME)
            vfsext_node = vol_node.get_node(VFSEXT_NAME)
            vfsext_node.append_child(PARTITION_LIST_NAME)
            partlist_node = vfsext_node.get_node(PARTITION_LIST_NAME)

            for element in vfs_element_list:
                partlist_node.append_child(PARTITION_NAME, element)

        vol_name = vol_node.get_node(NAME_NAME).get_text()
        if is_enc:
            enc_part_name = self.__vol_name2enc_part_name(vol_name)
            vol_node.append_child(NAME_NAME,
                                  enc_part_name,
                                  {TYPE_ATTR: PARTITION_ATTR_VAL})
            return

        part_name = self.__vol_name2first_part_name(vol_name)
        vol_node.append_child(NAME_NAME, part_name,
                              {TYPE_ATTR: PARTITION_ATTR_VAL})
        return

    def __append_enc_partition_name(self, vol_node):
        """Append the name tag with partition type attribure"""

        vol_name = vol_node.get_node(NAME_NAME).get_text()
        enc_part_name = self.__vol_name2enc_part_name(vol_name)
        vol_node.append_child(NAME_NAME, enc_part_name,
                              {TYPE_ATTR: CRYPT_ATTR_VAL})
        return

    def __send_multiple_request(self, target_module, req_param_list=[]):
        """Send request to pal module(multiple entry)"""

        entrylist = request.EntryList()
        for req_param in req_param_list:
            entrylist.append_entry(request.Entry(req_param[REQ_TYPE_KEY],
                                                 req_param[REQ_NODE_KEY]))
        return self.send_entrylist(entrylist, target_module).get_entry_list()

    def __send_request(self, target_module, type, child_xml=""):
        """Send request to pal module(single entry)"""

        entry = request.Entry(type, child_xml)
        return self.send_entry(entry, target_module)

    def __send_request_with_file(self, target_module,
                                 type, file, child_xml=""):
        """Send request to pal module(single entry)"""

        entry = request.Entry(type, child_xml)
        return self.send_entry_with_file(entry, target_module, file)

    def __set_volume_cache_value(self, vol_name, info_dict):
        """Save volume status to temp file"""

        dummy_path = os.path.join(VOLUME_CACHE_FILE_TMPL % (vol_name, "dummy"))
        dir_path = os.path.dirname(dummy_path)
        if not os.path.isdir(dir_path):
            os.makedirs(dir_path)

        for key, value in info_dict.iteritems():
            file_path = os.path.join(dir_path, key)

            with fileut.FLock(config.LOCK_DIR, VOL_LOCK_NAME, exclusive=False):
                with open(file_path, "w") as fh:
                    fh.write(value)
        return

    def __get_volume_cache_value(self, vol_name, file_name):
        """Load volume status to temp file"""

        file_path = \
            os.path.join(VOLUME_CACHE_FILE_TMPL % (vol_name, file_name))

        if not os.path.isfile(file_path):
            return ""

        with fileut.FLock(config.LOCK_DIR, VOL_LOCK_NAME, exclusive=False):
            with open(file_path, "r") as fh:
                return fh.read().rstrip()

    def __load_config(self, conffile_name):
        """Load config data"""

        load_entry = request.Entry(ReqTypes.LOAD)
        result_entry = self.send_entry_with_file(load_entry,
                                                 pal_info.PAL_CONFFILE_PATH,
                                                 conffile_name)
        return result_entry.get()

    # FIXME ライブラリ化
    def __is_current_raid_mode_vfsext(self):

        VFSEXT_FSTYPE = "vfsext"

        conf_node = self.__load_config(config.RAIDCTRL_CONF_FILE)
        level = conf_node.get_value("raidlist/raid/level")
        if VFSEXT_FSTYPE == config.RAID_MODE_TO_REAL_RAID_MODE[level]:
            return True
        return False

    def __force_make_dirs(self, dir_path):
        if not os.path.isdir(dir_path):
            os.makedirs(dir_path)

    def __get_macaddr(self):
        """Get system MAC address"""

        try:
            mac_file_path = os.path.join(config.CONF_DIR, "lockkey", "mac")
            self.__force_make_dirs(os.path.dirname(mac_file_path))

            if not os.path.isfile(mac_file_path):
                mac = util.get_mac_address("netif1").replace(":", "").upper()
                with open(mac_file_path, "w") as fh:
                    fh.write(mac)
                return mac

            with open(mac_file_path, "r") as fh:
                mac = fh.read().rstrip()
                return mac.upper()
        except:
            try:
                return util.get_mac_address("netif1").replace(":", "").upper()
            except:
                pass

        return "000000000000"

    def __dispatch_disk_update_event(self):
        try:
            dispatch_event('disk', 'update', "raidctrl")
        except:
            self.exception("fail to __dispatch_disk_update_event")
        return

    def __dispatch_raid_update_event(self):
        try:
            dispatch_event('raid', 'update', "raidctrl")
        except:
            self.exception("fail to __dispatch_raid_update_event")
        return

    def __dispatch_mount_event(self, vol_name,
                               is_previous=False,
                               is_mountfail=False):

        volume_node = XmlNode(MOUNT_UNMOUNT_EVENT_BODY_BASE_XML % vol_name)

        if not is_previous:
            if is_mountfail:
                volume_node.set_attrs({MOUNT_FAIL_ATTR: TRUE_ATTR_VAL})
            else:
                volume_node.set_attrs({MOUNT_FAIL_ATTR: FALSE_ATTR_VAL})

        dispatch_event('volume', 'mount', volume_node, is_previous)
        return

    def __dispatch_unmount_event(self, vol_name, is_previous=False):

        volume_node = XmlNode(MOUNT_UNMOUNT_EVENT_BODY_BASE_XML % vol_name)
        dispatch_event('volume', 'unmount', volume_node, is_previous)
        return

    def __dispatch_mkfsprepare_event(self, vol_name):

        try:
            volume_node = XmlNode(MKFSPREPARE_EVENT_BODY_BASE_XML % vol_name)
            dispatch_event('volume', 'mkfsprepare', volume_node)
        except:
            self.exception("fail to __dispatch_mkfsprepare_event")
            return DISPACH_FAIL
        return DISPACH_OK

    def __delete_disk_from_system(self, disk_name):

        try:
            util.delete_disk_from_system(disk_name)
        except:
            self.exception("fail to delete_disk_from_system(%s)" % disk_name)
            pass

    def __is_volume_have_non_enc_filesystem(self, vol_name):
        """暗号化されていないファイルシステムが存在するかどうか"""

        QUERY_REQ_BASE_XML = '<volctrl>' + \
                             ' <volume>' + \
                             '  <name type="disk">%s</name>' + \
                             ' </volume>' + \
                             '</volctrl>'
        FSYS_XPATH = "./volume/filesystem"
        PART_NAME_XPATH = "./volume/name[@type='partition']"

        disk_name = VOL_INFO[vol_name]["dev"]
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.QUERY,
                                           QUERY_REQ_BASE_XML % disk_name)
        try:
            if result_entry.get().get_node(
                    PART_NAME_XPATH).is_set_to_true(ENCRYPT_ATTR):
                return False
        except:
            pass

        if result_entry.get().get_value(FSYS_XPATH, None) is not None:
            return True

        return False

    def __is_encrypted_volume(self, vol_name):

        if INTERNAL_VOL == vol_name:

            volctrl_conf_node = self.__load_config(config.VOLCTRL_CONF_FILE)
            encrypt_node = volctrl_conf_node.get_node(INTRNAL_ENCRIPT_XPATH)
            if encrypt_node.is_enabled():
                return True
            return False

        if vol_name.startswith("usb"):

            if not self.__is_external_vfsext_setting(vol_name):
                if self.__is_volume_have_non_enc_filesystem(vol_name):
                    return False

            disk_name = self.__vol_name2hdd_name_list(vol_name)[0]

            if not self.__is_device_exist(disk_name):
                return False

            result_entry = \
                self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                    ReqTypes.CONTROL,
                                    CHECK_ENC_SIG_REQ_XML_TMPL % disk_name)
            if result_entry.get_result() == Results.OK:
                return True
            return False

        raise self.__InternalError

    def __is_lockkey(self, vol_name, is_need_mount=False):

        if not vol_name.startswith("usb"):
            return False

        if is_need_mount:
            self.__mount_lockkey(vol_name)

        mnt_path = VOL_INFO[vol_name]["mnt_point"]
        key_file_pattern = \
            os.path.join(mnt_path, config.LOCKKEY_DIR_PREFIX)
        key_file_list = glob.glob(key_file_pattern + "*/*")

        if is_need_mount:
            self.__unmount_lockkey(vol_name)

        if 0 < len(key_file_list):
            return True

        return False

    def __get_volume_info_from_pal(self, vol_name):

        QUERY_REQ_BASE_XML = '<volctrl>' + \
                             ' <volume>' + \
                             '  <name type="disk">%s</name>' + \
                             ' </volume>' + \
                             '</volctrl>'

        if self.__is_current_raid_mode_vfsext() and \
           INTERNAL_VOL == vol_name:
            result_entry = self.__get_vfs_query()
        else:
            disk_name = VOL_INFO[vol_name]["dev"]
            result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                               ReqTypes.QUERY,
                                               QUERY_REQ_BASE_XML % disk_name)

        if result_entry.get_result() == Results.DEVICE_NOT_FOUND:
            return {STATUS_NAME: UNCONNECTED_STATUS,
                    NAME_NAME: "",
                    FSYS_NAME: "",
                    TOTAL_NAME: "",
                    USE_NAME: "",
                    ENCRYPT_ATTR: ""}

        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        volctrl_node = result_entry.get()
        info_dict = {}
        info_dict[STATUS_NAME] = volctrl_node.get_value(VOL_STATUS_XPATH, "")
        info_dict[NAME_NAME] = volctrl_node.get_value(PART_NAME_XPATH, "")
        info_dict[FSYS_NAME] = volctrl_node.get_value(FILESYSTEM_XPATH, "")
        info_dict[TOTAL_NAME] = volctrl_node.get_value(TOTAL_SIZE_XPATH, "")
        info_dict[USE_NAME] = volctrl_node.get_value(USE_SIZE_XPATH, "")

        try:
            encdevflg = volctrl_node.get_node(PART_NAME_XPATH).\
                            is_set_to_true(ENCRYPT_ATTR)
            info_dict[ENCRYPT_ATTR] = str(encdevflg).lower()
        except:
            info_dict[ENCRYPT_ATTR] = "false"

        return info_dict

    def __get_usable_query_response(self, status, part_name):

        QUERY_REQ_BASE_XML = '<volctrl>' + \
                             ' <volume>' + \
                             '  <name type="partition">%s</name>' + \
                             ' </volume>' + \
                             '</volctrl>'
        if not part_name:
            return None

        if MOUNTED_STATUS != status and ENC_MOUNTED_STATUS != status:
            return None

        result_entry = \
            self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                ReqTypes.QUERY,
                                QUERY_REQ_BASE_XML % part_name)
        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        # 暗号化ボリューム
        if ENC_MOUNTED_STATUS == status:
            result_entry.get().get_node(FILESYSTEM_XPATH).set_attrs(
                {ENCRYPT_ATTR: TRUE_ATTR_VAL})
            status = MOUNTED_STATUS

        result_entry.get().get_node(VOL_STATUS_XPATH).set_text(status)
        return result_entry

    def __is_device_exist(self, dev_name):
        """Check wheather specified device exists."""

        result_entry = self.__send_request(pal_info.PAL_DEVCTRL_PATH,
                                           ReqTypes.CHK_EXIST,
                                           CHK_EXIST_REQ_XML % dev_name)
        if Results.OK == result_entry.get_result():
            return True

        return False

    def __get_enable_keyserver_node(self):

        KEYSERVER_ENABLE_XPATH = "keyserverlist/keyserver[@enable='true']"
        conf_entry = self.__load_config(config.LOCKKEY_CONF_FILE)
        enable_kersrv_node_list = \
            conf_entry.get_node_list(KEYSERVER_ENABLE_XPATH)

        if 0 == len(enable_kersrv_node_list):
            raise self.__KeyserverNotFound

        keyserverlist_node = XmlNode(KEYSERVER_LIST_XML)
        for keysrv_node in enable_kersrv_node_list:
            keyserverlist_node.append_child(keysrv_node)

        return keyserverlist_node

    def __delete_all_partition(self, vol_name):
        """Delete All partition.
           Specified volume must be the volume of USB external device.
        """

        if vol_name not in config.STOR_IF_TO_VOL["usb"].values():
            raise

        dev_name = self.__vol_name2hdd_name_list(vol_name)[0]

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.DELPART,
                                           ALLDELPART_REQ_XML % dev_name)
        if Results.OK != result_entry.get_result():
            raise self.__DeleteError
        return

    def __get_hdd_size(self, dev_name):

        DISK_SIZE_XPATH = "./disk/size"

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.QUERY,
                                           QUERY_DISK_REQ_XML % dev_name)

        if Results.OK != result_entry.get_result():
            raise self.__DeviceNotFound

        return int(result_entry.get().get_node(DISK_SIZE_XPATH).get_text())

    def __get_ptable_and_last_lba(self, disk_size):

        last_lba = 0
        ptable = ""

        if MAX_2TB_SECTORS < disk_size:
            ptable = GPT_PTABLE
            last_lba = disk_size - GPT_PART_START - 1 - \
                config.LANDISK_INFO_SECTOR_SIZE

        else:
            ptable = MBR_PTABLE
            last_lba = \
                (disk_size / 255 / 63) * 255 * 63 - 1 - \
                    config.LANDISK_INFO_SECTOR_SIZE

        return ptable, last_lba

    def __mkpart(self, vol_name, filesystem):

        dev_name = self.__vol_name2hdd_name_list(vol_name)[0]

        size = self.__get_hdd_size(dev_name)

        if FAT_FSYS == filesystem:
            if MAX_2TB_SECTORS < size:
                size = MAX_2TB_SECTORS

        ptable, last_lba = self.__get_ptable_and_last_lba(size)

        if ptable == MBR_PTABLE:
            req_xml = MKPART_REQ_XML % (dev_name,
                                        MBR_PTABLE,
                                        MBR_PART_START,
                                        str(last_lba),
                                        LABEL_STR,
                                        filesystem)
        elif ptable == GPT_PTABLE:
            req_xml = MKPART_REQ_XML % (dev_name,
                                        GPT_PTABLE,
                                        GPT_PART_START,
                                        str(last_lba),
                                        LABEL_STR,
                                        filesystem)

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.MKPART,
                                           req_xml)
        if Results.OK != result_entry.get_result():
            raise self.__MakepartError

        return

    def __is_need_bind_mount(self, vol_name):

        USB_PORT_ENABLE_MODE_XPATH = \
           "usblist/usb[@port='%s']/modelist/mode[@enable='true']"

        if INTERNAL_VOL == vol_name:
            return True

        if not vol_name.startswith("usb"):
            return False

        portconf_node = self.__load_config(config.PORTS_CONF_FILE)

        try:

            match = re.match("usb([1-9])", vol_name)
            port_num = match.group(1)
            mode = portconf_node.get_node(\
                USB_PORT_ENABLE_MODE_XPATH % port_num).get_text()
            if "share" == mode:
                return True
        except:
            return False

        return False

    def __get_raiddev_vs_part_name_table(self, disk_name, is_sys_only=False):

        table = {}

        for raiddev_name, element_list in \
                util.get_raid_element_info().iteritems():

            if is_sys_only:
                if raiddev_name not in config.SYSTEM_RAID_DEV_LIST:
                    continue

            for element in element_list:
                if element.startswith(disk_name):
                    table.update({raiddev_name: element})
        return table

    def __remove_disk_from_all_raid(self, disk_name, is_sys_only=False):

        REMOVE_REQ_BASE_XML = \
            '<raidctrl><raid>' + \
            '  <name type="raid"/>' + \
            '  <partition><name/></partition>' + \
            '</raid></raidctrl>'

        is_reqest_error = False

        # ディスクがどのRAIDにどのパーティションとして存在するか取得
        raid_part_table = \
            self.__get_raiddev_vs_part_name_table(disk_name, is_sys_only)

        raidctrl_node = XmlNode(REMOVE_REQ_BASE_XML)
        for raiddev_name, part_name in raid_part_table.iteritems():
            raidctrl_node.get_node("raid/name").set_text(raiddev_name)
            raidctrl_node.get_node("raid/partition/name").set_text(part_name)

            result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                               ReqTypes.REMOVE,
                                               raidctrl_node)

            if result_entry.get_result() != Results.OK:
                is_reqest_error = True

        self.__delete_disk_from_system(disk_name)

        if is_reqest_error:
            raise self.__RemoveError

        return

    def __vfs_add_partition(self, part_name, is_enc=False):

        VFS_ADD_BASE_REQ_XML = \
            '<volctrl>' + \
            ' <option vfsadd="true"/>' + \
            ' <volume>' + \
            '  <filesystem encrypt="%(is_enc)s"/>' + \
            '  <partition><name>%(part)s</name></partition>' + \
            ' </volume>' + \
            '</volctrl>'

        req_xml = VFS_ADD_BASE_REQ_XML % {"is_enc": str(is_enc).lower(),
                                          "part": part_name}
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.MOUNT,
                                           req_xml)
        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

    def __mkfs_vfs_partition(self, part_name, is_enc=False):

        def wait_mkfs_end():

            FILESYSTEM_CATEGORY = "filesystem"
            FORMATTING_STATE = "formatting"

            while(True):
                time.sleep(10)
                state = util.statemgr_query(FILESYSTEM_CATEGORY)
                if FORMATTING_STATE != state:
                    break

        for idx in range(0, 100):  # 100回LOOPして終わらなければアウト

            req_xml = MKFS_VFSADD_REQ_XML % (str(is_enc).lower(), part_name)
            result_entry = \
                self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                    ReqTypes.MKFS,
                                    req_xml)

            result_code = result_entry.get_result()
            if Results.OK == result_code:
                wait_mkfs_end()

                try:
                    disk_name = self.__partition2disk(part_name)
                    if "1" == util.get_disk_info(disk_name,
                                                 config.VFS_ADD_MKFS_ERR):
                        util.set_disk_info(disk_name,
                                           DISK_STATUS,
                                           DISK_FAIL_STATUS)
                        util.handle_error_disk(disk_name,
                                               DISK_FAIL_STATUS,
                                               True)
                        raise self.__MkfsError
                except:
                    self.exception(
                        "VFS_ADD_MKFS_ERR(%s)" % disk_name)
                    raise self.__MkfsError

                break

            if Results.BUSY_ERROR == result_code:
                time.sleep(10)
                continue

            raise self.__MkfsError

        return

    def __clear_vfs_autorebuild_flag(self, disk_name):
        """Clear need vfs autorebuild flag
           This flag file is for VFS only
        """

        try:
            util.set_disk_info(disk_name, config.NEED_VFS_AUTO_REBUILD, "0")
        except:
            pass

    def __all_clear_vfs_autorebuild_flag(self):

        for element in config.EXT_RAID_DEV_INFO.keys():
            try:
                disk_name = self.__partition2disk(element)
                if util.is_need_vfs_autorebuild(disk_name):
                    self.__clear_vfs_autorebuild_flag(disk_name)
            except:
                pass

    def __is_error_disk(self, disk_name):

        disk_status = util.get_disk_info(disk_name, "status")
        if disk_status in [DISK_FAIL_STATUS, DISK_SMARTERR_STATUS]:
            return True
        return False

    def __get_error_vfs_element_list(self, element_list):

        error_element_name_list = []
        for element in element_list:
            try:
                disk_name = self.__partition2disk(element)
                if self.__is_error_disk(disk_name):
                    error_element_name_list.append(element)
            except:
                continue

        return error_element_name_list

    def __get_usable_external_vfsext_port_element_list(self):
        """Get usable external vfsext port.
           ex.
              When the port mode of usb3 is vfsext and
              usb3 HDD is usable this device name is add to list
        """

        usable_element_list = []
        for element_name in util.get_support_external_vfsext_element_list():

            disk_name = self.__partition2disk(element_name)
            iface_name = config.DEV_TO_PORT_IF[disk_name]
            port_num = config.DEV_TO_PORT_NO[disk_name]

            if VFSEXT_PORT_MODE == \
                   util.get_current_port_mode(iface_name, port_num):

                # 存在確認
                if not self.__is_device_exist(disk_name):
                    continue

                usable_element_list.append(element_name)

        return usable_element_list

    def __do_vfs_mount_by_specify_elements(self, entry):

        VFSEXT_XPATH = "./volume/vfsext"

        VFS_MOUNT_XML_STR = \
            '<volctrl>' + \
            ' <volume>' + \
            '  <name type="partition">%s</name>' + \
            '  <dst>%s</dst>' + \
            '  <filesystem encrypt="%s">vfsext</filesystem>' + \
            ' </volume>' + \
            '</volctrl>'

        part_name = VOL_INFO[INTERNAL_VOL]["dev"]
        mount_point = self.__vol_name2mount_point(INTERNAL_VOL)

        # 暗号化チェック
        is_enc = False
        if self.__is_encrypted_volume(INTERNAL_VOL):
            is_enc = True
        req_xml = VFS_MOUNT_XML_STR % \
                      (part_name, mount_point, str(is_enc).lower())
        req_node = XmlNode(req_xml)

        # 上位から指定されたvfsextの要素を追加
        vfsext_node = entry.get().get_node(VFSEXT_XPATH)
        req_node.get_node(VOLUME_NAME).append_child(vfsext_node)

        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.MOUNT,
                                           req_node)
        result_code = result_entry.get_result()
        if result_code == Results.ALREADY_MOUNT:
            raise self.__AlreadyMount

        if result_code != Results.OK:
            raise self.__MountFail

        return

    def __get_autorebuild_result_node(self, auto_rebuild_reuslt_disk_map):

        MOUNT_RESULT_XML_TMPL = \
            '<result>' + \
            '  <autorebuild/>' + \
            '</result>'

        DISK_XML_TMPL = \
            '<disk>' + \
              '<name>%(name)s</name>' + \
              '<status>%(status)s</status>' + \
            '</disk>'

        result_node = XmlNode(MOUNT_RESULT_XML_TMPL)
        autorebuild_node = result_node.get_node(AUTOREBUILD_NAME)

        for disk_name, result in auto_rebuild_reuslt_disk_map.iteritems():
            autorebuild_node.append_child(
                XmlNode(DISK_XML_TMPL % {"name": disk_name,
                                         "status": result}))
        return result_node

    def __do_vfs_mount_by_auto_detect_elements(self, entry):

        VFS_MOUNT_XML_STR = \
            '<volctrl>' + \
            ' <volume>' + \
            '  <name type="partition">%s</name>' + \
            '  <dst>%s</dst>' + \
            '  <filesystem encrypt="%s">vfsext</filesystem>' + \
            '  <vfsext><partitionlist/></vfsext>' + \
            ' </volume>' + \
            '</volctrl>'

        PARTLIST_XPATH = "./volume/vfsext/partitionlist"

        vol_name = INTERNAL_VOL
        part_name = VOL_INFO[vol_name]["dev"]
        mount_point = self.__vol_name2mount_point(vol_name)

        vfsext_dev_info = util.get_vfsext_dev_info()

        # vfs elemen候補
        vfs_element_list = util.get_be_set_vfsexe_element_flag_list()

        # auto rebuild候補 element
        added_part_name_list = util.get_need_vfs_auto_rebuild_element_list()

        # 暗号化チェック
        is_enc = self.__is_encrypted_volume(INTERNAL_VOL)

        req_node = XmlNode(VFS_MOUNT_XML_STR % \
                           (part_name, mount_point, str(is_enc).lower()))
        partlist_node = req_node.get_node(PARTLIST_XPATH)
        for element in vfs_element_list:
            if not self.__is_device_exist(element):
                continue
            partlist_node.append_child(PARTITION_NAME, element)

        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.MOUNT,
                                           req_node)

        result_code = result_entry.get_result()
        is_already_mount = (result_code == Results.ALREADY_MOUNT)

        if result_code not in [Results.OK, Results.ALREADY_MOUNT]:
            raise self.__MountFail

        if len(added_part_name_list) == 0:
            if is_already_mount:
                raise self.__AlreadyMount
            return

        # VFS用autorebuild
        no_added_part_name_list = []
        reuslt_disk_map ={}
        for part_name in added_part_name_list:

            try:
                disk_name = self.__partition2disk(part_name)
                if is_enc:
                    enc_part_name = DEV_TO_ENC_DEV[part_name]
                    self.__do_enc_start_partition(part_name, enc_part_name)
                    self.__mkfs_vfs_partition(enc_part_name, True)
                else:
                    self.__mkfs_vfs_partition(part_name)

                reuslt_disk_map.update({disk_name: SUCCESS_TEXT})
            except:
                self.exception("fail to mkfs vfs element(%s)!! " % part_name)
                no_added_part_name_list.append(part_name)
                reuslt_disk_map.update({disk_name: FAIL_TEXT})

        if len(added_part_name_list) > 0:
            for added_part_name in added_part_name_list:
                if added_part_name not in no_added_part_name_list:

                    add_disk_name = self.__partition2disk(added_part_name)

                    try:
                        self.__vfs_add_partition(added_part_name, is_enc)
                        reuslt_disk_map.update({add_disk_name: SUCCESS_TEXT})
                    except:
                        reuslt_disk_map.update({add_disk_name: FAIL_TEXT})
                        continue

                    try:
                        util.set_vfsexe_element_flag(add_disk_name)
                    except:
                        self.exception("fail to set_vfsexe_element_flag()")
                        pass

        # 結果を返すために
        entry.get().append_child(
            self.__get_autorebuild_result_node(reuslt_disk_map))

        cur_vfs_element_list = util.get_current_vfs_element_list()
        for element in cur_vfs_element_list:
            if vfsext_dev_info[element]["is_external"]:
                vol_name = self.__dev_name2vol_name(element)
                self.__set_volume_cache_value(vol_name,
                                              {"status": VFSEXT_STATUS})
        if is_already_mount:
            raise self.__AlreadyMount

        return

    def __do_vfs_mount(self, entry):

        VFSEXT_PARTLIST_XPATH = "./volume/vfsext/partitionlist"

        if self.__is_exist_node(entry.get(), VFSEXT_PARTLIST_XPATH):
            self.__do_vfs_mount_by_specify_elements(entry)
            return

        self.__do_vfs_mount_by_auto_detect_elements(entry)
        return

    def __do_bind_mount(self, vol_name):

        mnt_point = self.__vol_name2mount_point(vol_name)
        if INTERNAL_VOL == vol_name:
            mnt_point = os.path.join(mnt_point, "share")
            if not os.path.isdir(mnt_point):
                os.makedirs(mnt_point)

        alt_mnt_point = self.__vol_name2alt_mount_point(vol_name)

        req_xml = BIND_MOUNT_XML_STR % (mnt_point, alt_mnt_point)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.MOUNT,
                                           req_xml)

        result_code = result_entry.get_result()
        if Results.OK == result_code or \
           Results.ALREADY_MOUNT == result_code:
            return

        raise self.__MountFail

    def __is_used_as_embedded_disk(self, disk_name):

        MIRROR_DISK_PART_LABEL_LIST = ("efisystem", "rootfs", "swap")

        PART_NAME_XPATH_TMPL = \
            "./disk/partitionlist/partition/name[text()='%s']"

        try:
            diskctrl_node = util.pal_diskctrl_query(disk_name).get()
        except:
            return False

        for part_label in MIRROR_DISK_PART_LABEL_LIST:
            if self.__is_exist_node(diskctrl_node,
                                    PART_NAME_XPATH_TMPL % part_label):
                return True

        return False

    def __get_system_partition_num_list(self, disk_name):

        UNUSABLE_PART_NAME_LIST = \
            ["Microsoft reserved partition".lower(),
             "EFI System Partition".lower(), ]

        PART_XPATH = "./disk/partitionlist/partition"

        try:
            diskctrl_node = util.pal_diskctrl_query(disk_name).get()
        except:
            return []

        sys_part_num_list = []
        part_node_list = diskctrl_node.get_node_list(PART_XPATH)
        for part_node in part_node_list:
            try:
                part_name = part_node.get_value(NAME_NAME, None)
                if part_name.lower() in UNUSABLE_PART_NAME_LIST:
                    part_num = part_node.get_value(NUMBER_NAME, None)
                    if not part_num:
                        continue
                    sys_part_num_list.append(int(part_num))
            except:
                continue

        return sys_part_num_list

    def __try_mount_all_target(self, entry, vol_name):

        if INTERNAL_VOL == vol_name:
            if self.__is_current_raid_mode_vfsext():
                try:
                    self.__do_vfs_mount(entry)
                except self.__AlreadyMount:
                    raise self.__AlreadyMount
                except self.__MountFail:
                    try:
                        self.__vfs_enc_stop()
                    except:
                        pass
                    raise
                return

        if self.__is_mirroring_setting(vol_name):
            raise self.__MountFail

        if vol_name.startswith("usb"):
            if self.__is_used_as_embedded_disk(
                self.__vol_name2hdd_name_list(vol_name)[0]):
                self.__set_volume_cache_value(vol_name,
                    {STATUS_NAME: EXIST_DISK_LABEL_STATUS})
                raise self.__DiskLabelExist

        mnt_point = self.__vol_name2mount_point(vol_name)
        part_name_list = []

        enc_part_name = self.__vol_name2enc_part_name(vol_name)
        if self.__is_device_exist(enc_part_name):
            part_name_list.append(enc_part_name)

        part_name_list.append(VOL_INFO[vol_name]["dev"])

        part_base = VOL_INFO[vol_name]["part_base"]
        if "" != part_base:
            # 外付けの場合
            disk_name = self.__vol_name2hdd_name_list(vol_name)[0]
            sys_part_num_list = \
                self.__get_system_partition_num_list(disk_name)

            for idx in range(1, 16):
                if idx in sys_part_num_list:
                    continue
                part_name_list.append(part_base + str(idx))

        for part_name in part_name_list:
            try:
                self.__do_mount(part_name, mnt_point)
                break
            except self.__AlreadyMount:
                raise self.__AlreadyMount
            except self.__MountFail:
                if enc_part_name == part_name:
                    raise self.__MountFail
                continue
        else:  # loop over
            raise self.__MountFail

        return

    def __do_mount(self, part_name, mount_point):

        if not self.__is_device_exist(part_name):
            raise self.__MountFail

        req_xml = MOUNT_XML_STR % (part_name, mount_point)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.MOUNT,
                                           req_xml)
        result_code = result_entry.get_result()

        if Results.OK == result_code:
            return

        if Results.ALREADY_MOUNT == result_code:
            raise self.__AlreadyMount

        raise self.__MountFail

    def __do_bind_unmount(self, vol_name):

        alt_mnt_point = self.__vol_name2alt_mount_point(vol_name)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.UNMOUNT,
                                           UNMOUNT_XML_STR % (alt_mnt_point))

        result_code = result_entry.get_result()

        if Results.OK == result_code:
            return

        if Results.ALREADY_UNMOUNT == result_code:
            raise self.__AlreadyUnmount

        raise self.__UnmountFail

    def __do_unmount(self, vol_name):

        mnt_point = self.__vol_name2mount_point(vol_name)
        req_xml = UNMOUNT_XML_STR % (mnt_point)
        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.UNMOUNT,
                                           req_xml)

        result_code = result_entry.get_result()

        if Results.OK == result_code:
            return

        if Results.ALREADY_UNMOUNT == result_code:
            raise self.__AlreadyUnmount

        raise self.__UnmountFail

    def __do_swap_get(self):
        """get current swap list"""

        SWAP_GET_REQ_XML = \
            """<volctrl>
                 <option operation="swapget"/>
               </volctrl>"""

        return self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                   ReqTypes.CONTROL,
                                   SWAP_GET_REQ_XML)

    def __do_swap_on(self, swap_part_name):
        """Enable swap"""

        SWAP_ON_REQ_XML = \
            """<volctrl>
                 <option operation="swapon"/>
                 <volume>
                   <name type="partition">%s</name>
                 </volume>
               </volctrl>"""

        return self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                   ReqTypes.CONTROL,
                                   SWAP_ON_REQ_XML % swap_part_name)

    def __do_swap_off(self, swap_part_name):
        """Disable swap"""

        SWAP_OFF_REQ_XML = \
            """<volctrl>
                 <option operation="swapoff"/>
                 <volume>
                   <name type="partition">%s</name>
                 </volume>
               </volctrl>"""

        return self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                   ReqTypes.CONTROL,
                                   SWAP_OFF_REQ_XML % swap_part_name)

    def __do_swap_create(self, swap_part_name):
        """Enable swap"""

        SWAP_CREATE_REQ_XML = \
            """<volctrl>
                 <option operation="swapcreate"/>
                 <volume>
                   <name type="partition">%s</name>
                 </volume>
               </volctrl>"""

        return self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                   ReqTypes.CONTROL,
                                   SWAP_CREATE_REQ_XML % swap_part_name)

    def __swap_restart(self, is_enc=False):

        def get_current_swap_list():

            PATRTITION_XPATH = "./partitionlist/partition"

            result_entry = self.__do_swap_get()

            self.debug(str(result_entry))
            if result_entry.get_result() != Results.OK:
                raise self.__RequestError

            part_node_list = result_entry.get().get_node_list(PATRTITION_XPATH)
            return [node.get_text() for node in part_node_list]

        def get_swap_list(is_enc=False):

            is_mirroring = False
            if self.__is_mirroring_setting(MIRROR_USB_VOLUME):

                status = self.__get_volume_cache_value(MIRROR_USB_VOLUME,
                                                       STATUS_NAME)
                if MIRRORING_STATUS == status:
                    is_mirroring = True

            arry_num = util.get_cuurent_array_num()

            swap_list = VOL_INFO[INTERNAL_VOL]["swap_dev_list"][arry_num]
            if is_mirroring:
                swap_list += \
                    VOL_INFO[MIRROR_USB_VOLUME]["swap_dev_list"][arry_num]

            if not is_enc:
                return swap_list

            enc_swap_list = []
            for swap in swap_list:
                enc_swap_list.append(DEV_TO_ENC_DEV[swap])

            return enc_swap_list

        def normal_current_swap_off():

            swap_list = get_current_swap_list()

            for swap in swap_list:
                try:
                    self.__do_swap_off(swap)
                except:
                    self.debug("fail to __swap_off(%s)" % swap)
                    continue

            # 暗号化パーティションが存在する場合は暗号化stop
            for enc_swap in get_swap_list(True):
                try:
                    self.__do_enc_stop_partition(enc_swap)
                except:
                    self.debug("fail to enc swap stop")
                    continue

        def all_swap_create(swap_part_name_list):

            for swap_part_name in swap_part_name_list:
                try:
                    self.__do_swap_create(swap_part_name)
                except:
                    self.debug("fail to __swap_create(%s)" % swap_part_name)
                    continue

        def all_swap_on(swap_part_name_list):

            for swap_part_name in swap_part_name_list:
                try:
                    self.__do_swap_on(swap_part_name)
                except:
                    self.debug("fail to __swap_on(%s)" % swap_part_name)
                    continue

        def enc_start_swap():

            part_list = get_swap_list(False)
            self.debug(str(part_list))

            enc_part_list = get_swap_list(True)

            part_enc_map = dict(zip(part_list, enc_part_list))

            for part_name, enc_part_name in part_enc_map.iteritems():
                try:
                    self.__do_enc_start_partition(part_name, enc_part_name)
                except:
                    self.debug("fail to enc start " + part_name)

        normal_current_swap_off()
        if is_enc:
            enc_start_swap()

        swap_list = get_swap_list(is_enc)
        all_swap_create(swap_list)
        all_swap_on(swap_list)
        return

    def __do_encrypt_start(self, vol_name):

        CTRL_ENC_BASE_XML = '<volctrl><volume/></volctrl>'

        enc_part_name = self.__vol_name2enc_part_name(vol_name)
        if self.__is_device_exist(enc_part_name):
            return

        volctrl_node = XmlNode(CTRL_ENC_BASE_XML)

        # append partition name
        vol_node = volctrl_node.get_node(VOLUME_NAME)

        vol_node.append_child(NAME_NAME,
                              self.__vol_name2first_part_name(vol_name),
                              {TYPE_ATTR: PARTITION_ATTR_VAL})
        vol_node.append_child(NAME_NAME,
                              enc_part_name,
                              {TYPE_ATTR: CRYPT_ATTR_VAL})

        ctrl_entry = request.Entry(ReqTypes.CONTROL, volctrl_node)

        # set key file
        ctrl_entry.set_file(DECODED_LOCK_KEY_FILE_PATH)

        # set option
        ctrl_entry.set_options({OPERATION_OPT: ENC_START_OPT_VAL})

        # send request
        result_entry = self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)
        if result_entry.get_result() != Results.OK:
            raise self.__StartError

        return

    def __do_enc_start_partition(self, part_name, enc_part_name):

        ctrl_entry = \
            request.Entry(ReqTypes.CONTROL,
                          CTRL_ENC_BASE_XML % (part_name, enc_part_name))
        # set key file
        ctrl_entry.set_file(DECODED_LOCK_KEY_FILE_PATH)

        # set option
        ctrl_entry.set_options({OPERATION_OPT: ENC_START_OPT_VAL})

        # send request
        result_entry = self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)
        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        return

    def __do_enc_stop_partition(self, enc_dev_name):

        CTRL_ENC_BASE_XML = '<volctrl><volume/></volctrl>'
        volctrl_node = XmlNode(CTRL_ENC_BASE_XML)

        # append partition name
        vol_node = volctrl_node.get_node(VOLUME_NAME)

        vol_node.append_child(NAME_NAME,
                              enc_dev_name,
                              {TYPE_ATTR: CRYPT_ATTR_VAL})

        ctrl_entry = request.Entry(ReqTypes.CONTROL, volctrl_node)

        # set option
        ctrl_entry.set_options({OPERATION_OPT: ENC_STOP_OPT_VAL})

        # send request
        result_entry = self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)

        return result_entry.get_result()

    def __do_encrypt_vfsext_start(self,
                                  is_chk_external_status=False,
                                  is_chk_external_enc_config=False):

        # 非暗号で取得
        vfs_element_list = \
            self.__get_enable_vfs_element_list(
                is_enc=False,
                is_chk_external_status=is_chk_external_status)

        for part_name in vfs_element_list:

            if not self.__is_device_exist(part_name):
                continue

            disk_name = self.__partition2disk(part_name)
            if util.is_need_vfs_autorebuild(disk_name):
                continue

            if is_chk_external_enc_config:
                if config.EXT_RAID_DEV_INFO[part_name]["is_external"]:
                    external_vol_name = self.__dev_name2vol_name(part_name)
                    if not self.__is_encrypted_volume(external_vol_name):
                        continue

            enc_part_name = DEV_TO_ENC_DEV[part_name]
            ctrl_entry = \
                request.Entry(ReqTypes.CONTROL,
                              CTRL_ENC_BASE_XML % (part_name, enc_part_name))
            # set key file
            ctrl_entry.set_file(DECODED_LOCK_KEY_FILE_PATH)

            # set option
            ctrl_entry.set_options({OPERATION_OPT: ENC_START_OPT_VAL})

            # send request
            self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)

    def __sub_get_current_lockkey_info(self, vol_name):
        """Get MAC and lockkey file path
           MAC addr is NOT gotten from pal_network. MAC addr is gotten by
           parsing lockkey file name.
        """

        mnt_path = VOL_INFO[vol_name]["mnt_point"]

        key_file_pattern = os.path.join(mnt_path, config.LOCKKEY_DIR_PREFIX)
        key_file_list = glob.glob(key_file_pattern + "*/*")

        if 0 == len(key_file_list):
            raise self.__KeyFileNotFound

        match = \
            re.match(key_file_pattern + "[0-9A-Za-z]{12}/([0-9A-Za-z]{12})",
                     key_file_list[0])
        if not match:
            raise self.__KeyFileNotFound

        return (match.group(1), key_file_list[0])

    def __get_current_lockkey_info(self):
        """Get lockkey infomation from connected lockkey."""

        for vol_name in config.LOCKKEY_VOL_LIST:

            try:
                (mac, lockkey_path) = \
                    self.__sub_get_current_lockkey_info(vol_name)

                lockkey_dev_name = VOL_INFO[vol_name]["dev"]

                return (mac, lockkey_dev_name, lockkey_path)

            except self.__KeyFileNotFound:
                continue

        raise self.__KeyFileNotFound

    def __mount_lockkey(self, vol_name=config.LOCKKEY_VOL_LIST[0]):

        part_name = self.__vol_name2first_part_name(vol_name)
        mnt_point = self.__vol_name2mount_point(vol_name)
        try:
            self.__do_mount(part_name, mnt_point)
        except self.__MountFail:
            pass
        except self.__AlreadyMount:
            pass
        return

    def __unmount_lockkey(self, vol_name=config.LOCKKEY_VOL_LIST[0]):

        retry = 3
        while(retry):
            try:
                self.__do_unmount(vol_name)
            except self.__UnmountFail:
                # 何らかの理由でリソースBSYだった場合などのために再試行
                time.sleep(1)
                retry -= 1
                continue
            except self.__AlreadyUnmount:
                break
            else:
                break
        return

    def __do_extract_usb_lockkey(self):

        self.__mount_lockkey()

        (mac, key_dev_name, key_path) = self.__get_current_lockkey_info()

        if mac != self.__get_macaddr():
            raise self.__ExtractLockkeyError

        req_xml = EXTRACT_USB_LOCKKEY_REQ_XML % (mac,
                                                 key_path,
                                                 DECODED_LOCK_KEY_FILE_PATH,
                                                 key_dev_name)
        result_entry = self.__send_request(pal_info.PAL_LOCKKEY_PATH,
                                           ReqTypes.SET_LOCKKEY,
                                           req_xml)
        self.__unmount_lockkey()

        if Results.OK != result_entry.get_result():
            raise self.__ExtractLockkeyError
        return

    def __do_extract_keyserver_lockkey(self):

        req_xml = EXTRACT_KEYSRV_REQ_BASE_XML % (self.__get_macaddr(),
                                                 DECODED_LOCK_KEY_FILE_PATH)

        req_node = XmlNode(req_xml)
        keyserverlist_node = self.__get_enable_keyserver_node()
        req_node.append_child(keyserverlist_node)

        result_entry = self.__send_request(pal_info.PAL_LOCKKEY_PATH,
                                           ReqTypes.SET_LOCKKEY,
                                           str(req_node))

        if Results.OK != result_entry.get_result():
            raise self.__KeyserverNotFound
        return

    def __do_encrypt_stop(self, vol_name):

        CTRL_ENC_BASE_XML = '<volctrl><volume/></volctrl>'
        volctrl_node = XmlNode(CTRL_ENC_BASE_XML)

        # append partition name
        vol_node = volctrl_node.get_node(VOLUME_NAME)

        vol_node.append_child(NAME_NAME,
                              self.__vol_name2enc_part_name(vol_name),
                              {TYPE_ATTR: CRYPT_ATTR_VAL})

        ctrl_entry = request.Entry(ReqTypes.CONTROL, volctrl_node)

        # set option
        ctrl_entry.set_options({OPERATION_OPT: ENC_STOP_OPT_VAL})

        # send request
        result_entry = self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)

        return result_entry.get_result()

    def __vfs_enc_stop(self):

        VFS_ENC_LIST = \
            [DEV_TO_ENC_DEV[element] \
                for element in util.get_vfsext_dev_info().keys()]

        CTRL_ENC_BASE_XML = \
            '<volctrl>' + \
             '<option operation="encstop" />' + \
             '<volume>' + \
              '<name type="crypt" />' + \
             '</volume>' + \
            '</volctrl>'
        ctrl_entry = request.Entry(ReqTypes.CONTROL, CTRL_ENC_BASE_XML)
        name_node = ctrl_entry.get().get_node('volume/name')

        for vfs_enc_name in VFS_ENC_LIST:

            if not self.__is_device_exist(vfs_enc_name):
                continue

            name_node.set_text(vfs_enc_name)
            result_entry = \
                self.send_entry(ctrl_entry, pal_info.PAL_VOLCTRL_PATH)

            if Results.OK != result_entry.get_result():
                raise self.__RequestError
        return

    def __enc_start(self, entry):
        """Start encrypt partition"""

        vol_node = entry.get().get_node(VOLUME_NAME)
        vol_name = vol_node.get_value(NAME_NAME)

        if self.__is_external_vfsext_setting(vol_name):
            entry.set_result(Results.OK)
            return entry

        if INTERNAL_VOL == vol_name:
            self.__swap_restart(True)

        if self.__is_current_raid_mode_vfsext():
            if INTERNAL_VOL == vol_name:
                self.__do_encrypt_vfsext_start(is_chk_external_status=False,
                                               is_chk_external_enc_config=True)
                entry.set_result(Results.OK)
                return entry

        self.__append_partition_name(vol_node)
        self.__append_enc_partition_name(vol_node)
        return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

    def __enc_stop(self, entry):
        """Stop encrypt partition"""

        vol_node = entry.get().get_node(VOLUME_NAME)
        vol_name = vol_node.get_node(NAME_NAME).get_text()

        enc_part_name = self.__vol_name2enc_part_name(vol_name)
        if not self.__is_device_exist(enc_part_name):
            entry.set_result(Results.OK)
            return entry

        self.__append_enc_partition_name(vol_node)
        return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

    def __swap_create(self, entry):
        """Create swap patrition"""

        return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

    def __swap_on(self, entry):
        """Enable swap"""

        return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

    def __swap_off(self, entry):
        """Disable swap"""

        return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

    def __write_enc_sig(self, entry):

        vol_name = entry.get().get_node(VOL_NAME_XPATH).get_text()
        disk_name = self.__vol_name2hdd_name_list(vol_name)[0]

        vol_node = entry.get().get_node(VOLUME_NAME)
        vol_node.append_child(NAME_NAME, disk_name,
                              {TYPE_ATTR: DISK_ATTR_VAL})

        return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

    def __check_enc_sig(self, entry):

        vol_name = entry.get().get_node(VOL_NAME_XPATH).get_text()
        disk_name = self.__vol_name2hdd_name_list(vol_name)[0]

        if vol_name.startswith("usb"):
            if not self.__is_external_vfsext_setting(vol_name):
                if self.__is_volume_have_non_enc_filesystem(vol_name):
                    entry.set_result(Results.CHECK_ERROR)
                    return entry

        vol_node = entry.get().get_node(VOLUME_NAME)
        vol_node.append_child(NAME_NAME, disk_name,
                              {TYPE_ATTR: DISK_ATTR_VAL})

        return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

    def __is_exist_volume_device(self, vol_name):

        disk_name = self.__vol_name2hdd_name_list(vol_name)[0]
        if not self.__is_device_exist(disk_name):
            return False
        return True

    def __is_mirroring_setting(self, vol_name):

        USB_MIRRORIGN_XPATH = \
            "usblist/usb[@port='%s']/modelist" + \
            "/mode[@enable='true' and text()='mirroring']"

        match = re.match("usb([1-9]+)", vol_name)
        if not match:
            return False

        ports_node = self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                               pal_info.PAL_CONFFILE_PATH,
                                               config.PORTS_CONF_FILE).get()
        try:
            ports_node.get_node(USB_MIRRORIGN_XPATH % match.group(1))
        except:
            return False

        return True

    def __is_external_vfsext_setting(self, vol_name):

        if not vol_name.startswith("usb"):
            return False

        disk_name = VOL_INFO[vol_name]["dev"]

        iface_name = config.DEV_TO_PORT_IF.get(disk_name)
        port_num = config.DEV_TO_PORT_NO.get(disk_name)

        if not iface_name or not port_num:
            return False

        if VFSEXT_PORT_MODE == \
               util.get_current_port_mode(iface_name, port_num):
            return True

        return False

    def __send_query_to_raidctrl(self, raiddev_name):
        """Send query request to pal_diskctrl"""

        QUERY_REQ_BASE_XML = '<raidctrl>' + \
                             ' <option element="true"/>' + \
                             ' <raid>' + \
                             '  <name type="raid">%s</name>' + \
                             ' </raid>' + \
                             '</raidctrl>'

        result_entry = self.__send_request(pal_info.PAL_RAIDCTRL_PATH,
                                           ReqTypes.QUERY,
                                           QUERY_REQ_BASE_XML % raiddev_name)

        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        return result_entry.get()

    def __get_current_raid_element_list(self, raiddev_name):

        PART_NAME_XPATH = "./raid/elementlist/element/name"
        try:
            raidctrl_node = self.__send_query_to_raidctrl(raiddev_name)
        except self.__RequestError:
            self.exception("fail to query to raidctrl(%s)" % raiddev_name)
            return []

        return [name_node.get_text() for name_node in \
                   raidctrl_node.get_node_list(PART_NAME_XPATH)]

    def __is_volume_join_mirroring(self, vol_name):
        """Check Whether specified volume is embedded into mirroring RAID"""

        mirror_raiddev_name = \
            VOL_INFO[INTERNAL_VOL]["dev"]

        element_name_list = \
            self.__get_current_raid_element_list(mirror_raiddev_name)

        for element_name in element_name_list:
            # ミラーリングRAIDに組み込まれている
            if element_name.startswith(self.__vol_name2dev_name(vol_name)):
                return True
        return False

    def __is_volume_join_external_vfsext(self, vol_name):

        disk_name = VOL_INFO[vol_name]["dev"]
        cur_vfs_element_list = util.get_current_vfs_element_list()

        for tgt_element in util.get_vfsext_dev_info().keys():
            if tgt_element.startswith(disk_name):
                break
        else:
            raise NotFoundError

        if tgt_element in cur_vfs_element_list:
            return True

        return False

    def __send_query_to_diskctrl(self, disk_name):
        """Send query request to pal_diskctrl"""

        QUERY_REQ_BASE_XML = '<diskctrl>' + \
                             ' <option getpartitioninfo="true" />' + \
                             ' <disk>' + \
                             '  <name>%s</name>' + \
                             ' </disk>' + \
                             '</diskctrl>'

        result_entry = self.__send_request(pal_info.PAL_DISKCTRL_PATH,
                                           ReqTypes.QUERY,
                                           QUERY_REQ_BASE_XML % disk_name)

        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        return result_entry.get()

    def __get_disk_size(self, disk_name):

        SIZE_XPATH = "./disk/size"
        diskctrl_node = self.__send_query_to_diskctrl(disk_name)
        return int(diskctrl_node.get_node(SIZE_XPATH).get_text())

    def __get_min_mirroring_disk_size(self):

        RAID_LEVEL_XPATH = "./raid/level"
        LBA_START_XPATH = "./disk/partitionlist/partition[number='%s']/start"
        LBA_END_XPATH = "./disk/partitionlist/partition[number='%s']/end"

        if INTERNAL_DEV:
            # FIXME VFSの判定も入れる
            raidctrl_node = self.__send_query_to_raidctrl(INTERNAL_DEV)
            raid_level = raidctrl_node.get_node(RAID_LEVEL_XPATH).get_text()

            # FIXME 決め打ち
            element_name_list = self.__get_current_raid_element_list(
                                                                INTERNAL_DEV)
        else:
            raid_level = SINGLE_RAID_LEVEL
            element_name_list = []
            mirror_dev = VOL_INFO[INTERNAL_VOL]["dev"]
            for element in config.RAID_TO_ELEMENT[1][mirror_dev]:
                try:
                    disk_name = self.__partition2disk(element)
                    if disk_name in config.SATA_HDD_DEV_LIST:
                        element_name_list.append(element)
                except:
                    # Ignore error because "element" may be not partition.
                    pass

        if len(element_name_list) == 0:
            raise self.__InternalError

        match = re.match("([a-zA-Z]+[1-9]+)p[1-9]+", element_name_list[0])
        if not match:
            raise self.__InternalError
        diskctrl_node = self.__send_query_to_diskctrl(match.group(1))

        size_dict = {}
        for part_num in ["2", "5", "6"]:  # md2,md5,md6
            start = \
               diskctrl_node.get_node(LBA_START_XPATH % part_num).get_text()
            end = \
               diskctrl_node.get_node(LBA_END_XPATH % part_num).get_text()
            size_dict[part_num] = int(end) - int(start) + 1

        return size_dict["2"] + \
               size_dict["5"] + \
               size_dict["6"] * util.get_raid_min_num(raid_level)

    def __is_enough_size_for_mirroring(self, vol_name):

        if not vol_name.startswith("usb"):
            raise self.__InternalError  # 本来ありえない

        usb_disk_name = self.__vol_name2hdd_name_list(vol_name)[0]
        usb_disk_size = self.__get_disk_size(usb_disk_name)

        if usb_disk_size >= self.__get_min_mirroring_disk_size():
            return True
        return False

    def __get_cur_vol_info_for_mirroring(self, vol_name):

        mirr_info_dict = {STATUS_NAME: WAITING_STATUS,
                          NAME_NAME: "", FSYS_NAME: "",
                          TOTAL_NAME: "", USE_NAME: "", ENCRYPT_ATTR: ""}

        (is_mirrordisk, is_recoverable, is_match_system) = \
            util.check_mirrordisk(self.__vol_name2dev_name(vol_name))

        # 未接続
        if not self.__is_exist_volume_device(vol_name):
            mirr_info_dict[STATUS_NAME] = UNCONNECTED_STATUS

        # ミラーに組み込まれているか
        elif self.__is_volume_join_mirroring(vol_name):
            mirr_info_dict[STATUS_NAME] = MIRRORING_STATUS

        # ディスクサイズがミラーに必要最低限の値か
        elif not self.__is_enough_size_for_mirroring(vol_name):
            mirr_info_dict[STATUS_NAME] = CAPSHORTAGE_STATUS

        elif is_mirrordisk:
            if is_recoverable:
                mirr_info_dict[STATUS_NAME] = RECOVERABLE_MIRROR_STATUS
            else:
                mirr_info_dict[STATUS_NAME] = OTHER_MIRROR_STATUS

        # データが存在するので待ち状態
        elif util.is_volume_exist_any_files(vol_name):
            mirr_info_dict[STATUS_NAME] = WAITING_STATUS

        return mirr_info_dict

    def __get_cur_vol_info_for_external_vfsex(self, vol_name):

        mirr_info_dict = {STATUS_NAME: WAITING_STATUS,
                          NAME_NAME: "", FSYS_NAME: "",
                          TOTAL_NAME: "", USE_NAME: "", ENCRYPT_ATTR: ""}

        (is_mirrordisk, is_recoverable, is_match_system) = \
            util.check_mirrordisk(self.__vol_name2dev_name(vol_name))

        # 未接続
        if not self.__is_exist_volume_device(vol_name):
            mirr_info_dict[STATUS_NAME] = UNCONNECTED_STATUS

        # 拡張ボリュームに組み込まれているか
        elif self.__is_volume_join_external_vfsext(vol_name):
            mirr_info_dict[STATUS_NAME] = VFSEXT_STATUS

        # ディスクサイズがミラーに必要最低限の値か
        elif not util.is_enough_disk_size_vfsext(
                     VOL_INFO[vol_name]["dev"]):
            mirr_info_dict[STATUS_NAME] = CAPSHORTAGE_STATUS

        elif is_mirrordisk:
            if is_recoverable:
                mirr_info_dict[STATUS_NAME] = RECOVERABLE_MIRROR_STATUS
            else:
                mirr_info_dict[STATUS_NAME] = OTHER_MIRROR_STATUS

        # データが存在するので待ち状態
        elif util.is_volume_exist_any_files(vol_name):
            mirr_info_dict[STATUS_NAME] = WAITING_STATUS

        # 暗号化ロック状態
        elif self.__is_encrypted_volume(vol_name):
            mirr_info_dict[STATUS_NAME] = KEYWAITING_STATUS

        return mirr_info_dict

    def __get_updated_volume_info(self, vol_name):

        # 外付けミラーリング用ステータス
        if self.__is_mirroring_setting(vol_name):
            return self.__get_cur_vol_info_for_mirroring(vol_name)

        # 外付け拡張ボリューム用ステータス
        if self.__is_external_vfsext_setting(vol_name):
            return self.__get_cur_vol_info_for_external_vfsex(vol_name)

        is_encrypt = self.__is_encrypted_volume(vol_name)

        # PALから情報取得
        info_dict = self.__get_volume_info_from_pal(vol_name)

        (is_mirrordisk, is_recoverable, is_match_system) = \
            util.check_mirrordisk(self.__vol_name2dev_name(vol_name))

        if MOUNTED_STATUS == info_dict[STATUS_NAME]:
            if is_encrypt:
                info_dict[STATUS_NAME] = ENC_MOUNTED_STATUS

            elif self.__is_lockkey(vol_name, False):
                info_dict[STATUS_NAME] = LOCKKEY_STATUS

        elif self.__is_lockkey(vol_name, True):
            info_dict[STATUS_NAME] = LOCKKEY_STATUS

        elif is_encrypt:
            if "false" == info_dict[ENCRYPT_ATTR]:
                # アンマウント＋暗号化設定 => keywaiting
                info_dict[STATUS_NAME] = KEYWAITING_STATUS

        elif is_mirrordisk:
            if is_recoverable:
                info_dict[STATUS_NAME] = RECOVERABLE_MIRROR_STATUS
            else:
                info_dict[STATUS_NAME] = OTHER_MIRROR_STATUS

        elif self.__is_used_as_embedded_disk(
            self.__vol_name2hdd_name_list(vol_name)[0]):

            info_dict[STATUS_NAME] = EXIST_DISK_LABEL_STATUS

        return info_dict

    def __check_force_raid_start_done(self):

        is_done = False

        if "1" == util.get_raid_cache_value(MIRROR_DEV,
                                            FORCE_START_DONE):
            is_done = True

        elif "1" == util.get_raid_cache_value(INTERNAL_DEV,
                                              FORCE_START_DONE):
            is_done = True

        if is_done:
            util.set_raid_cache_value(MIRROR_DEV,
                                      FORCE_START_DONE,
                                      "0")
            if INTERNAL_DEV:
                util.set_raid_cache_value(INTERNAL_DEV,
                                          FORCE_START_DONE,
                                          "0")

        return is_done

    def __update_volume_status(self, vol_name):
        """Update volume status and write status into temp file """

        # cur_status = self.__get_volume_cache_value(vol_name, STATUS_NAME)
        # if CHECKING_STATUS == cur_status or FORMATTING_STATUS == cur_status:
        #    self.debug("updateing status is cancelled(%s)" % cur_status)
        #    raise self.__UpdateError

        if self.__is_current_raid_mode_vfsext():
            if INTERNAL_VOL == vol_name:
                return

        info_dict = self.__get_updated_volume_info(vol_name)

        self.__set_volume_cache_value(vol_name, info_dict)
        return

    def __update_handler(self, entry):

        vol_name = entry.get().get_node(VOL_NAME_XPATH).get_text()
        status = entry.get().get_value(VOL_STATUS_XPATH, None)
        if status is not None:
            if status not in VOLUME_STATUS_LIST:
                entry.set_result(Results.UPDATE_ERROR)
                return entry

            self.__set_volume_cache_value(vol_name, {STATUS_NAME: status})
            entry.set_result(Results.OK)
            return entry

        try:
            self.__update_volume_status(vol_name)
        except self.__DeviceNotFound:
            entry.set_result(Results.DEVICE_NOT_FOUND)
        except self.__UpdateError:
            entry.set_result(Results.UPDATE_ERROR)
        else:
            entry.set_result(Results.OK)

        return entry

    def __get_vfs_query(self):

        FILESYSTEM_XPATH = "./volume/filesystem"

        result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                           ReqTypes.QUERY,
                                           VFS_QUERY_XML)

        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

        is_enc = \
            self.__is_encrypted_volume(INTERNAL_VOL)

        status = result_entry.get().get_value(VOL_STATUS_XPATH)
        if is_enc and UNMOUNTED == status:
            if not result_entry.get().get_node(FILESYSTEM_XPATH).\
                is_set_to_true(ENCRYPT_ATTR):

                result_entry.get().get_node(VOL_STATUS_XPATH).\
                    set_text(KEYWAITING_STATUS)

        if is_enc and MOUNTED == status:
            result_entry.get().get_node(FILESYSTEM_XPATH).\
                set_bool_attr(ENCRYPT_ATTR, is_enc)

        return result_entry

    def __query_handler(self, entry):
        """Handle "query" request."""

        vol_name = entry.get().get_node(VOL_NAME_XPATH).get_text()

        if self.__is_current_raid_mode_vfsext():
            if INTERNAL_VOL == vol_name:
                result_entry = self.__get_vfs_query()
                result_entry.set_result(Results.OK)
                return result_entry

        status = self.__get_volume_cache_value(vol_name, STATUS_NAME)

        # 一回もアップデートされていない
        if not status:
            try:
                self.__update_volume_status(vol_name)
                status = self.__get_volume_cache_value(vol_name, STATUS_NAME)
            except self.__UpdateError:
                entry.set_result(Results.UPDATE_ERROR)
                return entry

        # 別のシステムのミラーHDD
        #if OTHER_MIRROR_STATUS == status:
        if status in [OTHER_MIRROR_STATUS, RECOVERABLE_MIRROR_STATUS]:
            vol_node = entry.get().get_node(VOLUME_NAME)
            vol_node.append_child(STATUS_NAME, status)
            entry.set_result(Results.OK)
            return entry

        part_name = self.__get_volume_cache_value(vol_name, NAME_NAME)

        if self.__is_mirroring_setting(vol_name) or \
               self.__is_external_vfsext_setting(vol_name):

            disk_name = self.__vol_name2dev_name(vol_name)
            if FAIL_STATUS == util.get_disk_info(disk_name, "status"):
                vol_node = entry.get().get_node(VOLUME_NAME)
                vol_node.append_child(STATUS_NAME, FAIL_STATUS)
                entry.set_result(Results.OK)
                return entry

        # 未接続
        if UNCONNECTED_STATUS == status:
            entry.set_result(Results.DEVICE_NOT_FOUND)
            return entry

        # ディスクラベルが存在する（USBミラーの残骸）
        if EXIST_DISK_LABEL_STATUS == status:
            vol_node = entry.get().get_node(VOLUME_NAME)
            vol_node.append_child(STATUS_NAME, UNFORMATTED_STATUS)
            entry.set_result(Results.OK)
            return entry

        # マウントされていない
        if MOUNTED_STATUS != status and ENC_MOUNTED_STATUS != status:
            vol_node = entry.get().get_node(VOLUME_NAME)

            if ENC_FORMATTING_STATUS == status:
                vol_node.append_child(STATUS_NAME, FORMATTING_STATUS)
            elif ENC_CHECKING_STATUS == status:
                vol_node.append_child(STATUS_NAME, CHECKING_STATUS)
            else:
                vol_node.append_child(STATUS_NAME, status)

            if LOCKKEY_STATUS == status:
                entry.set_result(Results.OK)
                return entry

            # ファイルシステム
            fstype = self.__get_volume_cache_value(vol_name, FSYS_NAME)
            if fstype:
                vol_node.append_child(FSYS_NAME, fstype)

                encrypt_attr = \
                    self.__get_volume_cache_value(vol_name, ENCRYPT_ATTR)

                if TRUE_ATTR_VAL == encrypt_attr:
                    vol_node.get_node(FSYS_NAME).set_attrs(
                        {ENCRYPT_ATTR: TRUE_ATTR_VAL})

                elif status in [ENC_CHECKING_STATUS, ENC_FORMATTING_STATUS]:
                    vol_node.get_node(FSYS_NAME).set_attrs(
                        {ENCRYPT_ATTR: TRUE_ATTR_VAL})

            capa_node = XmlNode(CAPACITY_NAME)
            # 全容量
            total = self.__get_volume_cache_value(vol_name, TOTAL_NAME)
            if total:
                capa_node.append_child(TOTAL_NAME, total)

            # 使用容量
            use = self.__get_volume_cache_value(vol_name, USE_NAME)
            if use:
                capa_node.append_child(USE_NAME, use)

            if 0 < len(capa_node.get_node_list("*")):
                vol_node.append_child(capa_node)

            entry.set_result(Results.OK)
            return entry

        # マウント済みなので現在の状態取得
        try:
            result_entry = self.__get_usable_query_response(status, part_name)
            result_entry.get().append_child(
                entry.get().get_node(VOL_NAME_XPATH))
        except self.__DeviceNotFound:
            entry.set_result(Results.DEVICE_NOT_FOUND)
            return entry

        return result_entry

    def __control_handler(self, entry):
        """Handle "control" request."""

        op_func = \
            {ENC_START_ATTR_VAL: self.__enc_start,
             ENC_STOP_ATTR_VAL: self.__enc_stop,
             SWAP_CREATE_ATTR_VAL: self.__swap_create,
             SWAP_ON_ATTR_VAL: self.__swap_on,
             SWAP_OFF_ATTR_VAL: self.__swap_off,
             WRITE_ENC_SIG_ATTR_VAL: self.__write_enc_sig,
             CHK_ENC_SIG_ATTR_VAL: self.__check_enc_sig}

        op = entry.get_options()[OPERATION_OPT]
        return op_func[op](entry)

    def __mount_handler(self, entry):
        """Handle "mount" request."""

        def dispatch_pre_mount_event(vol_name):
            try:
                self.__dispatch_mount_event(vol_name, True)
            except:
                pass

        def dispatch_post_mount_event(vol_name, is_mountfail=False):
            try:
                self.__dispatch_mount_event(vol_name, False, is_mountfail)
            except:
                pass

        if self.__is_exist_node(entry.get(), OPTION_NAME):
            if entry.get().get_node(OPTION_NAME).\
                    is_set_to_true(VFSREMOUNT_ATTR):
                return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

            if entry.get().get_node(OPTION_NAME).\
                    is_set_to_true(VFSADD_ATTR):
                return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)
                

        vol_name = entry.get().get_node(VOL_NAME_XPATH).get_text()

        dispatch_pre_mount_event(vol_name)

        if self.__is_lockkey(vol_name, True):
            dispatch_post_mount_event(vol_name, is_mountfail=True)
            entry.set_result(Results.MOUNT_ERROR)
            return entry

        status = self.__get_volume_cache_value(vol_name, STATUS_NAME)
        if KEYWAITING_STATUS == status:
            dispatch_post_mount_event(vol_name, is_mountfail=True)
            entry.set_result(Results.UNLOCK_ERROR)
            return entry

        try:
            self.__try_mount_all_target(entry, vol_name)
            if self.__is_need_bind_mount(vol_name):
                self.__do_bind_mount(vol_name)

        except self.__MountFail:
            entry.set_result(Results.MOUNT_ERROR)
            dispatch_post_mount_event(vol_name, is_mountfail=True)
        except self.__DiskLabelExist:
            dispatch_post_mount_event(vol_name, is_mountfail=True)
            entry.set_result(Results.MOUNT_ERROR)
            return entry
        except self.__AlreadyMount:
            dispatch_post_mount_event(vol_name, is_mountfail=True)
            entry.set_result(Results.OK)  # Alreadyの場合はOK
        else:
            dispatch_post_mount_event(vol_name)
            entry.set_result(Results.OK)
        finally:
            if INTERNAL_VOL == vol_name:
                self.__all_clear_vfs_autorebuild_flag()

        self.__update_volume_status(vol_name)

        if INTERNAL_VOL == vol_name:
            if self.__is_current_raid_mode_vfsext() or \
                   self.__check_force_raid_start_done:
                try:
                    self.__dispatch_raid_update_event()
                except:
                    pass

                try:
                    self.__dispatch_disk_update_event()
                except:
                    pass

        try:
            util.spindown_unuse_disk()
        except:
            self.exception("fail to util.spindown_unuse_disk()")

        return entry

    def __unmount_handler(self, entry):
        """Handle "mount" request."""

        is_need_raid_update = True

        if self.__is_exist_node(entry.get(), OPTION_NAME):
            if entry.get().get_node(OPTION_NAME).is_set_to_true(NOUPDATE_ATTR):
                is_need_raid_update = False

        vol_name = entry.get().get_node(VOL_NAME_XPATH).get_text()

        # unmountイベントディスパッチ
        self.__dispatch_unmount_event(vol_name, True)

        is_unmount_done = False

        try:
            self.__do_bind_unmount(vol_name)
        except self.__UnmountFail:
            pass
        except self.__AlreadyUnmount:
            pass
        else:
            is_unmount_done = True

        try:
            self.__do_unmount(vol_name)
        except self.__UnmountFail:
            entry.set_result(Results.UNMOUNT_ERROR)
        except self.__AlreadyUnmount:
            entry.set_result(Results.OK)
        else:
            is_unmount_done = True
            entry.set_result(Results.OK)

        if self.__is_current_raid_mode_vfsext():
            if INTERNAL_VOL == vol_name:
                try:
                    util.vfsext_unmount()
                except:
                    self.exception("fail to util.vfsext_unmount()")
                    entry.set_result(Results.UNMOUNT_ERROR)
                    is_unmount_done = False
                else:
                    is_unmount_done = True

        if is_unmount_done:
            if not self.__is_mirroring_setting(vol_name):
                try:
                    self.__update_volume_status(vol_name)
                except:
                    pass

        try:
            self.__dispatch_unmount_event(vol_name)
        except:
            pass

        if INTERNAL_VOL == vol_name:
            if is_need_raid_update:
                try:
                    self.__dispatch_raid_update_event()
                except:
                    pass

        return entry

    def __save_enc_config(self, volume, is_enc):
        """volume is the node that be sent to pal_volctrl or volume name"""

        ENCRYPT_XPATH = "volumelist/volume/name[@type='volume']/../encrypt"
        VFS_ELEMENT_XPATH = "vfsext/partitionlist/partition"

        vol_name = None
        vol_node = None
        if isinstance(volume, str):
            vol_name = volume
        elif isinstance(volume, XmlNode):
            vol_name = volume.get_node(NAME_NAME).get_text()
            vol_node = volume
        else:
            raise InvalidArgError

        disk_name = self.__vol_name2hdd_name_list(vol_name)[0]

        if vol_name.startswith("usb"):

            if is_enc:
                req_xml = WRITE_SIG_REQ_XML_TMPL % disk_name
                self.__set_volume_cache_value(vol_name,
                                              {ENCRYPT_ATTR: TRUE_TEXT})
            else:
                req_xml = CLEAR_SIG_REQ_XML_TMPL % disk_name
                self.__set_volume_cache_value(vol_name,
                                              {ENCRYPT_ATTR: FALSE_TEXT})

            result_entry = self.__send_request(pal_info.PAL_VOLCTRL_PATH,
                                               ReqTypes.CONTROL,
                                               req_xml)
            if Results.OK != result_entry.get_result():
                raise self.__SaveError

            return

        if INTERNAL_VOL == vol_name:
            volumelist_node = self.__load_config(config.VOLCTRL_CONF_FILE)
            encrypt_node = volumelist_node.get_node(ENCRYPT_XPATH)
            encrypt_node.enable(is_enc)

            if is_enc:
                self.__set_volume_cache_value(vol_name,
                                              {ENCRYPT_ATTR: TRUE_TEXT})
            else:
                self.__set_volume_cache_value(vol_name,
                                              {ENCRYPT_ATTR: FALSE_TEXT})

            result_entry = \
                self.__send_request_with_file(pal_info.PAL_CONFFILE_PATH,
                                              ReqTypes.SAVE,
                                              config.VOLCTRL_CONF_FILE,
                                              volumelist_node)

            if Results.OK != result_entry.get_result():
                raise self.__SaveError

            if vol_node is None:
                return

            vfs_element_node_list = vol_node.get_node_list(VFS_ELEMENT_XPATH)
            for vfs_element_node in vfs_element_node_list:
                vfs_vol_name = \
                    self.__dev_name2vol_name(vfs_element_node.get_text())

                if vfs_vol_name.startswith("usb"):
                    self.__save_enc_config(vfs_vol_name, is_enc)

            return

        raise InvalidArgError

    def __get_enable_vfs_element_list(self, is_enc=False,
                                      is_chk_external_status=False):
        """Get enable vfs element list
           The returned element list can be able to be used as mkfs, fsck.
        """

        element_list = []
        for element, info in util.get_vfsext_dev_info().iteritems():

            if not self.__is_device_exist(element):
                continue

            disk_name = self.__partition2disk(element)
            try:
                disk_status = util.get_disk_info(disk_name, "status")
                if disk_status not in [DISK_NORMAL_STATUS,
                                       DISK_FAIL_PENDING_STATUS,
                                       DISK_SMARTERR_PENDING_STATUS]:
                    continue
            except:
                continue

            if info["is_external"]:
                iface_name = config.DEV_TO_PORT_IF[disk_name]
                port_num = config.DEV_TO_PORT_NO[disk_name]
                if VFSEXT_PORT_MODE != \
                       util.get_current_port_mode(iface_name, port_num):
                    continue

                if is_chk_external_status:
                    vol_name = self.__dev_name2vol_name(element)
                    vol_status = \
                        self.__get_volume_cache_value(vol_name, "status")
                    if vol_status not in [VFSEXT_STATUS]:
                        continue

            if is_enc:
                element_list.append(DEV_TO_ENC_DEV[element])
            else:
                element_list.append(element)

        element_list.sort()

        return element_list

    def __is_supported_mkfs_fsxk_port_mode(self, entry):

        vol_node = entry.get().get_node(VOLUME_NAME)
        vol_name = vol_node.get_node(NAME_NAME).get_text()

        if vol_name.startswith("usb"):
            if self.__is_mirroring_setting(vol_name):
                return False
            if self.__is_external_vfsext_setting(vol_name):
                return False
        return True

    def __pre_mkfs(self, entry):

        FILESYSTEM_XPATH = "./volume/filesystem"

        self.debug(str(entry))
        is_current_vfsext = False
        if self.__is_current_raid_mode_vfsext():
            is_current_vfsext = True

        vol_node = entry.get().get_node(VOLUME_NAME)
        vol_name = vol_node.get_node(NAME_NAME).get_text()
        filesystem = entry.get().get_node(FILESYSTEM_XPATH).get_text()

        # デバイスが存在しなければエラー終了
        if not is_current_vfsext:
            if not self.__is_device_exist(self.__vol_name2dev_name(vol_name)):
                return Results.DEVICE_NOT_FOUND

        try:
            self.__dispatch_unmount_event(vol_name, True)
        except:
            pass

        # アンマウント
        try:

            # VFSは特別扱い
            if is_current_vfsext:
                try:
                    self.__do_bind_unmount(vol_name)
                except:
                    pass

                try:
                    self.__do_unmount(vol_name)
                except:
                    pass

                try:
                    if INTERNAL_VOL == vol_name:
                        util.vfsext_unmount()
                except:
                    pass
            else:
                if self.__is_need_bind_mount(vol_name):

                    try:
                        self.__do_bind_unmount(vol_name)
                    except self.__AlreadyUnmount:
                        pass

                try:
                    self.__do_unmount(vol_name)
                except self.__AlreadyUnmount:
                    pass

        except self.__UnmountFail:
            # mountされてなかっただけかもしれないのでスルー
            pass

        self.debug(vol_name)
        if is_current_vfsext and INTERNAL_VOL == vol_name:

            # ロック
            if self.__is_encrypted_volume(vol_name):
                try:
                    self.__vfs_enc_stop()
                except:
                    pass

            # FIXME CGI
            self.debug("change xfs -> vfsext")
            entry.get().get_node("volume/filesystem").set_text("vfsext")

        try:
            self.__dispatch_unmount_event(vol_name)
        except:
            pass

        enc_dev_name = self.__vol_name2enc_part_name(vol_name)
        if self.__is_device_exist(enc_dev_name):

            # stop encrypt
            result_code = self.__do_encrypt_stop(vol_name)
            if Results.OK != result_code and \
               Results.DEVICE_NOT_FOUND != result_code:
                return Results.MKFS_ERROR

        # 新規でパーティションを作成する
        try:
            if vol_name in config.STOR_IF_TO_VOL["usb"].values():
                self.__delete_all_partition(vol_name)
                self.__mkpart(vol_name, filesystem)
        except self.__DeleteError:
            return Results.DELETE_ERROR
        except self.__MakepartError:
            return Results.MAKE_ERROR

        # 暗号化フォーマットの場合は暗号デバイス作成
        is_enc = vol_node.is_set_to_true(ENCRYPT_ATTR)
        if is_enc:

            # ロックキーをデコード
            try:
                self.__do_extract_keyserver_lockkey()
            except self.__KeyserverNotFound:
                self.__do_extract_usb_lockkey()

            if is_current_vfsext and INTERNAL_VOL == vol_name:
                self.__do_encrypt_vfsext_start(is_chk_external_status=True)
            else:
                # start encrypt
                try:
                    self.__do_encrypt_start(vol_name)
                except:
                    return Results.MKFS_ERROR

        if INTERNAL_VOL == vol_name:
            self.__swap_restart(is_enc)

        # PALにMKFSリクエスト
        if is_enc:
            self.__append_partition_name(vol_node, True, is_current_vfsext)
        else:
            self.__append_partition_name(vol_node, False, is_current_vfsext)

        try:
            self.__save_enc_config(vol_node, is_enc)
        except self.__SaveError:
            return Results.MKFS_ERROR

        return Results.OK

    def __mkfs_handler(self, entry):
        """Handle "mkfs" request."""

        if not self.__is_supported_mkfs_fsxk_port_mode(entry):
            entry.set_result(Results.NOT_SUPPORTED)
            return entry

        if CHECKING_STATE == util.statemgr_query(FILESYSTEM_CATEGORY):
            entry.set_result(Results.BUSY_ERROR)
            return entry

        vol_name = entry.get().get_value(VOL_NAME_PATH)
        if self.__dispatch_mkfsprepare_event(vol_name) == DISPACH_FAIL:
            entry.set_result(Results.MKFS_ERROR)
            return entry

        # ステートマネージャー
        try:
            util.statemgr_change(FILESYSTEM_CATEGORY, FORMATTING_STATE)
        except:
            self.exception("fail change state -> mkfs")
            entry.set_result(Results.BUSY_ERROR)
            return entry

        try:
            resultcode = self.__pre_mkfs(entry)
            if resultcode != Results.OK:
                util.statemgr_change(FILESYSTEM_CATEGORY, NOOPERATION_STATE)
                entry.set_result(resultcode)
                return entry
        except:
            util.statemgr_change(FILESYSTEM_CATEGORY, NOOPERATION_STATE)
            entry.set_result(Results.MKFS_ERROR)
            return entry

        util.statemgr_change(FILESYSTEM_CATEGORY, NOOPERATION_STATE)

        return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

    def __pre_fsck(self, entry):

        is_current_vfsext = False
        if self.__is_current_raid_mode_vfsext():
            is_current_vfsext = True

        vol_node = entry.get().get_node(VOLUME_NAME)
        vol_name = vol_node.get_node(NAME_NAME).get_text()

        # デバイスが存在しなければエラー終了
        if not is_current_vfsext:
            if not self.__is_device_exist(self.__vol_name2dev_name(vol_name)):
                return Results.DEVICE_NOT_FOUND

        try:
            self.__dispatch_unmount_event(vol_name, True)
        except:
            pass

        try:

            # VFSは特別扱い
            if is_current_vfsext:
                try:
                    self.__do_bind_unmount(vol_name)
                except:
                    pass

                try:
                    self.__do_unmount(vol_name)
                except:
                    pass

                try:
                    util.vfsext_unmount()
                except:
                    pass
            else:
                if self.__is_need_bind_mount(vol_name):

                    try:
                        self.__do_bind_unmount(vol_name)
                    except self.__AlreadyUnmount:
                        pass

                try:
                    self.__do_unmount(vol_name)
                except self.__AlreadyUnmount:
                    pass

        except self.__UnmountFail:
            # mountされてなかっただけかもしれないのでスルー
            pass

        # VFS設定の場合
        self.debug(vol_name)
        if is_current_vfsext and INTERNAL_VOL == vol_name:
            try:
                util.vfsext_unmount()
            except:
                pass
            # FIXME CGI
            self.debug("change xfs -> vfsext")
            entry.get().get_node("volume/filesystem").set_text("vfsext")

        try:
            self.__dispatch_unmount_event(vol_name)
        except:
            pass

        if self.__is_encrypted_volume(vol_name):
            if is_current_vfsext and INTERNAL_VOL == vol_name:
                pass
            else:
                try:
                    self.__do_encrypt_start(vol_name)
                except:
                    return Results.FSCK_ERROR

            self.__append_partition_name(vol_node, True, is_current_vfsext)
        else:
            self.__append_partition_name(vol_node, False, is_current_vfsext)

        return Results.OK

    def __fsck_handler(self, entry):
        """Handle "fsck" request."""

        def is_supported_filesystem(entry):

            vol_node = entry.get().get_node(VOLUME_NAME)
            vol_name = vol_node.get_node(NAME_NAME).get_text()

            if vol_name.startswith("sata"):
                return True

            if vol_name.startswith("usb"):
                fstype = self.__get_volume_cache_value(vol_name, FSYS_NAME)
                if fstype in config.SUPPORTED_FSCK_FSYS:
                    return True

            return False

        if not self.__is_supported_mkfs_fsxk_port_mode(entry):
            entry.set_result(Results.NOT_SUPPORTED)
            return entry

        if not is_supported_filesystem(entry):
            entry.set_result(Results.NOT_SUPPORTED)
            return entry

        if CHECKING_STATE == util.statemgr_query(FILESYSTEM_CATEGORY):
            entry.set_result(Results.BUSY_ERROR)
            return entry

        try:
            util.statemgr_change(FILESYSTEM_CATEGORY, CHECKING_STATE)
        except:
            self.exception("fail change state -> checking")
            entry.set_result(Results.BUSY_ERROR)
            return entry

        try:
            resultcode = self.__pre_fsck(entry)
            if resultcode != Results.OK:
                util.statemgr_change(FILESYSTEM_CATEGORY, NOOPERATION_STATE)
                entry.set_result(resultcode)
                return entry
        except:
            util.statemgr_change(FILESYSTEM_CATEGORY, NOOPERATION_STATE)
            entry.set_result(Results.FSCK_ERROR)
            return entry

        util.statemgr_change(FILESYSTEM_CATEGORY, NOOPERATION_STATE)

        return self.send_entry(entry, pal_info.PAL_VOLCTRL_PATH)

    #
    # Constractor
    #

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_save_handler = lambda entry: self.send_entry_with_file(entry,
            pal_info.PAL_CONFFILE_PATH, config.VOLCTRL_CONF_FILE)

        self.set_handler(ReqTypes.LOAD, load_save_handler)
        self.set_handler(ReqTypes.SAVE, load_save_handler)
        self.set_handler(ReqTypes.UPDATE, self.__update_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.CONTROL, self.__control_handler)
        self.set_handler(ReqTypes.MOUNT, self.__mount_handler)
        self.set_handler(ReqTypes.UNMOUNT, self.__unmount_handler)
        self.set_handler(ReqTypes.MKFS, self.__mkfs_handler)
        self.set_handler(ReqTypes.FSCK, self.__fsck_handler)

#
# Main
#

if __name__ == "__main__":
    FalVolctrl(option=falcmd.FalCmdOption()).handle_request()
