#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""For FalNarsus data
"""

import fal_config as config
import pal_info
from fal_info import (ReqTypes, Results)
from iobase.exception import (NotFoundError,)
from iobase.xmldata import (XmlNode, )
import util

NARSUS_XML_VER = "2.01"

VFSEXT_STR = u"拡張ボリューム"

VOL_STATUS_NAME = "status"
MIRRORING_STATUS = "mirroring"

LANDISK_INFO_BASE_XML = \
    '<narsus><landiskinfo isskey="%s" update="%s" version="%s"/></narsus>'

PRODUCT_BASE_XML = \
    '<product><name/><firmware/></product>'

SYS_INFO_BASE_XML = \
    '<systeminfo>' + \
     '<uptime>' + \
      '<fan>' + \
       '<time><hour/><minute/></time>' + \
      '</fan>' + \
      '<power>' + \
       '<time><hour/><minute/></time>' + \
      '</power>' + \
      '<device>' + \
       '<time><hour/><minute/></time>' + \
      '</device>' + \
     '</uptime>' + \
     '<temperature/>' + \
     '<fanrpm/>' + \
    '</systeminfo>'

STATUS_INFO_BASE_XML = \
   '<statusinfo>' + \
     '<status condition="OK">' + \
      '<notify type="notify">' + \
       '<param>OFF</param>' + \
      '</notify>' + \
      '<error type="quota">' + \
       '<param>OK</param>' + \
      '</error>' + \
      '<error type="ups">' + \
       '<param>OK</param>' + \
      '</error>' + \
      '<error type="diskerror">' + \
       '<param>OK</param>' + \
      '</error>' + \
      '<error type="capacity">' + \
       '<param>OK</param>' + \
      '</error>' + \
     '</status>' + \
    '</statusinfo>'

NETWORK_BASE_XML = \
    '<network>' + \
     '<tcpip>' + \
      '<lan>' + \
       '<mac/>' + \
      '</lan>' + \
     '</tcpip>' + \
    '</network>'

DEVICE_PORT_XML = '<deviceport>' + \
                    '<sataport/>' + \
                    '<usbport/>' + \
                    '<esataport><port connected="false" number="1" /></esataport>' + \
                  '</deviceport>'

PORT_XML = \
    '<port number="%s" connected="false"/>'

SATA_DISK_XML = \
    '<device type="DISK">' + \
          '<disk>' + \
            '<smart/>' + \
            '<error/>' + \
            '<scsiinfo>' + \
              '<name/>' + \
              '<vendor/>' + \
              '<firmware/>' + \
            '</scsiinfo>' + \
            '<atainfo>' + \
              '<model/>' + \
              '<firmware/>' + \
              '<serialnumber/>' + \
            '</atainfo>' + \
          '</disk>' + \
    '</device>'

USB_DISK_XML = \
    '<device type="DISK">' + \
       '<disk>' + \
          '<volume>' + \
            '<capacity>' + \
              '<use>0</use>' + \
              '<total>0</total>' + \
            '</capacity>' + \
          '</volume>' + \
          '<scsiinfo>' + \
            '<name/>' + \
            '<firmware/>' + \
            '<vendor/>' + \
          '</scsiinfo>' + \
       '</disk>' + \
    '</device>'

USB_UPS_XML = \
   '<device type="UPS">' + \
      '<ups>' + \
          '<battery/>' + \
          '<usbinfo>' + \
            '<name/>' + \
            '<vendor/>' + \
            '<firmware/>' + \
          '</usbinfo>' + \
      '</ups>' + \
    '</device>'

SERIAL_UPS_XML = \
   '<device type="UPS">' + \
      '<ups>' + \
          '<battery/>' + \
          '<serialinfo>' + \
            '<name>unknown</name>' + \
            '<vendor>unknown</vendor>' + \
            '<firmware>unknown</firmware>' + \
          '</serialinfo>' + \
      '</ups>' + \
    '</device>'

USB_LPR_XML = \
   '<device type="LPR">' + \
      '<lpr>' + \
          '<usbinfo>' + \
            '<name/>' + \
            '<vendor/>' + \
            '<firmware/>' + \
          '</usbinfo>' + \
      '</lpr>' + \
    '</device>'

VOLELEMENT_TAGS = ""
VOLELEMENT_TAG = \
    '<volelement>//deviceport/sataport/port[number="%s"]</volelement>'

for port_num in util.get_enable_sata_port_list():
    VOLELEMENT_TAGS += VOLELEMENT_TAG % port_num

VOLLIST_BASE_XML = \
    '<vollist>' + \
     '<vol level="RAID_X" type="raid">' + \
      '<capacity>' + \
       '<use>0</use>' + \
       '<total>0</total>' + \
      '</capacity>' + \
      '<volelementlist>' + \
      VOLELEMENT_TAGS + \
      '</volelementlist>' + \
     '</vol>' + \
    '</vollist>'

VOLLINKLIST_XML = '<vol><vollinklist/></vol>'
USB_VOLELEMENT_TAG = \
    '<volelement>//deviceport/usbport/port[number="%s"]</volelement>'

SMART_EXT_ID_XPATH_MAP = {
    # read
    "61456": "./read/total",
    "61457": "./read/range0_20ms",
    "61458": "./read/range20_40ms",
    "61459": "./read/range40_60ms",
    "61460": "./read/range60_80ms",
    "61461": "./read/range80_100ms",
    "61462": "./read/range100_120ms",
    "61463": "./read/range120_140ms",
    "61464": "./read/range140_160ms",
    "61465": "./read/range160_180ms",
    "61466": "./read/range180_200ms",
    "61467": "./read/range200_500ms",
    "61468": "./read/range500_1s",
    "61469": "./read/range1_5s",
    "61470": "./read/range5_10s",
    "61471": "./read/range10_15s",
    "61472": "./read/range15s_",
    # write
    "61488": "./write/total",
    "61489": "./write/range0_20ms",
    "61490": "./write/range20_40ms",
    "61491": "./write/range40_60ms",
    "61492": "./write/range60_80ms",
    "61493": "./write/range80_100ms",
    "61494": "./write/range100_120ms",
    "61495": "./write/range120_140ms",
    "61496": "./write/range140_160ms",
    "61497": "./write/range160_180ms",
    "61498": "./write/range180_200ms",
    "61499": "./write/range200_500ms",
    "61500": "./write/range500_1s",
    "61501": "./write/range1_5s",
    "61502": "./write/range5_10s",
    "61503": "./write/range10_15s",
    "61504": "./write/range15s_"
}

# XML Transfer データ構造
# MODULE: PALモジュール名,
# REQ_TYPE: リクエストタイプ名,
# REQ_XML: リクエストXML,
# REQ_XML_ARG_TYPES: REQ_XMLの引数"名"のリスト(%sがついている場合)
#                    引数名にしたがって__update_landiskinfo_node()が%sを変換
# SRC_DST_LST:
#    PALの出力XMLのどの値をlandiskinfoノードのどのパスに挿入するかのリスト
#    リストの要素は辞書
#       必須キー :
#                  SRC_PATH => PALの出力から取得する値のxpath
#                  DST_PATH => 取得した値のlandiskinfoノードへの挿入先xpath
#
#       オプションキー:
#                  DST_ARG_TYPES => DST_PATHの引数"名"のリスト
#                                   (%sがついている場合)
#                  DST_ATTR      => 挿入先が属性の場合の属性名
#                  IS_TRANSFER_NODE => SRC_PATHのノードごとDST_PATH以下に追加する(True, False)
#                                   デフォルト False

PAL_MODULE = "palmodule"
REQ_TYPE = "module"
REQ_XML = "reqxml"
SRC_DST_LST = "srcdstlist"
SRC_PATH = "srcpath"
DST_PATH = "dstpath"
DST_ARG_TYPES = "dstargtypes"
REQ_XML_ARG_TYPES = "reqxmlargtypes"
DST_ATTR = "dstattr"
DEVNAME_KEY = "devname"
PORTNUM_KEY = "portnum"
ARG_GETTER = "arggetter"
IS_TRANSFER_NODE = "istransfernode"
IS_UPDATE_NODE = "isupdatenode"
TEXT_CONVERTER = "textconverter"
WITH_FILE = "withfile"
RESULT_CONVERTER = "resultconverter"

INTERNAL_PARTITION_NAME = \
    config.VOL_INFO[config.STOR_IF_TO_VOL["internal"][1]]["dev"]

def conv_upper(str):
    return str.upper()

SYSINFO_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_SYSINFO_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<sysinfo><option shortver="true" detail="true"/></sysinfo>',
     SRC_DST_LST: [
         # model
         {SRC_PATH: "./model",
          DST_PATH: "./product/name"},

         # firmware
         {SRC_PATH: "./fwversion",
          DST_PATH: "./product/firmware"},

         # fan uptime
         {SRC_PATH: "./uptime/fan/time/hour",
          DST_PATH: "./systeminfo/uptime/fan/time/hour"},

         {SRC_PATH: "./uptime/fan/time/minutes",
          DST_PATH: "./systeminfo/uptime/fan/time/minute"},

         # power uptime
         {SRC_PATH: "./uptime/power/time/hour",
          DST_PATH: "./systeminfo/uptime/power/time/hour"},

         {SRC_PATH: "./uptime/power/time/minutes",
          DST_PATH: "./systeminfo/uptime/power/time/minute"},

         # device uptime
         {SRC_PATH: "./uptime/device/time/hour",
          DST_PATH: "./systeminfo/uptime/device/time/hour"},

         {SRC_PATH: "./uptime/device/time/minutes",
          DST_PATH: "./systeminfo/uptime/device/time/minute"},

         # temperature
         {SRC_PATH: "./temperature[@type='mb']",
          DST_PATH: "./systeminfo/temperature"},

         # fan
         {SRC_PATH: "./fanrpm",
          DST_PATH: "./systeminfo/fanrpm"}
    ]}

NETWORK_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_NETWORK_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: "<network/>",
     SRC_DST_LST: [
         # model
         {SRC_PATH: "./netiflist/netif[@dev='netif1']/mac",
          DST_PATH: "./network/tcpip/lan/mac"},
    ]}


# INTERNAL_VOLUME_DATA_TRANSFER = \
#     {PAL_MODULE: pal_info.PAL_VOLCTRL_PATH,
#      REQ_TYPE: ReqTypes.QUERY,
#      REQ_XML: '<volctrl><volume>' + \
#               '<name type="partition">' + \
#               INTERNAL_PARTITION_NAME + \
#               '</name>' + \
#               '</volume></volctrl>',
#
#      SRC_DST_LST: [
#          # total capacity
#          {SRC_PATH: "./volume/capacity/total",
#           DST_PATH: "./vollist/vol/capacity/total"},
#
#          # use capacity
#          {SRC_PATH: "./volume/capacity/use",
#           DST_PATH: "./vollist/vol/capacity/use"}
#      ]}


INTERNAL_VOLUME_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_VOLCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML_ARG_TYPES: (DEVNAME_KEY,),
     ARG_GETTER: "",
     REQ_XML: '<volctrl><volume>' + \
              '<name type="partition">%s</name>' + \
              '</volume></volctrl>',

     SRC_DST_LST: [
         #total capacity
         {SRC_PATH: "./volume/capacity/total",
          DST_PATH: "./vollist/vol/capacity/total"},

         #use capacity
         {SRC_PATH: "./volume/capacity/use",
          DST_PATH: "./vollist/vol/capacity/use"}
     ]}

VFSEXT_INTERNAL_VOLUME_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_VOLCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<volctrl>' + \
              ' <volume>' + \
              '  <filesystem>vfsext</filesystem>' + \
              ' </volume>' + \
              '</volctrl>',

     SRC_DST_LST: [
         #total capacity
         {SRC_PATH: "./volume/capacity/total",
          DST_PATH: "./vollist/vol/capacity/total"},

         #use capacity
         {SRC_PATH: "./volume/capacity/use",
          DST_PATH: "./vollist/vol/capacity/use"}
     ]}


def convert_raid_level(pal_result_entry):

    RAID_LEVEL_XPATH = "raidlist/raid/level"

    try:
        raidctrl_node = pal_result_entry.get()
        raid_level = raidctrl_node.get_value(RAID_LEVEL_XPATH)
        raidctrl_node.get_node(RAID_LEVEL_XPATH).set_text(
            config.RAID_MODE_TO_REAL_RAID_MODE[raid_level])
    except:
        return pal_result_entry.get()

    return raidctrl_node

RAID_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_CONFFILE_PATH,
     REQ_TYPE: ReqTypes.LOAD,
     REQ_XML: "<raidctrl/>",
     WITH_FILE: config.RAIDCTRL_CONF_FILE,
     RESULT_CONVERTER: convert_raid_level,
     SRC_DST_LST: [
         # level
         {SRC_PATH: "./raidlist/raid/level",
          DST_PATH: "./vollist/vol",
          DST_ATTR: "level",
          TEXT_CONVERTER: conv_upper}
     ]}


SATA_DISK_SCSI_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_DEVCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<devctrl><device><name>%s</name></device></devctrl>',
     REQ_XML_ARG_TYPES: (DEVNAME_KEY,),
     ARG_GETTER: "",
     SRC_DST_LST: [
         # device name
         {SRC_PATH: "./device/disk/model",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
               "/device/disk/scsiinfo/name",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # vendor
         {SRC_PATH: "./device/disk/vendor",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
               "/device/disk/scsiinfo/vendor",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # firmware
         {SRC_PATH: "./device/disk/rev",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
               "/device/disk/scsiinfo/firmware",
          DST_ARG_TYPES: (PORTNUM_KEY,)}
     ]}

SATA_DISK_ERR_CHK_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_DISKCTRL_PATH,
     REQ_TYPE: ReqTypes.CHECK,
     REQ_XML: '<diskctrl><disk><name>%s</name></disk></diskctrl>',
     REQ_XML_ARG_TYPES: (DEVNAME_KEY,),
     ARG_GETTER: "",
     SRC_DST_LST: [
         # device name
         {SRC_PATH: "./disk/diskerrcnt",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
               "/device/disk/error",
          DST_ARG_TYPES: (PORTNUM_KEY,),
          TEXT_CONVERTER: {"noerror": "OK",
                           "error": "NG"}},

         {SRC_PATH: "./disk/smartstatus",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
               "/device/disk/smart",
          DST_ARG_TYPES: (PORTNUM_KEY,),
          TEXT_CONVERTER: {"noerror": "OK",
                           "error": "NG"}},
     ]}

SATA_DISK_ATA_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_DISKCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<diskctrl><option ata="true"/><disk><name>%s</name></disk></diskctrl>',
     REQ_XML_ARG_TYPES: (DEVNAME_KEY,),
     ARG_GETTER: "",
     SRC_DST_LST: [
         # model
         {SRC_PATH: "./disk/atainfo/model",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
               "/device/disk/atainfo/model",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # firmware
         {SRC_PATH: "./disk/atainfo/firmware",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
               "/device/disk/atainfo/firmware",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # serialnumber
         {SRC_PATH: "./disk/atainfo/serialnumber",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
               "/device/disk/atainfo/serialnumber",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # smartinfo
         {SRC_PATH: "./disk/smartinfo",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
               "/device/disk",
          DST_ARG_TYPES: (PORTNUM_KEY,),
          IS_TRANSFER_NODE: True},
     ]}

SMART_EXT_INFO_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_DISKCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<diskctrl><option ata="true"/><disk><name>%s</name></disk></diskctrl>',
     REQ_XML_ARG_TYPES: (DEVNAME_KEY,),
     ARG_GETTER: "",
     RESULT_CONVERTER: "",
     SRC_DST_LST: [
         # smartinfo
         {SRC_PATH: "./disk/smartdatalist",
          DST_PATH: "./deviceport/sataport/port[@number='%s']" + \
                      "/device/disk/smartinfo/smartdatalist",
          DST_ARG_TYPES: (PORTNUM_KEY,),
          IS_UPDATE_NODE: True, }
     ]}

USB_DISK_SCSI_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_DEVCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<devctrl><device><name>%s</name></device></devctrl>',
     REQ_XML_ARG_TYPES: (DEVNAME_KEY,),
     ARG_GETTER: "",
     SRC_DST_LST: [
         # device name
         {SRC_PATH: "./device/disk/model",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/disk/scsiinfo/name",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # vendor
         {SRC_PATH: "./device/disk/vendor",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/disk/scsiinfo/vendor",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # firmware
         {SRC_PATH: "./device/disk/rev",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/disk/scsiinfo/firmware",
          DST_ARG_TYPES: (PORTNUM_KEY,)}
     ]}


def convert_for_mirroring(pal_result_entry):

    REQ_XML = '<volctrl><volume>' + \
              '<name type="partition">%s</name>' + \
              '</volume></volctrl>'
    DISK_NAME_XPATH = "volume/name[@type='disk']"
    USE_SIZE_XPATH = "./volume/capacity/use"
    TOTAL_SIZE_XPATH = "./volume/capacity/total"

    try:
        volctrl_node = pal_result_entry.get()
        disk_name = volctrl_node.get_node(DISK_NAME_XPATH).get_text()
        vol_name = config.DEV_TO_VOL.get(disk_name)
        if MIRRORING_STATUS == \
               util.get_volume_cache_value(vol_name, VOL_STATUS_NAME):

            result_entry = util._send_request(pal_info.PAL_VOLCTRL_PATH,
                                              ReqTypes.QUERY,
                                              REQ_XML % config.MIRROR_DEV)
            if result_entry.get_result() != Results.OK:
                return volctrl_node

            use_size = \
                result_entry.get().get_node(USE_SIZE_XPATH).get_text()
            volctrl_node.get_node(USE_SIZE_XPATH).set_text(use_size)

            total_size = \
                result_entry.get().get_node(TOTAL_SIZE_XPATH).get_text()
            volctrl_node.get_node(TOTAL_SIZE_XPATH).set_text(total_size)
    except:
        return pal_result_entry.get()

    return volctrl_node


USB_VOLUME_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_VOLCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<volctrl><volume>' + \
              '<name type="disk">%s</name>' + \
              '</volume></volctrl>',
     REQ_XML_ARG_TYPES: (DEVNAME_KEY,),
     ARG_GETTER: "",
     RESULT_CONVERTER: convert_for_mirroring,
     SRC_DST_LST: [
         # total capacity
         {SRC_PATH: "./volume/capacity/total",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/disk/volume/capacity/total",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # use capacity
         {SRC_PATH: "./volume/capacity/use",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
                "/device/disk/volume/capacity/use",
          DST_ARG_TYPES: (PORTNUM_KEY,)}
     ]}


USB_VFSEXT_VOLUME_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_VOLCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<volctrl>' + \
              ' <volume>' + \
              '  <filesystem>vfsext</filesystem>' + \
              ' </volume>' + \
              '</volctrl>',

     SRC_DST_LST: [
         #total capacity
         {SRC_PATH: "./volume/capacity/total",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/disk/volume/capacity/total",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         #use capacity
         {SRC_PATH: "./volume/capacity/use",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
                "/device/disk/volume/capacity/use",
          DST_ARG_TYPES: (PORTNUM_KEY,)}
     ]}


USB_UPS_COMMON_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_DEVCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<devctrl><device><name>%s</name></device></devctrl>',
     REQ_XML_ARG_TYPES: (DEVNAME_KEY,),
     ARG_GETTER: "",
     SRC_DST_LST: [
         # device name
         {SRC_PATH: "./device/usb/product",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups/usbinfo/name",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # vendor
         {SRC_PATH: "./device/usb/manufacturer",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups/usbinfo/vendor",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # firmware
         {SRC_PATH: "./device/usb/rev",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups/usbinfo/firmware",
          DST_ARG_TYPES: (PORTNUM_KEY,)}
     ]}


USB_UPS_DETAIL_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_UPS_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<ups/>',
     ARG_GETTER: "",
     SRC_DST_LST: [
         # battery
         {SRC_PATH: "./battery",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups/battery",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # status attribute
         {SRC_PATH: "./status",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups",
          DST_ATTR: "status",
          DST_ARG_TYPES: (PORTNUM_KEY,)}
     ]}


SERIAL_UPS_COMMON_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_UPS_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<ups/>',
     ARG_GETTER: "",
     SRC_DST_LST: [
         # device name
         {SRC_PATH: "./model",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups/usbinfo/name",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # vendor
         {SRC_PATH: "./vendor",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups/usbinfo/vendor",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # firmware
         {SRC_PATH: "./firmware",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups/usbinfo/firmware",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # battery
         {SRC_PATH: "./battery",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups/battery",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # status attribute
         {SRC_PATH: "./status",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/ups",
          DST_ATTR: "status",
          DST_ARG_TYPES: (PORTNUM_KEY,)}
     ]}

USB_LPR_COMMON_DATA_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_DEVCTRL_PATH,
     REQ_TYPE: ReqTypes.QUERY,
     REQ_XML: '<devctrl><device><name>%s</name></device></devctrl>',
     REQ_XML_ARG_TYPES: (DEVNAME_KEY,),
     ARG_GETTER: "",
     SRC_DST_LST: [
         # device name
         {SRC_PATH: "./device/usb/product",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/lpr/usbinfo/name",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # vendor
         {SRC_PATH: "./device/usb/manufacturer",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/lpr/usbinfo/vendor",
          DST_ARG_TYPES: (PORTNUM_KEY,)},

         # firmware
         {SRC_PATH: "./device/usb/rev",
          DST_PATH: "./deviceport/usbport/port[@number='%s']" + \
               "/device/lpr/usbinfo/firmware",
          DST_ARG_TYPES: (PORTNUM_KEY,)}
     ]}


def get_vollinklist(pal_result_entry):

    TYPE_ATTR = "type"
    VOLLINKLIST_NAME = "vollinklist"
    MIRRORING_VALUE = "mirroring"
    USB_MIRRORIGN_XPATH = "usblist/usb[@port='%s']/modelist" + \
                            "/mode[@enable='true' and text()='mirroring']"

    CHK_EXIST_XML = '<devctrl><device><name>%s</name></device></devctrl>'


    portconfig_node = pal_result_entry.get()

    volel_node_list = []
    for port in config.USB_PORT_NO_LIST:

        dev_name = "usbd" + port
        result_entry = util._send_request(pal_info.PAL_DEVCTRL_PATH,
                                          ReqTypes.CHK_EXIST,
                                          CHK_EXIST_XML % dev_name)
        if result_entry.get_result() != Results.OK:
            continue

        try:
            # xpathの存在確認
            portconfig_node.get_node(USB_MIRRORIGN_XPATH % port)
            volel_node = XmlNode(USB_VOLELEMENT_TAG % port)
            volel_node.set_attrs({TYPE_ATTR: MIRRORING_VALUE})
            volel_node_list += [volel_node]

        except NotFoundError:
            continue

    if len(volel_node_list) == 0:
        raise NotFoundError

    vol_node = XmlNode(VOLLINKLIST_XML)
    vollinklist_node = vol_node.get_node(VOLLINKLIST_NAME)
    for volel_node in volel_node_list:
        vollinklist_node.append_child(volel_node)

    return vol_node


VOL_LINK_LIST_TRANSFER = \
    {PAL_MODULE: pal_info.PAL_CONFFILE_PATH,
     REQ_TYPE: ReqTypes.LOAD,
     REQ_XML: "<portconfig/>",
     WITH_FILE: config.PORTS_CONF_FILE,
     RESULT_CONVERTER: get_vollinklist,
     SRC_DST_LST: [
         {SRC_PATH: "./vollinklist",
          DST_PATH: "./vollist/vol",
          IS_TRANSFER_NODE: True,}
     ]}