#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Define utility functions for FAL commands."""


from __future__ import with_statement

import os
import re
import copy

import iobase.xmldata as xmldata
import iobase.exception as exception
import ionas.request as request
import pal_info
import fal_config as config
import melody_data
import iobase.fileut as fileut
from pal_info import (ReqTypes, Results)
from product_conf.devinfo import (VOL_INFO)

TEMP_PACFILE = os.path.join(config.TEMP_DIR, "temp.pac")

PROXY_NAME = "proxy"
MANUAL_NAME = "manual"
PACFILE_NAME = "pacfile"
URL_NAME = "url"
SERVERLIST_NAME = "serverlist"
SERVER_NAME = "server"
HOST_NAME = "host"
PORT_NAME = "port"
DATA_NAME = "data"
VOLUME_NAME = "volume"

NETIF_PATH = "netiflist/netif"
MAC_NAME = "mac"
DEV_ATTR = "dev"

MAC_SEP = ":"

STATUS_LED_BASE_REQ_XML = \
  '<led>' + \
  ' <status color=\"%s\">%s</status>' + \
  '</led>'

PLAY_MELODY_BASE_REQ_XML = \
  '<buzzer>' + \
  ' <score>%s</score>' + \
  '</buzzer>'

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR_NAME = "category"
ACTION_ATTR_NAME = "action"
PREVIOUS_ATTR_NAME = "previous"

TRUE = xmldata.XmlConfNode.TRUE_TEXT
FALSE = xmldata.XmlConfNode.FALSE_TEXT

CATEGORYNAMELIST_NAME = "categorynamelist"
SERVICELIST_NAME = "servicelist"
SERVICE_NAME = "service"
NAME_NAME = "name"
SERVICE_NAME_PATH = './servicelist/service/name'

CATEGORY_SERVICE_NAME_XPATH_TMPL = \
            './categorylist/category/name[text()="%s"]/../servicelist//name'
SERVICE_NAME_XPATH_TMPL = './servicelist/service/name[text()="%s"]/..'

ALL_CATEGORY = "all"

STATELIST_NAME = "statelist"
STATE_NAME = "state"
TIMESTAMP_NAME = "timestamp"
UUID_NAME = "uuid"
MIRR_SET_NAME = "mirrset"

STATE_PATH = "./statelist/state"

PACKAGE_PATH = "packagelist/package"
INTERNALNAME_PATH = "./internalname"
VERSION_PATH = "./version"
PACKAGE_INTERNALNAME_PATH = "packagelist/package/internalname"

DISK_INFO_LOCK_NAME = "disk_info_cache"
VOL_LOCK_NAME = "volume_info_cache"
RAID_LOCK_NAME = "raid_info_cache"


# DISK TEMP STATUS
NORMAL_DISK_STATUS = "normal"
FAIL_DISK_STATUS = "fail"
UNPLUG_DISK_STATUS = "unplug"

# VOLUME CACHE TYPE
STATUS_VOL_CACHE = "status"

# VOLUME TEMP STATUS
MOUNTED_STATUS = "mounted"
ENC_MOUNTED_STATUS = "encmounted"
UNMOUNTED_STATUS = "unmounted"
LOCKKEY_STATUS = "lockkey"
KEYWAITING_STATUS = "keywaiting"
MOUNTFAIL_STATUS = "mountfail"
WAITING_STATUS = "waiting"
CAPSHORTAGE_STATUS = "capshortage"
CHECKING_STATUS = "checking"
ENC_CHECKING_STATUS = "encchecking"
FORMATTING_STATUS = "formatting"
ENC_FORMATTING_STATUS = "encformatting"
UNFORMATTED_STATUS = "unformated"
MIRRORING_STATUS = "mirroring"
EXIST_DISK_LABEL_STATUS = "existdisklabel"
FAIL_STATUS = "fail"

SERIES_PATH = "./series"
MODEL_PATH = "./model"
FWVERSION_PATH = "./fwversion"
FWVERSION_RE = "([0-9]+\.[0-9]+).*"

GPT_PART_START = 40


class _NotMatch:
    """Exception class for representing that "re.match()" return None."""

    pass


class RequestError(Exception):
    """Request error"""

    def get_result(self):
        """Return the first value of "args" as result code.
           If exception occured, then empty string is returned.
        """

        try:
            return self.args[0]
        except:
            return ""


class FileNotFound(Exception):
    """File Not Found"""

    pass


def _dev_to_port_info(dev_name, pattern):
    """Return tuple of port I/F name and number if "dev_name" match with
       "pattern".
    """

    match = re.match(pattern, dev_name.strip())
    if match == None:
        raise _NotMatch

    return (match.group(1), match.group(2))


def _get_entry(type, base_xml_dict, file_name=None, altfile_name=None,
        opt_attrs=None):
    """Create entry and return it."""

    return request.Entry(type, base_xml_dict[type], file_name=file_name,
                            altfile_name=altfile_name, opt_attrs=opt_attrs)


def _is_exist_node(node, node_path):
    """Check whether node_path exists in specified node"""

    try:
        node.get_node(node_path)
    except exception.NotFoundError:
        return False
    return True


def _partition2disk(part_name):
    """Convert partition name to corresponding disk name."""

    match = re.match("([a-zA-Z]+[0-9]+)", part_name)

    disk_name = match.group(1)

    if disk_name not in config.SATA_HDD_DEV_LIST + config.USB_HDD_DEV_LIST:
        raise exception.NotFoundError

    return disk_name


def _request_entry(pal_cmd, entry, dry_run, debug):
    """Send "entry" to "pal_cmd" and return entry in response data."""

    return request.Request(pal_cmd).request(entry,
                                    dry_run=dry_run, debug=debug).get_entry()


def _urlaccess_request(url, type, send_file, data, receive_file, proxy,
        dry_run, debug):
    """Create a entry for "pal_urlaccess" and send it to "pal_urlaccess"."""

    entry = _get_entry(type, pal_info.PAL_URLACCESS_REQ_BASE_XML,
                        file_name=send_file, altfile_name=receive_file)
    entry.get().get_node(URL_NAME).set_text(url)

    if data:
        entry.get().append_child(DATA_NAME, data)

    if proxy != None:
        entry.get().append_child(proxy)

    response_entry = _request_entry(pal_info.PAL_URLACCESS_PATH,
                        entry, dry_run, debug)

    if response_entry.get_result() == Results.OK:
        return response_entry

    if response_entry.get_result() == Results.SEND_ERROR:
        raise SendError

    if response_entry.get_result() == Results.RECEIVE_ERROR:
        raise ReceiveError

    if response_entry.get_result() == Results.CONNECT_ERROR:
        raise ConnectError

    raise InternalError


def load_config(conf_file, dry_run=False, debug=False):
    """Load config setting and return XML data ."""

    entry = _get_entry(
        ReqTypes.LOAD, pal_info.PAL_CONFFILE_REQ_BASE_XML,
        file_name=conf_file)
    return _request_entry(
        pal_info.PAL_CONFFILE_PATH, entry, dry_run, debug).get()


def save_config(conf_file, config_xml, dry_run=False, debug=False):
    """save "config_xml" XML data to "conf_file"."""

    entry = _get_entry(
        ReqTypes.SAVE, pal_info.PAL_CONFFILE_REQ_BASE_XML,
        file_name=conf_file)
    entry.set(config_xml)
    _request_entry(
        pal_info.PAL_CONFFILE_PATH, entry, dry_run, debug)


def _get_proxy_config(dry_run, debug):
    """Load proxy config setting and return XML data ."""

    return load_config(config.PROXY_CONF_FILE, dry_run, debug)


def _get_manual_proxy(proxy_config):
    """Return a proxy server list node which is set by manual."""

    manual = proxy_config.get_node(MANUAL_NAME)
    if not manual.is_enabled():
        raise NoProxy()

    serverlist = xmldata.XmlConfNode(SERVERLIST_NAME)
    server = xmldata.XmlConfNode(SERVER_NAME)
    server.append_child(manual.get_node(HOST_NAME).get_clone())
    server.append_child(manual.get_node(PORT_NAME).get_clone())
    serverlist.append_child(server)
    return serverlist


def _download_pacfile(pacfile, dry_run, debug):
    """Download a pac file as "local_path"."""

    pacfile_url = pacfile.get_node(URL_NAME).get_text()
    if not pacfile_url:
        raise NoProxy()

    urlaccess_receive(pacfile_url, TEMP_PACFILE, proxy=None,
                        dry_run=dry_run, debug=debug)
    return TEMP_PACFILE


def _get_proxy_from_pacfile(proxy_config, url, dry_run, debug):
    """Return a proxy "serverlist" node got from a pacfile"""

    pacfile = proxy_config.get_node(PACFILE_NAME)
    if not pacfile.is_enabled():
        raise NoProxy()

    local_pacfile = _download_pacfile(pacfile, dry_run, debug)
    entry = _get_entry(ReqTypes.QUERY, pal_info.PAL_PROXY_REQ_BASE_XML,
                        file_name=local_pacfile)
    entry.get().get_node(URL_NAME).set_text(url)

    response_entry = _request_entry(pal_info.PAL_PROXY_PATH, entry,
                                    dry_run, debug)

    try:
        os.remove(local_pacfile)
    except:
        pass

    if response_entry.get_result() == Results.NO_PROXY:
        raise NoProxy()

    if response_entry.get_result() != Results.OK:
        raise InternalError

    return response_entry.get().get_node(SERVERLIST_NAME)


def _get_servicectrl_config(dry_run, debug):
    """Load servicectrl config setting and return XML data ."""

    return load_config(config.SERVICECTRL_CONF_FILE, dry_run, debug)


def _save_servicectrl_config(config_xml, dry_run, debug):
    """Save servicectrl config setting."""

    save_config(config.SERVICECTRL_CONF_FILE, config_xml, dry_run, debug)


def _get_enabled_service_list(servicectrl_config, service_list):
    """Return enabled service list."""

    new_service_list = []
    for service_name in service_list:
        service = servicectrl_config.get_node(SERVICE_NAME_XPATH_TMPL %
                    (service_name,))
        if service.is_enabled():
            new_service_list.append(service_name)

    return new_service_list


def _get_service_list(category_list, enabled_only, dry_run, debug):
    """Return service list for "category_list"."""

    servicectrl_config = _get_servicectrl_config(dry_run, debug)
    service_list = []
    for category in category_list:
        if category == ALL_CATEGORY:
            service_list[len(service_list):] = \
                servicectrl_config.get_value_list(SERVICE_NAME_PATH)
        else:
            service_list[len(service_list):] = \
                servicectrl_config.get_value_list(
                    CATEGORY_SERVICE_NAME_XPATH_TMPL % (category,))

    if enabled_only:
        service_list = _get_enabled_service_list(
                            servicectrl_config, service_list)
    return service_list


def _add_servicelist(entry, category_list, service_list, enabled_only,
        dry_run, debug):
    """Add "service" entry to "servicelist" node in "entry" for
       "category_list" and "service_list".
    """

    servicelist = entry.get().get_node(SERVICELIST_NAME)
    appended_service_list = []
    for service_name in _get_service_list(category_list, enabled_only,
                                            dry_run, debug) + \
                                    list(service_list):

        if service_name in appended_service_list:
            continue

        appended_service_list.append(service_name)
        service = xmldata.XmlConfNode(SERVICE_NAME)
        service.append_child(NAME_NAME, service_name)
        servicelist.append_child(service)


def _servicectrl_request(type, category_list, service_list, enabled_only=False,
        dry_run=None, debug=None):
    """Send request to "pal_servicectrl"."""

    entry = _get_entry(type, pal_info.PAL_SERVICECTRL_REQ_BASE_XML)
    _add_servicelist(entry, category_list, service_list, enabled_only,
                        dry_run, debug)
    response_entry = _request_entry(pal_info.PAL_SERVICECTRL_PATH, entry,
                            dry_run, debug)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get()


def _statemgr_request(type, category, state=None,
        dry_run=None, debug=None):
    """Send request to "pal_statemgr"."""

    entry = _get_entry(type, pal_info.PAL_STATEMGR_REQ_BASE_XML)
    if type == ReqTypes.QUERY:
        entry.get().get_node(STATELIST_NAME).append_child(
                            STATE_NAME, attrs={CATEGORY_ATTR_NAME: category})
    else:
        state_node = entry.get().get_node(STATE_NAME)
        state_node.set_attrs({CATEGORY_ATTR_NAME: category})
        state_node.set_text(state)

    response_entry = _request_entry(pal_info.PAL_STATEMGR_PATH, entry,
                            dry_run, debug)

    if response_entry.get_result() == Results.CHANGE_ERROR:
        raise ChangeError
    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get()


#
# Public classes
#
class SendError(Exception):
    """Send error"""

    pass


class ReceiveError(Exception):
    """Receive error"""

    pass


class ConnectError(Exception):
    """Connect error"""

    pass


class InternalError(Exception):
    """Internal error"""

    pass


class NoProxy(Exception):
    """The proxy is not found"""

    pass


class EventError(Exception):
    """Event error"""

    pass


class RequestError(Exception):
    """Request error"""

    pass


class ChangeError(Exception):
    """Change error"""

    pass


#
# Public functions
#
def dev_to_port_info(dev_name):
    """Return tuple of port I/F name and number."""

    dev_name = os.path.basename(dev_name)
    try:
        return _dev_to_port_info(dev_name, "(.+)d([0-9]+).*")
    except:
        pass

    try:
        return _dev_to_port_info(dev_name, "(.+)ups([0-9]+).*")
    except:
        pass

    try:
        return _dev_to_port_info(dev_name, "(.+)lp([0-9]+).*")
    except:
        pass

    return ("", "")


def urlaccess_send(url, data, is_file=False, proxy=None,
        dry_run=None, debug=None):
    """Send "data" to "url" by "pal_urlaccess".
       If "is_file" is "True", then "data" is assumed to be file path.
    """

    if is_file:
        send_file = data
        data = None
    else:
        send_file = None

    _urlaccess_request(url, ReqTypes.SEND, send_file=send_file, data=data,
                        receive_file=None, proxy=proxy,
                        dry_run=dry_run, debug=debug)


def urlaccess_receive(url, receive_file=None, proxy=None,
        dry_run=None, debug=None):
    """Receive data from "url" by "pal_urlaccess" and return it
       or save it to "receive_file".
    """

    response_entry = _urlaccess_request(url, ReqTypes.RECEIVE, send_file=None,
                                    data=None, receive_file=receive_file,
                                    proxy=proxy, dry_run=dry_run, debug=debug)
    return response_entry.get().get_value(DATA_NAME)


def urlaccess_get(url, data, is_file=False, receive_file=None, proxy=None,
            dry_run=None, debug=None):
    """Send "data" to "url" by "pal_urlaccess", receive data and return it
       or save it to "receive_file".
       If "is_file" is "True", then "data" is assumed to be file path.
    """

    if is_file:
        send_file = data
        data = None
    else:
        send_file = None

    response_entry = _urlaccess_request(url, ReqTypes.GET, send_file=send_file,
                                    data=data, receive_file=receive_file,
                                    proxy=proxy, dry_run=dry_run, debug=debug)
    return response_entry.get().get_value(DATA_NAME)


def get_proxy(url, dry_run=None, debug=None):
    """Return "proxy" XmlNode instance of the proxy servers for "url".
       Raise "NoProxy" exception if no server is found.
    """
    proxy_config = _get_proxy_config(dry_run, debug)
    if not proxy_config.is_enabled():
        raise NoProxy()

    try:
        serverlist = _get_manual_proxy(proxy_config)
    except NoProxy:
        serverlist = _get_proxy_from_pacfile(proxy_config, url, dry_run, debug)

    proxy = xmldata.XmlConfNode(PROXY_NAME)
    proxy.append_child(serverlist)
    return proxy


def _send_request(target_module, type, child_xml="",
                 dry_run=None, debug=None):
    """Send entry to specified target pal module."""

    entry = request.Entry(type, child_xml)
    return _request_entry(target_module, entry, dry_run=None, debug=None)


def get_mac_address(netif, no_separator=False, dry_run=None, debug=None):
    """Return a MAC address string of "netif" interface.
       If "no_separator" is True, ":" is removed.
    """

    entry = _get_entry(ReqTypes.QUERY, pal_info.PAL_NETWORK_REQ_BASE_XML)
    response_entry = _request_entry(pal_info.PAL_NETWORK_PATH, entry,
                            dry_run, debug)

    if response_entry.get_result() != Results.OK:
        raise InternalError

    network = response_entry.get()
    mac_addr = network.get_node(NETIF_PATH,
                            {DEV_ATTR: netif}).get_node(MAC_NAME).get_text()
    if not mac_addr:
        raise InternalError

    if no_separator:
        mac_addr = mac_addr.replace(MAC_SEP, "")

    return mac_addr


def change_status_led_mode_for_setting(led_mode, dry_run=None, debug=None):
    """Change STATUS LED to specified mode.
       "led_mode" can be set "blink" or "on".
    """

    entry = request.Entry(ReqTypes.CONTROL,
                          STATUS_LED_BASE_REQ_XML % ("green", led_mode))
    _request_entry(pal_info.PAL_LED_PATH, entry, dry_run, debug)
    return


def beep_buzzer_for_setting(is_error=False, dry_run=None, debug=None):
    """Beep buzzer for setting"""

    if is_error:
        req_xml = PLAY_MELODY_BASE_REQ_XML % melody_data.ERROR
    else:
        req_xml = PLAY_MELODY_BASE_REQ_XML % melody_data.OK

    entry = request.Entry(ReqTypes.PLAY, req_xml)
    _request_entry(pal_info.PAL_BUZZER_PATH, entry, dry_run, debug)
    return


def dispatch_event(category, action, body, previous=False,
        dry_run=None, debug=None):
    """Dispatch event by "pal_events"."""

    entry = _get_entry(ReqTypes.EVENT, pal_info.PAL_EVENTS_REQ_BASE_XML)
    attrs = {CATEGORY_ATTR_NAME: category, ACTION_ATTR_NAME: action}
    if previous:
        attrs[PREVIOUS_ATTR_NAME] = TRUE
    entry.get().get_node(EVENTINFO_NAME).set_attrs(attrs)
    entry.get().get_node(EVENTINFO_NAME).append_child(body)

    response_entry = _request_entry(pal_info.PAL_EVENTS_PATH, entry,
                            dry_run, debug)

    if response_entry.get_result() != Results.OK:
        raise EventError


def enable_service(func_name, enable=True, do_start_stop=False,
        dry_run=None, debug=None):
    """Enable or disable service and start or stop it by "pal_servicectrl"
       if "do_start_stop" is True.
    """

    service_name = config.FUNC_TO_SERVICE_MAP[func_name]
    servicectrl_config = _get_servicectrl_config(dry_run, debug)
    try:
        service = servicectrl_config.get_node(SERVICE_NAME_XPATH_TMPL %
                    (service_name,))
    except:
        return

    if service.is_enabled() == enable:
        return

    service.enable(enable)
    _save_servicectrl_config(servicectrl_config, dry_run, debug)
    if do_start_stop:
        if enable:
            servicectrl_start([], [service_name], dry_run, debug)
        else:
            servicectrl_stop([], [service_name], dry_run, debug)


def servicectrl_query(category_list, service_list, dry_run=None, debug=None):
    """Get service status by "pal_servicectrl"."""

    return _servicectrl_request(ReqTypes.QUERY, category_list, service_list,
                            False, dry_run, debug)


def servicectrl_start(category_list, service_list, dry_run=None, debug=None):
    """Start service by "pal_servicectrl"."""

    return _servicectrl_request(ReqTypes.START, category_list, service_list,
                            True, dry_run, debug)


def servicectrl_stop(category_list, service_list, dry_run=None, debug=None):
    """Stop service by "pal_servicectrl"."""

    return _servicectrl_request(ReqTypes.STOP, category_list, service_list,
                            True, dry_run, debug)


def servicectrl_suspend(category_list, service_list, dry_run=None, debug=None):
    """Suspend service by "pal_servicectrl"."""

    return _servicectrl_request(ReqTypes.SUSPEND, category_list, service_list,
                            False, dry_run, debug)


def servicectrl_resume(category_list, service_list, dry_run=None, debug=None):
    """Resume service by "pal_servicectrl"."""

    return _servicectrl_request(ReqTypes.RESUME, category_list, service_list,
                            False, dry_run, debug)


def statemgr_query(category, dry_run=None, debug=None):
    """Return the state of "category" by "pal_statemgr"."""

    result = _statemgr_request(ReqTypes.QUERY, category,
                    dry_run=None, debug=None)
    return result.get_node(STATE_PATH).get_text()


def statemgr_set(category, state, dry_run=None, debug=None):
    """Set the state of "category" to "state" by "pal_statemgr"."""

    _statemgr_request(ReqTypes.SET, category, state, dry_run=None, debug=None)


def statemgr_change(category, state, dry_run=None, debug=None):
    """Change the state of "category" to "state" by "pal_statemgr"."""

    _statemgr_request(ReqTypes.CHANGE, category, state,
                        dry_run=None, debug=None)


def is_device_exist(dev_name, dry_run=None, debug=None):
    """Check whether specified device exist"""

    check_exist_req_xml = \
        '<devctrl><device><name>%s</name></device></devctrl>' % dev_name

    entry = request.Entry(ReqTypes.CHK_EXIST, check_exist_req_xml)
    response_entry = _request_entry(pal_info.PAL_DEVCTRL_PATH, entry,
                            dry_run, debug)

    if response_entry.get_result() == Results.OK:
        return True

    return False


def is_installed_package(package_name, dry_run=None, debug=None):

    try:
        installed_info = load_config(
            config.INSTALLED_PACKAGES_FILE, dry_run, debug)
        installed_package_list = installed_info.get_value_list(
            PACKAGE_INTERNALNAME_PATH)
    except:
        return False

    if package_name in installed_package_list:
        return True
    return False


def get_package_version(package_name, dry_run=None, debug=None):
    """Get specified package version."""

    try:
        installed_info = load_config(
            config.INSTALLED_PACKAGES_FILE, dry_run, debug)
        package_list = installed_info.get_node_list(PACKAGE_PATH)
        for package in package_list:
            current_name = package.get_node(INTERNALNAME_PATH).get_text()
            if current_name == package_name:
                current_version_str = package.get_node(VERSION_PATH).get_text()
                return float(current_version_str)
    except:
        pass

    raise exception.NotFoundError(package_name)


def vfsext_unmount():

    UNMOUNT_XML_STR = \
        '<volctrl><volume><dst>%s</dst></volume></volctrl>'

    for element, info in get_vfsext_dev_info().iteritems():
        req_xml = UNMOUNT_XML_STR % (info["mnt_point"])
        entry = request.Entry(ReqTypes.UNMOUNT, req_xml)
        result_entry = _request_entry(pal_info.PAL_VOLCTRL_PATH, entry,
                                      dry_run=None, debug=None)
    return


def set_used_port(port, service, dry_run=None, debug=None):

    conf = load_config(config.VALIDATES_XML_FILE, dry_run, debug)

    try:
        node = conf.get_node(
            "nwports/used/portlist/port[@service='%s']" % service)
        node.set_text(port)
    except exception.NotFoundError:
        par_node = conf.get_node("nwports/used/portlist")
        par_node.append_child("port", port, {"service": service})

    save_config(config.VALIDATES_XML_FILE, conf, dry_run, debug)


def unset_used_port(service, dry_run=None, debug=None):

    conf = load_config(config.VALIDATES_XML_FILE, dry_run, debug)

    try:
        par_node = conf.get_node("nwports/used/portlist")
        par_node.delete_child("port", {"service": service})
    except exception.NotFoundError:
        pass

    save_config(config.VALIDATES_XML_FILE, conf, dry_run, debug)


def increment_disk_errorcount(disk_name):

    INCERRCNT_REQ_XML_TMPL = \
        """<volctrl>
             <option operation="setdisklabel" incerrcnt="true"/>
             <volume>
               <name type="disk">%s</name>
             </volume>
           </volctrl>"""

    req_xml = \
        INCERRCNT_REQ_XML_TMPL % disk_name

    entry = request.Entry(ReqTypes.CONTROL, req_xml)
    result_entry = _request_entry(pal_info.PAL_VOLCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    result_code = result_entry.get_result()
    if Results.OK != result_code:
        raise RequestError

    return


def set_disklabel_value(disk_name, target, value):

    SET_DISKLABEL_REQ_XML_TMPL = \
        """<volctrl>
             <option operation="setdisklabel"/>
             <volume>
               <name type="disk">%(disk)s</name>
               <disklabel>
                 <%(target)s>%(value)s</%(target)s>
               </disklabel>
             </volume>
           </volctrl>"""

    req_xml = \
        SET_DISKLABEL_REQ_XML_TMPL % {"disk": disk_name,
                                      "target": target,
                                      "value": value}

    entry = request.Entry(ReqTypes.CONTROL, req_xml)
    result_entry = _request_entry(pal_info.PAL_VOLCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    result_code = result_entry.get_result()
    if Results.OK != result_code:
        raise RequestError

    return


def set_disklabel_infos(disk_name, disk_label_info_dict):

    SET_DISKLABEL_REQ_XML_TMPL = \
        """<volctrl>
             <option operation="setdisklabel"/>
             <volume>
               <name type="disk">%s</name>
               <disklabel/>
             </volume>
           </volctrl>"""

    DISK_LABEL_XPATH = "./volume/disklabel"

    volctrl_node = xmldata.XmlNode(SET_DISKLABEL_REQ_XML_TMPL % disk_name)
    disklabel_node = volctrl_node.get_node(DISK_LABEL_XPATH)

    for item, value in disk_label_info_dict.iteritems():
        disklabel_node.append_child(item, value)

    entry = request.Entry(ReqTypes.CONTROL, volctrl_node)
    result_entry = _request_entry(pal_info.PAL_VOLCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    result_code = result_entry.get_result()
    if Results.OK != result_code:
        raise RequestError

    return


def set_disk_info(disk_name, info_type, value):
    """Save disk info to temp file"""

    info_path = \
        os.path.join(config.DISK_CACHE_FILE_TMPL % (disk_name, info_type))

    dir_path = os.path.dirname(info_path)
    if not os.path.isdir(dir_path):
        os.makedirs(dir_path)

    with fileut.FLock(config.LOCK_DIR, DISK_INFO_LOCK_NAME, exclusive=False):
        with open(info_path, "w") as fh:
            fh.write(value)
    return


def get_disk_info(disk_name, info_type):
    """Load disk info to temp file"""

    info_path = \
        os.path.join(config.DISK_CACHE_FILE_TMPL % (disk_name, info_type))

    if not os.path.isfile(info_path):
        return ""

    with fileut.FLock(config.LOCK_DIR, DISK_INFO_LOCK_NAME, exclusive=False):
        with open(info_path, "r") as fh:
            return fh.read().rstrip()


def is_volume_exist_any_files(vol_name):

    CHK_FILE_EXIST_REQ_XML_TMPL = \
        """<volctrl>
             <option operation="checkfileexist"/>
             <volume>
               <name type="disk">%s</name>
             </volume>
           </volctrl>"""

    req_xml = \
        CHK_FILE_EXIST_REQ_XML_TMPL % VOL_INFO[vol_name]["dev"]

    entry = request.Entry(ReqTypes.CONTROL, req_xml)
    result_entry = _request_entry(pal_info.PAL_VOLCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    result_code = result_entry.get_result()
    if Results.OK == result_code:  # ファイルが存在する
        return True

    if Results.FILE_NOT_FOUND == result_code:  # ファイルが存在しない
        return False

    if Results.DEVICE_NOT_FOUND == result_code:
        return False

    # 予定外のエラーが出た場合は安全のためにwaiting状態
    return True


def get_disk_label_info(disk_name):

    # Return XML
    # <volctrl><volume>
    #  <disklabel>
    #   <errorcount>
    #   <magic>
    #   <md5sum>
    #   <mindisk>
    #   <raidmode>
    #   <usage>
    #   <version>

    disk_label_info_dict = \
        {"errorcount": "",
         "magic": "",
         "md5sum": "",
         "mindisk": "",
         "raidmode": "",
         "usage": "",
         "model": "",
         "version": "",
         "default_swap_size": ""}

    GET_DISK_LABEL_REQ_XML_TMPL = \
        """<volctrl>
             <option operation="getdisklabel"/>
             <volume>
               <name type="disk">%s</name>
             </volume>
           </volctrl>"""

    req_xml = \
        GET_DISK_LABEL_REQ_XML_TMPL % disk_name

    entry = request.Entry(ReqTypes.CONTROL, req_xml)
    result_entry = _request_entry(pal_info.PAL_VOLCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    result_code = result_entry.get_result()
    if Results.OK != result_code:
        raise exception.NotFoundError

    for key in disk_label_info_dict.keys():
        disk_label_info_dict[key] = \
            result_entry.get().get_value("./volume/disklabel/%s" % key)

    return disk_label_info_dict


def is_errorcount_over(disk_name):

    ERROR_CNT_THRESHOLD = 100
    try:
        info_dict = get_disk_label_info(disk_name)
        if int(info_dict["errorcount"]) >= ERROR_CNT_THRESHOLD:
            return True
    except:
        pass

    return False


def is_smart_error(disk_name):
    """Check smart error"""

    CHECK_SMART_ERROR_XML = \
        '<diskctrl><disk><name>%s</name></disk></diskctrl>'

    SMART_STATUS_XPATH = "disk/smartstatus"

    entry = request.Entry(ReqTypes.CHECK, CHECK_SMART_ERROR_XML % disk_name)
    result_entry = _request_entry(pal_info.PAL_DISKCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    if Results.OK != result_entry.get_result():
        return False

    smart_result = \
        result_entry.get().get_node(SMART_STATUS_XPATH).get_text()

    if smart_result == "error":
        return True

    return False


def set_volume_cache_value(vol_name, file_name, value):

    dir_path = os.path.dirname(config.VOLUME_CACHE_FILE_TMPL) % vol_name
    if not os.path.isdir(dir_path):
        os.makedirs(dir_path)

    file_path = \
        os.path.join(config.VOLUME_CACHE_FILE_TMPL % (vol_name, file_name))

    with fileut.FLock(config.LOCK_DIR, VOL_LOCK_NAME, exclusive=False):
        with open(file_path, "w") as fh:
            return fh.write(value)


def clear_volume_cache_value(vol_name, file_name):

    file_path = \
        os.path.join(config.VOLUME_CACHE_FILE_TMPL % (vol_name, file_name))

    with fileut.FLock(config.LOCK_DIR, VOL_LOCK_NAME, exclusive=False):
        if os.path.exists(file_path):
            os.remove(file_path)


def get_volume_cache_value(vol_name, file_name):

    file_path = \
        os.path.join(config.VOLUME_CACHE_FILE_TMPL % (vol_name, file_name))

    if not os.path.isfile(file_path):
        return ""

    with fileut.FLock(config.LOCK_DIR, VOL_LOCK_NAME, exclusive=False):
        with open(file_path, "r") as fh:
            return fh.read().rstrip()


def get_raid_cache_value(raid_name, file_name):

    file_path = \
        os.path.join(config.RAID_CACHE_FILE_TMPL % (raid_name, file_name))

    if not os.path.isfile(file_path):
        return ""

    with fileut.FLock(config.LOCK_DIR, RAID_LOCK_NAME, exclusive=False):
        with open(file_path, "r") as fh:
            return fh.read().rstrip()


def set_raid_cache_value(raid_name, file_name, value):

    file_path = \
        os.path.join(config.RAID_CACHE_FILE_TMPL % (raid_name, file_name))

    dir_path = os.path.dirname(file_path)
    if not os.path.isdir(dir_path):
        os.makedirs(dir_path)

    with fileut.FLock(config.LOCK_DIR, RAID_LOCK_NAME, exclusive=False):
        with open(file_path, "w") as fh:
            return fh.write(value)


def pal_volctrl_query_with_vfsext():

    VFS_QUERY_XML = """<volctrl>
                        <volume>
                         <filesystem>vfsext</filesystem>
                        </volume>
                       </volctrl>"""

    FILESYSTEM_XPATH = "./volume/filesystem"

    entry = request.Entry(ReqTypes.QUERY, VFS_QUERY_XML)
    result_entry = _request_entry(pal_info.PAL_VOLCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    if result_entry.get_result() != Results.OK:
        raise RequestError

    return result_entry.get()


def pal_diskctrl_query(disk_name, is_get_partition_info=True):
    """Send query request to pal_diskctrl"""

    DISK_NAME_XPATH = "./disk/name"

    if is_get_partition_info:
        QUERY_REQ_BASE_XML = \
            '<diskctrl>' + \
             '<option getpartitioninfo="true"/>' + \
             '<disk><name/></disk>' + \
            '</diskctrl>'
    else:
        QUERY_REQ_BASE_XML = '<diskctrl><disk><name/></disk></diskctrl>'

    diskctrl_node = xmldata.XmlNode(QUERY_REQ_BASE_XML)
    diskctrl_node.get_node(DISK_NAME_XPATH).set_text(disk_name)

    entry = request.Entry(ReqTypes.QUERY, diskctrl_node)
    result_entry = _request_entry(pal_info.PAL_DISKCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    if result_entry.get_result() != Results.OK:
        raise RequestError

    return result_entry


def get_disk_size(disk_name):

    SIZE_XPATH = "./disk/size"
    diskctrl_node = pal_diskctrl_query(disk_name, False).get()
    return int(diskctrl_node.get_node(SIZE_XPATH).get_text())


def is_disk_join_vfsext(disk_name):

    VFS_QUERY_XML = '<volctrl>' + \
                    ' <volume>' + \
                    '  <filesystem>vfsext</filesystem>' + \
                    ' </volume>' + \
                    '</volctrl>'

    PARTITION_XPATH = "volume/partitionlist/partition"

    match = re.match("^[a-zA-Z]+([0-9]+)$", disk_name)
    if not match:
        return False

    entry = request.Entry(ReqTypes.QUERY, VFS_QUERY_XML)
    result_entry = _request_entry(pal_info.PAL_VOLCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    if result_entry.get_result() != Results.OK:
        raise RequestError

    part_node_list = result_entry.get().get_node_list(PARTITION_XPATH)
    for part_node in part_node_list:

        try:
            part_name = part_node.get_text()
        except:
            continue

        if part_name.startswith(disk_name):
            return True
    return False


def delete_disk_from_system(disk_name):

    DISK_DELETE_XML_TMPL = \
        '<diskctrl><disk><name>%s</name></disk></diskctrl>'

    entry = request.Entry(ReqTypes.DELETE,
                          DISK_DELETE_XML_TMPL % disk_name)
    result_entry = _request_entry(pal_info.PAL_DISKCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    if result_entry.get_result() != Results.OK:
        raise RequestError

    return


def handle_error_disk(disk_name, status=FAIL_STATUS, add_fail=False):
    """dispatch raid-fail event.
       part_name must be specified partition name.
    """

    CATEGORY_RAID = "raid"
    FAIL_ACTION = "fail"
    STATUS_XPATH = "./status"
    ADD_FAIL_ATTR = "addfail"

    RAID_FAIL_EVENT_BODY_XML_TMPL = \
        '<raid><name>%s</name><status>%s</status></raid>'

    try:
        body_node = xmldata.XmlConfNode(
                        RAID_FAIL_EVENT_BODY_XML_TMPL % (disk_name, status))
        body_node.set_bool_attr(ADD_FAIL_ATTR, add_fail)
        dispatch_event(CATEGORY_RAID, FAIL_ACTION, body_node)
    except:
        raise

    return


def is_partition_exist_in_raid(raid_name, part_name):

    CHK_DISK_EXIST_REQ_XML = \
        """<raidctrl>
            <raid><name>%s</name></raid>
            <partition><name>%s</name></partition>
           </raidctrl>"""

    entry = request.Entry(ReqTypes.CHK_EXIST,
                          CHK_DISK_EXIST_REQ_XML % (raid_name, part_name))
    result_entry = _request_entry(pal_info.PAL_RAIDCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    if Results.OK == result_entry.get_result():
        return True

    if Results.DEVICE_NOT_FOUND == result_entry.get_result():
        return False

    raise RequestError


def is_disk_exist_in_raid(disk_name):

    CHK_DISK_EXIST_REQ_XML = \
        """<raidctrl>
            <disk><name>%s</name></disk>
           </raidctrl>"""

    entry = request.Entry(ReqTypes.CHK_EXIST,
                          CHK_DISK_EXIST_REQ_XML % disk_name)
    result_entry = _request_entry(pal_info.PAL_RAIDCTRL_PATH, entry,
                                  dry_run=None, debug=None)

    if Results.OK == result_entry.get_result():
        return True

    if Results.DEVICE_NOT_FOUND == result_entry.get_result():
        return False

    raise RequestError


def disk_name2external_vfs_element_name(disk_name):

    for element, info in get_vfsext_dev_info().iteritems():

        if not info["is_external"]:
            continue

        if element.startswith(disk_name):
            return element

    raise exception.NotFoundError


def disk_name2vfs_element_name(disk_name, vfsext_mode=None):

    for element, info in get_vfsext_dev_info(
        vfsext_mode=vfsext_mode).iteritems():

        if element.startswith(disk_name):
            return element

    raise exception.NotFoundError


def get_support_external_vfsext_element_list(is_get_as_disk_name=False):
    """get vfsext element list that supports the port mode of 'vfsext'"""

    element_list = []
    for element, info in get_vfsext_dev_info().iteritems():

        if not info["is_external"]:
            continue

        if is_get_as_disk_name:
            element_list.append(_partition2disk(element))
        else:
            element_list.append(element)

    return  element_list


def get_current_vfs_element_list():

    PART_XPATH = "./volume/partitionlist/partition"

    volctrl_node = pal_volctrl_query_with_vfsext()

    return sorted([node.get_text() \
                      for node in volctrl_node.get_node_list(PART_XPATH)])


def get_vfs_element_list_on_config(vol_name=config.INTERNAL_VOL):
    """設定ファイル上の必要なVFSEXTエレメント"""

    RAID_LEVEL_XPATH = \
        'raidlist/raid/name[text()="%s"]/../level' % vol_name

    raidctrl_config = \
        load_config(config.RAIDCTRL_CONF_FILE, dry_run=None, debug=None)
    raid_mode = raidctrl_config.get_node(RAID_LEVEL_XPATH).get_text()

    if not config.RAID_MODE_INFO[raid_mode]["is_ext_raid"]:
        return []

    element_list = []
    for element, info in config.EXT_RAID_DEV_INFO.iteritems():
        if not info["is_external"]:
            element_list.append(element)

    element_list += get_external_vfsext_port_mode_element_list()
    return element_list


def get_current_port_mode(iface, port_num):

    MODE_XPATH = \
        "./%(iface)slist/%(iface)s[@port=%(port_num)s]" % \
            {"iface": iface, "port_num": port_num} + \
                "/modelist/mode[@enable='true']"

    portconf_node = load_config(
        config.PORTS_CONF_FILE, dry_run=None, debug=None)
    return portconf_node.get_node(MODE_XPATH).get_text()


def get_external_vfsext_port_mode_element_list():

    VFSEXT_PORT_MODE = "vfsext"

    element_list = []
    for element, info in get_vfsext_dev_info().iteritems():

        if not info["is_external"]:
            continue

        disk_name = _partition2disk(element)

        # 設定ファイル上でポートがVFSEXTか
        iface_name = config.DEV_TO_PORT_IF[disk_name]
        port_num = config.DEV_TO_PORT_NO[disk_name]

        try:
            port_mode = get_current_port_mode(iface_name, port_num)
        except:
            continue

        if VFSEXT_PORT_MODE == port_mode:
            element_list.append(element)

    return element_list


def get_vfsext_mirrset_no_by_disk_name(disk_name, vfsext_mode=None):

    for element, info in get_vfsext_dev_info(
        vfsext_mode=vfsext_mode).iteritems():

        if element.startswith(disk_name):
            return info["mirrset"][0], info["mirrset"][1]

    raise exception.NotFoundError


def get_vfsext_pair_element_name_by_mirrset_no(major, minor, vfsext_mode=None):
    """Return element name of pair name of specified
       major - minor number element"""

    if minor != 1 and minor != 2:
        raise exception.NotFoundError
    pair_minor = 1 if minor == 2 else 2

    for element, info in get_vfsext_dev_info(vfsext_mode).iteritems():
        try:
            if info["mirrset"] == (major, pair_minor):
                return element
        except:
            continue

    raise exception.NotFoundError


def get_vfsext_element_name_by_mirrset_no(major, minor):

    for element, info in get_vfsext_dev_info().iteritems():
        try:
            if info["mirrset"] == (major, minor):
                return element
        except:
            continue

    raise exception.NotFoundError


def _get_idema_sector_size(size_gb):  # GB

    return 97696368 + (1953504 * (size_gb - 50))


def get_external_vfsext_max_end_part_sector(disk_name):

    disk_size = get_disk_size(disk_name)
    end_sector = disk_size - GPT_PART_START - 1 \
                     - config.LANDISK_INFO_SECTOR_SIZE
    return (end_sector & ~(end_sector & 7)) - 1


def get_external_vfsext_end_part_sector(disk_name):

    GB_100_UNIT = 100 * 1000 * 1000 * 1000
    AJUST_SECTOR_SIZE = 256 * 1024 * 1024 / 512  # 256MB

    disk_size = get_disk_size(disk_name)
    gb = 100 * int(round(disk_size * 512.0 / GB_100_UNIT))

    if gb > 0:
        idema_size = _get_idema_sector_size(gb)
        vfs_disk_size = idema_size - AJUST_SECTOR_SIZE
    else:
        vfs_disk_size = disk_size

    # 必要な領域を引く
    end_sector = vfs_disk_size - GPT_PART_START - 1 \
                     - config.LANDISK_INFO_SECTOR_SIZE

    # 8 sector境界に合わせる
    return (end_sector & ~(end_sector & 7)) - 1


def is_enough_disk_size_vfsext(dst_disk_name, is_use_flag=False, vfsext_mode=None):

    GPT_PART_START = 40

    VFS_QUERY_XML = '<volctrl>' + \
                    ' <volume>' + \
                    '  <filesystem>vfsext</filesystem>' + \
                    ' </volume>' + \
                    '</volctrl>'
    MOUNTED_STATUS = "mounted"

    def get_min_size_vfsext_disk_size():

        DEFAULT_MIN_SIZE = 0

        STATUS_XPATH = "volume/status"
        PARTITION_XPATH = "volume/partitionlist/partition"

        result_entry = _send_request(pal_info.PAL_VOLCTRL_PATH,
                                     ReqTypes.QUERY,
                                     VFS_QUERY_XML)
        if result_entry.get_result() != Results.OK:
            raise RequestError

        if MOUNTED_STATUS != result_entry.get().get_value(STATUS_XPATH):
            return DEFAULT_MIN_SIZE

        part_node_list = result_entry.get().get_node_list(PARTITION_XPATH)
        min_disk_size = None
        for part_node in part_node_list:
            part_name = part_node.get_text()

            disk_size = get_disk_size(_partition2disk(part_name))

            if min_disk_size is None:
                min_disk_size = disk_size

            elif min_disk_size > disk_size:
                min_disk_size = disk_size

        if min_disk_size is None:
            return DEFAULT_MIN_SIZE

        return min_disk_size

    def get_mirror_set_partner_disk_name(dst_disk_name):

        (major, minor) = get_vfsext_mirrset_no_by_disk_name(dst_disk_name, vfsext_mode)
        element = get_vfsext_pair_element_name_by_mirrset_no(major, minor, vfsext_mode)
        return _partition2disk(element)

    is_external = False
    for element, info in config.EXT_RAID_DEV_INFO.iteritems():
        if element.startswith(dst_disk_name):
            is_external = info["is_external"]
            break

    is_cmp_with_pair_disk = False
    try:
        pair_disk_name = get_mirror_set_partner_disk_name(dst_disk_name)

        if is_use_flag:
            vfs_element_list = get_be_set_vfsexe_element_flag_list(
                               vfsext_mode=vfsext_mode)
            for vfs_element in vfs_element_list:
                if vfs_element.startswith(pair_disk_name):
                    is_cmp_with_pair_disk = True
        else:
            if is_disk_join_vfsext(pair_disk_name):
                is_cmp_with_pair_disk = True
    except:
        # dst_disk_name may be USB and its pair may be not supported.
        pass

    # ミラーセットのペアがvfsに組み込まれている場合は
    # そっちと比較
    # 存在しない場合は最小のディスクと比較
    if is_cmp_with_pair_disk:
        if is_external:
            src_disk_size = \
                get_external_vfsext_end_part_sector(pair_disk_name) + 1 + \
                    GPT_PART_START + config.LANDISK_INFO_SECTOR_SIZE
        else:
            src_disk_size = get_disk_size(pair_disk_name)
    else:
        if is_external:
            return True
        else:
            src_disk_size = get_min_size_vfsext_disk_size()

    dst_disk_size = get_disk_size(dst_disk_name)
    if src_disk_size > dst_disk_size:
        return False

    return True


def get_cur_vfsext_element_info(element_name):

    info_table = {TIMESTAMP_NAME: "",
                  UUID_NAME: "",
                  MIRR_SET_NAME: ""}

    VFSEXT_ELEMENT_INFO_REQ_XML_TMPL = \
        """<volctrl>
            <option operation="vfsextelementinfo"/>
            <volume><name type="partition">%s</name></volume>
           </volctrl>"""

    VFSEXT_ELEMENT_INFO_REQ_XML_TMPL % element_name
    result_entry = \
        _send_request(pal_info.PAL_VOLCTRL_PATH,
                      ReqTypes.CONTROL,
                      VFSEXT_ELEMENT_INFO_REQ_XML_TMPL % element_name)

    result_code = result_entry.get_result()

    if Results.FILE_NOT_FOUND == result_code:
        raise FileNotFound

    if Results.OK != result_code:
        raise RequestError

    vol_node = result_entry.get().get_node(VOLUME_NAME)
    for tag_name in info_table.keys():
        info_table[tag_name] = vol_node.get_value(tag_name, "")

    return info_table


def get_vfsext_uuid_match_element_list(vfs_element_list):

    UNKNOWN_VALUE = -1
    info_table = {}
    for element in vfs_element_list:
        try:
            info = get_cur_vfsext_element_info(element)
            info_table[element] = {TIMESTAMP_NAME: info[TIMESTAMP_NAME],
                                   UUID_NAME: info[UUID_NAME],
                                   MIRR_SET_NAME: info[MIRR_SET_NAME], }
        except:
            info_table[element] = {TIMESTAMP_NAME: UNKNOWN_VALUE,
                                   UUID_NAME: UNKNOWN_VALUE,
                                   MIRR_SET_NAME: UNKNOWN_VALUE, }

    uuid_match_table = {}
    for element, info in info_table.iteritems():
        uuid = info[UUID_NAME]
        if uuid not in uuid_match_table.keys():
            uuid_match_table[uuid] = [element]
        else:
            uuid_match_table[uuid].append(element)

    max_len = 0
    max_match_uuid = None
    for uuid, match_list in uuid_match_table.iteritems():

        if UNKNOWN_VALUE == uuid:
            continue

        if max_len < len(match_list):
            max_len = len(match_list)
            max_match_uuid = uuid

    if max_match_uuid is None:
        raise exception.NotFoundError

    if 0 == len(uuid_match_table[max_match_uuid]):
        raise exception.NotFoundError

    un_match_element_list = \
        [el for el in vfs_element_list \
             if el not in uuid_match_table[max_match_uuid]]

    return (uuid_match_table[max_match_uuid], un_match_element_list)


def get_vfsext_dev_info(vfsext_mode=None, vol_name=config.INTERNAL_VOL):

    RAID_LEVEL_XPATH = \
        'raidlist/raid/name[text()="%s"]/../level' % vol_name

    vfsext_dev_info = {}

    if vfsext_mode is None:
        raidctrl_config = load_config(
            config.RAIDCTRL_CONF_FILE, dry_run=None, debug=None)
        vfsext_mode = raidctrl_config.get_node(RAID_LEVEL_XPATH).get_text()

    if not config.RAID_MODE_INFO[vfsext_mode]["is_ext_raid"]:
        raise exception.NotFoundError

    array_num = config.RAID_MODE_INFO[vfsext_mode]["array_num"]
    dev_name = config.VOL_INFO[vol_name]["dev"]
    element_list = config.EXT_RAID_TO_ELEMENT[array_num][dev_name]
    for element in element_list:
        vfsext_dev_info[element] = config.EXT_RAID_DEV_INFO[element]

    return vfsext_dev_info


def save_cuurent_array_num(vol_name=config.INTERNAL_VOL):

    RAID_LEVEL_XPATH = \
        "raidlist/raid/name[text()='%s']/../level" % vol_name

    raidctrl_conf_node = load_config(
        config.RAIDCTRL_CONF_FILE, dry_run=None, debug=None)
    raid_mode = raidctrl_conf_node.get_value(RAID_LEVEL_XPATH)

    file_path = config.CUR_ARRAY_NUM_TMPL % vol_name

    dir_path = os.path.dirname(file_path)
    if not os.path.isdir(dir_path):
        os.makedirs(dir_path)

    with open(file_path, "w") as fh:
        fh.write(str(config.RAID_MODE_INFO[raid_mode]["array_num"]))

    return


def get_current_raid_mode(vol_name=config.INTERNAL_VOL):

    RAID_LEVEL_XPATH = \
        "raidlist/raid/name[text()='%s']/../level" % vol_name

    raidctrl_conf_node = load_config(
        config.RAIDCTRL_CONF_FILE, dry_run=None, debug=None)
    return raidctrl_conf_node.get_value(RAID_LEVEL_XPATH)


def get_cuurent_array_num(vol_name=config.INTERNAL_VOL):

    raid_mode = get_current_raid_mode(vol_name)
    return config.RAID_MODE_INFO[raid_mode]["array_num"]


def get_max_array_num():

    dev_num = 0
    for dev in config.SATA_HDD_DEV_LIST:
        if is_device_exist(dev):
            dev_num += 1
        else:
            break

    array_num_list = config.VOL_INFO[config.INTERNAL_VOL]["mode_list"].keys()

    max_array_num = 0
    for array_num in array_num_list:
        if array_num <= dev_num:
            if max_array_num < array_num:
                max_array_num = array_num

    return max_array_num


def get_raid_element_info(raid_mode=None):

    ext_raid_to_elemnet = copy.deepcopy(config.EXT_RAID_TO_ELEMENT)

    element_info = {}
    if raid_mode is None:
        raid_mode = get_current_raid_mode()
        array_num = get_cuurent_array_num()
    else:
        array_num = config.RAID_MODE_INFO[raid_mode]["array_num"]

    if config.RAID_MODE_INFO[raid_mode]["is_ext_raid"]:
        element_info = ext_raid_to_elemnet[array_num]
        for raid_name in config.SYSTEM_RAID_DEV_LIST:
            if raid_name not in config.RAID_TO_ELEMENT[array_num]:
                continue
            element_info[raid_name] = \
                config.RAID_TO_ELEMENT[array_num][raid_name]
        return element_info

    return config.RAID_TO_ELEMENT[array_num]


def get_ideal_sata_hdd_list():

    array_num = get_cuurent_array_num()
    element_list = config.HDD_DEV_LIST_IN_VOL[array_num][config.INTERNAL_VOL]
    return [el for el in element_list if el.startswith("satad") ]


def get_enable_sata_port_list():

    return [config.DEV_TO_PORT_NO[dev] for dev in get_ideal_sata_hdd_list()]


def _is_volume_encrypted(vol_name=config.INTERNAL_VOL):

    ENCRIPT_XPATH = \
        "./volumelist/volume[name='%s']/encrypt" % vol_name

    volctrl_conf_node = load_config(
        config.VOLCTRL_CONF_FILE, dry_run=None, debug=None)

    encrypt_node = volctrl_conf_node.get_node(ENCRIPT_XPATH)
    if encrypt_node.is_enabled():
        return True
    return False


def _vfs_enc_stop_partition(vfs_enc_name):

    CTRL_ENC_TMPL = \
        '<volctrl>' + \
         '<option operation="encstop" />' + \
         '<volume>' + \
          '<name type="crypt">%s</name>' + \
         '</volume>' + \
        '</volctrl>'

    if not is_device_exist(vfs_enc_name):
        return

    result_entry = _send_request(pal_info.PAL_VOLCTRL_PATH,
                                 ReqTypes.CONTROL,
                                 CTRL_ENC_TMPL % vfs_enc_name)

    if Results.OK != result_entry.get_result():
        raise RequestError
    return


def remove_vfs_partition(part_name):

    VFS_REMOVE_BASE_REQ_XML = \
        '<volctrl>' + \
        ' <option vfsremove="true"/>' + \
        ' <volume>' + \
        '  <filesystem encrypt="%(is_enc)s"/>' + \
        '  <partition><name>%(part)s</name></partition>' + \
        ' </volume>' + \
        '</volctrl>'

    UNMOUNT_XML_STR = \
        '<volctrl><volume><dst>%s</dst></volume></volctrl>'

    # vfs専用remount実行
    is_enc = _is_volume_encrypted()

    req_xml = VFS_REMOVE_BASE_REQ_XML % {"is_enc": str(is_enc).lower(),
                                         "part": part_name}

    result_entry = _send_request(pal_info.PAL_VOLCTRL_PATH,
                                 ReqTypes.MOUNT,
                                 req_xml)

    if Results.OK != result_entry.get_result():
        raise RequestError

    # 対象パーティションを普通にunmount
    vfsext_dev_info = get_vfsext_dev_info()
    mnt_point = vfsext_dev_info[part_name]["mnt_point"]
    req_xml = UNMOUNT_XML_STR % (mnt_point)
    _send_request(pal_info.PAL_VOLCTRL_PATH,
                  ReqTypes.UNMOUNT,
                  req_xml)

    if is_enc:
        enc_part_name = config.DEV_TO_ENC_DEV[part_name]
        _vfs_enc_stop_partition(enc_part_name)

    return


def is_uuid_same_to_cur_vfsext_element(tgt_disk_name):

    # 確認元
    try:
        tgt_element_name = disk_name2vfs_element_name(tgt_disk_name)
        tgt_element_info = get_cur_vfsext_element_info(tgt_element_name)

        # 確認先
        cur_element_name = get_current_vfs_element_list()[0]
        cur_element_info = get_cur_vfsext_element_info(cur_element_name)
        return tgt_element_info[UUID_NAME] == cur_element_info[UUID_NAME]
    except:
        return False


def get_raid_min_num(raid_level):

    array_num = None
    if raid_level in config.RAID_MODE_INFO:
        array_num = config.RAID_MODE_INFO[raid_level]["array_num"]

    raid_min_num_tbl = \
        {"raid6": array_num - 2 if array_num > 2 else 2,
         "raid5": array_num - 1 if array_num > 1 else 1,
         "raid1": 1,
         "raid0": array_num,
         "vfsext": 1,
         "single": 1}

    real_mode = config.RAID_MODE_TO_REAL_RAID_MODE[raid_level]
    return raid_min_num_tbl[real_mode]


def is_have_system_partition(disk_name):

    DISK_PART_LABEL_LIST = ("efisystem", "rootfs", "swap", "disklabel")

    PART_NAME_XPATH_TMPL = \
        "./disk/partitionlist/partition/name[text()='%s']"

    try:
        diskctrl_node = pal_diskctrl_query(disk_name).get()
    except:
        return False

    for part_label in DISK_PART_LABEL_LIST:
        if _is_exist_node(diskctrl_node,
                          PART_NAME_XPATH_TMPL % part_label):
            return True
    return False


def set_vfsexe_element_flag(disk_name):

    set_disk_info(disk_name, config.VFS_ELEMENT, "1")


def clear_vfsexe_element_flag(disk_name):

    set_disk_info(disk_name, config.VFS_ELEMENT, "0")


def is_be_set_vfsexe_element_flag(disk_name):

    try:
        if "1" == get_disk_info(disk_name,
                                config.VFS_ELEMENT):
            return True
    except:
        pass

    return False


def get_be_set_vfsexe_element_flag_list(vfsext_mode=None):

    vfs_element_list = []
    for element in get_vfsext_dev_info(vfsext_mode=vfsext_mode).keys():
        disk_name = _partition2disk(element)

        if is_be_set_vfsexe_element_flag(disk_name):
            vfs_element_list.append(element)

    return vfs_element_list


def is_need_vfs_autorebuild(disk_name):
    """Check need vfs autorebuild flag"""

    try:
        if "1" == get_disk_info(disk_name,
                                config.NEED_VFS_AUTO_REBUILD):
            return True
    except:
        pass

    return False


def get_need_vfs_auto_rebuild_element_list():

    vfs_element_list = []
    for element in get_vfsext_dev_info().keys():
        disk_name = _partition2disk(element)

        if is_need_vfs_autorebuild(disk_name):
            vfs_element_list.append(element)

    return vfs_element_list


def get_sysinfo(is_version_full=True, dry_run=None, debug=None):
    """Return the tuple of product series, model and firmware version.
       If "is_version_full" is Flase, aditional string is removed from version.
    """

    series = ""
    model = ""
    version = ""
    try:
        response_entry = _request_entry(pal_info.PAL_SYSINFO_PATH,
                            request.Entry(ReqTypes.QUERY), dry_run, debug)

        if response_entry.get_result() != Results.OK:
            return ("", "", "")

        series = response_entry.get().get_value(SERIES_PATH)
        model = response_entry.get().get_value(MODEL_PATH)
        version = response_entry.get().get_value(FWVERSION_PATH)

        if not is_version_full:
            match = re.match(FWVERSION_RE, version)
            if match != None:
                version = match.group(1)

    except:
        pass

    return (series, model, version)


# FIXME: 暫定実装
def encode_share_id(share_name, length=32):
    """Encode share ID from share name."""

    import hashlib

    md5 = hashlib.md5()
    md5.update(share_name)
    hex32 = md5.hexdigest()
    share_id = (("%%0%dd" % length) % int(hex32, 16))[:length]
    return share_id


def check_root_node_name(entry, root_node_name):
    """Check root node name of entry."""

    try:
        root_node = entry.get()
        if root_node.get_node_name() != root_node_name:
            raise exception.InvalidArgError("Invalid root node name.")
    except:
        raise


def check_node_existence(entry, xpath):
    """Check node existence in entry."""

    try:
        entry.get().get_node(xpath)
    except:
        raise exception.InvalidArgError("node \"" + xpath + "\" not found.")


def check_node_value_existence(entry, xpath):
    """Check node value existence in entry."""

    try:
        value = entry.get().get_value(xpath)
        if not value:
            raise exception.InvalidArgError(
                "value \"" + xpath + "\" is empty.")
    except:
        raise


def spindown(disk_name):
    """Spin down disk."""

    SPINDOWN_REQ_XML_TMPL = \
       '<diskctrl>' + \
        '<option operation="spindown"/>' + \
          '<disk><name>%s</name></disk>' + \
       '</diskctrl>'

    entry = request.Entry(ReqTypes.CONTROL, SPINDOWN_REQ_XML_TMPL % disk_name)
    response_entry = request.Request(pal_info.PAL_DISKCTRL_PATH
                                        ).request(entry).get_entry()
    if response_entry.get_result() != Results.OK:
        raise RequestError


def spindown_unuse_disk(vol_name=config.INTERNAL_VOL):

    cur_raid_level = get_current_raid_mode()

    cur_array_num = \
        config.RAID_MODE_INFO[cur_raid_level]["array_num"]
    cur_disk_list = \
        config.HDD_DEV_LIST_IN_VOL[cur_array_num][config.INTERNAL_VOL]

    for disk_name in config.SATA_HDD_DEV_LIST:
        if disk_name not in cur_disk_list:
            spindown(disk_name)


def __raiddevname_to_elementnames(raiddevname, default=[]):
    elementnames = default
    try:
        elementnames = \
            list(config.RAID_TO_ELEMENT[get_cuurent_array_num()][raiddevname])
    except:
        pass
    return elementnames


def __raiddevname_to_elementinfo(raiddevname):
    elementinfo = {}

    raidctrl_node = xmldata.XmlNode("""<raidctrl><raid><name type="raid"/></raid><partitionlist/></raidctrl>""")
    partitionlist_node = raidctrl_node.get_node("partitionlist")
    for elementname in __raiddevname_to_elementnames(raiddevname):
        partition_node = xmldata.XmlNode("<partition><name/></partition>")
        partition_node.get_node("name").set_text(elementname)
        partitionlist_node.append_child(partition_node)
    raidctrl_node.get_node("raid/name").set_text(raiddevname)

    entry = _send_request(pal_info.PAL_RAIDCTRL_PATH,
                          ReqTypes.GET_ELEMENT_INFO,
                          raidctrl_node)

    for partition_node in entry.get().get_node_list("partitionlist/partition"):
        partition_name = partition_node.get_value("name")
        if partition_node.is_set_to_true("element"):
            elementinfo[partition_name] = {
                "uuid": partition_node.get_value("uuid", None),
                # "updatetime": ,
                # "events": ,
                "join": partition_node.is_set_to_true("join"),
            }
        else:
            elementinfo[partition_name] = {
                "join": partition_node.is_set_to_true("join"),
            }

    return elementinfo


def __partitionname_to_devicename(partitionname):
    """
    partitionname_to_devicename("satad1p2")
    """
    return re.sub(r"p[0-9]+$", u"", partitionname)  # FIXME


empty_object = object()
def check_mirrordisk(device_name):

    uuid1, uuid2 = None, None
    model1, model2 = None, None
    for element_partition_name, element_info in __raiddevname_to_elementinfo(config.SYSTEM_RAID_DEV_LIST[0]).iteritems():
        if element_partition_name.startswith("satad") and element_info.get("join"):
            if not uuid1:
                uuid1 = element_info.get("uuid", None)
                if uuid1:
                    # 未接続ではdisklabelを取得できないためuuidの取得をもって接続状態に代える
                    model1 = get_disk_label_info(__partitionname_to_devicename(element_partition_name)).get("model", object())
        elif element_partition_name.startswith("usbd"):
            if element_partition_name.startswith(device_name):
                uuid2 = element_info.get("uuid", None)
                if uuid2:
                    # 未接続ではdisklabelを取得できないためuuidの取得をもって接続状態に代える
                    model2 = get_disk_label_info(device_name).get("model", object())

    is_mirrordisk, is_recoverable, is_match_system = False, False, False
    if uuid1 and uuid2:
        # ミラーディスク  FIXME
        is_mirrordisk = True
        if uuid1 == uuid2:
            if model1 == model2:
                # 本システムのミラーディスク
                is_match_system = True
                is_recoverable = True
        else:
            # 本システムのミラーディスクでない
            if model1 == model2:
                # 復帰可能なミラーディスク
                is_recoverable = True
    else:
        # ミラーディスクでない
        pass

    return (is_mirrordisk, is_recoverable, is_match_system)


if __name__ == "__main__":
    pass
