#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling MailNotice event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import fal_info
import fal_msginfo as msginfo
import libevh.util as util
from iobase.xmldata import XmlNode
from iobase.exception import (NotFoundError, InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)
from fal_msginfo import get_category_str

NOTICE_CONF_FILE = "config-notice.xml"
MESSAGES_XML_FILE = "messages.xml"

EVENTINFO_NAME = "eventinfo"
NAME_NAME = "name"
MAILCONF_NAME = "mailconf"
CODE_NAME = "code"
MESSAGEINFO_NAME = "messageinfo"

CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
DETAIL_ATTR = "detail"
LEVEL_ATTR = "level"
DATE_ATTR = "date"

EVENTLOG_CATEGORY = "eventlog"
ADD_ACTION = "add"

TARGET_LOG_XPATH = "eventinfo/eventlog/log"
MESSAGESS_XPATH = "eventinfo/eventlog/messages"


NOTIFY_CATEGORY_TABLE = \
{"system": ("System Initialization", "System", "Firmware", "Package", "Tools"),
 "backup": ("Backup", "QuickCopy", "Replication", "Data Copy", ),
 "log": ("Log", ),
 "temperature": ("Temperature/FAN", ),
 "disk": ("Disk", "Active Repair", "RAID",
          "Lock Key", "Volume", "RAID Volume", ),
 "volume": ("Capacity", ),
 "quota": ("Quota", ),
 "ups":  ("UPS", ),
}

DEFAULT_NOTIFY_CATEGORY = "system"

MESSAGE_XML_BASE = \
    "<message>" + \
    " <header>" + \
    "  <subject/>" + \
    " </header>" + \
    " <body/>" + \
    "</message>"

LEVEL_STR_MAP = \
    {"0": u"情報",
     "1": u"警告",
     "2": u"エラー"}


class EvhMailNotice(evhcmd.EvhCmd):
    """MailNotice Event Hnadler command implementation class."""

    class __SendError(Exception):
        "Send Error"
        pass

    class __InternalError(Exception):
        "Internal Error"
        pass

    def __validate_target_event(self, entry):

        if EVENTLOG_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if ADD_ACTION != entry.get().get_node(EVENTINFO_NAME).\
                             get_attr_value(ACTION_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        return

    def __send_request(self, target_module, type, child_node=""):
        """Send request to pal module(single entry)"""

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __send_mail(self, mailsend_node):

        result_entry = self.__send_request(fal_info.FAL_MAILSEND_PATH,
                                           ReqTypes.SEND,
                                           mailsend_node)

        if result_entry.get_result() != Results.OK:
            raise self.__SendError

        return

    def __save_mail_log(self, notify_name, mail_addr):

        LOG_MESSAGE_TABLE = \
            {"system": msginfo.Msg.MSG_MAIL_SEND_SYSTEM_EVENT,
             "backup": msginfo.Msg.MSG_MAIL_SEND_BACKUP,
             "temperature": msginfo.Msg.MSG_MAIL_SEND_TEMP_FAN,
             "disk": msginfo.Msg.MSG_MAIL_SEND_DISKERROR,
             "volume": msginfo.Msg.MSG_MAIL_SEND_CAPACITY,
             "quota": msginfo.Msg.MSG_MAIL_SEND_QUOTA,
             "ups": msginfo.Msg.MSG_MAIL_SEND_UPS}

        if notify_name in LOG_MESSAGE_TABLE:
            util.save_event_log(LOG_MESSAGE_TABLE[notify_name],
                    arg_dict={"mail_address": mail_addr})
        return

    def __save_mailerror_log(self, notify_name, mail_addr):

        LOG_MESSAGE_TABLE = \
            {"system": msginfo.Msg.ERR_MAIL_SEND_SYSTEM_EVENT,
             "backup": msginfo.Msg.ERR_MAIL_SEND_BACKUP,
             "temperature": msginfo.Msg.ERR_MAIL_SEND_TEMP_FAN,
             "disk": msginfo.Msg.ERR_MAIL_SEND_DISKERROR,
             "volume": msginfo.Msg.ERR_MAIL_SEND_CAPACITY,
             "quota": msginfo.Msg.ERR_MAIL_SEND_QUOTA,
             "ups": msginfo.Msg.ERR_MAIL_SEND_UPS}

        if notify_name in LOG_MESSAGE_TABLE:
            util.save_event_log(LOG_MESSAGE_TABLE[notify_name],
                    arg_dict={"mail_address": mail_addr})
        return

    def __load_notice_node(self):
        """Load mailnotice file"""

        return self.send_entry(request.Entry(ReqTypes.LOAD),
                               fal_info.FAL_NOTICE_PATH).get()

    def __get_message_str(self, log_node):

        MESSAGES_REQ_XML = '<messages/>'
        messages_node = XmlNode(MESSAGES_REQ_XML)

        msginfo_node = log_node.get_clone()
        msginfo_node.set_node_name(MESSAGEINFO_NAME)

        messages_node.append_child(msginfo_node)
        res_message_node = self.__send_request(fal_info.FAL_MESSAGES_PATH,
                                               ReqTypes.QUERY,
                                               messages_node).get()
        return res_message_node.get_node("messageinfo/message").get_text()

    def __merge_auth_node(self,
                          detail_mainconf_node,
                          basic_mailconf_node):

        AUTH_XPATH = "./auth"
        TYPE_ATTR = "type"
        USER_XPATH = "./auth[@type='%s']/user"
        PASSWORD_XPATH = "./auth[@type='%s']/password"
        POP_HOST_XPATH = "./auth[@type='pop']/pop3/host"
        POP_PORT_XPATH = "./auth[@type='pop']/pop3/port"

        NONE_AUTH_ATTR_VAL = "no"
        POP_B4_SMTP_AUTH_ATTR_VAL = "pop"
        PLAIN_AUTH_ATTR_VAL = "plain"
        LOGIN_AUTH_ATTR_VAL = "login"
        MD5_AUTH_ATTR_VAL = "crammd5"

        type = \
            detail_mainconf_node.get_node(AUTH_XPATH).get_attr_value(TYPE_ATTR)

        if NONE_AUTH_ATTR_VAL == type:
            return

        if PLAIN_AUTH_ATTR_VAL == type or \
           LOGIN_AUTH_ATTR_VAL == type or \
           MD5_AUTH_ATTR_VAL == type:

            if "" == detail_mainconf_node.get_node(USER_XPATH % type):
                user = basic_mailconf_node.get_value(USER_XPATH % type)
                detail_mainconf_node.get_node(
                    USER_XPATH % type).set_text(user)

            if "" == detail_mainconf_node.get_node(PASSWORD_XPATH % type):
                passwd = basic_mailconf_node.get_value(PASSWORD_XPATH % type)
                detail_mainconf_node.get_node(
                    PASSWORD_XPATH % type).set_text(passwd)

            return

        if POP_B4_SMTP_AUTH_ATTR_VAL == type:

            if "" == detail_mainconf_node.get_node(POP_HOST_XPATH):
                host = basic_mailconf_node.get_value(POP_HOST_XPATH)
                detail_mainconf_node.get_node(POP_HOST_XPATH).set_text(host)

            if "" == detail_mainconf_node.get_node(POP_PORT_XPATH):
                port = basic_mailconf_node.get_value(POP_PORT_XPATH)
                detail_mainconf_node.get_node(POP_PORT_XPATH).set_text(port)

            return

        return

    def __merge_mail_conf_node(self,
                               detail_mainconf_node,
                               basic_mailconf_node):

        HOST_XPATH = "./host[@type='smtp']"
        PORT_XPATH = "./port[@type='smtp']"
        FROMADDR_XPATH = "./fromaddr"
        ENCODE_XPATH = "./encode"

        param_xpaths = \
            (HOST_XPATH, PORT_XPATH, FROMADDR_XPATH, ENCODE_XPATH)

        for xpath in param_xpaths:

            if "" == detail_mainconf_node.get_value(xpath):
                value = basic_mailconf_node.get_value(xpath)
                detail_mainconf_node.get_node(xpath).set_text(value)

        self.__merge_auth_node(basic_mailconf_node, detail_mainconf_node)

    def __get_enable_mailconf_node_list(self, notify_name):

#        MAILBASIC_XPATH = "mailbasic/mailconf"
        MAILBASIC_XPATH = "mailbasic"
        ENABLE_MAILCONF_XPATH_TMPL = \
            "./mailnotice/mailconflist" + \
            "/mailconf[@enable='true']" + \
            "/notifylist" + \
            "/notify[@enable='true' and @name='%s']" + \
            "/../.."

        notice_node = self.__load_notice_node()

        try:
            mailbasic_node = notice_node.get_node(MAILBASIC_XPATH)
        except:
            mailbasic_node = None

        mailconf_node_list = \
            notice_node.get_node_list(ENABLE_MAILCONF_XPATH_TMPL % notify_name)

        for mailconf_node in mailconf_node_list:
            if mailconf_node.is_set_to_true(DETAIL_ATTR):
                self.__merge_mail_conf_node(mailconf_node,
                    mailbasic_node.get_node(MAILCONF_NAME))
                continue

            if mailbasic_node is None:
                continue

            mailconf_node.update(mailbasic_node.get_node(MAILCONF_NAME),
                                  override=True)

        return mailconf_node_list

    def __category2notify_name(self, category_name):

        for notify_name, category_name_list in \
            NOTIFY_CATEGORY_TABLE.iteritems():

            if category_name in category_name_list:
                return notify_name

        return DEFAULT_NOTIFY_CATEGORY

    def __get_category_info(self, msg_code, category_code):
        """Return notify name, mail title and body"""

        TARGET_MESSAGE_XPATH = \
            'messageinfolist/messageinfo/code[text()="%s"]/../message'
        TARGET_CODE_XPATH = 'categorylist/category/code[text()="%s"]/..'

        category_name = \
            self.messages_node.get_node(TARGET_CODE_XPATH % category_code).\
                get_value(NAME_NAME)

        notify_name = self.__category2notify_name(category_name)

        if not notify_name:
            raise self.__InternalError

        body_str = \
            self.messages_node.get_node(
                TARGET_MESSAGE_XPATH % msg_code).get_text()

        # ex. エラー: UPS
        subject = self.level_str + u": " + self.category_str

        return (notify_name, subject, body_str)

    def __get_log_date(self, log_node):

        return log_node.get_attr_value(DATE_ATTR)

    def __send_notify_mail(self, log_node):

        SUBJECT_XMLPATH = "./header/subject"
        BODY_NAME = "body"
        MAILSEND_NAME = "mailsend"

        msg_code = log_node.get_value(CODE_NAME)
        msg_category_code = log_node.get_attr_value(CATEGORY_ATTR)

        self.category_str = \
            unicode(get_category_str(msg_category_code), "utf-8")

        (notify_name, subject, body_str) = \
            self.__get_category_info(msg_code, msg_category_code)

        mailconf_node_list = self.__get_enable_mailconf_node_list(notify_name)

        if len(mailconf_node_list) == 0:
            return

        body_str = self.__get_log_date(log_node)
        body_str += "  "
        body_str += self.__get_message_str(log_node)

        message_node = XmlNode(MESSAGE_XML_BASE)
        message_node.get_node(SUBJECT_XMLPATH).set_text(subject)
        message_node.get_node(BODY_NAME).set_text(body_str)

        for mailconf_node in mailconf_node_list:
            mail_addr = \
                mailconf_node.get_node("./toaddrlist/toaddr").get_text()
            mailsend_node = XmlNode(MAILSEND_NAME)
            mailsend_node.append_child(mailconf_node)
            mailsend_node.append_child(message_node)

            try:
                self.__send_mail(mailsend_node)
                self.__save_mail_log(notify_name, mail_addr)
            except self.__SendError:
                self.__save_mailerror_log(notify_name, mail_addr)
                continue
        return

    def __is_notify(self, code):

        try:
            messageinfo_node = self.messages_node.get_node(
                    'messageinfolist/messageinfo[code="%s"]' % code)
            return messageinfo_node.get_node('noticeinfo/mail').is_enabled()
        except:
            return False

    def __get_log_level_str(self, log_node):

        try:
            level = log_node.get_attr_value(LEVEL_ATTR)
        except:
            level = "0"

        return LEVEL_STR_MAP[level]

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            self.__validate_target_event(entry)

            try:
                log_node = entry.get().get_node(TARGET_LOG_XPATH)
            except NotFoundError:
                entry.set_result(Results.OK)
                return entry

            self.level_str = self.__get_log_level_str(log_node)

            self.messages_node = entry.get().get_node(MESSAGESS_XPATH)

            code = log_node.get_value('code', None)
            if not code:
                raise InvalidArgError
            if not self.__is_notify(code):
                entry.set_result(Results.OK)
                return entry

            self.__send_notify_mail(log_node)
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhMailNotice(option=evhcmd.EvhCmdOption()).handle_request()
