#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling Announce event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import pal_info
import fal_info
from fal_msginfo import Msg
from iobase.exception import (NotFoundError, InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf import SUDO_CMD

ANNOUNCE_FILE = "announce.xml"

EVENTINFO_NAME = "eventinfo"

CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

EVENTLOG_CATEGORY = "eventlog"
ADD_ACTION = "add"

TARGET_LOG_XPATH = "eventinfo/eventlog/log"
MESSAGESS_XPATH = "eventinfo/eventlog/messages"


class EvhAnnounce(evhcmd.EvhCmd):
    """Announce Event Hnadler command implementation class."""

    def __validate_target_event(self, entry):

        if EVENTLOG_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if ADD_ACTION != entry.get().get_node(EVENTINFO_NAME).\
                             get_attr_value(ACTION_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        return

    def __load_announcelist_node(self):
        """Load announce file"""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                         pal_info.PAL_CONFFILE_PATH,
                                         ANNOUNCE_FILE).get()

    def __load_snmptrap_config(self):
        entry = self.send_entry(
                request.Entry(ReqTypes.LOAD),
                fal_info.FAL_SNMP_PATH)
        trap_node = entry.get().get_node('trap')
        enable = trap_node.is_enabled()
        host = trap_node.get_node('host').get_text()
        port = trap_node.get_node('port').get_text()
        return enable, host, port

    def _code(self, constname, default=None):
        try:
            return getattr(Msg, constname)
        except:
            return default

    def _logging(self, constname, arglist={}):
        try:
            code = self._code(constname, default=constname)
            xml = """<eventlog>
                <log>
                    <code>%(code)s</code>
                    <arglist>%(arglist)s</arglist>
                </log>
                </eventlog>""" % {
                    'code': code,
                    'arglist': ''.join(['<arg name="%(k)s">%(v)s</arg>' % {
                        'k':k, 'v':v} for k, v in arglist.iteritems()]),
                }
            response = request.Request(
                    fal_info.FAL_EVENTLOG_PATH,
                    filter_cmd=SUDO_CMD).request(request.Entry('add', xml))
            entry = response.get_entrylist().get_entry()
            if int(entry.get_result()):
                return False
            else:
                return True
        except:
            return False

    def __is_enable_smtptrap(self, code):

        try:
            messageinfo_node = self.messages_node.get_node(
                    'messageinfolist/messageinfo[code="%s"]' % code)
            return messageinfo_node.get_node(
                'noticeinfo/snmptrap').is_enabled()
        except:
            return False

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            self.__validate_target_event(entry)

            try:
                log_node = entry.get().get_node(TARGET_LOG_XPATH)
            except NotFoundError:
                entry.set_result(Results.OK)
                return entry

            self.messages_node = entry.get().get_node(MESSAGESS_XPATH)

            code = log_node.get_value('code', None)
            if not code:
                raise InvalidArgError
            if not self.__is_enable_smtptrap(code):
                entry.set_result(Results.OK)
                return entry

            try:
                enable, host, port = self.__load_snmptrap_config()
            except NotFoundError:
                # イレギュラー状態だがエラーとすべきか?
                entry.set_result(Results.OK)
                return entry

            if not enable:
                entry.set_result(Results.OK)
                return entry

            code_node = log_node.get_node('code')
            try:
                arglist_node = log_node.get_node('arglist')
            except:
                arglist_node = None

            if code_node.get_text() == self._code('ERR_SNMP_TRAP_SEND'):
                entry.set_result(Results.OK)
                return entry

            trap_entry = request.Entry(ReqTypes.SEND, """<snmp>
                <trap>
                    <host>%(host)s</host>
                    <port>%(port)s</port>
                </trap>
            </snmp>""" % {
                'host': host,
                'port': port,
            })
            trap_entry.get().append_child('messageinfo')
            trap_entry.get().get_node('messageinfo').append_child(code_node)
            if arglist_node != None: 
                trap_entry.get().get_node('messageinfo').append_child(
                                                                arglist_node)
            result = self.send_entry(
                    trap_entry,
                    pal_info.PAL_SNMP_PATH).get_result()
            if result == Results.OK:
                self._logging('MSG_SNMP_TRAP_SEND')
            else:
                self._logging('ERR_SNMP_TRAP_SEND')
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhAnnounce(option=evhcmd.EvhCmdOption()).handle_request()
