#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling iosupdate event"""

import libevh.evhcmd as evhcmd
from ionas.request import Entry
from iobase.exception import NotFoundError
from fal_info import (FAL_SERVICECTRL_PATH, FAL_FIRMWARE_PATH,
            FAL_PACKAGES_PATH, Results, ReqTypes)


SERVICE_ENTRY = "<servicectrl><servicelist><service><name>iosupdate" + \
            "</name></service></servicelist></servicectrl>"
SERVICE_PATH = "servicelist/service"
  
ITEM_LIST = ["updatenotice", "autoupdate"]


class IOSUpdate(evhcmd.EvhCmd):
    """IOSUpdate Event Hnadler command implementation class."""

    def __start_or_stop_service(self, type):
        entry = Entry(type, SERVICE_ENTRY)
        response = self.send_entry(entry, FAL_SERVICECTRL_PATH)
        return response

    def __enable_service(self, is_enable):
        entry = Entry(ReqTypes.APPLY, SERVICE_ENTRY)

        entry.get().get_node(SERVICE_PATH).enable(is_enable)
        response = self.send_entry(entry, FAL_SERVICECTRL_PATH)
        return response

    def __iosupdate_is_enabled(self):
        for cmd in [FAL_FIRMWARE_PATH, FAL_PACKAGES_PATH]:
            conf = self.send_entry(Entry(ReqTypes.LOAD), cmd)
            for name in ITEM_LIST:
                try:
                    if conf.get().get_node(name).is_enabled():
                        return True
                except NotFoundError:
                    pass
        return False

    def __event_handler(self, entry):
        """Handle "event" request."""

        if self.__iosupdate_is_enabled():
            service_result = self.__enable_service(True)
            if service_result.get_result() == Results.OK:
                self.__start_or_stop_service(ReqTypes.STOP)
                service_result = self.__start_or_stop_service(ReqTypes.START)
        else:
            service_result = self.__enable_service(False)
            if service_result.get_result() == Results.OK:
                service_result = self.__start_or_stop_service(ReqTypes.STOP)

        if service_result.get_result() != Results.OK:
            entry.set_result(service_result.get_result())
            return entry

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    IOSUpdate(option=evhcmd.EvhCmdOption()).handle_request()
