#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling LCM event"""

from __future__ import with_statement
import os
import re
from datetime import datetime
import ionas.request as request
import libevh.evhcmd as evhcmd
from fal_info import (FAL_LCM_PATH, FAL_SYSINFO_PATH, FAL_VOLCTRL_PATH,
                      FAL_NETWORK_PATH, FAL_ANNOUNCE_PATH, FAL_MESSAGES_PATH)
from iobase.exception import (InvalidArgError, NotFoundError)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf import *
import libevh.util as util
from fal_msginfo import (Msg, get_query_entry)

# LCM EVENT
PUSH_SELECT_LCM_EVENT = "pushselect"
PUSH_ENTER_LCM_EVENT = "pushenter"
CLEAR_LCM_EVENT = "clear"
BOOT_LCM_EVENT = "boot"
SHUTDOWN_LCM_EVENT = "shutdown"
REBOOT_LCM_EVENT = "reboot"
ADD_LCM_EVENT = "add"
APPLY_EVENT = "apply"
RESTART_EVENT = "restart"
EVENTLOG_CATEGORY = "eventlog"
CATEGORY_ATTR = "category"

# XML tag name
EVENT_NAME = "event"
EVENTINFO_NAME = "eventinfo"
MODEL_NAME = "model"
FW_NAME = "fwversion"
HOST_NAME = "hostdesc/name"

# XML path
LCM_DISP_NAME_XPATH = "./eventinfo/lcm/lcmevent/name"

# Attribute
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

### LCM STATE IDENTIFYER ###
# Basic
INIT_STATE = "init"
NOTIFY_STATE = "notify"
LAN1_IPADDR_STATE = "lan1_ipaddr"
LAN2_IPADDR_STATE = "lan2_ipaddr"
MODEL_STATE = "model"
FREE_SPACE_STATE = "freespace"
DATETIME_STATE = "datetime"

# お知らせ表示モード
ANNOUNCE_STATE = "announce"

# 表示される順番
LCM_BASIC_INFO_LIST = \
    [NOTIFY_STATE,
     MODEL_STATE,
     LAN1_IPADDR_STATE,
     LAN2_IPADDR_STATE,
     FREE_SPACE_STATE,
     DATETIME_STATE]

# Temp file name
CUR_POPUP_FILE = "/var/tmp/lcm/popup"
POPUP_EXIST = "1"
POPUP_NOT_EXIST = "0"

CUR_BASIC_INFO_FILE = "/var/tmp/lcm/basic_info"
CUR_ANNOUNCE_INDEX_FILE = "/var/tmp/lcm/announce_index"

LCM_STR_XML = \
    '<lcm>' + \
    ' <stringlist>' + \
    '  <string line="1">%s</string>' + \
    '  <string line="2">%s</string>' + \
    ' </stringlist>' + \
    '</lcm>'

QUERY_VOLCTRL_XML = \
    '<volctrl><volume>' + \
    ' <name type="volume">%s</name>' + \
    '</volume></volctrl>'

QUERY_NETWORK_XML = '<network/>'

NOTIFY_DEL_XML = '<announce><index>%s</index></announce>'
ERR_LEVEL = "2"


class LcmEventHnadler(evhcmd.EvhCmd):
    """FAL LCM Event Hnadler command implementation class."""

    class __PassHandling(Exception):
        "Pass Handling"
        pass

    class __RequestError(Exception):
        "Request Error"
        pass

    #
    # Utility method
    #

    def __force_make_file_path(self, file_path, value=""):
        """make specified directory."""

        dir_path = os.path.dirname(file_path)
        if not os.path.isdir(dir_path):
            os.makedirs(dir_path)
        with open(file_path, "w") as fh:
            fh.write(value)
        return

    def __check_correct_root_node(self, entry):
        """check whether the [node_name] is equal to the name of[node].
        """

        node_name = entry.get().get_node_name()
        if node_name != EVENT_NAME:
            raise InvalidArgError(node_name)

    def __send_request(self, type, target_module, child_node=""):

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __is_booting_state(self):
        """Check system state booting."""

        if util.BOOTING_STATE == util.get_system_state():
            return True

        return False

    def __is_shutdown_state(self):
        """Check system state shutdown."""

        if util.SHUTDOWN_STATE == util.get_system_state():
            return True

        return False

    def __get_lcm_disp_strs(self, node=None):
        try:
            if node:
                code = node.get_value("code")
            msg_info = self.__send_msg_request(code, for_lcm=True)
            l1 = msg_info.get_value("messageinfo/message/line[@no='1']")
            l2 = msg_info.get_value("messageinfo/message/line[@no='2']")
        except:
            return ("", "")
        return (l1, l2)

    def __get_lcm_message(self, code=""):
        if not code:
            return ""
        msg_info = self.__send_msg_request(code, for_lcm=False)
        return msg_info.get_value("messageinfo/message")

    def __send_msg_request(self, code, for_lcm):
        entry = get_query_entry(code, for_lcm=for_lcm)
        response = request.Request(FAL_MESSAGES_PATH).request(entry)
        result = response.get_entrylist().get_entry()
        return result.get()

    def __disp_strs_in_lcm(self, line1, line2):
        request.Request(FAL_LCM_PATH).request(request.Entry(
                ReqTypes.DISPLAY,
                LCM_STR_XML % (line1, line2)))
        return

    def __get_announce_node_list(self):

        response = request.Request(FAL_ANNOUNCE_PATH).request(
                request.Entry(ReqTypes.QUERY, "<announce />"))
        entry = response.get_entrylist().get_entry()
        return entry.get().get_node_list("announce")

    #
    # Product information getter
    #

    def __get_notify(self):

        l1 = self.__get_lcm_message(code=Msg.MSG_LCM_NOTICE_STR)
        l2 = self.__get_lcm_message(code=Msg.MSG_LCM_NO_NOTICE)
        try:
            response = request.Request(FAL_ANNOUNCE_PATH).request(
            request.Entry(ReqTypes.QUERY, "<announce />"))
            entry = response.get_entrylist().get_entry()
            num = len(entry.get().get_node_list("announce"))
            if 0 < num:
                l2_pre = self.__get_lcm_message(code=Msg.MSG_LCM_NOTICE_EXIST)
                l2 = l2_pre + "(%s)" % str(num)
        except:
            pass

        return (l1, l2)

    def __get_ipaddr(self, lan_name):
        lan_map = {"lan1": "L1:", "lan2": "L2:"}

        IPADDR_XPATH = "./netiflist/netif[@name='%s']" + \
                       "/protocollist/protocol[@family='inet4']" + \
                       "/addrlist/addr"
        MAC_XPATH = "./netiflist/netif[@name='%s']/mac"

        try:
            result_entry = self.__send_request(ReqTypes.QUERY,
                                           FAL_NETWORK_PATH,
                                           QUERY_NETWORK_XML)
        except:
            result_entry = None

        try:
            ipaddr = result_entry.get().get_node(
                        IPADDR_XPATH % lan_name).get_text()
        except:
            ipaddr = ""
        if not ipaddr:
            ipaddr = self.__get_lcm_message(code=Msg.MSG_LCM_NO_LINK)

        try:
            mac = result_entry.get().get_node(MAC_XPATH % lan_name).get_text()
        except:
            mac = ""
        if not mac:
            mac = self.__get_lcm_message(code=Msg.ERR_LCM_MAC_UNKNOWN)

        return (lan_map[lan_name] + ipaddr, mac)

    def __get_ipaddr_lan1(self):

        return self.__get_ipaddr("lan1")

    def __get_ipaddr_lan2(self):

        return self.__get_ipaddr("lan2")

    def __get_model_name(self):

        try:
            result_entry = \
                self.__send_request(ReqTypes.QUERY, FAL_SYSINFO_PATH)

            if Results.OK != result_entry.get_result():
                raise self.__RequestError

            host_name = result_entry.get().get_node(HOST_NAME).get_text()
            model_name = result_entry.get().get_node(MODEL_NAME).get_text()
            fw_ver = result_entry.get().get_node(FW_NAME).get_text()
            fw_str = re.sub("\(.+\)", "", fw_ver)

            return (model_name + " " + fw_str, host_name)
        except:
            l1 = self.__get_lcm_message(code=Msg.ERR_LCM_MODEL_UNKNOWN)
            l2 = self.__get_lcm_message(code=Msg.ERR_LCM_HOST_UNKNOWN)
            return (l1, l2)

    def __get_datetime(self):
        dt = datetime.now()
        date_str = dt.strftime("%Y/%m/%d")
        time_str = dt.strftime("%H:%M")
        return (date_str, time_str)

    def __get_freespace(self):

        l1 = self.__get_lcm_message(code=Msg.MSG_LCM_FREE_SPACE)
        try:
            query_xml = QUERY_VOLCTRL_XML % STOR_IF_TO_VOL["internal"][1]

            result_entry = self.__send_request(ReqTypes.QUERY,
                                               FAL_VOLCTRL_PATH,
                                               query_xml)
            if Results.OK != result_entry.get_result():
                raise self.__RequestError

            USE_CAPA_XPATH = "volume/capacity/use"
            TOT_CAPA_XPATH = "volume/capacity/total"
            use_capa = result_entry.get().get_node(USE_CAPA_XPATH).get_text()
            tot_capa = result_entry.get().get_node(TOT_CAPA_XPATH).get_text()

            freespace = int(tot_capa) - int(use_capa)
            freespace = freespace / (1000 * 1000 * 1000)
            totalspace = int(tot_capa) / (1000 * 1000 * 1000)
            l2 = str(freespace) + "/" + str(totalspace) + " GB"
        except:
            l2 = self.__get_lcm_message(code=Msg.MSG_LCM_UNKNOWN_STR)

        return (l1, l2)

    def __get_lcm_sub_string(self, lcm_state):

        info_getter_handlers = \
            {NOTIFY_STATE: self.__get_notify,
             MODEL_STATE: self.__get_model_name,
             LAN1_IPADDR_STATE: self.__get_ipaddr_lan1,
             LAN2_IPADDR_STATE: self.__get_ipaddr_lan2,
             FREE_SPACE_STATE: self.__get_freespace,
             DATETIME_STATE: self.__get_datetime}

        l1, l2 = info_getter_handlers[lcm_state]()
        return (l1, l2)

    #
    # LCM control method
    #

    def __display_basic_info(self, lcm_disp_name):
        """Display specified basic info"""

        l1, l2 = self.__get_lcm_sub_string(lcm_disp_name)
        self.__disp_strs_in_lcm(l1, l2)

        if lcm_disp_name in LCM_BASIC_INFO_LIST:
            self.__save_basic_info_disp_name(lcm_disp_name)
        return

    def __display_next_basic_info(self, lcm_state):
        """Display next basic info of specified basic info"""

        cur_idx = LCM_BASIC_INFO_LIST.index(lcm_state)
        self.__display_basic_info(LCM_BASIC_INFO_LIST[cur_idx + 1])

        # POPUP表示解除
        self.__set_cur_popup_msg_code()
        return

    def __display_init_basic_info(self):

        self.__display_basic_info(NOTIFY_STATE)
        self.__save_next_announce_index(-1)

        # POPUP表示解除
        self.__set_cur_popup_msg_code()
        return

    def __delete_notify(self, idx):
        entry = request.Entry(ReqTypes.DELETE, NOTIFY_DEL_XML % idx)
        request.Request(FAL_ANNOUNCE_PATH).request(entry)
        return

    def __display_notify_detail(self):
        """Display notify detail info"""

        def _get_announce(index):
            """Return current index and announce node"""

            response = request.Request(FAL_ANNOUNCE_PATH).request(
                    request.Entry(ReqTypes.QUERY, "<announce />"))
            entry = response.get_entrylist().get_entry()

            announce_node_list = entry.get().get_node_list("announce")
            if len(announce_node_list) <= 0:
                return -1, None
            if len(announce_node_list) < index + 1:
                return 0, entry.get().get_node_list("announce")[0]

            return index, entry.get().get_node_list("announce")[index]
            """<announce category="CATEGORY" level="LEVEL">
                <code>CODE</code>
                <arglist><arg name="NAME">ARG</arg></arglist>
            </announce>"""

        idx = self.__get_current_announce_index()
        if -1 == idx:
            idx = 0

        cur_idx, announce_node = _get_announce(idx)
        if cur_idx >= 0:
            l1, l2 = self.__get_lcm_disp_strs(node=announce_node)
            if l1 + l2:
                self.__disp_strs_in_lcm(l1, l2)
                if announce_node.get_attr_value("level") == ERR_LEVEL:
                    self.__save_next_announce_index(cur_idx + 1)
                else:
                    self.__delete_notify(cur_idx)
                    self.__save_next_announce_index(cur_idx)
            # POPUP表示解除
            self.__set_cur_popup_msg_code()
        else:
            self.__display_init_basic_info()
        self.__update_status_led()

        return

    def __display_announce(self, announce_index=-1):

        # from evh_announce add

        def _get_announce(announce_index):
            response = request.Request(FAL_ANNOUNCE_PATH).request(
                    request.Entry(ReqTypes.QUERY, "<announce />"))
            entry = response.get_entrylist().get_entry()

            if 0 == len(entry.get().get_node_list("announce")):
                raise NotFoundError

            return entry.get().get_node_list("announce")[announce_index]
            """<announce category="CATEGORY" level="LEVEL">
                <code>CODE</code>
                <arglist><arg name="NAME">ARG</arg></arglist>
            </announce>"""

        announce_node = _get_announce(announce_index)
        l1, l2 = self.__get_lcm_disp_strs(node=announce_node)
        if l1 + l2:
            self.__disp_strs_in_lcm(l1, l2)

        self.__set_cur_popup_msg_code(
                announce_node.get_node("code").get_text())
        return

    #
    # LCM temporary info file operater
    #

    def __write_value(self, file_path, value):

        with open(file_path, "w") as fh:
            fh.write(value)
        return

    def __read_value(self, file_path):

        with open(file_path, "r") as fh:
            return fh.read()

    def __save_basic_info_disp_name(self, disp_name):

        self.__write_value(CUR_BASIC_INFO_FILE, disp_name)
        return

    def __save_next_announce_index(self, next_index):

        self.__write_value(CUR_ANNOUNCE_INDEX_FILE, str(next_index))
        return

    def __set_cur_popup_msg_code(self, msg_code=""):

        self.__write_value(CUR_POPUP_FILE, msg_code)
        return

    def __get_cur_popup_msg_code(self):

        return self.__read_value(CUR_POPUP_FILE)

    def __get_current_basic_info_disp_name(self):

        return self.__read_value(CUR_BASIC_INFO_FILE)

    def __is_during_popup(self):

        try:
            if POPUP_EXIST == self.__read_value(CUR_POPUP_FILE):
                return True
        except:
            pass

        return False

    def __get_current_announce_index(self):

        try:
            idx = int(self.__read_value(CUR_ANNOUNCE_INDEX_FILE).rstrip())
        except:
            return -1

        return idx

    def __get_currnt_disp_name(self):

        if -1 != self.__get_current_announce_index():
            return ANNOUNCE_STATE

        basic_disp_name = self.__get_current_basic_info_disp_name()
        return basic_disp_name

    #
    # XML parser
    #

    def __get_lcm_event_type(self, entry):
        """Get LCM event name."""

        try:
            event_info_node = entry.get().get_node(EVENTINFO_NAME)
            action = event_info_node.get_attr_value(ACTION_ATTR)
            is_previous = event_info_node.is_set_to_true(PREVIOUS_ATTR)
        except:
            return ""

        if RESTART_EVENT == action and is_previous:
            raise self.__PassHandling

        if APPLY_EVENT == action and is_previous:
            raise self.__PassHandling

        if BOOT_LCM_EVENT == action and is_previous:
            raise self.__PassHandling

        if SHUTDOWN_LCM_EVENT == action and not is_previous:
            raise self.__PassHandling

        if REBOOT_LCM_EVENT == action and not is_previous:
            raise self.__PassHandling

        if PUSH_SELECT_LCM_EVENT == action and is_previous:
            raise self.__PassHandling

        if CLEAR_LCM_EVENT == action and is_previous:
            raise self.__PassHandling

        if ADD_LCM_EVENT == action:
            if is_previous:
                raise self.__PassHandling

            if EVENTLOG_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
                raise InvalidArgError

        return action

    #
    # LCM event handlers
    #

    def __bootup_handler(self):

        try:
            self.__display_announce()
        except NotFoundError:
            # お知らせがないなら起動完了メッセージ表示
            pass

        return

    def __push_enter_handler(self):
        """PUSH_ENTER_LCM_EVENT hander"""

        lcm_disp_name = self.__get_currnt_disp_name()

        state_handlers = {INIT_STATE: self.__display_init_basic_info,
                          NOTIFY_STATE: self.__display_notify_detail,
                          LAN1_IPADDR_STATE: self.__display_notify_detail,
                          LAN2_IPADDR_STATE: self.__display_notify_detail,
                          MODEL_STATE: self.__display_notify_detail,
                          FREE_SPACE_STATE: self.__display_notify_detail,
                          DATETIME_STATE: self.__display_notify_detail,
                          ANNOUNCE_STATE: self.__display_init_basic_info}

        state_handlers[lcm_disp_name]()
        return

    def __push_select_handler(self):
        """PUSH_SELECT_LCM_EVENT hander"""

        lcm_disp_name = self.__get_currnt_disp_name()
        display_next = lambda: self.__display_next_basic_info(lcm_disp_name)
        state_handlers = {INIT_STATE: self.__display_init_basic_info,
                          NOTIFY_STATE: display_next,
                          MODEL_STATE: display_next,
                          LAN1_IPADDR_STATE: display_next,
                          LAN2_IPADDR_STATE: display_next,
                          FREE_SPACE_STATE: display_next,
                          DATETIME_STATE: self.__display_init_basic_info,
                          ANNOUNCE_STATE: self.__display_notify_detail}

        state_handlers[lcm_disp_name]()
        return

    def __apply_or_restart_handler(self):
        lcm_disp_name = self.__get_currnt_disp_name()
        if lcm_disp_name in [
            LAN1_IPADDR_STATE, LAN2_IPADDR_STATE, MODEL_STATE]:
            self.__display_basic_info(lcm_disp_name)

        return

    def __clear_handler(self):
        """CLEAR_LCM_EVENT hander"""

        if not util.is_boot_complete():
            return

        if self.__is_shutdown_state():
            return

        self.__update_status_led()
        if not self.__get_cur_popup_msg_code():
            lcm_disp_name = self.__get_currnt_disp_name()
            if lcm_disp_name in [INIT_STATE, NOTIFY_STATE, ANNOUNCE_STATE]:
                self.__display_init_basic_info()
            return

        announce_node_list = self.__get_announce_node_list()
        for announce_node in announce_node_list:
            code = announce_node.get_value("code")
            if code == self.__get_cur_popup_msg_code():
                return

        # お知らせファイルに現在表示している
        self.__display_init_basic_info()
        return

    def __update_status_led(self):
        util.fal_led_update()
        return

    def __add_handler(self, entry):
        """ADD_LCM_EVENT handler"""

        MESSAGESS_XPATH = "eventinfo/eventlog/messages"
        TARGET_LOG_XPATH = "eventinfo/eventlog/log"
        messages_node = entry.get().get_node(MESSAGESS_XPATH)

        def _is_lcm_enable(code):

            try:
                messageinfo_node = messages_node.get_node(
                    'messageinfolist/messageinfo[code="%s"]' % code)
                return messageinfo_node.get_node('noticeinfo/lcm').\
                    is_enabled()
            except:
                return False

        try:
            log_node = entry.get().get_node(TARGET_LOG_XPATH)
        except NotFoundError:
            entry.set_result(Results.OK)
            return entry

        code = log_node.get_value('code', None)
        if not code:
            raise InvalidArgError

        if not _is_lcm_enable(code) or not util.is_boot_complete():
            entry.set_result(Results.OK)
            return entry

        l1, l2 = self.__get_lcm_disp_strs(node=log_node)
        if l1 + l2:
            self.__disp_strs_in_lcm(l1, l2)

        # 現在表示しているメッセージコードを保存
        self.__set_cur_popup_msg_code(code)
        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        if self.__is_booting_state():
            entry.set_result(Results.OK)
            return entry

        try:
            try:
                lcm_evt = self.__get_lcm_event_type(entry)
            except self.__PassHandling:
                entry.set_result(Results.OK)
                return entry

            # LCM-EVENT vs corresponding handler dictionary
            # These handler have no argment

            add_handler = lambda: self.__add_handler(entry)
            self.__lcm_event_handler = \
                {PUSH_SELECT_LCM_EVENT: self.__push_select_handler,
                 PUSH_ENTER_LCM_EVENT: self.__push_enter_handler,
                 ADD_LCM_EVENT: add_handler,
                 CLEAR_LCM_EVENT: self.__clear_handler,
                 BOOT_LCM_EVENT: self.__bootup_handler,
                 APPLY_EVENT: self.__apply_or_restart_handler,
                 RESTART_EVENT: self.__apply_or_restart_handler}

            self.__lcm_event_handler[lcm_evt]()
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

        # status file
        if not os.path.exists(CUR_BASIC_INFO_FILE):
            self.__force_make_file_path(CUR_BASIC_INFO_FILE, INIT_STATE)

        if not os.path.exists(CUR_POPUP_FILE):
            self.__force_make_file_path(CUR_POPUP_FILE, POPUP_NOT_EXIST)

        if not os.path.exists(CUR_ANNOUNCE_INDEX_FILE):
            self.__force_make_file_path(CUR_ANNOUNCE_INDEX_FILE, "-1")

#
# Main
#

if __name__ == "__main__":
    LcmEventHnadler(option=evhcmd.EvhCmdOption()).handle_request()
