#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling RaidFail event"""

from __future__ import with_statement

import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
import fal_msginfo as msginfo
import ionas.device as device
from iobase.exception import (InvalidArgError, NotFoundError)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf.devinfo import (SATA_HDD_DEV_LIST,
                                  USB_HDD_DEV_LIST,
                                  SYSTEM_RAID_DEV_LIST)
from fal_info import FAL_RAIDCTRL_PATH
import iobase.fileut as fileut
import evh_config as config
from evh_config import (INTERNAL_VOL, INTERNAL_DEV, MIRROR_DEV)


EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
OPERATION_ATTR = "operation"
ADD_FAIL_ATTR = "addfail"
RAID_CATEGORY = "raid"
RESYNC_OP = "resync"

TARGET_DEV_NAME_XPATH = "eventinfo/raid/name"
STATUS_NAME_XPATH = "eventinfo/raid/status"
RAID_XPATH = "eventinfo/raid"

FAIL_ACTION = "fail"
DEGRADE_ACTION = "degrade"

REDUDANCY_ATTR = "redudancy"
REBUILD_TARGET_ATTR = "rebuildtarget"

STATUS_TYPE = "status"
FAIL_STATUS = "fail"
CRASH_STATUS = "crash"

DISK_FAIL_STATUS = "fail"
DISK_SMARTERR_STATUS = "smarterror"
DISK_CAPSHORTAGE_STATUS = "capshortage"

MIRRORING = "mirroring"
EXTERNAL_VFSEXT = "vfsext"

SATA_DETECT_FAIL_LOG_CODE_MAP = {
        "satad1": msginfo.Msg.ERR_DISK_SATA1_DETECT_ERROR,
        "satad2": msginfo.Msg.ERR_DISK_SATA2_DETECT_ERROR,
        "satad3": msginfo.Msg.ERR_DISK_SATA3_DETECT_ERROR,
        "satad4": msginfo.Msg.ERR_DISK_SATA4_DETECT_ERROR,
        "satad5": msginfo.Msg.ERR_DISK_SATA5_DETECT_ERROR,
        "satad6": msginfo.Msg.ERR_DISK_SATA6_DETECT_ERROR,
    }

SATA_ADD_FAIL_LOG_CODE_MAP = {
        "satad1": msginfo.Msg.ERR_DISK_SATA1_FAIL,
        "satad2": msginfo.Msg.ERR_DISK_SATA2_FAIL,
        "satad3": msginfo.Msg.ERR_DISK_SATA3_FAIL,
        "satad4": msginfo.Msg.ERR_DISK_SATA4_FAIL,
        "satad5": msginfo.Msg.ERR_DISK_SATA5_FAIL,
        "satad6": msginfo.Msg.ERR_DISK_SATA6_FAIL,
    }

SATA_CAPSHORTAGE_LOG_MAP = {
        "satad1": msginfo.Msg.ERR_DISK_SATA1_CAP_SHORTAGE,
        "satad2": msginfo.Msg.ERR_DISK_SATA2_CAP_SHORTAGE,
        "satad3": msginfo.Msg.ERR_DISK_SATA3_CAP_SHORTAGE,
        "satad4": msginfo.Msg.ERR_DISK_SATA4_CAP_SHORTAGE,
        "satad5": msginfo.Msg.ERR_DISK_SATA5_CAP_SHORTAGE,
        "satad6": msginfo.Msg.ERR_DISK_SATA6_CAP_SHORTAGE,
    }

USB_DETECT_FAIL_LOG_CODE_MAP = {
        "usbd2": {
              MIRRORING: msginfo.Msg.ERR_DISK_USB2_DETECT_ERROR,
            },
        "usbd3": {
              MIRRORING: msginfo.Msg.ERR_DISK_USB3_DETECT_ERROR,
              EXTERNAL_VFSEXT: msginfo.Msg.ERR_DISK_USB3_DETECT_ERROR,
            },
        "usbd4": {
              EXTERNAL_VFSEXT: msginfo.Msg.ERR_DISK_USB4_DETECT_ERROR,
            }
    }

USB_ADD_FAIL_LOG_CODE_MAP = {
        "usbd2": {
              MIRRORING: msginfo.Msg.ERR_DISK_USB2_FAIL,
            },
        "usbd3": {
              MIRRORING: msginfo.Msg.ERR_DISK_USB3_FAIL,
              EXTERNAL_VFSEXT: msginfo.Msg.ERR_DISK_USB3_FAIL,
            },
        "usbd4": {
              EXTERNAL_VFSEXT: msginfo.Msg.ERR_DISK_USB4_FAIL,
            }
    }

USB_CAPSHORTAGE_LOG_MAP = {
        "usbd2": {
              MIRRORING: msginfo.Msg.ERR_RAID_MIRROR_CAP_SHORTAGE_USB2,
            },
        "usbd3": {
              MIRRORING: msginfo.Msg.ERR_RAID_MIRROR_CAP_SHORTAGE_USB3,
              EXTERNAL_VFSEXT: msginfo.Msg.ERR_DISK_USB3_CAP_SHORTAGE,
            },
        "usbd4": {
              EXTERNAL_VFSEXT: msginfo.Msg.ERR_DISK_USB4_CAP_SHORTAGE,
            }
    }

SATA_DISK_ERR_LOG_MAP = {
        DISK_FAIL_STATUS: {
             False: SATA_DETECT_FAIL_LOG_CODE_MAP,
             True: SATA_ADD_FAIL_LOG_CODE_MAP,
           },
        DISK_SMARTERR_STATUS: {
             False: SATA_DETECT_FAIL_LOG_CODE_MAP,
             True: SATA_ADD_FAIL_LOG_CODE_MAP,
           },
        DISK_CAPSHORTAGE_STATUS: {
             False: SATA_CAPSHORTAGE_LOG_MAP,
             True: SATA_CAPSHORTAGE_LOG_MAP,
           },
    }

USB_DISK_ERR_LOG_MAP = {
        DISK_FAIL_STATUS: {
             False: USB_DETECT_FAIL_LOG_CODE_MAP,
             True: USB_ADD_FAIL_LOG_CODE_MAP,
           },
        DISK_SMARTERR_STATUS: {
             False: USB_DETECT_FAIL_LOG_CODE_MAP,
             True: USB_ADD_FAIL_LOG_CODE_MAP,
           },
        DISK_CAPSHORTAGE_STATUS: {
             False: USB_CAPSHORTAGE_LOG_MAP,
             True: USB_CAPSHORTAGE_LOG_MAP,
           },
    }


class RaidFail(evhcmd.EvhCmd):
    """Raid Fail Event Hnadler command implementation class."""

    class __InternalError(Exception):
        pass

    class __RemoveFail(Exception):
        pass

    def __get_event_action(self, entry):

        ENABLE_ACTION_LIST = (FAIL_ACTION, DEGRADE_ACTION)

        if RAID_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError("Previous is True !!")

        action = entry.get().get_node(EVENTINFO_NAME).\
            get_attr_value(ACTION_ATTR)

        if action not in ENABLE_ACTION_LIST:
            raise InvalidArgError("Invalid action !!" + action)

        return action

    def __send_request(self, req_type, target_module, child_node=""):

        entry = request.Entry(req_type, child_node)
        return self.send_entry(entry, target_module)

    def __part_name2disk_name(self, part_name):

        for disk_name in SATA_HDD_DEV_LIST + USB_HDD_DEV_LIST:
            if part_name.startswith(disk_name):
                return disk_name

        raise NotFoundError

    def __part_name2raid_name(self, part_name):

        for raid_name, element_list in \
                util.get_raid_element_info().iteritems():

            if part_name in element_list:
                return raid_name

    def __disk_name2main_raid_name(self, disk_name):

        for raid_name, element_list in \
                util.get_raid_element_info().iteritems():

            if raid_name in SYSTEM_RAID_DEV_LIST:
                continue

            for element in element_list:
                if element.startswith(disk_name):
                    return raid_name

        raise NotFoundError

    def __is_current_raid_mode_vfsext(self):

        VFSEXT_FSTYPE = "vfsext"

        raidctrl_node = \
            self.__send_request(ReqTypes.LOAD, FAL_RAIDCTRL_PATH).get()
        level = raidctrl_node.get_value("raidlist/raid/level")
        if VFSEXT_FSTYPE == level:
            return True
        return False

    # RAIDの冗長性チェック
    def __is_crash_if_remove(self, raid_name, disk_name):
        """Check Raid redudancy"""

        try:
            return util.is_crash_if_remove(raid_name, disk_name)
        except:
            return False

    def __is_element_exist_in_raid(self, raid_name, part_name):

        CHK_DISK_EXIST_REQ_XML = \
            """<raidctrl>
                <raid><name>%s</name></raid>
                <partition><name>%s</name></partition>
               </raidctrl>"""

        result_entry = \
            self.__send_request(ReqTypes.CHK_EXIST,
                                FAL_RAIDCTRL_PATH,
                                CHK_DISK_EXIST_REQ_XML % \
                                    (raid_name, part_name))
        if Results.OK == result_entry.get_result():
            return True

        if Results.DEVICE_NOT_FOUND == result_entry.get_result():
            return False

        raise self.__InternalError

    def __is_disk_exist_in_raid(self, disk_name):

        CHK_DISK_EXIST_REQ_XML = \
            """<raidctrl>
                <disk><name>%s</name></disk>
               </raidctrl>"""

        result_entry = \
            self.__send_request(ReqTypes.CHK_EXIST,
                                FAL_RAIDCTRL_PATH,
                                CHK_DISK_EXIST_REQ_XML % disk_name)
        return Results.OK == result_entry.get_result()

    def __is_raid_dirty(self, entry):

        try:
            operation = \
                entry.get().get_node(RAID_XPATH).get_attr_value(OPERATION_ATTR)
        except:
            return False

        if RESYNC_OP == operation:
            return True

        return False

    def __dirty_handling(self, raid_name, disk_name, is_need_remove=False):

        try:
            self.__crash_handling(raid_name, disk_name)
        except:
            self.exception("fail to crash handling")
            pass

        if is_need_remove:
            # 故障ログを残す
            if disk_name in SATA_DETECT_FAIL_LOG_CODE_MAP:
                try:
                    util.save_event_log(
                        SATA_DETECT_FAIL_LOG_CODE_MAP[disk_name])
                except:
                    self.exception("fail to save log (do nothing)")
                    pass
        return

    def __crash_handling(self, raid_name, disk_name):

        if util.is_raid_crash_or_stop(raid_name):
            return

        is_system_crash = False
        is_remove_fail = False

        # INTERNAL_DEVがcrashする場合はまずMIRROR_DEVから削除する。
        # この時MIRROR_DEVに冗長性がない場合はMIRROR_DEVをまずcrashさせる。
        if raid_name == INTERNAL_DEV:
            if self.__is_crash_if_remove(MIRROR_DEV, INTERNAL_DEV):
                try:
                    util.unmount_internal_volume()
                    util.fal_raidctrl_stop(MIRROR_DEV, True)
                    is_system_crash = True
                except:
                    self.exception("fail to stop parent_raid_name")
                    pass
            else:
                try:
                    util.fal_raidctrl_remove_partition(MIRROR_DEV, raid_name)
                except:
                    self.exception("fail to remove from MIRROR_DEV")
                    pass

            try:
                util.fal_raidctrl_stop(raid_name, True)
            except:
                self.exception("fail to stop %s" % raid_name)

            try:
                util.fal_raidctrl_remove(disk_name)
            except:
                is_remove_fail = True
                self.exception("fail to remove %s" % disk_name)

        elif raid_name == MIRROR_DEV:
            try:
                util.unmount_internal_volume()
                util.fal_raidctrl_stop(raid_name, True)
                is_system_crash = True
            except:
                self.exception("fail to stop parent_raid_name")
                pass

        if is_system_crash:
            try:
                util.unmount_all_usb_volume()
            except:
                pass

        try:
            util.fal_volctrl_update(INTERNAL_VOL)
        except:
            self.exception("fail to util.fal_volctrl_update(INTERNAL_VOL)")
            pass

        if is_remove_fail:
            raise self.__RemoveFail

        return

    def __dispatch_update_event(self):

        try:
            util.dispatch_event(util.RAID_CATEGORY, util.UPDATE_ACTION)
        except:
            self.exception("fail to dispatch raid-update")
            pass

        try:
            util.dispatch_event(util.DISK_CATEGORY, util.UPDATE_ACTION)
        except:
            self.exception("fail to dispatch disk-update")
            pass

    def __disk_name2raw_disk_name(self, disk_name):

        return self.dev_mgr.get_kdev_name(
                    self.dev_mgr.get_base_dev_name(disk_name))

    def __save_sata_log(self, disk_name, disk_status, is_add_fail):

        if disk_status in SATA_DISK_ERR_LOG_MAP.keys():
            if disk_name in SATA_DISK_ERR_LOG_MAP[
                                disk_status][is_add_fail].keys():
                try:
                    util.save_event_log(
                        SATA_DISK_ERR_LOG_MAP[
                            disk_status][is_add_fail][disk_name])
                except:
                    self.exception("fail to save log (do nothing)")
        return

    def __save_usb_log(self, disk_name, disk_status, is_add_fail, usb_mode):

        if disk_status in USB_DISK_ERR_LOG_MAP.keys():
            if disk_name in USB_DISK_ERR_LOG_MAP[
                                disk_status][is_add_fail].keys():
                try:
                    util.save_event_log(
                        USB_DISK_ERR_LOG_MAP[
                            disk_status][is_add_fail][disk_name][usb_mode])
                except:
                    self.exception("fail to save log (do nothing)")
        return

    def __external_vfsext_disk_fail_handling(self, disk_name,
                                             disk_status, is_add_fail):

        self.__save_usb_log(disk_name, disk_status,
                            is_add_fail, EXTERNAL_VFSEXT)
        try:
            util.fal_diskctrl_update(disk_name, STATUS_TYPE, disk_status)
        except:
            self.exception("fail to update disk status(fail) - " + \
                           "disk name : %s" % disk_name)
            pass

        try:
            if DISK_CAPSHORTAGE_STATUS != disk_status:
                util.delete_disk_from_system(disk_name)
        except:
            self.exception("fail to delete_disk_from_system(%s)" % disk_name)
            pass

        self.__dispatch_update_event()
        return

    def __action_fail(self, entry):
        """Fail event handler"""

        util.wait_for_raid_operable()

        is_remove_fail = False
        self.debug(str(entry))
        target_name = entry.get().get_value(TARGET_DEV_NAME_XPATH)

        disk_status = entry.get().get_value(STATUS_NAME_XPATH, None)
        if disk_status is None:
            disk_status = FAIL_STATUS

        if target_name.startswith("md"):
            return

        # 非ミラー設定でUSBディスクがfailになるのは意図的に切り離すため
        # fail処理はしない
        #  => 起動時に非ミラー設定なのにUSBのパーティションがmd2, md5に付い
        #     ていた場合は切り離す仕様
        usb_mode = None
        if target_name.startswith("usb"):
            if util.is_external_vfsext_setting(
                   self.__part_name2disk_name(target_name)):
                usb_mode = EXTERNAL_VFSEXT
            elif util.is_mirroring_setting(
                     self.__part_name2disk_name(target_name)):
                usb_mode = MIRRORING

        disk_name = None
        main_raid_name = None
        is_need_remove = False
        is_add_fail = False

        if util.is_disk_name(target_name):
            disk_name = target_name
            if EXTERNAL_VFSEXT != usb_mode:
                main_raid_name = self.__disk_name2main_raid_name(disk_name)
            is_need_remove = True
            is_add_fail = \
                entry.get().get_node(RAID_XPATH).is_set_to_true(ADD_FAIL_ATTR)

        elif util.is_raid_element_name(target_name):
            disk_name = self.__part_name2disk_name(target_name)
            raid_name = self.__part_name2raid_name(target_name)
            if EXTERNAL_VFSEXT != usb_mode:
                main_raid_name = self.__disk_name2main_raid_name(disk_name)

            is_need_remove = True
            if not self.__is_element_exist_in_raid(raid_name, target_name) or \
                   not util.is_device_exist(disk_name):
                is_need_remove = False

        self.info("target_name : %s" % target_name)
        self.info("disk_name : %s" % disk_name)
        self.info("is_need_remove == %s" % str(is_need_remove))

        if EXTERNAL_VFSEXT == usb_mode:
            self.__external_vfsext_disk_fail_handling(disk_name,
                                                      disk_status,
                                                      is_add_fail)
            return

        # HDDがすでに取り外されている場合でもdirtyな状態の場合は
        # RAID崩壊処理が必要
        if self.__is_raid_dirty(entry):
            if disk_name:
                self.__dirty_handling(main_raid_name,
                                      disk_name,
                                      is_need_remove)

            self.__dispatch_update_event()
            return

        if not is_need_remove:
            return

        if self.__is_crash_if_remove(main_raid_name, disk_name) and \
               not util.is_raid_crash_or_stop(main_raid_name):

            self.info("__is_crash_if_remove() : true")

            # 冗長性がなくremoveするとRAID崩壊する状況
            try:
                self.__crash_handling(main_raid_name, disk_name)
            except self.__RemoveFail:
                is_remove_fail = True
            except:
                self.exception("fail to __crash_handling() - " + \
                               "raid name : %s (do nothing)" % main_raid_name)
                pass

        else:
            self.info("__is_crash_if_remove() : false")
            if self.__is_disk_exist_in_raid(disk_name):

                try:
                    util.fal_raidctrl_remove(disk_name)
                except:
                    self.exception("fail to remove %s" % disk_name)
                    is_remove_fail = True

        # 故障ログを残す
        if disk_name.startswith("sata"):
            self.__save_sata_log(disk_name, disk_status, is_add_fail)
        elif disk_name.startswith("usb"):
            if usb_mode:
                self.__save_usb_log(disk_name, disk_status,
                                    is_add_fail, usb_mode)

        try:
            util.fal_diskctrl_update(disk_name, STATUS_TYPE,
                                     disk_status, not is_add_fail)
        except:
            self.exception("fail to update disk status(fail) - " + \
                           "disk name : %s" % disk_name)
            pass

        try:
            if DISK_CAPSHORTAGE_STATUS == disk_status and \
                disk_name.startswith("usbd"):
                pass
            else:
                if not is_remove_fail:
                    util.delete_disk_from_system(disk_name)
        except:
            self.exception("fail to delete_disk_from_system(%s)" % disk_name)
            pass

        self.__dispatch_update_event()
        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        action = self.__get_event_action(entry)

        action_func = \
            {FAIL_ACTION: self.__action_fail}

        try:
            with fileut.FLock(config.LOCK_DIR,
                              config.DISK_EVENT_LOCK_FILE,
                              exclusive=True):

                action_func[action](entry)
        except NotFoundError:
            self.exception("action_func - NotFoundError")
            entry.set_result(Results.DELETE_ERROR)
            return entry
        except:
            self.exception("fail to action_func")
            raise

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)
        self.dev_mgr = device.DevMgr(config.DEV_DIR, config.DEV_LINK_DIR,
                                     config.DEV_FILE_DIR, config.HDD_DEV_LIST)
        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    RaidFail(option=evhcmd.EvhCmdOption()).handle_request()
