#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling "raidactiverepair" event"""

import os
import re
import libevh.evhcmd as evhcmd
import evh_config as config
import libevh.util as util
from iobase.exception import (InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)
from fal_msginfo import Msg
from evh_config import (INTERNAL_DEV, MIRROR_DEV, TARGET_RAID_DEV_LIST)

CATEGORY_RAIDACTIVEREPAIR = "raidactiverepair"
EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
PROGRESS_ACTION = "progress"
STATE_ATTR = "state"
PROGRESS_FILE_PATH = os.path.join(config.VOLUME_STATUS_DIR, "activerepair")

RAID_NAME_XPATH = "eventinfo/progress/name"
PROGRESS_XPATH = "eventinfo/progress"

START_STATE = "start"
PROGRESS_STATE = "progress"
SUCCESS_STATE = "success"
ERROR_STATE = "error"

OTHER_DEV = "dummy_dev"

LOG_CODE_MAP = {
        START_STATE: {
                INTERNAL_DEV: Msg.MSG_ACTIVE_REPAIR_INTERNAL_START,
                MIRROR_DEV: Msg.MSG_ACTIVE_REPAIR_MIRROR_START,
            },
        PROGRESS_STATE: {
                INTERNAL_DEV: Msg.MSG_ACTIVE_REPAIR_INTERNAL_PROGRESS,
                MIRROR_DEV: Msg.MSG_ACTIVE_REPAIR_MIRROR_PROGRESS,
            },
        SUCCESS_STATE: {
                INTERNAL_DEV: Msg.MSG_ACTIVE_REPAIR_INTERNAL_END,
                MIRROR_DEV: Msg.MSG_ACTIVE_REPAIR_MIRROR_END,
            },
        ERROR_STATE: {
                INTERNAL_DEV: Msg.ERR_ACTIVE_REPAIR_INTERNAL_REPAIR,
                MIRROR_DEV: Msg.ERR_ACTIVE_REPAIR_MIRROR_REPAIR,
                OTHER_DEV: Msg.ERR_ACTIVE_REPAIR_SYSTEM_REPAIR,
            },
    }


class ActiverepairEventHnadler(evhcmd.EvhCmd):
    """ACTIVEREPAIR Event Hnadler command implementation class."""

    def __validate_eventinfo(self, entry):
        """Validate a evntinfo in "entry"."""

        eventinfo = entry.get().get_node(EVENTINFO_NAME)
        category = entry.get().get_node(EVENTINFO_NAME).get_attr_value(
                                                            CATEGORY_ATTR)

        if eventinfo.get_attr_value(CATEGORY_ATTR) != \
                                        CATEGORY_RAIDACTIVEREPAIR:

            raise InvalidArgError("Invalid category: " +
                            eventinfo.get_attr_value(CATEGORY_ATTR))

        if eventinfo.is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError("Previous is True!!")

        action = eventinfo. get_attr_value(ACTION_ATTR)

        if action != PROGRESS_ACTION:
            raise InvalidArgError("Invalid action !!" + action)

        return

    def __is_target(self, raid_name, state):
        """Check if "raid_name" deive is target in "state" state."""

        if not state in LOG_CODE_MAP:
            return False

        if not raid_name in LOG_CODE_MAP[state]:
            return False

        return True

    def __handle_state(self, entry, raid_name, state, progress):
        """Raidrebuild progress event handler for each "state" except "*%"."""

        if raid_name in LOG_CODE_MAP[state]:
            if progress:
                util.save_event_log(LOG_CODE_MAP[state][raid_name],
                            {"progress": progress})
            else:
                util.save_event_log(LOG_CODE_MAP[state][raid_name])

        util.save_progress(entry, PROGRESS_FILE_PATH)
        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self.__validate_eventinfo(entry)

        raid_name = entry.get().get_value(RAID_NAME_XPATH)
        if not raid_name in TARGET_RAID_DEV_LIST:
            raid_name = OTHER_DEV

        state = \
            entry.get().get_node(PROGRESS_XPATH).get_attr_value(STATE_ATTR)

        match = re.match("([0-9]+)%", state)
        progress = ""
        if match != None:
            state = PROGRESS_STATE
            progress = match.group(1)

        if not self.__is_target(raid_name, state):
            # Ignore event
            entry.set_result(Results.OK)
            return entry

        self.__handle_state(entry, raid_name, state, progress),
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    ActiverepairEventHnadler(option=evhcmd.EvhCmdOption()).handle_request()
