#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling UsbdPlug event"""

from __future__ import with_statement

import ionas.request as request
import ionas.device as device
import libevh.evhcmd as evhcmd
from fal_info import (FAL_RAIDCTRL_PATH, FAL_VOLCTRL_PATH)
import fal_info as info
import fal_msginfo as msginfo
from iobase.exception import (InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf.devinfo import (SYSTEM_RAID_DEV_LIST, SATA_HDD_DEV_LIST)
from iobase.exception import (NotFoundError)
import libevh.util as util
import iobase.fileut as fileut
import evh_config as config
from evh_config import (INTERNAL_VOL, INTERNAL_DEV, MIRROR_DEV)

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
REDUDANCY_ATTR = "redudancy"
REBUILD_TARGET_ATTR = "rebuildtarget"

SATAD_CATEGORY = "satad"
PLUG_ACTION = "plug"
UNPLUG_ACTION = "unplug"

DEV_NAME_XPATH = "eventinfo/device/name"

ADD_DISK_TO_RAID_REQ_XML = \
    '<raidctrl>' + \
    ' <raid>' + \
    '  <name type="disk">%s</name>' + \
    ' </raid>' + \
    '</raidctrl>'

CHECK_ADDDISK_REQ_XML = \
    '<raidctrl>' + \
    ' <option operation="adddisk"/>' + \
    ' <raid>' + \
    '  <name type="volume">sataraid1</name>' + \
    '  <level>%(level)s</level>' + \
    ' </raid>' + \
    '<disk><name>%(disk)s</name></disk>' + \
    '</raidctrl>'

CHECK_CHANGE_MODE_REQ_XML = \
    '<raidctrl>' + \
    ' <option operation="changemode"/>' + \
    ' <raid>' + \
    '  <name type="volume">%(volume)s</name>' + \
    '  <level>%(level)s</level>' + \
    ' </raid>' + \
    '</raidctrl>'

EXTEND_REQ_XML_TMPL = \
    '<raidctrl>' + \
    ' <raid>' + \
    '  <name type="volume">%(volume)s</name>' + \
    '  <level>%(level)s</level>' + \
    ' </raid>' + \
    '</raidctrl>'

STATUS_TYPE = "status"
NORMAL_STATUS = "normal"
CAPSHORTAGE_STATUS = "capshortage"
FAIL_STATUS = "fail"
SMARTERROR_STATUS = "smarterror"
UNPLUG_STATUS = "unplug"
CRASH_STATUS = "crash"

FAIL_LOG_CODE_MAP = {
        "satad1": msginfo.Msg.ERR_DISK_SATA1_FAIL,
        "satad2": msginfo.Msg.ERR_DISK_SATA2_FAIL,
        "satad3": msginfo.Msg.ERR_DISK_SATA3_FAIL,
        "satad4": msginfo.Msg.ERR_DISK_SATA4_FAIL,
        "satad5": msginfo.Msg.ERR_DISK_SATA5_FAIL,
        "satad6": msginfo.Msg.ERR_DISK_SATA6_FAIL,
    }

CAP_SHORTAGE_LOG_CODE_MAP = {
        "satad1": msginfo.Msg.ERR_DISK_SATA1_CAP_SHORTAGE,
        "satad2": msginfo.Msg.ERR_DISK_SATA2_CAP_SHORTAGE,
        "satad3": msginfo.Msg.ERR_DISK_SATA3_CAP_SHORTAGE,
        "satad4": msginfo.Msg.ERR_DISK_SATA4_CAP_SHORTAGE,
        "satad5": msginfo.Msg.ERR_DISK_SATA5_CAP_SHORTAGE,
        "satad6": msginfo.Msg.ERR_DISK_SATA6_CAP_SHORTAGE,
    }

EXTEND_VOLUME_LOG_CODE_MAP = {
        "vfsext_6": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_6", None),
        "vfsext_5": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_5", None),
        "vfsext_4": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_4", None),
        "vfsext_3": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_3", None),
        "vfsext_2": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_2", None),
        "vfsext_1": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_1", None),
    }

EXTEND_VOLUME_ERR_LOG_CODE_MAP = {
        "vfsext_6": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_6", None),
        "vfsext_5": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_5", None),
        "vfsext_4": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_4", None),
        "vfsext_3": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_3", None),
        "vfsext_2": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_2", None),
        "vfsext_1": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_1", None),
    }


class EvhSatadPlugUnplug(evhcmd.EvhCmd):
    """SatadPlugUnplug Event Hnadler command implementation class."""

    class __InternalError(Exception):
        pass

    class __RequestError(Exception):
        "Request Error"
        pass

    class __NeedDeleteDisk(Exception):
        "Need delete disk"
        pass

    class __NeedSpindown(Exception):
        pass

    def __get_event_action(self, entry):

        if SATAD_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        if PLUG_ACTION == entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            return PLUG_ACTION

        if UNPLUG_ACTION == entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            return UNPLUG_ACTION

        raise InvalidArgError

    def __send_request(self, type, target_module, child_node=""):
        """Send request"""

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __disk_name2raid_name(self, disk_name):

        for raid_name, element_list in \
                util.get_raid_element_info().iteritems():

            if raid_name in SYSTEM_RAID_DEV_LIST:
                continue

            for element in element_list:
                if element.startswith(disk_name):
                    return raid_name

        raise NotFoundError

    def __disk_name2raw_disk_name(self, disk_name):

        return self.dev_mgr.get_kdev_name(
                    self.dev_mgr.get_base_dev_name(disk_name))

    def __is_encrypt_setting(self):

        INTRNAL_ENCRIPT_XPATH = \
            "./volumelist/volume[name='%s']/encrypt" % INTERNAL_VOL

        volctrl_conf_node = self.__send_request(ReqTypes.LOAD,
                                                FAL_VOLCTRL_PATH).get()
        encrypt_node = volctrl_conf_node.get_node(INTRNAL_ENCRIPT_XPATH)
        if encrypt_node.is_enabled():
            return True
        return False

    def __add_disk_to_raid(self, disk_name):
        """Add disk to raid"""

        result_entry = \
            self.__send_request(ReqTypes.ADD,
                                FAL_RAIDCTRL_PATH,
                                ADD_DISK_TO_RAID_REQ_XML % disk_name)

        return result_entry.get_result()

    def __is_raid_element(self, disk_name):

        for element_list in util.get_raid_element_info().values():

            for element in element_list:
                if element.startswith(disk_name):
                    return True

        return False

    def __update_disk_status(self, disk_name, status):

        try:
            util.fal_diskctrl_update(disk_name, STATUS_TYPE, status)
        except:
            self.exception("fail to fal_diskctrl_update " + status)
            pass

    def __is_able_to_change_mode(self, ext_raid_level):

        result_entry = \
            self.__send_request(ReqTypes.CHECK,
                                FAL_RAIDCTRL_PATH,
                                CHECK_CHANGE_MODE_REQ_XML %  \
                                    {"volume": config.INTERNAL_VOL,
                                     "level": ext_raid_level})
        if Results.OK == result_entry.get_result():
            return True
        return False

    def __test_add_disk(self, ext_raid_level, disk_name):

        result_entry = \
            self.__send_request(ReqTypes.CHECK,
                                FAL_RAIDCTRL_PATH,
                                CHECK_ADDDISK_REQ_XML % \
                                    {"level": ext_raid_level,
                                     "disk": disk_name})
        result_code = result_entry.get_result()

        if Results.OK == result_code:
            self.__update_disk_status(disk_name, NORMAL_STATUS)

        elif Results.CAP_SHORTAGE == result_code:
            self.__update_disk_status(disk_name, CAPSHORTAGE_STATUS)
            if disk_name in CAP_SHORTAGE_LOG_CODE_MAP:
                util.save_event_log(CAP_SHORTAGE_LOG_CODE_MAP[disk_name])

        elif Results.DISK_ERROR == result_code:
            self.__update_disk_status(disk_name, FAIL_STATUS)
            if disk_name in FAIL_LOG_CODE_MAP:
                util.save_event_log(FAIL_LOG_CODE_MAP[disk_name])
        else:
            self.__update_disk_status(disk_name, FAIL_STATUS)
            if disk_name in FAIL_LOG_CODE_MAP:
                util.save_event_log(FAIL_LOG_CODE_MAP[disk_name])

    def __do_extend_disk(self, ext_raid_level):

        result_entry = \
            self.__send_request(ReqTypes.EXTEND,
                                FAL_RAIDCTRL_PATH,
                                EXTEND_REQ_XML_TMPL % \
                                    {"volume": config.INTERNAL_VOL,
                                     "level": ext_raid_level})
        return result_entry.get_result()

    def __do_plug_extend_disk(self, disk_name):

        try:
            self.__update_disk_status(disk_name, NORMAL_STATUS)
            util.dispatch_event(util.DISK_CATEGORY, util.UPDATE_ACTION)
        except:
            self.exception("fail to update disk state")
            pass

        cur_raid_level = util.get_current_raid_level()

        for ext_raid_level in config.EXTEND_RAID_MODE_MAP.get(cur_raid_level):

            array_num = config.RAID_MODE_INFO[ext_raid_level]["array_num"]
            disk_list = config.HDD_DEV_LIST_IN_VOL[array_num][INTERNAL_VOL]

            if disk_name not in disk_list:
                continue

            self.__test_add_disk(ext_raid_level, disk_name)

            if not util.is_auto_extend():
                raise self.__NeedSpindown

            if self.__is_able_to_change_mode(ext_raid_level):

                result_code = self.__do_extend_disk(ext_raid_level)

                if Results.OK == result_code:
                    if util.is_current_raid_mode_vfsext():
                        util.set_quota_limit(INTERNAL_VOL)
                    util.save_event_log(
                        EXTEND_VOLUME_LOG_CODE_MAP[ext_raid_level])
                else:
                    util.save_event_log(
                        EXTEND_VOLUME_ERR_LOG_CODE_MAP[ext_raid_level])
                return

        raise self.__NeedSpindown

    def __do_plug_normal_disk(self, disk_name):

        try:
            self.__update_disk_status(disk_name, NORMAL_STATUS)
            util.dispatch_event(util.DISK_CATEGORY, util.UPDATE_ACTION)
        except:
            self.exception("fail to update disk state")
            pass

        try:
            if self.__is_encrypt_setting():
                util.extract_enckey()
        except:
            self.exception("fail to extract enckey")

        result_code = self.__add_disk_to_raid(disk_name)

        is_need_delete_disk = True

        if Results.OK == result_code:
            if util.is_current_raid_mode_vfsext():
                util.set_quota_limit(INTERNAL_VOL)
            self.__update_disk_status(disk_name, NORMAL_STATUS)
            is_need_delete_disk = False

        elif Results.CAP_SHORTAGE == result_code:
            self.__update_disk_status(disk_name, CAPSHORTAGE_STATUS)
            if disk_name in CAP_SHORTAGE_LOG_CODE_MAP:
                util.save_event_log(CAP_SHORTAGE_LOG_CODE_MAP[disk_name])

        elif Results.DISK_ERROR == result_code:
            self.__update_disk_status(disk_name, FAIL_STATUS)
            if disk_name in FAIL_LOG_CODE_MAP:
                util.save_event_log(FAIL_LOG_CODE_MAP[disk_name])
        else:
            self.__update_disk_status(disk_name, FAIL_STATUS)
            if disk_name in FAIL_LOG_CODE_MAP:
                util.save_event_log(FAIL_LOG_CODE_MAP[disk_name])

        # buzzer
        if Results.OK != result_code:
            util.play_melody(info.BUZZER_ERROR)

        if is_need_delete_disk:
            raise self.__NeedDeleteDisk

        return

    def __action_plug(self, disk_name):
        """action handler for 'plug'"""

        if disk_name not in SATA_HDD_DEV_LIST:
            self.error("Unknown sata disk is plugged")
            return

        is_spindown = False
        try:
            if self.__is_raid_element(disk_name):
                self.__do_plug_normal_disk(disk_name)
            else:
                self.__do_plug_extend_disk(disk_name)

        except self.__NeedDeleteDisk:
            self.__delete_disk_from_system(disk_name)
            raise
        except self.__NeedSpindown:
            is_spindown = True
        except:
            self.exception("fail to do_plug")
            raise
        finally:
            util.dispatch_event(util.DISK_CATEGORY, util.UPDATE_ACTION)
            util.dispatch_event(util.RAID_CATEGORY, util.UPDATE_ACTION)
            if is_spindown:
                util.spindown(disk_name)

    def __get_disk_exist_info(self, disk_name):

        CHK_DISK_EXIST_REQ_XML = \
            """<raidctrl>
                <disk><name>%s</name></disk>
               </raidctrl>"""

        result_entry = \
            self.__send_request(ReqTypes.CHK_EXIST,
                                FAL_RAIDCTRL_PATH,
                                CHK_DISK_EXIST_REQ_XML % disk_name)
        if Results.OK != result_entry.get_result():
            return False, False

        is_rebuild_target = False
        disk_node = result_entry.get().get_node("disk")
        is_rebuild_target = disk_node.is_set_to_true(REBUILD_TARGET_ATTR)
        return True, is_rebuild_target

    # RAIDの冗長性チェック
    def __is_crash_if_remove(self, raid_name, disk_name):
        """Check Raid redudancy"""

        try:
            return util.is_crash_if_remove(raid_name, disk_name)
        except:
            return False

    def __delete_disk_from_system(self, disk_name):

        try:
            util.delete_disk_from_system(disk_name)
        except:
            self.exception("fail to delete_disk_from_system(%s)" % disk_name)
            pass

    def __crash_handling(self, raid_name, disk_name):

        if util.is_raid_crash_or_stop(raid_name):
            return

        is_system_crash = False

        # INTERNAL_DEVがcrashする場合はまずMIRROR_DEVから削除する。
        # この時MIRROR_DEVに冗長性がない場合はMIRROR_DEVをまずcrashさせる。
        if raid_name == INTERNAL_DEV:
            if self.__is_crash_if_remove(MIRROR_DEV, INTERNAL_DEV):
                try:
                    util.unmount_internal_volume()
                    util.fal_raidctrl_stop(MIRROR_DEV, True)
                    is_system_crash = True
                except:
                    self.exception("fail to stop parent_raid_name")
                    pass
            else:
                try:
                    util.fal_raidctrl_remove_partition(MIRROR_DEV, raid_name)
                except:
                    self.exception("fail to remove from MIRROR_DEV")
                    pass
            try:
                util.fal_raidctrl_stop(raid_name, True)
            except:
                self.exception("fail to stop %s" % raid_name)

            try:
                util.fal_raidctrl_remove(disk_name)
            except:
                self.exception("fail to remove %s" % disk_name)

        elif raid_name == MIRROR_DEV:
            try:
                util.unmount_internal_volume()
                util.fal_raidctrl_stop(raid_name, True)
                is_system_crash = True
            except:
                self.exception("fail to stop parent_raid_name")
                pass

        if is_system_crash:
            try:
                util.unmount_all_usb_volume()
            except:
                pass

        try:
            util.fal_volctrl_update(INTERNAL_VOL)
        except:
            self.exception("fail to util.fal_volctrl_update(INTERNAL_VOL)")
            pass

        return

    def __action_unplug(self, disk_name):
        """action handler for 'unplug'"""

        def do_unplug():

            is_exist, is_rebuild_target = self.__get_disk_exist_info(disk_name)
            if not is_exist:
                util.dispatch_event(util.DISK_CATEGORY, util.UPDATE_ACTION)
                util.dispatch_event(util.RAID_CATEGORY, util.UPDATE_ACTION)
                return

            raid_name = self.__disk_name2raid_name(disk_name)

            if self.__is_crash_if_remove(raid_name, disk_name):

                self.debug("__is_crash_if_remove true")
                self.debug(disk_name + " " + raid_name)
                # 冗長性がなくremoveするとRAID崩壊する状況
                self.__crash_handling(raid_name, disk_name)

            else:
                try:
                    util.fal_raidctrl_remove(disk_name)
                except:
                    self.exception("fail to remove %s" % disk_name)

        if disk_name not in SATA_HDD_DEV_LIST:
            self.info("Unknown sata disk is unplugged")
            return

        try:
            do_unplug()
        except:
            self.exception("fail to do_unplug")
            raise
        finally:
            util.fal_diskctrl_update(disk_name, STATUS_TYPE, UNPLUG_STATUS)
            self.__delete_disk_from_system(disk_name)
            util.dispatch_event(util.DISK_CATEGORY, util.UPDATE_ACTION)
            util.dispatch_event(util.RAID_CATEGORY, util.UPDATE_ACTION)

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        action = self.__get_event_action(entry)
        disk_name = entry.get().get_node(DEV_NAME_XPATH).get_text()

        if not disk_name.startswith("satad"):
            entry.set_result(Results.INVALID_XML)
            return entry

        action = entry.get().get_node(EVENTINFO_NAME).\
                     get_attr_value(ACTION_ATTR)

        action_func = {PLUG_ACTION: self.__action_plug,
                       UNPLUG_ACTION: self.__action_unplug}

        try:
            with fileut.FLock(config.LOCK_DIR,
                              config.DISK_EVENT_LOCK_FILE,
                              exclusive=True):

                action_func[action](disk_name)
        except:
            self.exception("fail to action_func")
            raise

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.dev_mgr = device.DevMgr(config.DEV_DIR, config.DEV_LINK_DIR,
                                config.DEV_FILE_DIR, config.HDD_DEV_LIST)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhSatadPlugUnplug(option=evhcmd.EvhCmdOption()).handle_request()
