#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling UsbdPlug event"""

import os
import re
import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
import evh_config as config
from evh_config import DECODED_LOCK_KEY_FILE_PATH
from fal_info import (FAL_LOCKKEY_PATH, FAL_VOLCTRL_PATH, FAL_PORTS_PATH)
from iobase.exception import (NotFoundError, InvalidArgError)
from product_conf import (VOL_LIST)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf.devinfo import (DEV_TO_PORT_NO, DEV_TO_PORT_IF, VOL_INFO,
                                  LOCKKEY_VOL_LIST, USB_HDD_DEV_LIST)

BOOTING_STATE = "booting"

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

USBD_CATEGORY = "usbd"
PLUG_ACTION = "plug"

VOLUME_NAME_XPATH = "eventinfo/device/name"

MOUNT_BASE_REQ_XML = \
    '<volctrl>' + \
    ' <volume>' + \
    '  <name type="volume">%s</name>' + \
    ' </volume>' + \
    '</volctrl>'

CHECK_ENC_SIG_REQ_XML_TMPL = \
    '<volctrl>' + \
    '  <option operation="chkencsig"/>' + \
    '  <volume>' + \
    '   <name type="volume">%s</name>' + \
    '  </volume>' + \
    '</volctrl>'

USB_PORT_ENABLE_MODE_XPATH = \
    "usblist/usb[@port='%s']/modelist/mode[@enable='true']"

RAID_VOL_CATEGORY = "raidvol"
RUNNING_STATE = "running"

# Volume status
KEYWAITING_STATUS = "keywaiting"


class EvhLockkeyPlug(evhcmd.EvhCmd):
    """UsbdPlug Event Hnadler command implementation class."""

    class __MountError(Exception):
        pass

    def __validate_event(self, entry):

        if USBD_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if PLUG_ACTION != entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        return

    def __dev_name2vol_name(self, dev_name):

        for vol_name, vol_info in VOL_INFO.iteritems():

            if dev_name.startswith(vol_info["dev"]):
                return vol_name

        return ""

    def __is_usb_vol_encypted(self, vol_name):

        result_entry = \
            self.__send_request(ReqTypes.CONTROL, FAL_VOLCTRL_PATH,
                                CHECK_ENC_SIG_REQ_XML_TMPL % vol_name)
        if result_entry.get_result() == Results.OK:
            return True

        return False

    def __get_exist_key_waiting_volume_list(self):

        return [vol_name for vol_name in VOL_LIST \
            if KEYWAITING_STATUS == util.get_volume_status(vol_name)]

    def __send_request(self, type, target_module, child_node=""):

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __load_port_config(self):

        return self.__send_request(ReqTypes.LOAD, FAL_PORTS_PATH).get()

    def __share_mount(self, vol_name):

        result_entry = \
            self.__send_request(ReqTypes.MOUNT, FAL_VOLCTRL_PATH,
                                MOUNT_BASE_REQ_XML % vol_name)
        if result_entry.get_result() != Results.OK:
            raise self.__MountError
        return

    def __is_need_mount(self, vol_name):

        if vol_name == util.get_internal_vol_name():
            return True

        match = re.match("usb([1-9]+)", vol_name)
        if not match:
            return False

        port_num = match.group(1)

        port_conf_node = self.__load_port_config()
        mode = port_conf_node.get_node(
            USB_PORT_ENABLE_MODE_XPATH % port_num).get_text()

        if "share" == mode:
            return True

        return False

    def __try_unlock_volume(self, vol_name):

        waiting_vol_list = self.__get_exist_key_waiting_volume_list()
        if len(waiting_vol_list) == 0:
            return

        portconf_node = self.__load_port_config()

        for vol_name in waiting_vol_list:
            try:
                if self.__is_need_mount(vol_name):
                    self.__share_mount(vol_name)

                if vol_name == util.get_internal_vol_name():
                    util.set_state_manager(RAID_VOL_CATEGORY, RUNNING_STATE)
            except:
                continue

        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self.__validate_event(entry)
        dev_name = entry.get().get_node(VOLUME_NAME_XPATH).get_text()
        vol_name = self.__dev_name2vol_name(dev_name)

        if vol_name not in LOCKKEY_VOL_LIST:
            entry.set_result(Results.OK)
            return entry

        if BOOTING_STATE == util.get_system_state():
            entry.set_result(Results.OK)
            return entry

        if not util.is_boot_complete():
            entry.set_result(Results.OK)
            return entry

        try:
            self.__try_unlock_volume(vol_name)
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhLockkeyPlug(option=evhcmd.EvhCmdOption()).handle_request()
