#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling UPS event"""

import re
import libevh.evhcmd as evhcmd
from ionas.request import Entry
from fal_info import (FAL_SERVICECTRL_PATH, FAL_UPS_PATH, FAL_BUZZER_PATH)
from product_conf.reqinfo import (Results, ReqTypes)
import libevh.util as util
from fal_msginfo import Msg


UPS_SERVICE_ENTRY = "<servicectrl><servicelist><service><name>%s</name>" + \
            "</service></servicelist></servicectrl>"
BUZZER_ENTRY = "<buzzer><melody type='ups'/></buzzer>"

EVENTINFO_NAME = "eventinfo"
DEVICE_XPATH = "eventinfo/device/name"
USBUPS = "usbups"
SERIALUPS = "serialups"
ACTION_ATTR = "action"
CATEGORY_ATTR = "category"
PLUG = "plug"
UNPLUG = "unplug"
OMRONSSD = "omronssd"
APCUPSD = "apcupsd"
NUT = "nut"
WARNING_XPATH = "warning"


class EvhUPS(evhcmd.EvhCmd):
    """UPS Event Hnadler command implementation class."""

    def __start_or_stop_service(self, type, service_name):
        entry = Entry(type, UPS_SERVICE_ENTRY % service_name)
        response = self.send_entry(entry, FAL_SERVICECTRL_PATH)
        return response

    def __update_request(self):
        entry = Entry(ReqTypes.UPDATE)
        return self.send_entry(entry, FAL_UPS_PATH)

    def __get_service_name(self, entry):
        ups_dev = entry.get().get_node(DEVICE_XPATH).get_text()
        if re.match(".*\.apc$", ups_dev):
            return APCUPSD
        elif re.match(".*\.omron$", ups_dev):
            return OMRONSSD
        elif re.match(".*\.fuji$", ups_dev):
            return NUT
        return ""

    def __warning_is_enabled(self):
        conf = self.send_entry(Entry(ReqTypes.LOAD), FAL_UPS_PATH)
        if conf.get().get_node(WARNING_XPATH).is_enabled():
            return True
        else:
            return False

    def __control_buzzer(self, type):
        entry = Entry(type, BUZZER_ENTRY)
        buzz = self.send_entry(entry, FAL_BUZZER_PATH)
        return

    def __usbups_process(self, entry):

        action = entry.get().get_node(
                    EVENTINFO_NAME).get_attr_value(ACTION_ATTR)
        service_name = self.__get_service_name(entry)
        if not service_name:
            # Force exit without error because of the unsuppored device.
            entry.set_result(Results.OK)
            return entry

        if action == PLUG:
            if self.__warning_is_enabled():
                util.save_event_log(Msg.MSG_UPS_DETECT_UPS)
                self.__control_buzzer(ReqTypes.STOP)
            update_result = self.__update_request()
            if update_result.get_result() != Results.OK:
                entry.set_result(update_result.get_result())
                return entry
            start = self.__start_or_stop_service(ReqTypes.START, service_name)
            if start.get_result() != Results.OK:
                entry.set_result(start.get_result())
                return entry
            entry.set_result(Results.OK)
            return entry

        if action == UNPLUG:
            if self.__warning_is_enabled():
                util.save_event_log(Msg.ERR_UPS_DETECT_NO_UPS)
                self.__control_buzzer(ReqTypes.PLAY)
            stop = self.__start_or_stop_service(ReqTypes.STOP, service_name)
            if stop.get_result() != Results.OK:
                entry.set_result(stop.get_result())
                return entry
            entry.set_result(Results.OK)
            return entry

        entry.set_result(Results.EVENT_ERROR)
        return entry

    def __serialups_process(self, entry):

        action = entry.get().get_node(
                    EVENTINFO_NAME).get_attr_value(ACTION_ATTR)
        if action == PLUG:
            if self.__warning_is_enabled():
                util.save_event_log(Msg.MSG_UPS_DETECT_UPS)
                self.__control_buzzer(ReqTypes.STOP)
            update_result = self.__update_request()
            if update_result.get_result() != Results.OK:
                entry.set_result(update_result.get_result())
                return entry
            start = self.__start_or_stop_service(ReqTypes.START, APCUPSD)
            if start.get_result() != Results.OK:
                entry.set_result(start.get_result())
                return entry
            entry.set_result(Results.OK)
            return entry

        if action == UNPLUG:
            if self.__warning_is_enabled():
                util.save_event_log(Msg.ERR_UPS_DETECT_NO_UPS)
                self.__control_buzzer(ReqTypes.PLAY)
            stop = self.__start_or_stop_service(ReqTypes.STOP, APCUPSD)
            if stop.get_result() != Results.OK:
                entry.set_result(stop.get_result())
                return entry
            entry.set_result(Results.OK)
            return entry

        entry.set_result(Results.EVENT_ERROR)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        category = entry.get().get_node(
                    EVENTINFO_NAME).get_attr_value(CATEGORY_ATTR)

        if category == USBUPS:
            result = self.__usbups_process(entry)
        elif category == SERIALUPS:
            result = self.__serialups_process(entry)
        else:
            entry.set_result(Results.EVENT_ERROR)
            return entry
        return result

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhUPS(option=evhcmd.EvhCmdOption()).handle_request()
