#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling UsbdPlug event"""

import os
import sys
import re
import fal_info as info
import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
import fal_msginfo as msginfo
from evh_config import DECODED_LOCK_KEY_FILE_PATH
from fal_info import (FAL_VOLCTRL_PATH, FAL_LOCKKEY_PATH, FAL_SERVICECTRL_PATH)
from iobase.exception import (InvalidArgError)
from iobase.cmdexec import (CmdExec, PIPE, NULL)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf import (UNLOCKD_CMD, PS_CMD, STOR_IF_TO_VOL)
from product_conf.devinfo import (LOCKKEY_VOL_LIST)

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

VOLUME_CATEGORY = "volume"
MOUNT_ACTION = "mount"

VOLUME_NAME_XPATH = "eventinfo/volume/name"

START_ENC_REQ_XML_TMPL = \
    '<volctrl>' + \
    ' <option operation="encstart"/>' + \
    ' <volume>' + \
    '  <name type="volume">%s</name>' + \
    ' </volume>' + \
    '</volctrl>'

STOP_ENC_REQ_XML_TMPL = \
    '<volctrl>' + \
    ' <option operation="encstop"/>' + \
    ' <volume>' + \
    '  <name type="volume">%s</name>' + \
    ' </volume>' + \
    '</volctrl>'

EXTRACT_LOCKKEY_REQ_XML = \
    '<lockkey>' + \
    ' <option operation="extract"/>' + \
    '</lockkey>'

CHECK_ENC_SIG_REQ_XML_TMPL = \
    '<volctrl>' + \
    '  <option operation="chkencsig"/>' + \
    '  <volume>' + \
    '   <name type="volume">%s</name>' + \
    '  </volume>' + \
    '</volctrl>'

INTRNAL_ENCRIPT_XPATH = \
    "./volumelist/volume[name='sataraid1']/encrypt"

ENABLE_KEYSERVER_XPATH = \
    "./keyserverlist/keyserver[@enable='true']"

RAID_VOL_CATEGORY = "raidvol"
KEYWAITING_STATE = "keywaiting"

### Command ###
PS_AX_ARG = [PS_CMD, "ax"]

KEYWAITING_STATUS = "keywaiting"
LOCKKEY_STATUS = "lockkey"
RUNNING_STATE = "running"

INTERNAL_NAME = "internal"
USB_NAME = "usb"

USB_UNLOCK_FAIL_LOG_MSG_LIST = \
    [msginfo.Msg.ERR_LOCKKEY_USB1_SECURITY_UNLOCK,
     msginfo.Msg.ERR_LOCKKEY_USB2_SECURITY_UNLOCK,
     msginfo.Msg.ERR_LOCKKEY_USB3_SECURITY_UNLOCK,
     msginfo.Msg.ERR_LOCKKEY_USB4_SECURITY_UNLOCK]

INTERNAL_UNLOCK_FAIL_LOG_MSG_LIST = \
    [msginfo.Msg.ERR_LOCKKEY_SECURITY_UNLOCK]

USB_UNLOCK_FAIL_LOG_CODE_MAP = \
    dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems()]),
             USB_UNLOCK_FAIL_LOG_MSG_LIST))

INTERNAL_UNLOCK_FAIL_LOG_CODE_MAP = \
    dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[INTERNAL_NAME].iteritems()]),
             INTERNAL_UNLOCK_FAIL_LOG_MSG_LIST))

UNLOCK_FAIL_LOG_CODE_MAP = USB_UNLOCK_FAIL_LOG_CODE_MAP
UNLOCK_FAIL_LOG_CODE_MAP.update(INTERNAL_UNLOCK_FAIL_LOG_CODE_MAP)

DISK_SERVICECTRL_REQ_XML = """<servicectrl>
    <categorylist>
        <category><name>disk</name></category>
    </categorylist>
</servicectrl>
"""


class EvhUsbdPlug(evhcmd.EvhCmd):
    """UsbdPlug Event Hnadler command implementation class."""

    class __ExtractError(Exception):

        pass

    class __StartEncError(Exception):

        pass

    class __StopEncError(Exception):

        pass

    class __DeviceNotFound(Exception):
        "Lockkey Device Not Found Error"
        pass

    class __NoLockkey(Exception):
        "No Lockkey"
        pass

    class __UnknownFilesystem(Exception):
        "Unknown Filesystem"
        pass

    def __validate_event(self, entry):

        if VOLUME_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if not entry.get().get_node(EVENTINFO_NAME)\
            .is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        if MOUNT_ACTION != entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            raise InvalidArgError

        return

    def __send_request(self, req_type, target_module, child_node=""):

        entry = request.Entry(req_type, child_node)
        return self.send_entry(entry, target_module)

    def __is_enable_keyserver(self):

        # lockkey設定ファイルからkeyserverの有無を判別
        lockkey_conf_node = \
            self.__send_request(ReqTypes.LOAD, FAL_LOCKKEY_PATH).get()

        keyserver_node_list = \
            lockkey_conf_node.get_node_list(ENABLE_KEYSERVER_XPATH)

        if len(keyserver_node_list) > 0:
            return True

        return False

    def __is_exist_unlock_deamon(self):
        """Get the PID of spindown daemon."""

        unlockd_name = os.path.basename(UNLOCKD_CMD)
        daemon_serch_reg_str = ".*%s.*" % unlockd_name

        proc = CmdExec(PS_AX_ARG,
                       stdout=PIPE, stdin=NULL, stderr=NULL).execute()
        for line in proc.stdout:
            match = re.search(daemon_serch_reg_str, line)

            if not match:
                continue

            return True
        return False

    def __start_unlock_daemon(self):

        if self.__is_exist_unlock_deamon():
            return

        CmdExec(UNLOCKD_CMD,
                stdin=NULL, stdout=NULL, stderr=NULL,
                wait=False, raise_exc=True, close_fds=True).execute()
        return

    def __extract_enckey(self):

        result_entry = \
            self.__send_request(ReqTypes.SET_LOCKKEY,
                                FAL_LOCKKEY_PATH,
                                EXTRACT_LOCKKEY_REQ_XML)
        result_code = result_entry.get_result()

        if Results.OK == result_entry.get_result():
            return

        if Results.DEVICE_NOT_FOUND == result_code:
            raise self.__DeviceNotFound

        if Results.NO_LOCKKEY == result_code:
            raise self.__NoLockkey

        raise self.__ExtractError

    def __start_enc_volume(self, vol_name):

        ctrl_entry = request.Entry(ReqTypes.CONTROL,
                                   START_ENC_REQ_XML_TMPL % vol_name)
        ctrl_entry.set_file(DECODED_LOCK_KEY_FILE_PATH)

        result_entry = self.send_entry(ctrl_entry, FAL_VOLCTRL_PATH)

        if result_entry.get_result() != Results.OK:
            raise self.__StartEncError
        return

    def __stop_enc_volume(self, vol_name):

        ctrl_entry = request.Entry(ReqTypes.CONTROL,
                                   STOP_ENC_REQ_XML_TMPL % vol_name)

        result_entry = self.send_entry(ctrl_entry, FAL_VOLCTRL_PATH)

        if result_entry.get_result() != Results.OK:
            raise self.__StopEncError
        return

    def __unlock_volume(self, vol_name):

        self.__extract_enckey()
        self.__start_enc_volume(vol_name)
        util.fal_volctrl_update(vol_name)

        volctrl_node = util.fal_volctrl_query(vol_name)
        filesystem = volctrl_node.get_value("volume/filesystem")
        if not filesystem:
            self.__stop_enc_volume(vol_name)
            raise self.__UnknownFilesystem
        return

    def __start_suspend_disk_service(self):

        self.__send_request(ReqTypes.START,
                            FAL_SERVICECTRL_PATH,
                            DISK_SERVICECTRL_REQ_XML)

        self.__send_request(ReqTypes.SUSPEND,
                            FAL_SERVICECTRL_PATH,
                            DISK_SERVICECTRL_REQ_XML)

    def __stop_disk_service(self):

        self.__send_request(ReqTypes.STOP,
                            FAL_SERVICECTRL_PATH,
                            DISK_SERVICECTRL_REQ_XML)

    def __ctrl_disk_service(self, vol_name):

        if vol_name == STOR_IF_TO_VOL[INTERNAL_NAME][1]:

            if KEYWAITING_STATUS == util.get_volume_status(vol_name):
                self.__stop_disk_service()
            else:
                self.__start_suspend_disk_service()

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""
        self.__validate_event(entry)

        vol_name = entry.get().get_node(VOLUME_NAME_XPATH).get_text()

        util.fal_volctrl_update(vol_name)

        is_enable_keyserver = self.__is_enable_keyserver()

        if KEYWAITING_STATUS != util.get_volume_status(vol_name):
            self.__ctrl_disk_service(vol_name)
            entry.set_result(Results.OK)
            return entry

        # アンロック処理開始
        try:
            self.__unlock_volume(vol_name)
        except self.__DeviceNotFound:
            # USB1が存在しない時は
            # アンロック失敗ログは残さない
            if is_enable_keyserver:
                self.__start_unlock_daemon()
            if vol_name == STOR_IF_TO_VOL[INTERNAL_NAME][1]:
                # 構成異常と同じ音
                util.play_melody(info.BUZZER_LOCKMODE)
            entry.set_result(Results.SET_ERROR)
            return entry

        except:
            self.exception("fail to __unlock_volume")
            if is_enable_keyserver:
                self.__start_unlock_daemon()
            if vol_name == STOR_IF_TO_VOL[INTERNAL_NAME][1]:
                util.set_state_manager(RAID_VOL_CATEGORY, KEYWAITING_STATE)
                # 構成異常と同じ音
                util.play_melody(info.BUZZER_LOCKMODE)

            if self.__NoLockkey == sys.exc_info()[0]:
                self.error("NoLockkey exceoption")
                util.save_event_log(msginfo.Msg.ERR_LOCKKEY_NO_KEY_FILE)
            else:
                if vol_name in UNLOCK_FAIL_LOG_CODE_MAP:
                    util.save_event_log(UNLOCK_FAIL_LOG_CODE_MAP[vol_name])

            entry.set_result(Results.SET_ERROR)
            return entry

        self.__ctrl_disk_service(vol_name)

        if vol_name == STOR_IF_TO_VOL[INTERNAL_NAME][1]:
            util.set_state_manager(RAID_VOL_CATEGORY, RUNNING_STATE)
            util.stop_melody(info.BUZZER_LOCKMODE)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhUsbdPlug(option=evhcmd.EvhCmdOption()).handle_request()
