#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command to start or stop services for "disk" category."""

import ionas.request as request
import libevh.evhcmd as evhcmd
from fal_info import (FAL_SERVICECTRL_PATH,)
from iobase.exception import (NotFoundError, InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
VOLUME_CATEGORY = "volume"
MOUNT_ACTION = "mount"
UNMOUNT_ACTION = "unmount"
SYSTEM_CATEGORY = "system"
BOOT_ACTION = "boot"

DISK_SERVICECTRL_REQ_XML = """<servicectrl>
    <categorylist>
        <category><name>disk</name></category>
    </categorylist>
</servicectrl>
"""


class EvhDiskService(evhcmd.EvhCmd):
    """Event Handler to start or stop services for "disk" category."""

    def __send_request(self, target, type, body):
        """Send request entry to "target".""" 

        entry = request.Entry(type, body)
        return self.send_entry(entry, target)

    def __event_handler(self, entry):
        """Handle "event" request."""

        eventinfo = entry.get().get_node(EVENTINFO_NAME)
        if eventinfo.get_attr_value(CATEGORY_ATTR) == SYSTEM_CATEGORY:
            if eventinfo.get_attr_value(ACTION_ATTR) == BOOT_ACTION:
                self.__send_request(FAL_SERVICECTRL_PATH,
                                ReqTypes.SUSPEND,
                                DISK_SERVICECTRL_REQ_XML)
        elif eventinfo.get_attr_value(CATEGORY_ATTR) == VOLUME_CATEGORY:
            action = eventinfo.get_attr_value(ACTION_ATTR)
            if action == MOUNT_ACTION or action == UNMOUNT_ACTION:
                try:
                    previous = eventinfo.get_attr_value(PREVIOUS_ATTR)
                except:
                    previous = "false"
                if previous == "true":
                    self.__send_request(FAL_SERVICECTRL_PATH,
                                    ReqTypes.SUSPEND,
                                    DISK_SERVICECTRL_REQ_XML)
                else:
                    self.__send_request(FAL_SERVICECTRL_PATH,
                                    ReqTypes.RESUME,
                                    DISK_SERVICECTRL_REQ_XML)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhDiskService(option=evhcmd.EvhCmdOption()).handle_request()
