#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling Printer event"""

import os

import libevh.evhcmd as evhcmd
from ionas.request import Entry
from fal_info import (FAL_SERVICECTRL_PATH, FAL_PRINTER_PATH, FAL_VOLCTRL_PATH)
from pal_info import (PAL_SAMBA_PATH, Results, ReqTypes)
import evh_config as config


SERVICE_ENTRY = "<servicectrl><servicelist><service><name>%s</name>" + \
        "</service></servicelist></servicectrl>"
QUERY_MNT_ENTRY = "<volctrl><volume><name type='volume'>sataraid1</name>" + \
        "</volume></volctrl>"

EVENTINFO_NAME = "eventinfo"
ACTION_ATTR = "action"
CATEGORY_ATTR = "category"
USBLP = "usblp"
PLUG = "plug"
UNPLUG = "unplug"
VOLUME = "volume"
MOUNT = "mount"
UNMOUNT = "unmount"
CUPS_SERVICE = "cups"
SMB_SERVICE = "samba"

MNT_STATUS_XPATH = "./volume/status"

PRINTER_FLAG_DIR = config.TEMP_DIR + "/printer"
PRINTER_FLAG_PATH = PRINTER_FLAG_DIR + "/printer_enable"


class EvhPrinter(evhcmd.EvhCmd):
    """Printer Event Hnadler command implementation class."""

    def __send_service_request(self, type, service):
        entry = Entry(type, SERVICE_ENTRY % service)
        response = self.send_entry(entry, FAL_SERVICECTRL_PATH)
        return response

    def __sataraid1_is_mounted(self):
        try:
            mnt_entry = Entry(ReqTypes.QUERY, QUERY_MNT_ENTRY)
            response = self.send_entry(mnt_entry, FAL_VOLCTRL_PATH)
            mnt_status = response.get().get_node(MNT_STATUS_XPATH).get_text()
            if mnt_status == "mounted":
                return True
        except:
            return False
        return False

    def __update_printer_conf(self, type):
        response = self.send_entry(Entry(type), FAL_PRINTER_PATH)
        return response

    def __printer_is_connected(self):
        for dev in config.USB_LP_DEV_LIST:
            if os.path.exists(config.DEV_DIR + "/" + dev):
                return True
        return False

    def __update_flag_file(self, create=True):
        if create:
            if not os.path.exists(PRINTER_FLAG_DIR):
                os.makedirs(PRINTER_FLAG_DIR)
            open(PRINTER_FLAG_PATH, "w").close()
        else:
            try:
                os.remove(PRINTER_FLAG_PATH)
            except:
                pass
        return

    def __update_smb_conf(self, do_restart=True):
        if do_restart:
            suspend = self.__send_service_request(ReqTypes.SUSPEND, SMB_SERVICE)
            if suspend.get_result() != Results.OK:
                return suspend

        response = self.send_entry(Entry(ReqTypes.UPDATE), PAL_SAMBA_PATH)

        if do_restart:
            resume = self.__send_service_request(ReqTypes.RESUME, SMB_SERVICE)
            if resume.get_result() != Results.OK:
                return resume
        return response

    def __plug_process(self, entry):
        if self.__sataraid1_is_mounted():
            cups_conf = self.__update_printer_conf(ReqTypes.ENABLE)
            if cups_conf.get_result() != Results.OK:
                entry.set_result(cups_conf.get_result())
                return entry
            start = self.__send_service_request(ReqTypes.START, CUPS_SERVICE)
            if start.get_result() != Results.OK:
                entry.set_result(start.get_result())
                return entry
            self.__update_flag_file(True)
            smb_result = self.__update_smb_conf(do_restart=True)
            if smb_result.get_result() != Results.OK:
                entry.set_result(smb_result.get_result())
                return entry
        entry.set_result(Results.OK)
        return entry

    def __unplug_process(self, entry):
        cups_conf = self.__update_printer_conf(ReqTypes.DISABLE)
        if cups_conf.get_result() != Results.OK:
            entry.set_result(cups_conf.get_result())
            return entry
        stop = self.__send_service_request(ReqTypes.STOP, CUPS_SERVICE)
        if stop.get_result() != Results.OK:
            entry.set_result(stop.get_result())
            return entry
        self.__update_flag_file(False)
        smb_result = self.__update_smb_conf(do_restart=True)
        if smb_result.get_result() != Results.OK:
            entry.set_result(smb_result.get_result())
            return entry
        entry.set_result(Results.OK)
        return entry

    def __mount_process(self, entry):
        if self.__printer_is_connected():
            cups_conf = self.__update_printer_conf(ReqTypes.ENABLE)
            if cups_conf.get_result() != Results.OK:
                entry.set_result(cups_conf.get_result())
                return entry
            start = self.__send_service_request(ReqTypes.START, CUPS_SERVICE)
            if start.get_result() != Results.OK:
                entry.set_result(start.get_result())
                return entry
            self.__update_flag_file(True)
            smb_result = self.__update_smb_conf(do_restart=False)
            if smb_result.get_result() != Results.OK:
                entry.set_result(smb_result.get_result())
                return entry
        entry.set_result(Results.OK)
        return entry

    def __unmount_process(self, entry):
        cups_conf = self.__update_printer_conf(ReqTypes.DISABLE)
        if cups_conf.get_result() != Results.OK:
            entry.set_result(cups_conf.get_result())
            return entry
        stop = self.__send_service_request(ReqTypes.STOP, CUPS_SERVICE)
        if stop.get_result() != Results.OK:
            entry.set_result(stop.get_result())
            return entry
        self.__update_flag_file(False)
        smb_result = self.__update_smb_conf(do_restart=False)
        if smb_result.get_result() != Results.OK:
            entry.set_result(smb_result.get_result())
            return entry
        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        evt_info = entry.get().get_node(EVENTINFO_NAME)
        category = evt_info.get_attr_value(CATEGORY_ATTR)
        if not category in [USBLP, VOLUME]:
            entry.set_result(Results.EVENT_ERROR)
            return entry

        action = evt_info.get_attr_value(ACTION_ATTR)
        if action == PLUG:
            result = self.__plug_process(entry)
        elif action == UNPLUG:
            result = self.__unplug_process(entry)
        elif action == MOUNT:
            if evt_info.get_node("volume/name").get_text() != \
                            config.LPR_SPOOL_VOL:
                entry.set_result(Results.OK)
                return entry
            result = self.__mount_process(entry)
        elif action == UNMOUNT:
            if evt_info.get_node("volume/name").get_text() != \
                            config.LPR_SPOOL_VOL:
                entry.set_result(Results.OK)
                return entry
            result = self.__unmount_process(entry)
        else:
            entry.set_result(Results.EVENT_ERROR)
            return entry

        return result

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhPrinter(option=evhcmd.EvhCmdOption()).handle_request()
