#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling Announce event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
import fal_info
from iobase.exception import (NotFoundError, InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)

EVENTINFO_NAME = "eventinfo"
ANNOUNCE_NAME = "announce"
CODE_NAME = "code"

CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
TIMESTAMP_ATTR = "timestamp"
DATE_ATTR = "date"
LEVEL_ATTR = "level"
CAUSE_ATTR = "cause"

EVENTLOG_CATEGORY = "eventlog"
LCM_CATEGORY = "lcm"
ADD_ACTION = "add"
CLEAR_ACTION = "clear"

TARGET_LOG_XPATH = "eventinfo/eventlog/log[@level='1' or @level='2']"
MESSAGES_XPATH = "eventinfo/eventlog/messages"
MESSAGEINFO_XPATH = MESSAGES_XPATH + "/messageinfolist/messageinfo"
RELATEDCODELIST_XPATH = "noticeinfo/relatedcodelist"
NOTIFY_XPATH = "noticeinfo/notify"


class EvhAnnounce(evhcmd.EvhCmd):
    """Announce Event Hnadler command implementation class."""

    def __validate_target_event(self, entry):
        """Validate target event entry."""

        eventinfo_node = entry.get().get_node(EVENTINFO_NAME)
        if EVENTLOG_CATEGORY != eventinfo_node.get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if ADD_ACTION != eventinfo_node.get_attr_value(ACTION_ATTR):
            raise InvalidArgError

        if eventinfo_node.is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

    def __send_request(self, type, target_module, child_node=""):
        """Send request to module."""

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __clear_recovered_announce(self, entry):
        """If log level is normal and if same category announce exists
           clear target announce
        """

        LEVEL0_LOG_XPATH = "eventinfo/eventlog/log[@level='0']"
        ANNOUNCE_XML_BASE = "<announce><option code='%s'/></announce>"

        try:
            log_node = entry.get().get_node(LEVEL0_LOG_XPATH)
            if not self.__is_enable_announce(entry):
                return
            code = log_node.get_value(CODE_NAME, None)
        except:
            return

        announce_xml = ANNOUNCE_XML_BASE % code
        result_entry = self.__send_request(
            ReqTypes.CLEAR, fal_info.FAL_ANNOUNCE_PATH, announce_xml)
        if result_entry.get_result() == Results.OK:
            try:
                self.__dispatch_event()
            except:
                self.exception("fail to '__dispatch_event'")
                pass
        return

    def __dispatch_event(self):
        """Dispatch "LCM clear" event."""

        entry = request.Entry(
                ReqTypes.EVENT,
                fal_info.FAL_EVENTS_REQ_BASE_XML[ReqTypes.EVENT])
        entry.get().get_node(EVENTINFO_NAME).set_attrs(
            {CATEGORY_ATTR: LCM_CATEGORY, ACTION_ATTR: CLEAR_ACTION})
        request.Request(fal_info.FAL_EVENTS_PATH).request(entry)

    def __is_enable_announce(self, entry):
        """Check if the code is announce target."""

        try:
            messageinfo = entry.get().get_node(MESSAGEINFO_XPATH)
        except:
            self.exception("messageinfo load failed.")
            return False

        try:
            if messageinfo.get_node(RELATEDCODELIST_XPATH):
                return True
        except:
            pass

        try:
            return messageinfo.get_node(NOTIFY_XPATH).is_enabled()
        except:
            return False

    def __update_status_led(self):
        """Update status LED."""

        try:
            util.fal_led_update()
        except:
            self.exception("fail to fal_led_update")
        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self.__validate_target_event(entry)

        self.messages_node = entry.get().get_node(MESSAGES_XPATH)

        try:
            log_node = entry.get().get_node(TARGET_LOG_XPATH)
        except NotFoundError:
            self.__clear_recovered_announce(entry)
            self.__update_status_led()
            entry.set_result(Results.OK)
            return entry

        # <log> から <annouce> に変更(以下のログは一緒)
        log_node.set_node_name(ANNOUNCE_NAME)
        log_node.delete_attrs((TIMESTAMP_ATTR, DATE_ATTR))

        result_entry = self.__send_request(
            ReqTypes.ADD, fal_info.FAL_ANNOUNCE_PATH, log_node.to_str())
        if result_entry.get_result() != Results.OK:
            entry.set_result(result_entry.get_result())
            return entry

        self.__update_status_led()
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)
        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhAnnounce(option=evhcmd.EvhCmdOption()).handle_request()
