#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling volume-mount post event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
from fal_info import (FAL_VOLCTRL_PATH)
from iobase.exception import (InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)
from evh_config import (INTERNAL_VOL)

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
MOUNT_FAIL_ATTR = "mountfail"

VOLUME_CATEGORY = "volume"
MOUNT_ACTION = "mount"

VOLUME_NAME_XPATH = "eventinfo/volume/name"
VOLUME_XPATH = "eventinfo/volume"

FORMATTING_STATE = "formatting"
CHECKING_STATE = "checking"


class EvhChkUnlock(evhcmd.EvhCmd):
    """UsbdPlug Event Hnadler command implementation class."""

    def __validate_event(self, entry):

        if VOLUME_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).\
                   is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        if MOUNT_ACTION != entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            raise InvalidArgError

        return

    def __send_request(self, req_type, target_module, child_node=""):

        entry = request.Entry(req_type, child_node)
        return self.send_entry(entry, target_module)

    def __load_volctrl_config(self):

        return self.__send_request(ReqTypes.LOAD,
                                   FAL_VOLCTRL_PATH).get()

    def __is_encrypted_volume(self, vol_name):

        INTRNAL_ENCRIPT_XPATH = \
            "./volumelist/volume[name='%s']/encrypt" % INTERNAL_VOL

        CHECK_ENC_SIG_REQ_XML_TMPL = \
            '<volctrl>' + \
            '  <option operation="chkencsig"/>' + \
            '  <volume>' + \
            '   <name type="volume">%s</name>' + \
            '  </volume>' + \
            '</volctrl>'

        if INTERNAL_VOL == vol_name:

            volctrl_conf_node = self.__load_volctrl_config()
            encrypt_node = volctrl_conf_node.get_node(INTRNAL_ENCRIPT_XPATH)
            if encrypt_node.is_enabled():
                return True
            return False

        if vol_name.startswith("usb"):
            result_entry = \
                self.__send_request(ReqTypes.CONTROL,
                                    FAL_VOLCTRL_PATH,
                                    CHECK_ENC_SIG_REQ_XML_TMPL % vol_name)
            if result_entry.get_result() == Results.OK:
                return True
            return False

        raise self.__InternalError

    #
    # Request handlers
    #
    def __event_handler(self, entry):
        """Handle "event" request."""
        self.__validate_event(entry)

        vol_name = entry.get().get_node(VOLUME_NAME_XPATH).get_text()

        if not self.__is_encrypted_volume(vol_name):
            entry.set_result(Results.OK)
            return entry

        is_mountfail = \
            entry.get().get_node(VOLUME_XPATH).is_set_to_true(MOUNT_FAIL_ATTR)

        util.fal_volctrl_update(vol_name)

        fs_state = util.get_filesystem_state()

        # アンロック成功
        if not is_mountfail:

            # mkfs, fsck時も本イベントが発生する
            if fs_state == FORMATTING_STATE or fs_state == CHECKING_STATE:
                entry.set_result(Results.OK)
                return entry

            try:
                util.save_unlock_event_log(vol_name)
            except:
                self.exception("fail save_unlock_event_log()")
                pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhChkUnlock(option=evhcmd.EvhCmdOption()).handle_request()
