#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling system-boot event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
from fal_info import FAL_DISKCTRL_PATH
from iobase.exception import (InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf.devinfo import (SATA_HDD_DEV_LIST, USB_HDD_DEV_LIST)

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
SYSTEM_CATEGORY = "system"
BOOT_ACTION = "boot"

STATUS_TYPE = "status"
FAIL_STATUS = "fail"
SMARTERROR_STATUS = "smarterror"
NORMAL_STATUS = "normal"
UNPLUG_STATUS = "unplug"


class EvhDiskCheck(evhcmd.EvhCmd):
    """UsbdPlug Event Hnadler command implementation class."""

    class __RequestError(Exception):
        "Request Error"
        pass

    def __get_event_action(self, entry):

        if SYSTEM_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        is_previous = \
            entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR)

        if BOOT_ACTION == entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            if not is_previous:
                return BOOT_ACTION

    def __send_request(self, type, target_module, child_node=""):

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __get_disk_error(self, disk_name):

        ERROR_VAL = "error"

        CHECK_ERROR_XML = \
            '<diskctrl><disk><name>%s</name></disk></diskctrl>'

        SMART_STATUS_XPATH = "disk/smartstatus"
        ERRCNT_STATUS_XPATH = "disk/diskerrcnt"

        result_entry = self.__send_request(ReqTypes.CHECK,
                                           FAL_DISKCTRL_PATH,
                                           CHECK_ERROR_XML % disk_name)
        if Results.OK != result_entry.get_result():
            raise self.__RequestError

        smart_result = \
            result_entry.get().get_node(SMART_STATUS_XPATH).get_text()

        errcnt_result = \
            result_entry.get().get_node(ERRCNT_STATUS_XPATH).get_text()

        return (ERROR_VAL == smart_result,
                ERROR_VAL == errcnt_result)

    def __check_and_save_disk_info(self, disk_name):

        if not util.is_device_exist(disk_name):
            util.fal_diskctrl_update(disk_name,
                                     STATUS_TYPE,
                                     UNPLUG_STATUS)
            return

        is_smart_error, is_errcnt_over = self.__get_disk_error(disk_name)

        if is_errcnt_over:
            util.fal_diskctrl_update(disk_name,
                                     STATUS_TYPE,
                                     FAIL_STATUS)
        elif is_smart_error:
            util.fal_diskctrl_update(disk_name,
                                     STATUS_TYPE,
                                     SMARTERROR_STATUS)
        else:
            util.fal_diskctrl_update(disk_name,
                                     STATUS_TYPE,
                                     NORMAL_STATUS)
        return

    def __action_boot(self, entry):

        for disk_name in SATA_HDD_DEV_LIST + USB_HDD_DEV_LIST:

            if disk_name.startswith("usbd"):
                if not util.is_mirroring_setting(disk_name) and \
                       not util.is_external_vfsext_setting(disk_name):
                    continue

            try:
                self.__check_and_save_disk_info(disk_name)
            except:
                self.exception("Fail to __check_and_save_disk_info()")
                continue
        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        action = self.__get_event_action(entry)

        self.__action_func = \
            {BOOT_ACTION: self.__action_boot}

        try:
            self.__action_func[action](entry)
        except:
            self.exception("Fail to __action_func() - do nothing")
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, evhcmd.EvhCmdOption())

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhDiskCheck().handle_request()
