#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling RAID update event"""


from product_conf.reqinfo import (Results, ReqTypes)
from product_conf.devinfo import (SATA_HDD_DEV_LIST, DEV_TO_PORT_NO)

import libevh.evhcmd as evhcmd
import libevh.util as util


EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"

DISK_CATEGORY = "disk"
UPDATE_ACTION = "update"

RAIDVOL_CATEGORY = "raidvol"
CRASH_STATUS = "crash"

NORMAL_STATUS = "normal"
SMARTERROR_STATUS = "smarterror"
LASTSMARTERROR_STATUS = "lastsmarterror"
CAPSHORTAGE_STATUS = "capshortage"
FAIL_STATUS = "fail"
UNPLUG_STATUS = "unplug"

RED_BLINK_STATUS_LIST = (FAIL_STATUS, CAPSHORTAGE_STATUS,
                         SMARTERROR_STATUS, LASTSMARTERROR_STATUS)
BLUE_ON_STATUS_LIST = (NORMAL_STATUS,)
LED_OFF_STATUS_LIST = (UNPLUG_STATUS,)

DONT_CHANGE_DISK_LED_RAID_STATE = (CRASH_STATUS,)

DISK_STATUS_XPATH = "disk/status"
DISK_NAME_XPATH = "eventinfo/disk/name"


class EvhDiskLed(evhcmd.EvhCmd):
    """Disk Update Event Hnadler command implementation class."""

    def __update_disk_state(self):

        RED_COLOR = "red"
        BLUE_COLOR = "blue"
        ON_MODE = "on"
        OFF_MODE = "off"
        BLINK_MODE = "blink"
        HDDNUM_KEY = "hddnum"
        COLOR_KEY = "color"
        MODE_KEY = "mode"

        hdd_led_params_list = []

        for disk_name in SATA_HDD_DEV_LIST:
            color = ""
            mode = ""
            port_num = ""

            try:
                disk_status = util.get_disk_status(disk_name)
            except util.RequestError, e:
                if e.info == Results.DEVICE_NOT_FOUND:
                    disk_status = UNPLUG_STATUS
                else:
                    self.exception(
                        "util.get_disk_status: fail - do nothing")
                    return

            port_num = DEV_TO_PORT_NO[disk_name]

            if disk_status in RED_BLINK_STATUS_LIST:
                color = RED_COLOR
                mode = BLINK_MODE
            elif disk_status in BLUE_ON_STATUS_LIST:
                color = BLUE_COLOR
                mode = ON_MODE
            elif disk_status in LED_OFF_STATUS_LIST:
                color = BLUE_COLOR
                mode = OFF_MODE
            else:
                # 未使用ポートだが接続されている状態
                color = BLUE_COLOR
                mode = ON_MODE

            if color and mode and port_num:
                hdd_led_params_list.append({HDDNUM_KEY: port_num,
                                            COLOR_KEY: color,
                                            MODE_KEY: mode})
        util.change_hdd_led(hdd_led_params_list)
        return

    def __event_handler(self, entry):
        """Handle "event" request."""

        eventinfo = entry.get().get_node(EVENTINFO_NAME)
        if eventinfo.get_attr_value(CATEGORY_ATTR) == DISK_CATEGORY:
            if eventinfo.get_attr_value(ACTION_ATTR) == UPDATE_ACTION:
                try:
                    cur_raid_state = util.get_raid_state()
                    if cur_raid_state in DONT_CHANGE_DISK_LED_RAID_STATE:
                        entry.set_result(Results.OK)
                        return entry

                    self.__update_disk_state()
                except:
                    self.exception(
                        "__update_disk_state: fail - do nothing")
                    # Ignore error

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhDiskLed(option=evhcmd.EvhCmdOption()).handle_request()
