#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""
Handler for volume:mount event which is first event for the internal volume
after install.
"""

import os
import stat
import evh_config as config
import libevh.evhcmd as evhcmd
import libevh.util as util
import fal_info
import ionas.request as request
from product_conf.reqinfo import (Results, ReqTypes)


CATEGORY = "volume"
ACTION = "mount"
PRIORITY = "30"

VOLUME_NAME_PATH = "eventinfo/volume/name"
EVENTS_XML_TMPL = "<events>" + \
                """<eventinfo category="%s" action="%s" priority="%s"/>""" + \
                "</events>"
MNT_POINT = "mnt_point"
SHARE_PATH = "sharelist/share"
PATH_NAME = "path"
UID_NAME = "uid"
GID_NAME = "gid"

SHARE_MODE = stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO

SWAP_FILE_NAME = "swap.bin"


class FirstMountHandler(evhcmd.EvhCmd):
    """Event handler to process for first mount of the internal volume."""

    def __get_share_list(self, root_path):
        """Return target share xml node list."""

        share_list = []
        try:
            shares_node = self.send_entry(request.Entry(ReqTypes.LOAD),
                                fal_info.FAL_SHARES_PATH).get()

            for share in shares_node.get_node_list(SHARE_PATH):
                if share.get_value(PATH_NAME, "").startswith(root_path):
                    share_list.append(share)
        except:
            self.exception("Can't get share list!!")

        return share_list

    def __try_create_directories_for_share(self, root_path):
        """Try creating directories for share under "root_path"."""

        try:
            for share in self.__get_share_list(root_path):
                path = share.get_value(PATH_NAME)

                self.warning("check : " + path + "\n")
                if not os.path.exists(path):
                    os.makedirs(path)
                    os.chmod(path, SHARE_MODE)
                    self.warning("chmod ok : " + path + "\n")
                    try:
                        uid = int(share.get_value(UID_NAME))
                        gid = int(share.get_value(GID_NAME))
                        self.warning(
                            "uid:" + str(uid) + "  gid:" + str(gid) + "\n")
                        os.chown(path, uid, gid)
                    except:
                        self.exception("catch excption")
                        pass
        except:
            self.exception("catch excption2")
            pass

    def __try_remove_swap_file(self, root_path):
        """Try removing swap file."""

        try:
            swap_file_path = os.path.join(root_path, SWAP_FILE_NAME)
            if os.path.exists(swap_file_path):
                os.remove(swap_file_path)
        except:
            self.exception("swap file remove failed.")
            pass

    def __get_self_handler_real_path(self):
        """Return the self handler real path."""

        return util.get_handler_real_path(__file__)

    def __unregist_self(self, handler, category, action, priority):
        """Unregist the self handler."""

        req_xml = EVENTS_XML_TMPL % (category, action, priority)
        entry = request.Entry(ReqTypes.UNREGISTER,
                        req_xml, file_name=handler)
        self.send_entry(entry, fal_info.FAL_EVENTS_PATH)

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            # Target is internal volume.
            target_vol = entry.get().get_node(VOLUME_NAME_PATH).get_text()
            if target_vol == config.INTERNAL_VOL:

                handler = self.__get_self_handler_real_path()
                self.__try_create_directories_for_share(
                    config.VOL_INFO[target_vol][MNT_POINT])
                self.__try_remove_swap_file(
                    config.VOL_INFO[target_vol][MNT_POINT])
                self.__unregist_self(handler, CATEGORY, ACTION, PRIORITY)
        except:
            # Non target events
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)
        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    FirstMountHandler(option=evhcmd.EvhCmdOption()).handle_request()
