#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling Lock event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
from fal_info import (FAL_VOLCTRL_PATH)
from iobase.exception import (InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
ENCRYPT_ATTR = "encrypt"

VOLUME_CATEGORY = "volume"
UNMOUNT_ACTION = "unmount"

VOLUME_NAME_XPATH = "eventinfo/volume/name"

STOP_ENC_REQ_XML_TMPL = \
    '<volctrl>' + \
    ' <option operation="encstop"/>' + \
    ' <volume>' + \
    '  <name type="volume">%s</name>' + \
    ' </volume>' + \
    '</volctrl>'


class EvhLock(evhcmd.EvhCmd):
    """Lock Event Hnadler command implementation class."""

    class __StopEncError(Exception):

        pass

    def __validate_event(self, entry):

        if VOLUME_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME)\
            .is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        if UNMOUNT_ACTION != entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            raise InvalidArgError

        return

    def __send_request(self, req_type, target_module, child_node=""):

        entry = request.Entry(req_type, child_node)
        return self.send_entry(entry, target_module)

    def __stop_enc_volume(self, vol_name):

        ctrl_entry = request.Entry(ReqTypes.CONTROL,
                                   STOP_ENC_REQ_XML_TMPL % vol_name)
        result_entry = self.send_entry(ctrl_entry, FAL_VOLCTRL_PATH)

        if result_entry.get_result() != Results.OK:
            raise self.__StopEncError
        return

    def __is_encrypted_volume(self, volume):

        volctrl_node = util.fal_volctrl_query(volume)
        try:
            return volctrl_node.get_node("volume/filesystem").\
                       is_set_to_true(ENCRYPT_ATTR)
        except:
            return False

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""
        self.__validate_event(entry)

        vol_name = entry.get().get_node(VOLUME_NAME_XPATH).get_text()

        try:
            self.__stop_enc_volume(vol_name)
        except:
            entry.set_result(Results.SET_ERROR)
            return entry

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhLock(option=evhcmd.EvhCmdOption()).handle_request()
