#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling Announce event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import fal_info
from xml.sax.saxutils import escape
from iobase.exception import (NotFoundError, InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)
from iobase.xmldata import XmlNode
from datetime import datetime


MESSAGES_XML_FILE = "messages.xml"

EVENTINFO_NAME = "eventinfo"
LANDISKINFO_NAME = "landiskinfo"

CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
TYPE_ATTR = "type"

EVENTLOG_CATEGORY = "eventlog"
ADD_ACTION = "add"

TARGET_LOG_XPATH = "eventinfo/eventlog/log"
MESSAGESS_XPATH = "eventinfo/eventlog/messages"

LANDISKEVENT_XML_BASE = \
    '<landiskevent update="%(date)s" version="%(ver)s">' + \
    '<event type="%(type)s" category="%(category)s">' + \
    '<code>%(code)s</code>' + \
    '</event>' + \
    '</landiskevent>'

URL_ACCESS_SEND_REQ_XML_TMPL = \
    '<urlaccess>' + \
    '<url>%(url)s</url>' + \
    '<data>%(data)s</data>' + \
    '</urlaccess>'

LANDISKEVENT_XML_VER = "1.00"


class EvhNarsusHandler(evhcmd.EvhCmd):
    """Event log saved Event Hnadler for Narsus"""

    class __GetError(Exception):
        """Get error"""
        pass

    class __SendError(Exception):
        """Send error"""
        pass

    def __validate_target_event(self, entry):

        if EVENTLOG_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if ADD_ACTION != entry.get().get_node(EVENTINFO_NAME).\
                             get_attr_value(ACTION_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        return

    def __is_narsus_enable(self):

        narsus_node = \
            self.send_entry(request.Entry(ReqTypes.LOAD),
                            fal_info.FAL_NARSUS_PATH).get()

        if narsus_node.get_node("service").is_enabled():
            return True
        return False

    def __get_event_notify_dst_url(self):

        EVENT_URL_XPATH = "./urllist/url[@type='event']"

        narsus_node = self.send_entry(request.Entry(ReqTypes.QUERY),
                                      fal_info.FAL_NARSUS_PATH).get()
        return narsus_node.get_node(EVENT_URL_XPATH).get_text()

    def __is_narsus_notify(self, code):

        try:
            messageinfo_node = self.messages_node.get_node(
                    'messageinfolist/messageinfo[code="%s"]' % code)
        except:
            return False
        try:
            return messageinfo_node.get_node('noticeinfo/narsus').is_enabled()
        except:
            return False

    def __get_event_type(self, code):

        xpath = \
            'messageinfolist/messageinfo[code="%s"]/noticeinfo/narsus' % code

        return self.messages_node.get_node(xpath).get_attr_value(TYPE_ATTR)

    def __get_category_code(self, msg_code):
        """Return the category code for the "msg_code"."""

        MESSAGE_CODE_UNIT = 10000
        CATEGORY_CODE_UNIT = 100
        UNKNOWN_CATEGORY = "0"

        try:
            if not isinstance(msg_code, int):
                msg_code = int(msg_code)
            return str((msg_code / MESSAGE_CODE_UNIT / CATEGORY_CODE_UNIT) *
                        CATEGORY_CODE_UNIT)
        except:
            return UNKNOWN_CATEGORY

    def __get_current_datetime(self):
        """Get current datetime."""

        now = datetime.now()
        return now.strftime("%Y/%m/%d %H:%M:%S")

    def __get_landiskinfo_node(self):

        get_entry = request.Entry(ReqTypes.GET)
        result_entry = self.send_entry(get_entry, fal_info.FAL_NARSUS_PATH)
        if result_entry.get_result() != Results.OK:
            raise self.__GetError
        return result_entry.get().get_node(LANDISKINFO_NAME)

    def __get_landiskevent_node(self, log_node):

        CODE_NAME = "code"
        ARGLIST_NAME = "arglist"

        code = log_node.get_value(CODE_NAME, None)
        if code is None:
            raise self.__GetError

        params = {"date": self.__get_current_datetime(),
                  "ver": LANDISKEVENT_XML_VER,
                  "type": self.__get_event_type(code),
                  "category": self.__get_category_code(code),
                  "code": code}

        landiskevent_node = XmlNode(LANDISKEVENT_XML_BASE % params)

        # arglist
        try:
            arglist_node = log_node.get_node(ARGLIST_NAME)
        except NotFoundError:
            arglist_node = None

        if arglist_node is not None:
            landiskevent_node.get_node("event").append_child(arglist_node)

        # landiskinfo
        landiskinfo_node = self.__get_landiskinfo_node()
        landiskevent_node.append_child(landiskinfo_node)

        return landiskevent_node

    def __urlaccess_send(self, url, data):

        data = escape(data)
        req_xml = URL_ACCESS_SEND_REQ_XML_TMPL % {"url": url, "data": data}
        get_entry = request.Entry(ReqTypes.GET, req_xml)
        result_entry = self.send_entry(get_entry, fal_info.FAL_URLACCESS_PATH)

        if result_entry.get_result() != Results.OK:
            raise self.__SendError

        response_node = XmlNode(result_entry.get().get_node("data").get_text())
        status = response_node.get_node("status").get_text()
        if status != "0":
            raise self.__SendError

        return

    def __urlaccess_send_with_file(self, url, file_name):

        req_xml = URL_ACCESS_SEND_REQ_XML_TMPL % {"url": url, "data": ""}
        get_entry = request.Entry(ReqTypes.GET, req_xml)
        result_entry = self.send_entry_with_file(get_entry,
                                                 fal_info.FAL_URLACCESS_PATHi,
                                                 file_name)
        if result_entry.get_result() != Results.OK:
            raise self.__SendError
        return

    def __send_landisk_event(self, log_node):

        url = self.__get_event_notify_dst_url()
        landiskevent_node = self.__get_landiskevent_node(log_node)
        self.__urlaccess_send(url, str(landiskevent_node))

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            self.__validate_target_event(entry)

            if not self.__is_narsus_enable():
                entry.set_result(Results.OK)
                return entry

            try:
                log_node = entry.get().get_node(TARGET_LOG_XPATH)
            except NotFoundError:
                entry.set_result(Results.OK)
                return entry

            self.messages_node = entry.get().get_node(MESSAGESS_XPATH)

            code = log_node.get_value('code', None)
            if not code:
                raise InvalidArgError
            if not self.__is_narsus_notify(code):
                entry.set_result(Results.OK)
                return entry

            # NarSuSサーバーへのデータ送信
            try:
                self.__send_landisk_event(log_node)
            except self.__SendError:
                self.exception("fail to send landisk_event !!")
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.messages_node = None
        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhNarsusHandler(option=evhcmd.EvhCmdOption()).handle_request()
