#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling port mode modify event"""

import os
import libevh.evhcmd as evhcmd
from ionas.request import Entry
from iobase.exception import NotFoundError
from pal_info import PAL_PROFTPD_PATH
from fal_info import (FAL_VOLCTRL_PATH, ReqTypes, Results)
import evh_config as config


FILE_DIR_LIST = [config.PROFTPD_PART_DIR, config.PROFTPD_ANON_PART_DIR]
EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"

USB_ORIG_SUFFIX = ".real"
#See pal_proftpd
VOL_ENTRY = "<volctrl><volume><name type='volume'>%s</name></volume></volctrl>"


class Ports(evhcmd.EvhCmd):
    """Port Mode Event Hnadler command implementation class."""

    def __get_vol_status(self, vol):
        try:
            entry = Entry(ReqTypes.QUERY, VOL_ENTRY % vol)
            response = self.send_entry(entry, FAL_VOLCTRL_PATH)
            return response.get().get_node("volume/status").get_text()
        except:
            return "disconnected"

    def __mount_or_unmount(self, type, vol):
        vol_status = self.__get_vol_status(vol)
        if type == "mount":
            if vol_status == "mounted":
                return Results.OK
        else:
            if vol_status == "unmounted" or vol_status == "disconnected":
                return Results.OK
        dev_name = config.VOL_INFO[vol]["dev"]
        dev_path = os.path.join(config.DEV_DIR, dev_name)
        if not os.path.exists(dev_path):
            return Results.OK

        entry = Entry(type, VOL_ENTRY % vol)
        response = self.send_entry(entry, FAL_VOLCTRL_PATH)
        return response.get_result()

    def __remove_usb_share_link(self, port):
        for dir in FILE_DIR_LIST:
            for suffix in [".ro", ".rw"]:
                file_name = port + suffix
                file_path = os.path.join(dir, port, file_name)
                if not os.path.exists(file_path):
                    continue
                os.remove(file_path)
        return

    def __create_usb_share_link(self, port):
        for dir in FILE_DIR_LIST:
            for suffix in [".ro", ".rw"]:
                file_name = port + suffix
                file_path = os.path.join(dir, port, file_name)
                if os.path.exists(file_path):
                    continue
                src_file_path = file_path + USB_ORIG_SUFFIX
                if os.path.exists(src_file_path):
                    os.symlink(src_file_path, file_path)
        return

    def __update_request(self, entry):
        event_data = entry.get().get_node(EVENTINFO_NAME)
        usblist = event_data.get_node_list("./ports/usblist/usb")
        if len(usblist) < 1:
            entry.set_result(Results.OK)
            return entry
        for usb in usblist:
            port_num = usb.get_attr_value("port")
            enable_mode = usb.get_node(
                            "modelist/mode[@enable='true']").get_text()
            if enable_mode == "share":
                self.__create_usb_share_link("usb" + port_num)
                result = self.__mount_or_unmount(
                            ReqTypes.MOUNT, "usb" + port_num)
            else:
                self.__remove_usb_share_link("usb" + port_num)
                result = self.__mount_or_unmount(
                            ReqTypes.UNMOUNT, "usb" + port_num)
            if result != Results.OK:
                entry.set_result(result)
                return entry
        return self.__update_conf_file()

    def __update_conf_file(self):
        return self.send_entry(Entry(ReqTypes.UPDATE), PAL_PROFTPD_PATH)

    def __event_handler(self, entry):
        """Handle "event" request."""

        category = entry.get().get_node(
                    EVENTINFO_NAME).get_attr_value(CATEGORY_ATTR)
        if category != "ports":
            entry.set_result(Results.EVENT_ERROR)
            return entry
            
        update_result = self.__update_request(entry)
        if update_result.get_result() != Results.OK:
            entry.set_result(update_result.get_result())
            return entry

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    Ports(option=evhcmd.EvhCmdOption()).handle_request()
