#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling MKFS event"""

from __future__ import with_statement
import os
import ionas.request as request
import libevh.evhcmd as evhcmd
import evh_config as config
import libevh.util as util
from iobase.xmldata import (XmlNode)
from fal_info import (FAL_QUOTA_PATH, FAL_USERS_PATH, FAL_GROUPS_PATH,
                        FAL_EVENTS_PATH)
from iobase.exception import (InvalidArgError, NotFoundError)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf import *
from product_conf.devinfo import *

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

EVENTINFO_VOLUME_PATH = EVENTINFO_NAME + "/volume"
MOUNTFAIL_ATTR = "mountfail"

VOLUME_CATEGORY = "volume"
MKFS_ACTION = "mkfs"
MOUNT_ACTION = "mount"
PRIORITY = "50"

PYC_SUFFIX = ".pyc"
PY_SUFFIX = ".py"

TMP_CUR_QUOTA_XML_FILE = \
    os.path.join(config.TEMP_DIR, "quotanodebeforemkfs")

EVENTS_XML_TMPL = "<events>" + \
                """<eventinfo category="%s" action="%s" priority="%s"/>""" + \
                "</events>"


class EvhQuotaRestore(evhcmd.EvhCmd):
    """Init Share Event Hnadler command implementation class."""

    class __RequestError(Exception):
        "RequestError"
        pass

    #
    # Utility method
    #

    def __get_action(self, entry):
        """Return the action of eventinfo in "entry"."""

        return entry.get().get_node(EVENTINFO_NAME).get_attr_value(ACTION_ATTR)

    def __validate_target_event(self, entry):

        if VOLUME_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                  get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if not self.__get_action(entry) in (MKFS_ACTION, MOUNT_ACTION):
            raise InvalidArgError

        return

    def __send_request(self, type, target_module, child_node=""):

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __is_previous(self, entry):

        return entry.get().get_node(
                   EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR)

    def __is_mountfail(self, entry):

        try:
            return entry.get().get_node(
                        EVENTINFO_VOLUME_PATH).is_set_to_true(MOUNTFAIL_ATTR)
        except:
            return False

    def __get_self_handler_real_path(self):
        """Return the self handler real path."""

        (base, suffix) = os.path.splitext(__file__)
        handler_path = __file__
        if suffix == PYC_SUFFIX:
            handler_path = base + PY_SUFFIX

        if os.path.islink(handler_path):
            return os.path.abspath(os.path.join(os.path.dirname(handler_path),
                        os.readlink(handler_path)))
        return os.path.abspath(handler_path)


    def __regist_self(self, category, action, priority):
        """Regist the self handler."""

        req_xml = EVENTS_XML_TMPL % (category, action, priority)
        handler = self.__get_self_handler_real_path()
        entry = request.Entry(ReqTypes.REGISTER,
                        req_xml, file_name=handler)
        self.send_entry(entry, FAL_EVENTS_PATH)

    def __unregist_self(self, category, action, priority):
        """Unregist the self handler."""

        req_xml = EVENTS_XML_TMPL % (category, action, priority)
        handler = self.__get_self_handler_real_path()
        entry = request.Entry(ReqTypes.UNREGISTER,
                        req_xml, file_name=handler)
        self.send_entry(entry, FAL_EVENTS_PATH)

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            self.__validate_target_event(entry)

            internal_mnt_point = \
                    config.VOL_INFO[util.get_internal_vol_name()]["mnt_point"]

            if self.__get_action(entry) == MKFS_ACTION:
                if not util.is_internal_dev_name(entry):
                    entry.set_result(Results.OK)
                    return entry

                if not self.__is_previous(entry):
                    self.__regist_self(VOLUME_CATEGORY, MOUNT_ACTION, PRIORITY)
            elif self.__get_action(entry) == MOUNT_ACTION:
                if (not util.is_internal_vol_name(entry) or
                        self.__is_mountfail(entry)):

                    entry.set_result(Results.OK)
                    return entry
                util.set_quota_limit(util.get_internal_vol_name())
                self.__unregist_self(VOLUME_CATEGORY, MOUNT_ACTION, PRIORITY)

        except:
            self.exception("fail to mkfs event")
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhQuotaRestore(option=evhcmd.EvhCmdOption()).handle_request()
