#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling "raidrebuild" event"""

import re
import time
import libevh.evhcmd as evhcmd
import libevh.util as util
import fal_msginfo as msginfo
from iobase.exception import (InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)
from evh_config import (INTERNAL_DEV, MIRROR_DEV, TARGET_RAID_DEV_LIST)

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
RAIDREBUILD_CATEGORY = "raidrebuild"
PROGRESS_ACTION = "progress"
STATE_ATTR = "state"


RAID_NAME_XPATH = "eventinfo/progress/name"
PROGRESS_XPATH = "eventinfo/progress"

START_STATE = "start"
COMPLETE_STATE = "complete"
PROGRESS_STATE = "progress"

LOG_CODE_MAP = {
        START_STATE: {
                INTERNAL_DEV: msginfo.Msg.MSG_RAID_INTERNAL_REBUILD_START,
                MIRROR_DEV: msginfo.Msg.MSG_RAID_MIRROR_REBUILD_START,
            },
        PROGRESS_STATE: {
                INTERNAL_DEV: msginfo.Msg.MSG_RAID_INTERNAL_REBUILD_PROGRESS,
                MIRROR_DEV: msginfo.Msg.MSG_RAID_MIRROR_REBUILD_PROGRESS,
            },
        COMPLETE_STATE: {
                INTERNAL_DEV: msginfo.Msg.MSG_RAID_INTERNAL_REBUILD_END,
                MIRROR_DEV: msginfo.Msg.MSG_RAID_MIRROR_REBUILD_END,
            },
    }


class RaidRebuild(evhcmd.EvhCmd):
    """UsbdPlug Event Hnadler command implementation class."""

    def __validate_event_info(self, entry):

        if RAIDREBUILD_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError("Previous is True !!")

        action = entry.get().get_node(EVENTINFO_NAME).\
            get_attr_value(ACTION_ATTR)

        if action != PROGRESS_ACTION:
            raise InvalidArgError("Invalid action !!" + action)

        return

    def __state_start(self, entry, raid_name):
        """Rebuild start event handler"""

        if raid_name in LOG_CODE_MAP[START_STATE]:
            util.save_event_log(LOG_CODE_MAP[START_STATE][raid_name])

        util.dispatch_event(util.RAID_CATEGORY, util.UPDATE_ACTION)
        return

    def __state_progress(self, entry, raid_name):
        """Rebuild progress event handler"""

        progress = \
            entry.get().get_node(PROGRESS_XPATH).get_attr_value(STATE_ATTR)
        match = re.match("(.*)%", progress)
        if not match:
            return
        progress = match.group(1)
        if raid_name in LOG_CODE_MAP[PROGRESS_STATE]:
            util.save_event_log(LOG_CODE_MAP[PROGRESS_STATE][raid_name],
                            {"progress": progress})
        return

    def __state_complete(self, entry, raid_name):
        """Rebuild complete event handler"""

        raid_name = entry.get().get_value(RAID_NAME_XPATH)
        if raid_name in LOG_CODE_MAP[COMPLETE_STATE]:
            util.save_event_log(LOG_CODE_MAP[COMPLETE_STATE][raid_name])
        util.dispatch_event(util.RAID_CATEGORY, util.UPDATE_ACTION)
        for idx in range(1, 4):
            util.play_melody("raidok")
            time.sleep(0.5)
        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self.__validate_event_info(entry)

        raid_name = entry.get().get_value(RAID_NAME_XPATH)
        if not raid_name in TARGET_RAID_DEV_LIST:
            # Ignore event
            entry.set_result(Results.OK)
            return entry

        state = \
            entry.get().get_node(PROGRESS_XPATH).get_attr_value(STATE_ATTR)

        if re.match("[0-9]+%", state):
            state = PROGRESS_STATE

        state_func = \
            {START_STATE: self.__state_start,
             COMPLETE_STATE: self.__state_complete,
             PROGRESS_STATE: self.__state_progress}

        state_func[state](entry, raid_name)
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    RaidRebuild(option=evhcmd.EvhCmdOption()).handle_request()
