#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling RAID update event"""


from __future__ import with_statement
import iobase.fileut as fileut
import fal_info as info
import fal_msginfo as msginfo
import product_conf
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf.devinfo import (SATA_HDD_DEV_LIST, DEV_TO_PORT_NO)

import libevh.evhcmd as evhcmd
import libevh.util as util
from evh_config import (INTERNAL_VOL, INTERNAL_DEV, MIRROR_DEV)


EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"

RAID_CATEGORY = "raid"
UPDATE_ACTION = "update"

RAIDVOL_CATEGORY = "raidvol"

RUNNING_STATUS = "running"
STOP_STATUS = "stop"
REBUILDING_STATUS = "rebuilding"
DEGRADE_STATUS = "degrade"
FAIL_STATUS = "fail"
CRASH_STATUS = "crash"
MOUNTFAIL_STATUS = "mountfail"
KEYWAITING_STATUS = "keywaiting"
PREPARING_STATUS = "preparing"
MOUNTED_STATUS = "mounted"

RUNNING_STATE = "running"
STOP_STATE = "stop"
DEGRADE_STATE = "degrade"
FAIL_STATE = "fail"
CRASH_STATE = "crash"
REBUILDING_STATE = "rebuilding"
MOUNTFAIL_STATE = "mountfail"
KEYWAITING_STATE = "keywaiting"

RAID_STATUS_XPATH = "raid/status"
VOLUME_STATUS_XPATH = "volume/status"

NORMAL_DISK_STATUS = "normal"
SMARTERROR_DISK_STATUS = "smarterror"
LASTSMARTERROR_DISK_STATUS = "lastsmarterror"
CAPSHORTAGE_DISK_STATUS = "capshortage"
FAIL_DISK_STATUS = "fail"
UNPLUG_DISK_STATUS = "unplug"

CRASH_HDD_LED_TABLE = {
    "red_on_status_list":
        (NORMAL_DISK_STATUS,),
    "red_blink_status_list":
        (FAIL_DISK_STATUS, CAPSHORTAGE_DISK_STATUS,
         SMARTERROR_DISK_STATUS, LASTSMARTERROR_DISK_STATUS),
    "led_off_status_list":
        (UNPLUG_DISK_STATUS,)}
RUNNING_HDD_LED_TABLE = {
    "blue_on_status_list":
        (NORMAL_DISK_STATUS,)}

LOCK_NAME = "evh_raidupdate"


class EvhRaidUpdate(evhcmd.EvhCmd):
    """RAID Update Event Hnadler command implementation class."""

    def __get_internal_raid_status(self):
        """Return the internal RAID status."""

        try:
            if INTERNAL_DEV:
                raid_info = util.fal_raidctrl_query(INTERNAL_DEV)
                return raid_info.get_node(RAID_STATUS_XPATH).get_text()
            else:
                return STOP_STATUS
        except:
            return STOP_STATUS

    def __get_mirror_raid_status(self):
        """Return the mirroring RAID status."""

        try:
            raid_info = util.fal_raidctrl_query(MIRROR_DEV)
            return raid_info.get_node(RAID_STATUS_XPATH).get_text()
        except:
            return STOP_STATUS

    def __get_internal_volume_status(self):
        """Return the internal volume status."""

        try:
            raid_info = util.fal_volctrl_query(INTERNAL_VOL)
            return raid_info.get_node(VOLUME_STATUS_XPATH).get_text()
        except:
            return PREPARING_STATUS

    def __update_raidvol_state(self, cur_raidvol_state,
            internal_volume_status):
        """Update "raidvol" state and return it."""

        if (internal_volume_status != KEYWAITING_STATUS or
            cur_raidvol_state == KEYWAITING_STATE):

            util.stop_melody(info.BUZZER_LOCKMODE)

        if internal_volume_status == PREPARING_STATUS:
            return PREPARING_STATUS

        if internal_volume_status == MOUNTFAIL_STATUS:
            if cur_raidvol_state != MOUNTFAIL_STATE:
                util.set_state_manager(RAIDVOL_CATEGORY, MOUNTFAIL_STATE)
            return MOUNTFAIL_STATE

        if internal_volume_status == KEYWAITING_STATUS:

            if not util.is_tried_first_mount():
                return PREPARING_STATUS

            if cur_raidvol_state != KEYWAITING_STATE:
                util.set_state_manager(RAIDVOL_CATEGORY, KEYWAITING_STATE)
                util.save_event_log(msginfo.Msg.ERR_LOCKKEY_SECURITY_LOCK_MODE)
                util.play_melody(info.BUZZER_LOCKMODE)
            return KEYWAITING_STATE

        if cur_raidvol_state != RUNNING_STATE:
            util.set_state_manager(RAIDVOL_CATEGORY, RUNNING_STATE)
            self.__change_hdd_led(RUNNING_HDD_LED_TABLE)
        return RUNNING_STATE

    def __change_hdd_led(self, led_table):

        RED_COLOR = "red"
        BLUE_COLOR = "blue"
        ON_MODE = "on"
        BLINK_MODE = "blink"
        HDDNUM_KEY = "hddnum"
        COLOR_KEY = "color"
        MODE_KEY = "mode"

        hdd_led_params_list = []

        for disk_name in SATA_HDD_DEV_LIST:
            color = ""
            mode = ""
            port_num = ""

            try:
                disk_status = util.get_disk_status(disk_name)
            except util.RequestError, e:
                if e.info == Results.DEVICE_NOT_FOUND:
                    disk_status = UNPLUG_DISK_STATUS
                else:
                    raise e

            port_num = DEV_TO_PORT_NO[disk_name]

            if disk_status in led_table.get("red_blink_status_list", []):
                color = RED_COLOR
                mode = BLINK_MODE
            elif disk_status in led_table.get("red_on_status_list", []):
                color = RED_COLOR
                mode = ON_MODE
            elif disk_status in led_table.get("blue_on_status_list", []):
                color = BLUE_COLOR
                mode = ON_MODE
            elif disk_status in led_table.get("led_off_status_list", []):
                pass

            if color and mode and port_num:
                hdd_led_params_list.append({HDDNUM_KEY: port_num,
                                            COLOR_KEY: color,
                                            MODE_KEY: mode})
        util.change_hdd_led(hdd_led_params_list)
        return

    def __update_raid_raidvol_state(self):
        """Update "raid" and "raidvol" state."""

        self.debug("Update raid and raidvol state")

        cur_raid_state = util.get_raid_state()
        cur_raidvol_state = util.get_raidvol_state()
        internal_raid_status = self.__get_internal_raid_status()
        mirror_raid_status = self.__get_mirror_raid_status()
        internal_volume_status = self.__get_internal_volume_status()
        if not INTERNAL_DEV and internal_volume_status == MOUNTED_STATUS:
            internal_raid_status = RUNNING_STATUS

        self.debug("Current raid state: " + cur_raid_state)
        self.debug("Current raidvol state: " + cur_raidvol_state)
        self.debug("Internal raid status: " + internal_raid_status)
        self.debug("Mirror raid status: " + mirror_raid_status)
        self.debug("Internal volume status: " + internal_volume_status)

        if not self.__is_able_to_raid_update(internal_volume_status):
            return

        if (internal_volume_status != KEYWAITING_STATUS and
            (mirror_raid_status == STOP_STATUS or
                internal_raid_status == STOP_STATUS)):

            if cur_raid_state == CRASH_STATE:
                return

            util.set_state_manager(RAID_CATEGORY, STOP_STATE)
            return

        if mirror_raid_status == FAIL_STATUS:
            if cur_raid_state != FAIL_STATE:
                util.set_state_manager(RAID_CATEGORY, FAIL_STATE)
                util.set_state_manager(RAIDVOL_CATEGORY, FAIL_STATE)
                util.save_event_log(msginfo.Msg.ERR_RAID_FAIL)
                util.play_melody(info.BUZZER_RAIDFAIL)
            return

        if mirror_raid_status == CRASH_STATUS:
            if cur_raid_state != CRASH_STATE:
                util.set_state_manager(RAID_CATEGORY, CRASH_STATE)
                util.set_state_manager(RAIDVOL_CATEGORY, FAIL_STATE)
                util.save_event_log(msginfo.Msg.ERR_RAID_CRASH)
                self.__change_hdd_led(CRASH_HDD_LED_TABLE)
                util.play_melody(info.BUZZER_RAIDCRASH)
            else:
                # diskが抜けて崩壊後、
                # そのdiskが挿された場合等の為
                # LEDのみ更新
                self.__change_hdd_led(CRASH_HDD_LED_TABLE)
            return

        if (mirror_raid_status == DEGRADE_STATUS or
            internal_raid_status == DEGRADE_STATUS):

            if cur_raid_state != DEGRADE_STATE:
                util.set_state_manager(RAID_CATEGORY, DEGRADE_STATE)
                util.save_event_log(msginfo.Msg.ERR_RAID_DEGRADE)
                util.play_melody(info.BUZZER_RAIDFAIL)

            self.__update_raidvol_state(cur_raidvol_state,
                                        internal_volume_status)
            return

        if cur_raid_state != RUNNING_STATE:
            if cur_raid_state in (CRASH_STATE, FAIL_STATE, DEGRADE_STATE):
                util.save_event_log(msginfo.Msg.MSG_RAID_RECOVER)
            util.set_state_manager(RAID_CATEGORY, RUNNING_STATE)

            # stop RAID error melody.
            util.stop_melody(info.BUZZER_RAIDFAIL)
            util.stop_melody(info.BUZZER_RAIDCRASH)

            util.play_melody(info.BUZZER_OK)

        if (mirror_raid_status == REBUILDING_STATUS or
            internal_raid_status == REBUILDING_STATUS):

            if (self.__update_raidvol_state(cur_raidvol_state,
                        internal_volume_status) == RUNNING_STATE):

                if cur_raid_state != REBUILDING_STATE:
                    util.set_state_manager(RAIDVOL_CATEGORY, REBUILDING_STATE)

            return

        self.__update_raidvol_state(cur_raidvol_state, internal_volume_status)
        return

    def __is_able_to_raid_update(self, internal_volume_status):

        if internal_volume_status == KEYWAITING_STATUS and \
               util.is_boot_complete():

            return False

        return True

    def __event_handler(self, entry):
        """Handle "event" request."""

        with fileut.FLock(product_conf.LOCK_DIR, LOCK_NAME, True):
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == RAID_CATEGORY:
                if eventinfo.get_attr_value(ACTION_ATTR) == UPDATE_ACTION:
                    try:
                        self.__update_raid_raidvol_state()

                    except:
                        self.exception(
                            "__update_raid_raidvol_state: fail - do nothing")
                        # Ignore error

            entry.set_result(Results.OK)
            return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhRaidUpdate(evhcmd.EvhCmdOption()).handle_request()
