#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling MKFS event"""

from __future__ import with_statement
import os
import libevh.evhcmd as evhcmd
import evh_config as config
import libevh.util as util
from iobase.xmldata import (XmlNode)
from fal_info import (FAL_SHARES_PATH, FAL_EVENTS_PATH)
from iobase.exception import (InvalidArgError,)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf import *
from product_conf.devinfo import *
from product_conf import SUDO_CMD
from ionas.request import Request, Entry, EntryList
from evh_config import VOL_INFO, EXCLUDE_SHARE_INIT_LIST

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
SHARE_XPATH = "./sharelist/share"

VOLUME_CATEGORY = "volume"
MKFS_ACTION = "mkfs"
MOUNT_ACTION = "mount"

DEFAULT_SHARES_CONF_FILE = \
    os.path.join(config.DEFAULT_XML_CONF_DIR, config.SHARES_CONF_FILE)

EVENT_XML = "<events>" + \
              "<eventinfo category='volume' action='mount' priority='70'/>" + \
            "</events>"

SELF_HANDLER = "/mnt/data/libexec/evh/evh_share_init.py"


class Share(object):

    def _request(self, entry, fal=FAL_SHARES_PATH):
        request = Request(fal, filter_cmd=SUDO_CMD)
        response = request.request(entry)
        return response

    def _reqentry(self, entry, fal=FAL_SHARES_PATH):
        try:
            return self._request(entry, fal=fal).get_entrylist().get_entry()
        except:
            return None

    def _load(self):
        entry = Entry(ReqTypes.LOAD, "<shares />")
        entry = self._reqentry(entry)
        return entry

    def _delete(self, name):
        entry = Entry(ReqTypes.DELETE,
                      "<shares><share><name /></share></shares>")
        entry.get().get_node("share/name").set_text(name)
        entry = self._reqentry(entry)
        return entry

    def delete(self, name):
        entry = self._delete(name)
        if entry is not None and entry.get_result() == Results.OK:
            return True
        else:
            return False

    def _get_default_share_node_list(self):
        with open(DEFAULT_SHARES_CONF_FILE, "r") as fh:
            shares_node = XmlNode(fh.read())

        return shares_node.get_node_list(SHARE_XPATH)

    def _get_current_share_name_list(self):
        names = []
        for share in self._load().get().get_node_list("sharelist/share"):
            name = share.get_value("name")
            if name:
                names.append(name)

        return names

    def _create(self, exclude=[]):
        share_node_list = self._get_default_share_node_list()

        result = True

        for share_node in share_node_list:
            if share_node.get_value("name") in exclude:
                continue
            shares_node = XmlNode("shares")
            shares_node.append_child(share_node)
            entry = self._reqentry(Entry(ReqTypes.ADD, shares_node))
            if not (entry is not None and
                    entry.get_result() == Results.OK):
                result = False

        return result

    def create(self, exclude=[]):
        exclude = exclude + EXCLUDE_SHARE_INIT_LIST + \
                            self._get_current_share_name_list()
        return self._create(exclude)

    def clear(self, exclude=[]):
        exclude = exclude + EXCLUDE_SHARE_INIT_LIST

        names = self._get_current_share_name_list()
        names = [name for name in names if name not in exclude]
        deleted = []
        for name in names:
            if name in exclude:
                continue
            if self.delete(name):
                deleted.append(name)
        if deleted == names:
            return True
        else:
            return False


class EvhInitShare(evhcmd.EvhCmd):
    """Init Share Event Hnadler command implementation class."""

    #
    # Utility method
    #

    def __get_action_method(self, entry):

        category = \
            entry.get().get_node(EVENTINFO_NAME).get_attr_value(CATEGORY_ATTR)

        action = \
            entry.get().get_node(EVENTINFO_NAME).get_attr_value(ACTION_ATTR)

        is_previous = \
            entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR)

        if VOLUME_CATEGORY == category and MKFS_ACTION == action:
            if True == is_previous:
                return self.__pre_mkfs
            else:
                return self.__post_mkfs

        if VOLUME_CATEGORY == category and MOUNT_ACTION == action and \
           False == is_previous:
            return self.__create_init_share

        raise InvalidArgError

    def __is_internal_vol_name(self, entry):

        VOLUME_XPATH = "./eventinfo/volume/name"

        vol_name = entry.get().get_value(VOLUME_XPATH)

        if util.get_internal_vol_name() == vol_name:
            return True

        return False

    def __pre_mkfs(self, entry):

        if not util.is_internal_dev_name(entry):
            entry.set_result(Results.OK)
            return entry

        Share().clear()
        entry.set_result(Results.OK)
        return entry

    def __regist_or_unregist(self, type):

        self.send_entry_with_file(Entry(type, EVENT_XML),
                                  FAL_EVENTS_PATH,
                                  SELF_HANDLER)

    def __post_mkfs(self, entry):

        if not util.is_internal_dev_name(entry):
            entry.set_result(Results.OK)
            return entry

        self.__regist_or_unregist(ReqTypes.REGISTER)

        entry.set_result(Results.OK)
        return entry

    def __create_init_share(self, entry):

        if not self.__is_internal_vol_name(entry):
            entry.set_result(Results.OK)
            return entry

        try:
            Share().create()
        finally:
            self.__regist_or_unregist(ReqTypes.UNREGISTER)

        entry.set_result(Results.OK)
        return entry

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        handle_method = self.__get_action_method(entry)
        return handle_method(entry)

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhInitShare(option=evhcmd.EvhCmdOption()).handle_request()
