#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling UsbdPlug event"""

from __future__ import with_statement

import os
import re
import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
import fal_msginfo as msginfo
import fal_info
from fal_info import (FAL_VOLCTRL_PATH, FAL_RAIDCTRL_PATH,
                      FAL_BUZZER_PATH, FAL_PORTS_PATH,
                      FAL_EVENTS_PATH, FAL_LED_PATH,
                      FAL_ANNOUNCE_PATH, FAL_SERVICECTRL_PATH,
                      FAL_LOCKKEY_PATH)
from iobase.exception import (InvalidArgError)
from product_conf import (VOL_LIST, CONF_DIR)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf.devinfo import (SATA_HDD_DEV_LIST, HDD_DEV_LIST_IN_VOL,
                                  SYSTEM_RAID_DEV_LIST, LOCKKEY_VOL_LIST)
from evh_config import (INTERNAL_DEV, MIRROR_DEV, BOOT_COMPLETE_FILE_PATH)
import evh_config as config

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

RAID_NAME = "raid"
NAME_NAME = "name"
TYPE_ATTR = "type"
TYPE_PARTITION = "partition"

RAID_START_REQ_XML_TMPL = \
    '<raidctrl>' + \
    ' <option force="false" autorebuild="true"/>' + \
    '  <raid><name type="volume">%s</name></raid>' + \
    '</raidctrl>'

MOUNT_VOLUME_REQ_XML_TMPL = \
    '<volctrl>' + \
    ' <volume><name type="volume">%s</name></volume>' + \
    '</volctrl>'

PLAY_BOOT_MELODY_XML = '<buzzer><melody type="bootup"/></buzzer>'
PLAY_ERROR_MELODY_XML = '<buzzer><melody type="error"/></buzzer>'

CHECK_SMART_ERROR_XML = \
    '<diskctrl><disk><name>%s</name></disk></diskctrl>'

USB_PORT_ENABLE_MODE_XPATH = \
    "usblist/usb[@port='%s']/modelist/mode[@enable='true']"

VOLUME_STATUS_XPATH = "volume/status"

SYSTEM_CATEGORY = "system"

BOOT_ACTION = "boot"

STATUS_NAME = "status"
NAME_NAME = "name"

# For state managr
RAID_VOL_CATEGORY = "raidvol"
RAID_CATEGORY = "raid"
MOUNT_FAIL_STATE = "mountfail"
FAIL_STATE = "fail"
RUNNING_STATE = "running"
KEYWAITING_STATE = "keywaiting"

STATUS_TYPE = "status"
KEYWAITING_STATUS = "keywaiting"
LOCKKEY_STATUS = "lockkey"
LASTSMARTERROR_STATUS = "lastsmarterror"
SMARTERROR_STATUS = "smarterror"
CAPSHORTAGE_STATUS = "capshortage"
FAIL_STATUS = "fail"
OTHER_MIRROR_STATUS = "othermirror"

# extend raid result
EXTEND_SUCCESS = "success"
EXTEND_FAIL = "fail"
EXTEND_NOEXEC = "noexec"

# extend mount result
EXTEND_MOUNT_SUCCESS = "success"
EXTEND_MOUNT_FAIL = "fail"

EXTEND_KEY = "extend"
LEVEL_KEY = "level"
STATUS_KEY = "status"
AUTOREBUILD_KEY = "autorebuild"

FAIL_LOG_CODE_MAP = {
        "satad1": msginfo.Msg.ERR_DISK_SATA1_DETECT_ERROR,
        "satad2": msginfo.Msg.ERR_DISK_SATA2_DETECT_ERROR,
        "satad3": msginfo.Msg.ERR_DISK_SATA3_DETECT_ERROR,
        "satad4": msginfo.Msg.ERR_DISK_SATA4_DETECT_ERROR,
        "satad5": msginfo.Msg.ERR_DISK_SATA5_DETECT_ERROR,
        "satad6": msginfo.Msg.ERR_DISK_SATA6_DETECT_ERROR,
        "usbd3": msginfo.Msg.ERR_DISK_USB3_DETECT_ERROR,
    }

EXTEND_VOLUME_LOG_CODE_MAP = {
        "vfsext_6": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_6", None),
        "vfsext_5": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_5", None),
        "vfsext_4": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_4", None),
        "vfsext_3": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_3", None),
        "vfsext_2": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_2", None),
        "vfsext_1": getattr(msginfo.Msg, "MSG_RAID_MODE_AUTO_EXTEND_VFSEXT_1", None),
    }

EXTEND_VOLUME_ERR_LOG_CODE_MAP = {
        "vfsext_6": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_6", None),
        "vfsext_5": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_5", None),
        "vfsext_4": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_4", None),
        "vfsext_3": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_3", None),
        "vfsext_2": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_2", None),
        "vfsext_1": getattr(msginfo.Msg, "ERR_RAID_MODE_AUTO_EXTEND_VFSEXT_1", None),
    }

DISK_SERVICECTRL_REQ_XML = """<servicectrl>
    <categorylist>
        <category><name>disk</name></category>
    </categorylist>
</servicectrl>
"""

CUR_ARRAY_NUM_TMPL = os.path.join(CONF_DIR,"raid", "%s", "cur_array_num")


class EvhSystemBoot(evhcmd.EvhCmd):
    """UsbdPlug Event Hnadler command implementation class."""

    class __StartError(Exception):
        "Start Error"
        pass

    class __MountError(Exception):
        "Mount Error"
        pass

    class __RequestError(Exception):
        "Request Error"
        pass

    class __KeyWaiting(Exception):
        "KeyWaiting"
        pass

    def __get_event_action(self, entry):

        if SYSTEM_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        is_previous = \
            entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR)

        if BOOT_ACTION == entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            if not is_previous:
                return BOOT_ACTION

    def __send_request(self, type, target_module, child_node=""):

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __change_status_led(self, state):
        """state must be specified 'on' or 'blink' or 'off'"""

        CHG_STATE_LED_REQ_XML = \
            '<led><status color="green">%s</status></led>'

        self.__send_request(ReqTypes.CONTROL,
                            FAL_LED_PATH,
                            CHG_STATE_LED_REQ_XML % state)
        return

    def __stop_disk_service(self):

        self.__send_request(ReqTypes.STOP,
                            FAL_SERVICECTRL_PATH,
                            DISK_SERVICECTRL_REQ_XML)

    def __raid_start(self):

        result_map = {EXTEND_KEY: {LEVEL_KEY: "",
                                   STATUS_KEY: ""}}

        result_entry = self.__send_request(ReqTypes.START,
                                           FAL_RAIDCTRL_PATH,
                                           RAID_START_REQ_XML_TMPL % \
                                               util.get_internal_vol_name())
        result_code = result_entry.get_result()

        result_map[EXTEND_KEY][LEVEL_KEY] = \
            result_entry.get().get_value("result/extend/level")

        result_map[EXTEND_KEY][STATUS_KEY] = \
            result_entry.get().get_value("result/extend/status")

        if Results.OK == result_code:
            return result_map

        # autorebuildで容量不足のHDDがあった
        if Results.CAP_SHORTAGE == result_code:
            return result_map

        raise self.__StartError

    def __load_port_config(self):

        return self.__send_request(ReqTypes.LOAD, FAL_PORTS_PATH).get()

    def __get_current_raid_level(self, volume):
        """Return current raid level."""

        RAID_LEVEL_XPATH = \
            "raidlist/raid/name[text()='%s']/../level" % volume

        return self.__send_request(ReqTypes.LOAD,
                        FAL_RAIDCTRL_PATH).get().get_value(RAID_LEVEL_XPATH)

    def __is_current_raid_mode_vfsext(self):

        VFSEXT_FSTYPE = "vfsext"

        level = self.__get_current_raid_level(config.INTERNAL_VOL)
        if VFSEXT_FSTYPE == level:
            return True
        return False

    def __is_need_mount(self, vol_name):

        if vol_name == util.get_internal_vol_name():
            return True

        match = re.match("usb([1-9]+)", vol_name)
        if not match:
            return False

        port_num = match.group(1)

        port_conf_node = self.__load_port_config()
        mode = port_conf_node.get_node(
            USB_PORT_ENABLE_MODE_XPATH % port_num).get_text()

        if "share" == mode:
            return True

        return False

    def __is_exist_announce(self):

        result_entry = self.__send_request(ReqTypes.QUERY, FAL_ANNOUNCE_PATH)
        if Results.OK != result_entry.get_result():
            return False

        return len(result_entry.get().get_node_list("announce")) > 0

    def __mount(self, vol_name):

        result_map = {AUTOREBUILD_KEY:{}}

        self.__check_keywaiting(vol_name)

        result_entry = \
            self.__send_request(ReqTypes.MOUNT,
                                FAL_VOLCTRL_PATH,
                                MOUNT_VOLUME_REQ_XML_TMPL % vol_name)
                                
        if Results.OK != result_entry.get_result():
            raise self.__MountError

        disk_node_list = \
            result_entry.get().get_node_list("result/autorebuild/disk")
 
        for disk_node in disk_node_list:
            result_map[AUTOREBUILD_KEY].update(
                {disk_node.get_value(NAME_NAME): disk_node.get_value(STATUS_NAME)})

        return result_map

    def __try_mount_all_usb_share_volume(self):

        for vol_name in VOL_LIST:

            if vol_name == util.get_internal_vol_name():
                continue

            if not util.is_device_exist(config.VOL_INFO[vol_name]["dev"]):
                continue

            if not self.__is_need_mount(vol_name):
                continue

            try:
                self.__mount(vol_name)
            except:
                continue

    def __play_boot_melody(self):

        self.__send_request(ReqTypes.PLAY,
                            FAL_BUZZER_PATH,
                            PLAY_BOOT_MELODY_XML)
        return

    def __play_error_melody(self):

        self.__send_request(ReqTypes.PLAY,
                            FAL_BUZZER_PATH,
                            PLAY_ERROR_MELODY_XML)
        return

    def __dispatch_event(self, category, action, body_node=None):
        """Dispatch the HDL-H internal event."""

        entry = request.Entry(ReqTypes.EVENT,
                        fal_info.FAL_EVENTS_REQ_BASE_XML[ReqTypes.EVENT])

        entry.get().get_node(EVENTINFO_NAME).set_attrs(
                        {CATEGORY_ATTR: category, ACTION_ATTR: action})

        if body_node is not None:
            entry.get().get_node(EVENTINFO_NAME).append_child(body_node)

        # Ignore error.
        request.Request(FAL_EVENTS_PATH).request(entry)

    def __get_mirror_usbd(self):
        """Return mirror USB device name."""

        raid_mode = self.__get_current_raid_level(config.INTERNAL_VOL)
        array_num = config.RAID_MODE_INFO[raid_mode]["array_num"]
        internal_dev = config.VOL_INFO[config.INTERNAL_VOL]["dev"]

        for element in config.RAID_TO_ELEMENT[array_num][internal_dev]:
            if element in config.USB_HDD_DEV_LIST:
                return element
        return ""

    def __remove_usbdisk_from_system_raid(self):

        # mirroring設定でない場合はusbのパーティションをsystem用raid
        # からremoveする

        mirror_usbd = self.__get_mirror_usbd()
        if not mirror_usbd:
            return
        
        if util.is_mirroring_setting(mirror_usbd):
            return

        for raid_name, element_list in \
                util.get_raid_element_info().iteritems():

            if raid_name not in SYSTEM_RAID_DEV_LIST:
                continue

            for element in element_list:

                if not element.startswith(mirror_usbd):
                    continue

                try:
                    util.fal_raidctrl_remove_partition(raid_name, element)
                except:
                    self.exception("fail to remove %s from %s" % \
                                   (element, raid_name))
        return

    def __update_all_volume_info(self):

        for vol_name in VOL_LIST:
            try:
                util.fal_volctrl_update(vol_name)
            except:
                continue
        return

    def __get_internal_volume_status(self):
        """Return the internal volume status."""

        raid_info = util.fal_volctrl_query(util.get_internal_vol_name())
        return raid_info.get_node(VOLUME_STATUS_XPATH).get_text()

    def __is_enable_keyserver(self):

        ENABLE_KEYSERVER_XPATH = \
            "./keyserverlist/keyserver[@enable='true']"

        # lockkey設定ファイルからkeyserverの有無を判別
        lockkey_conf_node = \
            self.__send_request(ReqTypes.LOAD, FAL_LOCKKEY_PATH).get()

        keyserver_node_list = \
            lockkey_conf_node.get_node_list(ENABLE_KEYSERVER_XPATH)

        if len(keyserver_node_list) > 0:
            return True

        return False

    def __check_keywaiting(self, vol_name):

        if KEYWAITING_STATUS == util.get_volume_status(vol_name):

            if self.__is_enable_keyserver():
                return

            lockkey_status = util.get_volume_status(LOCKKEY_VOL_LIST[0])
            if not lockkey_status:
                util.save_event_log(msginfo.Msg.ERR_LOCKKEY_DISK_NOT_FOUND)
                if vol_name == util.get_internal_vol_name():
                    self.__stop_disk_service()
                raise self.__KeyWaiting

            if LOCKKEY_STATUS != lockkey_status:
                util.save_event_log(msginfo.Msg.ERR_LOCKKEY_NO_KEY_FILE)
                if vol_name == util.get_internal_vol_name():
                    self.__stop_disk_service()
                raise self.__KeyWaiting

    def __create_boot_complete_file(self):
        """Create "boot_complete" file."""

        with open(BOOT_COMPLETE_FILE_PATH, "a"):
            pass

    def save_cuurent_array_num(self, vol_name=config.INTERNAL_VOL):

        raid_mode = self.__get_current_raid_level(vol_name)

        file_path = CUR_ARRAY_NUM_TMPL % vol_name

        dir_path = os.path.dirname(file_path)
        if not os.path.isdir(dir_path):
            os.makedirs(dir_path)

        if not os.path.isfile(file_path):
            with open(file_path, "w") as fh:
                fh.write(str(config.RAID_MODE_INFO[raid_mode]["array_num"]))

        return

    def __handle_extend_volume(self, raid_result_map, mount_result_map):

        ext_raid_level = raid_result_map[EXTEND_KEY][LEVEL_KEY]
        extend_raid_status = raid_result_map[EXTEND_KEY][STATUS_KEY]

        if not extend_raid_status:
            return

        if EXTEND_NOEXEC == extend_raid_status:
            return

        if EXTEND_FAIL == extend_raid_status:
            util.save_event_log(EXTEND_VOLUME_ERR_LOG_CODE_MAP[ext_raid_level])
            return

        if EXTEND_SUCCESS == extend_raid_status:
            for disk, extend_mnt_status in \
                    mount_result_map[AUTOREBUILD_KEY].iteritems():

                # 一つでも成功してれば成功ログを残す
                if extend_mnt_status ==  EXTEND_MOUNT_SUCCESS:
                    util.save_event_log(EXTEND_VOLUME_LOG_CODE_MAP[ext_raid_level])
                    return
            util.save_event_log(EXTEND_VOLUME_ERR_LOG_CODE_MAP[ext_raid_level])
        return

    def __action_boot(self, entry):

        try:
            self.save_cuurent_array_num()
        except:
            self.exception("fail to save_cuurent_array_num()")

        try:
            self.__remove_usbdisk_from_system_raid()
        except:
            self.exception("fail to remove_usbdisk")

        try:
            for mirr_vol_name in config.MIRRORING_VOL_LIST:
                try:
                    if util.is_other_mirror(mirr_vol_name):
                        util.fal_volctrl_update(mirr_vol_name,
                                                OTHER_MIRROR_STATUS)
                except:
                    pass
        except:
            pass


        try:
            raid_result_map = self.__raid_start()
        except self.__StartError:

            self.exception("fail raid_start")

            try:
                util.fal_raidctrl_stop(MIRROR_DEV, is_crash=True)
            except:
                pass

            try:
                util.fal_raidctrl_stop(INTERNAL_DEV, is_crash=True)
            except:
                pass

            try:
                util.unmount_all_usb_volume()
            except:
                pass

            util.dispatch_event(util.RAID_CATEGORY, util.UPDATE_ACTION)
            util.dispatch_event(util.DISK_CATEGORY, util.UPDATE_ACTION)
            self.__create_boot_complete_file()
            util.save_event_log(msginfo.Msg.MSG_SYSTEM_BOOT_DONE)
            return

        except:
            self.exception("fail to raid_start by unknown reason")
            self.__create_boot_complete_file()
            return

        is_mount_fail = False
        try:
            mount_result_map = self.__mount(util.get_internal_vol_name())
            util.save_event_log(msginfo.Msg.MSG_RAID_VOLUME_MOUNT)
        except:
            is_mount_fail = True
            self.exception("fail to mount internal volume status")
            pass

        try:
            self.__handle_extend_volume(raid_result_map, mount_result_map)
        except:
            self.exception("fail to __handle_extend_volume")

        try:
            util.save_tried_first_mount_file()
        except:
            pass

        try:
            util.fal_volctrl_update(util.get_internal_vol_name())
            if is_mount_fail:
                if KEYWAITING_STATE != self.__get_internal_volume_status():
                    util.save_event_log(msginfo.Msg.ERR_RAID_VOLUME_MOUNT)
        except:
            self.exception("fail to get internal volume status")
            pass

        self.__update_all_volume_info()

        util.dispatch_event(util.RAID_CATEGORY, util.UPDATE_ACTION)
        util.dispatch_event(util.DISK_CATEGORY, util.UPDATE_ACTION)

        self.__create_boot_complete_file()

        util.save_event_log(msginfo.Msg.MSG_SYSTEM_BOOT_DONE)
        self.__change_status_led("on")

        if self.__is_exist_announce():
            self.__play_error_melody()
        else:
            self.__play_boot_melody()

        try:
            util.spindown_unuse_disk()
        except:
            self.exception("fail to spindown_unuse_disk()")

        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        action = self.__get_event_action(entry)

        self.__action_func = \
            {BOOT_ACTION: self.__action_boot}

        self.__action_func[action](entry)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhSystemBoot(option=evhcmd.EvhCmdOption()).handle_request()
