#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling UsbdPlug event"""

from __future__ import with_statement

import os

import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
import fal_msginfo as msginfo
from iobase.cmdexec import (CmdExec, NULL)
from fal_info import (FAL_VOLCTRL_PATH, FAL_PORTS_PATH,
                      FAL_RAIDCTRL_PATH, FAL_LOCKKEY_PATH, FAL_FEATURES_PATH)
from iobase.exception import (InvalidArgError, NotFoundError)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf.devinfo import (DEV_TO_PORT_NO, VOL_INFO, LOCKKEY_VOL_LIST,
                                  USB_HDD_DEV_LIST,
                                  SYSTEM_RAID_DEV_LIST)
from product_conf import (STOR_IF_TO_VOL, LOCK_DIR)
from evh_config import (INTERNAL_VOL, INTERNAL_DEV, MIRROR_DEV, QUICKCOPY_ARGS,
                        PLUGED_LOCK_BASE)

import iobase.fileut as fileut
import evh_config as config
import ionas.device as device

BOOTING_STATE = "booting"

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
REDUDANCY_ATTR = "redudancy"
REBUILD_TARGET_ATTR = "rebuildtarget"

USBD_CATEGORY = "usbd"
PLUG_ACTION = "plug"
UNPLUG_ACTION = "unplug"

MIRRORING_STATUS = "mirroring"
VFSEXT_STATUS = "vfsext"
WAITING_STATUS = "waiting"
CAPSHORTAGE_STATUS = "capshortage"
UNFORMATED_STATUS = "unformated"
KEYWAITING_STATUS = "keywaiting"
MOUNTED_STATUS = "mounted"
MOUNTFAIL_STATUS = "mountfail"
LOCKKEY_STATUS = "lockkey"
UNCONNECTED_STATUS = "unconnected"
FAIL_STATUS = "fail"
CRASH_STATUS = "crash"
UNPLUG_STATUS = "unplug"
NORMAL_STATUS = "normal"
UNKNOWN_STATUS = "unknown"
OTHER_MIRROR_STATUS = "othermirror"
RECOVERABLE_MIRROR_STATUS = "recoverablemirror"

START_KEY = "start"

STATUS_TYPE = "status"

VOLUME_NAME_XPATH = "eventinfo/device/name"

SYMBL_MIRROR_NAME = VOL_INFO[INTERNAL_VOL]["dev"]

MOUNT_BASE_REQ_XML = \
    '<volctrl>' + \
    ' <volume>' + \
    '  <name type="volume">%s</name>' + \
    ' </volume>' + \
    '</volctrl>'

UNMOUNT_BASE_REQ_XML = \
    '<volctrl>' + \
    ' <volume>' + \
    '  <name type="volume">%s</name>' + \
    ' </volume>' + \
    '</volctrl>'

USB_PORT_ENABLE_MODE_XPATH = \
    "usblist/usb[@port='%s']/modelist/mode[@enable='true']"

ADD_TO_RAID_REQ_XML = \
    '<raidctrl>' + \
    ' <raid>' + \
    '  <name type="disk">%s</name>' + \
    ' </raid>' + \
    '</raidctrl>'

MOUNT_INTERNAL_VOL_REQ_XML = \
    '<volctrl>' + \
    '<volume>' + \
    '<name type="volume">%s</name>' % INTERNAL_VOL + \
    '</volume>' + \
    '</volctrl>'

UNMOUNT_INTERNAL_VOL_REQ_XML = \
    '<volctrl>' + \
    '<volume>' + \
    '<name type="volume">%s</name>' % INTERNAL_VOL + \
    '</volume>' + \
    '</volctrl>'

USB_NAME = "usb"
SHARE_PORT_MODE = "share"
MIRRORING_PORT_MODE = "mirroring"
VFSEXT_PORT_MODE = "vfsext"
QUICKCOPY_MODE = "quickcopy"

FEATURES_NAME = "features"
BUTTON_PATH = "deviceinfo/buttonlist/button"
NAME_ATTR = "name"
FUNC_NAME = "func"

UNFORMATED_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_VOLUME_USB1_UNFORMATTED,
     msginfo.Msg.ERR_VOLUME_USB2_UNFORMATTED,
     msginfo.Msg.ERR_VOLUME_USB3_UNFORMATTED,
     msginfo.Msg.ERR_VOLUME_USB4_UNFORMATTED]

KEYWAITING_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_LOCKKEY_USB1_SECURITY_LOCK_MODE,
     msginfo.Msg.ERR_LOCKKEY_USB2_SECURITY_LOCK_MODE,
     msginfo.Msg.ERR_LOCKKEY_USB3_SECURITY_LOCK_MODE,
     msginfo.Msg.ERR_LOCKKEY_USB4_SECURITY_LOCK_MODE]

MOUNTED_STATUS_MSG_LIST = \
    [msginfo.Msg.MSG_VOLUME_USB1_MOUNT,
     msginfo.Msg.MSG_VOLUME_USB2_MOUNT,
     msginfo.Msg.MSG_VOLUME_USB3_MOUNT,
     msginfo.Msg.MSG_VOLUME_USB4_MOUNT]

MOUNTFAIL_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_VOLUME_USB1_MOUNT,
     msginfo.Msg.ERR_VOLUME_USB2_MOUNT,
     msginfo.Msg.ERR_VOLUME_USB3_MOUNT,
     msginfo.Msg.ERR_VOLUME_USB4_MOUNT]

VFSEXT_START_MSG_LIST = \
    [msginfo.Msg.MSG_RAID_USB3_VFSEXT_START,
     msginfo.Msg.MSG_RAID_USB4_VFSEXT_START]

CAPSHORTAGE_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_RAID_USB3_VFSEXT_CAP_SHORTAGE,
     msginfo.Msg.ERR_RAID_USB4_VFSEXT_CAP_SHORTAGE]

WAITING_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_RAID_USB3_VFSEXT_WAITING,
     msginfo.Msg.ERR_RAID_USB4_VFSEXT_WAITING]

FAIL_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_RAID_USB3_VFSEXT_FAIL,
     msginfo.Msg.ERR_RAID_USB4_VFSEXT_FAIL]

UNKNOWN_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_RAID_USB3_VFSEXT_START,
     msginfo.Msg.ERR_RAID_USB4_VFSEXT_START]

MIRROR_START_MSG_LIST = \
    [msginfo.Msg.MSG_RAID_MIRROR_START_TO_USB2,
     msginfo.Msg.MSG_RAID_MIRROR_START_TO_USB3]

MIRROR_CAPSHORTAGE_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_RAID_MIRROR_CAP_SHORTAGE_USB2,
     msginfo.Msg.ERR_RAID_MIRROR_CAP_SHORTAGE_USB3]

MIRROR_WAITING_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_RAID_MIRROR_WAITING_USB2,
     msginfo.Msg.ERR_RAID_MIRROR_WAITING_USB3]

MIRROR_FAIL_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_RAID_MIRROR_FAIL_USB2,
     msginfo.Msg.ERR_RAID_MIRROR_FAIL_USB3]

MIRROR_UNKNOWN_STATUS_MSG_LIST = \
    [msginfo.Msg.ERR_RAID_MIRROR_START_TO_USB2,
     msginfo.Msg.ERR_RAID_MIRROR_START_TO_USB3]

VOLUME_LOG_CODE_MAP = {
    UNFORMATED_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems()]),
             UNFORMATED_STATUS_MSG_LIST)),

    KEYWAITING_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems()]),
             KEYWAITING_STATUS_MSG_LIST)),
    MOUNTED_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems()]),
             MOUNTED_STATUS_MSG_LIST)),
    MOUNTFAIL_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems()]),
             MOUNTFAIL_STATUS_MSG_LIST)),
    }

VFSEXT_START_LOG_CODE_MAP = \
    dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (3, 4)]),
             VFSEXT_START_MSG_LIST))

VFSEXT_ERR_LOG_CODE_MAP = {
    CAPSHORTAGE_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (3, 4)]),
             CAPSHORTAGE_STATUS_MSG_LIST)),

    WAITING_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (3, 4)]),
             WAITING_STATUS_MSG_LIST)),

    FAIL_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (3, 4)]),
             FAIL_STATUS_MSG_LIST)),

    OTHER_MIRROR_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (3, 4)]),
             WAITING_STATUS_MSG_LIST)),

    RECOVERABLE_MIRROR_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (3, 4)]),
             WAITING_STATUS_MSG_LIST)),

    UNKNOWN_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (3, 4)]),
             UNKNOWN_STATUS_MSG_LIST)),
    }

MIRROR_LOG_CODE_MAP = {
    START_KEY:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (2, 3)]),
             MIRROR_START_MSG_LIST)),

    CAPSHORTAGE_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (2, 3)]),
             MIRROR_CAPSHORTAGE_STATUS_MSG_LIST)),

    CAPSHORTAGE_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (2, 3)]),
             MIRROR_CAPSHORTAGE_STATUS_MSG_LIST)),

    WAITING_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (2, 3)]),
             MIRROR_WAITING_STATUS_MSG_LIST)),

    FAIL_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (2, 3)]),
             MIRROR_FAIL_STATUS_MSG_LIST)),

    OTHER_MIRROR_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (2, 3)]),
             MIRROR_WAITING_STATUS_MSG_LIST)),

    RECOVERABLE_MIRROR_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (2, 3)]),
             MIRROR_WAITING_STATUS_MSG_LIST)),

    UNKNOWN_STATUS:
        dict(zip(sorted([vol for port, vol in
             STOR_IF_TO_VOL[USB_NAME].iteritems() if port in (2, 3)]),
             MIRROR_UNKNOWN_STATUS_MSG_LIST)),
    }


class EvhUsbdPlug(evhcmd.EvhCmd):
    """UsbdPlug Event Hnadler command implementation class."""

    class __MountFail(Exception):
        "Mount Fail"
        pass

    class __FailDisk(Exception):
        "Fail Disk detected"
        pass

    class __RequestError(Exception):
        "RequestError"
        pass

    class __KeyWaiting(Exception):
        pass

    class __UnmountError(Exception):
        pass

    class __InternalError(Exception):
        pass

    def __get_event_action(self, entry):

        if USBD_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        if PLUG_ACTION == entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            return PLUG_ACTION

        if UNPLUG_ACTION == entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            return UNPLUG_ACTION

        raise InvalidArgError

    def __dev_name2vol_name(self, dev_name):

        for vol_name, vol_info in VOL_INFO.iteritems():

            if dev_name.startswith(vol_info["dev"]):
                return vol_name

        return ""

    def __send_request(self, type, target_module, child_node=""):

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    def __is_enable_keyserver(self):

        ENABLE_KEYSERVER_XPATH = \
            "./keyserverlist/keyserver[@enable='true']"

        # lockkey設定ファイルからkeyserverの有無を判別
        lockkey_conf_node = \
            self.__send_request(ReqTypes.LOAD, FAL_LOCKKEY_PATH).get()

        keyserver_node_list = \
            lockkey_conf_node.get_node_list(ENABLE_KEYSERVER_XPATH)

        if len(keyserver_node_list) > 0:
            return True

        return False

    def __share_mount(self, vol_name):

        if CRASH_STATUS == util.get_raid_state():
            return

        if KEYWAITING_STATUS == util.get_volume_status(vol_name):

            # キーサーバー有効か無効かチェック
            if not self.__is_enable_keyserver():
                lockkey_status = util.get_volume_status(LOCKKEY_VOL_LIST[0])
                if not lockkey_status:
                    util.save_event_log(msginfo.Msg.ERR_LOCKKEY_DISK_NOT_FOUND)
                    raise self.__KeyWaiting

                if LOCKKEY_STATUS != lockkey_status:
                    util.save_event_log(msginfo.Msg.ERR_LOCKKEY_NO_KEY_FILE)
                    raise self.__KeyWaiting

        result_entry = self.__send_request(ReqTypes.MOUNT, FAL_VOLCTRL_PATH,
                                           MOUNT_BASE_REQ_XML % vol_name)
        result_code = result_entry.get_result()
        if Results.OK == result_code or Results.ALREADY_MOUNT == result_code:
            return

        if KEYWAITING_STATUS == util.get_volume_status(vol_name):
            raise self.__KeyWaiting
        raise self.__MountFail

    def __share_unmount(self, vol_name):

        result_entry = \
            self.__send_request(ReqTypes.UNMOUNT, FAL_VOLCTRL_PATH,
                                UNMOUNT_BASE_REQ_XML % vol_name)

        result_code = result_entry.get_result()
        if Results.OK == result_code:
            return

        if Results.ALREADY_UNMOUNT == result_code:
            return

        raise self.__UnmountError

    def __is_encrypted_volume(self, volume):

        ENCRYPT_ATTR = "encrypt"

        volctrl_node = util.fal_volctrl_query(INTERNAL_VOL)
        try:
            return volctrl_node.get_node("volume/filesystem").\
                       is_set_to_true(ENCRYPT_ATTR)
        except:
            return False

    def __add_to_raid_with_remake_enc(self, dev_name):

        # 内蔵ボリュームunmount
        result_entry = self.__send_request(ReqTypes.UNMOUNT,
                                           FAL_VOLCTRL_PATH,
                                           UNMOUNT_INTERNAL_VOL_REQ_XML)
        result_code = result_entry.get_result()
        if Results.OK != result_code:
            raise self.__RequestError

        # RAIDにdevice add
        result_entry = self.__send_request(ReqTypes.ADD,
                                           FAL_RAIDCTRL_PATH,
                                           ADD_TO_RAID_REQ_XML % dev_name)
        result_code = result_entry.get_result()

        if Results.DISK_ERROR == result_code:
            raise self.__FailDisk

        if Results.MKFS_ERROR == result_code:
            raise self.__FailDisk

        if Results.ADD_ERROR == result_code:
            raise self.__FailDisk
        

        # 内蔵ボリュームmount
        result_entry = self.__send_request(ReqTypes.MOUNT,
                                           FAL_VOLCTRL_PATH,
                                           MOUNT_INTERNAL_VOL_REQ_XML)
        result_code = result_entry.get_result()
        if Results.OK != result_code:
            raise self.__RequestError

    def __add_to_raid(self, dev_name):

        result_entry = self.__send_request(ReqTypes.ADD,
                                           FAL_RAIDCTRL_PATH,
                                           ADD_TO_RAID_REQ_XML % dev_name)

        result_code = result_entry.get_result()

        if Results.OK == result_code:
            return

        if Results.DISK_ERROR == result_code:
            raise self.__FailDisk

        if Results.MKFS_ERROR == result_code:
            raise self.__FailDisk

        if Results.ADD_ERROR == result_code:
            raise self.__FailDisk

        raise self.__RequestError

    def __load_port_config(self):

        return self.__send_request(ReqTypes.LOAD, FAL_PORTS_PATH).get()

    def __get_volume_status(self, vol_name):

        STATUS_XPATH = "./volume/status"
        try:
            volctrl_node = util.fal_volctrl_query(vol_name)
        except:
            return UNCONNECTED_STATUS

        return volctrl_node.get_value(STATUS_XPATH)

    def __log_mirroring_event(self, vol_name):

        try:
            status = util.get_volume_status(vol_name)

            if status == MIRRORING_STATUS:
                return

            if (status == CAPSHORTAGE_STATUS or
                status == WAITING_STATUS or
                status == FAIL_STATUS or 
                status == OTHER_MIRROR_STATUS or
                status == RECOVERABLE_MIRROR_STATUS):

                util.save_event_log(MIRROR_LOG_CODE_MAP[status][vol_name])
            else:
                util.save_event_log(
                        MIRROR_LOG_CODE_MAP[UNKNOWN_STATUS][vol_name])
        except:
            pass

    def __log_vfsext_event(self, vol_name):

        try:
            status = util.get_volume_status(vol_name)

            if status == VFSEXT_STATUS:
                return

            try:
                util.save_event_log(
                    VFSEXT_ERR_LOG_CODE_MAP[status][vol_name])
            except KeyError:
                util.save_event_log(
                    VFSEXT_ERR_LOG_CODE_MAP[UNKNOWN_STATUS][vol_name])
        except:
            pass

    def __reset_bgcmd_progress_state(self, dev_name):

        body_xml = \
            '<progress state="NOT_EXEC"><target>%s</target></progress>' \
            % dev_name

        util.dispatch_event("volumemkfs", "progress", body_xml)
        util.dispatch_event("volumefsck", "progress", body_xml)

    def __is_mirror_element_disk(self, disk_name):

        for element in util.get_raid_element_info()[SYMBL_MIRROR_NAME]:

            if element.startswith(disk_name):
                return True

        return False

    # RAIDの冗長性チェック
    def __is_crash_if_remove(self, raid_name, disk_name):
        """Check Raid redudancy"""

        try:
            return util.is_crash_if_remove(raid_name, disk_name)
        except:
            return False

    def __is_mirror_redundancy(self):
        """Check Raid redudancy"""

        STATUS_XPATH = "raid/status"

        try:
            raidctrl_node = util.fal_raidctrl_query(MIRROR_DEV)
        except:
            raise self.__InternalError

        return raidctrl_node.get_node(STATUS_XPATH).\
            is_set_to_true(REDUDANCY_ATTR)

    def __get_disk_exist_info(self, disk_name):

        CHK_DISK_EXIST_REQ_XML = \
            """<raidctrl>
                <disk><name>%s</name></disk>
               </raidctrl>"""

        result_entry = \
            self.__send_request(ReqTypes.CHK_EXIST,
                                FAL_RAIDCTRL_PATH,
                                CHK_DISK_EXIST_REQ_XML % disk_name)
        if Results.OK != result_entry.get_result():
            return False, False

        is_rebuild_target = False
        disk_node = result_entry.get().get_node("disk")
        is_rebuild_target = disk_node.is_set_to_true(REBUILD_TARGET_ATTR)
        return True, is_rebuild_target

    def __disk_name2raw_disk_name(self, disk_name):

        return self.dev_mgr.get_kdev_name(
                    self.dev_mgr.get_base_dev_name(disk_name))

    def __disk_name2raid_name(self, disk_name):

        if util.is_current_raid_mode_vfsext():
            return INTERNAL_DEV

        for raid_name, element_list in \
                util.get_raid_element_info().iteritems():

            if raid_name in SYSTEM_RAID_DEV_LIST:
                continue

            for element in element_list:
                if element.startswith(disk_name):
                    return raid_name

        raise NotFoundError

    def __crash_mirror_raid(self):

        try:
            if util.is_raid_crash_or_stop(MIRROR_DEV):
                return
        except:
            pass

        try:
            util.unmount_internal_volume()
            util.fal_raidctrl_stop(MIRROR_DEV, True)
        except:
            self.exception("fail to stop parent_raid_name")
            pass

        try:
            util.unmount_all_usb_volume()
        except:
            pass

        return

    def __delete_disk_from_system(self, disk_name):

        try:
            util.delete_disk_from_system(disk_name)
        except:
            self.exception("fail to delete_disk_from_system(%s)" % disk_name)
            pass

    def __remove_disk_raid(self, disk_name):

        is_exist, is_rebuild_target = self.__get_disk_exist_info(disk_name)
        if not is_exist:
            return

        raid_name = self.__disk_name2raid_name(disk_name)
        if self.__is_crash_if_remove(raid_name, disk_name):
            self.__crash_mirror_raid()
            return

        try:
            util.fal_raidctrl_remove(disk_name)
        except:
            self.exception("fail to remove %s" % disk_name)

        return

    def __is_func_button_supported(self):

        fetures = self.__send_request(ReqTypes.QUERY, FAL_FEATURES_PATH,
                            FEATURES_NAME).get()

        for button in fetures.get_node_list(BUTTON_PATH):
            if button.get_attr_value(NAME_ATTR) == FUNC_NAME:
                return True

        return False

    def __action_plug(self, dev_name, mode, is_force):

        result_code = Results.OK
        vol_name = self.__dev_name2vol_name(dev_name)
        is_just_pluged = False
        pluged_lock_path = os.path.join(LOCK_DIR,
                                        str(vol_name) + PLUGED_LOCK_BASE)
        if not os.path.exists(pluged_lock_path):
            is_just_pluged = True
            with open(pluged_lock_path, "w"):
                pass

        mirror_state = util.check_mirror_state(vol_name)

        if self.__dev_name2vol_name(dev_name) in config.MIRRORING_VOL_LIST:
            if mirror_state in [OTHER_MIRROR_STATUS, RECOVERABLE_MIRROR_STATUS]:
                if not is_force:
                    util.fal_volctrl_update(vol_name, mirror_state)

        if mode in [MIRRORING_PORT_MODE, VFSEXT_PORT_MODE]:

            if not util.is_boot_complete():
                return Results.OK

            try:
                if self.__is_encrypted_volume(INTERNAL_VOL) and \
                   mode == MIRRORING_PORT_MODE:
                    self.__add_to_raid_with_remake_enc(dev_name)
    
                else:
                    self.__add_to_raid(dev_name)

                if MIRRORING_PORT_MODE == mode:
                    util.save_event_log(
                        MIRROR_LOG_CODE_MAP[START_KEY][vol_name])

                elif VFSEXT_PORT_MODE == mode:
                    util.set_quota_limit(INTERNAL_VOL)
                    util.save_event_log(
                        VFSEXT_START_LOG_CODE_MAP[vol_name])

                try:
                    util.fal_diskctrl_update(dev_name,
                                             STATUS_TYPE,
                                             NORMAL_STATUS)
                except:
                    self.exception("fail to fal_diskctrl_update")

            except self.__FailDisk:
                result_code = Results.ADD_ERROR
                try:
                    util.fal_diskctrl_update(dev_name,
                                             STATUS_TYPE,
                                             FAIL_STATUS)
                except:
                    self.exception("fail to fal_diskctrl_update")

            except:
                result_code = Results.ADD_ERROR

            # fal_volctrlが内部的に
            # 保持しているデータを更新
            try:
                util.fal_volctrl_update(vol_name)
            except:
                self.exception("fail to fal_volctrl_update")

            if not is_force:
                if MIRRORING_PORT_MODE == mode:
                    self.__log_mirroring_event(vol_name)
                if VFSEXT_PORT_MODE == mode:
                    self.__log_vfsext_event(vol_name)

            util.dispatch_event(util.RAID_CATEGORY, util.UPDATE_ACTION)
            return result_code

        # mount or umount
        elif SHARE_PORT_MODE == mode:

            try:
                util.fal_volctrl_update(vol_name)
            except:
                self.exception("fail to fal_volctrl_update")

            status = self.__get_volume_status(vol_name)
            if status != LOCKKEY_STATUS:
                try:
                    self.__share_mount(vol_name)
                except self.__MountFail:
                    util.save_event_log(
                        VOLUME_LOG_CODE_MAP[MOUNTFAIL_STATUS][vol_name])
                    result_code = Results.MOUNT_ERROR
                except self.__KeyWaiting:
                    result_code = Results.MOUNT_ERROR

        # fal_volctrlが内部的に
        # 保持しているデータを更新
        try:
            util.fal_volctrl_update(vol_name)
        except:
            self.exception("fail to fal_volctrl_update")

        try:
            util.fal_diskctrl_update(dev_name,
                                     STATUS_TYPE,
                                     NORMAL_STATUS)
        except:
            self.exception("fail to fal_diskctrl_update")

        if QUICKCOPY_MODE == mode:
            status = self.__get_volume_status(vol_name)
            if (status != LOCKKEY_STATUS and
                is_just_pluged and
                util.is_mounted(INTERNAL_VOL) and
                not self.__is_func_button_supported()):

                CmdExec(QUICKCOPY_ARGS).execute(stdout=NULL, stderr=NULL,
                                                wait=False, raise_exc=False)
        else:
            # ログ保存
            status = util.get_volume_status(vol_name)
            if (status in VOLUME_LOG_CODE_MAP and
                vol_name in VOLUME_LOG_CODE_MAP[status]):
                util.save_event_log(VOLUME_LOG_CODE_MAP[status][vol_name])

        return result_code

    def __action_unplug(self, dev_name, mode):

        result_code = Results.OK
        vol_name = self.__dev_name2vol_name(dev_name)

        pluged_lock_path = os.path.join(LOCK_DIR,
                                        str(vol_name) + PLUGED_LOCK_BASE)
        if os.path.exists(pluged_lock_path):
            os.remove(pluged_lock_path)

        if SHARE_PORT_MODE == mode:
            try:
                self.__share_unmount(vol_name)
            except:
                result_code = Results.UNMOUNT_ERROR

        elif MIRRORING_PORT_MODE == mode:

            if self.__is_mirror_element_disk(dev_name):

                try:
                    self.__remove_disk_raid(dev_name)
                except:
                    self.exception("fail to do_unplug")
                finally:
                    util.fal_diskctrl_update(dev_name,
                                             STATUS_TYPE,
                                             UNPLUG_STATUS)
                    util.dispatch_event(util.RAID_CATEGORY,
                                        util.UPDATE_ACTION)
        elif VFSEXT_PORT_MODE == mode:
            try:
                self.__remove_disk_raid(dev_name)
            except:
                self.exception("fail to do_unplug")
            finally:
                util.fal_diskctrl_update(dev_name,
                                         STATUS_TYPE,
                                         UNPLUG_STATUS)
                util.dispatch_event(util.RAID_CATEGORY,
                                    util.UPDATE_ACTION)

        # fal_volctrlが内部的に保持
        # しているデータを更新(重要)
        util.fal_volctrl_update(vol_name)
        self.__delete_disk_from_system(dev_name)
        return result_code

    #
    # Request handlers
    #

    def __action_plug_unplug(self, entry):

        action = self.__get_event_action(entry)

        dev_name = entry.get().get_node(VOLUME_NAME_XPATH).get_text()

        if dev_name not in USB_HDD_DEV_LIST:
            self.info("Unknown sata disk is plugged or unplugged")
            entry.set_result(Results.OK)
            return entry

        port_num = DEV_TO_PORT_NO[dev_name]
        portconf_node = self.__load_port_config()
        mode = portconf_node.get_node(
                   USB_PORT_ENABLE_MODE_XPATH % port_num).get_text()

        self.__reset_bgcmd_progress_state(dev_name)

        is_force = False
        try:
            opt_node = entry.get().get_node("eventinfo/device/option")
            is_force = opt_node.is_set_to_true("force")
        except:
            pass

        result_code = Results.OK

        if PLUG_ACTION == action:
            result_code = self.__action_plug(dev_name, mode, is_force)
        elif UNPLUG_ACTION == action:
            result_code = self.__action_unplug(dev_name, mode)

        entry.set_result(result_code)
        return entry

    def __event_handler(self, entry):

        try:
            with fileut.FLock(config.LOCK_DIR,
                              config.DISK_EVENT_LOCK_FILE,
                              exclusive=True):

                return self.__action_plug_unplug(entry)
        except:
            self.exception("fail to action plug and unplug")
            raise

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.dev_mgr = device.DevMgr(config.DEV_DIR, config.DEV_LINK_DIR,
                        config.DEV_FILE_DIR, config.HDD_DEV_LIST)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhUsbdPlug(option=evhcmd.EvhCmdOption()).handle_request()
