#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Define utility functions for EVH commands."""

from __future__ import with_statement
import os
import re
import stat
import shutil
import fal_info
import ionas.request as request
from iobase.xmldata import (XmlNode, XmlConfNode)
from iobase.exception import (IoBaseException, NotFoundError, InvalidArgError)
from product_conf import (STOR_IF_TO_VOL, MAN_DIR, DEFAULT_XML_CONF_DIR)
from product_conf.devinfo import *
from product_conf.reqinfo import (Results, ReqTypes)
import evh_config as config
import fal_msginfo as msginfo

import pal_info


PYC_SUFFIX = ".pyc"
PY_SUFFIX = ".py"

# XML Tag Name
STATE_NAME = "state"
ARGLIST_NAME = "arglist"
ARG_NAME = "arg"
LOG_NAME = "log"
EVENTINFO_NAME = "eventinfo"

# X path
ACTION_XPATH = "eventinfo/@action"
VOL_NAME_XPATH = "eventinfo/volume/name"
PROGRESS_XPATH = \
    "./eventinfo/progress"
TARGET_XPATH = \
    "./eventinfo/progress/target"
USB_PORT_MODE_PATH = "usblist/usb[@port='%s']/modelist/mode[@enable='true']"
STATE_XPATH = "./statelist/state"
MAN_XPATH = "sharelist/share[name='LAN DISK Manual']"

# XML Tag Value
SUCCESS_STATUS_VAL = "success"
ERROR_STATUS_VAL = "error"
TRUE = XmlConfNode.TRUE_TEXT

### XML Attribute ###
NAME_ATTR = "name"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

# Some value
PROCESSING_STR = "PROCESSING:"
COMPLETE_STR = "COMPLETE:"

BOOTING_STATE = "booting"
PREPARING_STATE = "preparing"
SHUTDOWN_STATE = "shutdown"

RAID_CATEGORY = "raid"
DISK_CATEGORY = "disk"
UPDATE_ACTION = "update"

STATEMGR_SYSTEM_QUERY_XML = \
    '<statemgr><statelist><state category="system"/>' + \
        '</statelist></statemgr>'

STATEMGR_QUERY_XML_TMPL = '<statemgr><statelist><state category="%s"/>' + \
                              '</statelist></statemgr>'
SET_STATE_REQ_XML_TMPL = \
    '<statemgr><state category="%(category)s">%(state)s</state></statemgr>'

VOLUME_QUERY_REQ_XML_TMPL = \
    '<volctrl><volume><name type="volume">%s</name></volume></volctrl>'

MOUNT_REQ_XML = \
    '<volctrl><volume><name type="volume">%s</name></volume></volctrl>'

MOUNT_INTERNAL_REQ_XML = \
    MOUNT_REQ_XML % STOR_IF_TO_VOL["internal"][1]

UNMOUNT_REQ_XML = \
    """<volctrl>
         <option noupdate="true"/>
         <volume>
          <name type="volume">%s</name>
         </volume>
       </volctrl>"""

UNMOUNT_INTERNAL_REQ_XML = \
    UNMOUNT_REQ_XML % STOR_IF_TO_VOL["internal"][1]

PLAY_MELODY_BASE_REQ_XML_TMPL = \
  '<buzzer>' + \
  ' <melody type="%s"/>' + \
  '</buzzer>'

# mkfs後のevh_share_initよりpriorityを下げる
MAN_SET_EVENT_XML = \
  "<events>" + \
  " <eventinfo category='volume' action='mount' priority='75'/>" + \
  "</events>"

SHARE_MODE = "share"

STATUS_PATH = "volume/status"
STATUS_MOUNTED = "mounted"

CRASH_STATUS = "crash"
STOP_STATUS = "stop"

INTERNAL_VOL = STOR_IF_TO_VOL["internal"][1]
INTERNAL_VOL_INFO = VOL_INFO[INTERNAL_VOL]

SHARE_PATH = os.path.join(INTERNAL_VOL_INFO["mnt_point"],
                          INTERNAL_VOL_INFO["share_root"])

DEFAULT_SHARES_XML_PATH = os.path.join(
    DEFAULT_XML_CONF_DIR, config.SHARES_CONF_FILE)

MAN_SHARE_NAME = "LAN DISK Manual"
MAN_SHARE_PATH = os.path.join(SHARE_PATH, MAN_SHARE_NAME)

MODE_CH_FILE = (stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)

MIRRORING_PORT_MODE = "mirroring"
VFSEXT_PORT_MODE = "vfsext"

QUOTA_XML = "<quota/>"
USERLIST_NAME = "userlist"
GROUPLIST_NAME = "grouplist"
NAME_NAME = "name"
UNAME_NAME = "uname"
USER_PATH = USERLIST_NAME + "/user"
SHARE_PATH = "sharelist/share"
GROUP_PATH = GROUPLIST_NAME + "/group"
LIMIT_PATH = "quota/limit"
SHARE_LIMIT_PATH = "shareservice/msshare/quota/limit"
DEFAULT_LIMIT = "0.0"
USER_XML_FOR_QUOTA_TMPL = """<user><name>%s</name><quota volume="%s">""" + \
                                """<limit>%s</limit></quota></user>"""
GROUP_XML_FOR_QUOTA_TMPL = """<group><name>%s</name><quota volume="%s">""" + \
                                """<limit>%s</limit></quota></group>"""

MNT_POINT_KEY = "mnt_point"


class RequestError(IoBaseException):
    """Request error"""

    pass


class ChangeError(Exception):
    """Change error"""

    pass


class InternalError(Exception):
    """Internal error"""

    pass


def _force_make_dir(dir_path):
    """make specified directory."""

    if not os.path.isdir(dir_path):
        os.makedirs(dir_path)

    return


def _save_value(save_file_path, value):

    with open(save_file_path, "w") as fh:
        fh.write(value)
    return


def _get_user_quota_list(volume):
    """Return userlist XML data for applying quota which include for share."""

    userlist = XmlNode(USERLIST_NAME)
    try:
        mnt_point = VOL_INFO[volume][MNT_POINT_KEY]
        users = request_entry(fal_info.FAL_USERS_PATH,
                                request.Entry(ReqTypes.LOAD)).get()
        for user in users.get_node_list(USER_PATH):
            try:
                limit = user.get_value(LIMIT_PATH, DEFAULT_LIMIT)
                if float(limit) != 0:
                    userlist.append_child(XmlNode(USER_XML_FOR_QUOTA_TMPL %
                                        (user.get_node(NAME_NAME).get_text(),
                                            mnt_point, limit)))
            except:
                pass

        shares = request_entry(fal_info.FAL_SHARES_PATH,
                                request.Entry(ReqTypes.LOAD)).get()
        for share in shares.get_node_list(SHARE_PATH):
            try:
                limit = share.get_value(SHARE_LIMIT_PATH, DEFAULT_LIMIT)
                if float(limit) != 0:
                    userlist.append_child(XmlNode(USER_XML_FOR_QUOTA_TMPL %
                                        (share.get_node(UNAME_NAME).get_text(),
                                            mnt_point, limit)))
            except:
                pass
    except:
        pass

    return userlist


def _get_group_quota_list(volume):
    """Return grouplist XML data for applying quota."""

    grouplist = XmlNode(GROUPLIST_NAME)
    try:
        mnt_point = VOL_INFO[volume][MNT_POINT_KEY]
        groups = request_entry(fal_info.FAL_GROUPS_PATH,
                                request.Entry(ReqTypes.LOAD)).get()
        for group in groups.get_node_list(GROUP_PATH):
            try:
                limit = group.get_value(LIMIT_PATH, DEFAULT_LIMIT)
                if float(limit) != 0:
                    grouplist.append_child(XmlNode(GROUP_XML_FOR_QUOTA_TMPL %
                                        (group.get_node(NAME_NAME).get_text(),
                                            mnt_point, limit)))
            except:
                pass
    except:
        pass

    return grouplist


def get_internal_vol_name():

    return STOR_IF_TO_VOL["internal"][1]


def save_progress(entry, save_file_path):

    _force_make_dir(os.path.dirname(save_file_path))

    state = \
        entry.get().get_node(PROGRESS_XPATH).get_attr_value(STATE_NAME)

    if SUCCESS_STATUS_VAL == state or ERROR_STATUS_VAL == state:
        _save_value(save_file_path, COMPLETE_STR + state)
        return

    elif re.match(".*%", state):
        _save_value(save_file_path, PROCESSING_STR + state)

    else:
        _save_value(save_file_path, state)

    return


def dev_name2vol_name(dev_name):

    for vol_name, vol_info in VOL_INFO.iteritems():

        if dev_name.startswith(vol_info["dev"]):
            return vol_name

    return ""


def request_entry(fal_pal_cmd, entry):
    """Send "entry" to "fal/pal_cmd" and return the entry in response data."""

    return request.Request(fal_pal_cmd).request(entry).get_entry()


def get_handler_real_path(handler_path):
    """Return the "handler_path" handler real path."""

    (base, suffix) = os.path.splitext(handler_path)
    target_path = handler_path
    if suffix == PYC_SUFFIX:
        target_path = base + PY_SUFFIX

    if os.path.islink(target_path):
        return os.path.abspath(os.path.join(os.path.dirname(target_path),
                    os.readlink(target_path)))
    return os.path.abspath(target_path)


#
# For APP layer event handler
#


def get_system_state():
    """Return the state of "system"."""

    try:
        entry = request.Entry(ReqTypes.QUERY, STATEMGR_SYSTEM_QUERY_XML)
        response_entry = request_entry(fal_info.FAL_STATEMGR_PATH, entry)

        if response_entry.get_result() != Results.OK:
            return BOOTING_STATE

        return response_entry.get().get_node(STATE_XPATH).get_text()
    except:
        return BOOTING_STATE


def get_raid_state():
    """Return the state of "raidvol"."""

    try:
        entry = \
             request.Entry(ReqTypes.QUERY, STATEMGR_QUERY_XML_TMPL % "raid")
        response_entry = request_entry(fal_info.FAL_STATEMGR_PATH, entry)

        if response_entry.get_result() != Results.OK:
            return PREPARING_STATE

        return response_entry.get().get_node(STATE_XPATH).get_text()
    except:
        return PREPARING_STATE


def get_raidvol_state():
    """Return the state of "raidvol"."""

    try:
        entry = \
             request.Entry(ReqTypes.QUERY, STATEMGR_QUERY_XML_TMPL % "raidvol")
        response_entry = request_entry(fal_info.FAL_STATEMGR_PATH, entry)

        if response_entry.get_result() != Results.OK:
            return PREPARING_STATE

        return response_entry.get().get_node(STATE_XPATH).get_text()
    except:
        return PREPARING_STATE


def get_filesystem_state():
    """Return the state of "filesystem"."""

    try:
        entry = \
             request.Entry(ReqTypes.QUERY, STATEMGR_QUERY_XML_TMPL % "filesystem")
        response_entry = request_entry(fal_info.FAL_STATEMGR_PATH, entry)

        if response_entry.get_result() != Results.OK:
            return PREPARING_STATE

        return response_entry.get().get_node(STATE_XPATH).get_text()
    except:
        return PREPARING_STATE

def set_state_manager(category, state):

    entry = request.Entry(ReqTypes.SET, SET_STATE_REQ_XML_TMPL % \
                                            {'category': category,
                                             'state': state})
    response_entry = request_entry(fal_info.FAL_STATEMGR_PATH, entry)

    if response_entry.get_result() == Results.CHANGE_ERROR:
        raise ChangeError
    if response_entry.get_result() != Results.OK:
        raise RequestError

    return


def save_event_log(msg_code, arg_dict={}):
    """save event log."""

    EVENTLOG_XML_TMPL = '<eventlog><log><code>%s</code></log></eventlog>'

    eventlog_node = XmlNode(EVENTLOG_XML_TMPL % msg_code)

    arglist_node = XmlNode(ARGLIST_NAME)
    for arg_name, arg_value in arg_dict.iteritems():
        arglist_node.append_child(ARG_NAME, arg_value, {NAME_ATTR: arg_name})
        eventlog_node.get_node(LOG_NAME).append_child(arglist_node)

    entry = request.Entry(ReqTypes.ADD, eventlog_node)
    request_entry(fal_info.FAL_EVENTLOG_PATH, entry)
    return


def is_device_exist(dev_name, dry_run=None, debug=None):
    """Check whether specified device exist"""

    check_exist_req_xml = \
        '<devctrl><device><name>%s</name></device></devctrl>' % dev_name

    entry = request.Entry(ReqTypes.CHK_EXIST, check_exist_req_xml)
    response_entry = request_entry(fal_info.FAL_DEVCTRL_PATH, entry)

    if response_entry.get_result() == Results.OK:
        return True

    return False


def get_volume_status(vol_name):
    """Check whether specified device exist"""

    VOLUME_QUERY_REQ_XML_TMPL % vol_name

    entry = request.Entry(ReqTypes.QUERY,
                          VOLUME_QUERY_REQ_XML_TMPL % vol_name)
    response_entry = request_entry(fal_info.FAL_VOLCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        return ""

    return response_entry.get().get_value("volume/status")


def play_melody(melody):
    """Play melody"""

    entry = request.Entry(ReqTypes.PLAY,
                          PLAY_MELODY_BASE_REQ_XML_TMPL % melody)
    return request_entry(fal_info.FAL_BUZZER_PATH, entry)


def stop_melody(melody=None):
    """Stop melody.
       If 'melody' is specified just corresponding melody is stopped.
    """

    STOP_WITH_MELODY_REQ_XML = "<buzzer><melody type='%s'/></buzzer>"
    STOP_REQ_XML = "<buzzer/>"

    if melody:
        req_xml = STOP_WITH_MELODY_REQ_XML % melody
    else:
        req_xml = STOP_REQ_XML

    entry = request.Entry(ReqTypes.STOP, req_xml)
    return request_entry(fal_info.FAL_BUZZER_PATH, entry)


def change_status_led(color, mode):
    """Change STATUS LED to specified mode.
       "color" can be set "green" or "red"
       "led_mode" can be set "blink" or "on".
    """

    STATUS_LED_BASE_REQ_XML = \
        "<led><status color=\"%s\">%s</status></led>"

    entry = request.Entry(ReqTypes.CONTROL,
                          STATUS_LED_BASE_REQ_XML % (color, mode))
    response_entry = request_entry(fal_info.FAL_LED_PATH, entry)
    if response_entry.get_result() != Results.OK:
        raise RequestError

    return


def change_hdd_led(hdd_led_params_list):
    """Change HDD LED to specified mode.
       hdd_led_params_list is the list of dictionarys
          [{"hddnum": "1", "color": "red", "mode": "on" },
           {"hddnum": "2", "color": "red", "mode": "blink" },,,]
    """

    LED_BASE_XML = "<led><hdd><statuslist/></hdd></led>"
    STATUS_XML_TMPL = \
        "<status color='%(color)s' hddnum='%(hddnum)s'>%(mode)s</status>"
    STATUSLIST_XPATH = "hdd/statuslist"

    led_node = XmlNode(LED_BASE_XML)
    led_node.get_node(STATUSLIST_XPATH)
    for params in hdd_led_params_list:
        status_node = XmlNode(STATUS_XML_TMPL % params)
        led_node.get_node(STATUSLIST_XPATH).append_child(status_node)

    entry = request.Entry(ReqTypes.CONTROL, led_node)
    response_entry = request_entry(fal_info.FAL_LED_PATH, entry)
    if response_entry.get_result() != Results.OK:
        raise RequestError

    return


def mount_internal_volume():
    """Unmount internal volume"""

    entry = request.Entry(ReqTypes.MOUNT,
                          MOUNT_INTERNAL_REQ_XML)
    response_entry = request_entry(fal_info.FAL_VOLCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError
    return


def unmount_internal_volume():
    """Unmount internal volume"""

    entry = request.Entry(ReqTypes.UNMOUNT,
                          UNMOUNT_INTERNAL_REQ_XML)
    response_entry = request_entry(fal_info.FAL_VOLCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError
    return


def unmount_volume(vol_name):

    entry = request.Entry(ReqTypes.UNMOUNT,
                          UNMOUNT_REQ_XML % vol_name)
    response_entry = request_entry(fal_info.FAL_VOLCTRL_PATH, entry)
    if response_entry.get_result() != Results.OK:
        raise RequestError
    return


def unmount_all_usb_volume():

    for vol_name in VOL_INFO.keys():
        try:
            unmount_volume(vol_name)
        except:
            continue


def is_mirroring_setting(usb_disk_name="usbd3"):

    USB_PORT_ENABLE_MODE_XPATH = \
        "usblist/usb[@port='%d']/modelist/mode[@enable='true']"

    if not usb_disk_name.startswith("usbd"):
        return False

    entry = request.Entry(ReqTypes.LOAD)
    portconf_node = request_entry(fal_info.FAL_PORTS_PATH, entry).get()
    port_num = int(config.DEV_TO_PORT_NO[usb_disk_name])
    mode = portconf_node.get_node(
               USB_PORT_ENABLE_MODE_XPATH % port_num).get_text()

    return MIRRORING_PORT_MODE == mode


def is_external_vfsext_setting(usb_disk_name):

    USB_PORT_ENABLE_MODE_XPATH = \
        "usblist/usb[@port='%d']/modelist/mode[@enable='true']"

    if not usb_disk_name.startswith("usbd"):
        return False

    entry = request.Entry(ReqTypes.LOAD)
    portconf_node = request_entry(fal_info.FAL_PORTS_PATH, entry).get()
    port_num = int(config.DEV_TO_PORT_NO[usb_disk_name])
    mode = portconf_node.get_node(
               USB_PORT_ENABLE_MODE_XPATH % port_num).get_text()

    return VFSEXT_PORT_MODE == mode


def is_share_mode_by_port_num(port_num):

    port_config = request_entry(
        fal_info.FAL_PORTS_PATH,
        request.Entry(ReqTypes.LOAD)).get()

    mode = port_config.get_value(USB_PORT_MODE_PATH % port_num)

    if mode == SHARE_MODE:
        return True
    return False


def is_share_mode_by_vol_name(vol_name):

    USB_LIST = ["usb1", "usb2", "usb3", "usb4"]

    if not vol_name in USB_LIST:
        return True

    port_num = re.sub(r"^usb", "", vol_name)

    if is_share_mode_by_port_num(port_num):
        return True
    return False


def is_mounted(vol_name):

    result_entry = request_entry(
        fal_info.FAL_VOLCTRL_PATH,
        request.Entry(ReqTypes.QUERY, XmlNode(
            VOLUME_QUERY_REQ_XML_TMPL % vol_name)))

    if result_entry.get_result() == Results.DEVICE_NOT_FOUND:
        return False

    if result_entry.get().get_value(STATUS_PATH) == STATUS_MOUNTED:
        return True

    return False


def is_mounted_internal_volume():

    return is_mounted(INTERNAL_VOL)


def is_internal_vol_mnt_event(entry):

    body = entry.get()

    action = body.get_value(ACTION_XPATH)
    vol_name = body.get_value(VOL_NAME_XPATH)

    if vol_name != INTERNAL_VOL:
        if action != "mount":
            return False
    return True


def dispatch_event(category, action, body=None, previous=False):
    """Dispatch event."""

    entry = request.Entry(ReqTypes.EVENT,
                fal_info.FAL_EVENTS_REQ_BASE_XML[ReqTypes.EVENT])

    attrs = {CATEGORY_ATTR: category, ACTION_ATTR: action}
    if previous:
        attrs[PREVIOUS_ATTR] = TRUE

    entry.get().get_node(EVENTINFO_NAME).set_attrs(attrs)
    if body != None:
        if isinstance(body, (str, unicode)):
            body = XmlNode(body)
        entry.get().get_node(EVENTINFO_NAME).append_child(body)

    return request_entry(fal_info.FAL_EVENTS_PATH, entry)


#
# FAL_RAIDCTRL Control Functions
#

RAID_QUERY_REQ_XML_TMPL = \
    '<raidctrl><raid><name type="raid">%s</name></raid></raidctrl>'

RAID_START_REQ_XML_TMPL = \
    """<raidctrl>
         <option force="false" autorebuild="false"/>
         <raid>
          <name type="volume">sataraid1</name>
         </raid>
       </raidctrl>"""

RAID_STOP_REQ_XML_TMPL = \
    '<raidctrl><raid><name type="raid">%s</name></raid></raidctrl>'

RAID_CRASH_STOP_REQ_XML_TMPL = \
    """<raidctrl><option crash="true"/>
         <raid><name type="raid">%s</name></raid>
       </raidctrl>"""

RAID_CRASH_STOP_AS_VOL_REQ_XML_TMPL = \
    """<raidctrl><option crash="true"/>
         <raid><name type="volume">%s</name></raid>
       </raidctrl>"""

RAID_STOP_AS_VOL_REQ_XML_TMPL = \
    """<raidctrl>
         <raid><name type="volume">%s</name></raid>
       </raidctrl>"""

RAID_REMOVE_REQ_XML_TMPL = \
    '<raidctrl><disk><name>%s</name></disk></raidctrl>'

RAID_REMOVE_PART_REQ_XML_TMPL = \
    """<raidctrl>
         <raid>
           <name type="raid">%s</name>
           <partition>
             <name>%s</name>
           </partition>
         </raid>
       </raidctrl>"""

RAID_CHK_CRASH_WITH_DISK_REQ_XML_TMPL = \
    """<raidctrl>
         <option type="crash"/>
         <disk><name>%s</name></disk>
       </raidctrl>"""

RAID_CHK_CRASH_WITH_RAID_REQ_XML_TMPL = \
    """<raidctrl>
         <option type="crash"/>
         <disk><name>%s</name></disk>
       </raidctrl>"""

RAID_CHK_GET_OPERATION_REQ_XML = \
    """<raidctrl>
         <option type="getoperation"/>
       </raidctrl>"""
       
RAID_CHK_OTHER_MIRROR_REQ_XML_TMPL = \
    """<raidctrl>
         <option type="othermirror"/>
         <volume><name>%s</name></volume>
       </raidctrl>"""

INTERNAL_DEV = "md6"
MIRROR_DEV = "md13"


def fal_raidctrl_query(raid_name, depth=None):

    # FIXME type="volume"の場合にはまだ対応していない

    entry = request.Entry(ReqTypes.QUERY,
                          RAID_QUERY_REQ_XML_TMPL % raid_name)
    response_entry = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get()


def fal_raidctrl_stop(raid_name, is_crash=False, is_volume=False):

    if is_volume:
        if is_crash:
            entry = request.Entry(ReqTypes.STOP,
                                  RAID_CRASH_STOP_AS_VOL_REQ_XML_TMPL % \
                                  raid_name)
        else:
            entry = request.Entry(ReqTypes.STOP,
                                  RAID_STOP_AS_VOL_REQ_XML_TMPL % raid_name)
    else:
        if is_crash:
            entry = request.Entry(ReqTypes.STOP,
                                  RAID_CRASH_STOP_REQ_XML_TMPL % raid_name)
        else:
            entry = request.Entry(ReqTypes.STOP,
                                  RAID_STOP_REQ_XML_TMPL % raid_name)

    response_entry = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get()


def fal_raidctrl_remove(disk_name):

    entry = request.Entry(ReqTypes.REMOVE,
                          RAID_REMOVE_REQ_XML_TMPL % disk_name)
    response_entry = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get()


def fal_raidctrl_remove_partition(raid_name, part_name):

    entry = \
        request.Entry(ReqTypes.REMOVE,
                      RAID_REMOVE_PART_REQ_XML_TMPL % (raid_name, part_name))
    response_entry = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get()


def fal_raidctrl_start():

    entry = request.Entry(ReqTypes.START, RAID_START_REQ_XML_TMPL)

    response_entry = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get()


def is_raid_crash_or_stop(raid_name):

    STATUS_XPATH = "raid/status"

    try:
        raidctrl_node = fal_raidctrl_query(raid_name)
    except:
        return False

    status = raidctrl_node.get_value(STATUS_XPATH)
    if CRASH_STATUS == status or STOP_STATUS == status:
        return True

    return False


#
# FAL_VOLCTRL Control Functions
#

VOLUME_UPDATE_XML_TMPL = \
    '<volctrl><volume><name type="volume">%s</name></volume></volctrl>'


def fal_volctrl_update(vol_name, status=""):

    VOLUME_UPDATE_XML_TMPL = \
        '<volctrl><volume><name type="volume">%s</name></volume></volctrl>'

    entry = request.Entry(ReqTypes.UPDATE,
                          VOLUME_UPDATE_XML_TMPL % vol_name)
    if status:
        entry.get().get_node("volume").append_child("status", status)

    response_entry = request_entry(fal_info.FAL_VOLCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return


def fal_volctrl_query(vol_name):

    VOLUME_QUERY_XML_TMPL = \
        '<volctrl><volume><name type="volume">%s</name></volume></volctrl>'

    entry = request.Entry(ReqTypes.QUERY,
                          VOLUME_QUERY_XML_TMPL % vol_name)
    response_entry = request_entry(fal_info.FAL_VOLCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get()


#
# FAL_DISKCTRL Control Functions
#

def fal_disk_query(disk_name):

    DISK_QUERY_XML_TMPL = \
        '<diskctrl><disk><name>%s</name></disk></diskctrl>'

    entry = request.Entry(ReqTypes.QUERY,
                          DISK_QUERY_XML_TMPL % vol_name)
    response_entry = request_entry(fal_info.FAL_VOLCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get()


#
# Ports event utils
#

VOL_ENTRY = "<volctrl><volume><name type='volume'>%s</name></volume></volctrl>"
EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
USB_ORIG_SUFFIX = ".real"


def ports_event(entry, falpal, file_dir):
    def __get_vol_status(vol):
        try:
            entry = request.Entry(ReqTypes.QUERY, VOL_ENTRY % vol)
            response = request_entry(fal_info.FAL_VOLCTRL_PATH, entry)
            return response.get().get_node("volume/status").get_text()
        except:
            return "disconnected"

    def __mount_or_unmount(type, vol):
        vol_status = __get_vol_status(vol)
        if type == "mount":
            if vol_status == "mounted":
                return Results.OK
        else:
            if vol_status == "unmounted" or vol_status == "disconnected":
                return Results.OK
        dev_name = VOL_INFO[vol]["dev"]
        dev_path = os.path.join(config.DEV_DIR, dev_name)
        if not os.path.exists(dev_path):
            return Results.OK

        entry = request.Entry(type, VOL_ENTRY % vol)
        response = request_entry(fal_info.FAL_VOLCTRL_PATH, entry)
        return response.get_result()

    def __remove_usb_share_link(port):
        for suffix in [".ro", ".rw"]:
            file_name = port + suffix
            file_path = os.path.join(file_dir, port, file_name)
            if not os.path.exists(file_path):
                continue
            os.remove(file_path)
        return

    def __create_usb_share_link(port):
        for suffix in [".ro", ".rw"]:
            file_name = port + suffix
            file_path = os.path.join(file_dir, port, file_name)
            if os.path.exists(file_path):
                continue
            src_file_path = file_path + USB_ORIG_SUFFIX
            if os.path.exists(src_file_path):
                os.symlink(src_file_path, file_path)
        return

    def __update_request(entry):
        event_data = entry.get().get_node(EVENTINFO_NAME)
        usblist = event_data.get_node_list("./ports/usblist/usb")
        if len(usblist) < 1:
            entry.set_result(Results.OK)
            return entry
        for usb in usblist:
            port_num = usb.get_attr_value("port")
            enable_mode = usb.get_node(
                            "modelist/mode[@enable='true']").get_text()
            if enable_mode == "share":
                __create_usb_share_link("usb" + port_num)
                result = __mount_or_unmount(
                            ReqTypes.MOUNT, "usb" + port_num)
            else:
                __remove_usb_share_link("usb" + port_num)
                result = __mount_or_unmount(
                            ReqTypes.UNMOUNT, "usb" + port_num)
            if result != Results.OK:
                entry.set_result(result)
                return entry
        return __update_conf_file()

    def __update_conf_file():
        return request_entry(falpal, request.Entry(ReqTypes.UPDATE))

    ### Main ##############

    category = entry.get().get_node(
                EVENTINFO_NAME).get_attr_value(CATEGORY_ATTR)
    if category != "ports":
        entry.set_result(Results.EVENT_ERROR)
        return entry

    update_result = __update_request(entry)
    if update_result.get_result() != Results.OK:
        entry.set_result(update_result.get_result())
        return entry

    entry.set_result(Results.OK)
    return entry


#
# FAL_DISKCTRL Control Functions
#

def fal_diskctrl_update(disk_name, info_type, value, is_inc_err=False):

    DISK_UPDATE_XML_TMPL = \
        """<diskctrl>
            <option incerr="false"/>
            <disk><name>%s</name></disk>
           </diskctrl>"""

    entry = request.Entry(ReqTypes.UPDATE,
                          DISK_UPDATE_XML_TMPL % disk_name)

    entry.get().get_node("./disk").append_child(info_type)
    entry.get().get_node("./disk/%s" % info_type).set_text(value)

    if is_inc_err:
        entry.get().get_node("./option").set_attrs({"incerr": "true"})

    response_entry = request_entry(fal_info.FAL_DISKCTRL_PATH, entry)
    if response_entry.get_result() != Results.OK:
        raise RequestError

    return


def get_disk_status(disk_name):

    DISK_QUERY_XML_TMPL = \
        '<diskctrl><disk><name>%s</name></disk></diskctrl>'

    entry = request.Entry(ReqTypes.QUERY,
                          DISK_QUERY_XML_TMPL % disk_name)

    response_entry = request_entry(fal_info.FAL_DISKCTRL_PATH, entry)
    result = response_entry.get_result()
    if result != Results.OK:
        # DEVICE_NOT_FOUND等のresultcodeを
        # 呼び出し元で判断するためresultを引数に。
        raise RequestError(result)

    return response_entry.get().get_value("./disk/status")


def delete_disk_from_system(disk_name):

    DISK_DELETE_XML_TMPL = \
        '<diskctrl><disk><name>%s</name></disk></diskctrl>'

    entry = request.Entry(ReqTypes.DELETE,
                          DISK_DELETE_XML_TMPL % disk_name)
    response_entry = request_entry(fal_info.FAL_DISKCTRL_PATH, entry)
    if response_entry.get_result() != Results.OK:
        raise RequestError

    return


#
# FAL_LED Control Functions
#


def fal_led_update():

    entry = request.Entry(ReqTypes.UPDATE)
    response_entry = request_entry(fal_info.FAL_LED_PATH, entry)
    if response_entry.get_result() != Results.OK:
        raise RequestError

    return


def is_boot_complete():

    if os.path.exists(config.BOOT_COMPLETE_FILE_PATH):
        return True
    return False


def is_tried_first_mount():

    if os.path.exists(config.TRIED_FIRST_MOUNT_FILE_PATH):
        return True
    return False


def save_tried_first_mount_file():

    open(config.TRIED_FIRST_MOUNT_FILE_PATH, "w").close()
    return


#
# Manuals Control Functions
#


def copy_manuals(man_list=[]):

    if not os.path.exists(MAN_DIR):
        return

    for man in os.listdir(MAN_DIR):
        if man_list and not man in man_list:
            continue

        src_path = os.path.join(MAN_DIR, man)
        dst_path = os.path.join(MAN_SHARE_PATH, man)

        try:
            shutil.copy(src_path, dst_path)
            os.chmod(dst_path, MODE_CH_FILE)
        except BaseException, ins:
            if os.path.exists(dst_path):
                os.unlink(dst_path)
            raise ins


def _set_manual(man_list=[]):

    share_conf = request_entry(fal_info.FAL_SHARES_PATH,
                               request.Entry(ReqTypes.LOAD)).get()

    try:
        share_conf.get_node(MAN_XPATH)
    except NotFoundError:
        if not is_mounted_internal_volume():
            raise Exception

        shares_node = XmlNode(open(DEFAULT_SHARES_XML_PATH).read())
        landiskmanual_node = shares_node.get_node(MAN_XPATH)

        req_node = XmlNode("shares")
        req_node.append_child(landiskmanual_node)

        res_entry = request_entry(
            fal_info.FAL_SHARES_PATH,
            request.Entry(ReqTypes.ADD, req_node))

        if res_entry.get_result() != Results.OK:
            raise Exception

    copy_manuals(man_list)


def regist_man_set_event(entry, handler):
    if is_internal_dev_name(entry):
        entry = request.Entry(ReqTypes.REGISTER,
            MAN_SET_EVENT_XML, file_name=handler)
        request_entry(fal_info.FAL_EVENTS_PATH, entry)
    entry.set_result(Results.OK)
    return entry


def unregist_man_set_event(handler):
    entry = request.Entry(ReqTypes.UNREGISTER,
        MAN_SET_EVENT_XML, file_name=handler)
    request_entry(fal_info.FAL_EVENTS_PATH, entry)


def set_manual(entry, man_list, handler):
    if is_internal_vol_name(entry):
        _set_manual(man_list)
        unregist_man_set_event(handler)
    entry.set_result(Results.OK)
    return entry


#
# Check internal name functions
#
def is_internal_dev_name(entry):

    DEVICE_XPATH = "./eventinfo/device/name"

    dev_name = entry.get().get_value(DEVICE_XPATH)

    internal_dev_name = VOL_INFO[get_internal_vol_name()]["dev"]

    if internal_dev_name == dev_name:
        return True

    if DEV_TO_ENC_DEV[internal_dev_name] == \
           os.path.basename(dev_name):
        return True

    return False


def is_internal_vol_name(entry):

    VOLUME_XPATH = "./eventinfo/volume/name"

    vol_name = entry.get().get_value(VOLUME_XPATH)

    if get_internal_vol_name() == vol_name:
        return True

    return False


def check_crash(remove_target_name):

    NAME_NAME = "name"
    RESULT_NAME = "result"
    CRASH_RESULT = "crash"
    NO_CRASH_RESULT = "nocrash"
    RAID_XPATH = "result/raidlist/raid"

    if remove_target_name in config.HDD_DEV_LIST:
        req_xml = RAID_CHK_CRASH_WITH_DISK_REQ_XML_TMPL % remove_target_name
    elif remove_target_name in config.RAID_DEV_LIST:
        req_xml = RAID_CHK_CRASH_WITH_RAID_REQ_XML_TMPL % remove_target_name
    else:
        raise InvalidArgError

    entry = request.Entry(ReqTypes.CHECK, req_xml)

    response_entry = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    crash_map = {}
    raid_node_list = response_entry.get().get_node_list(RAID_XPATH)
    for raid_node in raid_node_list:

        raid_name = raid_node.get_node(NAME_NAME).get_text()
        result = raid_node.get_node(RESULT_NAME).get_text()

        if CRASH_RESULT == result:
            crash_map[raid_name] = True
        elif NO_CRASH_RESULT == result:
            crash_map[raid_name] = False
        else:
            pass

    return crash_map


def get_product_real_raiddev_name(raiddev_name):

    if raiddev_name in config.RAID_TO_REAL_NAME:
        return config.RAID_TO_REAL_NAME[raiddev_name]

    return raiddev_name


def is_crash_if_remove(raid_name, remove_target_name):

    crash_map = check_crash(remove_target_name)

    real_raid_name = get_product_real_raiddev_name(raid_name)
    if real_raid_name in crash_map.keys():
        return crash_map.get(real_raid_name)

    raise NotFoundError


def is_disk_name(dev_name):

    return dev_name in \
        SATA_HDD_DEV_LIST + USB_HDD_DEV_LIST + ESATA_HDD_DEV_LIST


def is_raid_element_name(dev_name):

    MISSING_DEVICE = "missing"

    for raid_name, element_list in get_raid_element_info().iteritems():

        tmp_element_list = list(element_list)

        while MISSING_DEVICE in tmp_element_list:
            tmp_element_list.remove(MISSING_DEVICE)

        if dev_name in tmp_element_list:
            return True

    return False


def is_raid_busy():

    OPERATION_NAME = "operation"
    NON_RAID_OPERATION = "none"

    entry = request.Entry(ReqTypes.CHECK,
                          RAID_CHK_GET_OPERATION_REQ_XML)

    response_entry = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    op = response_entry.get().get_node(OPERATION_NAME).get_text()
    return NON_RAID_OPERATION != op


def wait_for_raid_operable():

    while True:
        if not is_raid_busy():
            break


def _get_cuurent_array_num(vol_name=config.INTERNAL_VOL):

    RAID_LEVEL_XPATH = \
        "raidlist/raid/name[text()='%s']/../level" % vol_name

    entry = request.Entry(ReqTypes.LOAD)
    raidctrl_conf_node = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry).get()
    raid_mode = raidctrl_conf_node.get_value(RAID_LEVEL_XPATH)

    return config.RAID_MODE_INFO[raid_mode]["array_num"]


def get_raid_element_info(raid_mode=None):

    if raid_mode is None:
        # 現在の設定から
        return config.RAID_TO_ELEMENT[_get_cuurent_array_num()]

    # 指定されたraid modeから
    array_num = config.RAID_MODE_INFO[raid_mode]["array_num"]
    return config.RAID_TO_ELEMENT[array_num]

def get_current_raid_level(vol_name=config.INTERNAL_VOL):

    RAID_LEVEL_XPATH = \
        "raidlist/raid/name[text()='%s']/../level" % vol_name

    entry = request.Entry(ReqTypes.LOAD)
    raidctrl_conf_node = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry).get()
    return raidctrl_conf_node.get_value(RAID_LEVEL_XPATH)

def is_auto_extend(vol_name=config.INTERNAL_VOL):

    AUTO_EXTEND_ATTR = "autoextend"
    RAID_XPATH = \
        "raidlist/raid/name[text()='%s']/.." % vol_name

    entry = request.Entry(ReqTypes.LOAD)
    raidctrl_conf_node = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry).get()
    return raidctrl_conf_node.get_node(
               RAID_XPATH).is_set_to_true(AUTO_EXTEND_ATTR)

def is_current_raid_mode_vfsext(vol_name=config.INTERNAL_VOL):

    RAID_LEVEL_XPATH = \
        "raidlist/raid/name[text()='%s']/../level" % vol_name

    entry = request.Entry(ReqTypes.LOAD)
    raidctrl_conf_node = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry).get()
    raid_mode = raidctrl_conf_node.get_value(RAID_LEVEL_XPATH)

    return config.RAID_MODE_INFO[raid_mode]["is_ext_raid"]


def save_unlock_event_log(vol_name):

    UNLOCK_LOG_CODE_MAP = {}
    messages = {
        "internal": {
            1: msginfo.Msg.MSG_LOCKKEY_SECURITY_UNLOCK,
        },
        "usb": {
            1: msginfo.Msg.MSG_LOCKKEY_USB1_SECURITY_UNLOCK,
            2: msginfo.Msg.MSG_LOCKKEY_USB2_SECURITY_UNLOCK,
            3: msginfo.Msg.MSG_LOCKKEY_USB3_SECURITY_UNLOCK,
            4: msginfo.Msg.MSG_LOCKKEY_USB4_SECURITY_UNLOCK,
        },
    }
    for storif, vols in STOR_IF_TO_VOL.iteritems():
        for i, vol in vols.iteritems():
            try:
                UNLOCK_LOG_CODE_MAP[vol] = messages[storif][i]
            except:
                continue

    if vol_name in UNLOCK_LOG_CODE_MAP:
        save_event_log(UNLOCK_LOG_CODE_MAP[vol_name])
    return


def extract_enckey():

    EXTRACT_LOCKKEY_REQ_XML = \
        '<lockkey>' + \
        ' <option operation="extract"/>' + \
        '</lockkey>'

    entry = request.Entry(ReqTypes.SET_LOCKKEY, EXTRACT_LOCKKEY_REQ_XML)
    result_entry = request_entry(fal_info.FAL_LOCKKEY_PATH, entry)
    result_code = result_entry.get_result()

    if Results.NO_LOCKKEY == result_code:
        raise RequestError
    return


def set_quota_limit(volume):
    """Set quota limit to "volume"."""

    quota_entry = request.Entry(ReqTypes.APPLY, QUOTA_XML)
    quota_entry.get().append_child(_get_user_quota_list(volume))
    quota_entry.get().append_child(_get_group_quota_list(volume))

    response_entry = request.Request(
        fal_info.FAL_QUOTA_PATH).request(quota_entry).get_entry()
    if response_entry.get_result() != Results.OK:
        raise RequestError


def load_config(conf_file):
    """Load config setting and return XML data ."""

    entry = request.Entry(
        ReqTypes.LOAD, pal_info.PAL_CONFFILE_REQ_BASE_XML[ReqTypes.LOAD],
        file_name=conf_file)
    return request_entry(pal_info.PAL_CONFFILE_PATH, entry).get()

def spindown(disk_name):
    """Spin down disk."""

    SPINDOWN_REQ_XML_TMPL = \
       '<diskctrl>' + \
        '<option operation="spindown"/>' + \
          '<disk><name>%s</name></disk>' + \
       '</diskctrl>'

    entry = request.Entry(ReqTypes.CONTROL, SPINDOWN_REQ_XML_TMPL % disk_name)
    response_entry = request.Request(fal_info.FAL_DISKCTRL_PATH
                                        ).request(entry).get_entry()
    if response_entry.get_result() != Results.OK:
        raise RequestError


def spindown_unuse_disk(vol_name=config.INTERNAL_VOL):

    RAID_LEVEL_XPATH = \
        "raidlist/raid/name[text()='%s']/../level" % vol_name

    entry = request.Entry(ReqTypes.LOAD)
    raidctrl_conf_node = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry).get()
    cur_raid_level = raidctrl_conf_node.get_value(RAID_LEVEL_XPATH)

    cur_array_num = \
        config.RAID_MODE_INFO[cur_raid_level]["array_num"]
    cur_disk_list = \
        config.HDD_DEV_LIST_IN_VOL[cur_array_num][config.INTERNAL_VOL]

    for disk_name in config.SATA_HDD_DEV_LIST:
        if disk_name not in cur_disk_list:
            spindown(disk_name)


def check_mirror_state(vol_name):

    RESULT_NAME = "result"

    entry = request.Entry(ReqTypes.CHECK,
                          RAID_CHK_OTHER_MIRROR_REQ_XML_TMPL % vol_name)
    response_entry = request_entry(fal_info.FAL_RAIDCTRL_PATH, entry)

    if response_entry.get_result() != Results.OK:
        raise RequestError

    return response_entry.get().get_value(RESULT_NAME)
