# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command
from ionas.request import Entry
from iobase.xmldata import xpath_escape
from network import Network
from sysinfo import SysInfo
from share import Share
from volctrl import Volctrl
from fal_info import Results

import re

SHARE_PATH_XPATH = "sharelist/share[name=%s]/path"
FILESYSTEM_KEY = "filesystem"

EXCLUSIVE_FS_LIST = ["xfs", "ext3", "ext4", "vfsext"]

VOL_SEP = "/"
VOLUME_IDX = 1

UNC_RE = r"^\\\\([^\\]+)\\(.+)$"


class Backups(Command):
#    COMMAND_NAME = 'fal_backups'
    try:
        from fal_info import FAL_BACKUPS_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_backups'

    TYPE_LOAD = 'load'
    TYPE_QUERY = 'query'
    TYPE_RESTORE = 'restore'
    TYPE_EXECUTE = 'execute'
    TYPE_STOP = 'stop'

    def __init__(self, *args, **kwargs):
        super(Backups, self).__init__(*args, **kwargs)
        self.conf_dict = {}
        self.local_host_list = []

    def _load(self):
        entry = Entry(self.TYPE_LOAD, '<backups />')
        return self.request(entry)

    def _query(self):
        entry = Entry(
                self.TYPE_QUERY,
                """<backups><option detail="true" /></backups>""")
        return self.request(entry)

    def _restore(self, jobno, generation):
        entry = Entry(
                self.TYPE_RESTORE,
                """<backups>
                    <backup>
                        <job>
                            <id>%(jobno)s</id>
                            <generation>
                                <id>%(generation)s</id>
                            </generation>
                        </job>
                    </backup>
                </backups>""" % {
                    'jobno': jobno,
                    'generation': generation,
                })
        return self.request(entry)

    def _execute(self, jobid):
        entry = Entry(self.TYPE_EXECUTE, """<backups>
            <backup>
                <job>
                    <id>%(id)s</id>
                </job>
            </backup>
        </backups>""" % {'id': jobid})

        try:
            entry.get().get_node('backup').delete_child('job')
            entry.get().get_node('backup').append_child(self.load(to_dict=False).get().get_node('backup/joblist/job[id="%s"]' % jobid))
        except:
            pass

        return self.request(entry)

    def _stop(self, jobid, function):
        entry = Entry(self.TYPE_STOP, """<backups>
            <%(function)s>
                <job>
                    <id>%(id)s</id>
                </job>
            </%(function)s>
        </backups>""" % {
            "id": jobid,
            "function": function,
        })
        return self.request(entry)

    def __get_schedule_dict(self, node):
        try:
            sch_node = node.get_node("schedule")
            return {
                "enable": self.text_to_bool(sch_node.get_value("@enable")),
                "day": sch_node.get_value_list("day"),
                "hour": sch_node.get_value("hour"),
                "time": sch_node.get_value("time")}
        except:
            return {}

    def __get_sourcelist(self, node):
        try:
            list_ = []
            src_node_list = node.get_node_list("sourcelist/source")
            if not src_node_list:
                # データコピー時
                src_node_list = node.get_node_list("source")
            for src_node in src_node_list:
                list_ += [{
                    "unc": src_node.get_value("unc", do_strip=False),
                    "user": src_node.get_value("user", do_strip=False),
                    "passwd": src_node.get_value("passwd", do_strip=False)}]
            return list_
        except:
            return []

    def __get_destination(self, node):
        try:
            dst_node = node.get_node("destination")
            return {
                "unc": dst_node.get_value("unc", do_strip=False),
                "user": dst_node.get_value("user", do_strip=False),
                "passwd": dst_node.get_value("passwd", do_strip=False)}
        except:
            return {}

    def __get_job_dict(self, node):
        try:
            return {
                "name": node.get_value("name", do_strip=False),
                "times": node.get_value("times"),
                "schedule": self.__get_schedule_dict(node),
                "trash": self.text_to_bool(node.get_value("trash/@enable")),
                "shutdown": self.text_to_bool(
                    node.get_value("shutdown/@enable")),
                "all_local": self.text_to_bool(
                    node.get_value("all_local/@enable")),
                "sourcelist": self.__get_sourcelist(node),
                "destination": self.__get_destination(node)}
        except:
            return {}

    def __get_jobs_dict(self, node):
        try:
            dict_ = {}
            for job_node in node.get_node_list("joblist/job"):
                job_id = job_node.get_value("id")
                dict_[job_id] = self.__get_job_dict(job_node)
            return dict_
        except:
            return {}

    def __get_dict(self, node):
        try:
            dict_ = {}
            for n in node.get_node_list("*"):
                category = n.get_node_name()
                dict_[category] = self.__get_jobs_dict(n)
            return dict_
        except:
            return {}

    def __to_dict(self, entry):
        try:
            return self.__get_dict(entry.get())
        except:
            return {}

    def __get_local_host_list(self):
        try:
            si = SysInfo()
            nw = Network()
            local_host_name = si.query()["host"]["name"].lower()
            local_host_addr_list = [
                node.get_text().lower()
                for node
                in nw.query(to_dict=False).get().get_node_list(
                    "netiflist/netif/protocollist/protocol/addrlist/addr")]
            return ["localhost", "127.0.0.1", local_host_name] +\
                [local_host_addr_list]
        except:
            return []

    def __unc_to_host_share(self, unc):
        match = re.match(UNC_RE, unc)
        if match:
            host = match.group(1)
            share = match.group(2)
        else:
            host = "localhost"
            share = unc
        return (host, share)

    def __get_local_share_path(self, share_name):
        share = Share()
        share_conf = share.load(to_dict=False).get()
        return share_conf.get_value(
            SHARE_PATH_XPATH % xpath_escape(share_name), "")

    def __path_to_vol_info_dict(self, path):
        vc = Volctrl()
        vol = path.lstrip(VOL_SEP).split(VOL_SEP)[VOLUME_IDX]
        return vc.query(vol)

    def __path_to_filesystem(self, path):
        vol_info_dict = self.__path_to_vol_info_dict(path)
        return vol_info_dict.get(FILESYSTEM_KEY)

    def __get_share_fs(self, share_name):
        return self.__path_to_filesystem(
            self.__get_local_share_path(share_name))

    def query(self, to_dict=True):
        def _to_dict(entry):
            dict_ = dict()
            try:
                root_nd = entry.get()
                for func in ['backup', 'datacopy']:
                    dict_[func] = dict()
                    func_nd = root_nd.get_node(func)
                    for job_nd in func_nd.get_node_list('joblist/job'):
                        id_ = job_nd.get_value('id')
                        dict_[func][id_] = dict()
                        dict_[func][id_]['id'] = id_
                        dict_[func][id_]['status'] = job_nd.get_value('status')
                        dict_[func][id_]['generationlist'] = [nd.get_value('id') \
                                for nd in job_nd.get_node_list('generationlist/generation')]

                for func in ['restore']:
                    dict_[func] = dict()
                    func_nd = root_nd.get_node(func)
                    for job_nd in func_nd.get_node_list('joblist/job'):
                        id_ = job_nd.get_value('id')
                        dict_[func][id_] = dict()
                        dict_[func][id_]['id'] = id_
                        dict_[func][id_]['status'] = job_nd.get_value('status')
            except:
                pass
            return dict_
        entry = self._query()
        return _to_dict(entry) if to_dict else entry
        #return self._query()

    def load(self, to_dict=True):
        entry = self._load()
        return self.__to_dict(entry) if to_dict else entry

    def restore(self, jobno, generation, name="Job", to_bool=True, log=True):
        entry = self._restore(jobno, generation)
        result = entry.get_result()
        _MSGCODE = {
            Results.OK: 'MSG_BACKUP_RESTORE_START',
            Results.QUEUE: 'MSG_BACKUP_RESTORE_QUEUE',
            Results.BUSY_ERROR: 'MSG_BACKUP_STATE_CHANGE',
            None: 'ERR_BACKUP_RESTORE_OPERATION',
        }
        _code = _MSGCODE.get(result, _MSGCODE[None])
        _arglist = {'name':name} if _code == 'MSG_BACKUP_RESTORE_QUEUE' else {}
        self._set_message(_code, arglist=_arglist, log=log)
        return (result in [Results.OK, Results.QUEUE]) if to_bool else entry

    def execute(self, jobid, log=True, to_bool=True, default_messages={}):

        try:
            name = self.load(to_dict=False).get().get_node('backup/joblist/job[id="%s"]' % jobid).get_value('name', '')
        except:
            name = ''

        _default_messages = {
            Results.OK: ['MSG_BACKUP_START', {}],
            Results.QUEUE: ['MSG_BACKUP_QUEUE', {'name': name}],
            Results.BUSY_ERROR: ['MSG_BACKUP_STATE_CHANGE', {}],
            None: ['ERR_BACKUP_OPERATION', {}],
        }
        _default_messages.update(default_messages)

        entry = self._execute(jobid)
        result = entry.get_result()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK, Results.QUEUE]) if to_bool else entry

    def stop(self, jobid, function="backup", log=True, to_bool=True, default_messages={}):
        _default_messages = {
            Results.OK: ['MSG_BACKUP_EXEC_STOP', {}],
            None: ['ERR_BACKUP_EXEC_STOP', {}],
        }
        _default_messages.update(default_messages)

        entry = self._stop(jobid, function=function)
        result = entry.get_result()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK]) if to_bool else entry

    def is_bkup_dst(self, share_name,
        category="backup", fs_list=EXCLUSIVE_FS_LIST):
        try:
            if not self.conf_dict:
                self.conf_dict = self.load()
            if not self.local_host_list:
                self.local_host_list = self.__get_local_host_list()
            for job_id in self.conf_dict[category].keys():
                dst_unc = self.conf_dict.get(
                    category, {}).get(
                    job_id, {}).get(
                    "destination", {}).get(
                    "unc")
                dst_host, dst_share = self.__unc_to_host_share(dst_unc)
                if dst_host.lower() in self.local_host_list:
                    if dst_share.lower() == share_name.lower():
                        if fs_list:
                            if self.__get_share_fs(share_name) in fs_list:
                                return True
                        else:
                            return True
            return False
        except:
            return False

    def get_local_dst_list(self, category="backup"):
        try:
            dst_list = []
            if not self.conf_dict:
                self.conf_dict = self.load()
            if not self.local_host_list:
                self.local_host_list = self.__get_local_host_list()
            for job_id in self.conf_dict[category].keys():
                unc = self.conf_dict.get(
                    category, {}).get(
                    job_id, {}).get(
                    "destination", {}).get(
                    "unc")
                host, share = self.__unc_to_host_share(unc)
                if host.lower() in self.local_host_list:
                    dst_list += [share]
            return dst_list
        except:
            return []

    def get_local_src_list(self, category="backup"):
        try:
            src_list = []
            if not self.conf_dict:
                self.conf_dict = self.load()
            if not self.local_host_list:
                self.local_host_list = self.__get_local_host_list()
            for job_id in self.conf_dict[category].keys():
                src_info_list = self.conf_dict.get(
                    category, {}).get(
                    job_id, {}).get(
                    "sourcelist", [])
                for src_info in src_info_list:
                    unc = src_info.get("unc")
                    host, share = self.__unc_to_host_share(unc)
                    if host.lower() in self.local_host_list:
                        src_list += [share]
            return src_list
        except:
            return []

    def get_bkup_local_share_list(self, category="backup", dst=True, src=True):
        try:
            dst_list = []
            src_list = []
            if dst:
                dst_list = self.get_local_dst_list(category)
            if src:
                src_list = self.get_local_src_list(category)
            return (dst_list, src_list)
        except:
            return ([], [])

    def is_running(self,
                   jobid,
                   runnings=['PROCESSING', 'QUEUE'],  # NOT_EXEC, ABORT, COMPLETE
                   function='backup'):
        try:
            if self.query()[function][str(jobid)]['status'] in runnings:
                return True
            return False
        except:
            return False

    def schedule_enable(self,
                        jobid,
                        enabled,
                        func=u"backup",  # or u"datacopy"
                        *args, **kwargs):
        try:
            jobid = unicode(jobid)
            enabled = bool(enabled)
            e1 = self.request(Entry(u"load", u"""<backups />"""))
            for job_node in e1.get().get_node_list(u"%s/joblist/job" % func):
                if job_node.get_value(u"id", u"") == jobid:
                    job_node.get_node(u"schedule").enable(enabled)
                    e2 = Entry(u"modify", u"""<backups><%s /></backups>""" % func)
                    e2.get().get_node(func).append_child(job_node)
                    self.request(e2)
                    break
            return True
        except:
            return False


if __name__ == '__main__':
    i = Backups()

    print i.query()
