# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from fal_info import Results


class Group(Command):
    try:
        from fal_info import FAL_GROUPS_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_groups.py'

    TYPE_LOAD = 'load'
    TYPE_ADD = 'add'
    TYPE_MODIFY = 'modify'
    TYPE_DELETE = 'delete'
    TYPE_QUERY = 'query'
    TYPE_DOMAIN_LIST = 'domain_list'

    def load(self, to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            users = entry.get()
            for user in users.get_node_list(
                    'grouplist/group[not(@hidden="true")]'):  # FIXME xpath
                try:
                    name = user.get_value('name')
                    dict_[name] = {
                        'name': name,
                        'gid': user.get_value('id'),
                        #'gid': user.get_value('gid'),
                        'quota': user.get_value('quota/limit', None),
                        'members': [nd.get_value('name') \
                                for nd in user.get_node_list('userlist/user')]
                    }
                except:
                    continue
            return dict_
        entry = self.request(Entry(self.TYPE_LOAD, '<groups />'))
        return _to_dict(entry) if to_dict else entry

    def _domain_list(self):
        entry = Entry(self.TYPE_DOMAIN_LIST, '<groups />')
        return self.request(entry)

    def domain_list(self, to_dict=True):
        def _to_dict(entry):
            """<entry type="domain_list" result="0">
                <groups>
                    <grouplist>
                        <group>
                            <upn>DOMAIN@GROUP001</upn>
                            <domain>DOMAIN</domain>
                            <name>GROUP001</name>
                            <quota><limit>100</limit></quota>
                        </group>
                    </grouplist>
                </groups>
            </entry>"""
            dict_ = {}
            try:
                for gnd in entry.get().get_node_list('grouplist/group'):
                    upn = gnd.get_value('upn', None)
                    domain = gnd.get_value('domain', None)
                    name = gnd.get_value('name', None)
                    qlimit = gnd.get_value('qlimit', None)
                    if upn:
                        upn = upn if isinstance(upn, unicode) else upn.decode('utf-8')
                        dict_[upn] = {}
                        dict_[upn]['upn'] = upn
                        dict_[upn]['domain'] = domain
                        dict_[upn]['name'] = name
                        dict_[upn]['quota'] = qlimit
            except:
                pass
            return dict_
        entry = self._domain_list()
        return _to_dict(entry) if to_dict else entry

    def _req_add_or_mod(self,
            type_,
            name,
            gid=None,
            quota=None,
            members=None):
        entry = Entry(type_, '<groups><group /></groups>')
        group_node = entry.get().get_node('group')
        if isinstance(name, (str, unicode)):
            group_node.append_child('name', text=name)
        else:
            raise  # FIXME
        if isinstance(gid, (int)):
            group_node.append_child('id', text=str(gid))
        if isinstance(quota, (int, float)):
            group_node.append_child('quota')
            group_node.get_node('quota').append_child('limit', text=str(quota))
        if isinstance(members, (list, tuple)):
            group_node.append_child('userlist')
            userlist_node = group_node.get_node('userlist')
            for member in members:
                user_node = XmlNode('<user />')
                user_node.append_child('name', text=member)
                userlist_node.append_child(user_node)
        return self.request(entry)

    def add(self, name, gid=None, quota=None, members=None):
        entry = self._req_add_or_mod(
                self.TYPE_ADD,
                name,
                gid=gid,
                quota=quota,
                members=members)
        arglist = {'name': name}
        result = self.get_result()
        if result:
            if result == int(Results.MAX_EXCEED_ERROR):
                self._set_message('ERR_GROUP_NO_MORE', arglist=arglist, log=True)
            else:
                self._set_message('ERR_GROUP_ADD', arglist=arglist, log=True)
        else:
            self._set_message('MSG_GROUP_ADD', arglist=arglist, log=True)
        return entry

    def modify(self, name, gid=None, quota=None, members=None, log=True):
        entry = self._req_add_or_mod(
                self.TYPE_MODIFY,
                name,
                gid=gid,
                quota=quota,
                members=members)
        arglist = {'name': name}
        if self.get_result():
            self._set_message(
                    'ERR_GROUP_INFO_CHANGE',
                    arglist=arglist,
                    log=log)
        else:
            self._set_message(
                    'MSG_GROUP_INFO_CHANGE',
                    arglist=arglist,
                    log=log)
        return entry

    def delete(self, name):
        entry = Entry(self.TYPE_DELETE, '<groups><group /></groups>')
        user = entry.get().get_node('group')
        if isinstance(name, (str, unicode)):
            user.append_child('name', text=name)
        else:
            raise  # FIXME
        entry = self.request(entry)
        arglist = {'name': name}
        if self.get_result():
            self._set_message('ERR_GROUP_DELETE', arglist=arglist, log=True)
        else:
            self._set_message('MSG_GROUP_DELETE', arglist=arglist, log=True)
        return entry

    def query(self, to_dict=True, all=False):
        def _to_dict(entry, all=False):
            try:
                return dict([
                        (n.get_value('name'), n.get_value('quota/used')) \
                        for n in entry.get().get_node_list('grouplist/group') \
                            if all or not n.is_set_to_true('hidden')])
            except:
                return dict([])
        entry = self.request(Entry(self.TYPE_QUERY, '<groups />'))
        return _to_dict(entry, all=all) if to_dict else entry

    # ここから ===
    # 以下メソッドはwbinfoを直接実行している
    # 設計思想上は好ましくないがFALを使うと非常に遅いのでこうする
    def wbinfo_g(self):
        try:
            from subprocess import Popen, PIPE
            proc = Popen(['wbinfo', '-g'], stdout=PIPE)
            out, err = proc.communicate()
            return sorted([upn for upn in out.splitlines()])
        except:
            return []

    def wbinfo_i(self, gname):
        info = dict()
        keys = ['name', 'passwd', 'gid', 'accounts']
        for k in keys:
            info[k] = None
        try:
            from subprocess import Popen, PIPE
            proc = Popen(['wbinfo', '--group-info', gname], stdout=PIPE)
            out, err = proc.communicate()
            for k, v in zip(keys, out.strip().split(':')):
                info[k] = v
            info['accounts'] = info['accounts'].split(',') if info['accounts'] else []
            return info
        except:
            return info
    # === ここまで

if __name__ == '__main__':
    g = Group()
    for name in g.wbinfo_g():
        print g.wbinfo_i(name)
