# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from product_conf import STOR_IF_TO_VOL
from fal_info import Results
import datetime
from project.utils import get_mirroring_device


class Raidctrl(Command):
    try:
        from fal_info import FAL_RAIDCTRL_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_raidctrl.py'

    TYPE_APPLY = 'apply'
    TYPE_QUERY = 'query'
    TYPE_CREATE = 'create'
    TYPE_START = 'start'
    TYPE_STOP = 'stop'
    TYPE_ACTIVEREPAIR = 'activerepair'
    TYPE_LOAD = 'load'
    TYPE_GETMODELIST = 'getraidmodelist'
    TYPE_REMOVE = 'remove'

    VOLUME = STOR_IF_TO_VOL

    def __init__(self, *args, **kwargs):
        super(Raidctrl, self).__init__(*args, **kwargs)
        self.__query_cache = None

    def get_autoextend(self):
        try:
            entry = self._load()
            return self._load().get().get_node('raidlist/raid[name/text()="sataraid1"]').is_set_to_true('autoextend')
        except:
            return False

    def set_autoextend(self, autoextend=None, log=True):
        if isinstance(autoextend, bool):
            entry = Entry('apply', """<raidctrl>
                <raidlist>
                    <raid>
                        <name type="volume" />
                    </raid>
                </raidlist>
            </raidctrl>""")
            entry.get().get_node('raidlist/raid').set_bool_attr('autoextend', autoextend)
            entry.get().get_node('raidlist/raid/name').set_text('sataraid1')
            result = getattr(self.request(entry), 'get_result', lambda: None)()
            constname = {
                Results.OK: 'MSG_RAID_CHANGE',
                None: 'ERR_RAID_CHANGE',
            }.get(result, 'ERR_RAID_CHANGE')
            self._set_message(constname, log=log)
            return (result in [Results.OK])
        return False

    def extend(self, mode, log=True):
        entry = Entry('extend', """<raidctrl>
            <raid>
                <name type="volume">sataraid1</name>
                <level/>
            </raid>
        </raidctrl>""")
        entry.get().get_node('raid/level').set_text(mode)
        result = getattr(self.request(entry), 'get_result', lambda: None)()
        constname = {
            Results.OK: 'MSG_RAID_MODE_EXTEND_VFSEXT_6',
            None: 'ERR_RAID_MODE_EXTEND_VFSEXT_6',
        }.get(result, 'ERR_RAID_MODE_EXTEND_VFSEXT_6')
        self._set_message(constname, log=log)
        return (result in [Results.OK])

    def check(self, operation='changemode', mode=''):
        if operation == 'changemode':
            entry = Entry('check', """<raidctrl>
                <option operation="changemode"/>
                <raid>
                    <name type="volume">sataraid1</name>
                    <level/>
                </raid>
            </raidctrl>""")
            entry.get().get_node('raid/level').set_text(mode)
            result = getattr(self.request(entry), 'get_result', lambda: None)()
            return (result in [Results.OK])
        elif operation == 'adddisk':
            pass
        return True

    def _remove(self, disk, user=True):
        xml = """<raidctrl>
                <option user="%(user)s" />
                <disk>
                    <name>%(disk)s</name>
                </disk>
            </raidctrl>""" % {
                'user': str(user).lower(),
                'disk': disk}
        return self.request(Entry(self.TYPE_REMOVE, xml))

    def remove(self,
            disk,  # 'satad1', 'satad2', 'usbd1', etc...
            user=True,
            to_bool=True):
        entry = self._remove(disk, user=user)
        if to_bool:
            return entry.get_result() in [Results.OK, Results.ALREADY_REMOVED]
        else:
            return entry

    def _getraidmode(self):
        xml = """<raidctrl>
               <raid>
                   <name type="volume">sataraid1</name>
               </raid>
            </raidctrl>"""
        return self.request(Entry(self.TYPE_GETMODELIST, xml))

    def getraidmode(self, to_list=True):
        def _to_list(entry):
            list_ = []
            try:
                raidctrl_node = entry.get()
                list_ = [node.get_text() for node in \
                        raidctrl_node.get_node_list('raid/modelist/mode')]
            except:
                list_ = []
            return list_
        entry = self._getraidmode()
        return _to_list(entry) if to_list else entry

    def _load(self):
        return self.request(Entry(self.TYPE_LOAD, """<raidctrl />"""))

    def load(self, to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            try:
                sche_node = entry.get().get_node('activerepair/schedule')
                dict_['enable'] = sche_node.is_enabled()
                dict_['day'] = sche_node.get_value('day', None)
                hour = sche_node.get_value('time/hour', 0)
                minute = sche_node.get_value('time/minute', 0)
                dict_['time'] = datetime.time(int(hour), int(minute))
            except:
                pass
            return dict_
        entry = self._load()
        return _to_dict(entry) if to_dict else entry

    def query(self,
            volume='sataraid1',
            depth=2,
            to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            try:
                raid_node = entry.get().get_node('raid')
                dict_['name'] = raid_node.get_value('name', None)
                dict_['status'] = raid_node.get_value('status', None)
                dict_['progress'] = raid_node.get_value('progress', None)
                dict_['size'] = raid_node.get_value('size', None)
                dict_['level'] = raid_node.get_value('level', None)
                dict_['number'] = raid_node.get_value('number', None)
                dict_['elementlist'] = {}
                for enode in raid_node.get_node_list('elementlist/element'):
                    ename = enode.get_value('name', None)
                    if ename is None:
                        continue
                    dict_['elementlist'][ename] = {
                        'name': ename,
                    }
            except:
                pass
            return dict_
        entry = self.request(Entry(self.TYPE_QUERY, """<raidctrl>
            <raid depth="%(depth)s">
                <name type="volume">%(name)s</name>
            </raid>
        </raidctrl>""" % {
            'depth': depth,
            'name': volume,
        }))
        entry.get_result()
        self.__query_cache = _to_dict(entry)
        return self.__query_cache if to_dict else entry

    def create(self,
#            volume=VOLUME['internal'][1],
            volume=None,
            level='vfsext',
            to_bool=True, log=True):
        entry = Entry(self.TYPE_CREATE, """<raidctrl>
            <raid>
                <name type="volume">%(volume)s</name>
                <level>%(level)s</level>
            </raid>
        </raidctrl>""" % {
            'volume': volume,
            'level': level,
        })
        entry = self.request(entry)
        result = self.get_result(entry=entry)
        modelist = (
            'vfsext', 'vfsext_6', 'vfsext_4', 'vfsext_3', 'vfsext_2',
            'raid0',  'raid0_6',  'raid0_4',  'raid0_3',  'raid0_2',
            'raid1',              'raid1_4',  'raid1_3',  'raid1_2',
            'raid5',  'raid5_6',  'raid5_4',
            'raid6',  'raid6_6',  'raid6_4',
        )
        if int(result) == int(Results.OK):
            codes = dict([(m, "MSG_RAID_MODE_CHANGE_%s" % m.upper()) for m in modelist])
            codes.update({  # NOTE: 不要なはずだが一応過去互換保証のため
                'raid6': 'MSG_RAID_MODE_CHANGE_RAID6',
                'raid5': 'MSG_RAID_MODE_CHANGE_RAID5',
                'raid1': 'MSG_RAID_MODE_CHANGE_RAID1',
                'raid0': 'MSG_RAID_MODE_CHANGE_RAID0',
                'vfsext': 'MSG_RAID_MODE_CHANGE_VFSEXT',
                })
            self._set_message(codes.get(level, 'MSG_RAID_MODE_CHANGE'), log=log)
            result = True
        elif int(result) == int(Results.DEVICE_NOT_FOUND):
            self._set_message('ERR_RAID_MODE_CHANGE_DIVICE_NOT_FOUND', log=log)
            result = False
        else:
            codes = dict([(m, "ERR_RAID_MODE_CHANGE_%s" % m.upper()) for m in modelist])
            codes.update({  # NOTE: 不要なはずだが一応過去互換保証のため
                'raid6': 'ERR_RAID_MODE_CHANGE_RAID6',
                'raid5': 'ERR_RAID_MODE_CHANGE_RAID5',
                'raid1': 'ERR_RAID_MODE_CHANGE_RAID1',
                'raid0': 'ERR_RAID_MODE_CHANGE_RAID0',
                'vfsext': 'ERR_RAID_MODE_CHANGE_VFSEXT',
                })
            self._set_message(codes.get(level, 'ERR_RAID_MODE_CHANGE'), log=log)
            result = False
        return result if to_bool else entry

    def mirrorrestore(self, log=True):
        entry = Entry(self.TYPE_CREATE, """<raidctrl>
            <option recovery="true" />
            <raid>
                <name type="volume">sataraid1</name>
            </raid>
        </raidctrl>""")
        entry = self.request(entry)
        mirroring_device = get_mirroring_device(default=(u"usb", 3))
        if entry.get_result() == Results.OK:
            if mirroring_device == (u"usb", 2):
                self._set_message(u"MSG_RAID_MIRROR_START_FROM_USB2", log=log)
            else:
                self._set_message(u"MSG_RAID_MIRROR_START_FROM_USB3", log=log)
            return True
        else:
            if mirroring_device == (u"usb", 2):
                self._set_message(u"ERR_RAID_MIRROR_START_FROM_USB2", log=log)
            else:
                self._set_message(u"ERR_RAID_MIRROR_START_FROM_USB3", log=log)
            return False

    def _start(self,
#            volume=VOLUME['internal'][1],
            volume=None,
            force=False,
            autorebuild=False):
        entry = Entry(self.TYPE_START, """<raidctrl>
            <option force="%(force)s" autorebuild="%(autorebuild)s" />
            <raid>
                <name type="volume">%(volume)s</name>
            </raid>
        </raidctrl>""" % {
            'volume': volume,
            'force': self.bool_to_text(force),
            'autorebuild': self.bool_to_text(autorebuild),
        })
        return self.request(entry)

    def _stop(self, volume=None):
        entry = Entry(self.TYPE_STOP, """<raidctrl>
            <raid><name type="volume">%s</name></raid>
        </raidctrl>""" % volume)
        return self.request(entry)

    def stop(self, volume=VOLUME['internal'][1], to_bool=True, log=True):
        entry = self._stop(volume=volume)
        result = entry.get_result()
        if int(result) == int(Results.OK):
            result = True
        else:
            result = False
        return result if to_bool else entry

    def forcestart(self,
            volume=VOLUME['internal'][1],
            to_bool=True, log=True):
        entry = self._start(volume=volume, force=True, autorebuild=False)
        result = self.get_result(entry=entry)
        if int(result) == int(Results.OK):
            self._set_message('MSG_RAID_FORCE_START', log=log)
            result = True
        else:
            self._set_message('ERR_RAID_FORCE_START', log=log)
            result = False
        return result if to_bool else entry

    def activerepair_getstatus(self):
        _dict = {}
        try:
            entry = Entry(self.TYPE_ACTIVEREPAIR,
                          """<raidctrl><option operation="getstatus"/></raidctrl>""")
            entry = self.request(entry)
            result = getattr(entry, 'get_result', lambda: None)()

            _dict['status'] = entry.get().get_value('status', None)
            _dict['result'] = entry.get().get_value('result', None)
            _dict['timestamp'] = entry.get().get_value('timestamp', None)
        except:
            pass
        return _dict

    def activerepair(self,
            volume=VOLUME['internal'][1],
            operation='stop',  # start / stop / schedule
            schedule=True,
            day=None,  # Sun / Mon / Tue / Web / Thu / Fri / Sat
            time=None,  # type == datetime.time
            to_bool=True, log=True):
        entry = Entry(self.TYPE_ACTIVEREPAIR, """<raidctrl>
            <option operation="%(operation)s" />
            <raid>
                <name type="volume">%(volume)s</name>
            </raid>
        </raidctrl>""" % {
            'operation': operation,
            'volume': volume,
        })
        if operation == 'schedule':
            if schedule and day and isinstance(time, datetime.time):
                entry.get().append_child(XmlNode("""<activerepair>
                    <schedule enable="%(enable)s">
                        <day>%(day)s</day>
                        <time>
                            <hour>%(hour)s</hour>
                            <minute>%(minute)s</minute>
                        </time>
                    </schedule>
                </activerepair>""" % {
#                    'enable': str(schedule).lower(),
                    'enable': 'true',
                    'day': day,
                    'hour': time.hour,
                    'minute': time.minute
                }))
            elif not schedule:
                entry.get().append_child(XmlNode("""<activerepair>
                    <schedule enable="false" />
                </activerepair>"""))
            else:
                return False
        entry = self.request(entry)
        result = self.get_result(entry=entry)
        if operation == 'start':
            if int(result) == int(Results.OK):
                self._set_message('MSG_ACTIVE_REPAIR_START', log=log)
            elif int(result) == int(Results.BUSY_ERROR):
                self._set_message('ERR_ACTIVE_REPAIR_START', log=log)  # FIXME
            else:
                self._set_message('ERR_ACTIVE_REPAIR_START', log=log)
        elif operation == 'stop':
            if int(result) == int(Results.OK):
                self._set_message('MSG_ACTIVE_REPAIR_STOP', log=log)
            else:
                self._set_message('ERR_ACTIVE_REPAIR_STOP', log=log)
        elif operation == 'schedule':
            if int(result) == int(Results.OK):
                self._set_message('MSG_ACTIVE_REPAIR_CHANGE', log=log)
            else:
                self._set_message('ERR_ACTIVE_REPAIR_CHANGE', log=log)
        result = True if int(result) == int(Results.OK) else False
        return result if to_bool else entry

    def humanize_status(self, status=None):
        info = self.query()
        status = status or info['status']
        progress = info['progress']
        _humanize = {
            'fail': _(u'RaidStatusFail'),
            'degrade': _(u'RaidStatusDegrade'),
            'stop': _(u'RaidStatusStop'),
            'running': _(u'RaidStatusRunning'),
            'rebuilding': "%s(%s%%)" % (_(u'RaidStatusRebuilding'), progress),
            'activerepair': _(u'RaidStatusActiveRepair'),
        }
        return _humanize.get(status, _(u'unknown'))

    def status(self,
            volume='sataraid1',
            depth=2,
            humanize=False,
            cache=False):
        info = self.__query_cache if cache and self.__query_cache else \
                self.query(volume=volume, depth=depth)
        status = info.get('status', None)
        progress = info.get('progress', _(u'unknown')) or _(u'unknown')
        _humanize = {
            'fail': _(u'RaidStatusFail'),
            'crash': _(u'RaidStatusCrash'),
            'degrade': _(u'RaidStatusDegrade'),
            'stop': _(u'RaidStatusStop'),
            'running': _(u'RaidStatusRunning'),
            'rebuilding': "%s(%s%%)" % (_(u'RaidStatusRebuilding'), progress),
            'activerepair': _(u'RaidStatusActiveRepair'),
        }
        return _humanize.get(status, _(u'unknown')) if humanize else status

    def trans(self, text):
        """内部的な値(今の所RAIDモード表記のみ)をユーザーへの表記に変換"""
        return {
            'single'  : _(u"Single Drive"),
            'vfsext'  : _(u"VFS"),
            'vfsext_6': _(u"VFS(6)"),
            'vfsext_4': _(u"VFS(4)"),
            'vfsext_3': _(u"VFS(3)"),
            'vfsext_2': _(u"VFS(2)"),
            'raid0'   : _(u"RAID0"),
            'raid0_6' : _(u"RAID0(6)"),
            'raid0_4' : _(u"RAID0(4)"),
            'raid0_3' : _(u"RAID0(3)"),
            'raid0_2' : _(u"RAID0(2)"),
            'raid1'   : _(u"RAID1"),
            'raid1_4' : _(u"RAID1(4)"),
            'raid1_3' : _(u"RAID1(3)"),
            'raid1_2' : _(u"RAID1(2)"),
            'raid5'   : _(u"RAID5"),
            'raid5_6' : _(u"RAID5(6)"),
            'raid5_4' : _(u"RAID5(4)"),
            'raid6'   : _(u"RAID6"),
            'raid6_6' : _(u"RAID6(6)"),
            'raid6_4' : _(u"RAID6(4)"),
        }.get(text, text)


if __name__ == '__main__':
    i = Raidctrl()

    print i.get_autoextend()
    print i.check(mode='vfsext_6')
    print i.activerepair_getstatus()

#    print i.query(volume=i.VOLUME['internal'][1], depth=1)
#    print i.query(volume=i.VOLUME['internal'][1], depth=2)

#    print "GETRAIDMODE: %s" % " ".join(i.getraidmode())
#    print "REMOVE: %s" % i.remove('satad1')

#    from datetime import time
#    print i.activerepair(operation="schedule", day="Sun", time=time(12, 34))
#    print i.load()

#    print i.status()
#    print i.status(humanize=True)

#    vol = i.VOLUME['internal'][1]

#    print "STOP: %s" % i.stop(volume=vol)
#    print i.create(volume=vol, level="raid6")
#    print "CREATE: %s (%s)" % (i.get_message(), i.get_result())
#    print i.query(volume=vol)
#
#    print "STOP: %s" % i.stop(volume=vol)
#    print i.create(volume=vol, level="raid0")
#    print "CREATE: %s (%s)" % (i.get_message(), i.get_result())
#    print i.query(volume=vol)

#    print i.activerepair(volume=i.VOLUME['internal'][1], operation="start")
#    print "ACTIVEREPAR: %s (%s)" % (i.get_message(), i.get_result())
#    import time
#    time.sleep(5)
#    print i.activerepair(volume=i.VOLUME['internal'][1], operation="stop")
#    print "ACTIVEREPAR: %s (%s)" % (i.get_message(), i.get_result())

#    print i.forcestart(volume=i.VOLUME['internal'][1])
#    print "FORCE START: %s (%s)" % (i.get_message(), i.get_result())
