# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.conf import settings
from django.utils.translation import ugettext as _
from django.forms.widgets import MultiWidget, Select
from datetime import datetime

__all__ = ('SelectDateTimeWidget',)


class SelectDateTimeWidget(MultiWidget):
    """
    for DateTimeField
    """

    NONE_VALUE = '-'

    YYYYMMDD = 'YYYY_MM_DD'
    YYYYMMDDhh = 'YYYY_MM_DD_hh'
    YYYYMMDDhhmm = 'YYYY_MM_DD_hh_mm'
    YYYYMMDDhhmmss = 'YYYY_MM_DD_hh_mm_ss'

    FORMATS = {
        'ja': {
            YYYYMMDD:       u'%(YYYY)s/%(MM)s/%(DD)s',
            YYYYMMDDhh:     u'%(YYYY)s/%(MM)s/%(DD)s %(hh)s',
            YYYYMMDDhhmm:   u'%(YYYY)s/%(MM)s/%(DD)s %(hh)s:%(mm)s',
            YYYYMMDDhhmmss: u'%(YYYY)s/%(MM)s/%(DD)s %(hh)s:%(mm)s:%(ss)s',
        },
        'en-us': {
            YYYYMMDD:       u'%(MM)s/%(DD)s/%(YYYY)s',
            YYYYMMDDhh:     u'%(MM)s/%(DD)s/%(YYYY)s %(hh)s',
            YYYYMMDDhhmm:   u'%(MM)s/%(DD)s/%(YYYY)s %(hh)s:%(mm)s',
            YYYYMMDDhhmmss: u'%(MM)s/%(DD)s/%(YYYY)s %(hh)s:%(mm)s:%(ss)s',
        },
    }

    def __init__(self, attrs=None,
            required=True,
            years=range(2013, 2038), months=range(1, 13), days=range(1, 32),
            hours=range(0, 24), minutes=range(0, 60), seconds=range(0, 60),
            show_hours=True, show_minutes=True, show_seconds=True):

        self.required = required
        self.show_hours = show_hours
        self.show_minutes = show_minutes
        self.show_seconds = show_seconds

        years = [(y, y) for y in years]
        months = [(m, "%02d" % m) for m in months]
        days = [(d, "%02d" % d) for d in days]
        hours = [(h, "%02d" % h) for h in hours]
        minutes = [(m, "%02d" % m) for m in minutes]
        seconds = [(s, "%02d" % s) for s in seconds]
        if not self.required:
            years = [(self.NONE_VALUE, _(u'year'))] + years
            months = [(self.NONE_VALUE, _(u'month'))] + months
            days = [(self.NONE_VALUE, _(u'day'))] + days
            hours = [(self.NONE_VALUE, _(u'hour'))] + hours
            minutes = [(self.NONE_VALUE, _(u'minute'))] + minutes
            seconds = [(self.NONE_VALUE, _(u'second'))] + seconds
        _widgets = (
            Select(attrs=attrs, choices=years),
            Select(attrs=attrs, choices=months),
            Select(attrs=attrs, choices=days),
            Select(attrs=attrs, choices=hours),
            Select(attrs=attrs, choices=minutes),
            Select(attrs=attrs, choices=seconds),
        )
        super(SelectDateTimeWidget, self).__init__(_widgets, attrs)

    def decompress(self, value):
        if isinstance(value, datetime):
            date = value.date()
            time = value.time()
            return [date.year, date.month, date.day,
                    time.hour, time.minute, time.second]
        return [None, None, None,
                None, None, None]

    def format_output(self, rendered_widgets):
        datadict = {
            'YYYY': rendered_widgets[0],
            'MM': rendered_widgets[1],
            'DD': rendered_widgets[2],
            'hh': rendered_widgets[3],
            'mm': rendered_widgets[4],
            'ss': rendered_widgets[5],
        }
        try:
            lang = settings.LANGUAGE_CODE
            if self.show_hours:
                format_ = self.FORMATS[lang][self.YYYYMMDDhh]
                if self.show_minutes:
                    format_ = self.FORMATS[lang][self.YYYYMMDDhhmm]
                    if self.show_seconds:
                        format_ = self.FORMATS[lang][self.YYYYMMDDhhmmss]
            else:
                format_ = self.FORMATS[lang][self.YYYYMMDD]
        except:
            if self.show_hours:
                format_ = self.FORMATS['ja'][self.YYYYMMDDhh]
                if self.show_minutes:
                    format_ = self.FORMATS['ja'][self.YYYYMMDDhhmm]
                    if self.show_seconds:
                        format_ = self.FORMATS['ja'][self.YYYYMMDDhhmmss]
            else:
                format_ = self.FORMATS['ja'][self.YYYYMMDD]
        return format_ % datadict

    def value_from_datadict(self, data, files, name):
        datalist = [widget.value_from_datadict(data, files, name + "_%s" % i)
                for i, widget in enumerate(self.widgets)]
        if datalist[0] == self.NONE_VALUE:
            return None
        datalist[1] = 1 if datalist[1] in (self.NONE_VALUE) else datalist[1]
        datalist[2] = 1 if datalist[2] in (self.NONE_VALUE) else datalist[2]
        datalist[3] = 0 if datalist[3] in (self.NONE_VALUE, None) else datalist[3]
        datalist[4] = 0 if datalist[4] in (self.NONE_VALUE, None) else datalist[4]
        datalist[5] = 0 if datalist[5] in (self.NONE_VALUE, None) else datalist[5]
        try:
            value = datetime(*[int(data) for data in datalist])
        except ValueError:
            return datalist
            #return datetime.now()  # FIXME None?
        else:
            return value
