# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.conf import settings as django_settings
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils import simplejson
from django.http import HttpResponse, HttpResponseRedirect
import sys
import os
import re
import copy
import logging
from root.modules.utils import importer as imp, to_importable_path as to_ipath, is_django_app
from project.utils import is_crash, is_keywait
from project.auth import Authentication
from project.fal.command import get_message as fgettext
from project.fal.user import User


def _get_help_mode():
    try:
        return bool(int(User().load(all=True)["admin"]["help"]))
    except:
        return True


class View(object):
    KEY_RESPONSE = 'response'
    KEY_REDIRECT = 'redirect'
    KEY_TEMPLATE = 'template'
    KEY_TEMPLATE_DIC = 'template_dic'

    views = []  # ex) ['save', 'add', 'delete']

    logger = logging.getLogger("custom")

    def __init__(self):
        self.CURRENT = os.path.dirname(sys.modules[self.__module__].__file__)
        self.TEMPLATES = os.path.join(self.CURRENT, 'templates')

    @classmethod
    def as_view(cls):
        self = cls()
        return self.__view

    def __view(self, request, *args, **kwargs):
        view = getattr(self,
                'view_' + self.__get_requested_view(request),
                self.view)
        try:
            params = self._view(view, request, *args, **kwargs)
        except:
            import traceback
            self.logger.debug(traceback.format_exc())
            raise

        if self.KEY_RESPONSE in params and params[self.KEY_RESPONSE]:
            return params[self.KEY_RESPONSE]
        else:
            if self.KEY_REDIRECT in params and params[self.KEY_REDIRECT]:
                return HttpResponseRedirect(params[self.KEY_REDIRECT])
            else:
                return render_to_response(
                        params[self.KEY_TEMPLATE],
                        params[self.KEY_TEMPLATE_DIC],
                        context_instance=RequestContext(request))

    def __get_requested_view(self, request):
        if request.method == 'POST':
            for v in self.views:
                if v in request.POST:
                    return v
        return ''

    def _view(self, view, request, *args, **kwargs):
        params = {
            self.KEY_RESPONSE: None,
            self.KEY_REDIRECT: None,
            self.KEY_TEMPLATE: None,
            self.KEY_TEMPLATE_DIC: {}}
        params.update(view(request, *args, **kwargs))
        return params


class RootView(View):
    REQUIRED_LOGIN = True
    NOT_LOGINED_URL = '/'  # FIXME 名前 非ログイン時のリダイレクト先
    RESET_LOGOUT_COUNT = True

    AJAXFORM_HTML = 'ajaxform.html'  # FIXME

    def __init__(self):
        super(RootView, self).__init__()
        self.TEMPLATE = os.path.join(self.TEMPLATES, 'page.html')

    def _view(self, view, request, *args, **kwargs):
        self.session = self.Session(
                request.session,
                # FIXME [6:]
                map(lambda k: k.upper(), self.CURRENT.split(os.sep)[6:]))
        self.action = self.Action(getattr(self, 'set_message', lambda msg: msg))
        self.messages = []

        auth = Authentication()
        if (self.REQUIRED_LOGIN and not auth.is_logined(request, reset_logout_count=self.RESET_LOGOUT_COUNT)) \
                or (request.path != self.NOT_LOGINED_URL and request.COOKIES.get(django_settings.SESSION_COOKIE_NAME, None) is None):
            # ログインしてない
            params = super(RootView, self)._view(
                    lambda r, *args, **kwargs: {},  # ビュー関数のダミー
                    request, *args, **kwargs)
            params.update({
                self.KEY_RESPONSE: None,
                self.KEY_REDIRECT: self.NOT_LOGINED_URL})
        else:
            product = 'HDL-H'
            hostname = 'landisk'
            manual = 'http://www.iodata.jp/product/hdd/lanhdd/hdl6-h/support.htm',
            try:
                from fal.sysinfo import SysInfo
                sysinfo = SysInfo()
                info = sysinfo.query()
                product = info['model']
                hostname = info['host']['name']
                manual = 'http://www.iodata.jp/product/hdd/lanhdd/%s/support.htm' % info['series'].lower()
                if (info["series"] in ["HDL-AA", "HDL-AAW", "HDL2-AA", "HDL2-AAW", "HDL2-X", "HDL4-X"]) or (os.path.exists('/usr/local/share/manual')):
                    manual = "/manual/"
            except:
                pass
            params = super(RootView, self)._view(view, request, *args, **kwargs)
            params[self.KEY_TEMPLATE_DIC].update({
                'AJAXFORM_HTML': self.AJAXFORM_HTML,
                'MESSAGES': self.messages,
                'MESSAGE_IS_ERROR': getattr(self, 'message_is_error', False),
                'HOSTNAME': hostname,
                'PRODUCT': product,
                'MANUAL_URL': manual,
            })
        if request.is_ajax() and not params[self.KEY_RESPONSE]:
            params[self.KEY_RESPONSE] = self._to_ajaxform(params)
        return params

    def _to_ajaxform(self, params):
        json = {
            'redirect': '',
            'uplist': {}}  # {id1: [p1, p2, p3], id2: [pa, pb, pc]} な形式
        if params[self.KEY_REDIRECT]:
            json['redirect'] = params[self.KEY_REDIRECT]
        else:
            msgs = []
            for msg in self.messages:
                msgs.append(msg)
            if self.KEY_TEMPLATE_DIC in params \
                    and 'form' in params[self.KEY_TEMPLATE_DIC]:
                form = params[self.KEY_TEMPLATE_DIC]['form']
                NON_FIELD_ERRORS = '__all__'  # FIXME django.forms.NON_FIELD_ERRORS
                if NON_FIELD_ERRORS in form.errors:
                    for error in form.errors[NON_FIELD_ERRORS]:
                        msgs.append(error)
                for field in form:
                    for error in field.errors:
                        msgs.append('%s: %s' % (field.label, error))
            json['uplist']['ajaxform_modified'] = msgs
            if getattr(self, 'message_is_error', False):
                json['uplist_is_error'] = 'yes'
            else:
                json['uplist_is_error'] = 'no'

        return HttpResponse(
                simplejson.dumps(json),
                mimetype="application/json; charset=UTF-8;")

    def set_message(self, msg):  # FIXME 微妙 いらないかも
        self.messages.append(msg)

    def processing(self, state="busy", constname="ERR_SYSTEM_STATE_CHANGE"):
        return self.Processing(
                self,
                set_message=self.set_message,
                state=state,
                constname=constname)

    class Processing(object):
        class __StateError(Exception):
            pass

        def __init__(self,
                view,
                set_message=lambda msg: "",
                state="busy",
                constname="ERR_SYSTEM_STATE_CHANGE"):
            from fal.buzzer import Buzzer
            from fal.led import LED
            from fal.statemgr import Statemgr

            self._view = view
            self._set_message = set_message

            self._result = False
            self._constname = constname
            self._state = state

            self._buz = Buzzer()
            self._led = LED()
            self._statemgr = Statemgr()

        def __enter__(self):
            self.result(False)

            if self._statemgr.change("system", self._state, log=False):
                self._state_change = True
            else:
                self._state_change = False
                self._set_message(fgettext(self._constname, is_log=True))

            self._led.status(True, blink=True, color='green')

            return self

        def __exit__(self, typ, val, tb):
            if getattr(self, "_state_change", False) and self.result():
                self._view.message_is_error = False
                self._buz.ok()
            else:
                self._view.message_is_error = True
                self._buz.error()
            self._led.status(True, blink=False, color='green')
            if typ is not self.__StateError:
                self._statemgr.change('system', 'running', log=False)

            try:
                if tb:
                    import traceback
                    logging.getLogger("custom").debug(
                            "".join(traceback.format_exception(typ, val, tb)))
            except:
                pass

            return True

        def result(self, result=None):
            if isinstance(result, bool):
                self._result = result
            return self._result

        def break_if_state_change(self):
            if not getattr(self, "_state_change", False):
                raise self.__StateError()
#                raise Exception("StateError")


    class Action(object):
        def __init__(self, set_message):
            from fal.buzzer import Buzzer
            from fal.led import LED
            from fal.statemgr import Statemgr
            self._buz = Buzzer()
            self._led = LED()
            self._statemgr = Statemgr()

            self._set_message = set_message

        def start(self,
                state='busy',
                constname="ERR_SYSTEM_STATE_CHANGE", arglist={}):
            self._state = self._statemgr.change('system', state, log=False)
            if not self._state:
                self._set_message(
                        fgettext(constname, arglist=arglist, is_log=True))
                return False
            self._led.status(True, blink=True, color='green')
            return True

        def end(self, result=True):
            if result:
                self._buz.ok()
            else:
                self._buz.error()
            self._led.status(True, blink=False, color='green')
            if self._state:
                self._statemgr.change('system', 'running', log=False)
            return True

    class Session(object):
        def __init__(self, session, keys):
            self._prefix = '-'.join(keys)
            self._root_session = session
            if not self._prefix in self._root_session:
                self._root_session[self._prefix] = {}
            self._session = self._root_session[self._prefix]

        def __contains__(self, key):
            return key in self._session

        def __getitem__(self, key):
            return self._session[key]

        def __setitem__(self, key, value):
            self._session[key] = value
            self._root_session[self._prefix] = self._root_session[self._prefix]

        def __delitem__(self, key):
            del self._session[key]
            self._root_session[self._prefix] = self._root_session[self._prefix]

        def get(self, key, default=None):
            return self._session.get(key, default)

        def pop(self, key, *args):
            self._root_session[self._prefix] = self._root_session[self._prefix]
            return self._session.pop(key, *args)

        def update(self, dict_):
            self._session.update(dict_)
            self._root_session[self._prefix] = self._root_session[self._prefix]

        def has_key(self, key):
            return key in self._session

        def keys(self):
            return self._session.keys()

        def values(self):
            return self._session.values()

        def items(self):
            return self._session.items()

        def iterkeys(self):
            return self._session.iterkeys()

        def itervalues(self):
            return self._session.itervalues()

        def iteritems(self):
            return self._session.iteritems()

        def clear(self):
            self._session.clear()
            self._root_session[self._prefix] = self._root_session[self._prefix]

        def save(self):
            self._root_session.save()

        def load(self):
            return self._root_session.load()[self._prefix]


class AdminView(RootView):
#    ADMIN_BASE_HTML = os.path.join('admin', 'templates', 'base.html')
#    ADMIN_STATUS_HTML = os.path.join('admin', 'templates', 'status.html')
#    ADMIN_MENU_LIST_HTML = os.path.join('admin', 'templates', 'mlist.html')

    def _view(self, view, request, *args, **kwargs):
        params = super(AdminView, self)._view(view, request, *args, **kwargs)

        from root import admin
        admin_dir = os.path.abspath(os.path.dirname(admin.__file__))

        params[self.KEY_TEMPLATE_DIC].update({
            'INIT_URL': request.session['INIT_URL'] if 'INIT_URL' in request.session else '',
            'SLIDES': self._get_slides(admin_dir),
            'STATUS': self._get_status(),
#            'URL_LOGOUT': '/logout',
#            'ADMIN_BASE_HTML': self.ADMIN_BASE_HTML,
#            'ADMIN_STATUS_HTML': self.ADMIN_STATUS_HTML,
#            'ADMIN_MENU_LIST_HTML': self.ADMIN_MENU_LIST_HTML,
#            'MENU_LIST': self._get_menu(admin_dir),
        })
        return params

    def _get_status(self):
        return {
            'landiskname': 'landisk-abcdef',
            'version': '1.00',
            'available_capacity': '9,876 GB',
            'total_capacity': '12,345 GB',
            'notice': 'critical error'}

    def _get_slides(self, admin_dir):
        slides = []
        slides_dir = os.path.join(admin_dir, 'slides')
        if os.path.isdir(slides_dir):
            for name in sorted(os.listdir(slides_dir)):
                file = os.path.join(slides_dir, name)
                if os.path.isfile(file):
                    try:
                        s = imp(to_ipath(file))
                        slides.append({
                            'template': os.path.join(slides_dir, s.template),
                            'template_dic': s.template_dic})
                    except:
                        continue
        return slides

    def _get_menu(self, directory):
        """
        ex) return [
            {~~~, child: None},
            {~~~, child: None},
            {~~~, child: [
                {~~~, child: [
                    {~~~, child: None}
                ]},
                {~~~, child: None}
            ]},
            ~~~
        ]
        """
        menu = []
        try:
            for name in os.listdir(directory):
                path = os.path.join(directory, name)
                if is_django_app(path):
                    try:
                        settings = imp(to_ipath(os.path.join(path, 'settings.py')))
                        if not settings.IS_MENU_ITEM:
                            continue
                        settings.NAME_FORDISPLAY
                        settings.MENU_ORDER
                        settings.ID
                        settings.MENU_URL
                        settings.MENU_ICON
                    except:
                        continue

                    try:
                        if settings.IS_MENU_TAIL:
                            func = lambda d: []
                        else:
                            func = self._get_menu
                    except:
                        func = self._get_menu

                    menu.append({
                        'id': settings.ID,
                        'name': settings.NAME_FORDISPLAY,
                        'url': settings.MENU_URL,
                        'order': settings.MENU_ORDER,
                        'icon': 'img/icon/' + settings.MENU_ICON,
                        'child': func(path)})
        except:
            pass
        return menu


class FuncView(RootView):
    IN_FRAME = True
    FUNC_BASE_HTML = os.path.join('admin', 'templates', 'func', 'base.html')
    NOT_LOGINED_URL = '/admin/timeout/timeout'  # FIXME
    _NOT_LOGINED_URL = copy.copy(NOT_LOGINED_URL)

    TITLE = None

    def _view(self, view, request, *args, **kwargs):
        self.NOT_LOGINED_URL = self._NOT_LOGINED_URL + request.path
        params = super(FuncView, self)._view(view, request, *args, **kwargs)

        if ((not self._crash_safe()) and is_crash()) or \
                ((not self._lockkey_safe()) and is_keywait()):
            params[self.KEY_RESPONSE] = None
            params[self.KEY_REDIRECT] = self._NOT_LOGINED_URL
            return params

        if self.TITLE:
            pass
        else:
            try:
                settings = imp(
                        to_ipath(os.path.join(self.CURRENT, 'settings.py')))
                self.TITLE = settings.NAME_FORDISPLAY
                if settings.IS_MENU_ITEM and getattr(self, 'IN_FRAME', False):
                    request.session['FRAME_SRC'] = request.path
            except:
                self.TITLE = ''

        current, path = [], request.path  # ex) /admin/network/menu
        dir_ = self.CURRENT
        for p in reversed(path.split('/')[2:-1]):
            try:
                settings = imp(to_ipath(os.path.join(dir_, 'settings')))
                if not getattr(settings, 'IS_MENU_ITEM', False):
                    raise
            except:
                continue
            finally:
                dir_ = os.path.dirname(dir_)
            current.insert(0, {
                'name': getattr(settings, 'NAME_FORDISPLAY', p),
                'path': getattr(settings, 'MENU_URL', ''),  #FIXME default
                'icon': getattr(settings, 'MENU_ICON', ''),  #FIXME default
            })

        params[self.KEY_TEMPLATE_DIC].update({
            'current': current,
            'HELP_MODE': _get_help_mode(),
            'BACK': current[-2]['path'] if len(current) >= 2 else '/admin/menu',
            'FUNC_BASE_HTML': self.FUNC_BASE_HTML,
            'TITLE': self.TITLE})
        return params

    CRASH_SAFE_DEFAULT = False  # 基本 unsafe

    def _crash_safe(self):
        try:
            settings = imp(to_ipath(os.path.join(self.CURRENT, 'settings')))
            reload(settings)
            safe = getattr(settings, 'crash_safe', lambda: self.CRASH_SAFE_DEFAULT)
            return safe()
        except:
            return True

    def _lockkey_safe(self):
        try:
            settings = imp(to_ipath(os.path.join(self.CURRENT, 'settings')))
            reload(settings)
            safe = getattr(settings, 'lockkey_safe', self._crash_safe)
            return safe()
        except:
            return True


class MenuView(FuncView):
    def view(self, request, *args, **kwargs):
        crash = is_crash()
        keywait = is_keywait()
        items = []
        for name in os.listdir(self.CURRENT):
            dir_ = os.path.join(self.CURRENT, name)
            if is_django_app(dir_):
                try:
                    settings = imp(to_ipath(os.path.join(dir_, 'settings')))
                    reload(settings)
                except:
                    continue
                crash_safe = getattr(settings, 'crash_safe', lambda: False)
                lockkey_safe = getattr(settings, 'lockkey_safe', crash_safe)
                if getattr(settings, 'IS_MENU_ITEM', False) \
                        and (crash_safe() or not crash) \
                        and (lockkey_safe() or not keywait):
                    item = {}
                    item['name'] = getattr(settings, 'NAME_FORDISPLAY', name)
                    item['path'] = getattr(settings, 'MENU_URL', '')  # FIXME: default
                    item['icon'] = getattr(settings, 'MENU_ICON', '')  # FIXME: default
                    item['order'] = getattr(settings, 'MENU_ORDER', 100)
                    item['help_text'] = getattr(settings, 'MENU_HELP_TEXT', u"")
                    item['help_text'] = item['help_text'] if item['help_text'] else item['name']
                    items.append(item)
                    #items.append({
                    #    'name': getattr(settings, 'NAME_FORDISPLAY', name),
                    #    'path': getattr(settings, 'MENU_URL', ''),  #FIXME def
                    #    'icon': getattr(settings, 'MENU_ICON', ''),  #FIXME def
                    #    'order': getattr(settings, 'MENU_ORDER', 100),
                    #    'help_text': getattr(settings, 'MENU_HELP_TEXT', ''),
                    #})
        request.session['FRAME_SRC'] = request.path
        return {
            self.KEY_TEMPLATE: 'admin/templates/func/menu.html',
            self.KEY_TEMPLATE_DIC: {'items': items}}

    CRASH_SAFE_DEFAULT = True  # メニュー系ページは基本 safe


class StatusView(RootView):
    """
    状態取得用ベースビュークラス
    任意の処理の状態を取得し、それをJSON形式で返すビュー
    処理状態はファイルに記録されているとする
    JSONデータの形式は
        {'status': 状態(UIに表示可能な形式の文字列であること)}

    使い方
        1. views.py にこのクラスを継承して状態取得ビューを作る
        2. urls.py にそのビューへのマッピングを書く
        3. html で status.js を読み込むように書く
        4. html で 取得した情報を表示したい要素に任意の一意のIDをつける
        5. html で new Status(id, url, {options}) でインスタンス作成
            id は手順4のID
            url は手順2でマッピングしたURL
            options は種々のオプションを指定する連想配列
        6. そのインスタンス.get() する
    """
    STATUS_FILE_PATH = "/mnt/data/conf/volume/status/usb1/mkfs"  # dummy

    def view(self, request):
        return {self.KEY_RESPONSE: HttpResponse(
                simplejson.dumps({'status': self._status()}),
                mimetype="application/json; charset=UTF-8;")}

    def _status(self):
        """
            return "未実行"
            return "実行中(xx%)"
            return "完了(成功/エラー)"
        """
        try:
            fpath = self.STATUS_FILE_PATH
            with open(fpath, "r") as f:
                sts = f.read().rstrip().split(":")
            if sts[0] == "NOT_EXEC":
                return "%s" % (_(u"NOT_EXEC"))
            elif sts[0] == "PROCESSING":
                try:
                    progress = sts[1].decode('utf-8')
                    return "%s(%s)" % (_(u"PROCESSING"), progress)
                except:
                    progress = _(u'unknown')
                    return "%s" % _(u"PROCESSING")
                #return "%s(%s)" % (_(u"PROCESSING"), progress)
            elif sts[0] == "COMPLETE":
                result = {
                    "success": _(u"success"),
                    "error": _(u"error")}.get(sts[1], _(u"unknown"))
                return "%s(%s)" % (_(u"COMPLETE"), result)
            elif sts[0] == "CANCEL":
                return "%s" % (_(u"CANCEL"))
            else:
                return "%s" % _(u"unknown")
        except:
            return "%s" % _(u"unknown")
