# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

from datetime import datetime
from ionas.request import (Request, Entry)
from product_conf import (SUDO_CMD, STOR_IF_TO_VOL)
from iobase.exception import NotFoundError
from project.fal.command import (logging, encode_share_id)
from fal_info import (
    FAL_SYSINFO_PATH, FAL_USERS_PATH, FAL_NETWORK_PATH,
    FAL_DATETIME_PATH, FAL_SHARES_PATH, ReqTypes, Results)


NTP_ENTRY = "<datetime><ntp enable='true'><serveraddress/><sync>" + \
            "<boot enable='false'/><time enable='false'/></sync></ntp>" + \
            "</datetime>"    # FIXME
MANUAL_ENTRY = "<datetime><manual><date><year/>" + \
            "<month/><day/><hour/><min/><sec/></date></manual></datetime>"
LAN_ENTRY = "<network><familyconfig><inet4><dns/><gateway/></inet4>" + \
        "</familyconfig><netiflist><netif name='lan1'><protocollist>" + \
        "<protocol family='inet4' enable='true'><dhcp/>" + \
        "</protocol></protocollist></netif></netiflist></network>"
SHARES_ENTRY = "<shares/>"
INET_XPATH = "./familyconfig/inet4"
PROTOCOL_XPATH = "./netiflist/netif[@name='lan1']/protocollist/" + \
        "protocol[@family='inet4']"
NTPSRV_XPATH = "./ntp/serveraddress"
BOOT_XPATH = "./ntp/sync/boot"
TIME_XPATH = "./ntp/sync/time"
YEAR_XPATH = "./manual/date/year"
MONTH_XPATH = "./manual/date/month"
DAY_XPATH = "./manual/date/day"
HOUR_XPATH = "./manual/date/hour"
MIN_XPATH = "./manual/date/min"
SEC_XPATH = "./manual/date/sec"

EXCLUDE_SHARES_LIST = ['LAN DISK Manual', ]
DETAILPERM_XPATH = "permission/detailpermission"
NOGUEST_XPATH = "shareservice/noguest"

SHARE_ACCESS_NOCHANGE = 'nochange'
SHARE_ACCESS_EVERYONE = 'everyone'
SHARE_ACCESS_ADMIN = 'admin'


class InitSetting(object):

    def load_request(self, cmd):
        result = Request(cmd, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        return result

    def get_admin_info(self):
        admin_dic = {"mail": "", "password": "", "pw_confirm": ""}
        config = self.load_request(FAL_USERS_PATH)
        adminuser = config.get_node("userlist/user[@admin='true']")
        admin_dic["mail"] = adminuser.get_node(
            "mailaddr").get_text()
        admin_dic["password"] = adminuser.get_node(
            "passwd").get_text(do_strip=False)
        admin_dic["pw_confirm"] = admin_dic["password"]
        return admin_dic

    def get_landisk_name(self):
        info = {}
        result = self.load_request(FAL_SYSINFO_PATH)
        info["name"] = result.get_node("hostdesc/name").get_text()
        info["desc"] = result.get_node("hostdesc/desc").get_text()
        info["location"] = result.get_node("hostdesc/location").get_text()
        return info

    def get_lan1ipv4_info(self):
        info_dic = {"dhcp": "auto"}
        result = self.load_request(FAL_NETWORK_PATH)
        if not result.get_node(PROTOCOL_XPATH).get_node("dhcp").is_enabled():
            info_dic["dhcp"] = "custom"

        entry = Entry(ReqTypes.QUERY, "<network/>")
        query_info = Request(FAL_NETWORK_PATH, filter_cmd=SUDO_CMD).request(
                               entry).get_entrylist().get_entry().get()
        try:
            addr = query_info.get_node(PROTOCOL_XPATH).get_node(
                "addrlist/addr")
            info_dic["ipaddr"] = addr.get_text()
            info_dic["subnet"] = addr.get_attr_value("netmask")
        except NotFoundError:
            pass
        try:
            inet = query_info.get_node(INET_XPATH)
            info_dic["gateway"] = inet.get_node("gateway/addr").get_text()
            info_dic["dnssrv"] = inet.get_node("dns/addrlist/addr").get_text()
        except NotFoundError:
            pass
        return info_dic

    def get_time_info(self):
        info_dic = {"sync": 1, "boot": False, "daily": False}
        result = self.load_request(FAL_DATETIME_PATH)
        if not result.get_node("ntp").is_enabled():
            info_dic["sync"] = 0
        try:
            info_dic["ntpsrv"] = result.get_node(NTPSRV_XPATH).get_text()
            if result.get_node(BOOT_XPATH).is_enabled():
                info_dic["boot"] = True
            if result.get_node(TIME_XPATH).is_enabled():
                info_dic["daily"] = True
                hour = result.get_node(
                            TIME_XPATH).get_node("hour").get_text()
                info_dic["hour"] = int(hour)
                minute = result.get_node(
                            TIME_XPATH).get_node("minute").get_text()
                info_dic["minute"] = int(minute)
        except NotFoundError:
            pass

        # FIXME: FAL/PAL の呼出が必要?
        info_dic["datetime"] = datetime.today()

        return info_dic

    def __is_exclude_share(self, share_node):

        volume = None

        try:
            name = share_node.get_value('name')
            if name in EXCLUDE_SHARES_LIST:
                return True
        except:
            return True

        try:
            volume = share_node.get_value('volume')
            if volume in STOR_IF_TO_VOL['usb'].values():
                return False
        except:
            pass

        try:
            if share_node.is_set_to_true('undeletable'):
                return True
        except:
            pass

        return False

    def get_shares_info(self):
        info = []
        result = self.load_request(FAL_SHARES_PATH)
        for share_node in result.get_node_list('sharelist/share'):
            if self.__is_exclude_share(share_node):
                continue

            info_dic = {
                'id': encode_share_id(share_node.get_value('name')),
                'name': share_node.get_value('name'),
                'access': SHARE_ACCESS_NOCHANGE,
            }
            info.append(info_dic)

        info.sort(key=lambda x: x['name'])
        return info

    def send_request(self, cmd, entry):
        try:
            response = Request(cmd,
                filter_cmd=SUDO_CMD).request(entry).get_entrylist().get_entry()
            if response.get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def sysinfo_set(self, data):
        config = self.load_request(FAL_SYSINFO_PATH)
        config.get_node("hostdesc/name").set_text(data["name"])
        config.get_node("hostdesc/location").set_text(data["location"])
        try:
            config.get_node("hostdesc/desc").set_text(data["desc"])
        except:
            pass
        entry = Entry(ReqTypes.APPLY, config)
        return self.send_request(FAL_SYSINFO_PATH, entry)

    def sync_now(self):
        entry = Entry(ReqTypes.EXECUTE)
        return self.send_request(FAL_DATETIME_PATH, entry)

    def network_set(self, data):
        entry = Entry(ReqTypes.APPLY, LAN_ENTRY)
        protocol = entry.get().get_node(PROTOCOL_XPATH)
        inet = entry.get().get_node(INET_XPATH)
        if data["dhcp"] == "auto":
            protocol.get_node("dhcp").enable()
            inet.get_node("dns").set_attrs({"source": "lan1"})
            inet.get_node("gateway").set_attrs({"source": "lan1"})
        else:
            protocol.get_node("dhcp").enable(False)
            protocol.append_child("addrlist")
            protocol.get_node("addrlist").append_child(
                    "addr", data["ipaddr"], {"netmask": data["subnet"]})
            inet.get_node("dns").set_attrs({"source": "static"})
            inet.get_node("gateway").set_attrs({"source": "static"})
            inet.get_node("dns").append_child("addrlist")
            inet.get_node("dns/addrlist").append_child("addr", data["dnssrv"])
            inet.get_node("gateway").append_child("addr", data["gateway"])
        return self.send_request(FAL_NETWORK_PATH, entry)

    def time_set(self, data):
        if data["sync"] == "1":
            entry = Entry(ReqTypes.APPLY, NTP_ENTRY)
            entry.get().get_node("ntp").set_attrs({"enable": "true"})
            entry.get().get_node(NTPSRV_XPATH).set_text(data["ntpsrv"])
            if data["boot"]:
                entry.get().get_node(BOOT_XPATH).set_attrs({"enable": "true"})
            if data["daily"]:
                entry.get().get_node(TIME_XPATH).set_attrs({"enable": "true"})
                entry.get().get_node(TIME_XPATH).append_child(
                                                "hour", data["hour"])
                entry.get().get_node(TIME_XPATH).append_child(
                                                "minute", data["minute"])
        else:
            entry = Entry(ReqTypes.APPLY, MANUAL_ENTRY)
            entry.get().get_node(YEAR_XPATH).set_text(
                                        str(data["datetime"].year))
            entry.get().get_node(MONTH_XPATH).set_text(
                                        str(data["datetime"].month))
            entry.get().get_node(DAY_XPATH).set_text(str(data["datetime"].day))
            entry.get().get_node(HOUR_XPATH).set_text(
                                        str(data["datetime"].hour))
            entry.get().get_node(MIN_XPATH).set_text(
                                        str(data["datetime"].minute))
            entry.get().get_node(SEC_XPATH).set_text(
                                        str(data["datetime"].second))
            config = self.load_request(FAL_DATETIME_PATH)
            entry.get().append_child(config.get_node("ntp"))
            entry.get().get_node("ntp").enable(False)
        return self.send_request(FAL_DATETIME_PATH, entry)

    def __update_config_everyone_share(self, share_node):
        share_node.get_node(DETAILPERM_XPATH).set_attrs({'enable': 'false'})
        share_node.get_node(NOGUEST_XPATH).set_attrs({'enable': 'false'})

    def __update_config_admin_share(self, share_node):
        share_node.get_node(DETAILPERM_XPATH).set_attrs({'enable': 'true'})

    def shares_set(self, data):
        config = self.load_request(FAL_SHARES_PATH)
        result = True

        for share in data['shares']:
            if share['access'] == SHARE_ACCESS_NOCHANGE:
                continue

            try:
                share_node = config.get_node(
                    "sharelist/share/name[text()='%s']/.." % share['name'])
                share_node = share_node.get_clone()
            except:
                continue

            try:
                if share['access'] == SHARE_ACCESS_EVERYONE:
                    self.__update_config_everyone_share(share_node)
                elif share['access'] == SHARE_ACCESS_ADMIN:
                    self.__update_config_admin_share(share_node)
            except:
                logging(
                    self.get_code("ERR_SHARE_INFO_CHANGE"),
                    arglist={'name': share['name']})
                result = False
                continue

            for delete_node in ['uid', 'gid', 'uname']:
                try:
                    share_node.delete_child(delete_node)
                except:
                    pass

            entry = Entry(ReqTypes.MODIFY, "<shares/>")
            entry.get().append_child(share_node)
            if not self.send_request(FAL_SHARES_PATH, entry):
                logging(
                    self.get_code("ERR_SHARE_INFO_CHANGE"),
                    arglist={'name': share['name']})
                result = False

            logging(
                self.get_code("MSG_SHARE_INFO_CHANGE"),
                arglist={'name': share['name']})

        return result

    def get_code(self, msg):
        try:
            from fal_msginfo import Msg
            return getattr(Msg, msg)
        except:
            return None
