# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import datetime
import re
import string
from iobase.xmldata import XmlNode, escape
from iobase.exception import NotFoundError
from ionas.request import (Request, Entry, EntryList)
from product_conf import SUDO_CMD
from fal_info import (FAL_BACKUPS_PATH, ReqTypes, Results)
from ...modules.share_ctrl import ShareCtrl


#ADD_OR_MOD_ENTRY = '<backups><backup><job><name>%(name)s</name><times>%(times)s' + \
#            '</times><schedule/><id/><trash enable="%(trash)s"/><shutdown ' + \
#            'enable="%(shutdown)s"/><all_local enable="%(all_local)s"/>' + \
#            '<sourcelist/><destination><unc>%(destunc)s</unc><user>' + \
#            '%(destuser)s</user><passwd>%(destpswd)s</passwd>' + \
#            '</destination></job></backup></backups>'
ADD_OR_MOD_ENTRY = """<backups>
    <backup>
        <job full="%(full)s">
            <name>%(name)s</name>
            <times>%(times)s</times>
            <schedule/>
            <id/>
            <trash enable="%(trash)s"/>
            <shutdown enable="%(shutdown)s"/>
            <all_local enable="%(all_local)s"/>
            <sourcelist/>
            <destination>
                <unc>%(destunc)s</unc>
                <user>%(destuser)s</user>
                <passwd>%(destpswd)s</passwd>
            </destination>
        </job>
    </backup>
</backups>"""
DEL_ENTRY = "<backups><backup><job><id/><name/></job></backup></backups>"
EXEC_ENTRY = "<backups><backup/></backups>"

JOB_XPATH = "backup/joblist/job"
NAME_XPATH = "backup/job/name"
ID_XPATH = "backup/job/id"
SCHEDULE_XPATH = "backup/job/schedule"
SOURCELIST_XPATH = "backup/job/sourcelist"

XPATH_JOB_BYID = 'backup/joblist/job[id="%s"]'
XPATH_JOB_BYNAME = 'backup/joblist/job[name="%s"]'
TRUE = "true"


class Backup(object):

    def delete_job(self, id):
        return self.send_request(self.delete_entry(id))

    def delete_entry(self, id):
        entry = Entry(ReqTypes.DELETE, DEL_ENTRY)
        target_job = self.get_target_job_node(id=id)
        if not target_job:
            return entry
        entry.get().get_node(ID_XPATH).set_text(id)
        name = target_job.get_node("name").get_text(False)
        entry.get().get_node(NAME_XPATH).set_text(name)
        return entry

    def execute_job(self, id):
        entry = Entry(ReqTypes.EXECUTE, EXEC_ENTRY)
        target_job = self.get_target_job_node(id=id)
        if not target_job:
            return False
        entry.get().get_node("backup").append_child(target_job)
        return self.send_request(entry, False)

    def send_request(self, req_entry, bool=True):
        try:
            response = Request(FAL_BACKUPS_PATH, filter_cmd=SUDO_CMD).request(req_entry)
            res_entry = response.get_entrylist().get_entry()
            result = res_entry.get_result()
            if bool:
                if result == Results.OK:
                    return True
                else:
                    return False
            else:
                return result
        except:
            return False

    def add_job(self, data):
        entry_str = ADD_OR_MOD_ENTRY % {
                "name": data["name"],
                "times": data["times"],
                "trash": str(data["trash"]).lower(),
                "shutdown": str(data["shutdown"]).lower(),
                "all_local": str(data["all_local"]).lower(),
                "destunc": escape(data["dest"]),
                "destuser": escape(data["destuser"]),
                "destpswd": escape(data["destpswd"]),
                "full": str(data["full"]).lower(),
        }
        entry = Entry(ReqTypes.ADD, entry_str)
        sche_node = entry.get().get_node(SCHEDULE_XPATH)
        self.make_schedule_node(sche_node, data)
        sourlist_node = entry.get().get_node(SOURCELIST_XPATH)
        self.make_sourcelist_node(sourlist_node, data)

        ShareCtrl().set_share_ro(data["dest"])

        return self.send_request(entry)

    def make_schedule_node(self, sche_node, data):
        enable_dic = {"1": "true", "0": "false"}
        sche_node.set_attrs({"enable": enable_dic[data["schedule"]]})
        if data["schedule"] == "1":
            day_dic = {"0": "Sun", "1": "Mon", "2": "Tue", "3": "Wed",
                       "4": "Thu", "5": "Fri", "6": "Sat"}
            for day in data["day"]:
                sche_node.append_child("day", day_dic[day])
            time_node = XmlNode("time")
            time_node.append_child("hour", str(data["time"].hour))
            time_node.append_child("minute", str(data["time"].minute))
            sche_node.append_child(time_node)
        return 
        
    def make_sourcelist_node(self, sourlist_node, data):
        if data["sourcelist"] == "":
            return
        for info in re.split("\b", data["sourcelist"]):
            items = re.split("\t", info)
            source_node = XmlNode("source")
            source_node.append_child("unc", items[0])
            source_node.append_child("user", items[1])
            source_node.append_child("passwd", items[2])
            sourlist_node.append_child(source_node)
        return 

    def modify_job(self, data, id):
        entry_str = ADD_OR_MOD_ENTRY % {
                "name": data["name"],
                "times": data["times"],
                "trash": str(data["trash"]).lower(),
                "shutdown": str(data["shutdown"]).lower(),
                "all_local": str(data["all_local"]).lower(),
                "destunc": escape(data["dest"]),
                "destuser": escape(data["destuser"]),
                "destpswd": escape(data["destpswd"]),
                "full": str(data["full"]).lower(),
        }
        entry = Entry(ReqTypes.MODIFY, entry_str)
        entry.get().get_node(ID_XPATH).set_text(id)
        sche_node = entry.get().get_node(SCHEDULE_XPATH)
        self.make_schedule_node(sche_node, data)
        sourlist_node = entry.get().get_node(SOURCELIST_XPATH)
        self.make_sourcelist_node(sourlist_node, data)

        ShareCtrl().set_share_ro(data["dest"])

        return self.send_request(entry)

    def get_job_list(self):
        try:
            result = Request(FAL_BACKUPS_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
            job_list = result.get().get_node_list(JOB_XPATH)
        except NotFoundError:
            job_list = []
        return job_list

    def delete_or_execute_init_info(self, id):
        info_dic = {"id": id, "name": "", "dest": "", "source": [], "all_local": "false"}
        target_job = self.get_target_job_node(id=id)
        if not target_job:
            return None
        try:
            info_dic["name"] = target_job.get_node("name").get_text(False)
            info_dic["dest"] = target_job.get_node("destination/unc").get_text(False)
            for node in target_job.get_node_list("sourcelist/source"):
                info_dic["source"].append(node.get_node("unc").get_text(False))
            if target_job.get_node("all_local").is_enabled():
                info_dic["all_local"] = "true"
        except:
            pass
        return info_dic

    def get_jobinfo_list(self):
        #各ジョブの情報を取得
        job_list = []
        for job in self.get_job_list():
            info_dic = {
                "id": "",
                "name": "",
                "schedule": {},
            }
            info_dic["id"] = job.get_node("id").get_text()
            info_dic["name"] = job.get_node("name").get_text(False)
            try:
                schedule = job.get_node("schedule")
                info_dic["schedule"]["enabled"] = schedule.is_enabled()
                if schedule.get_attr_value("enable") == TRUE:
                    day_list = []
                    for node in schedule.get_node_list("day"):
                        day_list.append(node.get_text())
                    info_dic["schedule"]["day"] = day_list
                    hour = schedule.get_node("time").get_node("hour").get_text()
                    if len(hour) == 1:
                        hour = "0" + hour
                    minu = schedule.get_node("time").get_node("minute").get_text()
                    if len(minu) == 1:
                        minu = "0" + minu
                    info_dic["schedule"]["time"] = hour + ":" + minu
            except NotFoundError:
                pass

            job_list.append(info_dic)
        return job_list

    def get_target_job_node(self, name="", id=""):
        try:
            result = Request(FAL_BACKUPS_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
            if id:
                target_job = result.get().get_node(XPATH_JOB_BYID % id)
            elif name:
                target_job = result.get().get_node(XPATH_JOB_BYNAME % name)
            else:
                return None
            return target_job
        except:
            return None

    def modify_init_basic_info(self, id):
        info_dic = {"name": "",
                    "times": 0,
                    "all_local": False,
                    "trash": False,
                    "shutdown": False,
                    "schedule": 0,
                    "day": [],
                    "dest": "",
                    "destuser": "",
                    "destpswd": "",
                    "time": datetime.time(12, 0),
                    "full": False,
        }
        target_job = self.get_target_job_node(id=id)
        if not target_job:
            return info_dic
        try:
            info_dic["name"] = target_job.get_node("name").get_text(False)
        except:
            pass
        try:
            times = target_job.get_node("times").get_text()
            #if times in ["0", "1", "2", "3", "4", "5"]:
            #    info_dic["times"] = string.atoi(times)
            info_dic["times"] = int(times)
        except:
            pass
        try:
            for item in ("all_local", "trash", "shutdown"):
                if target_job.get_node(item).get_attr_value("enable") == "true":
                    info_dic[item] = True
        except:
            pass
        try:
            info_dic["full"] = target_job.get_attr_value("full") == "true"
        except:
            pass
        try:
            sche_node = target_job.get_node("schedule")
            if sche_node.get_attr_value("enable") == "true":
                day_dic = {"Sun": 0, "Mon": 1, "Tue": 2, "Wed": 3, "Thu": 4,
                       "Fri": 5, "Sat": 6}
                info_dic["schedule"] = 1
                for node in sche_node.get_node_list("day"):
                    info_dic["day"].append(day_dic[node.get_text()])
                hour = sche_node.get_node("time").get_node("hour").get_text()
                minu = sche_node.get_node("time").get_node("minute").get_text()
                info_dic["time"] = datetime.time(string.atoi(hour),
                                             string.atoi(minu))
        except:
            pass
        try:
            dest_node = target_job.get_node("destination")
            info_dic["dest"] = dest_node.get_node("unc").get_text(False)
            info_dic["destuser"] = dest_node.get_node("user").get_text(False)
            info_dic["destpswd"] = dest_node.get_node("passwd").get_text(False)
        except:
            pass
        return info_dic

    def modify_init_sourcelist_info(self, id):
        sour_list = []
        target_job = self.get_target_job_node(id=id)
        if not target_job:
            return sour_list
        for node in target_job.get_node_list("sourcelist/source"):
            try:
                unc = node.get_node("unc").get_text(False)
                if unc == "":
                    break
                source = {
                    "unc": unc,
                    "user": node.get_node("user").get_text(False),
                    "pswd": node.get_node("passwd").get_text(False)}
                sour_list.append(source)
            except:
                pass
        return sour_list

    def multi_del_request_entry_list(self, idlist=[]):
        entry_list = EntryList()
        if not idlist:
            return entry_list
        for id in idlist:
            entry_list.append_entry(self.delete_entry(id))
        return entry_list

    def id_to_jobinfo(self, id):
        for jobinfo in self.get_jobinfo_list(): 
            if jobinfo["id"] == id:
                return jobinfo
