# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry, EntryList)
from product_conf import (SUDO_CMD, STOR_IF_TO_VOL)
from iobase.xmldata import XmlConfNode, xpath_escape
from iobase.exception import ExecError, NotFoundError

from fal_info import (FAL_NETWORK_PATH, FAL_SYSINFO_PATH,
    FAL_SHARES_PATH, FAL_VOLCTRL_PATH, FAL_PORTS_PATH,
    ReqTypes, Results)

import re

NAME_NAME = "name"
SHARE_LIST_PATH =\
     "sharelist/share"
MS_SHARE_LIST_PATH =\
     "sharelist/share/shareservice/msshare[@enable='true']/../.."
USB_SHARE_DICT = dict(zip(
    STOR_IF_TO_VOL["usb"].values(),
    STOR_IF_TO_VOL["usb"].keys()))
VOLCTRL_QUERY_STR = """<volctrl><volume>
    <name type="volume">%s</name></volume></volctrl>"""
UNC_RE = r"^\\\\([^\\]+)\\(.+)$"
UNC_FMT = "\\\\%s\\%s"


def _request(fal, data):
    return Request(fal, filter_cmd=SUDO_CMD).request(data)


def request(fal, type=None, data=None):
    if type:
        if not data:
            return _request(fal, Entry(type)).\
                get_entrylist().get_entry()
        if isinstance(data, str):
            return _request(fal, Entry(type, XmlConfNode(data))).\
                get_entrylist().get_entry()
        if isinstance(data, XmlConfNode):
            return _request(fal, Entry(type, data)).\
                get_entrylist().get_entry()
    else:
        if isinstance(data, EntryList):
            return  _request(fal, data).\
                get_entrylist()
        if (isinstance(data, Entry) or
            isinstance(data, XmlConfNode)):
            return  _request(fal, data).\
                get_entrylist().get_entry()


def get_tip_node(node, fullpath):
    for path in fullpath.split("/"):
        try:
            node.get_node(path)
        except NotFoundError:
            node.append_child(path)
        finally:
            node = node.get_node(path)
    return node


class ShareCtrl():
    def __init__(self):
        # FALへのLOADを１度で済ますためクラス化し
        # メンバにもつ
        self.share_config = None
        self.port_config = None

    def _get_local_host_name(self):
        HOST_NAME_PATH = "hostdesc/name"

        sysinfo_config = request(FAL_SYSINFO_PATH, ReqTypes.LOAD).get()
        return sysinfo_config.get_node(HOST_NAME_PATH).get_text().lower()

    def _get_local_addr_list(self):
        NETWORK_NODE_STR = "<network/>"
        ADDR_LIST_PATH = "netiflist/netif/protocollist/protocol/addrlist/addr"

        network_config = request(FAL_NETWORK_PATH, ReqTypes.QUERY,
            XmlConfNode(NETWORK_NODE_STR)).get()

        return [
            node.get_text().lower()
            for node
            in network_config.get_node_list(ADDR_LIST_PATH)]

    def _is_local_share(self, unc):
        LOCALHOST_NAME = "localhost"
        LOOPBACK_ADDR = "127.0.0.1"

        local_list = [LOCALHOST_NAME] + [LOOPBACK_ADDR] + [
            self._get_local_host_name()] + self._get_local_addr_list()

        match = re.match(UNC_RE, unc)
        if match:
            host_name = match.group(1)
            if host_name.lower() in local_list:
                return True
        else:
            # UNC形式でなければローカル共有として処理
            return True
        return False

    def _is_exclusive_share(self, unc):
        EXCLUSIVE_FS_LIST = ["xfs", "ext3", "ext4", "vfsext"]
        PATH_NAME = "path"
        VOL_IDX = 1
        VOL_SEP = "/"
        FILESYSTEM_PATH = "volume/filesystem"

        share_node = self._get_share_node(unc)
        share_path = share_node.get_value(PATH_NAME)

        volume = share_path.lstrip(VOL_SEP).split(VOL_SEP)[VOL_IDX]

        res_entry = request(FAL_VOLCTRL_PATH, ReqTypes.QUERY,
            VOLCTRL_QUERY_STR % volume)

        if res_entry.get_result() == Results.DEVICE_NOT_FOUND:
            return False

        fs = res_entry.get().get_value(FILESYSTEM_PATH)

        if fs in EXCLUSIVE_FS_LIST:
            return True
        return False

    def _is_local_exclusive_share(self, unc):
        if not self._is_local_share(unc):
            return False

        self.share_config = request(FAL_SHARES_PATH, ReqTypes.LOAD).get()

        if not self._is_exclusive_share(unc):
            return False
        return True

    def _get_share_node(self, unc):
        SHARE_PATH = "sharelist/share[name=%s]"

        share_name = re.match(UNC_RE, unc).group(2)

        return self.share_config.get_node(
            SHARE_PATH % xpath_escape(share_name))

    def _unc_to_host_share(self, unc):
        match = re.match(UNC_RE, unc)
        if match:
            host = match.group(1)
            share = match.group(2)
        else:
            # UNC形式でなければローカル共有名として処理
            host = "localhost"
            share = unc
        return (host, share)

    def _get_msnetwork_share_list(self):
        return self._get_local_share_list(MS_SHARE_LIST_PATH)

    def _get_local_share_list(self, path=SHARE_LIST_PATH):
        local_share_list = [
            node.get_value(NAME_NAME)
            for node
            in self.share_config.get_node_list(path)]

        return local_share_list

    def _is_share_mode_port(self, port):
        PORT_MODE_PATH =\
            "usblist/usb[@port='%s']/modelist/mode[@enable='true']"
        SHARE_MODE = "share"

        if not self.port_config:
            self.port_config = request(FAL_PORTS_PATH, ReqTypes.LOAD).get()

        if self.port_config.get_value(PORT_MODE_PATH % port) == SHARE_MODE:
            return True
        return False

    def _get_share_mode_share_list(self, share_list):
        not_share_mode_port_list = []

        for share_name in share_list:
            if share_name in USB_SHARE_DICT.keys():
                if not self._is_share_mode_port(USB_SHARE_DICT[share_name]):
                    not_share_mode_port_list += [share_name]

        for not_share_port in not_share_mode_port_list:
            share_list.remove(not_share_port)

        return share_list

    def _get_mounted_share_list(self, share_list):
        STATUS_PATH = "volume/status"
        MOUNTED_STR = "mounted"

        unmounted_share_list = []

        for share_name in share_list:
            if share_name in USB_SHARE_DICT.keys():
                res_entry = request(FAL_VOLCTRL_PATH, ReqTypes.QUERY,
                    VOLCTRL_QUERY_STR % share_name)

                if res_entry.get_result() == Results.DEVICE_NOT_FOUND:
                    unmounted_share_list += [share_name]
                    continue

                if res_entry.get_result() != Results.OK:
                    raise ExecError(FAL_VOLCTRL_PATH)

                if res_entry.get().get_value(STATUS_PATH) != MOUNTED_STR:
                    unmounted_share_list += [share_name]

        for unmounted_share in unmounted_share_list:
            share_list.remove(unmounted_share)

        return share_list

    def get_local_share_list(self):
        self.share_config = request(FAL_SHARES_PATH, ReqTypes.LOAD).get()

        return self._get_local_share_list()

    def set_share_ro(self, unc):
        SHARES_NODE = "<shares><share><name>%s</name></share></shares>"
        NAME_PATH = "name"
        READONLY_PATH = "permission/readonly"
        SHARE_READONLY_PATH = "share/" + READONLY_PATH

        try:
            # uncにUNC形式でなく
            # ローカル共有名がくる場合あるので
            # UNC形式に正規化
            unc = UNC_FMT % self._unc_to_host_share(unc)

            if not self._is_local_exclusive_share(unc):
                return

            share_node = self._get_share_node(unc)
            if share_node.get_node(READONLY_PATH).is_enabled():
                return

            shares_node = XmlConfNode(
                SHARES_NODE % share_node.get_value(NAME_PATH))

            get_tip_node(shares_node, SHARE_READONLY_PATH).enable()

            if request(FAL_SHARES_PATH,
                ReqTypes.MODIFY, shares_node).get_result() != Results.OK:
                raise ExecError(FAL_SHARES_PATH)

        except NotFoundError:
            pass
