# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core.exceptions import ValidationError
from django.forms import CharField
from project.view import FuncView
from project.fal.command import (get_message, logging)
from root.modules.form import BaseForm
import settings
from django import forms
from project.forms.fields.extras.ipv4address import IPv4AddressField
from project.forms.fields.extras.ipv4v6address import IPv4v6AddressField
from project.forms.fields.extras.ipv4subnetmask import IPv4SubnetMaskField
from project.forms.fields.extras.jumboframesize import JumboFrameSizeField
from project.forms.fields.radio import RadioField
from ... modules.network import Network
from ... modules.utils import ip_v4_netmask_validate


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'ipv4']))
    
class View(FuncView):
    class Form(BaseForm):
        dhcp = RadioField(
                label=_(u'IP Address Setting Method'),
                required=True,
                choices=(('disable', _(u'Disable')),
                     ('auto', _(u'Obtain an IP address automatically(DHCP)')),
                     ('custom', _(u'Use the following IP address'))),
                initial='disable')

        # self._ip_v4_netmask_validateにて
        # netmask考慮した禁則をするので
        # IPv4AddressFieldは使用せずCharField
        ipaddr = CharField(label=_(u'IP Address'),
                required=False,
                widget=forms.TextInput(attrs={"class": "lan2ipv4"}),
                initial='')

        netmask = IPv4SubnetMaskField(
                label=_(u'Subnet Mask'),
                required=False,
                attrs={"class": "lan2ipv4"},
                initial='255.255.255.0')
        framesize = JumboFrameSizeField(
                label=_(u'Frame Size'),
                initial=1500,
                required=False,
                widget=forms.TextInput(attrs={"class": "lan2ipv4"}))
        wol = RadioField(
                label=_(u'Wake On LAN'),
                required=False,
                choices=(('true', _(u'Enable')),
                         ('false', _(u'Disable'))),
                initial='false')
        common = RadioField(
                choices=(('auto',_(u'Auto')),
                              ('custom',_(u'Custom Settings'))),
                label=_(u'Common Setting'),
                required=False,
                initial='auto')
        #mainif = RadioField(
        #        choices=(('lan1',_(u'LAN1')), ('lan2',_(u'LAN2'))),
        #        label=_(u'Main Interface'),
        #        required=False,
        #        initial='lan1')
        gateway = IPv4AddressField(
                label=_(u'Default Gateway'),
                required=False,
                widget=forms.TextInput(attrs={"class": "lan2ipv4"}),
                initial='')
        dns = IPv4v6AddressField(
                label=_(u'DNS Server'),
                required=False,
                widget=forms.TextInput(attrs={"class": "lan2ipv4"}),
                initial='')

        def __init__(self, *args, **kwargs):
            super(View.Form, self).__init__(*args, **kwargs)
            from project.forms.fields.extras.network import MainInterfaceField
            _mainif = MainInterfaceField(label=_(u"Main Interface"), required=False)
            if len(_mainif.choices) > 1:
                self.fields[u"mainif"] = _mainif

        def _dhcp_is_enabled(self):
            network = Network()
            if network.dhcp_is_set_enabled("lan1", "inet4"):
                return True
            return False

        def clean(self):
            if self.cleaned_data.get('dhcp') != "disable":
                if "framesize" in self.cleaned_data.keys():
                    if self.cleaned_data.get('framesize') == None:
                        raise ValidationError(_(u'"Frame Size" is required.'))
            if self.cleaned_data.get('dhcp') == "custom":
                ipaddr = self.cleaned_data.get('ipaddr')
                netmask = self.cleaned_data.get('netmask')
                if ipaddr == "":
                    raise ValidationError(_(u'"IP Address" is required.'))
                if netmask == "":
                    raise ValidationError(_(u'"Subnet Mask" is required.'))
                ip_v4_netmask_validate(ipaddr, netmask)
            if self.cleaned_data.get('common') == "auto":
                if self.cleaned_data.get('dhcp') == "custom":
                    if not self._dhcp_is_enabled():
                        #raise ValidationError(_(u'DHCP is disable. "Common Setting" can not be set to "Auto".'))
                        raise ValidationError(_(u'DHCP is disable. "%(common_setting)s" can not be set to "%(auto)s".') % {
                            "common_setting": _(u"Common Setting"),
                            "auto": _("Auto"),
                        })
            return self.cleaned_data

    views = ['apply']
    network = Network()

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': self.Form(
                        initial=self.network.get_ipv4_info(netif="lan2"))}}

    def view_apply(self, request):
        with self.processing(constname="ERR_NET_STATE_CHANGE") as p:
            form = self.Form(extdir='fields', data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.network.lan2_ipv4_set(data=form.cleaned_data):
                    logging(self.network.get_msg_code("MSG_NET_CHANGE"))
                    self.set_message(_(u'Setting up the network. Please wait until STATUS lamp blinking stops.'))
                    rst = True
                else:
                    self.set_message(get_message('ERR_NET_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form}}


ipv4 = View.as_view()
