# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import re
from iobase.exception import NotFoundError
from ionas.request import (Request, Entry)
from project.fal.command import logging
from product_conf import SUDO_CMD
from fal_info import (FAL_FIRMWARE_PATH, FAL_PACKAGES_PATH,
                FAL_SYSINFO_PATH, ReqTypes, Results)

FIRM_ENTRY = "<firmware/>"
ENABLE = "enable"
TRUE = "true"

VERSION_RE = "\d+\.\d+"


class Firmware(object):
    def update_request(self, file=None):
        if file:
            entry = Entry(ReqTypes.EXECUTE, FIRM_ENTRY)
            entry.set_file(file)
        else:
            entry = Entry(ReqTypes.UPDATE, FIRM_ENTRY)
        result = self.send_request(entry)
        return result.get_result()

    def apply_request(self, type, data={}):
        enable_dic = {"1": "true", "0": "false"}
        entry = Entry(ReqTypes.APPLY, FIRM_ENTRY)
        entry.get().append_child(
                type, attrs={"enable": enable_dic[data[type]]})
        if data[type] == "1":
            entry.get().append_child("time")
            entry.get().get_node("time").append_child(
                                    "hour", text=data["time"])
            package_set = self.set_time_to_package_config(data["time"])
            if not package_set:
                return package_set
        result = self.send_request(entry)
        if result.get_result() != Results.OK:
            return False
        return True

    def send_request(self, entry):
        response = Request(FAL_FIRMWARE_PATH,
                filter_cmd=SUDO_CMD).request(entry)
        return response.get_entrylist().get_entry()

    def __get_code(self, msg):
        try:
            from fal_msginfo import Msg
            return getattr(Msg, msg)
        except:
            return None

    def set_time_to_package_config(self, time):
        package_conf = self.load_config(FAL_PACKAGES_PATH)
        package_conf.get_node("time/hour").set_text(time)
        save_entry = Entry(ReqTypes.SAVE, package_conf)
        result = Request(FAL_PACKAGES_PATH, filter_cmd=SUDO_CMD).request(
                save_entry).get_entrylist().get_entry()
        if result.get_result() == Results.OK:
            return True
        else:
            return False

    def load_config(self, cmd):
        try:
            result = Request(cmd, filter_cmd=SUDO_CMD).request(
                Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        except:
            result = ""
        return result

    def get_config_init(self, type):
        init = {type: 0}
        result = self.load_config(FAL_FIRMWARE_PATH)
        if result == "":
            return init
        if result.get_node(type).is_enabled():
            init[type] = 1
        try:
            init["time"] = result.get_node("time/hour").get_text()
        except NotFoundError:
            pass
        return init

    def query_firmware(self):
        """Query firmware info from fal_firmware."""

        try:
            src_entry = Entry(ReqTypes.QUERY, "<firmware/>")
            src_request = Request(FAL_FIRMWARE_PATH, filter_cmd=SUDO_CMD)
            dst_response_node = src_request.request(src_entry)
            dst_entry_node = dst_response_node.get_entrylist().get_entry()
            if dst_entry_node.get_result() != Results.OK:
                if dst_entry_node.get_result() != Results.DOWNLOAD_ERROR:
                    logging(self.__get_code("ERR_FIRMWARE_CONNECT"))
                return None

            dst_firmware_node = dst_entry_node.get()
            return dst_firmware_node
        except:
            return None

    def get_model_name_from_firm_data(self, firmware_node):
        """Get firmware model name from firmware data."""

        try:
            xpath = "software/name"
            model_name = firmware_node.get_node(xpath).get_text()
            return model_name
        except:
            return None

    def get_firm_version_from_firm_data(self, firmware_node):
        """Get firmware version number from firmware data."""

        try:
            xpath = "software/version"
            firm_version = firmware_node.get_node(xpath).get_text()
            return float(re.findall(VERSION_RE, firm_version)[0])
        except:
            return 0

    def query_sysinfo(self):
        """Query sysinfo from fal_sysinfo."""

        try:
            src_entry = Entry(ReqTypes.QUERY, "<sysinfo/>")
            src_request = Request(FAL_SYSINFO_PATH, filter_cmd=SUDO_CMD)
            dst_response_node = src_request.request(src_entry)
            dst_entry_node = dst_response_node.get_entrylist().get_entry()
            if dst_entry_node.get_result() != Results.OK:
                return None

            dst_sysinfo_node = dst_entry_node.get()
            return dst_sysinfo_node
        except:
            return None

    def get_model_name_from_sysinfo(self, sysinfo_node):
        """Get firmware model name from sysinfo."""

        try:
            xpath = "series"
            model_name = sysinfo_node.get_node(xpath).get_text()
            return model_name
        except:
            return None

    def get_firm_version_from_sysinfo(self, sysinfo_node):
        """Get firmware version number from sysinfo."""

        try:
            xpath = "fwversion"
            firm_version = sysinfo_node.get_node(xpath).get_text()
            return float(re.findall(VERSION_RE, firm_version)[0])
        except:
            return 0

    def get_cur_ver(self):
        """Get current firmware version."""

        sysinfo_node = self.query_sysinfo()
        if not sysinfo_node:
            return 0

        return self.get_firm_version_from_sysinfo(sysinfo_node)

    def get_firm_info_data(self):
        infos = {"changes": [], "new_ver": 0, "connect": "no"}

        # firmware update data
        firmware_node = self.query_firmware()
        if not firmware_node:
            return infos

        infos["connect"] = "ok"
        released_model = self.get_model_name_from_firm_data(firmware_node)
        if not released_model:
            return infos

        released_version = self.get_firm_version_from_firm_data(firmware_node)
        if not released_version:
            return infos

        # current running firmware
        sysinfo_node = self.query_sysinfo()
        if not sysinfo_node:
            return infos

        current_model = self.get_model_name_from_sysinfo(sysinfo_node)
        if not current_model:
            return infos

        current_version = self.get_firm_version_from_sysinfo(sysinfo_node)
        if not current_version:
            return infos

        # check model
        if released_model != current_model:
            return infos

        # check version
        if released_version > current_version:
            infos["new_ver"] = released_version
            xpath = "software/changelist/change"
            for change in firmware_node.get_node_list(xpath):
                change_dic = {"items": [], "version": ""}
                ver = change.get_attr_value("version")
                if float(re.findall(VERSION_RE, ver)[0]) <= current_version:
                    continue
                change_dic["version"] = ver
                for item in change.get_node_list("itemlist/item"):
                    change_dic["items"].append(item.get_text())
                infos["changes"].append(change_dic)
        else:
            logging(self.__get_code("MSG_FIRMWARE_LATEST_FIRMWARE"))
        return infos
