# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import re
from iobase.exception import NotFoundError
from ionas.request import (Request, Entry)
from product_conf import SUDO_CMD
from iobase.xmldata import XmlNode
from fal_info import (FAL_FIRMWARE_PATH, FAL_PACKAGES_PATH, ReqTypes, Results)

UNINSTALL_ENTRY = "<packages><packagelist/></packages>"
INSTALL_ENTRY = "<packages><packagelist/></packages>"
NOTICE_ENTRY = "<packages><updatenotice enable='false'/></packages>"
AUTOUPDATE_ENTRY = "<packages><autoupdate enable='false'/></packages>"
ID_XPATH = "./id"
NOTICE_XPATH = "./updatenotice"
AUTOUPDATE_XPATH = "./autoupdate"
PACKAGE_XPATH = "./packagelist/package"
VERSION_RE = "\d+\.\d+"      # FIXME


class Package(object):
    def send_request(self, req_entry):
        try:
            response = Request(FAL_PACKAGES_PATH, filter_cmd=SUDO_CMD).request(req_entry)
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def uninstall_package(self, id_list=[]):
        return self.send_request(self.delete_entry(id_list))

    def install_package(self, id_list=[]):
        return self.send_request(self.add_entry(id_list))

    def delete_entry(self, id_list=[]):
        entry = Entry(ReqTypes.UNINSTALL, UNINSTALL_ENTRY)
        pack_list = entry.get().get_node("packagelist")
        for id in id_list:
            package = XmlNode("package")
            package.append_child("packageid", id)
            pack_list.append_child(package)
        return entry

    def add_entry(self, id_list=[]):
        entry = Entry(ReqTypes.INSTALL, INSTALL_ENTRY)
        pack_list = entry.get().get_node("packagelist")
        for id in id_list:
            package = XmlNode("package")
            package.append_child("packageid", id)
            pack_list.append_child(package)
        return entry

    def notice_set(self, data):
        entry = Entry(ReqTypes.APPLY, NOTICE_ENTRY)
        if data["notice"] == "1":
            notice_node = entry.get().get_node(NOTICE_XPATH)
            notice_node.set_attrs({"enable": "true"})
            if data["installed"] == True:
                notice_node.set_attrs({"installed": "true"})
            else:
                notice_node.set_attrs({"installed": "false"})
            entry.get().append_child("time")
            entry.get().get_node("time").append_child("hour", text=data["time"])
            firm_time_set = self.set_time_to_firm_config(data["time"])
            if not firm_time_set:
                return firm_time_set
        return self.send_request(entry)

    def auto_set(self, data):
        entry = Entry(ReqTypes.APPLY, AUTOUPDATE_ENTRY)
        if data["auto"] == "1":
            auto_node = entry.get().get_node(AUTOUPDATE_XPATH)
            auto_node.set_attrs({"enable": "true"})
            entry.get().append_child("time")
            entry.get().get_node("time").append_child("hour", text=data["time"])
            firm_time_set = self.set_time_to_firm_config(data["time"])
            if not firm_time_set:
                return firm_time_set
        return self.send_request(entry)

    def set_time_to_firm_config(self, time):
        firm_conf = self.load_config(FAL_FIRMWARE_PATH)
        firm_conf.get_node("time/hour").set_text(time)
        save_entry = Entry(ReqTypes.SAVE, firm_conf)
        result = Request(FAL_FIRMWARE_PATH, filter_cmd=SUDO_CMD).request(
                save_entry).get_entrylist().get_entry()
        if result.get_result() == Results.OK:
            return True
        else:
            return False

    def load_config(self, cmd):
        try:
            result = Request(cmd, filter_cmd=SUDO_CMD).request(
                Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        except:
            result = ""
        return result

    def get_notice_init_info(self):
        info_dic = {"notice": 0,
                    "installed": False,
                    "time": ""}
        result = self.load_config(FAL_PACKAGES_PATH)
        if result == "":
            return info_dic
        notice_node = result.get_node(NOTICE_XPATH)
        if notice_node.is_enabled():
            info_dic["notice"] = 1
        if notice_node.is_set_to_true("installed"):
            info_dic["installed"] = True
        try:
            info_dic["time"] = result.get_node("time/hour").get_text()
        except:
            pass
        return info_dic

    def get_auto_init_info(self):
        info_dic = {"auto": 0, "time": ""}
        result = self.load_config(FAL_PACKAGES_PATH)
        if result == "":
            return info_dic
        auto_node = result.get_node(AUTOUPDATE_XPATH)
        if auto_node.is_enabled():
            info_dic["auto"] = 1
        try:
            info_dic["time"] = result.get_node("time/hour").get_text()
        except:
            pass
        return info_dic

    def get_package_list(self):
        package_list = []
        try:
            result = Request(FAL_PACKAGES_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LIST)).get_entrylist().get_entry()
            if result.get_result() != Results.OK:
                return package_list
        except:
            return package_list
        for node in result.get().get_node_list(PACKAGE_XPATH):
            infos = {"name": "", "version": "",
                "description": "", "changes": [], "targets": []}
            infos["name"] = node.get_node("name").get_text()
            infos["version"] = node.get_node("version").get_text()
            infos["description"] = node.get_node("description").get_text()
            infos["id"] = node.get_node("packageid").get_text()
            infos["icon"] = node.get_value("icon", None)
            infos["icon_url"] = "/".join(
                    ["/media/img/pkgicon", os.path.basename(infos["icon"])])
            try:
                changes = node.get_node(
                    "changelist/change[@version='%s']" % infos['version'])
                for item in changes.get_node_list("itemlist/item"):
                    infos["changes"].append(item.get_text())
            except NotFoundError:
                pass
            try:
                for target in node.get_node_list("targetlist/target"):
                    infos["targets"].append(
                        {"name": target.get_node("name").get_text(),
                         "version": target.get_node("version").get_text()})
            except NotFoundError:
                pass
            package_list.append(infos)
        return package_list

    def get_package_info_by_id(self, id, version=None, type="detail"):
        if type == "update":
            package_list = self.get_package_release_list()[1]
        else:
            package_list = self.get_package_list()
        for package in package_list:
            if package["id"] == id:
                if version is None:
                    return package
                else:
                    if package["version"] == version:
                        return package
        return {}

    def get_package_release_list(self):
        datalist = []
        result = Request(FAL_PACKAGES_PATH, filter_cmd=SUDO_CMD).request(
                 Entry(ReqTypes.QUERY)).get_entrylist().get_entry()    #FIXME
        if result.get_result() == Results.DOWNLOAD_ERROR:
            return (False, datalist)
        for node in result.get().get_node_list(PACKAGE_XPATH):
            infos = {"name": "", "version": "",
                "description": "", "changes": [], "targets": []}
            #infos["available"] = node.is_set_to_true('available')
            infos["available"] = node.get_attr_value('available', default="true")
            infos["available"] = True if infos["available"] == "true" else False
            infos["version"] = node.get_node("version").get_text()
            infos["description"] = node.get_node("description").get_text()
            infos["id"] = node.get_node("packageid").get_text()
            infos["icon"] = node.get_value("icon", None)
            infos["icon_url"] = "/".join(
                    ["/media/img/pkgicon", os.path.basename(infos["icon"])])
            try:
                old_pkg = self.get_package_info_by_id(
                            infos["id"], type="detail")
                if old_pkg.get("version"):
                    old_ver = float(re.findall(
                            VERSION_RE, old_pkg.get("version"))[0])
                    new_ver = float(re.findall(VERSION_RE, infos["version"])[0])
                    if old_ver >= new_ver:
                        continue
            except:
                pass

            infos["name"] = node.get_node("name").get_text()
            try:
                changes = node.get_node(
                    "changelist/change[@version='%s']" % infos['version'])
                for item in changes.get_node_list("itemlist/item"):
                    infos["changes"].append(item.get_text())
            except NotFoundError:
                pass
            try:
                for target in node.get_node_list("targetlist/target"):
                    infos["targets"].append(
                        {"name": target.get_node("name").get_text(),
                         "version": target.get_node("version").get_text()})
            except NotFoundError:
                pass
            datalist.append(infos)
        return (True, datalist)

