# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os

from django.utils.translation import ugettext as _
from django.conf import settings
from django.http import HttpResponseRedirect
from django.core import validators
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.view import StatusView
from root.modules.form import BaseForm

from project.fal.raidctrl import Raidctrl
from project.fal.lockkey import Lockkey
from project.fal.volctrl import Volctrl
from project.fal.devctrl import Devctrl
from project.fal.diskctrl import Diskctrl, SATA_PORT_NO_LIST
from project.forms.fields.extras.raid import RaidModeField
from project.forms.fields.extras.enable import EnableField


VOLUME_NAME = "sataraid1"


def __get_modelist(volumename):
    for raidvolume in settings.FEATURES(["raidvolumelist"], default=[]):
        if raidvolume.get("name") == volumename:
            return raidvolume.get("modelist", [])

RAID_IS_SUPPORTED = bool(len(__get_modelist(VOLUME_NAME)))
ENCRYPTION_IS_SUPPORTED = "volume_encryption" in settings.FEATURES(["functionlist"], default=[])


def usb1_is_connected():
    return "usbd1" in Devctrl().list().keys()


class InternalVolumeFormatForm(BaseForm):

    def __init__(self, *args, **kwargs):
        super(InternalVolumeFormatForm, self).__init__(*args, **kwargs)
        if RAID_IS_SUPPORTED:
            self.fields['raid_mode'] = RaidModeField(label=_(u"RAID Mode"), required=False)
        if ENCRYPTION_IS_SUPPORTED:
            self.fields['encrypt'] = EnableField(label=_(u'Encrypt'))


class InternalVolumeFormatView(FuncView):

    views = ['execute']

    def view(self, request):
        form = InternalVolumeFormatForm(initial={
            'raid_mode': Raidctrl().query().get('level', 'raid6'),
            'encrypt': Volctrl().query().get('encrypt', False),
        })
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                "RAID_is_supported": RAID_IS_SUPPORTED,
                "encryption_is_supported": ENCRYPTION_IS_SUPPORTED,
                'format_status': MkfsStatusView()._status(),
                'raid_status': RaidStatusView()._status(),
                'form': form,
            },
        }

    def view_execute(self, request):
        form = InternalVolumeFormatForm(data=request.POST)
        with self.processing(constname="ERR_VOLUME_STATE_CHANGE") as p:
            p.break_if_state_change()
            if form.is_valid():
                result = True

                encrypt = False
                if ENCRYPTION_IS_SUPPORTED:
                    encrypt = form.cleaned_data['encrypt']
                    if encrypt:
                        if usb1_is_connected():
                            lockkey = Lockkey()
                            if not lockkey.create():
                                from fal_info import Results
                                if str(lockkey.get_result()) == str(Results.ALREADY_EXIST):
                                    pass
                                elif str(lockkey.get_result()) == str(Results.UNKNOWN_LOCKKEY):
                                    result = False
                                    self.set_message(_(u'USB1 is already lockkey of another LAN DISK.'))
                                else:
                                    result = False
                                    self.set_message(_(u'Failed to create lockkey.'))
                        else:
                            result = False
                            self.set_message(_(u'Please connect lockkey drive.'))

                if result and RAID_IS_SUPPORTED:
                    raidmode = form.cleaned_data["raid_mode"]
                    if raidmode:
                        raidctrl = Raidctrl()
                        if not raidctrl.create(volume=VOLUME_NAME, level=raidmode):
                            result = False
                            self.set_message(raidctrl.get_message())
                            diskctrl = Diskctrl()
                            try:
                                disknolist = range(1, int(mode.split('_')[-1]) + 1)
                            except:
                                disknolist = SATA_PORT_NO_LIST
                            for diskno in disknolist:
                                if not diskobj.is_connected(disk=diskno):
                                    self.set_message(_(u'disk%(diskno)s is not connected.') % {'diskno': diskno})
                                if diskobj.is_capshortage(disk=diskno):
                                    self.set_message(_(u'disk%(diskno)s is capshortage.') % {'diskno': diskno})
                                if diskobj.is_fail(disk=diskno):
                                    self.set_message(_(u'disk%(diskno)s is fail.') % {'diskno': diskno})

                if result:
                    volctrl = Volctrl()
                    result = volctrl.mkfs(volume=VOLUME_NAME, fs="xfs", encrypt=encrypt)
                    self.set_message(volctrl.get_message())

                p.result(result)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                "RAID_is_supported": RAID_IS_SUPPORTED,
                "encryption_is_supported": ENCRYPTION_IS_SUPPORTED,
                'format_status': MkfsStatusView()._status(),
                'raid_status': RaidStatusView()._status(),
                'form': form,
            },
        }

    """
    def view_execute(self, request):
        with self.processing(constname="ERR_VOLUME_STATE_CHANGE") as p:
            form = InternalVolumeFormatForm(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                mode = form.cleaned_data['raid_mode']
                encrypt = form.cleaned_data['encrypt']

                try:
                    if encrypt:
                        if 'usbd1' not in Devctrl().list().keys():
                            self.set_message(
                                    _(u'Please connect lockkey drive.'))
                            raise
                        # 暗号化ならばロックキー作成
                        lobj = Lockkey()
                        if not lobj.create():
                            from fal_info import Results
                            result = int(lobj.get_result())
                            if result == int(Results.ALREADY_EXIST):
                                pass
                            elif result == int(Results.UNKNOWN_LOCKKEY):
                                self.set_message(_(u'USB1 is already lockkey of another LAN DISK.'))
                                raise
                            else:
                                self.set_message(_(u'Failed to create lockkey.'))
                                raise

                    if mode:
                        robj = Raidctrl()
                        vol = robj.VOLUME['internal'][1]
                        if not robj.create(volume=vol, level=mode):
                            self.set_message(robj.get_message())

                            dobj = Diskctrl()
                            #for diskno in SATA_PORT_NO_LIST:
                            try:
                                disknolist = range(1, int(mode.split('_')[-1]) + 1)
                            except:
                                disknolist = SATA_PORT_NO_LIST
                            for diskno in disknolist:
                                if not dobj.is_connected(disk=diskno):
                                    self.set_message(_(u'disk%(diskno)s is not connected.') % {'diskno': diskno})
                                if dobj.is_capshortage(disk=diskno):
                                    self.set_message(_(u'disk%(diskno)s is capshortage.') % {'diskno': diskno})
                                if dobj.is_fail(disk=diskno):
                                    self.set_message(_(u'disk%(diskno)s is fail.') % {'diskno': diskno})

                            raise

                    vobj = Volctrl()
                    p.result(vobj.mkfs(volume=vobj.VOLUME['internal'][1],
                                       fs='xfs',
                                       encrypt=encrypt))
                    self.set_message(vobj.get_message())
                except:
                    p.result(False)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'format_status': MkfsStatusView()._status(),
                'raid_status': RaidStatusView()._status(),
                'form': form}}
    """


class MkfsStatusView(StatusView):
    STATUS_FILE_PATH = "/mnt/data/conf/volume/status/sataraid1/mkfs"

    def _status(self):
        try:
            from project.utils import is_keywait
            if is_keywait():
                return _(u'VolumeStatusKeywaiting')
            return super(MkfsStatusView, self)._status()
        except:
            return super(MkfsStatusView, self)._status()


class RaidStatusView(StatusView):
    def _status(self):
        robj = Raidctrl()
        rinfo = robj.query()
        return robj.status(humanize=True)
