# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.conf import settings
from django import forms
from django.core.exceptions import ValidationError
from project.view import RootView
import os
import re
from datetime import datetime

from project.fal.sysinfo import SysInfo
from project.fal.fdatetime import DateTime
from project.fal.eco import Eco
from project.fal.power import Power

from project.fal.raidctrl import Raidctrl
from project.fal.lockkey import Lockkey
from project.fal.volctrl import Volctrl
from project.fal.devctrl import Devctrl
from project.fal.diskctrl import Diskctrl, SATA_PORT_NO_LIST

from project.forms.fields.extras.landiskname import LandiskNameField
from project.forms.fields.extras.comment import CommentField
from project.forms.fields.extras.fqdn import FQDNField
from project.forms.fields.extras.ipv4address import IPv4AddressField
from project.forms.fields.extras.ipv4subnetmask import IPv4SubnetMaskField
from project.forms.fields.extras.jumboframesize import JumboFrameSizeField
from project.forms.fields.extras.raid import RaidModeField
from project.forms.fields.extras.enable import EnableField
from project.forms.fields.extras.spindown import SpindownTimeField
from project.forms.fields.extras.microsoftnetwork import MicrosoftNetworkMethodField

from project.forms.fields.radio import RadioField
from project.forms.fields.checkbox import CheckboxField


INTERNAL_VOLUME_NAME = "sataraid1"


def __get_modelist(volumename):
    for raidvolume in settings.FEATURES(["raidvolumelist"], default=[]):
        if raidvolume.get("name") == volumename:
            return raidvolume.get("modelist", [])
    return []

RAID_IS_SUPPORTED = bool(len(__get_modelist(INTERNAL_VOLUME_NAME)))
ENCRYPTION_IS_SUPPORTED = "volume_encryption" in settings.FEATURES(["functionlist"], default=[])
LAN2_IS_SUPPORTED = bool(len([netif for netif in settings.FEATURES(["deviceinfo", "netiflist"]) if str(netif["port"]) == "2"]))

WEEKDAYS = ('sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat')


class DateField(forms.DateField):

    def __init__(self, attrs=None, *args, **kwargs):
        if "input_formats" not in kwargs:
            kwargs["input_formats"] = ['%Y/%m/%d']
        if "widget" not in kwargs:
            kwargs["widget"] = self.widget(
                attrs=attrs,
                format=kwargs["input_formats"][0])
        super(DateField, self).__init__(*args, **kwargs)


class TimeField(forms.TimeField):

    def __init__(self, attrs=None, *args, **kwargs):
        if 'input_formats' not in kwargs:
            kwargs['input_formats'] = ['%H:%M']
        if 'widget' not in kwargs:
            kwargs['widget'] = self.widget(
                attrs=attrs,
                format=kwargs['input_formats'][0])
        super(TimeField, self).__init__(*args, **kwargs)


class InitialSettingForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(InitialSettingForm, self).__init__(*args, **kwargs)

        # 名前設定
        self.fields['landiskname'] = LandiskNameField(
            label=_(u"LAN DISK Name"))
        self.fields['landiskdesc'] = CommentField(
            label=_(u"LAN DISK Description"),
            required=False)
        self.fields['landisklocation'] = CommentField(
            label=_(u"LAN DISK Location"),
            required=False)

        # ネットワーク設定
        self.fields['lan1_dhcp'] = RadioField(
            widget=forms.widgets.RadioSelect(),
            label=_(u"IP Address Setting Method"),
            choices=(('auto', _(u"Obtain an IP address automatically(DHCP)")),
                     ('custom', _(u'Use the following IP address'))),
            initial='auto',
            required=True)
        self.fields['lan1_ipaddr'] = forms.CharField(
            label=_(u"IP Address"),
            required=False)
        self.fields['lan1_netmask'] = IPv4SubnetMaskField(
            label=_(u"Subnet Mask"),
            required=False)
        self.fields['lan1_framesize'] = JumboFrameSizeField(
            label=_(u"Frame Size"),
            initial=1500,
            required=True)
        self.fields['lan1_wol'] = EnableField(
            label=_(u"Wake On LAN"),
            initial=False,
            required=True)
        if LAN2_IS_SUPPORTED:
            self.fields['lan2_dhcp'] = RadioField(widget=forms.widgets.RadioSelect(),
                                                  label=_(u"IP Address Setting Method"),
                                                  choices=(('disable', _(u"Disable")),
                                                           ('auto', _(u"Obtain an IP address automatically(DHCP)")),
                                                           ('custom', _(u'Use the following IP address'))),
                                                  initial='auto',
                                                  required=True)
            self.fields['lan2_ipaddr'] = forms.CharField(label=_(u"IP Address"),
                                                         required=False)
            self.fields['lan2_netmask'] = IPv4SubnetMaskField(label=_(u"Subnet Mask"),
                                                              required=False)
            self.fields['lan2_framesize'] = JumboFrameSizeField(label=_(u"Frame Size"),
                                                                initial=1500,
                                                                required=True)
            self.fields['lan2_wol'] = EnableField(label=_(u"Wake On LAN"),
                                                  initial=False)
        self.fields['nw_common'] = RadioField(widget=forms.widgets.RadioSelect(),
                                              label=_(u"Common Setting"),
                                              choices=(('auto', _(u"Auto")), ('custom', _(u"Custom Settings"))),
                                              initial='auto',
                                              required=True)
        #self.fields['nw_mainif'] = RadioField(
        #    label=_(u"Main Interface"),
        #    choices=(('lan1', _(u"LAN1")),
        #             ('lan2', _(u"LAN2"))),
        #    initial='lan1',
        #    required=True)
        from project.forms.fields.extras.network import MainInterfaceField
        _nw_mainif = MainInterfaceField()
        if len(_nw_mainif.choices) > 1:
            self.fields['nw_mainif'] = _nw_mainif
        self.fields['nw_gateway'] = IPv4AddressField(
            label=_(u"Default Gateway"),
            required=False)
        self.fields['nw_dns'] = IPv4AddressField(
            label=_(u"DNS Server"),
            required=False)

        # Microsoftネットワーク設定
        from project.forms.fields.extras.domain import WorkGroupField
        from project.forms.fields.extras.domain import NetBIOSDomainField
        from project.forms.fields.extras.domain import NtControllerField
        from project.forms.fields.extras.domain import AdControllerField
        from project.forms.fields.extras.domain import AdUserField
        from project.forms.fields.extras.domain import AdAdminPassField
        from project.forms.fields.extras.domain import DomainNameField
        from project.forms.fields.extras.domain import WinsSrvField
        self.fields['msnw_method'] = MicrosoftNetworkMethodField(
            label=_(u"Method"))
        self.fields['msnw_wg_name'] = WorkGroupField(
            label=_(u"WorkGroup Name"),
            max_length=15,
            required=False)
        self.fields['msnw_nt_name'] = NetBIOSDomainField(
            label=_(u"Domain Name"),
            max_length=15,
            required=False)
        self.fields['msnw_nt_dcname'] = NtControllerField(
            label=_(u"Domain Controller Name"),
            max_length=15,
            required=False)
        self.fields['msnw_ad_netbios'] = NetBIOSDomainField(
            label=_(u"Domain Name(NetBIOS Name)"),
            max_length=15,
            required=False)
        self.fields['msnw_ad_fulldns'] = DomainNameField(
            label=_(u"Domain Name(Full DNS Name)"),
            max_length=155,
            required=False)
        self.fields['msnw_ad_dcname'] = AdControllerField(
            label=_(u"Domain Controller Name"),
            max_length=155,
            required=False)
        self.fields['msnw_ad_adminname'] = AdUserField(
            label=_(u"Administrator Name"),
            max_length=20,
            required=False)
        self.fields['msnw_ad_passwd'] = AdAdminPassField(
            label=_(u"Administrator Password"),
            max_length=20,
            required=False)
        self.fields['msnw_winssrv'] = WinsSrvField(
            label=_(u"WINS Server"),
            required=False)

        # 時刻設定
        self.fields['dttype'] = RadioField(
            label=_(u"Datetime setting"),
            choices=(('m', _(u"manual")), ('s', _(u"syncronous"))),
            initial='m')
        self.fields['date'] = DateField(
            label=_('Date'),
            input_formats=[{
                'ja': "%Y/%m/%d",
            }.get('ja', "%Y/%m/%d")],
            required=False)
        self.fields['time'] = TimeField(
            label=_('Time'),
            required=False)
        self.fields['dtntp'] = FQDNField(
            label=_(u"NTP Server"),
            max_length=128,
            required=False)
        self.fields['dtsyncboot'] = CheckboxField(
            label=_(u"Boot"),
            required=False)
        self.fields['dtsyncdaily'] = CheckboxField(
            label=_(u"Daily"),
            required=False)
        self.fields['dtsyncdailytime'] = TimeField(
            label=_(u"Time"),
            required=False)

        # RAID設定
        if RAID_IS_SUPPORTED:
            self.fields['raidmode'] = RaidModeField(label=_(u"RAID Mode"),
                                                    required=False)
        if ENCRYPTION_IS_SUPPORTED:
            self.fields['encrypt'] = EnableField(label=_(u"Encrypt"))

        # 省電力設定
        self.fields['eco_time'] = SpindownTimeField(
            label=_(u"Time Setting"))
#        from product_conf import VOL_LIST
#        for vol in VOL_LIST:
#            self.fields["eco_%s" % vol] = EnableField(
#                label=vol)
        self.fields['eco_sataraid1'] = EnableField(label=_(u"Internal Volume"))
        for usb in settings.FEATURES(["deviceinfo", "usblist"], []):
            try:
                port = usb["port"]
                self.fields['eco_usb%s' % port] = EnableField(label=_(u"USB%(i)s" % {'i': port}))
            except:
                pass
        #self.fields['eco_usb1'] = EnableField(label=_(u"USB%(i)s" % {'i': 1}))
        #self.fields['eco_usb2'] = EnableField(label=_(u"USB%(i)s" % {'i': 2}))
        #self.fields['eco_usb3'] = EnableField(label=_(u"USB%(i)s" % {'i': 3}))
        #self.fields['eco_usb4'] = EnableField(label=_(u"USB%(i)s" % {'i': 4}))

        self.fields['schedule_shutdown_delay_on_accessing'] = CheckboxField(
            label=_(u"Shutdown delay on accessing."),
            required=False)
        for day in WEEKDAYS:
            _day = _(u"%s" % day.capitalize())
            self.fields["%s_on" % day] = CheckboxField(
                label=_(u"Initial Setting: On(%(day)s)") % {'day': _day})
            self.fields["%s_on_time" % day] = TimeField(
                label=_(u"Initial Setting: OnTime(%(day)s)") % {'day': _day})
            self.fields["%s_off" % day] = CheckboxField(
                label=_(u"Initial Setting: Off(%(day)s)") % {'day': _day})
            self.fields["%s_off_time" % day] = TimeField(
                label=_(u"Initial Setting: OffTime(%(day)s)") % {'day': _day})

    def clean(self):
        data = self.cleaned_data

        messages = []

        errors = {
            'required': [],
        }

        # ネットワーク設定
        for lanX in ('lan1', 'lan2'):
            if data.get("%s_dhcp" % lanX) == 'custom':
                if not data.get("%s_ipaddr" % lanX):
                    errors['required'].append("%s_ipaddr" % lanX)
                if not data.get("%s_netmask" % lanX):
                    errors['required'].append("%s_netmask" % lanX)
                from root.admin.network.modules.utils import ip_v4_netmask_validate
                ip_v4_netmask_validate(data.get("%s_ipaddr" % lanX),
                                       data.get("%s_netmask" % lanX))
        if ((data.get('nw_common') == 'auto') and \
            ((data.get('lan1_dhcp') != 'auto') and \
             (data.get('lan2_dhcp') != 'auto'))):
            #messages.append(_(u'DHCP is disable. "Common Setting" can not be set to "Auto".'))
            messages.append(_(u'DHCP is disable. "%(common_setting)s" can not be set to "%(auto)s".') % {
                "common_setting": _(u"Common Setting"),
                "auto": _("Auto"),
            })
        # Microsoftネットワーク設定
        method = data.get('msnw_method')
        if method == 'wg':
            if not data.get('msnw_wg_name'):
                errors["required"].append('msnw_wg_name')
        elif method == 'nt':
            if not data.get('msnw_nt_name'):
                errors["required"].append('msnw_nt_name')
            if not data.get('msnw_nt_dcname'):
                errors['required'].append('msnw_nt_dcname')
        elif method == 'ad':
            if not data.get('msnw_ad_netbios'):
                errors["required"].append('msnw_ad_netbios')
            if not data.get('msnw_ad_fulldns'):
                errors["required"].append('msnw_ad_fulldns')
            if not data.get('msnw_ad_dcname'):
                errors["required"].append('msnw_ad_dcname')
            if not data.get('msnw_ad_adminname'):
                errors["required"].append('msnw_ad_adminname')
            if not data.get('msnw_ad_passwd'):
                errors["required"].append('msnw_ad_passwd')
        # 時刻設定
        if data.get('dttype') == 's':
            if not data.get('dtntp'):
                errors['required'].append('dtntp')
            if (not data.get('dtsyncboot')) and (not data.get('dtsyncdaily')):
                messages.append(_(u'"Syncronous Timing" is required.'))
        elif data.get('dttype') == 'm':
            if (not data.get('date')) or (not data.get('time')):
                messages.append("%s: %s" % (_(u"Datetime"),
                                            _(u"This field is required.")))

        default_error_message = {
            'required': _(u"This field is required."),
        }
        for err, names in errors.iteritems():
            for name in names:
                messages.append("%s: %s" % (self.fields[name].label,
                                            default_error_message[err]))
        if len(messages):
            raise ValidationError(messages)

        return data


class InitialSettingView(RootView):

    views = ['apply']

    def __URL(self):
        if "narsus" in settings.FEATURES(["functionlist"]):
            return '/admin/narsus/guide'
        else:
            return '/admin/main'

    def view(self, request):
        form = InitialSettingForm(initial=self.__initial())

        try:
            SysInfo().clear()
        except:
            pass

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            "lan2_is_supported": LAN2_IS_SUPPORTED,
            "RAID_is_supported": RAID_IS_SUPPORTED,
            "encryption_is_supported": ENCRYPTION_IS_SUPPORTED,
            'form': form,
            'button_disabled': False,
            'URL': self.__URL(),
            'IS_LOGINED': True,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        form = InitialSettingForm(data=request.POST)

        result = False
        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                try:
                    # 名前設定
                    SysInfo().apply(
                        hostname=form.cleaned_data['landiskname'],
                        hostdesc=form.cleaned_data['landiskdesc'],
                        hostlocation=form.cleaned_data['landisklocation'])
                    # 時刻設定
                    if form.cleaned_data['dttype'] == 's':
                        DateTime().apply_config(config={
                            'ntp': {
                                'enabled': True,
                                'serveraddress': form.cleaned_data['dtntp'],
                                'sync': {
                                    'boot': {
                                        'enabled': form.cleaned_data['dtsyncboot'],
                                    },
                                    'time': {
                                        'enabled': form.cleaned_data['dtsyncdaily'],
                                        'time': form.cleaned_data['dtsyncdailytime'],
                                    },
                                },
                            },
                        })
                    else:
                        d = form.cleaned_data['date']
                        t = form.cleaned_data['time']
                        DateTime().apply_config(config={
                            'ntp': {
                                'enabled': False,
                            },
                            'manual': {
                                'datetime': datetime(d.year, d.month, d.day, t.hour, t.minute),
                            },
                        })
                    # RAID設定
                    raidmode = None
                    if RAID_IS_SUPPORTED:
                        raidmode = form.cleaned_data['raidmode']
                    encrypt = None
                    if ENCRYPTION_IS_SUPPORTED:
                        encrypt = form.cleaned_data['encrypt']
                    if not self.__raidchange(level=raidmode, encrypt=encrypt):
                        raise
                    # 省電力設定
                    eco_time = int(form.cleaned_data['eco_time'])
                    eco_enabled = bool(eco_time)
                    Eco().apply(config={
                        'time': {
                            'minute': eco_time,
                        },
                        'volumelist': [{
                                'enabled': eco_enabled and form.cleaned_data['eco_sataraid1'],
                                'name': 'sataraid1',
                            }] + [{
                                "enabled": eco_enabled and form.cleaned_data.get("eco_usb%s" % usb["port"]),
                                "name": "usb%s" % usb["port"]
                            } for usb in settings.FEATURES(["deviceinfo", "usblist"], []) if usb.get("port", None) is not None],
                    })
                    # 電源スケジュール設定
                    daylist = {}
                    for day in WEEKDAYS:
                        daylist[day] = {}
                        for onoff in ('on', 'off'):
                            daylist[day][onoff] = {}
                            daylist[day][onoff]['enable'] = form.cleaned_data["%s_%s" % (day, onoff)]
                            daylist[day][onoff]['time'] = form.cleaned_data["%s_%s_time" % (day, onoff)]
                    Power().apply2(config={
                        'schedule': {
                            'daylist': daylist,
                            'delay': form.cleaned_data['schedule_shutdown_delay_on_accessing'],
                        },
                    })
                    # ネットワーク設定 & Microsoftネットワーク
                    self._nwchange(form.cleaned_data)

                    self.set_message(_(u"Initial Setting: Now setting."))
                    p.result(True)
                except:
                    pass
            result = p.result()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            "lan2_is_supported": LAN2_IS_SUPPORTED,
            "RAID_is_supported": RAID_IS_SUPPORTED,
            "encryption_is_supported": ENCRYPTION_IS_SUPPORTED,
            'form': form,
            'button_disabled': result,
            'URL': self.__URL(),
            'IS_LOGINED': True,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __initial(self):

        initial = {}
        try:

            try:
                initial['landiskname'] = SysInfo().query()['host']['name']
                initial['landiskdesc'] = SysInfo().query()['host']['desc']
                initial['landisklocation'] = SysInfo().query()['host']['location']
            except:
                pass

            try:
                from .. admin.network.modules.network import Network
                nw = Network()
                info1 = nw.get_ipv4_info(netif='lan1')
                initial['lan1_dhcp'] = info1['dhcp']
                initial['lan1_ipaddr'] = info1.get('ipaddr', '')
                initial['lan1_netmask'] = info1.get('netmask', '255.255.255.0')
                initial['lan1_framesize'] = info1.get('framesize', '1500')
                initial['lan1_wol'] = info1.get('wol', 'false') == 'true'
                if LAN2_IS_SUPPORTED:
                    info2 = nw.get_ipv4_info(netif='lan2')
                    initial['lan2_dhcp'] = info2['dhcp']
                    initial['lan2_ipaddr'] = info2.get('ipaddr', '')
                    initial['lan2_netmask'] = info2.get('netmask', '255.255.255.0')
                    initial['lan2_framesize'] = info2.get('framesize', '1500')
                    initial['lan2_wol'] = info2.get('wol', 'false') == 'true'
                if info1.get('common'):
                    initial['nw_common'] = info1.get('common')
                if info1.get('mainif'):
                    initial['nw_mainif'] = info1.get('mainif')
                initial['nw_gateway'] = info1['gateway']
                initial['nw_dns'] = info1['dns']
            except:
                pass

            try:
                from .. admin.usergroup.msusergroup.modules.msnetwork import MSnetwork
                msnwinfo = MSnetwork().get_msnetwork_config()
                initial['msnw_method'] = {1:'wg',2:'nt',3:'ad'}.get(msnwinfo['method'], 'wg')
                initial['msnw_wg_name'] = msnwinfo['wg_name']
                initial['msnw_nt_name'] = msnwinfo['nt_name']
                initial['msnw_nt_dcname'] = msnwinfo['nt_dcname']
                initial['msnw_ad_netbios'] = msnwinfo['netbios']
                initial['msnw_ad_fulldns'] = msnwinfo['fulldns']
                initial['msnw_ad_adminname'] = msnwinfo['adminname']
                initial['msnw_ad_dcname'] = msnwinfo['ad_dcname']
                initial['msnw_ad_passwd'] = msnwinfo['passwd']
                initial['msnw_winssrv'] = msnwinfo['winssrv']
            except:
                pass

            try:
                dtinfo = DateTime().load()
                initial['dttype'] = 's' if dtinfo['ntp']['enabled'] else 'm'
                initial['date'] = datetime.now().date()
                initial['time'] = datetime.now().time()
                initial['dtntp'] = dtinfo['ntp']['srvaddr']
                initial['dtsyncboot'] = dtinfo['ntp']['sync']['boot']['enabled']
                initial['dtsyncdaily'] = dtinfo['ntp']['sync']['time']['enabled']
                initial['dtsyncdailytime'] = dtinfo['ntp']['sync']['time']['time']
            except:
                pass

            try:
                initial["encrypt"] = Volctrl().query().get("encrypt", False)
            except:
                pass

            try:
                ecoinfo = Eco().load()
                initial['eco_time'] = ecoinfo['time']['minute']
                for vol in ecoinfo['volumelist']:
                    initial["eco_%s" % vol['name']] = vol['enabled']
            except:
                pass

            try:
                pwinfo = Power().load(to_dict=True)
                initial['schedule_shutdown_delay_on_accessing'] = pwinfo['delay']
                for day in WEEKDAYS:
                    initial['%s_on' % day] = pwinfo[day]['on']
                    initial['%s_on_time' % day] = pwinfo[day]['ontime']
                    initial['%s_off' % day] = pwinfo[day]['off']
                    initial['%s_off_time' % day] = pwinfo[day]['offtime']
            except:
                pass

        except:
            pass
        return initial

    def __raidchange(self, level=None, encrypt=None):
        try:
            if ENCRYPTION_IS_SUPPORTED:
                if encrypt:
                    if 'usbd1' not in Devctrl().list().keys():
                       self.set_message(_(u'Please connect lockkey drive.'))
                       raise
                    lockkey = Lockkey()
                    if not lockkey.create():
                        from fal_info import Results
                        result = str(lockkey.get_result())
                        if result == str(Results.ALREADY_EXIST):
                            pass
                        elif result == str(Results.UNKNOWN_LOCKKEY):
                            self.set_message(_(u'USB1 is already lockkey of another LAN DISK.'))
                            raise
                        else:
                            self.set_message(_(u'Failed to create lockkey.'))
                            raise

            raid_created = False
            if RAID_IS_SUPPORTED:
                if level:
                    raidctrl = Raidctrl()
                    if not raidctrl.create(volume=INTERNAL_VOLUME_NAME, level=level):
                        self.set_message(raidctrl.get_message())
                        diskctrl = Diskctrl()
                        try:
                            disknolist = range(1, int(level.split('_')[-1]) + 1)
                        except:
                            disknolist = SATA_PORT_NO_LIST
                        for diskno in disknolist:
                            if not diskctrl.is_connected(disk=diskno):
                                self.set_message(_(u'disk%(diskno)s is not connected.') % {'diskno': diskno})
                            if dobj.is_capshortage(disk=diskno):
                                self.set_message(_(u'disk%(diskno)s is capshortage.') % {'diskno': diskno})
                            if dobj.is_fail(disk=diskno):
                                self.set_message(_(u'disk%(diskno)s is fail.') % {'diskno': diskno})
                        raise
                    else:
                        raid_created = True

            if (raid_created) or (encrypt != Volctrl().query().get("encrypt")):
                volctrl = Volctrl()
                volctrl.mkfs(volume=INTERNAL_VOLUME_NAME,
                             fs='xfs',
                             encrypt=encrypt)
            #self.set_message(vobj.get_message())
            return True
        except:
            return False

    def _nwchange(self, cleaned_data):
        try:
            from subprocess import call, Popen, PIPE, STDOUT
            from product_conf import SUDO_CMD
            cmd = [SUDO_CMD, '/usr/bin/python',
                   os.path.join(self.CURRENT, '..', '..', 'nwchgcmd.py'),
                   '--lan1-dhcp', cleaned_data['lan1_dhcp'],
                   '--lan1-ipaddr', cleaned_data['lan1_ipaddr'],
                   '--lan1-netmask', cleaned_data['lan1_netmask'],
                   '--lan1-framesize', str(cleaned_data['lan1_framesize']),
                   '--nw-common', cleaned_data['nw_common'],
                   #'--nw-mainif', cleaned_data['nw_mainif'],
                   '--nw-gateway', cleaned_data['nw_gateway'],
                   '--nw-dns', cleaned_data['nw_dns'],

                   '--method', cleaned_data['msnw_method'],
                   '--wg-name', cleaned_data['msnw_wg_name'],
                   '--nt-name', cleaned_data['msnw_nt_name'],
                   '--nt-dcname', cleaned_data['msnw_nt_dcname'],
                   '--ad-dcname', cleaned_data['msnw_ad_dcname'],
                   '--ad-netbios', cleaned_data['msnw_ad_netbios'],
                   '--ad-fulldns', cleaned_data['msnw_ad_fulldns'],
                   '--ad-adminname', cleaned_data['msnw_ad_adminname'],
                   '--ad-passwd', cleaned_data['msnw_ad_passwd'],

                   '--winssrv', cleaned_data['msnw_winssrv'],
                  ]
            if len(settings.NETWORK_INTERFACES) > 1:
                cmd = cmd + ['--nw-mainif', cleaned_data['nw_mainif']]
            if cleaned_data['lan1_wol']:
                cmd = cmd + ['--lan1-wol-on']
            if LAN2_IS_SUPPORTED:
                cmd = cmd + ['--lan2-dhcp', cleaned_data['lan2_dhcp'],
                             '--lan2-ipaddr', cleaned_data['lan2_ipaddr'],
                             '--lan2-netmask', cleaned_data['lan2_netmask'],
                             '--lan2-framesize', str(cleaned_data['lan2_framesize'])]
                if cleaned_data['lan2_wol']:
                    cmd = cmd + ['--lan2-wol-on']
            proc = Popen(cmd, stdout=PIPE, stderr=STDOUT)
        except:
            pass
