# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
import re
from project.view import RootView
from project.auth import Authentication
from project.fal.command import get_message as fgettext, logging
from project.fal.user import User
from project.fal.sysinfo import SysInfo

ADMIN = 'admin'
PASSWORD_MIN_LENGTH = 4


class LoginView(RootView):
    views = ['login']

    REQUIRED_LOGIN = False

    class Form(forms.Form):
        from project.forms.fields.password import PasswordField
        password = PasswordField(
                label=_(u'Administrator Password'),
                required=False)

    def __init__(self):
        super(LoginView, self).__init__()
        self.TEMPLATE = 'login.html'

    def __is_admin_password_qualified(self):
        """Check administrator password."""

        user = User()
        info = user.load(all=True)[ADMIN]
        if len(info['passwd']) >= PASSWORD_MIN_LENGTH:
            return True

        return False

    def view(self, request, url=None):
        request.session['FRAME_SRC'] = url  # /admin/main で利用
        form = self.Form()
        if 'MSG_LOGOUT' in request.session:
            self.set_message(request.session['MSG_LOGOUT'])
            del request.session['MSG_LOGOUT']
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'url_userpassword': '/userpassword',
                'form': form}}

    def view_login(self, request, url=None):
        form = self.Form(data=request.POST)
        auth = Authentication()

        if form.is_valid():
            if auth.login(request, form.cleaned_data['password']):
                # ログイン成功
                # 表示はされないが一応メッセージをセット
                self.set_message(fgettext(
                    'MSG_SYSTEM_WEB_LOGIN',
                    arglist={'ip_address': request.META['REMOTE_ADDR']},
                    is_log=True))
            else:
                # ログイン失敗
                # (パスワード不適合/余所でログイン済み)
                self.message_is_error = True
                self.set_message(fgettext(
                    'ERR_SYSTEM_WEB_LOGIN',
                    arglist={'ip_address': request.META['REMOTE_ADDR']},
                    is_log=True))
                if auth.is_logined_other(request):
                    logging(
                        'ERR_SYSTEM_WEB_MULTI_LOGIN',
                        arglist={'ip_address': request.META['REMOTE_ADDR']})

                if request.is_ajax():
                    return {
                        self.KEY_TEMPLATE: self.AJAXFORM_HTML,
                        self.KEY_TEMPLATE_DIC: {'form': form}}
                else:
                    return {
                        self.KEY_TEMPLATE: self.TEMPLATE,
                        self.KEY_TEMPLATE_DIC: {'form': form}}
        else:
            self.message_is_error = True
            self.set_message(fgettext(
                'ERR_SYSTEM_WEB_LOGIN',
                arglist={'ip_address': request.META['REMOTE_ADDR']},
                is_log=True))
            return {
                self.KEY_TEMPLATE: self.AJAXFORM_HTML if request.is_ajax() \
                else self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {'form': form}}

        if auth.is_logined(request):
            # ログイン済み
            # (ログイン成功/そもそもログアウトしていない)
            self.message_is_error = False
            if re.match('\/gate\/.*', request.session.get('FRAME_SRC', '')):
                return {self.KEY_REDIRECT: request.session['FRAME_SRC']}

            redirect = '/changelist/changelist'
            if not self.__is_admin_password_qualified():
                redirect = '/setpass'

            return {self.KEY_REDIRECT: redirect}
        else:
            # ログインしていない
            # (ログイン失敗)
            self.message_is_error = True
            return {
                self.KEY_TEMPLATE: self.AJAXFORM_HTML if request.is_ajax() \
                else self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {'form': form}}


class SetPassView(RootView):
    views = ['apply']

    REQUIRED_LOGIN = True
    NOT_LOGINED_URL = '/login'

    class Form(forms.Form):
        from project.forms.fields.extras.landiskpass import LanDiskPassField
        password = LanDiskPassField(
            label=_(u'Administrator Password'),
            min_length=PASSWORD_MIN_LENGTH,
            required=True)
        pw_confirm = LanDiskPassField(
            label=_(u'Administrator Password (Confirm)'),
            min_length=PASSWORD_MIN_LENGTH,
            required=True)

        def clean(self):
            from django import forms
            password = self.cleaned_data.get('password', None)
            pw_confirm = self.cleaned_data.get('pw_confirm', None)
            if password != pw_confirm:
                raise forms.ValidationError(_(u'Password unmatch.'))

            return self.cleaned_data

    def __init__(self):
        super(SetPassView, self).__init__()
        self.TEMPLATE = 'setpass.html'

    def __set_admin_password(self, password):
        """Set administrator password."""

        with self.processing(constname='ERR_ADMIN_STATE_CHANGE') as p:
            p.break_if_state_change()
            user = User()
            info = user.load(all=True)[ADMIN]
            user.adminmod(passwd=password, mailaddr=info['mailaddr'])
            self.set_message(user.get_message())
            result = user.get_result(boolean=True)
            p.result(result)
            return result

        return False

    def view(self, request, url=None):
        # /admin/main で利用
        request.session['FRAME_SRC'] = url

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': self.Form()}}

    def view_apply(self, request, url=None):
        form = self.Form(data=request.POST)

        try:
            if not form.is_valid():
                raise ValueError

            password = form.cleaned_data['password']
            if not self.__set_admin_password(password):
                raise ValueError

            redirect = "/admin/main"
            from django.conf import settings
            if "narsus" in settings.FEATURES(["functionlist"]):
                redirect = '/admin/narsus/guide'
            condition = SysInfo().query().get('condition')
            if condition == 'installed':  # 出荷時
                redirect = '/init/init'
            elif condition == 'updated':  # F/W更新時
                redirect = '/changelist/changelist'

            return {
                self.KEY_REDIRECT: redirect,
                self.KEY_RESPONSE: None}
        except ValueError:
            self.message_is_error = True
            self.set_message(fgettext('ERR_ADMIN_CHANGE', is_log=True))
            template = self.TEMPLATE
            if request.is_ajax():
                template = self.AJAXFORM_HTML
            return {
                self.KEY_TEMPLATE: template,
                self.KEY_TEMPLATE_DIC: {'form': form}}


class LogoutView(RootView):
    views = []

    REQUIRED_LOGIN = False

    def view(self, request):
        auth = Authentication()
        if auth.logout(request):
            request.session['MSG_LOGOUT'] = fgettext(
                    'MSG_SYSTEM_WEB_LOGOUT',
                    arglist={'ip_address': request.META['REMOTE_ADDR']},
                    is_log=True)
        else:
            request.session['MSG_LOGOUT'] = fgettext(
                    'MSG_SYSTEM_WEB_LOGOUT',
                    arglist={'ip_address': request.META['REMOTE_ADDR']},
                    is_log=True)
            pass
        return {self.KEY_REDIRECT: '/'}


login = LoginView.as_view()
setpass = SetPassView.as_view()
logout = LogoutView.as_view()
